!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
module compute_budgets

!-----------------------------------------------------------------------
! PURPOSE: check conservation of tracer-like quantities, check
!   conservation of mass, keep track of enstropy, globally-averaged
!   kinetic energy, etc.
!-----------------------------------------------------------------------

  use kinds
  use physical_parameters
  use time_manager_module
  use model_parameters
  use sigma_coordinate
  use tracer_names
  use grid_metrics
  use prognostics
  use global_reductions
  use adjustments

  implicit none
  save

  real(kind=dbl_kind), dimension(ntracer) ::  &
    glb_trcdiff = 0.0_dbl_kind, &
    glb_trcres  = 0.0_dbl_kind

!*********************************************************************

  contains

!*********************************************************************

  subroutine compute_global_budgets

    implicit none

    integer(kind=int_kind) :: k, nt, size_2d, size_3d

    real(kind=dbl_kind), dimension(iim,jjm,nsdm) ::  &
      work1_2d = 0.0_dbl_kind, &
      work2_2d = 0.0_dbl_kind, &
      work3_2d = 0.0_dbl_kind, &
      work4_2d = 0.0_dbl_kind, &
      work5_2d = 0.0_dbl_kind, &
      work6_2d = 0.0_dbl_kind, &
      work7_2d = 0.0_dbl_kind, &
      one_2d   = 1.0_dbl_kind, &
      area_mask

    real(kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) :: &
      work1 = 0.0_dbl_kind,   &
      work2 = 0.0_dbl_kind,   &
      work3 = 0.0_dbl_kind,   &
      work4 = 0.0_dbl_kind,   &
      work5 = 0.0_dbl_kind,   &
      work6 = 0.0_dbl_kind,   &
      work7 = 0.0_dbl_kind,   &
      work8 = 0.0_dbl_kind,   &
      one   = 1.0_dbl_kind,   &
      mass_n3 = 0.0_dbl_kind, &
      mass_n4 = 0.0_dbl_kind

    real(kind=dbl_kind), dimension(ntprog) :: mass = 0.0_dbl_kind
    real(kind=dbl_kind), dimension(nttend) :: mass_f = 0.0_dbl_kind

    real(kind=dbl_kind) ::  &
      tot_area  = 0.0_dbl_kind, &
      source    = 0.0_dbl_kind, &
      tot_mass  = 0.0_dbl_kind, &
      sum_grid  = 0.0_dbl_kind, &
      dcadjust  = 0.0_dbl_kind, &
      fill      = 0.0_dbl_kind, &
      error     = 0.0_dbl_kind, &
      rmin      = 0.0_dbl_kind, &
      rmax      = 0.0_dbl_kind

    size_2d = iim*jjm*nsdm
    size_3d = iim*jjm*nlm*nsdm
    area_mask = area * grid_center_mask
    tot_area = global_real_sum(area_mask,one_2d,size_2d)
    sum_grid = global_real_sum(grid_center_mask(1,1,1),one_2d(1,1,1),size_2d)

!----------------------------------------------------------------------------
! write header
!----------------------------------------------------------------------------
    if (my_task == 0) then
      write(6,20) tau,step_count
    endif

!----------------------------------------------------------------------------
! compute mass using pthick
!----------------------------------------------------------------------------
    area_mask = area * grid_center_mask
    do k=1,nlm
      mass_n3(:,:,k,:) = pthick(:,:,k,n3,:) * area_mask(:,:,:) * dsig(k)
      mass_n4(:,:,k,:) = pthick(:,:,k,n4,:) * area_mask(:,:,:) * dsig(k)
    enddo	
    mass(n3) = global_real_sum(mass_n3, one, size_3d) / tot_area
    mass(n4) = global_real_sum(mass_n4, one, size_3d) / tot_area
    if (my_task == 0) then
      write(6,30) mass(n3)+p_top, mass(n4)+p_top
    endif
    tot_mass = mass(n3)*tot_area

!----------------------------------------------------------------------------
! compute mass use surface pressure
!----------------------------------------------------------------------------
    work1_2d = ps(:,:,n3,:) * area_mask(:,:,:)
    work2_2d = ps(:,:,n4,:) * area_mask(:,:,:)
    mass(n3) = sum(work1_2d)/tot_area
    mass(n4) = sum(work2_2d)/tot_area
    mass(n3) = global_real_sum(work1_2d(:,:,:), one_2d(:,:,:), size_2d)
    mass(n4) = global_real_sum(work2_2d(:,:,:), one_2d(:,:,:), size_2d)
    mass = mass/tot_area
    if (my_task == 0) then
      write(6,35) mass(n3), mass(n4)
    endif

!----------------------------------------------------------------------------
! compute mass tendency using ps_f
!----------------------------------------------------------------------------
    mass_f(n3_f)=global_real_sum(ps_f(:,:,n3_f,:),area_mask(:,:,:),size_2d)
    mass_f(n2_f)=global_real_sum(ps_f(:,:,n2_f,:),area_mask(:,:,:),size_2d) 
    mass_f(n1_f)=global_real_sum(ps_f(:,:,n1_f,:),area_mask(:,:,:),size_2d)
    mass_f = mass_f/tot_area
    if (my_task == 0) then
      write(6,40) mass_f(n3_f),mass_f(n2_f),mass_f(n1_f)
    endif
#ifndef no_pbl
!----------------------------------------------------------------------------
! compute pbl thickness and tendency
!----------------------------------------------------------------------------
    work1_2d(:,:,:) = (ps(:,:,n3,:)-pb(:,:,n3,:)) * area_mask(:,:,:)
    work2_2d(:,:,:) = (ps(:,:,n4,:)-pb(:,:,n4,:)) * area_mask(:,:,:)
    mass(n3) = global_real_sum(work1_2d, one_2d, size_2d) / tot_area
    mass(n4) = global_real_sum(work2_2d, one_2d, size_2d) / tot_area
    mass_f(n3_f) = global_real_sum(entrainment, area_mask, size_2d) 
    mass_f(n2_f) = global_real_sum(-cumulusmf, area_mask, size_2d) 
    mass_f(n1_f) = global_real_sum(pleak, area_mask, size_2d)
    mass_f = grav*mass_f/tot_area
    rmin = global_real_minval(pthick(:,:,nlm,n4,:),size_2d)
    rmax = global_real_maxval(pthick(:,:,nlm,n4,:),size_2d)
    if (my_task == 0) then
      write(6,45) mass(n3), mass(n4), mass_f(n3_f),mass_f(n2_f),mass_f(n1_f), &
                  (mass(n4)-mass(n3))/dt,sum(mass_f(:)),rmin,rmax
    endif
#endif

!----------------------------------------------------------------------------
! compute globally averaged kinetic energy
!----------------------------------------------------------------------------
    mass(n3) = global_real_sum(ke(:,:,:,:),mass_n3(:,:,:,:),size_3d) / tot_mass
    mass(n4) = global_real_sum(relative(:,:,:,:),  &
                               mass_n3(:,:,:,:),size_3d) / tot_mass
    mass_f(n3_f) = global_real_sum((relative(:,:,:,:))**2, &
                               mass_n3(:,:,:,:),size_3d) / tot_mass
    if (my_task == 0) then
      write(6,46) mass(n3),mass(n4),mass_f(n3_f)
    endif

!----------------------------------------------------------------------------
! compute divergence statistics
!----------------------------------------------------------------------------
    mass(n4) = global_real_sum(div(:,:,:,n4,:),mass_n4(:,:,:,:),size_3d) / tot_mass
    mass(n3) = global_real_sum(div(:,:,:,n3,:),mass_n3(:,:,:,:), &
                               size_3d) / tot_mass
    mass_f(n3_f) = global_real_sum((div(:,:,:,n4,:))**2,mass_n4(:,:,:,:), &
                               size_3d) / tot_mass
    if (my_task == 0) then
      write(6,47) mass(n3),mass(n4),mass_f(n3_f)
    endif
	
!----------------------------------------------------------------------------
! compute potential temperature variance as a function of height
!----------------------------------------------------------------------------
    if (my_task == 0) write(6,148)
    do k=1,nlm
      mass(n4) = global_real_sum(th(:,:,k,n4,:),area_mask(:,:,:), &
                                 size_2d) / tot_area
      mass(n3) = global_real_sum((th(:,:,k,n4,:)-mass(n4))**2,area_mask(:,:,:), &
                                 size_2d) / tot_area
      mass_f(n3_f) = global_real_sum(div(:,:,k,n4,:),area_mask(:,:,:), &
                                 size_2d) / tot_area
      mass_f(n2_f) = global_real_sum((div(:,:,k,n4,:)-mass_f(n3_f))**2, &
                                     area_mask(:,:,:),size_2d) / tot_area
      mass_f(n1_f) = global_real_sum(eta(:,:,k,n4,:),area_mask(:,:,:), &
                                     size_2d) / tot_area
      source = global_real_sum((eta(:,:,k,n4,:)-mass_f(n1_f))**2, &
                               area_mask(:,:,:),size_2d) / tot_area
      if (my_task == 0) then
        write(6,48) k,mass(n4),mass(n3),mass_f(n3_f),mass_f(n2_f), &
                    mass_f(n1_f),source
      endif
    enddo
    if (my_task == 0) write(6,*)

!----------------------------------------------------------------------------
! compute potential temperature budget
!----------------------------------------------------------------------------
    if (my_task == 0) then
      write(6,49)
    endif
    do k=1,nlm
      work3(:,:,k,:) = th_f(:,:,k,n3_f,:) * area_mask(:,:,:)*dsig(k)
      work4(:,:,k,:) = th_f(:,:,k,n2_f,:) * area_mask(:,:,:)*dsig(k)
      work5(:,:,k,:) = th_f(:,:,k,n1_f,:) * area_mask(:,:,:)*dsig(k)
      work6(:,:,k,:) = th_src(:,:,k,:)    * area_mask(:,:,:)*dsig(k) 
      work7(:,:,k,:) = dcadt(:,:,k,:) * area_mask(:,:,:)*dsig(k)
    enddo 
    mass(n3) = global_real_sum(th(:,:,:,n3,:), mass_n3, size_3d) / tot_mass
    mass(n4) = global_real_sum(th(:,:,:,n4,:), mass_n4, size_3d) / tot_mass
    mass_f = c0
!#ifndef fluxcorrectedtransport
    mass_f(n3_f) = global_real_sum(work3, one, size_3d) / tot_mass
    mass_f(n2_f) = global_real_sum(work4, one, size_3d) / tot_mass
    mass_f(n1_f) = global_real_sum(work5, one, size_3d) / tot_mass
!#endif
    source       = global_real_sum(work6, one, size_3d) / tot_mass
    dcadjust     = global_real_sum(work7, one, size_3d) / tot_mass
    fill         = c0
    error = abs((mass(n4)-mass(n3))*invdt -  &
                (w3*mass_f(n3_f)+w2*mass_f(n2_f) +  &
                 w1*mass_f(n1_f)+source+dcadjust+fill))
    if (my_task == 0) then
      write(6,50) mass(n3), mass(n4), mass_f(n3_f), mass_f(n2_f),  &
        mass_f(n1_f), source, dcadjust, fill, (mass(n4)-mass(n3))/dt, &
        w3*mass_f(n3_f)+w2*mass_f(n2_f)+w1*mass_f(n1_f)+source+dcadjust+fill, &
        error
    endif

    do nt=1,ntracer
      if (my_task == 0) then
	write(6,55) tracer_name(nt)
      endif
      do k=1,nlm
        work3(:,:,k,:) = tracer_f(:,:,k,nt,n3_f,:) * area_mask(:,:,:)*dsig(k)
        work4(:,:,k,:) = tracer_f(:,:,k,nt,n2_f,:) * area_mask(:,:,:)*dsig(k)
        work5(:,:,k,:) = tracer_f(:,:,k,nt,n1_f,:) * area_mask(:,:,:)*dsig(k)
        work6(:,:,k,:) = tracer_src(:,:,k,nt,:)    * area_mask(:,:,:)*dsig(k)
        if (nt == iq) then
          work7(:,:,k,:) = dcadq(:,:,k,:) * area_mask(:,:,:)*dsig(k)
	else
          work7 = c0
	endif
	work8(:,:,k,:) = fill_rate(:,:,k,nt,:) * mass_n4(:,:,k,:)
      enddo
      mass(n3) = global_real_sum(tracer(:,:,:,nt,n3,:), mass_n3, size_3d)
      mass(n4) = global_real_sum(tracer(:,:,:,nt,n4,:), mass_n4, size_3d)
      mass = mass/tot_mass
#ifndef fluxcorrectedtransport
      mass_f(n3_f) = global_real_sum(work3, one, size_3d) / tot_mass
      mass_f(n2_f) = global_real_sum(work4, one, size_3d) / tot_mass
      mass_f(n1_f) = global_real_sum(work5, one, size_3d) / tot_mass
#endif
      source       = global_real_sum(work6, one, size_3d) / tot_mass
      if (nt == iq) then
        dcadjust       = global_real_sum(work7, one, size_3d) / tot_mass
      else
        dcadjust = c0
      endif
      fill         = global_real_sum(work8, one, size_3d) / tot_mass
      error = abs((mass(n4)-mass(n3))/dt - &
       (w3*mass_f(n3_f)+w2*mass_f(n2_f)+w1*mass_f(n1_f)+source+dcadjust+fill))
      if (my_task == 0) then
        write(6,50) mass(n3), mass(n4), mass_f(n3_f), mass_f(n2_f), &
           mass_f(n1_f), source, dcadjust, fill, (mass(n4)-mass(n3))/dt, &
           w3*mass_f(n3_f)+w2*mass_f(n2_f)+w1*mass_f(n1_f)+source+dcadjust+fill, &
      	   error
      endif
      glb_trcdiff(nt) = (mass(n4)-mass(n3))/dt
      glb_trcdiff(nt) = glb_trcdiff(nt)*tot_mass
      glb_trcres(nt)  = (w3*mass_f(n3_f)+w2*mass_f(n2_f)+w1*mass_f(n1_f) &
                        * dcadjust+fill)*tot_mass
    enddo

 20 format ( // &
               15x,' |---------------------------------------------------|'/ &
               15x,' |        COMPUTING BUDGETS for DYNAMICAL CORE       |'/ &
               15x,' |---------------------------------------------------|'/ &
               15x,'           TIME       = ', e20.10, ' hours' /  &
               15x,'           STEP_COUNT = ',i8 / )
 30 format ( ' MASS MEASURED USING PTHICK (Pa) ' /  &
                       '  @ n3        : ',e20.10, ' (Pa)' /  &
                       '  @ n4        : ',e20.10, ' (Pa)' /)
 35 format ( ' MASS MEASURED USING SURFACE PRESSURE (Pa) ' / &
                       '  @ n3        : ',e20.10, ' (Pa)' / &
                       '  @ n4        : ',e20.10, ' (Pa)' /)
 40 format ( ' MASS TENDENCY USING d(PS)/dt (Pa/s) ' / &
                       '  @ n3_f      : ',e20.10, ' (Pa/s)' / &
                       '  @ n2_f      : ',e20.10, ' (Pa/s)' / &
                       '  @ n1_f      : ',e20.10, ' (Pa/s)' / )
 46 format ( ' GLOBALLY AVERAGED KINETIC ENERGY PER UNIT MASS '/ &
     			'  kinetic energy @ n4     : ',e20.10, ' (m2/s2)' / &
                        '  relative vorticity @ n4 : ',e20.10, ' (1/s)' / &
                        '  entrophy @ n4           : ',e20.10, ' (1/s2)' / )
 47 format ( ' GLOBALLY AVERAGED DIVERGENCE DIAGNOSTICS '/ &
                        '  average divergence @ n4  : ',e20.10, ' (1/s)'/ &
                        '  average divergence @ n3  : ',e20.10, ' (1/s)' / &
                        '  divergence variance @ n4 : ',e20.10, ' (1/s2)' /)
 45 format ( ' PBL THICKNESS (Pa) and MASS FLUXES (Pa/s) ' / &
                        '  @ n3        : ',e20.10, ' (Pa)' / &
                        '  @ n4        : ',e20.10, ' (Pa)' / &
                        '  entrainment : ',e20.10, ' (Pa/s)' / &
                        '  cumulusmf   : ',e20.10, ' (Pa/s)' / &
                        '  pleak       : ',e20.10, ' (Pa/s)' / &
                        '  d(pb)/dt    : ',e20.10, ' (Pa/s)' / &
                        '  sum(e+c+p)  : ',e20.10, ' (Pa/s)' / &
                        '  minval      : ',e20.10, ' (Pa)  ' / &
                        '  maxval      : ',e20.10, ' (Pa)  ' /) 
 148 format (15x,'|------------------------------------------------------|'/,  &
             15x,'| MEAN and VARIANCE of THETA, DIVERGENCE and VORTICITY |'/,  &
             15x,'|------------------------------------------------------|'/,  &
                 '   k      mean(th)      var(th)      mean(div)     ',        &
                 ' var(div)     mean(eta)    var(eta)')     
 48  format (i4,6e14.4)
 49  format ( ' POTENTIAL TEMPERATURE BUDGET (K) and (K/s) ')
 50  format (        '  @ n3              : ',e20.10, ' (q)' / &
                     '  @ n4              : ',e20.10, ' (q)' / &
                     '  @ n3_f            : ',e20.10, ' (q*Pa/s)' / &
                     '  @ n2_f            : ',e20.10, ' (q*Pa/s)' / &
                     '  @ n1_f            : ',e20.10, ' (q*Pa/s)' / &
                     '  sources           : ',e20.10, ' (q*Pa/s)' / &
                     '  dcadj             : ',e20.10, ' (q*Pa/s)' / &
                     '  fill              : ',e20.10, ' (q*Pa/s)' / &
                     '  n4-n3             : ',e20.10, ' (q*Pa/s)' / &
                     '  sum(tendencies)   : ',e20.10, ' (q*Pa/s)' / &
                     '  abs(error)        : ',e20.10, ' (q*Pa/s)' / ) 
 55  format (a80)

  end subroutine compute_global_budgets

end module compute_budgets
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
