!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module domain_decomposition


!***********************************************************************
!
!	this module defines how the horizontal domain is decomposed
!	it also contains mpi specific information
!
!***********************************************************************

!-------------------------------------------------------------------------
!  VARIABLE DEFINITIONS
!
!	iim		= number of cells in i-direction on one panel
!	jjm		= number of cells in j-direction on one panel
!	isdm		= number of blocks in the i-direction on ONE panel
!			  must be an even number
!	jsdm		= number of blocks in the j-direction on ONE panel
!	npe_atmos	= number of processes for AGCM
!	sbdmn_max	= mask number subdomains
!	nsdm		= number of subdomains each process owns
!	ipad		= padding in the i-direction
!	jpad		= padding in the j-direction
!-------------------------------------------------------------------------

      use kinds
      use model_parameters

      implicit none
      save

#ifdef mpi 
      include 'mpif.h'
#endif

#ifdef mpi 
        integer (kind=int_kind), parameter ::                         &
                        isdm = 4,                                     &
! isdm = 2**partition_factor - used by fct
                        partition_factor=2,                           &
                        jsdm = isdm/2,                                &
                        npe_atmos = 002,                              &
                        sbdmn_max = 5*isdm*jsdm,                      &
                        nsdm = sbdmn_max/npe_atmos,                   &
                        ipad = 1,                                     & ! code only valid for ipad=1
                        jpad = 1,                                     & ! code only valid for jpad=1
                        iim = (im5 - 2)/isdm + 2*ipad,                &
                        jjm = (jm  - 2)/jsdm + 2*jpad
#else

        integer (kind=int_kind), parameter ::                         &
                        isdm = 2,                                     &
! isdm = 2**partition_factor - used by fct
                        partition_factor=1,                           &
                        jsdm = isdm/2,                                &
                        npe_atmos = 001,                              &
                        sbdmn_max = 5*isdm*jsdm,                      &
                        nsdm = sbdmn_max/npe_atmos,                   &
                        ipad = 1,                                     &
                        jpad = 1,                                     &
                        iim = (im5 - 2)/isdm + 2*ipad,                &
                        jjm = (jm  - 2)/jsdm + 2*jpad

#endif

        integer (kind=int_kind), parameter ::                         &
                        iimjjm = iim * jjm


#ifdef mpi
      integer (kind=int_kind) ::                                      &
                         ierr,                                        &
                         atmos_group,                                 &
                         MPI_COMM_ATMOS,                              &
                         MPI_GROUP_WORLD,                             &
                         mpi_float_type,                              &
		         mpi_int_type,                                &
                         my_task,                                     &
                         itask

#if ibm == 1
	integer (kind=int_kind) :: status(MPI_STATUS_SIZE)
#else
	integer (kind=int_kind) :: status
#endif

#else
      integer (kind=int_kind) ::                                      &
                         MPI_COMM_ATMOS=0,                            &
                         mpi_float_type=0,                            &
		         mpi_int_type=0
      integer (kind=int_kind), parameter :: my_task = 0
      integer (kind=int_kind) :: itask = 0
#endif


!***********************************************************************
#ifdef mpi

      contains

!***********************************************************************


!======================================================================
! BEGINNING OF SPECIFY_MPI_COMMUNICATOR
!======================================================================
        subroutine specify_mpi_communicator

!-------------------------------------------------------------------------
! LOCAL
!-------------------------------------------------------------------------
        integer i, pro_ranks(npe_atmos)

!-------------------------------------------------------------------------
! initialize MPI
!-------------------------------------------------------------------------
        call MPI_INIT(ierr)


!-------------------------------------------------------------------------
! define processor rank
!-------------------------------------------------------------------------
        do i = 1,npe_atmos
           pro_ranks (i) = i-1
        enddo

!-------------------------------------------------------------------------
! generate the MPI_COMM_ATMOS communicator
!-------------------------------------------------------------------------
        call MPI_COMM_GROUP(                &
             MPI_COMM_WORLD,                &
             MPI_GROUP_WORLD,               &
             ierr)

        call MPI_GROUP_INCL(                &
             MPI_GROUP_WORLD,               &
             npe_atmos,                     &
             pro_ranks(:),                  &
             atmos_group,                   &
             ierr)

        call MPI_COMM_CREATE(               &
             MPI_COMM_WORLD,                &
             atmos_group,                   &
             MPI_COMM_ATMOS,                &
             ierr)

        call MPI_COMM_RANK (MPI_COMM_ATMOS, my_task,ierr)
!       call MPI_COMM_SIZE (MPI_COMM_ATMOS, npe_atmos,ierr )

	end subroutine specify_mpi_communicator
!======================================================================
! END OF SPECIFY_MPI_COMMUNICATOR
!======================================================================


!======================================================================
! BEGINNING OF SPECIFY_MPI_PRECISION
!======================================================================
	subroutine specify_mpi_precision

!-------------------------------------------------------------------------
!  PURPOSE:  specify the type of messages. single precision
!	or double precision for floating. also type of integer
!-------------------------------------------------------------------------
        implicit none
        integer (kind=int_kind) :: errorcode, ierror

        if(dbl_kind == selected_real_kind(6)) then
           mpi_float_type = MPI_REAL
        elseif (dbl_kind == selected_real_kind(13)) then
           mpi_float_type = MPI_DOUBLE_PRECISiON
        else
           if(my_task == 0) then
             write(6,*)
             write(6,*) ' stop in specify_mpi_precision'
             write(6,*) ' mpi_precision of real numbers unknown'
             write(6,*)
             CALL MPI_ABORT(MPI_COMM_ATMOS, errorcode, ierror)
           endif
        endif

 	if(int_kind == kind(1)) then
 	  mpi_int_type = MPI_INTEGER
 	else
 	  if(my_task == 0) then
             write(6,*)
             write(6,*) ' stop in specify_mpi_precision'
 	     write(6,*) ' unknown integer type'
 	     CALL MPI_ABORT(MPI_COMM_ATMOS, errorcode, ierror)
 	  endif
 	endif

	end subroutine specify_mpi_precision
!======================================================================
! END OF SPECIFY_MPI_PRECISION
!======================================================================
#endif

	end module domain_decomposition

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
