!======================================================================
! PROGRAM BUGS
!======================================================================

#ifdef no_pbl
	program bugs
#else

#ifdef atm_component
        subroutine bugs( comp_init
     &,                  sst_prog
     &,                  down_heat_flux
     &,                  down_water_flux
     &,                  stress)
#else
	program bugs
#endif

#endif

!-----------------------------------------------------------------------
! PURPOSE : driver for the dynamical core and full gcm
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! DYNAMICAL CORE : LOG, LINEAGE and SUPPORT
!
!	This numerical model is not supported. Feel free to use it for
!	  academic purposes. If think you have found an error in the
!	  code, please inform us. We welcome any comments you might have
!	  on any aspect of the model.
!
!	We have put substantial effort into making this model readable
!	  and understandable. We have done this by using Fortran 90
!	  modules, defining variables, and using many comments. Anyone
!	  who takes the time to study the code and read the background
!	  papers should be able to use the model without many problems.
!
!	The original version of this SWM was written my Ross Heikes
!	  as a part of a Master's Thesis at CSU (1993). The basic
!	  2nd order accurate operators (Laplacian, Flux-Diverengce, and
!	  Jacobian) were included in the original version
!
!	While the original papers (listed below) use the twisted
!	  icosahedal grid, this swm does not used the twisted grid, it
!	  uses the spherical geodesic grid without the twisting.
!
!	The code was largely rewritten by Todd Ringler (1999) to form
!	  the basis of a dynamical core. Various operators related to
!	  calculating fluxes at cell walls were added.
!
!	A domain decomposition algorithm was implemented to make use
!	  of massively parallel computers. June 2000.
!
!       A new dynamical core based on Ringler and Randall (2002a,b)
!         was implement in April 2002.
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! DYNAMICAL CORE : REFERENCES 
!   on this swallow water model 
!	Heikes, R., 1993: The shallow water equations on a spherical 
!	   geodesic grid, Department of Atmospheric Science, Colorado 
!	   State University, Technical Report 524, pp. 175. 
!
!	Heikes, R. and R. A. Randall, 1995a: Numerical integration 
!	  of the shallow-water equations on a twisted icosahedral 
!	  grid. Part I: Basic design and results of tests. Mon. Wea. 
!	  Rev., 123, 1862-1880.
!
!	Heikes, R. and R. A. Randall, 1995b: Numerical integration of 
!	  the shallow-water equations on a twisted icosahedral grid. 
!	  Part II: A detailed description of the grid and an analysis 
!	  of numerical accuracy. Mon. Wea. Rev., 123, 1881-1887.
!
!   some relevant paper preceding the work of Heikes and Randall
!	Masuda, Y., and H. Ohnishi, 1986: An integration scheme of 
!	  the primitive equations model with an icosahedral-hexagonal 
!	  grid system and its application to the shallow water equations. 
!	  Short- and Medium-Range Numerical Weather Prediction. 
!	  Japan Meteorological Society, Tokyo, 317-326.
!
!	Sadourny, R., A. Arakawa, and Y. Mintz, 1968: Integration of the 
!	  nondivergent barotropic vorticity equation with an icosahedal-
!	  hexagonal grid for the sphere. Mon. Wea. Rev., 96, 351-356.
!
!	Williamson, D. L., 1968: Integration of the barotropic vorticity equation 
!	  on a spherical geodesic grid. Tellus, 20, 642-653.
!
!   work which uses this swm as a basis for further development
!	Ringler, T.D, R.P. Heikes, and D.A. Randall, 1998: Solving 
!	  the primitive equations on a spherical geodesic grid: A 
!	  technical report to a new class of dynamical cores. Department 
!	  of Atmospheric Science, Colorado State University, 
!	  Technical Report 665, pp. 62.
!
!	Ringler, T.D. and D.A. Randall, 2002a: A Potential Enstrophy
!	   and Energy Conserving Numerical Scheme for Solution of
!	   the Shallow-Water Equations on a Geodesic Grid. Mon. Wea.
!	   Rev., 130, 1397-1410.
!
!       Ringler, T.D. and D.A. Randall, 2002b: The ZM-grid: An
!	   Alternative to the Z-grid. Mon. Wea. Rev., 130, 1411-1422.
!-----------------------------------------------------------------------


	use physical_parameters
	use model_parameters
	use domain_decomposition
#ifndef no_pbl
	use domain_decomposition_sfc
#endif
      use sigma_coordinate
	use model_control
	use grid_mask
	use operators
	use grid_metrics
	use interpolate_to_latlon
	use timer
	use renormalization
	use initialize
	use source_terms
	use pre_step_module
	use step
	use post_step_module
	use adjustments
	use compute_budgets
	use qp_module
	use qp_ml
	use pbp_module
	use qp_pbp_diagnostics_module
	use gmeans_module

#ifndef no_pbl
	use eau_budget
#ifdef link_to_gcm_physics
	use qp_module_sfc
	use pbp_module_sfc
      use bc_update
	use flux_coupler_module
      use sfc_control_module
      use fluxcoupler_fields
	use initialize_pcontrol
	use sfc_restart
	use sfc_init
#endif
#endif

	implicit none
#ifndef no_pbl
#ifdef atm_component
        logical (kind=log_kind), intent(in) :: comp_init
        real (kind=dbl_kind), intent(out) ::
     &     down_heat_flux(iim,jjm,nsdm)
     &,    down_water_flux(iim,jjm,nsdm)
     &,    stress(nspace,ncorners,iim,jjm,nsdm)
        real(kind=dbl_kind), intent(in) :: sst_prog(iim,jjm,nsdm)
#endif
#endif

        integer (kind=int_kind) :: i, j  ! loop indices
	integer (kind=int_kind) :: k
        logical (kind=log_kind) :: fc_restart=.true.

#ifndef no_pbl
#ifdef atm_component
        if(comp_init) then
#endif
#endif

!-------------------------------------------------------------------------
! INITIALIZATION MPI if NECESSARY
!-------------------------------------------------------------------------
	call timer_start('total')
	call timer_start('model init')
#ifdef mpi
	call specify_mpi_communicator
#ifndef no_pbl
	call specify_mpi_sfc_communicator
#endif
	call specify_mpi_precision
#endif

!-------------------------------------------------------------------------
! INITIALIZATION MODEL STATE
!-------------------------------------------------------------------------
	call initialize_model

!-------------------------------------------------------------------------
! update time
!-------------------------------------------------------------------------
	  call timer_start('model crl')
          call time_manager

!------------------------------------------------------------------
! based on the time step, get pointers
!------------------------------------------------------------------
          call get_pointers
	  call timer_stop('model crl')

#ifndef no_pbl
#ifdef link_to_gcm_physics

!------------------------------------------------------------------
! initialize pcontrol: init cumulus kinetic energy and climatolical
!  ozone
!------------------------------------------------------------------
         call init_pcontrol

!------------------------------------------------------------------
! initialize the flux coupler
!------------------------------------------------------------------
         call init_flux_coupler

!------------------------------------------------------------------
! compute the time varying boundary conditions
!------------------------------------------------------------------
#ifdef atm_component
 	call bc_update_manager(sst_prog)
#else
 	call bc_update_manager
#endif

!------------------------------------------------------------------
! call the flux coupler to compute diagnostic information such
!  as stratus cloudiness and surface albedo needed for the initial
!  call to pcontrol
!------------------------------------------------------------------
         call flux_coupler(fc_restart)
#endif
#endif

!-----------------------------------------------------------------------
! initialize the qp and pbp table information
!-----------------------------------------------------------------------
        call initialize_pbpinfo
#ifndef no_pbl
#ifdef link_to_gcm_physics
        call initialize_pbpinfo_sfc
#endif
#endif

        call initialize_qpinfo
        call initialize_qpmlinfo
#ifndef no_pbl
#ifdef link_to_gcm_physics
        call initialize_qpinfo_sfc
#endif
#endif

!-----------------------------------------------------------------------
! initialize the qp and pbp files
!-----------------------------------------------------------------------

!  for the atmosphere
        call initialize_pbpfiles(simulation_tag)
#ifndef no_pbl
#ifdef link_to_gcm_physics
        call initialize_pbpfiles_sfc(simulation_tag)
#endif
#endif

        call initialize_qpfiles(simulation_tag, grid_center_mask)
        call initialize_qpmlfiles(simulation_tag, grid_center_mask)

!  for the surface
#ifndef no_pbl
#ifdef link_to_gcm_physics
        call initialize_qpfiles_sfc(simulation_tag)
#endif
#endif

!-----------------------------------------------------------------------
! initialize the global mean file
!-----------------------------------------------------------------------
	call initialize_globinfo

        call timer_stop('model init')
#ifndef no_pbl
#ifdef atm_component
          down_heat_flux = 0.0_dbl_kind
          down_water_flux = 0.0_dbl_kind
!      do ksdm = 1,nsdm
!        call average_to_cell_corners(
!     *                  ipad,
!     *                  iim,
!     *                  jjm,
!     *                  ndata,
!     *                  ncorners,
!     *                  s_weights(:,:,:,:,ksdm),
!     *                  drag_atm(:,:,2,ksdm),  
!     *                  drag_c(:,:,:,ksdm))  

!        enddo
!	call wrap_vertices(1,1,drag_c       (:,:,:,:))
          drag_c = 1.e-4  ! constant
          stress(1,:,:,:,:) = drag_c(:,:,:,:) * u(1,:,:,:,nlm,n4,:) 
          stress(2,:,:,:,:) = drag_c(:,:,:,:) * u(2,:,:,:,nlm,n4,:) 
        endif   ! if(comp_init) then
#endif
#endif


!-------------------------------------------------------------------------
! INTEGRATION : stop model based on logical halt
!-------------------------------------------------------------------------
#ifdef no_pbl
        do while(.not.l_halt)
#else
#ifdef atm_component
        if(.not.comp_init) then
#else
        do while(.not.l_halt)
#endif
#endif

!-------------------------------------------------------------------------
! compute qp interpolation weights for this timestep and update counter
!-------------------------------------------------------------------------
          call timer_start('qp')
	  call setqp_wrap
#ifndef no_pbl
#ifdef link_to_gcm_physics
	  call qp_sfc_increment
#endif
#endif
          call timer_stop('qp')

!-------------------------------------------------------------------------
! initialize sources
!-------------------------------------------------------------------------
          call timer_start('sources')
	  call init_sources
          call timer_stop('sources')

#ifndef no_pbl
#ifdef link_to_gcm_physics
!-------------------------------------------------------------------------
! compute the physics time rates of change
!   pcontrol uses time level n3, pthick(n3)*dsig (or plgl2, etc.)
!   can avoid "ghost cells" by masking with grid_center_mask(iim,jjm,ksmd)
!-------------------------------------------------------------------------
#ifdef eau_ng
          call pcontrol_ng
#else
          call pcontrol
#endif

!-------------------------------------------------------------------------
! compute the surface physics, get surface fluxes
!-------------------------------------------------------------------------
	  call flux_coupler(fc_restart)
#endif
#endif

!-------------------------------------------------------------------------
! compute diagnostic variable ect
!-------------------------------------------------------------------------
	  call pre_step

!-------------------------------------------------------------------------
! compute sources (like del4 diffusion) based on time level n3
!   note: get_sources is a driver for computing lots of stuff
!   check out the code
!-------------------------------------------------------------------------
          call timer_start('sources')
          call get_sources
          call timer_stop('sources')

!-------------------------------------------------------------------------
! move forward a time step
!-------------------------------------------------------------------------
 	  call step_dt(step_count)

!-------------------------------------------------------------------------
! make adjustments of state of atmosphere (like dcadj and hole filling)
!-------------------------------------------------------------------------
          call timer_start('adjust')
 	  call make_adjustments
          call timer_stop('adjust')

!-----------------------------------------------------------------
! wrap the data
!-----------------------------------------------------------------
        call timer_start('wrap')
        call wrap_prognostics
        call timer_stop('wrap')

!-----------------------------------------------------------------
! accumulate time average (qp) and time series (pbp) diagnostics
!-----------------------------------------------------------------
        call timer_start('qp')
	call qp_pbp_diagnostics(nsecond_dyn)
        call timer_stop('qp')

!-------------------------------------------------------------------------
! output results based on logicals
!-------------------------------------------------------------------------
          call timer_start('model crl')
	  if(l_renorm)	    call renormalize
          if(l_restart) then
	     call renormalize
             call write_restart
             fc_restart = .true.
#ifndef no_pbl
#ifdef link_to_gcm_physics
             call write_restart_physics
             call write_restart_ocean
             call write_restart_landice
             call write_restart_sib
             call write_restart_pbl
#endif
#endif
          endif

!-----------------------------------------------------------------
! compute diagnostically-derived quantities based on new time step
!-----------------------------------------------------------------
	call post_step

!-----------------------------------------------------------------------
! update fields that may be needed in the flux coupler
!-----------------------------------------------------------------------
#ifndef no_pbl
        spdm_atm(:,:,:) = spdm(:,:,:)
        shear1_atm(:,:,:) = shear1(:,:,:)
#endif

          if(l_output)      call write_prognostics
!         if(l_output)      call write_sib_prognostics

          if(l_diagnostics) call compute_global_budgets
#ifndef no_pbl
#ifdef link_to_gcm_physics
          if(l_diagnostics) call eau_conserve_mass
#endif
#endif

          if(l_diagnostics) call write_gmean_diagnostics

          call timer_stop('model crl')
          if(l_timer) then
             call timer_stop('total')
             call timer_write ()
             call timer_start('total')
          endif

          call timer_start('qp')
	  if(l_qp) then
	    call write_qp_diagnostics(grid_center_mask)
	    call write_qpml_diagnostics(grid_center_mask)
#ifndef no_pbl
#ifdef link_to_gcm_physics
	    call write_qp_diagnostics_sfc
#endif
#endif
          endif
 	  if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
             call write_hf_diagnostics(grid_center_mask)
#ifndef no_pbl
#ifdef link_to_gcm_physics
             call write_hf_diagnostics_sfc
#endif
#endif
          endif
#ifndef no_pbl
          if(mod(nsecond_dyn,86400).eq.0) call write_mjo_diagnostics
#endif
          call timer_stop('qp')

	  if(l_pbp) call write_pbp_diagnostics
#ifndef no_pbl
#ifdef link_to_gcm_physics
	  if(l_pbp) call write_pbp_diagnostics_sfc
#endif
#endif

!-------------------------------------------------------------------------
! update time
!-------------------------------------------------------------------------
          call timer_start('model crl')
          call time_manager

!------------------------------------------------------------------
! based on the time step, get pointers
!------------------------------------------------------------------
          call get_pointers
          call timer_stop('model crl')

#ifndef no_pbl
#ifdef atm_component
          down_heat_flux = radvdc_atm*(1._dbl_kind - alvdf_atm)
     &                   + radndc_atm*(1._dbl_kind - alndf_atm)
     &                   + radvbc_atm*(1._dbl_kind - alndr_atm)
     &                   + radnbc_atm*(1._dbl_kind - alndr_atm)
     &                   + dlwbot_atm -fss_atm -hltm*fws_atm
     &                   - stefan*sst_prog**4
          down_water_flux = dcupr_atm + dlspr_atm + dlsps_atm - fws_atm
      do ksdm = 1,nsdm
        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  drag_atm(:,:,2,ksdm),  
     *                  drag_c(:,:,:,ksdm))  

        enddo
	call wrap_vertices(1,1,drag_c       (:,:,:,:))
          stress(1,:,:,:,:) = drag_c(:,:,:,:) * u(1,:,:,:,nlm,n4,:) 
          stress(2,:,:,:,:) = drag_c(:,:,:,:) * u(2,:,:,:,nlm,n4,:) 
#endif
#endif
!------------------------------------------------------------------
! based on the time step, update the boundary conditions
!------------------------------------------------------------------
#ifndef no_pbl
#ifdef link_to_gcm_physics
          if(l_bc_update) call bc_update_manager

!------------------------------------------------------------------
! if we wrote a restart, recompute albedo and stratus for restartability
!------------------------------------------------------------------
	  if(fc_restart) call flux_coupler(fc_restart)
#endif
#endif

#ifdef no_pbl
        enddo   ! do while(.not.l_halt)

!-------------------------------------------------------------------------
! WRITE OUT TIMING
!-------------------------------------------------------------------------
	call timer_write
        call timer_stop('total')

!-------------------------------------------------------------------------
! CLOSE MPI
!-------------------------------------------------------------------------
#ifdef mpi
        call MPI_FINALIZE(ierr )
#endif
#else
#ifdef atm_component
        endif    ! if(.not.comp_init)
#else
        enddo   ! do while(.not.l_halt)

!-------------------------------------------------------------------------
! WRITE OUT TIMING
!-------------------------------------------------------------------------
	call timer_write
        call timer_stop('total')

!-------------------------------------------------------------------------
! CLOSE MPI
!-------------------------------------------------------------------------
#ifdef mpi
        call MPI_FINALIZE(ierr )
#endif
#endif
#endif

#ifdef no_pbl
	end program bugs
#else
#ifdef atm_component
	end subroutine bugs
#else
	end program bugs
#endif
#endif

!======================================================================
! END OF PROGRAM BUGS
!======================================================================
