!==============================================================================

 module eau_cldproperties

 use kinds
 use eau_params

 implicit none

 private
 public:: eau_cf, eau_effr, eau_ncw

!==============================================================================
!eau_cldproperties contains the subroutines that calculate the diagnostic cloud
!fraction and cloud optical properties needed by rad_control.
!Laura D. Fowler/mcbug (11-12-03).
!send comments to laura@mcbug.atmos.colostate.edu

!references:
!-----------
!Fowler,L.D.,and D.A. Randall,1996:Liquid and ice cloud microphysics in the 
!CSU General Circulation Model. Part II: Impact on cloudiness, the Earth's
!radiation budget, and the general circulation of the atmosphere.
!J. Climate,9,489-529.

!subroutines called:
!   eau_cf :computes the horizontal fraction of large-scale supersaturation and
!           anvil clouds as functions of the liquid, ice, and snow water paths.
!           This subroutine will disappear when the parameterization of prognos
!           tic cloud fraction is included.
!  eau_effr:calculates the effective radii of cloud water drops and cloud ice
!           crystals.
!   eau_ncw:specifies different droplet-concentration of warm clouds between
!           land and oceans.
! Laura D. Fowler/slikrock (06-07-00).

!==============================================================================

 contains

!==============================================================================
      
 subroutine eau_cf &
                ( nlen ,   len ,    nlm , &
                  npbl , cldpc ,    plg , &
                    tl ,    cw ,     ci , &
                 dlcup , dicup ,    clu , &
                   cld ,  cldt ,   cldw , &
                  cldi ,  cldm , cldt2d   &
                )

 use kinds
 use eau_params

 implicit none
!------------------------------------------------------------------------------
!eau_cf computes the horizontal fraction of large-scale supersaturation and
!anvil clouds as functions of the liquid, ice, and snow water content. This sub
!routine will disappear when the parameterization of prognostic cloud fraction
!is included.
!Laura D. Fowler/mcbug (06-07-00).
!send comments to laura@atmos.colostate.edu

!references:
!-----------
!Fowler,L.D.,and D.A. Randall,1996:Liquid and ice cloud microphysics in the 
!CSU General Circulation Model. Part II: Impact on cloudiness, the Earth's
!radiation budget, and the general circulation of the atmosphere.
!J. Climate,9,489-529.

!subroutines called:
!   none.

!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    nlen,            &!length of total domain.
    len,             &!length of sub-domain.
    nlm,             &!number of layers.
    npbl              !number of layers in the PBL.

 real (kind=dbl_kind), intent(in), dimension(nlen):: &
    cldpc             !fraction of PBL clouds                              (-).

 real (kind=dbl_kind), dimension(nlen,nlm):: &
    cw,              &!cloud water mixing ratio                        (kg/kg).
    ci,              &!cloud ice mixing ratio                          (kg/kg).
    tl,              &!temperature                                         (K).
    plg,             &!odd-level pressure                                 (Pa).
    dlcup,           &!cumulus detrainment of cloud water            (kg/kg/s).
    dicup             !cumulus detrainment of cloud ice              (kg/kg/s).

!output arguments:
!-----------------
 real (kind=dbl_kind), dimension(nlen):: &
    cldt2d            !vertically integrated cloud fraction                (-).

 real (kind=dbl_kind), dimension(nlen,nlm):: &
    cld,             &!large-scale saturation cloud fraction               (-).
    clu,             &!anvil cloud fraction                                (-).
    cldw,            &!fraction of water clouds                            (-).
    cldi,            &!fraction of ice and snow clouds                     (-).
    cldm,            &!fration of mixed-phase clouds                       (-).
    cldt              !total cloud fraction                                (-).
     
! local variables:

 integer (kind=int_kind):: i, l, lpbl
 real (kind=dbl_kind), dimension(nlen,nlm):: totwc, totwc0, totwc1

!------------------------------------------------------------------------------
 lpbl = nlm-npbl

!initialization of local and output variables:
 cldt2d(1:len)       = c0

 totwc(1:len,1:nlm)  = c0 ; totwc0(1:len,1:nlm) = c0
 totwc1(1:len,1:nlm) = c0 ; cld(1:len,1:nlm)    = c0
 clu(1:len,1:nlm)    = c0 ; cldt(1:len,1:nlm)   = c0
 cldw(1:len,1:nlm)   = c0 ; cldi(1:len,1:nlm)   = c0
 cldm(1:len,1:nlm)   = c0

!discrimination between large-scale saturation clouds and anvils:
 totwc(1:len,1:lpbl) = cw(1:len,1:lpbl)+ci(1:len,1:lpbl)
 do l = 1, lpbl
    do i = 1, len
       if(totwc(i,l) .gt. 1.e-05) then
          if(dlcup(i,l).gt.0. .or. dicup(i,l).gt.0.) then
             totwc0(i,l) = cw(i,l)+ci(i,l)
          else
             totwc1(i,l) = cw(i,l)+ci(i,l)
          endif
       endif
    enddo
 enddo
      
!computation of the cloud fraction:
 do i = 1, len
    do l = 1, lpbl
#ifdef no_stratosphere
       if(totwc0(i,l) > c0 .and. plg(i,l) >= 7000.0_dbl_kind) &
          clu(i,l) = c1
       if(totwc1(i,l) > c0 .and. plg(i,l) >= 7000.0_dbl_kind) &
          cld(i,l) = c1
#else
       if(totwc0(i,l) > c0) clu(i,l) = c1
       if(totwc1(i,l) > c0) cld(i,l) = c1
#endif
    enddo
    do l = lpbl+1, nlm
       cld(i,l) = cldpc(i)
    enddo
 enddo
      
!computation of the vertically-integrated cloud fraction, and fraction of water
!ice, or mixed-phase clouds:
 do l = 1, nlm
    do i = 1, len
       cldt(i,l) = max(cld(i,l),clu(i,l))
       if(tl(i,l) .ge. tbgmax) cldw(i,l) = cldt(i,l)
       if(tl(i,l) .lt. tbgmin) cldi(i,l) = cldt(i,l)
       if(tl(i,l).ge.tbgmin .and. tl(i,l).lt.tbgmax) cldm(i,l) = cldt(i,l)
    enddo
 enddo
 do l = 1, nlm
    cldt2d(1:len) = max(cldt2d(1:len),cldt(1:len,l))
 enddo

 end subroutine eau_cf
      
!-------------------------------------------------------------------------------
 subroutine eau_effr &
                ( nlen ,   len ,   nlm , &
                  npbl ,   ncw ,   plg , &
                    tl ,    cw ,    ci , &
                 pblcw , pblci ,  cldt , &
                 effrw , effri           &
                )

 use kinds
 use eau_params

 implicit none      
!------------------------------------------------------------------------------
!eau_effr calculates the effective radii of cloud water droplets and cloud ice
!crystals.
!Laura D. Fowler/slikrock (06-07-00).
!send comments to laura@atmos.colostate.edu

!subroutines called:
!   none.

!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    nlen,            &!length of total domain. 
    len,             &!length of sub-domain.
    nlm,             &!number of layers.
    npbl              !number of layers in the PBL.

 real (kind=dbl_kind), intent(in), dimension(nlen):: &
    ncw               !droplet concentration of water clouds          (per m3).

 real (kind=dbl_kind), intent(in), dimension(nlen,npbl):: &
    pblcw,           &!pbl cloud water mixing ratio                    (kg/kg).
    pblci             !pbl cloud ice mixing ratio                      (kg/kg).

 real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
    plg,             &!pressure                                          (hPa).
    tl,              &!temperature                                         (K).
    cw,              &!cloud water mixing ratio                        (kg/kg).
    ci,              &!cloud ice mixing ratio                          (kg/kg).
    cldt              !total cloud fraction                                (-).

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
    effrw,           &!effective radius of water and mixed-phase clouds   (mu).
    effri             !effective radius of ice clouds                     (mu).

! local variables:
 integer (kind=int_kind):: i, l, ll, lpbl

 real (kind=dbl_kind):: &
    den_w,           &!density of water                                (kg/m3).
    k_w,             &!constant                                            (-).
    const,           &!3/4*pi                                              (-).
    undef             !missing value
 data const,den_w, k_w, undef /2.3561945,1.e03,1.,1.e36/

 real (kind=dbl_kind), dimension(len,nlm):: &
    cont_w,          &!liquid water content                            (kg/m3).
    cont_i,          &!lce plus snow content                           (kg/m3).
    den               !air density                                     (kg/m3).

!------------------------------------------------------------------------------

 lpbl = nlm-npbl

!initialization of local and output variables:
 den(:,:)    = c0 ; cont_w(:,:) = c0
 cont_i(:,:) = c0
 
 effrw(1:len,1:nlm) = c0 ; effri(1:len,1:nlm) = c0

!effective radius of water and mixed phase clouds following Martin et al.
!(1994).
 do i = 1, len
    do l = 1, lpbl
       den(i,l)    = plg(i,l)/(287.*tl(i,l))
       cont_w(i,l) = den(i,l)*cw(i,l)*cldt(i,l)
    enddo
    do l = lpbl+1, nlm
       ll = l-lpbl    
       den(i,l)    = plg(i,l)/(287.*tl(i,l))
       cont_w(i,l) = den(i,l)*(cw(i,l)+pblcw(i,ll))*cldt(i,l)
    enddo
    do l = 1, nlm       
!      if(ncw(i) .ne. undef) then
!         effrw(i,l) = cont_w(i,l)/(den_w*k_w*ncw(i))
!         effrw(i,l) = (const*effrw(i,l))**0.3333
!         effrw(i,l) = effrw(i,l)*1.e06
!      endif
       if(cont_w(i,l) > c0) effrw(i,l) = 10.0_dbl_kind
    enddo
enddo

!effective radius of ice clouds following rotstayn (1997) and platt (1994,
!1997):
 do i = 1, len
    do l = 1, lpbl
       den(i,l)    = plg(i,l)/(287.0_dbl_kind*tl(i,l))
       cont_i(i,l) = den(i,l)*ci(i,l)*cldt(i,l)
    enddo
    do l = lpbl+1, nlm
       ll = l-lpbl
       den(i,l)    = plg(i,l)/(287.0_dbl_kind*tl(i,l))
       cont_i(i,l) = den(i,l)*(ci(i,l)+pblci(i,ll))*cldt(i,l)
    enddo
    do l = 1, nlm
!      effri(i,l) = 0.051*cont_i(i,l)**0.667
!      effri(i,l) = effri(i,l)*1.e06
       if(cont_i(i,l) > c0) effri(i,l) = 30.0_dbl_kind
    enddo
 enddo 

 end subroutine eau_effr
      
!------------------------------------------------------------------------------
 subroutine eau_ncw  &
                     (  nlen  ,     len , log_sea, &
                      log_ice , log_sib ,     ncw  &
                     )

 use kinds

 implicit none
      
!-----------------------------------------------------------------------

! eau_ncw specifies different droplet-concentration of warm clouds
! between land and oceans.
! Laura D. Fowler/slikrock (06-07-00).

! references:
! Martin, G.M., D.W. Johnson, and A. Spice, 1994: The measurement and
! parameterization of effective radius of droplets in warm stratocumulus
! clouds.
! J. Atmos. Sci., 51, 1823-1842.

! Rotstayn, L.D., 1997: A physically based scheme for the treatment of
! stratiform clouds and precipitation in large-scale models. I: Descrip
! tion and evaluation of the microphysical processes.
! Q.J.R. Meteorol. Soc., 123, 1227-1282.

! modifications:
!     none.

! subroutines called:
!     none.

! argument list variables:
!     input arguments:
!     ----------------

 integer (kind=int_kind), intent(in):: &
   nlen, &     !length of horizontal domain.
   len         !length of subdomain.

 logical (kind=log_kind), intent(in), dimension(nlen):: &
   log_sea, &  !true if it is an ocean point                     (-).
   log_ice, &  !true if it is an ice point                       (-).
   log_sib     !true if it is a sib point                        (-).
     

!     output arguments:
!     -----------------

 real (kind=dbl_kind), intent(out), dimension(nlen):: &
   ncw      !Droplet concentration                       (per m3).      

! local variables:

 integer (kind=int_kind):: n

 real (kind=dbl_kind):: &
   nc_land, &  !Droplet concentration over land             (per m3).
   nc_ocean    !Droplet concentration over oceans           (per m3).
 data nc_land, nc_ocean /5.e08, 1.e08/

 real (kind=dbl_kind):: undef
 data undef /1.e36/

!-----------------------------------------------------------------------

!---- fills out the ocean, land, and land ice points:

 do n = 1, len
    if (log_sea(n)) ncw(n) = nc_ocean
 enddo

 do n = 1, len
    if (log_ice(n)) ncw(n) = nc_land
 enddo 

 do n = 1, len
    if (log_sib(n)) ncw(n) = nc_land
 enddo 

 end subroutine eau_ncw

!-----------------------------------------------------------------------
!==============================================================================

 end module eau_cldproperties

!==============================================================================
