!------------------------------------------------------------------------------
subroutine eau_cup &
               (      nlen ,     len ,        nlm ,    lpbl , &
                     dtime ,   alpha ,        plg ,   plgl2 , &
                       bpg ,   bpgl2 ,        phi ,   phil2 , &
                        th ,     tl2 ,        ssl ,   ssll2 , &
                        ql ,     ql2 ,       qcwl ,   qcwl2 , &
                      qrwl ,   qrwl2 ,       qcil ,   qcil2 , &
                      qril ,   qril2 ,         qs ,    qsl2 , &
                      dplg ,     psb ,      hbase ,  wvbase , &
                    cwbase ,  rwbase ,     cibase ,  ribase , &
                       cke ,  cuprec , cuprec_pbl ,    cape , &
                        mb ,      hh ,        hhs ,    ssvl , &
                     dhcup ,   dtcup ,      dvcup ,   dlcup , &
                     drcup ,   dicup ,      dscup ,   tcuph , &
                     tcupv ,   tcupl ,      tcupr ,   tcupi , &
                     tcups ,   fcuph ,      fcupv ,   fcupl , &
                     fcupr ,   fcupi ,      fcups ,  masflx , &
                     lamda ,  cdwork ,   inc_diag             &
#ifdef passive_tracers
                , npassive , tracerp ,  tracerpl2 , trcbase , &
                   dtrccup , tcuptrc ,    fcuptrc             &
#endif
               )

use kinds
use eau_params
use timer
      
implicit none
!------------------------------------------------------------------------------
!eau_cup is the driver for the parameterization of convection.
!Laura D. Fowler/slikrock (03-01-01).
!send comments to laura@atmos.colostate.edu

!references:
!----------- 
!Arakawa, A., and W.H. Schubert 1974: "Interaction of a cumulus cloud ensemble
!   with the large-scale environment,Part I"; J. Atmos. Sci., 31, 674-701
!Randall, D. A., and D. - M. Pan, 1993: "Implementation of the Arakawa-Schubert
!   parameterization with a prognostic closure. The representation of Cumulus
!   Convection in Numerical Models, a Meteorological Monograph published by the
!   American Meteorological Society";
!   K. Emanuel and D. Raymond, Eds., 246pp.
!Ping, 1995: A parameterization of cumulus convection with multiple cloud base
!   levels. Atmospheric Science Paper 583, Dept. of Atmospheric Science.
!   Colorado State University. Fort Collins, Colorado.
!Fowler, L.D., and D.A. Randall, 2002: Interactions between cloud microphysics
!    and cumulus convection in the CSU General Circulation Model. J. Atmos.
!    Sci., 59, 3074-3098.

!modifications:
!   - even-level variables are passed in the argument list instead of being
!     computed locally in eau_cupl2.
!   - even-level variables are now dimensioned as nlm+1, and the code has been
!     corrected accordingly.
!      Laura D. Fowler/mcbug (01-01-03).

!subroutines called:
!   eau_cupl2     : interpolation of odd-level to even-level variables, and
!                   calculation of local variables needed in the cloud model.
!   eau_cupcld    : convective cloud model.
!   eau_cupls     : calculation of convective tendencies due to  microphysics
!                   in convective updrafts and transport by the convective
!                   eddies.
!   eau_cuptracers: convective transport of passive tracers.

!definitions:
!    "odd-level" variables refer to variables in the middle of layers (nlm).
!    "even-level" variables refer to variables at the edges of layers (nlm+1).

!input arguments:
!----------------
integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the pbl top.

real (kind=dbl_kind), intent(in):: &
   dtime              !time step                                           (s).

real (kind=dbl_kind), intent(in), dimension(nlen):: &
   psb                !pbl top pressure                                   (Pa).
     
real (kind=dbl_kind), intent(in), dimension(lpbl):: &
   alpha              !used to diagnose the cloud base mass flux

real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
   plg,              &!odd-level pressure                                 (Pa).
   bpg,              &!odd-level (pl/p0)**kapa                             (-).
   dplg,             &!odd-level pressure thickness                       (Pa).
   phi,              &!odd-level geopotential height                       (m).
   th,               &!odd-level potential temperature                     (K).
   ssl,              &!odd-level dry static energy                      (J/kg).
   ql,               &!odd-level water vapor mixing ratio              (kg/kg).
   qcwl,             &!odd-level cloud water mixing ratio              (kg/kg).
   qrwl,             &!odd-level rain mixing ratio                     (kg/kg).
   qcil,             &!odd-level cloud ice mixing ratio                (kg/kg).
   qril,             &!odd-level cloud snow mixing ratio               (kg/kg).
   qs,               &!odd-level saturation mixing ratio               (kg/kg).
   hbase,            &!moist static energy at convective cloud base     (J/kg).
   wvbase,           &!ql at convective cloud base                     (kg/kg).
   cwbase,           &!cw at convective cloud base                     (kg/kg).
   rwbase,           &!rw at convective cloud base                     (kg/kg).
   cibase,           &!ci at convective cloud base                     (kg/kg).
   ribase             !ri at convective cloud base                     (kg/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   plgl2,            &!even-level pressure                                (Pa).
   bpgl2,            &!even-level (pl2/p0)**kapa                           (-).
   phil2,            &!even-level geopotential height                      (m).
   tl2,              &!even-level  temperature                    (K).
   ssll2,            &!even-level dry static energy                     (J/kg).
   ql2,              &!even-level water vapor mixing ratio             (kg/kg).
   qcwl2,            &!even-level cloud water mixing ratio             (kg/kg).
   qrwl2,            &!even-level rain mixing ratio                    (kg/kg).
   qcil2,            &!even-level cloud ice mixing ratio               (kg/kg).
   qril2,            &!even-level snow mixing ratio                    (kg/kg).
   qsl2               !even-level saturation mixing ratio              (kg/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm,nlm):: &
   cke                !cumulus kinetic energy                         (Joules).

#ifdef passive_tracers
   integer (kind=int_kind), intent(in):: &
     npassive         !number of passive tracers.

   real (kind=dbl_kind), intent(in), dimension(nlen,nlm,npassive):: &
     trcbase          !tracerp at convective cloud base                (kg/kg).
   real (kind=dbl_kind), intent(in), dimension(nlen,nlm,npassive):: &
     tracerp          !odd-level passive tracer mixing ratio           (kg/kg).
   real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1,npassive):: &
     tracerpl2        !even-level passive tracer mixing ratio          (kg/kg).
#endif

!output arguments:
!-----------------
!microp. tendency: convective tendency due to cloud microphysics in the
!                  convective updraft.
!conv. transport : convective tendency due to circulation by convective
!                  eddies.
real (kind=dbl_kind), intent(out), dimension(nlen):: &
   cuprec,           &!cumulus precipitation rate                    (kg/m2/s).
   cuprec_pbl         !cumulus precipitation rate from PBL base      (kg/m2/s).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   mb,               &!cloud base mass flux                          (kg/m2/s).
   cape,             &!conditional available potential energy         (J/kg/s).
   hh,               &!odd-level generalized moist static energy        (J/kg).
   hhs,              &!odd-level saturation moist static energy         (J/kg).
   ssvl,             &!odd-level virtual dry static energy              (J/kg).
   dtcup,            &!odd-level cumulus heating rate                    (K/s).
   dhcup,            &!odd-level microp. tendency of moist stat. ener.(J/kg/s).
   dvcup,            &!odd-level microp. tendency of water vapor     (kg/kg/s).
   dlcup,            &!odd-level microp. tendency of cloud water     (kg/kg/s).
   drcup,            &!odd-level microp. tendency of rain            (kg/kg/s).
   dicup,            &!odd-level microp. tendency of cloud ice       (kg/kg/s).
   dscup,            &!odd-level microp. tendency of snow            (kg/kg/s).
   tcuph,            &!odd-level conv. transport of moist stat. ener. (J/kg/s).
   tcupv,            &!odd-level conv. transport of water vapor      (kg/kg/s).
   tcupl,            &!odd-level conv. transport of cloud water      (kg/kg/s).
   tcupr,            &!odd-level conv. transport of rain             (kg/kg/s).
   tcupi,            &!odd-level conv. transport cloud ice           (kg/kg/s).
   tcups              !odd-level conv. transport of snow             (kg/kg/s).

!fluxes due to circulation by convective eddies.
real (kind=dbl_kind), dimension(nlen,nlm+1):: &
   fcuph,            &!even-level flux of moist static                  (J/m2).
   fcupv,            &!even-level flux of water vapor                  (kg/m2).
   fcupl,            &!even-level flux of cloud water                  (kg/m2).
   fcupr,            &!even-level flux of rain                         (kg/m2).
   fcupi,            &!even-level flux of cloud ice                    (kg/m2).
   fcups              !even-level flux of snow                         (kg/m2).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm-1,nlm):: &
   lamda              !fractional entrainment rate                     (per m).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm,nlm):: &
   cdwork,           &!cloud work function                            (J/kg/s).
   masflx             !cloud base convective mass flux               (kg/m2/s).

real (kind=dbl_kind), intent(out), dimension(nlen,16):: &
   inc_diag           !diagnostic of incidence of convection               (-).

#ifdef passive_tracers
real (kind=dbl_kind),intent(out),dimension(nlen,nlm,npassive):: &
   dtrccup,          &!odd-level detrainment of passive tracers      (kg/kg/s).
   tcuptrc            !odd-level large-scale compensating subsidence (kg/kg/s).

real (kind=dbl_kind),intent(out),dimension(nlen,nlm+1,npassive):: &
   fcuptrc            !even-level flux of passive tracers due to convective
                      !transport                                     (kg/kg/s).
#endif

!local variables:

logical (kind=log_kind) :: & 
   dotv  = .false.    !true, if water loading of the virtual temperature.
logical (kind=log_kind), dimension(len,nlm):: &
   found              !true if the layer is convectively unstable.
       
integer (kind=int_kind):: & 
   i,ic,il,j,l,ll,lpblp1,k,kcount,kn,kp,kt,kccc,nunstsum,nlmp1,nlmm1,np
integer (kind=int_kind):: &
   loadw = 0          !set the water loading in the computation of the cloud
!                      work function and virtual temperature.
integer (kind=int_kind), dimension(nlm):: &
   nunst, nunst2
integer (kind=int_kind), dimension(len,nlm):: &
   ndxlst, kktopm
integer (kind=int_kind), dimension(len*nlm,2,nlm):: &
   ndxl2
     
real (kind=dbl_kind):: &
   dqcpmx , hltmi , tem1
data dqcpmx / 0.25 /

real (kind=dbl_kind), dimension(len):: &
   kktop, rat, dzm

real (kind=dbl_kind), dimension(len,nlm-1):: &
   thswhe, save1, save2

real (kind=dbl_kind), dimension(len,nlm):: &
   zl , tl , tvl , wghtl , gaml, imassl

real (kind=dbl_kind), dimension(len,nlm+1):: &
   hhsl2 , zl2 , tvl2 , ssvl2 , gaml2

real (kind=dbl_kind), dimension(nlen,nlm+1):: &
    hhl2
 
real (kind=dbl_kind), dimension(len,nlm-1,nlm-1,nlm):: &
   dqhcloud , dqvcloud , dqlcloud , dqrcloud, dqicloud , &
   dqscloud ,   pprodr

real (kind=dbl_kind), dimension(len,nlm,nlm-1,nlm):: &
    hcloud , qvcloud , qlcloud , qrcloud , qicloud , &
   qscloud

real (kind=dbl_kind), dimension(len,nlm,nlm,nlm):: &
   eta2

#ifdef passive_tracers
   real (kind=dbl_kind), dimension(len,nlm-1,nlm):: &
     zzctop           !height of the cloud top                             (m).
#endif

!==============================================================================
!initialize of local arrays:
!==============================================================================
!write(6,*) '--- enter eau_cup: len =',len

nlmp1 = nlm+1 ; nlmm1 = nlm-1 ; lpblp1 = lpbl+1

hltmi = 1/hltm

found  = .false. ; rat    = c1
nunst  = 0       ; nunst2 = 0
kktop  = nlm     ; kktopm = nlm
ndxlst = 0       ; ndxl2  = 0
eta2   = c0
      
pprodr(:,:,:,:)   = c0 ; dqhcloud(:,:,:,:) = c0
dqvcloud(:,:,:,:) = c0 ; dqlcloud(:,:,:,:) = c0
dqrcloud(:,:,:,:) = c0 ; dqicloud(:,:,:,:) = c0
dqscloud(:,:,:,:) = c0

hcloud(:,:,:,:)  = c0 ; qvcloud(:,:,:,:) = c0 ; qlcloud(:,:,:,:) = c0
qrcloud(:,:,:,:) = c0 ; qicloud(:,:,:,:) = c0 ; qscloud(:,:,:,:) = c0
      
imassl(:,:) = grav/dplg(1:len,:)

#ifdef passive_tracers
   zzctop(:,:,:) = c0
#endif

!==============================================================================
!computation of conditional instability (following lord, 1982;
!eqs. 2.50 and 2.83)
!==============================================================================

!computation of large-scale variables needed in the middle and at edges of
!layers:

call eau_cupl2 &
        ( nlen ,   len ,  nlm , loadw ,  plg , plgl2 ,  bpg , bpgl2 , &     
           phi , phil2 ,   th ,   ssl , ssll2 ,   ql ,  ql2 , &
          qcwl , qcwl2 , qrwl , qrwl2 , qcil , qcil2 , qril , qril2 , &
            qs ,  qsl2 ,   tl ,   tl2 ,  tvl ,  tvl2 , ssvl , ssvl2 , & 
            hh ,  hhl2 ,  hhs , hhsl2 ,   zl ,   zl2 , gaml , gaml2 , &
         wghtl                                                        &
        )

!computation of the "in-cloud" moist static energy at layer edges for a non-
!entraining layer:

do l = 1, nlmm1
   do i = 1, len
      save1(i,l) = (1.+gaml2(i,l+1))*spec_heat_cp*delta*tl2(i,l+1)            
      save2(i,l) = 1.+gaml2(i,l+1)*delta*hltmi*spec_heat_cp*tl2(i,l+1)
      thswhe(i,l) = save1(i,l)/save2(i,l)*(qsl2(i,l+1)-ql2(i,l+1))
      thswhe(i,l) = hhsl2(i,l+1) - thswhe(i,l)
                 
      if(thswhe(i,l) .lt. hhl2(i,l+1) .or. thswhe(i,l) .gt. hhsl2(i,l+1)) then
         thswhe(i,l) = hhl2(i,l+1)
 	 endif
   enddo
enddo

!find grid columns where there is conditional instability; hh(l) is greater
!than thswhe at some level - necessary conditions for a positive cloud work
!function.

do i = 1, len

!find lowest unstable cloud base:
   do k = nlmm1, ltpcup, -1
      found(i,nlm) = hh(i,nlm) .gt. thswhe(i,k)
      if(found(i,nlm)) goto 110
   enddo
110 continue

!find higher unstable cloud base: 
   do k = ltpcup+1, nlmm1
      kp = nlmm1+ltpcup+1-k
      do kt = kp-1, ltpcup, -1
         found(i,kp) = hhl2(i,kp+1) .gt. thswhe(i,kt)
         if(found(i,kp)) goto 120
      enddo       
120   continue
   enddo

   dzm(i) = (phil2(i,nlm) - phil2(i,nlm+1)) / grav
enddo

!---- create an index list for the unstable points; if there are no 
!     unstable points, go no further.

nunstsum = 0
do j = ncb, lpblp1
   kcount = 0
   do i = 1, len
      if(found(i,j)) then
         nunst(j) = nunst(j) + 1
	   kcount   = kcount + 1
	   ndxlst(kcount,j) = i
	endif
   enddo
   nunstsum = nunstsum + nunst(j)
enddo

if(nunstsum.eq.0) return

if(nunst(lpblp1) .ne. 0) then
   do kn = 1, nunst(lpblp1)
      il = ndxlst(kn,lpblp1)
      inc_diag(il,1) = inc_diag(il,1) + c1
   enddo
endif
do j = ncb, lpblp1
   if(nunst(j) .ne. 0) then
      do kn = 1, nunst(j)
         il = ndxlst(kn,j)
         inc_diag(il,2) = inc_diag(il,2) + c1
      enddo
   endif
enddo

!==============================================================================
!cloud model (Fowler and Randall, 2002)
!==============================================================================

do j = lpblp1, ncb, -1   
   if(nunst(j) .ne. 0) then

      call eau_cupcld &
               (                j ,              nlen ,               len , &
                              nlm ,              lpbl ,              dotv , &
                              plg ,                zl ,                tl , &
                               ql ,              qcwl ,              qrwl , &
                             qcil ,              qril ,                qs , &
                               hh ,               hhs ,              gaml , &
                            wghtl ,             plgl2 ,               zl2 , &
                              tl2 ,               ql2 ,             qcwl2 , &
                            qrwl2 ,             qcil2 ,             qril2 , &
                             hhl2 ,             hhsl2 ,             gaml2 , &
                       hbase(1,j) ,       wvbase(1,j) ,       cwbase(1,j) , &
                      rwbase(1,j) ,       cibase(1,j) ,       ribase(1,j) , &
                         nunst(j) ,       ndxlst(1,j) ,               tvl , &
                             tvl2 ,   hcloud(1,1,1,j) ,  qvcloud(1,1,1,j) , &
                 qlcloud(1,1,1,j) ,  qrcloud(1,1,1,j) ,  qicloud(1,1,1,j) , &
                 qscloud(1,1,1,j) , dqhcloud(1,1,1,j) , dqvcloud(1,1,1,j) , &
                dqlcloud(1,1,1,j) , dqrcloud(1,1,1,j) , dqicloud(1,1,1,j) , &
                dqscloud(1,1,1,j) ,   pprodr(1,1,1,j) ,     cdwork(1,1,j) , &
                    eta2(1,1,1,j) ,         nunst2(j) ,      ndxl2(1,1,j) , &
                        cape(1,j) ,      lamda(1,1,j) ,          inc_diag , &
                              dzm                                           &
#ifdef passive_tracers
                  , zzctop(1,1,j)                                           &
#endif
              )

  endif
enddo

if(nunst2(lpblp1) .ne. 0) then
   do kn = 1, nunst2(lpblp1)
      ic = ndxl2(kn,1,lpblp1) ! cloud top index
      il = ndxl2(kn,2,lpblp1) ! cloud top location index
      inc_diag(il,15) = inc_diag(il,15) + c1
   enddo
endif
do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then
      do kn = 1, nunst2(j)
         ic = ndxl2(kn,1,j) ! cloud top index
          il = ndxl2(kn,2,j) ! cloud top location index
          inc_diag(il,16) = inc_diag(il,16) + c1
       enddo
   endif
enddo

!==============================================================================
!cloud feedbacks on the large-scale fields:
!==============================================================================

kccc = 0
do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then
      kccc = kccc+nunst2(j)
      do kn = 1, nunst2(j)
         ic = ndxl2(kn,1,j) ! cloud top index
         il = ndxl2(kn,2,j) ! cloud top location index
!        cuin(il) = .true.
         if(kktop(il) .ge. ic) kktop(il) = ic
         if(kktopm(il,j) .ge. ic) kktopm(il,j) = ic           
      enddo            
   endif
enddo
if(kccc .eq. 0) return
            
!cumulus cloud base mass flux for each cloud type, and large-scale cumulus
!cloud base mass flux:

do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then
      do kn = 1, nunst2(j)
	   ic = ndxl2(kn,1,j) ! cloud top index.
	   il = ndxl2(kn,2,j) ! cloud top location.
   	   masflx(il,ic,j) = sqrt(cke(il,ic,j)/alpha(ic))
         mb(il,j) = mb(il,j) + masflx(il,ic,j)

!cumulus mass flux at each level, and for each cloud type:
         do l = ic, j
            eta2(il,l,ic,j) = eta2(il,l,ic,j)*masflx(il,ic,j)
         enddo               
      enddo
   endif
enddo        

!limit mass fluxes. rat ensures that the cumulus mass flux does not exceed the
!mass of the pbl.
      
do i = 1, len      
   rat(i) = c1
   tem1   = mb(i,lpblp1)*grav*dtime
   if(tem1 .gt. psb(i)) rat(i) = psb(i)/tem1     
enddo

do l = 1, lpblp1
   mb(1:len,l) = mb(1:len,l)*rat(:)
enddo
do l = 1, lpblp1
   do ll = 1, lpblp1
      masflx(1:len,l,ll) = masflx(1:len,l,ll)*rat(:)
   enddo
enddo

!large-scale feedbacks: tendencies due to cloud microphysical processes and
!transports due to convective circulations:

#ifdef eau_ng
 call eau_cuplsng &
         (   nlen ,     len ,     nlm ,    lpbl ,         rat , &
           nunst2 ,   ndxl2 ,  imassl ,      hh ,        hhl2 , &
               ql ,     ql2 ,    qcwl ,   qcwl2 ,        qrwl , &
            qrwl2 ,    qcil ,   qcil2 ,    qril ,       qril2 , &
             eta2 ,  hcloud , qvcloud , qlcloud ,     qrcloud , &
          qicloud , qscloud ,  pprodr ,  cuprec ,  cuprec_pbl , &
            dhcup ,   dtcup ,   dvcup ,   dlcup ,       drcup , &
            dicup ,   dscup ,   tcuph ,   tcupv ,       tcupl , &
            tcupr ,   tcupi ,   tcups ,   fcuph ,       fcupv , &
            fcupl ,   fcupr ,   fcupi ,   fcups                 & 
         )
#else
 call eau_cupls &
         (    nlen ,      len ,      nlm ,     lpbl ,         rat , &
            nunst2 ,    ndxl2 ,   imassl ,     hhl2 ,         ql2 , &
             qcwl2 ,    qrwl2 ,    qcil2 ,    qril2 ,        eta2 , &
            hcloud ,  qvcloud ,  qlcloud ,  qrcloud ,     qicloud , &
           qscloud , dqhcloud , dqvcloud , dqlcloud ,    dqrcloud , &
          dqicloud , dqscloud ,   pprodr ,   cuprec ,  cuprec_pbl , &
             dhcup ,    dtcup ,    dvcup ,    dlcup ,       drcup , &
             dicup ,    dscup ,    tcuph ,    tcupv ,       tcupl , &
             tcupr ,    tcupi ,    tcups ,    fcuph ,       fcupv , &
             fcupl ,    fcupr ,    fcupi ,    fcups                 &
         )
#endif
#ifdef passive_tracers
!tendencies of passive tracers due to convective transport:

   do np = 1, npassive

      call eau_cuptracers &
               (           nlen ,              len ,               nlm , &
                           lpbl ,           nunst2 ,             ndxl2 , &
                             zl ,              zl2 ,            imassl , &
                          lamda ,           zzctop ,              eta2 , &
                trcbase(1,1,np) ,  tracerp(1,1,np) , tracerpl2(1,1,np) , &
                dtrccup(1,1,np) ,  tcuptrc(1,1,np) ,   fcuptrc(1,1,np)   &
               )

      do l = 1, lpblp1
         dtrccup(1:len,l,np) = dtrccup(1:len,l,np)*rat(:)
         tcuptrc(1:len,l,np) = tcuptrc(1:len,l,np)*rat(:)
      enddo
      do l = 1, lpblp1+1
         fcuptrc(1:len,l,np) = fcuptrc(1:len,l,np)*rat(:)
      enddo
   enddo
#endif

!write(6,*) '--- end eau_cup'
end subroutine eau_cup

!------------------------------------------------------------------------------
