!------------------------------------------------------------------------------
 subroutine eau_cup_control &
                      (      nagrid ,       ksdm , &
                            do_phys ,        nlm , &
                               lpbl ,      dtdyn , &
                                plg ,      plgl2 , &
                                bpg ,      bpgl2 , &
                                phi ,      phil2 , &
                                 th ,        tl2 , &
                                ssl ,      ssll2 , & 
                             tracer ,   tracerl2 , &
                                 qs ,       qsl2 , &
                                cke ,        psb , &
                             cdwork ,     mb_pbl , &
                             cuprec ,      lamda , &
                             masflx ,      dt_th , &
                           dt_hydro , deauc_vtot   &
#ifdef passive_tracers
                       ,  dt_tracer                &
#endif
                      )   

 use kinds
 use eau_params
 use time_manager_module
 use tracer_names

#ifndef scm
 use qp_ml
#endif
      
 implicit none
!------------------------------------------------------------------------------
!eau_cup_control is the driver for the eau_cup parameterization of convection.
!eau_cup runs on the physical time-step.
!Laura D. Fowler/slikrock (03-01-01).

!send comments to laura@atmos.colostate.edu.

!modifications:
!    - added the calculation of the tendencies of passive tracers (as co2) due
!      to transport by convective circulations.    
!      Laura D. Fowler/mcbug (02-01-02).
!    - even-level variables are passed in the argument list instead of being
!      computed locally in eau_cupl2.
!      Laura D. Fowler/mcbug (01-01-03).

!subroutines called:
!    eau_cupalpha: set the value of alpha for the prognostic closure of the
!                  cke.
!    eau_cupinit : set the cloud base value of the moist static energy, water
!                  vapor, cloud water, cloud ice, rain, and snow.
!    eau_cup     : core of convective parameterization.
!    eau_cupqps  : global qp2s and qp3s diagnostics.
!    eau_cuppbps : point-by-point pbp1s and pbp2s diagnostics.

!definitions:
!    "odd-level" variables refer to variables in the middle of layers (nlm).
!    "even-level" variables refer to variables at the edges of layers (nlm+1).

!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    ksdm,            &!loop index.
    nagrid,          &!length of horizontal domain with ghost cells.
    nlm,             &!number of layers.
    lpbl              !number of layers above the pbl layer(s).

 logical (kind=log_kind), intent(in), dimension(nagrid):: &
    do_phys           !select the non-ghost cells                          (-).

 real (kind=dbl_kind), intent(in):: &
    dtdyn             !dynamical time-step                                 (s).

 real (kind=dbl_kind), intent(in), dimension(nagrid):: &
    psb               !pbl-top pressure                                   (Pa).

 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm):: &
    plg,             &!odd-level pressure                                 (Pa).
    bpg,             &!odd-level (pl/p0)**kapa                             (-).
    phi,             &!odd-level geopotential height                       (m).
    th,              &!odd-level potential temperature                     (K).
    ssl,             &!odd-level dry static energy                      (J/kg).
    qs                !odd-level saturation mixing ratio               (kg/kg).

 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm,nlm):: &
    cke               !cumulus kinetic energy                         (J/m2/s).
 
 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1):: &
    plgl2,           &!even-level pressure                                (Pa).
    bpgl2,           &!even-level (pl/p0)**kapa                            (-).
    phil2,           &!even-level geopotential height                      (m).
    tl2,             &!even-level temperature                              (K).
    ssll2,           &!even-level dry static energy.
    qsl2              !even-level saturation mixing ratio              (kg/kg).

 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm,ntracer):: &
    tracer            !odd-level water + passive tracer mixing ratios  (kg/kg).
     
 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1,ntracer):: &
    tracerl2          !even-level water+passive tracer mixing ratios   (kg/kg).

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out),dimension(nagrid):: &
    mb_pbl,          &!convective mass flux at PBL top               (kg/m2/s).
    cuprec            !convective precipitation rate                 (kg/m2/s).

 real (kind=dbl_kind), intent(out),dimension(nagrid,nlm):: &
    dt_th             !odd-level tendency of temperature                 (K/s).

 real (kind=dbl_kind), intent(out), dimension(nagrid,nwater):: &
    deauc_vtot        !vertically-integrated tendencies of "water"
                      !variables                                     (kg/m2/s).

 real (kind=dbl_kind), intent(out),dimension(nagrid,nlm,nwater):: &
    dt_hydro          !odd-level tendency of "water" variables       (kg/kg/s). 

 real (kind=dbl_kind), intent(out), dimension(nagrid,nlm-1,nlm) :: &
    lamda             !fractional entrainment rate                      (m^-1).

 real (kind=dbl_kind), intent(out),dimension(nagrid,nlm,nlm):: &
    cdwork,          &!cloud work function                            (J/kg/s).
    masflx            !cloud base convective mass flux               (kg/m2/s).

#ifdef passive_tracers
 real (kind=dbl_kind), intent(out),dimension(nagrid,nlm,npassive):: &
    dt_tracer         !odd-level tendency of passive tracers due to convective
                      !transport                                     (kg/kg/s).
#endif

!local variables output by eau_cup:
!microp. tendency: convective tendency due to cloud microphysics in the
!                  convective updraft.
!conv. transport : convective tendency due to circulation by convective
!                  eddies.
 real (kind=dbl_kind), dimension(lpbl):: &
    alpha             !constant used in prognostic closure of cke      (m4/kg).

 real (kind=dbl_kind), dimension(nagrid):: &
    cuprec_pbl        !convective precipitation rate for PBL base    (kg/m2/s).

 real (kind=dbl_kind), dimension(nagrid,nlm):: &
    mb,              &!cloud base mass flux                          (kg/m2/s).
    cape,            &!conditional available potential energy         (J/kg/s).
    hh,              &!odd-level generalized moist static energy        (J/kg).
    hhs,             &!odd-level saturation moist static energy         (J/kg).
    ssvl,            &!odd-level virtual dry static energy              (J/kg).
    dtcup,           &!odd-level cumulus heating rate                    (K/s).
    dhcup,           &!odd-level microp. tendency of moist stat. ener.(J/kg/s).
    dvcup,           &!odd-level microp. tendency of water vapor     (kg/kg/s).
    dlcup,           &!odd-level microp. tendency of cloud water     (kg/kg/s).
    drcup,           &!odd-level microp. tendency of rain            (kg/kg/s).
    dicup,           &!odd-level microp. tendency of cloud ice       (kg/kg/s).
    dscup,           &!odd-level microp. tendency of snow            (kg/kg/s).
    tcuph,           &!odd-level conv. transport of moist stat. ener. (J/kg/s).
    tcupv,           &!odd-level conv. transport of water vapor      (kg/kg/s).
    tcupl,           &!odd-level conv. transport of cloud water      (kg/kg/s).
    tcupr,           &!odd-level conv. transport of rain             (kg/kg/s).
    tcupi,           &!odd-level conv. transport cloud ice           (kg/kg/s).
    tcups             !odd-level conv. transport of snow             (kg/kg/s).

!fluxes due to circulation by convective eddies.
 real (kind=dbl_kind), dimension(nagrid,nlm+1):: &
    fcuph,           &!even-level flux of moist static                  (J/m2).
    fcupv,           &!even-level flux of water vapor                  (kg/m2).
    fcupl,           &!even-level flux of cloud water                  (kg/m2).
    fcupr,           &!even-level flux of rain                         (kg/m2).
    fcupi,           &!even-level flux of cloud ice                    (kg/m2).
    fcups             !even-level flux of snow                         (kg/m2).

!other local variables:
 integer (kind=int_kind):: i, itod, itod2, l, len, nlen, nlmm1, np

 real (kind=dbl_kind):: &
    dtphys,          &!physical time-step                                  (s).
    nbphys,          &!nb of physical time-steps per day                   (-).
    pcos1,           &!cosine of first diurnal harmonic                    (-).
    pcos2,           &!cosine of first diurnal harmonic                    (-).
    psin1,           &!sine of second diurnal harmonic                     (-).
    psin2,           &!sine of second diurnal harmonic                     (-).
    imass             !conversion factor                               (m2/kg).

 real (kind=dbl_kind), dimension(nagrid):: &
    deauc_iqt,       &!vertically-integrated deauc_iq                (kg/m2/s).
    deauc_icwt,      &!vertically-integrated deauc_icw               (kg/m2/s).
    deauc_irwt,      &!vertically-integrated deauc_irw               (kg/m2/s).
    deauc_icit,      &!vertically-integrated deauc_ici               (kg/m2/s).
    deauc_irit,      &!vertically-integrated deauc_iri               (kg/m2/s).
    tot_cup           !vertically-integrated total tendency          (kg/m2/s).

 real (kind=dbl_kind), dimension(nagrid,nlm):: &
    dplg,            &!layer pressure thickness                           (Pa).
    ql,              &!odd-level water vapor mixing ratio              (kg/kg).
    qcwl,            &!odd-level cloud water mixing ratio              (kg/kg).
    qrwl,            &!odd-level rain mixing ratio                     (kg/kg).
    qcil,            &!odd-level cloud ice mixing ratio                (kg/kg).
    qril,            &!odd-level snow mixing ratio                     (kg/kg).
    deauc_iq,        &!odd-level total tendency of water vapor       (kg/kg/s).
    deauc_icw,       &!odd-level total tendency of cloud water       (kg/kg/s).
    deauc_ici,       &!odd-level total tendency of cloud ice         (kg/kg/s).
    deauc_irw,       &!odd-level total tendency of rain              (kg/kg/s).
    deauc_iri         !odd-level total tendency of snow              (kg/kg/s).

 real (kind=dbl_kind), dimension(nagrid,nlm+1):: &
    ql2,             &!even-level water vapor mixing ratio             (kg/kg).
    qcwl2,           &!even-level cloud water mixing ratio             (kg/kg).
    qrwl2,           &!even-level rain mixing ratio                    (kg/kg).
    qcil2,           &!even-level cloud ice mixing ratio               (kg/kg).
    qril2             !even-level snow mixing ratio                    (kg/kg).

 real (kind=dbl_kind), dimension(nagrid,lpbl+1):: &
    hbase,           &!moist static energy at convective cloud base     (J/kg).
    qlbase,          &!ql at convective cloud base                     (kg/kg).
    cwbase,          &!cw at convective cloud base                     (kg/kg).
    rwbase,          &!rw at convective cloud base                     (kg/kg).
    cibase,          &!ci at convective cloud base                     (kg/kg).
    ribase            !ri at convective cloud base                     (kg/kg).

 real (kind=dbl_kind), dimension(nagrid,16):: inc_diag

#ifdef passive_tracers
  real (kind=dbl_kind), dimension(nagrid,npassive):: &
     deauc_trct       !vertically-integrated tendency of tracerl       (kg/m2).

  real (kind=dbl_kind), dimension(nagrid,lpbl+1,npassive):: &
     trcbase          !tracerl at convective cloud base                (kg/kg).

  real (kind=dbl_kind), dimension(nagrid,nlm,npassive):: &
     tracerp,        &!odd-level passive tracer mixing ratio           (kg/kg).
     dtrccup,        &!odd-level microp. tendency of tracerl         (kg/kg/s).
     tcuptrc,        &!odd-level conv. transport of tracerl          (kg/kg/s).
     deauc_trc        !odd-level total conv. tendency of tracerl     (kg/kg/s).

  real (kind=dbl_kind), dimension(nagrid,nlm+1,npassive):: &
     tracerpl2,      &!even-level passive tracer mixing ratio          (kg/kg).
     fcuptrc          !flux of tracerl due to cup transp             (kg/m2/s).
#endif

!------------------------------------------------------------------------------
!write(6,*) '--- enter eau_cup_control: ksdm=', ksdm, ' nagrid=', nagrid, &
!           ' nlm=', nlm,' lpbl=', lpbl
 nlmm1  = nlm-1
 dtphys = dtdyn*n_physics

!first and second diurnal harmonics:
 nbphys = 86400_dbl_kind/dtdyn 
 itod   = tofday*nbphys/24._dbl_kind + 1
 itod2  = 2 * itod - 1 
#ifndef scm
 pcos1  = pcos(itod)  ; psin1 = psin(itod)
 pcos2  = pcos(itod2) ; psin2 = psin(itod2)
#endif

!initialization of output variables:
 mb_pbl(:)       = c0 ; cuprec(:)       = c0 ; dt_th(:,:)    = c0
 lamda(:,:,:)    = c0 ; masflx(:,:,:)   = c0 ; cdwork(:,:,:) = c0
 dt_hydro(:,:,:) = c0 ; deauc_vtot(:,:) = c0

#ifdef passive_tracers
 dt_tracer(:,:,:) = c0
#endif

!initialization of local variables:
 tot_cup(:)    = c0 ; deauc_iqt(:)  = c0 ; deauc_icwt(:) = c0
 deauc_irwt(:) = c0 ; deauc_icit(:) = c0 ; deauc_irit(:) = c0

 ql(:,:)   = tracer(:,:,iq)  ; ql2(:,:)   = tracerl2(:,:,iq)
 qcwl(:,:) = tracer(:,:,icw) ; qcwl2(:,:) = tracerl2(:,:,icw)
 qrwl(:,:) = tracer(:,:,irw) ; qrwl2(:,:) = tracerl2(:,:,irw)
 qcil(:,:) = tracer(:,:,ici) ; qcil2(:,:) = tracerl2(:,:,ici)
 qril(:,:) = tracer(:,:,iri) ; qril2(:,:) = tracerl2(:,:,iri)

 do l = 1, nlm
    dplg(:,l) = plgl2(:,l+1)-plgl2(:,l)
 enddo

#ifdef passive_tracers
  tracerp(:,:,:)   = tracer(:,:,ipassive(:))
  tracerpl2(:,:,:) = tracerl2(:,:,ipassive(:))
#endif

!initialization of local eau_cup variables:
 cuprec_pbl(:)  = c0

 hbase(:,:)  = c0 ; qlbase(:,:) = c0 ; cwbase(:,:) = c0
 rwbase(:,:) = c0 ; cibase(:,:) = c0 ; ribase(:,:) = c0

 cape(:,:)      = c0 ; mb(:,:)        = c0 ; hh(:,:)        = c0
 hhs(:,:)       = c0 ; ssvl(:,:)      = c0 
 dhcup(:,:)     = c0 ; dtcup(:,:)     = c0 ; dvcup(:,:)     = c0
 dlcup(:,:)     = c0 ; drcup(:,:)     = c0 ; dicup(:,:)     = c0
 dscup(:,:)     = c0 ; tcuph(:,:)     = c0 ; tcupv(:,:)     = c0
 tcupl(:,:)     = c0 ; tcupr(:,:)     = c0 ; tcupi(:,:)     = c0
 tcups(:,:)     = c0 ; deauc_iq(:,:)  = c0 ; deauc_icw(:,:) = c0
 deauc_irw(:,:) = c0 ; deauc_ici(:,:) = c0 ; deauc_iri(:,:) = c0

 fcuph(:,:) = c0 ; fcupv(:,:) = c0 ; fcupl(:,:) = c0
 fcupr(:,:) = c0 ; fcupi(:,:) = c0 ; fcups(:,:) = c0

 inc_diag(:,:) = c0

#ifdef passive_tracers
  trcbase(:,:,:)  = c0 ; dtrccup(:,:,:)   = c0
  tcuptrc(:,:,:)  = c0 ; fcuptrc(:,:,:)   = c0
  deauc_trct(:,:) = c0 ; deauc_trc(:,:,:) = c0 
#endif

!----

 do i = 1, nagrid
    if(do_phys(i)) then
!      write(6,*) '--- eau_cup: i=', i

       nlen = nagrid
       len  = 1
            
!eau_cup parameterization of convection:

       call eau_cupalpha(lpbl,alpha)

       call eau_cupinit &  
                (       nlen ,            len ,            nlm ,        lpbl , &
                    plg(i,1) ,       bpg(i,1) ,       phi(i,1) ,     th(i,1) , &
                     ql(i,1) ,      qcwl(i,1) ,      qrwl(i,1) ,   qcil(i,1) , &
                   qril(i,1) ,     hbase(i,1) ,    qlbase(i,1) , cwbase(i,1) , &
                 rwbase(i,1) ,    cibase(i,1) ,    ribase(i,1)                 &
#ifdef passive_tracers
                  , npassive , tracerp(i,1,1) , trcbase(i,1,1)                 &
#endif
                )

       call eau_cup &
                (          nlen ,            len ,              nlm , &
                           lpbl ,         dtphys ,            alpha , &
                       plg(i,1) ,     plgl2(i,1) ,         bpg(i,1) , &
                     bpgl2(i,1) ,       phi(i,1) ,       phil2(i,1) , &
                        th(i,1) ,       tl2(i,1) ,         ssl(i,1) , &
                     ssll2(i,1) ,        ql(i,1) ,         ql2(i,1) , &
                      qcwl(i,1) ,     qcwl2(i,1) ,        qrwl(i,1) , &
                     qrwl2(i,1) ,      qcil(i,1) ,       qcil2(i,1) , &
                      qril(i,1) ,     qril2(i,1) ,          qs(i,1) , &
                      qsl2(i,1) ,      dplg(i,1) ,           psb(i) , &
                     hbase(i,1) ,    qlbase(i,1) ,      cwbase(i,1) , &
                    rwbase(i,1) ,    cibase(i,1) ,      ribase(i,1) , &
                     cke(i,1,1) ,      cuprec(i) ,    cuprec_pbl(i) , &
                      cape(i,1) ,        mb(i,1) ,          hh(i,1) , &
                       hhs(i,1) ,      ssvl(i,1) ,       dhcup(i,1) , &
                     dtcup(i,1) ,     dvcup(i,1) ,       dlcup(i,1) , &
                     drcup(i,1) ,     dicup(i,1) ,       dscup(i,1) , &
                     tcuph(i,1) ,     tcupv(i,1) ,       tcupl(i,1) , &
                     tcupr(i,1) ,     tcupi(i,1) ,       tcups(i,1) , &
                     fcuph(i,1) ,     fcupv(i,1) ,       fcupl(i,1) , &
                     fcupr(i,1) ,     fcupi(i,1) ,       fcups(i,1) , &
                  masflx(i,1,1) ,   lamda(i,1,1) ,    cdwork(i,1,1) , &
                  inc_diag(i,1)                                       &
#ifdef passive_tracers
                     , npassive , tracerp(i,1,1) , tracerpl2(i,1,1) , &
                 trcbase(i,1,1) , dtrccup(i,1,1) ,   tcuptrc(i,1,1) , &
                 fcuptrc(i,1,1)                                       &
#endif
                )

    endif ! end(do_phys)
 enddo    ! end nagrid loop

!eau_cup diagnostics:

#ifndef scm
 call eau_cupqps &
          (      ksdm , n_physics ,  nagrid ,      nlm ,      pcos1 , &
                pcos2 ,     psin1 ,   psin2 ,     dplg ,     cuprec , &
           cuprec_pbl ,      cape ,     mb  ,      cke ,     cdwork , &
               masflx ,     dtcup ,   dhcup ,    dvcup ,      dlcup , &
                drcup ,     dicup ,   dscup ,    tcuph ,      tcupv , &
                tcupl ,     tcupr ,   tcupi ,    tcups ,   inc_diag , &
                   hh                                                 &
#ifdef passive_tracers
           , npassive ,   dtrccup , tcuptrc                           &
#endif
          )
#endif

 call eau_cuppbps &
          (      ksdm , n_physics ,  n_pbp , nagrid ,    nlm , &
                   hh ,       hhs ,    ssl ,   ssvl , cuprec , &
           cuprec_pbl ,      cape ,     mb ,    cke , cdwork , &
               masflx ,     dtcup ,  dhcup ,  dvcup ,  dlcup , &
                drcup ,     dicup ,  dscup ,  tcuph ,  tcupv , &
                tcupl ,     tcupr ,  tcupi ,  tcups , inc_diag  &
          )

!-----
      
 do i = 1, nagrid
    if(do_phys(i)) then

!check conservation of the total water: 

       deauc_iq(i,:)  = dvcup(i,:) + tcupv(i,:)
       deauc_icw(i,:) = dlcup(i,:) + tcupl(i,:)
       deauc_irw(i,:) = drcup(i,:) + tcupr(i,:)
       deauc_ici(i,:) = dicup(i,:) + tcupi(i,:)
       deauc_iri(i,:) = dscup(i,:) + tcups(i,:)

       do l = 1, nlm
          deauc_iqt(i)  = deauc_iqt(i) +deauc_iq(i,l)*dplg(i,l)
          deauc_icwt(i) = deauc_icwt(i)+deauc_icw(i,l)*dplg(i,l)
          deauc_irwt(i) = deauc_irwt(i)+deauc_irw(i,l)*dplg(i,l)
          deauc_icit(i) = deauc_icit(i)+deauc_ici(i,l)*dplg(i,l)
 	    deauc_irit(i) = deauc_irit(i)+deauc_iri(i,l)*dplg(i,l) 
       enddo
       tot_cup(i) = deauc_iqt(i)  + deauc_icwt(i) + deauc_irwt(i) &
                  + deauc_icit(i) + deauc_irit(i)
       tot_cup(i) = tot_cup(i)/grav
!      if(cuprec(i).ne.c0) write(6,101) i,cuprec(i),tot_cup(i), &
!                             cuprec(i)+tot_cup(i)

       deauc_vtot(i,iq)  = deauc_iqt(i)
       deauc_vtot(i,icw) = deauc_icwt(i)
       deauc_vtot(i,irw) = deauc_irwt(i)
       deauc_vtot(i,ici) = deauc_icit(i)
       deauc_vtot(i,iri) = deauc_irit(i)

#ifdef passive_tracers
       do np = 1, npassive
          deauc_trc(i,:,np) = dtrccup(i,:,np) + tcuptrc(i,:,np)
          do l = 1, nlm
             deauc_trct(i,np) = deauc_trct(i,np) + deauc_trc(i,l,np)*dplg(i,l)
          enddo
          deauc_trct(i,np) = deauc_trct(i,np)/grav
!         write(6,103) i,deauc_trct(i,np)
       enddo
#endif

!adjustment of convective tendencies in the PBL for consistency with the
!calculation of the vertical flux divergence in the dynamical core:

       imass = grav / dplg(i,nlmm1)
       dtcup(i,nlmm1) = dtcup(i,nlmm1) - (fcuph(i,nlm)-hltm*fcupv(i,nlm) &
                      + lfus*(fcupi(i,nlm)+fcups(i,nlm))) * imass/spec_heat_cp
       tcupv(i,nlmm1) = tcupv(i,nlmm1) - fcupv(i,nlm)*imass
       tcupl(i,nlmm1) = tcupl(i,nlmm1) - fcupl(i,nlm)*imass
       tcupr(i,nlmm1) = tcupr(i,nlmm1) - fcupr(i,nlm)*imass
       tcupi(i,nlmm1) = tcupi(i,nlmm1) - fcupi(i,nlm)*imass 
       tcups(i,nlmm1) = tcups(i,nlmm1) - fcups(i,nlm)*imass

       imass = grav / dplg(i,nlm)
       dtcup(i,nlm)   = c0
       tcupv(i,nlm)   = tcupv(i,nlm) + fcupv(i,nlm)*imass
       tcupl(i,nlm)   = tcupl(i,nlm) + fcupl(i,nlm)*imass
       tcupr(i,nlm)   = tcupr(i,nlm) + fcupr(i,nlm)*imass
       tcupi(i,nlm)   = tcupi(i,nlm) + fcupi(i,nlm)*imass
       tcups(i,nlm)   = tcups(i,nlm) + fcups(i,nlm)*imass

#ifdef passive_tracers
       imass = grav / dplg(i,nlmm1)
       do np = 1, npassive
          tcuptrc(i,nlmm1,np) = tcuptrc(i,nlmm1,np) - fcuptrc(i,nlm,np)*imass
       enddo
       imass = grav / dplg(i,nlm)
       do np = 1, npassive
          tcuptrc(i,nlm,np) = tcuptrc(i,nlm,np) + fcuptrc(i,nlm,np)*imass
       enddo
#endif

    endif ! end(do_phys)
 enddo    ! end nagrid loop

#ifndef scm
 call qp3mlsum(dvcup(1,1),nagrid,c1,iqp3ml_tnmrc,ksdm,nagrid) 
 call qp3mlsum(tcupv(1,1),nagrid,c1,iqp3ml_tnmrc,ksdm,nagrid,step_sample=0) 
#endif

!convective tendencies needed for the dynamics:

 mb_pbl(:) = mb(:,nlm)

 dt_th(:,:) = dtcup(:,:)
 dt_hydro(:,:,iq)  = dvcup(:,:) + tcupv(:,:)
 dt_hydro(:,:,icw) = dlcup(:,:) + tcupl(:,:)
 dt_hydro(:,:,irw) = drcup(:,:) + tcupr(:,:)
 dt_hydro(:,:,ici) = dicup(:,:) + tcupi(:,:)
 dt_hydro(:,:,iri) = dscup(:,:) + tcups(:,:)

#ifdef passive_tracers
  dt_tracer(:,:,:) = dtrccup(:,:,:) + tcuptrc(:,:,:)      
#endif

!formats:
 100 format(i3,2(1x,e15.8))
 101 format('--- cup: ',i3,3(1x,e15.8))

!write(6,*) '--- end eau_cup_control'

 end subroutine eau_cup_control

!------------------------------------------------------------------------------
