!------------------------------------------------------------------------------
subroutine eau_cupl2 &
               (  nlen ,  len ,  nlm , loadw ,  plg , plgl2 ,  bpg , bpgl2 , &
                  phi , phil2 ,   th ,   ssl , ssll2 ,   ql ,   ql2 , &
                 qcwl , qcwl2 , qrwl , qrwl2 , qcil , qcil2 , qril , qril2 , &
                   qs ,  qsl2 ,   tl ,   tl2 ,  tvl ,  tvl2 , ssvl , ssvl2 , &
                   hh ,  hhl2 ,  hhs , hhsl2 ,   zl ,   zl2 , gaml , gaml2 , &
                wghtl                                                        &
               )
use kinds
use eau_params
use eau_util

implicit none
!------------------------------------------------------------------------------
!eau_cupl2 computes additional variables, such as the generalized moist static
!energy, that are defined in the middle and edges of individual layers.
!Laura D. Fowler/slikrock (12-31-97).

!send comments to laura@atmos.colostate.edu

!modifications:
!   - even-level variables are passed in the argument list instead of being
!     computed locally. only additional odd- and even-level variables needed by
!     the cloud model are now calculated in eau_cupl2.
!   - even-level variables are now dimensioned as nlm+1, and the code has been
!     corrected accordingly.
!     Laura D. Fowler/mcbug (01-01-03).

!subroutines called:
!    eau_sat: calculates saturation vapor pressures, saturation mixing
!             ratios, and saturation.

!argument list variables:
!input arguments:
!----------------
integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub domain.
   nlm,              &!number of layers.
   loadw              !set the water loading in the computation of the cloud
                      !work function and virtual temperature.

real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
   plg,              &!odd-level pressure                                 (Pa).
   bpg,              &!odd-level (pl/p0)**kapa                             (-).
   phi,              &!odd-level geopotential height                       (m).
   th,               &!odd-level potential temperature                     (K).
   ssl,              &!odd-level dry static energy                      (J/kg).
   ql,               &!odd-level water vapor mixing ratio              (kg/kg).
   qcwl,             &!odd-level cloud water mixing ratio              (kg/kg).
   qrwl,             &!odd-level rain mixing ratio                     (kg/kg).
   qcil,             &!odd-level cloud ice mixing ratio                (kg/kg).
   qril,             &!odd-level cloud snow mixing ratio               (kg/kg).
   qs                 !odd-level saturation mixing ratio               (kg/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   plgl2,            &!even-level pressure                               (hPa).
   bpgl2,            &!even-level (pl2/p0)**kapa                           (-).     
   phil2,            &!even-level geopotential height                      (m).
   tl2,              &!even-level temperature                    (K).
   ssll2,            &!even-level dry static energy                     (J/kg).
   ql2,              &!even-level water vapor mixing ratio             (kg/kg).
   qcwl2,            &!even-level cloud water mixing ratio             (kg/kg).
   qrwl2,            &!even-level rain mixing ratio                    (kg/kg).
   qcil2,            &!even-level cloud ice mixing ratio               (kg/kg).
   qril2,            &!even-level snow mixing ratio                    (kg/kg).
   qsl2               !even-level saturation mixing ratio              (kg/kg).
     
!output arguments:
!-----------------
real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   hh,               &!odd-level generalized moist static energy        (J/kg).
   hhs,              &!odd-level saturation moist static energy         (J/kg).
   ssvl               !odd-level virtual dry static energy              (J/kg).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm+1):: &
   hhl2               !even-level moist static energy                   (J/kg).

real (kind=dbl_kind), intent(out), dimension(len,nlm):: &
   zl,               &!odd-level height                                    (m).
   tl,               &!odd-level temperature                               (K).
   tvl,              &!odd-level virtual temperature                       (K).
   wghtl,            &!odd-level temperature weight                        (-).
   gaml               !odd-level temperature gradient                         .

real (kind=dbl_kind), intent(out), dimension(len,nlm+1):: &
   hhsl2,            &!even-level saturation moist static energy        (J/kg).
   zl2,              &!even-level height                                   (m).
   tvl2,             &!even-level virtual temperature                      (K).
   ssvl2,            &!even-level virtual dry static energy             (J/kg).
   gaml2              !even-level temperature gradient                        .

!local variables:
integer (kind=int_kind):: l
     
real (kind=dbl_kind):: cpi

real (kind=dbl_kind), dimension(len):: pres, pres2
real (kind=dbl_kind), dimension(len,nlm):: dqs, ql_tem
real (kind=dbl_kind), dimension(len,nlm+1):: dqsl2, tl2tem

!------------------------------------------------------------------------------

cpi      = c1/spec_heat_cp

!initialization of output arrays:

ssvl(1:len,:) = c0 ; hh(1:len,:)   = c0
hhs(1:len,:)  = c0 ; hhl2(1:len,:) = c0

zl  = c0   ; tl  = c0 ; tvl  = c0 ;  gaml = c0 ; wghtl = c0
zl2 = c0   ; tvl2 = c0 ; gaml2 = c0 ; hhsl2 = c0
ssvl2 = c0

!initialization of local arrays:

dqs = c0 ; ql_tem = c0 ; dqsl2 = c0

!odd-level variables needed by the cloud model:

zl (:,:) = phi(1:len,:)*gravi
tl(:,:)  = th(1:len,:)*bpg(1:len,:)
tvl(:,:) = tl(:,:)*(1.+delta*ql(1:len,:)        &
         - loadw*(qcwl(1:len,:) + qrwl(1:len,:) &
         + qcil(1:len,:) + qril(1:len,:)))
ssvl(1:len,:) = ssl(1:len,:) + spec_heat_cp*tl(:,:)*(delta*ql(1:len,:) &
              - loadw*(qcwl(1:len,:)+qrwl(1:len,:)              &
              +        qcil(1:len,:)+qril(1:len,:)))
hh(1:len,:)   = ssl(1:len,:) + hltm*ql(1:len,:) - lfus*qcil(1:len,:)
hhs(1:len,:)  = ssl(1:len,:) + hltm*qs(1:len,:)

ql_tem(:,:) = ql(1:len,:)
do l = 1, nlm
   pres(:) = plg(1:len,l)
   tl2tem(1:len,:) = tl2(1:len,:)
   call eau_sat(len,pres,tl2tem(1,l),dqsats=dqs(1,l),wght=wghtl(1,l))
enddo
gaml(:,:) = hltm*cpi*dqs(:,:) 

do l = 1, nlm+1
   pres2(:) = plgl2(1:len,l)
   call eau_sat(len,pres2 ,tl2(1,l),dqsats=dqsl2(1,l))
enddo

zl2(:,:)   = phil2(1:len,:)*gravi
ssvl2(:,:) = ssll2(1:len,:) + spec_heat_cp*tl2(1:len,:)*(delta*ql2(1:len,:) &
           - loadw*(qcwl2(1:len,:)+qrwl2(1:len,:)                &
	     + qcil2(1:len,:)+qril2(1:len,:))) 
hhl2(1:len,:) = ssll2(1:len,:) + hltm*ql2(1:len,:) - lfus*qcil2(1:len,:)
tvl2(:,:)     = tl2(1:len,:)*(1.+delta*ql2(1:len,:)      & 
              - loadw*(qcwl2(1:len,:)+qrwl2(1:len,:) & 
              + qcil2(1:len,:)+qril2(1:len,:)))
gaml2(:,:) = hltm*cpi*dqsl2(:,:)
hhsl2(:,:) = ssll2(1:len,:) + hltm*qsl2(1:len,:)

end subroutine eau_cupl2

!------------------------------------------------------------------------------
