!------------------------------------------------------------------------------
subroutine eau_cupls &
               (    nlen ,       len ,      nlm ,     lpbl ,        rat , &
                  nunst2 ,     ndxl2 ,   imassl ,     hhl2 ,        ql2 , &
                   qcwl2 ,     qrwl2 ,    qcil2 ,    qril2 ,       eta2 , &
                  hcloud ,   qvcloud ,  qlcloud ,  qrcloud ,    qicloud , &
                 qscloud ,  dqhcloud , dqvcloud , dqlcloud ,   dqrcloud , &
                dqicloud ,  dqscloud ,   pprodr ,   cuprec , cuprec_pbl , &
                   dhcup ,     dtcup ,    dvcup ,    dlcup ,      drcup , &
                   dicup ,     dscup ,    tcuph ,    tcupv ,      tcupl , &
                   tcupr ,     tcupi ,    tcups ,    fcuph ,      fcupv , &
                   fcupl ,     fcupr ,    fcupi ,    fcups                &
               )

use kinds
use eau_params

implicit none
!------------------------------------------------------------------------------
!eau_cupls calculates the large-scale tendencies of moist static energy, water
!vapor, cloud water, rain, cloud ice, and snow due to microphysical processes
!occurring in the convective updrafts.
!eau_cupls calculates the transport of moist static energy, water vapor, cloud
!water, rain, cloud ice, and snow due to convective circulations.
!Laura D. Fowler/slikrock (01-30-01).

!send comments to laura@atmos.colostate.edu.

!modifications:
!    none.

!subroutines called:
!    eau_cuptran: calculates the tendencies due to convective transport by
!                 convective circulations.

!definitions:
!    "odd-level" variables refer to variables in the middle of layers (nlm).
!    "even-level" variables refer to variables at the edges of layers (nlm+1).

!argument list variables:
!input arguments:
!----------------
integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the pbl top.

integer (kind=int_kind), intent(in), dimension(nlm):: &
   nunst2             !total number of cloud-tops per grid-point.

integer (kind=int_kind), intent(in), dimension(len*nlm,2,nlm):: &
   ndxl2              !indexes for cloud-top and cloud-top location.

real (kind=dbl_kind), intent(in), dimension(len):: &
   rat                !scale the convective tendencies when the mass flux is
                      !greater than the PBL depth.
 
real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   ql2,              &!even-level water vapor mixing ratio             (kg/kg).
   qcwl2,            &!even-level cloud water mixing ratio             (kg/kg).
   qrwl2,            &!even-level rain mixing ratio                    (kg/kg).
   qcil2,            &!even-level cloud ice mixing ratio               (kg/kg).
   qril2,            &!even-level snow mixing ratio                    (kg/kg).
   hhl2               !even-level moist static energy                   (J/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm):: &
   imassl             !inverse mass of layer                           (m2/kg).
     
real (kind=dbl_kind), intent(in), dimension(len,nlm-1,nlm-1,nlm):: &
   dqhcloud,         &!in-cloud tendency of moist static energy         (J/kg).
   dqvcloud,         &!in-cloud tendency of water vapor mixing ratio   (kg/kg).
   dqlcloud,         &!in-cloud tendency of cloud water mixing ratio   (kg/kg).
   dqrcloud,         &!in-cloud tendency of rain mixing ratio          (kg/kg).
   dqicloud,         &!in-cloud tendency of cloud ice mixing ratio     (kg/kg).
   dqscloud,         &!in-cloud tendency of snow mixing ratio          (kg/kg).
   pprodr             !in-cloud tendency of precipitation of rain      (kg/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm-1,nlm):: &
   hcloud,           &!in-cloud moist static energy                     (J/kg).
   qvcloud,          &!in-cloud water vapor mixing ratio               (kg/kg).
   qlcloud,          &!in-cloud cloud water mixing ratio               (kg/kg).
   qrcloud,          &!in-cloud rain mixing ratio                      (kg/kg).
   qicloud,          &!in-cloud cloud ice mixing ratio                 (kg/kg).
   qscloud            !in-cloud snow mixing ratio                      (kg/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm,nlm):: &
   eta2               !cumulus mass flux                               (kg/m2).

!output arguments:
!-----------------
!microp. tendency: convective tendency due to cloud microphysics in the
!                  convective updraft.
!conv. transport : convective tendency due to circulation by convective
!                  eddies.
real (kind=dbl_kind), intent(out), dimension(nlen):: &
   cuprec,           &!cumulus precipitation rate                    (kg/m2/s).
   cuprec_pbl         !cumulus precipitation rate from PBL base      (kg/m2/s).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   dtcup,            &!odd-level cumulus heating rate                    (K/s).
   dhcup,            &!odd-level microp. tendency of moist stat. ener.(J/kg/s).
   dvcup,            &!odd-level microp. tendency of water vapor     (kg/kg/s).
   dlcup,            &!odd-level microp. tendency of cloud water     (kg/kg/s).
   drcup,            &!odd-level microp. tendency of rain            (kg/kg/s).
   dicup,            &!odd-level microp. tendency of cloud ice       (kg/kg/s).
   dscup,            &!odd-level microp. tendency of snow            (kg/kg/s).
   tcuph,            &!odd-level conv. transport of moist stat. ener. (J/kg/s).
   tcupv,            &!odd-level conv. transport of water vapor      (kg/kg/s).
   tcupl,            &!odd-level conv. transport of cloud water      (kg/kg/s).
   tcupr,            &!odd-level conv. transport of rain             (kg/kg/s).
   tcupi,            &!odd-level conv. transport cloud ice           (kg/kg/s).
   tcups              !odd-level conv. transport of snow             (kg/kg/s).

!fluxes due to circulation by convective eddies.
real (kind=dbl_kind), dimension(nlen,nlm+1):: &
   fcuph,            &!even-level flux of moist static                  (J/m2).
   fcupv,            &!even-level flux of water vapor                  (kg/m2).
   fcupl,            &!even-level flux of cloud water                  (kg/m2).
   fcupr,            &!even-level flux of rain                         (kg/m2).
   fcupi,            &!even-level flux of cloud ice                    (kg/m2).
   fcups              !even-level flux of snow                         (kg/m2).

!local variables:
integer (kind=int_kind):: ic, il, ll, lpblp1, kn, j
     
real (kind=dbl_kind):: tem, cpi

real (kind=dbl_kind), dimension(len,nlm):: pprec, pprec_pbl

!------------------------------------------------------------------------------
!write(6,*) '--- enter eau_cupls:'

lpblp1 = lpbl+1
cpi   = c1/spec_heat_cp

pprec(:,:) = c0 ; pprec_pbl(:,:)  = c0

!microphysics tendencies and convective precipitation:

do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then

      do kn = 1, nunst2(j)
         ic = ndxl2(kn,1,j) !cloud top index
         il = ndxl2(kn,2,j) !cloud top location
         do ll = ic, j-1               
            tem = imassl(il,ll)*eta2(il,ll,ic,j)
            dhcup(il,ll) = dhcup(il,ll) + tem*dqhcloud(il,ll,ic,j)
            dvcup(il,ll) = dvcup(il,ll) - tem*dqvcloud(il,ll,ic,j)
            dlcup(il,ll) = dlcup(il,ll) + tem*dqlcloud(il,ll,ic,j)
            drcup(il,ll) = drcup(il,ll) + tem*dqrcloud(il,ll,ic,j)
            dicup(il,ll) = dicup(il,ll) + tem*dqicloud(il,ll,ic,j)
            dscup(il,ll) = dscup(il,ll) + tem*dqscloud(il,ll,ic,j)
            pprec(il,ll) = pprec(il,ll) + tem*pprodr(il,ll,ic,j) 
         enddo   
      enddo

   endif   
enddo

do il = 1, len
   do ll = 1, lpbl
      cuprec(il) = cuprec(il) + pprec(il,ll)/imassl(il,ll)
   enddo
enddo

!convective precipitation from PBL cloud base:

if(nunst2(lpblp1) .ne. 0) then
   do kn = 1, nunst2(lpblp1)
      ic = ndxl2(kn,1,lpblp1) !cloud top index
      il = ndxl2(kn,2,lpblp1) !cloud top location
      do ll = ic, lpbl               
         tem = imassl(il,ll)*eta2(il,ll,ic,lpblp1)
	   pprec_pbl(il,ll) = pprec_pbl(il,ll) + tem*pprodr(il,ll,ic,lpblp1)
      enddo 
   enddo
endif
do il = 1, len
   do ll = 1, lpbl
      cuprec_pbl(il) = cuprec_pbl(il) + pprec_pbl(il,ll)/imassl(il,ll)
   enddo
enddo

!convective transport:

!write(6,*) '--- moist static energy:'
call eau_cuptran &
         (  nlen ,    len ,   nlen ,  nlm ,   lpbl , &
          nunst2 ,  ndxl2 , imassl , eta2 , hcloud , &
            hhl2 , .true. ,  fcuph ,  tcuph          &
         )
!write(6,*) '--- water vapor:'
call eau_cuptran &
         (  nlen ,    len ,   nlen ,  nlm ,    lpbl , &
          nunst2 ,  ndxl2 , imassl , eta2 , qvcloud , &
             ql2 , .true. ,  fcupv ,  tcupv           &
         )
!write(6,*) '--- cloud water:'
call eau_cuptran &
         (  nlen ,    len ,   nlen ,  nlm ,    lpbl , &
          nunst2 ,  ndxl2 , imassl , eta2 , qlcloud , &
           qcwl2 , .true. ,  fcupl ,  tcupl           &
         )
!call eau_cuptran &
!         (  nlen ,     len ,   nlen ,   nlm ,    lpbl , &
!          nunst2 ,   ndxl2 , imassl ,  eta2 , qrcloud , &
!           qrwl2 , .false. ,  fcupr , tcupr             &
!         )
!write(6,*) '--- cloud ice:'
call eau_cuptran &
         (  nlen ,    len ,   nlen ,  nlm ,    lpbl , &
          nunst2 ,  ndxl2 , imassl , eta2 , qicloud , &
           qcil2 , .true. ,  fcupi ,  tcupi           &
         )
!write(6,*) '--- snow:'
call eau_cuptran &
         (  nlen ,     len ,   nlen ,   nlm ,    lpbl , &
          nunst2 ,   ndxl2 , imassl ,  eta2 , qscloud , &
           qril2 , .false. ,  fcups , tcups             &
         )

!convective heating rate:

do il = 1, len
   do ll = 1, lpblp1
      dtcup(il,ll) = tcuph(il,ll)+dhcup(il,ll)         &
                   - hltm*(dvcup(il,ll)+tcupv(il,ll)) &
                   + lfus*(dicup(il,ll)+tcupi(il,ll))  &
                   + lfus*(dscup(il,ll)+tcups(il,ll))
      dtcup(il,ll) = cpi*dtcup(il,ll)
   enddo
enddo

!scaling of the convective precipitation rates and tendencies when the
!mass flux is greater than the PBL depth:

cuprec(1:len)     = cuprec(1:len)*rat(:)
cuprec_pbl(1:len) = cuprec_pbl(1:len)*rat(:)

do ll = 1, lpblp1
   dhcup(1:len,ll) = dhcup(1:len,ll)*rat(:)
   dtcup(1:len,ll) = dtcup(1:len,ll)*rat(:)
   dvcup(1:len,ll) = dvcup(1:len,ll)*rat(:)
   dlcup(1:len,ll) = dlcup(1:len,ll)*rat(:)
   drcup(1:len,ll) = drcup(1:len,ll)*rat(:)
   dicup(1:len,ll) = dicup(1:len,ll)*rat(:)
   dscup(1:len,ll) = dscup(1:len,ll)*rat(:)
   tcuph(1:len,ll) = tcuph(1:len,ll)*rat(:)
   tcupv(1:len,ll) = tcupv(1:len,ll)*rat(:)
   tcupl(1:len,ll) = tcupl(1:len,ll)*rat(:)
   tcupr(1:len,ll) = tcupr(1:len,ll)*rat(:)
   tcupi(1:len,ll) = tcupi(1:len,ll)*rat(:)
   tcups(1:len,ll) = tcups(1:len,ll)*rat(:)
enddo

do ll = 1, lpblp1+1
   fcuph(1:len,ll) = fcuph(1:len,ll)*rat(:)
   fcupv(1:len,ll) = fcupv(1:len,ll)*rat(:)
   fcupl(1:len,ll) = fcupl(1:len,ll)*rat(:)
   fcupr(1:len,ll) = fcupr(1:len,ll)*rat(:)
   fcupi(1:len,ll) = fcupi(1:len,ll)*rat(:)
   fcups(1:len,ll) = fcups(1:len,ll)*rat(:)
enddo

!write(6,*) '--- end eau_cupls'
end subroutine eau_cupls

!------------------------------------------------------------------------------
