!------------------------------------------------------------------------------
subroutine eau_cupqps &
               (      ksdm , n_physics ,  nagrid ,   nlm ,    pcos1 , &
                     pcos2 ,     psin1 ,   psin2 ,  dplg ,   cuprec , &
                cuprec_pbl ,      cape ,      mb ,   cke ,   cdwork , &
                    masflx ,     dtcup ,   dhcup , dvcup ,    dlcup , &
                     drcup ,     dicup ,   dscup , tcuph ,    tcupv , &
                     tcupl ,     tcupr ,   tcupi , tcups , inc_diag , &
                       hhl                                            &  
#ifdef passive_tracers
                , npassive ,   dtrccup , tcuptrc                      &
#endif
               )
      
use kinds
use eau_params
use qp_module
use qp_ml
     
implicit none
!------------------------------------------------------------------------------
!eau_cupqps computes the 2D and 3D diagnostics that are related to the eau_cup
!cumulus parameterization.
!Laura D. Fowler/slikrock (04-01-01).

!send comments to laura@atmos.colostate.edu.

!modifications:
!    none.

!subroutines called:
!    qp2sum: saves diagnostics for QP2s diagnostics.
!    qp3sum: saves diagnostics for QP3s diagnostics.
!    hf2sum: saves high-frequency diagnostics.

!argument list variables:
!input arguments:
!----------------
!microp. tendency: convective tendency due to cloud microphysics in the
!                  convective updraft.
!conv. transport : convective tendency due to circulation by convective
!                  eddies.
integer(kind=int_kind), intent(in):: &
   ksdm,             &!loop index.
   n_physics,        &!nb of short-time steps per hour. 
   nagrid,           &!length of horizontal domain with ghost cells.
   nlm                !number of layers.

real(kind=dbl_kind), intent(in):: &
   pcos1,            &!cosine of first diurnal harmonic                    (-).
   pcos2,            &!cosine of first diurnal harmonic                    (-).
   psin1,            &!sine of second diurnal harmonic                     (-).
   psin2              !sine of second diurnal harmonic                     (-).

real(kind=dbl_kind), intent(in), dimension(nagrid):: &
   cuprec,           &!cumulus precipitation rate                    (kg/m2/s).
   cuprec_pbl         !precipitation rate from PBL cloud base        (kg/m2/s).
     
real(kind=dbl_kind), dimension(nagrid,nlm):: &
   dplg,             &!odd-level pressure thickness                       (Pa).
   mb,               &!cloud base mass flux                          (kg/m2/s).
   cape,             &!conditional available potential energy         (J/kg/s).
   dtcup,            &!odd-level cumulus heating rate                    (K/s).
   dhcup,            &!odd-level microp. tendency of moist stat. ener.(J/kg/s).
   dvcup,            &!odd-level microp. tendency of water vapor     (kg/kg/s).
   dlcup,            &!odd-level microp. tendency of cloud water     (kg/kg/s).
   drcup,            &!odd-level microp. tendency of rain            (kg/kg/s).
   dicup,            &!odd-level microp. tendency of cloud ice       (kg/kg/s).
   dscup,            &!odd-level microp. tendency of snow            (kg/kg/s).
   tcuph,            &!odd-level conv. transport of moist stat. ener. (J/kg/s).
   tcupv,            &!odd-level conv. transport of water vapor      (kg/kg/s).
   tcupl,            &!odd-level conv. transport of cloud water      (kg/kg/s).
   tcupr,            &!odd-level conv. transport of rain             (kg/kg/s).
   tcupi,            &!odd-level conv. transport cloud ice           (kg/kg/s).
   tcups,            &!odd-level conv. transport of snow             (kg/kg/s).
   hhl                !odd-level moist static energy                    (J/kg).

real(kind=dbl_kind), intent(in), dimension(nagrid,nlm,nlm):: &
   cke,              &!cumulus kinetic energy equation                  (J/m2).
   cdwork,           &!cloud work function                            (J/kg/s).
   masflx             !cloud base convective mass flux               (kg/m2/s).

real(kind=dbl_kind), intent(in), dimension(nagrid,16):: inc_diag

#ifdef passive_tracers
   integer (kind=int_kind), intent(in):: &
      npassive        !number of passive tracers.

   real (kind=dbl_kind), dimension(nagrid,nlm,npassive):: &
      dtrccup,       &!odd-level microp. tendency of tracerl         (kg/kg/s).
      tcuptrc         !odd-level conv. transport of tracerl          (kg/kg/s).
#endif

!local variables:
integer(kind=int_kind):: i, l, np

real(kind=dbl_kind):: x_phys

real(kind=dbl_kind), dimension(nagrid):: &
   cupinc,           &!incidence of convection                             (-).
   cke_pbl,          &!pbl cloud base cumulus kinetic energy            (J/m2).
   cdwork_pbl,       &!pbl cloud base cloud work function             (J/kg/s).
   dlcupt,           &!vertically integrated detrainment of cloud water(kg/m2).
   dicupt,           &!vertically integrated detrainment of cloud ice  (kg/m2).
   tot_dlcupt,       &!vertically integrated tendency of cloud water   (kg/m2).
   tot_dicupt         !vertically integrated tendency of cloud ice     (kg/m2).

real(kind=dbl_kind), dimension(nagrid,nlm):: &
   dqcup              !convective tendency of total water            (kg/kg/s).

!------------------------------------------------------------------------------

!initialization of local variables:

x_phys   = c1*n_physics

cupinc(:) = c0
do i = 1, nagrid
   if(mb(i,nlm) .ne. c0) cupinc(i) = cupinc(i) + c1
enddo

dqcup(:,:) = c0
dqcup(:,:) = dvcup(:,:) + dlcup(:,:) + drcup(:,:) + dicup(:,:) + dscup(:,:) &
           + tcupv(:,:) + tcupl(:,:) + tcupr(:,:) + tcupi(:,:) + tcups(:,:)

cke_pbl(:)    = ckemin
cdwork_pbl(:) = c0
do i = 1, nagrid
   do l= 1, nlm
      if(cke(i,l,nlm).gt.ckemin) cke_pbl(i) = cke_pbl(i) + cke(i,l,nlm)
      cdwork_pbl(i) = cdwork_pbl(i) + cdwork(i,l,nlm)
   enddo
enddo

!vertically integrated convective source terms:
dlcupt(:) = c0
dicupt(:) = c0
tot_dlcupt(:) = c0
tot_dicupt(:) = c0
do l = 1, nlm
   dlcupt(:)     = dlcupt(:) + dlcup(:,l)*dplg(:,l)*gravi
   dicupt(:)     = dicupt(:) + dicup(:,l)*dplg(:,l)*gravi
   tot_dlcupt(:) = tot_dlcupt(:) + (dlcup(:,l)+tcupl(:,l))*dplg(:,l)*gravi
   tot_dicupt(:) = tot_dicupt(:) + (dicup(:,l)+tcupi(:,l))*dplg(:,l)*gravi
 enddo

!qp2 diagnostics:

 call qp2sum(cupinc,c1,iqp2_cupinc,ksdm,nagrid)
 call qp2sum(cuprec,c1,iqp2_prc,ksdm,nagrid)
 call qp2sum(cuprec,pcos1,iqp2_cupcos1,ksdm,nagrid)
 call qp2sum(cuprec,pcos2,iqp2_cupcos2,ksdm,nagrid)
 call qp2sum(cuprec,psin1,iqp2_cupsin1,ksdm,nagrid)
 call qp2sum(cuprec,psin2,iqp2_cupsin2,ksdm,nagrid)
 call qp2sum(cuprec_pbl,c1,iqp2_cupprpbl,ksdm,nagrid)
 call qp2sum(cke_pbl,c1,iqp2_cupcke,ksdm,nagrid)
 call qp2sum(cdwork_pbl,c1,iqp2_cupcdwork,ksdm,nagrid)
 call qp2sum(cape(:,nlm),c1,iqp2_cupcape,ksdm,nagrid)
 call qp2sum(mb(:,nlm),c1,iqp2_cupmb,ksdm,nagrid)
 call qp2sum(cuprec,x_phys,iqp2_pr,ksdm,nagrid,step_sample=0)
 call qp2sum(cuprec,x_phys*pcos1,iqp2_prcos1,ksdm,nagrid,step_sample=0)
 call qp2sum(cuprec,x_phys*pcos2,iqp2_prcos2,ksdm,nagrid,step_sample=0)
 call qp2sum(cuprec,x_phys*psin1,iqp2_prsin1,ksdm,nagrid,step_sample=0)
 call qp2sum(cuprec,x_phys*psin2,iqp2_prsin2,ksdm,nagrid,step_sample=0)
 call qp2sum(dlcupt,c1,iqp2_dlcupt,ksdm,nagrid)
 call qp2sum(dicupt,c1,iqp2_dicupt,ksdm,nagrid)
 call qp2sum(tot_dlcupt,c1,iqp2_totdlcupt,ksdm,nagrid)
 call qp2sum(tot_dicupt,c1,iqp2_totdicupt,ksdm,nagrid)

!extra qp2 diagnostics used in the convective cloud model:

 call qp2sum(inc_diag(:,1) ,c1,iqp2_cupinc1 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,2) ,c1,iqp2_cupinc2 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,3) ,c1,iqp2_cupinc3 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,4) ,c1,iqp2_cupinc4 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,5) ,c1,iqp2_cupinc5 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,6) ,c1,iqp2_cupinc6 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,7) ,c1,iqp2_cupinc7 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,8) ,c1,iqp2_cupinc8 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,9) ,c1,iqp2_cupinc9 ,ksdm,nagrid)
 call qp2sum(inc_diag(:,10),c1,iqp2_cupinc10,ksdm,nagrid)
 call qp2sum(inc_diag(:,11),c1,iqp2_cupinc11,ksdm,nagrid)
 call qp2sum(inc_diag(:,12),c1,iqp2_cupinc12,ksdm,nagrid)
 call qp2sum(inc_diag(:,13),c1,iqp2_cupinc13,ksdm,nagrid)
 call qp2sum(inc_diag(:,14),c1,iqp2_cupinc14,ksdm,nagrid)
 call qp2sum(inc_diag(:,15),c1,iqp2_cupinc15,ksdm,nagrid)
 call qp2sum(inc_diag(:,16),c1,iqp2_cupinc16,ksdm,nagrid)

!qp3 diagnostics on physics time_step:

 call qp3sum(cke(1,1,nlm),nagrid,x_phys,iqp3_cke,ksdm,nagrid)
 call qp3sum(cdwork(1,1,nlm),nagrid,x_phys,iqp3_cdwork,ksdm,nagrid)
 call qp3sum(masflx(1,1,nlm),nagrid,x_phys,iqp3_masflx,ksdm,nagrid)
 call qp3sum(dtcup(1,1),nagrid,x_phys,iqp3_tntmc,ksdm,nagrid)
 call qp3sum(dqcup(1,1),nagrid,x_phys,iqp3_tnmrc,ksdm,nagrid) 

 call qp3sum(dhcup(1,1),nagrid,x_phys,iqp3_dhcup,ksdm,nagrid)
 call qp3sum(dvcup(1,1),nagrid,x_phys,iqp3_dvcup,ksdm,nagrid)
 call qp3sum(dlcup(1,1),nagrid,x_phys,iqp3_dlcup,ksdm,nagrid)
 call qp3sum(drcup(1,1),nagrid,x_phys,iqp3_drcup,ksdm,nagrid)
 call qp3sum(dicup(1,1),nagrid,x_phys,iqp3_dicup,ksdm,nagrid)
 call qp3sum(dscup(1,1),nagrid,x_phys,iqp3_dscup,ksdm,nagrid)
 call qp3sum(tcuph(1,1),nagrid,x_phys,iqp3_tcuph,ksdm,nagrid)
 call qp3sum(tcupv(1,1),nagrid,x_phys,iqp3_tcupv,ksdm,nagrid)
 call qp3sum(tcupl(1,1),nagrid,x_phys,iqp3_tcupl,ksdm,nagrid)
 call qp3sum(tcupr(1,1),nagrid,x_phys,iqp3_tcupr,ksdm,nagrid)
 call qp3sum(tcupi(1,1),nagrid,x_phys,iqp3_tcupi,ksdm,nagrid)
 call qp3sum(tcups(1,1),nagrid,x_phys,iqp3_tcups,ksdm,nagrid)

 call qp3mlsum(hhl(1,1),nagrid,c1,iqp3ml_hhl,ksdm,nagrid) 
 call qp3mlsum(masflx(1,1,nlm),nagrid,c1,iqp3ml_masflx,ksdm,nagrid)

#ifdef passive_tracers
   do np = 1, npassive
      call qp3sum(dtrccup(1,1,np),nagrid,x_phys,iqp3_dtrccup(np),ksdm,nagrid)
      call qp3sum(tcuptrc(1,1,np),nagrid,x_phys,iqp3_tcuptrc(np),ksdm,nagrid)
   enddo
#endif

!high-frequency hf2 diagnostics:

 call hf2sum(cuprec,c1,ihf2_prc,ksdm,nagrid)
 call hf2sum(cuprec,x_phys,ihf2_pr,ksdm,nagrid,step_sample=0)

end subroutine eau_cupqps

!------------------------------------------------------------------------------
      
