!------------------------------------------------------------------------------
subroutine eau_cupsubs &
               (  nlen ,     len ,   lene ,   nlm , lpbl , &
                nunst2 ,   ndxl2 , imassl ,  eta2 ,  xil , &
                  xile , entrain ,  dxcup , xflux          &
               )

use kinds
use eau_params

implicit none
!------------------------------------------------------------------------------
!eau_cupsubs calculates the compensating subsidence of a large-scale variable
!outside of the convective updrafts.
!Laura D. Fowler/mcbug (01-21-02).

!send comments to laura@atmos.colostate.edu

!modifications:
!   - reindexed the even-level variable xile.
!     Laura D. Fowler/littlebug (01-15-03).

!subroutines called:
!none.

!argument list variables:
!input arguments:
!----------------
logical (kind=log_kind), intent(in):: &
   entrain            !if true, large-scale variable is entrained inside the
                      !convective updraft.

integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub-domain.
   lene,             &!additional length of sub-domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the pbl top.
integer (kind=int_kind), intent(in), dimension(nlm):: &
   nunst2             !location of given cumulus cloud base.
integer (kind=int_kind), intent(in), dimension(len*nlm,2,nlm):: &
   ndxl2              !index for cloud-top and cloud-top location.    

real (kind=dbl_kind), intent(in), dimension(len,nlm):: &
   imassl             !odd-level inverse mass of a layer               (m2/kg).
real (kind=dbl_kind), intent(in), dimension(lene,nlm+1):: &
   xile               !even-level large-scale variable                     (?).
real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   xil                !odd-level large-scale variable                      (?).
real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm,nlm):: &
   eta2               !cumulus mass flux                               (kg/m2).                                        
     
!output arguments:
!-----------------
real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   dxcup  !odd-level divergence of the flux of variable xil due to 
          !large-scale compensating subsidence                             (?).
real (kind=dbl_kind), intent(out), dimension(nlen,nlm+1):: & 
   xflux  !even-level flux of variable xil due to large-scale compensating
          !subsidence                                                      (?).

!local variables:
integer (kind=int_kind):: ic, il, j, kn, l, lpblp1

real (kind=dbl_kind):: xmean
real (kind=dbl_kind), dimension(len,nlm):: xflux1, xflux2

!------------------------------------------------------------------------------

!initialize output and local arrays:

lpblp1 = lpbl+1

dxcup(1:len,1:nlm) = c0 ; xflux(1:len,1:nlm+1) = c0

!large-scale compensating subsidence:

do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then
      do kn = 1, nunst2(j)
         ic = ndxl2(kn,1,j) !cloud top index.
         il = ndxl2(kn,2,j) !cloud top location.

         xflux1(:,:) = c0
         xflux2(:,:) = c0

         xflux1(il,ic) = eta2(il,ic+1,ic,j)*xile(il,ic+1)
         if(entrain) then
            xmean = 0.5_dbl_kind*(xil(il,ic)+xile(il,ic+1))
            xflux2(il,ic) = eta2(il,ic+1,ic,j)*xmean
         else
            xflux2(il,ic) = eta2(il,ic+1,ic,j)*xil(il,ic)
         endif 
         do l = ic+1, j-1
            xflux1(il,l) = &
                eta2(il,l+1,ic,j)*xile(il,l+1)-eta2(il,l,ic,j)*xile(il,l)
            xflux2(il,l) = &
                xil(il,l)*(eta2(il,l+1,ic,j)-eta2(il,l,ic,j))
         enddo
         xflux1(il,j) = -eta2(il,j,ic,j)*xile(il,j)
         xflux2(il,j) = -eta2(il,j,ic,j)*xil(il,j)

         do l = ic, j
            dxcup(il,l) = dxcup(il,l) &
                        - (xflux1(il,l)-xflux2(il,l))*imassl(il,l)
         enddo
      enddo
   endif
enddo

!Only, the flux at the top of the PBL is needed for correction of the tendency
!(refer to the bottom of eau_cup_control):
!write(6,*)
!write(6,*) '--- subroutine eau_cupsubs:'
if(nunst2(lpblp1) > 0) then
   do kn = 1, nunst2(lpblp1)
      ic = ndxl2(kn,1,lpblp1) !cloud top index.
      il = ndxl2(kn,2,lpblp1) !cloud top location.

      xflux(il,lpblp1) = xflux(il,lpblp1) &
            + eta2(il,lpblp1,ic,lpblp1)*(xil(il,lpblp1)-xile(il,lpblp1))
!     write(6,100) nunst2(lpblp1),il,ic,xil(il,lpblp1),xile(il,lpblp1)
   enddo      
endif
!write(6,*)
!do il = 1, len
!  write(6,101) il,lpblp1,xflux(il,lpblp1)
!enddo
100 format(3(1x,i3),3(1x,e15.8))
101 format(2(1x,i3),1x,e15.8)

end subroutine eau_cupsubs

!------------------------------------------------------------------------------
