!------------------------------------------------------------------------------
subroutine eau_cuptracers &
               (   nlen ,     len ,      nlm ,    lpbl , &
                 nunst2 ,   ndxl2 ,       zz ,     zz1 , &
                 imassl ,   lamda ,   zzctop ,    eta2 , &
                trcbase , tracerl , tracerl2 , dtrccup , &
                tcuptrc , fcuptrc                        &
               )

use kinds
use eau_params
implicit none

!------------------------------------------------------------------------------
!eau_cuptracers calculates the tendency of a passive tracer due to detrainment
!at cloud-top, and large-scale compensating subsidence.
!Laura D. Fowler/mcbug (01-30-02).

!send comments to laura@atmos.colostate.edu

!modifications:
!    none

!subroutines called:
!    eau_cupsubs: calculates the tendencies due to large-scale compensating
!                 subsidence.

!argument list variables:
!input arguments:
!----------------
integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the pbl top.

integer (kind=int_kind), intent(in), dimension(nlm):: &
   nunst2             !total number of cloud-tops per grid-point.

integer (kind=int_kind), intent(in), dimension(len*nlm,2,nlm):: &
   ndxl2              !indexes for cloud-top and cloud-top location.

real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
   tracerl,          &!odd-level tracer mixing ratio                   (kg/kg).
   trcbase            !tracer at convective cloud base                 (kg/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   tracerl2           !even-level tracer mixing ratio                  (kg/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm-1,nlm):: &
   lamda              !fractional entrainment rate                  (per m^-1).

real (kind=dbl_kind), intent(in), dimension(len,nlm):: &
   imassl,           &!odd-level inverse mass of layer                 (m2/kg).
   zz                 !odd-level height                                    (m).

real (kind=dbl_kind), intent(in), dimension(len,nlm+1):: &
   zz1                !half-level height                                   (m).

real (kind=dbl_kind), intent(in), dimension(len,nlm-1,nlm):: &
   zzctop             !height of cloud-top                                 (m).

real (kind=dbl_kind), dimension(len,nlm,nlm,nlm):: &
   eta2               !mass flux                                     (kg/m2/s).

!output arguments:
!-----------------
real (kind=dbl_kind), dimension(nlen,nlm):: &
   dtrccup,          &!odd-level detrainment of tracer               (kg/kg/s).
   tcuptrc            !odd-level compensating subsidence of tracer   (kg/kg/s).

real (kind=dbl_kind), dimension(nlen,nlm+1):: &
   fcuptrc            !even-level flux of tracer due to ls subsidence(kg/m2/s).

!local variables:
integer (kind=int_kind):: i, j, k, kn, n

real (kind=dbl_kind):: tem, trcmean, zk, zkb1, zkb2

real (kind=dbl_kind), dimension(len):: zb  

real (kind=dbl_kind), dimension(len,nlm,nlm-1,nlm):: &
   tracercc           !in-cloud tracer mixing ratio                    (kg/kg).

!------------------------------------------------------------------------------

!initialization:

dtrccup(1:len,:)  = c0 ; tcuptrc(1:len,:)  = c0
fcuptrc(1:len,:)  = c0
     
tracercc(:,:,:,:) = c0

zk    = c0 ; zkb1 = c0 ; zkb2 = c0
zb(:) = c0

!lifting of passive tracers in the convective updrafts:

do j = ncb, nlm

   if(nunst2(j).ne.0) then
      do kn = 1, nunst2(j)
         k = ndxl2(kn,1,j) ! cloud-top index
         n = ndxl2(kn,2,j) ! cloud-top location

!cloud base:

         zb(n) = zz1(n,j)
         tracercc(n,j,k,j) = trcbase(n,j)

!higher cloud levels:

         do i = j-1, k+1, -1
            zk   = zz1(n,i)-zz1(n,i+1)
            zkb1 = zz1(n,i)-zb(n)
            zkb2 = zz1(n,i+1)-zb(n)
            tracercc(n,i,k,j) =                                &
                  ((1.0+lamda(n,k,j)*zkb2)*tracercc(n,i+1,k,j) &
                +  lamda(n,k,j)*tracerl(n,i)*zk)               &
                / (1.0+lamda(n,k,j)*zkb1)
               enddo

!cloud-top controlled by amiu:

            zk   = zzctop(n,k,j)-zz1(n,k+1)
            zkb1 = zzctop(n,k,j)-zb(n)
            zkb2 = zz1(n,k+1)-zb(n)

            trcmean = 0.5_dbl_kind*(tracerl(n,k)+tracerl2(n,k+1))
            tracercc(n,k,k,j) =                                   &
                     ((1.0+lamda(n,k,j)*zkb2)*tracercc(n,k+1,k,j) &
                   + lamda(n,k,j)*trcmean*zk+amiu                 &
                   * lamda(n,k,j)*(zz(n,k)-zb(n))*trcmean)        &
                   / (1.0+lamda(n,k,j)*zkb1                       &
                   + amiu*lamda(n,k,j)*(zz(n,k)-zb(n)))

         enddo
      endif

enddo

!detrainment of passive tracers at cloud-top:
do j = ncb, nlm
   if(nunst2(j) .ne. 0) then

      do kn = 1, nunst2(j)
         k = ndxl2(kn,1,j) ! cloud top index
         n = ndxl2(kn,2,j) ! cloud top location

         trcmean = 0.5*(tracerl(n,k)+tracerl2(n,k+1))
         tem = imassl(n,k)*eta2(n,k,k,j)
         dtrccup(n,k) = dtrccup(n,k) + tem*(tracercc(n,k,k,j)-trcmean)
      enddo

   endif
enddo
100 format(3(1x,i3),5(1x,e15.8))

!large-scale compensating subsidence of passive tracers:

call eau_cupsubs &
         (    nlen ,    len ,    nlen ,     nlm ,    lpbl , &
            nunst2 ,  ndxl2 ,  imassl ,    eta2 , tracerl , &
          tracerl2 , .true. , tcuptrc , fcuptrc             &            
         )

end subroutine eau_cuptracers

!------------------------------------------------------------------------------
