!-----------------------------------------------------------------------

 subroutine eau_farrays &
                (  nlen ,    len ,    nlm ,   lpbl, &
                  dtime ,     pl ,     tl ,    qwv, &
                    qcw ,    qrw ,    qci ,    qri, &
                  dlcup ,  drcup ,  dicup ,  dscup, &
                 fpracw , fpsaci , fpsacw , fprevp, &
                 fpsevp                             &
                )

 use kinds
 use eau_params      
 use eau_util

 implicit none
      
!-----------------------------------------------------------------------

! eau_farrays computes the "F-arrays" that are used in eau_liq for the
! computation of cloud microphysical processes, taking into account the
! horizontal cloud fraction.

! send comments to laura@atmos.colostate.edu.

!references:
! Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1995: Liquid and ice 
! cloud microphysics in the CSU GCM. Part I: Model description and simu 
! lated microphysical processes. J. of Climate, 9, 489-529.

! subroutines called:
!     eau_sat: computation of saturation vapor pressures, mixing ratios,
!              and saturation over water and ice.

! argument list variables:
!     input arguments:
!     ----------------

 integer (kind=int_kind), intent(in) :: &
   nlen, &   ! length of total domain.
   len, &    ! length of sub domain.
   nlm, &    ! number of layers.
   lpbl      ! number of layers above the PBL top.

 real (kind=dbl_kind), intent(in) :: &
   dtime     ! microphysical time-step                            (s).

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
   pl, &     ! full-level pressure                               (Pa).
   tl, &     ! full-level temperature                             (K).
   qwv, &    ! full-level water vapor mixing ratio            (kg/kg).
   qcw, &    ! full-level cloud water mixing ratio            (kg/kg).
   qrw, &    ! full-level rain mixing ratio                   (kg/kg).
   qci, &    ! full-level cloud ice mixing ratio              (kg/kg).
   qri       ! full-level snow mixing ratio                   (kg/kg).

 real (kind=dbl_kind), intent(in), dimension(nlen,nlm) :: &
   dlcup, &  ! eau_cup microphysics tendency of cloud water (kg/kg/s).
   drcup, &  ! eau_cup microphysics tendency of rain        (kg/kg/s).
   dicup, &  ! eau_cup microphysics tendency of cloud ice   (kg/kg/s).
   dscup     ! eau_cup microphysics tendency of snow        (kg/kg/s).

! output arguments:
! -----------------

 real (kind=dbl_kind), intent(out), dimension(len,nlm) :: &
   fpracw, & ! f-array: collection of cloud water by rain   (kg/kg/s).
   fpsaci, & ! f-array: collection of cloud ice by snow     (kg/kg/s).
   fpsacw, & ! f-array: collection of cloud water by snow   (kg/kg/s).
   fprevp, & ! f-array: Evaporation of rain                 (kg/kg/s).
   fpsevp    ! f-array: Evaporation of snow                 (kg/kg/s).

! local variables:

 integer (kind=int_kind) :: i,l

 real (kind=dbl_kind) :: acoef

 real (kind=dbl_kind), dimension(len,nlm) :: &
      sh ,     tv ,  rhol ,  rwslp ,    rwv ,   rislp, &
    qcwl ,   qrwl ,  qcil ,   qril , qsatsl , dqsatsl, &
    essl , dtessl , satsl ,  wghtl
 
 real (kind=dbl_kind) :: l_acoef, l_bcoef

 real (kind=dbl_kind), dimension(len,nlm) :: &
    f_aprime , f_bprime , f_asecond, f_bsecond, &
    f_acoef ,  f_bcoef

!-----------------------------------------------------------------------

! write(6,*) '--- enter eau_farrays:'

!---- initialize output and local arrays:

 fpracw(:,:) = c0 ; fpsaci(:,:) = c0
 fpsacw(:,:) = c0 ; fprevp(:,:) = c0
 fpsevp(:,:) = c0
      
 rwslp(:,:)  = c0 ; rwv(:,:) = c0
 rislp(:,:)  = c0
 f_asecond   = c0 ; f_bsecond = c0
 f_acoef     = c0 ; f_bcoef   = c0
      
!---- contribution from eaucup at the top of eauliq:

 qcwl(:,1:lpbl) = qcw(:,1:lpbl) + dlcup(1:len,1:lpbl)*dtime
 qrwl(:,1:lpbl) = qrw(:,1:lpbl) + drcup(1:len,1:lpbl)*dtime
 qcil(:,1:lpbl) = qci(:,1:lpbl) + dicup(1:len,1:lpbl)*dtime
 qril(:,1:lpbl) = qri(:,1:lpbl) + dscup(1:len,1:lpbl)*dtime

!---- computation of the air density:

 do l = 1, lpbl
    do i = 1, len
       sh(i,l)   = qwv(i,l)/(1.+qwv(i,l))
       tv(i,l)   = (c1+0.61*sh(i,l))*tl(i,l)
       rhol(i,l) = pl(i,l)/(rv*tv(i,l))
    enddo
 enddo

!---- computation of saturation values:

 do l = 1, lpbl

    call eau_sat &
             (              len ,             pl(1,l), &
                        tl(1,l) ,       qwvl=qwv(1,l), &
              qsats=qsatsl(1,l) , dqsats=dqsatsl(1,l), &
                  ess=essl(1,l) ,   dtess=dtessl(1,l), &
                sats=satsl(1,l) ,     wght=wghtl(1,l)  &
             )

 enddo
   
!---- computation of the fallspeeds for rain and snow:

 do l = 1, lpbl
    do i = 1, len
       if (qrwl(i,l) >= 1.e-09) then
          rwslp(i,l) = pielq*rhor*nzeror/ &
                     (rhol(i,l)*qrwl(i,l))
          rwslp(i,l) = rwslp(i,l)**0.25
          rwv(i,l)   = (-.267+206.e02/rwslp(i,l) &
                     -2.045e+07/(rwslp(i,l)**2)  &
                     +9.06e09/(rwslp(i,l)**3))
          rwv(i,l)   = rwv(i,l)*(pzero/pl(i,l))**0.4
          if (rwv(i,l) < c0) rwv(i,l) = c0
!          if (rwv(i,l).gt.5.) rwv(i,l) = 5.
       endif
    enddo
 enddo
      
 do l = 1, lpbl
    do i = 1, len
       if (qril(i,l) >= 1.e-09) then
          rislp(i,l) = pielq*rhos*nzeros/(rhol(i,l)*qril(i,l))
          rislp(i,l) = rislp(i,l)**0.25
       endif
    enddo
 enddo


!---- collection of cloud water by rain:

 do l = 1, lpbl
    do i = 1, len
       if (qrwl(i,l) >= 1.e-09 .and. tl(i,l) >= t00) then
          acoef = -.534*rwslp(i,l)+3.09e04-2.454e07/ &
            rwslp(i,l)+9.0609/rwslp(i,l)**2
          fpracw(i,l) = 0.25*erccoef*acoef*(rhol(i,l)/rhor)
          fpracw(i,l) = fpracw(i,l)*(pzero/pl(i,l))**0.4 
       endif
    enddo
 enddo

!---- collection of cloud water by snow:

 do l = 1, lpbl
    do i = 1, len
       if (qril(i,l) >= 1.e-09 .and. tl(i,l) < t00) then
          fpsacw(i,l) = 0.25*asecond*esccoef*gams1* &
                    (rhol(i,l)/rhos)/rislp(i,l)**(belq-1)
          fpsacw(i,l) = fpsacw(i,l)*(pzero/pl(i,l))**0.4 
       endif
    enddo
 enddo

!---- collection of cloud ice by snow:

 do l = 1, lpbl
    do i = 1, len
       if (qril(i,l) >= 1.e-09 .and. tl(i,l) < t00) then
          fpsaci(i,l) = 0.25*asecond*esicoef*gams1* &
                   (rhol(i,l)/rhos)/rislp(i,l)**(belq-1)
          fpsaci(i,l) = fpsaci(i,l)*(pzero/pl(i,l))**0.4 
       endif
    enddo
 enddo

!---- deposition/evaporation of rain:

 f_aprime = c0 ; f_bprime = c0
 f_acoef  = c0 ; f_bcoef  = c0
      
 l_acoef = (pielq*rhor*nzeror)**0.50
 l_bcoef = (pielq*rhor*nzeror)**0.25

 do i = 1, len
    do l = 1, lpbl
       if (qrwl(i,l) >= 1.e-09 .and. tl(i,l) >= t00) then
          f_aprime(i,l) = hltm/(rv*tl(i,l))-1.
          f_aprime(i,l) = hltm/(therco*tl(i,l))*f_aprime(i,l)
          f_bprime(i,l) = rv*tl(i,l)/(diffelq*essl(i,l))
               
          f_bcoef(i,l) = sqrt(aprime*rhol(i,l)/muelq)
          f_bcoef(i,l) = 0.31*gam3*(pzero/pl(i,l))**0.2*f_bcoef(i,l)
          f_bcoef(i,l) = f_bcoef(i,l)*l_bcoef/(rhol(i,l)*qrwl(i,l))**0.25
     
          f_acoef(i,l) = 0.78*l_acoef/(rhol(i,l)*qrwl(i,l))**0.50
     
          fprevp(i,l) = 2./rhor/(f_aprime(i,l)+f_bprime(i,l))
          fprevp(i,l) = fprevp(i,l)*(f_acoef(i,l) + f_bcoef(i,l))
       endif
    enddo
 enddo               

!---- deposition/evaporation of snow:

 f_aprime = c0 ; f_bprime = c0
 f_acoef  = c0 ; f_bcoef  = c0

 l_acoef = (pielq*rhos*nzeros)**(-1./8*(belq+1))
 l_bcoef = (pielq*rhos*nzeros)**(1./8*(belq-3))

 do i = 1, len
    do l = 1, lpbl         
       if (qril(i,l) >= 1.e-09 .and. tl(i,l) < t00) then
          f_asecond(i,l) = lsub/(rv*tl(i,l))-c1
          f_asecond(i,l) = lsub/(therco*tl(i,l))*f_asecond(i,l)
          f_bsecond(i,l) = rv*tl(i,l)/(diffelq*essl(i,l))

          f_bcoef(i,l) = sqrt(asecond*rhol(i,l)/muelq)
          f_bcoef(i,l) = 0.44*gams2*(pzero/pl(i,l))**0.2*f_bcoef(i,l)

          f_acoef(i,l) = 0.65/l_acoef  &
                       * (rhol(i,l)*qril(i,l))**(-1./8*(belq+1))  &
                       + f_bcoef(i,l)
          f_acoef(i,l) = (rhol(i,l)*qril(i,l))**(1./8*(belq-3))  &
                       / l_bcoef*f_acoef(i,l)
          fpsevp(i,l)  = 4./(pielq*rhos)  &
                         / (f_asecond(i,l)+f_bsecond(i,l))
          fpsevp(i,l)  = f_acoef(i,l)*fpsevp(i,l)
       endif
    enddo
 enddo

! write(6,*) '--- end eau_farrays'

 end subroutine eau_farrays

!-----------------------------------------------------------------------
