!-----------------------------------------------------------------------

 subroutine eau_liq &
                (    nlen ,      len, &
                      nlm ,     lpbl, &
                    dtdyn ,      pp2, &
                     bpp2 ,       pp, &
                      bpp ,      dpp, &
                       th ,      qwv, &
                      qcw ,      qrw, &
                      qci ,      qri, &
                    dlcup ,    drcup, &
                    dicup ,    dscup, &
                     d_th ,     d_wv, &
                     d_cw ,     d_rw, &
                     d_ci ,     d_ri, &
                    d_rwf ,    d_rif, &
                 elqrprec , elqsprec  &
                )
     
 use kinds
 use eau_params

 implicit none

!-----------------------------------------------------------------------

! eau_liq calculates the tendencies of temperature, and water vapor,
! cloud water, rain, cloud ice, and snow mixing ratios due to cloud
! microphysics processes.
! Laura D. Fowler/slikrock (02-18-00).

! note:
! This code is under development to include a prognostic equation of the
! fractional cloudiness. Send comments to laura@atmos.colostate.edu if
! want to modify the code.
! Laura D. Fowler/slikrock (02-01-01).

! references:
! Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1996: Liquid and Ice
! Cloud Microphysics in the CSU General Circulation Model.
! Part I: Model description and simulated Microphysical processes.
! Journal of Climate, 9, 489-529.

! Randall, D.A., and L.D. Fowler, 1999: EAUliq: The next generation.
! Dept. of Atmospheric Science Paper 673, Dept. of Atmospheric Science.
! Colorado State University, Fort Collins, Colorado, 65 pp.

! modifications:
!     none.

! subroutines called:

!      eau_farrays  :computes the effects of collection of cloud water
!                    by rain and snow, and collection of cloud ice by
!                    snow.
!                    computes the effects of deposition and evaporation
!                    of rain and snow.
!      eau_melt     :computes freezing/melting of cloud water/ice and
!                    rain/snow.
!      eau_gather   :gathers cloud water, cloud ice, rain, and snow into
!                    super-saturation and sub-saturated categories.
!      eau_subsat   :solves the cloud microphysics equations for sub-satu
!                    ration conditions.
!      eau_supsat   :solves the cloud microphysics equations for super-
!                    saturation conditions.
!      eau_prec     :computes the precipitation of rain and snow.
!      eau_mass     :computes the conservation of mass.

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: &
   nlen, &   ! Length of total domain.
   len, &    ! Length of sub domain.
   nlm, &    ! Number of layers.
   lpbl      ! Number of layers above the PBL top.

 real (kind=dbl_kind), intent(in) :: &
   dtdyn  !Dynamical time-step                                (s).

 real (kind=dbl_kind), intent(in), dimension(nlen,nlm) :: &
   pp, &     ! Layer pressure                                    (Pa).
   bpp, &    ! (pp/p0)**kapa                                      (-).
   dpp, &    ! Pressure thickness                                (Pa).
   th, &     ! Potential temperature                              (K).
   qwv, &    ! Water vapor mixing ratio                       (kg/kg).
   qcw, &    ! Cloud water mixing ratio                       (kg/kg).
   qrw, &    ! Rain mixing ratio                              (kg/kg).
   qci, &    ! Cloud ice mixing ratio                         (kg/kg).
   qri, &    ! Snow mixing ratio                              (kg/kg).
   dlcup, &  ! eau_cup microphysics tendency of cloud water (kg/kg/s).
   drcup, &  ! eau_cup microphysics tendency of rain        (kg/kg/s).
   dicup, &  ! eau_cup microphysics tendency of cloud ice   (kg/kg/s).
   dscup     ! eau_cup microphysics tendency of snow        (kg/kg/s).

 real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1) :: &
   pp2, &    ! Level pressure                                    (Pa).
   bpp2      ! (pp2/p0)**kapa                                     (-).

! output arguments:
! -----------------

 real (kind=dbl_kind), intent(out), dimension(nlen) :: &
   elqrprec, &  ! Precipitation of rain                        (kg/m2).
   elqsprec     ! Precipitation of snow                        (kg/m2).

 real (kind=dbl_kind), intent(out), dimension(nlen,nlm) :: &
   d_th, &      ! Tendency of temperature                        (K/s).
   d_wv, &      ! Tendency of water vapor mixing ratio       (kg/kg/s).
   d_cw, &      ! Tendency of cloud water mixing ratio       (kg/kg/s).
   d_rw, &      ! Tendency of rain mixing ratio              (kg/kg/s).
   d_ci, &      ! Tendency of cloud ice mixing ratio         (kg/kg/s).
   d_ri, &      ! Tendency of snow mixing ratio              (kg/kg/s).
   d_rwf, &     ! Tendency of rain due to precipitation      (kg/kg/s).
   d_rif        ! Tendency of snow due to precipitation      (kg/kg/s).

! local variables:

 logical (kind=log_kind), parameter :: w_diag = .false.
      
 integer (kind=int_kind) :: i, ielq, l, nn1, nn2, nn3, np1, np2, np3

 integer (kind=int_kind), dimension(len*nlm) :: &
   inn1, inn2, inn3, inp1, inp2, inp3
          
 real (kind=dbl_kind) :: dtdyni, dtelq, dtelqi

 real (kind=dbl_kind), dimension(len) :: &
     elqr ,    dqelqr ,      elqs , dqelqs, &
   dlcupt ,    drcupt ,    dicupt , dscupt, &
    d_wvt ,     d_cwt ,     d_rwt ,  d_cit, &
    d_rit , dcup_mass , delq_mass , dpgrav

 real (kind=dbl_kind), dimension(len,nlm) :: &
       thl ,      tl ,      pl ,     dpl ,     bpl, &
      qwvl ,    qcwl ,    qrwl ,    qcil ,    qril, &
   dneg_cw , dneg_rw , dneg_ci , dneg_ri ,   satsl, &
    qsatsl , dqsatsl ,   wghtl , fpracwl , fpsacwl, &
   fpsacil , fprevpl , fpsevpl , qrwfall , qrifall, &
   qprautl , qpsautl

 real (kind=dbl_kind), dimension(len,nlm+1) :: pl2, bpl2

!------------------------------------------------------------------------------

!     write(6,*) '--- enter eau_liq:'

 dtdyni   = 1/dtdyn
 if (instant_prec) then
    dtelq = dtdyn
 else
    dtelq    = dtdyn/nc1elq
 endif
 dtelqi   = 1/dtelq

!---- initialization of output variables:

 ielq = 0

 elqrprec(1:len) = c0 ; elqsprec(1:len) = c0
      
 d_th(1:len,:)  = c0 ; d_wv(1:len,:)  = c0
 d_cw(1:len,:)  = c0 ; d_rw(1:len,:)  = c0
 d_ci(1:len,:)  = c0 ; d_ri(1:len,:)  = c0
 d_rwf(1:len,:) = c0 ; d_rif(1:len,:) = c0

!---- initialization of local variables:

 elqr(:)   = c0
 elqs(:)   = c0

 thl(:,:)  = th(1:len,:)
 tl(:,:)   = th(1:len,:)*bpp(1:len,:)
 qwvl(:,:) = qwv(1:len,:)
 qcwl(:,:) = qcw(1:len,:)
 qrwl(:,:) = qrw(1:len,:)
 qcil(:,:) = qci(1:len,:)
 qril(:,:) = qri(1:len,:)
 bpl(:,:)  = bpp(1:len,:)
 bpl2(:,:) = bpp2(1:len,:)
 pl(:,:)   = pp(1:len,:)
 dpl(:,:)  = dpp(1:len,:)
 pl2(:,:)  = pp2(1:len,:)

 dneg_cw(1:len,:) = c0 ; dneg_rw(1:len,:) = c0 
 dneg_ci(1:len,:) = c0 ; dneg_ri(1:len,:) = c0

!---- compute f-arrays:

 call eau_farrays &
          (   nlen ,     len ,     nlm ,    lpbl, &
             dtelq ,      pl ,      tl ,    qwvl, &
              qcwl ,    qrwl ,    qcil ,    qril, &
             dlcup ,   drcup ,   dicup ,   dscup, &
           fpracwl , fpsacil , fpsacwl , fprevpl, &
           fpsevpl                                &
          )

 if (instant_prec) then
      
! no sub-cycling, precip falls out entirely
    if (w_diag) then
       write(6,*)
       write(6,*) '--- eau_liq instant_rec '
    endif

    wghtl(:,:)   = c0 ; qsatsl(:,:) = c0
    dqsatsl(:,:) = c0 ; satsl(:,:)  = c0

    dqelqr(:)    = c0 ; dqelqs(:)    = c0
    qprautl(:,:) = c0 ; qpsautl(:,:) = c0
    qrwfall(:,:) = c0 ; qrifall(:,:) = c0

!---- contribution from eaucup microphysics and transport:

    qcwl(:,1:nlm) = qcwl(:,1:nlm) + dlcup(1:len,1:nlm)*dtelq
    qrwl(:,1:nlm) = qrwl(:,1:nlm) + drcup(1:len,1:nlm)*dtelq
    qcil(:,1:nlm) = qcil(:,1:nlm) + dicup(1:len,1:nlm)*dtelq
    qril(:,1:nlm) = qril(:,1:nlm) + dscup(1:len,1:nlm)*dtelq

    do l = 1, nlm
       do i = 1, len
          if (qcwl(i,l) < c0) then
             dneg_cw(i,l) = dneg_cw(i,l) + qcwl(i,l)
             qcwl(i,l) = c0               
          endif   
          if (qrwl(i,l) < c0) then
             dneg_rw(i,l) = dneg_rw(i,l) + qrwl(i,l)              
             qrwl(i,l) = c0
          endif   
          if (qcil(i,l) < c0) then
             dneg_ci(i,l) = dneg_ci(i,l) + qcil(i,l)                                           
             qcil(i,l) = c0
          endif   
          if (qril(i,l) < c0) then
             dneg_ri(i,l) = dneg_ri(i,l) + qril(i,l)                                                          
             qril(i,l) = c0
          endif   
       enddo   
    enddo

!---- remove all water species (and cloud fraction) from the pbl:

    elqr(1:len)  = elqr(1:len)  &
                 + (qcwl(:,nlm)+qrwl(:,nlm))*dpl(:,nlm)*gravi
    elqs(1:len)  = elqs(1:len)  &
                 + (qcil(:,nlm)+qril(:,nlm))*dpl(:,nlm)*gravi
    qcwl(:,nlm)  = c0
    qrwl(:,nlm)  = c0
    qcil(:,nlm)  = c0
    qril(:,nlm)  = c0

!---- check 1 for conservation of mass and energy:

    if (w_diag) then
       write(6,*)
       write(6,*) '--- before microphysics:'
       call eau_mass &
                ( len ,  nlm , lpbl ,  dpl, &
                 qwvl , qcwl , qrwl , qcil, &
                 qril , elqr , elqs         &
                )
    endif


    do l = 1,lpbl
      
! the following eauliq operations in this loop are done one layer at a time.
! all precip falls out into the next layer, where it becomes the initialization
! of rain and snow for that layer.

!---- freezing/melting of cloud water/ice and rain/snow:
      call eau_melt &
               ( len       ,         1 , 1        , &
                 tl(1,l)   , qcwl(1,l) , qrwl(1,l), &
                 qcil(1,l) , qril(1,l)              &
               )

!---- gathering of grid-points:

      call eau_gather &
               (   len ,      1 ,   1                   , &
                   pl(1,l) ,     tl(1,l) ,   qwvl(1,l)  , &
                   qcwl(1,l) ,   qrwl(1,l) ,   qcil(1,l), &
                   qril(1,l) ,  wghtl(1,l) , qsatsl(1,l), &
                   dqsatsl(1,l) ,  satsl(1,l) ,    nn1  , &
                   nn2 ,    nn3 ,    np1                , &
                   np2 ,    np3 ,   inn1                , &
                   inn2 ,   inn3 ,   inp1               , &
                   inp2 ,   inp3                          &
               )

!---- subsaturation conditions:

      call eau_subsat &
               (    len ,       1 ,     nn1 ,     nn2                    , &
                    nn3 ,    inn1 ,    inn2 ,    inn3                    , &
                  dtelq ,      tl(1,l) ,    qwvl(1,l) ,    qcwl(1,l)     , &
                   qrwl(1,l) ,    qcil(1,l) ,    qril(1,l) ,   satsl(1,l), &
                fpracwl(1,l) , fpsacil(1,l) , fpsacwl(1,l) , fprevpl(1,l), &
                fpsevpl(1,l) ,   wghtl(1,l)                                &
               )

!---- supersaturation conditions:

      call eau_supsat &
               (    len ,     nlm ,     np1 ,     np2                    , &
                    np3 ,    inp1 ,    inp2 ,    inp3                    , &
                  dtelq ,      tl(1,l) ,    qwvl(1,l) ,    qcwl(1,l)     , &
                   qrwl(1,l) ,    qcil(1,l) ,    qril(1,l) ,  qsatsl(1,l), &
                dqsatsl(1,l) , fpracwl(1,l) , fpsacil(1,l) , fpsacwl(1,l), &
                fprevpl(1,l) , fpsevpl(1,l) ,   wghtl(1,l)                 &
               )

!---- additional autoconversion of cloud water/ice to rain/snow:
 
      do i = 1, len   
         if (qcwl(i,l) >= qcw0) then
            qprautl(i,l) = alphaelq*(qcwl(i,l)-qcw0)*dtelq
            qcwl(i,l)    = qcwl(i,l) - qprautl(i,l)
            qrwl(i,l)    = qrwl(i,l) + qprautl(i,l)
         endif
      enddo

      do i = 1, len   
         if (qcil(i,l) >= qci0) then
            qpsautl(i,l) = betaelq*exp(.025*(tl(i,l)-t00)) &
                         * (qcil(i,l)-qci0)*dtelq
            qcil(i,l)    = qcil(i,l) - qpsautl(i,l)
            qril(i,l)    = qril(i,l) + qpsautl(i,l)
         endif
      enddo

!---- falling of rain and snow:
!    all rain and snow fall into next layer
      do i = 1,len
         qril(i,l+1) = qril(i,l+1) + dpl(i,l) * qril(i,l) / dpl(i,l+1)
         qrifall(i,l) = qrifall(i,l) - qril(i,l) * dtelqi
         qrifall(i,l+1) = qrifall(i,l+1) + qril(i,l+1) * dtelqi
         qril(i,l) = c0
            
         qrwl(i,l+1) = qrwl(i,l+1) + dpl(i,l) * qrwl(i,l) / dpl(i,l+1)
         qrwfall(i,l) = qrwfall(i,l) - qrwl(i,l) * dtelqi
         qrwfall(i,l+1) = qrwfall(i,l+1) + qrwl(i,l+1) * dtelqi
         qrwl(i,l) = c0
      enddo

 
    enddo ! do l = 1,lpbl

    do i = 1,len      
       elqr(i) = elqr(i) + qrwl(i,lpbl+1) * dpl(i,lpbl+1) * gravi
       qrwl(i,lpbl+1) = c0
       qrwfall(i,lpbl+1) = c0
       elqs(i) = elqs(i) + qril(i,lpbl+1) * dpl(i,lpbl+1) * gravi
       qril(i,lpbl+1) = c0
       qrifall(i,lpbl+1) = c0
    enddo
    d_rwf(1:len,1:lpbl) = d_rwf(1:len,1:lpbl) + qrwfall(:,1:lpbl)
    d_rif(1:len,1:lpbl) = d_rif(1:len,1:lpbl) + qrifall(:,1:lpbl) 
      
    if (w_diag) then
       write(6,*)
       write(6,*) '--- after precipitation:'
       call eau_mass &
                ( len ,  nlm , lpbl ,  dpl, &
                 qwvl , qcwl , qrwl , qcil, &
                 qril , elqr , elqs         &
                )
    endif

 else !if instant_prec
      
!---- begin sub-cycling:

   do while (ielq < nc1elq)
      ielq = ielq + 1
      if (w_diag) then
         write(6,*)
         write(6,*) '--- eau_liq ielq = ', ielq
      endif

      wghtl(:,:)   = c0 ; qsatsl(:,:) = c0
      dqsatsl(:,:) = c0 ; satsl(:,:)  = c0

      dqelqr(:)    = c0 ; dqelqs(:)    = c0
      qprautl(:,:) = c0 ; qpsautl(:,:) = c0
      qrwfall(:,:) = c0 ; qrifall(:,:) = c0

!---- contribution from eaucup microphysics and transport:

      qcwl(:,1:nlm) = qcwl(:,1:nlm) + dlcup(1:len,1:nlm)*dtelq
      qrwl(:,1:nlm) = qrwl(:,1:nlm) + drcup(1:len,1:nlm)*dtelq
      qcil(:,1:nlm) = qcil(:,1:nlm) + dicup(1:len,1:nlm)*dtelq
      qril(:,1:nlm) = qril(:,1:nlm) + dscup(1:len,1:nlm)*dtelq

      do l = 1, nlm
         do i = 1, len
            if (qcwl(i,l) < c0) then
               dneg_cw(i,l) = dneg_cw(i,l) + qcwl(i,l)
               qcwl(i,l) = c0               
            endif   
            if (qrwl(i,l) < c0) then
               dneg_rw(i,l) = dneg_rw(i,l) + qrwl(i,l)              
               qrwl(i,l) = c0
            endif   
            if (qcil(i,l) < c0) then
               dneg_ci(i,l) = dneg_ci(i,l) + qcil(i,l)                                           
               qcil(i,l) = c0
            endif   
            if (qril(i,l) < c0) then
               dneg_ri(i,l) = dneg_ri(i,l) + qril(i,l)                                                          
               qril(i,l) = c0
            endif   
         enddo   
      enddo

!---- remove all water species (and cloud fraction) from the pbl:

      elqr(1:len)  = elqr(1:len) & 
                   + (qcwl(:,nlm)+qrwl(:,nlm))*dpl(:,nlm)*gravi
      elqs(1:len)  = elqs(1:len) &
                   + (qcil(:,nlm)+qril(:,nlm))*dpl(:,nlm)*gravi
      qcwl(:,nlm)  = c0
      qrwl(:,nlm)  = c0
      qcil(:,nlm)  = c0
      qril(:,nlm)  = c0

!---- check 1 for conservation of mass and energy:

      if (w_diag) then
         write(6,*)
         write(6,*) '--- before microphysics:'
         call eau_mass &
                  ( len ,  nlm , lpbl ,  dpl, &
                   qwvl , qcwl , qrwl , qcil, &
                   qril , elqr , elqs         &
                  )
      endif

!---- freezing/melting of cloud water/ice and rain/snow:

      call eau_melt &
               ( len ,  nlm , lpbl, &
                  tl , qcwl , qrwl, &
                qcil , qril         &
               )

!---- gathering of grid-points:

      call eau_gather &
               (    len ,    nlm ,   lpbl, &
                     pl ,     tl ,   qwvl, &
                   qcwl ,   qrwl ,   qcil, &
                   qril ,  wghtl , qsatsl, &
                dqsatsl ,  satsl ,    nn1, &
                    nn2 ,    nn3 ,    np1, &
                    np2 ,    np3 ,   inn1, &
                   inn2 ,   inn3 ,   inp1, &
                   inp2 ,   inp3           &
               )

!---- subsaturation conditions:

      call eau_subsat &
               (    len ,     nlm ,     nn1 ,     nn2, &
                    nn3 ,    inn1 ,    inn2 ,    inn3, &
                  dtelq ,      tl ,    qwvl ,    qcwl, &
                   qrwl ,    qcil ,    qril ,   satsl, &
                fpracwl , fpsacil , fpsacwl , fprevpl, &
                fpsevpl ,   wghtl                      &
               )

!---- supersaturation conditions:

      call eau_supsat &
               (    len ,     nlm ,     np1 ,     np2, &
                    np3 ,    inp1 ,    inp2 ,    inp3, &
                  dtelq ,      tl ,    qwvl ,    qcwl, &
                   qrwl ,    qcil ,    qril ,  qsatsl, &
                dqsatsl , fpracwl , fpsacil , fpsacwl, &
                fprevpl , fpsevpl ,   wghtl            &
               )

!---- additional autoconversion of cloud water/ice to rain/snow:
 
      do l = 1, lpbl 
         do i = 1, len   
            if (qcwl(i,l) >= qcw0) then
               qprautl(i,l) = alphaelq*(qcwl(i,l)-qcw0)*dtelq
               qcwl(i,l)    = qcwl(i,l) - qprautl(i,l)
               qrwl(i,l)    = qrwl(i,l) + qprautl(i,l)
            endif
         enddo
      enddo

      do l = 1, lpbl
         do i = 1, len   
            if (qcil(i,l) >= qci0) then
               qpsautl(i,l) = betaelq*exp(.025*(tl(i,l)-t00)) &
                            * (qcil(i,l)-qci0)*dtelq
               qcil(i,l)    = qcil(i,l) - qpsautl(i,l)
               qril(i,l)    = qril(i,l) + qpsautl(i,l)
            endif
         enddo
      enddo

!---- falling of rain and snow:

      call eau_prec &
               (   len ,     nlm ,    lpbl ,  dtelq, &
                   bpl ,      pl ,     dpl ,     tl, &
                  qwvl ,    qrwl ,    qril ,   bpl2, &
                   pl2 , qrwfall , qrifall , dqelqr, &
                dqelqs                               &
               )
 
      elqr(1:len) = elqr(1:len) + dqelqr(:)
      elqs(1:len) = elqs(1:len) + dqelqs(:)
      qrwl(:,1:lpbl) = qrwl(:,1:lpbl)+qrwfall(:,1:lpbl)*dtelq
      qril(:,1:lpbl) = qril(:,1:lpbl)+qrifall(:,1:lpbl)*dtelq
      d_rwf(1:len,1:lpbl) = d_rwf(1:len,1:lpbl) + qrwfall(:,1:lpbl)
      d_rif(1:len,1:lpbl) = d_rif(1:len,1:lpbl) + qrifall(:,1:lpbl) 

      if (w_diag) then
         write(6,*)
         write(6,*) '--- after precipitation:'
         call eau_mass &
                  ( len ,  nlm , lpbl ,  dpl, &
                   qwvl , qcwl , qrwl , qcil, &
                   qril , elqr , elqs         &
                  )
      endif

   enddo
  
 endif ! if(instant_prec)

!---- computes microphysical tendencies:

 d_th(1:len,:)   = (tl(:,:)-th(1:len,:)*bpp(1:len,:))*dtdyni
 d_wv(1:len,:)   = (qwvl(:,:)-qwv(1:len,:))*dtdyni      
 d_cw(1:len,:)   = (qcwl(:,:)-qcw(1:len,:)+dneg_cw(:,:))*dtdyni
 d_rw(1:len,:)   = (qrwl(:,:)-qrw(1:len,:)+dneg_rw(:,:))*dtdyni
 d_ci(1:len,:)   = (qcil(:,:)-qci(1:len,:)+dneg_ci(:,:))*dtdyni
 d_ri(1:len,:)   = (qril(:,:)-qri(1:len,:)+dneg_ri(:,:))*dtdyni

 elqrprec(1:len) = elqr(:)*dtdyni
 elqsprec(1:len) = elqs(:)*dtdyni

!---- check total water budget:

 if (w_diag) then
    delq_mass(:) = c0 ; dcup_mass(:) = c0
    dlcupt(:)    = c0 ; drcupt(:)    = c0
    dicupt(:)    = c0 ; dscupt(:)    = c0
    d_wvt(:)     = c0 ; d_cwt(:)     = c0
    d_rwt(:)     = c0 ; d_cit(:)     = c0
    d_rit(:)     = c0

    do l = 1, nlm
       dpgrav(:) = dpl(:,l)*gravi         
       d_wvt(:)  = d_wvt(:)  + d_wv(1:len,l)*dpgrav(:)
	   d_cwt(:)  = d_cwt(:)  + d_cw(1:len,l)*dpgrav(:)
	   d_rwt(:)  = d_rwt(:)  + d_rw(1:len,l)*dpgrav(:)
	   d_cit(:)  = d_cit(:)  + d_ci(1:len,l)*dpgrav(:)
	   d_rit(:)  = d_rit(:)  + d_ri(1:len,l)*dpgrav(:)
       dlcupt(:) = dlcupt(:) + dlcup(1:len,l)*dpgrav(:)
	   drcupt(:) = drcupt(:) + drcup(1:len,l)*dpgrav(:)
	   dicupt(:) = dicupt(:) + dicup(1:len,l)*dpgrav(:)
	   dscupt(:) = dscupt(:) + dscup(1:len,l)*dpgrav(:)
    enddo
    dcup_mass(:) = dlcupt(:) + drcupt(:) + dicupt(:) + dscupt(:)
    delq_mass(:) = d_wvt(:) + d_cwt(:) + d_rwt(:) + d_cit(:) + d_rit(:)
    delq_mass(:) = delq_mass(:) + (elqr(:)+elqs(:))*dtdyni
      
    do i = 1, len
       write(6,101) i, dcup_mass(i),delq_mass(i),dcup_mass(i)-delq_mass(i)
    enddo
 endif
 101 format('--- elq: ',i3,3(1x,e15.8),/)

! write(6,*) '--- end eau_liq'

 end subroutine eau_liq

!-----------------------------------------------------------------------
