!----------------------------------------------------------------------

 subroutine eau_liq_control             &
                (    nagrid ,     ksdm, &
                    do_phys ,      nlm, &
                       lpbl ,      nls, &
                      dtdyn ,    plgl2, &
                      bpgl2 ,    phil2, &
                        plg ,      bpg, &
                       phil ,      thp, &
                    tracerp , delq_cup, &
                   elqrprec , elqsprec, &
                     dt_thp , dt_hydro, &
                 deaul_vtot             &
                )   
    
 use kinds
 use eau_params
 use time_manager_module
 use tracer_names
      
 implicit none

!----------------------------------------------------------------------

! eau_control is the driver for the eau_liq parameterization of large-
! scale cloud microphysics. eau_liq runs on the dynamical time-step.
! Laura D. Fowler/slikrock (03-01-01).

! send comments to laura@atmos.colostate.edu.

! modifications:
!     none.

! subroutines called:
!     eau_liq     : core of parameterization of cloud microphysics.
!     eau_liqqps  : global qp2s and qp3s diagnostics.
!     eau_liqpbps : point-by-point pbp1s and pbp2s diagnostics.

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: &
   ksdm, &      ! loop index.
   nagrid, &    ! length of horizontal domain with ghost cells.
   nlm, &       ! number of layers.
   lpbl, &      ! number of layers above the PBL layer(s).
   nls          ! number of layers in the stratosphere.

 logical (kind=log_kind), intent(in), dimension(nagrid) :: &
   do_phys      ! select the non-ghost cells                      (-).

 real (kind=dbl_kind), intent(in) :: &
   dtdyn        ! dynamical time-step                             (s).

 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm) :: &
   plg, &       ! pressure                                       (Pa).
   bpg, &       ! (pl/p0)**kapa                                   (-).
   phil, &      ! geopotential                                    (m).
   thp          ! potential temperature                           (K).

 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1) :: &
   plgl2, &     ! pressure at layer edges                        (Pa).
   bpgl2, &     ! (pl/p0)**kapa at layer edges                    (-).
   phil2        ! geopotential at layer edges                     (m).

 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm,ntracer):: &
   tracerp      ! array of non-passive tracers                (kg/kg).
 
 real (kind=dbl_kind), intent(in), dimension(nagrid,nlm,nwater) :: &
   delq_cup     ! eau_cup convective tendencies             (kg/kg/s).
     
! output arguments:
! -----------------

 real (kind=dbl_kind),intent(out),dimension(nagrid) :: &
   elqrprec, &  ! large-scale precipitation of rain         (kg/m2/s).
   elqsprec     ! large-scale precipitation of snow         (kg/m2/s).

 real (kind=dbl_kind),intent(out),dimension(nagrid,nlm) :: &
   dt_thp       ! total tendency of potential temperature       (K/s).

 real (kind=dbl_kind), intent(out), dimension(nagrid,nwater) :: &
   deaul_vtot   ! vertically-integrated tendencies          (kg/m2/s).

 real (kind=dbl_kind),intent(out),dimension(nagrid,nlm,nwater) :: &
   dt_hydro     ! total tendency of hydro variables         (kg/kg/s). 

! local variables output by eau_liq:

 real (kind=dbl_kind), dimension(nagrid,nlm) :: &
   deaul_tp, &  ! tendency of temperature due to eau_liq        (K/s).
   deaul_iq, &  ! tendency of water vapor due to eau_liq    (kg/kg/s).
   deaul_icw, & ! tendency of cloud water due to eau_liq    (kg/kg/s).
   deaul_irw, & ! tendency of rain due to eau_liq           (kg/kg/s).
   deaul_ici, & ! tendency of cloud ice due to eau_liq      (kg/kg/s).
   deaul_iri, & ! tendency of snow due to eau_liq           (kg/kg/s).
   deaul_rwf, & ! tendency of rain due to precipitation     (kg/kg/s).
   deaul_rif    ! tendency of snow due to precipitation     (kg/kg/s).

! other local variables:

 logical (kind=log_kind), parameter :: w_diag = .true.

 integer (kind=int_kind) :: i, itod, itod2, l, len, nlen

 real (kind=dbl_kind) :: &
   nbdyn, &     ! nb of physical time-steps per day               (-).
   pcos1, &     ! cosine of first diurnal harmonic                (-).
   pcos2, &     ! cosine of first diurnal harmonic                (-).
   psin1, &     ! sine of second diurnal harmonic                 (-).
   psin2        ! sine of second diurnal harmonic                 (-).

 real (kind=dbl_kind), dimension(nagrid) :: &
    deaul_iqt , deaul_icwt , deaul_irwt, &
   deaul_icit , deaul_irit , deauc_icwt, &
   deauc_irwt , deauc_icit , deauc_irit, &
   tot_elq ,  tot_elqpr      

 real (kind=dbl_kind), dimension(nagrid,nlm) :: &
   dplg, &      ! layer pressure thickness                       (Pa).
   ql, &        ! water vapor mixing ratio                    (kg/kg).
   cw, &        ! cloud water mixing ratio                    (kg/kg).
   rw, &        ! rain mixing ratio                           (kg/kg).
   ci, &        ! cloud ice mixing ratio                      (kg/kg).
   ri, &        ! snow mixing ratio                           (kg/kg).
   deauc_icw, & ! total tend. of cloud water due to eau_cup (kg/kg/s).
   deauc_ici, & ! total tend. of cloud ice due to eau_cup   (kg/kg/s).
   deauc_irw, & ! total tend. of rain due to eau_cup        (kg/kg/s).
   deauc_iri    ! total tend. of snow due to eau_cup        (kg/kg/s).

! local variables for lsp_pbl - removing supersaturation from the
!  boundary layer
      real (kind=dbl_kind), dimension(nagrid) ::         &
       ddzl_tp   &! pbl temperature increment due to supersat. adj. (K)
     , ddzl_iq   &! pbl water vapor increment due to supersat. adj. (kg/kg)
     , dzlrprec  &! liquid precip increment due to supersat. adj. (kg/m2)
     , dzlsprec   ! frozen precip increment due to supersat. adj. (kg/m2)
      real (kind=dbl_kind) :: dtdyni

!----------------------------------------------------------------------

! write(6,*)
! write(6,*) '--- enter eau_liq_control: ksdm=', ksdm, ' nagrid='  &
!             nagrid,' nlm=', nlm,' lpbl=', lpbl

!---- first and second diurnal harmonics:

 dtdyni = 1._dbl_kind / dtdyn
 nbdyn  = 86400_dbl_kind/dtdyn
 itod   = tofday*nbdyn/24._dbl_kind + 1
 itod2  = 2 * itod - 1 
#ifndef scm
  pcos1  = pcos(itod)  ; psin1 = psin(itod)
  pcos2  = pcos(itod2) ; psin2 = psin(itod2)
#endif

! write(6,*)
! write(6,*) ' tofday =', tofday
! write(6,*) ' dtdyn  =', dtdyn
! write(6,*) ' nbdyn  =', nbdyn
! write(6,*) ' itod   =', itod
! write(6,*) ' itod2  =', itod2
! write(6,111) pcos1, psin1
! write(6,111) pcos2, psin2
! write(6,*)
! 111 format(2(1x,e15.8))

!---- initialization of output variables:

 elqrprec(:) = c0
 elqsprec(:) = c0
      
 dt_thp(:,:)     = c0
 dt_hydro(:,:,:) = c0

 deaul_vtot(:,:) = c0

!---- initialization of local variables:

 deauc_icwt(:) = c0 ; deauc_irwt(:) = c0
 deauc_icit(:) = c0 ; deauc_irit(:) = c0
 deaul_iqt(:)  = c0 ; deaul_icwt(:) = c0
 deaul_irwt(:) = c0 ; deaul_icit(:) = c0
 deaul_irit(:) = c0 ; tot_elqpr(:)  = c0
 tot_elq(:)    = c0

 do l = 1, nlm
    dplg(:,l) = plgl2(:,l+1)-plgl2(:,l)
 enddo

 ql(:,:) = tracerp(:,:,iq)  ; cw(:,:) = tracerp(:,:,icw)
 rw(:,:) = tracerp(:,:,irw) ; ci(:,:) = tracerp(:,:,ici)
 ri(:,:) = tracerp(:,:,iri)

 deauc_icw(:,:) = delq_cup(:,:,icw)
 deauc_irw(:,:) = delq_cup(:,:,irw)
 deauc_ici(:,:) = delq_cup(:,:,ici)
 deauc_iri(:,:) = delq_cup(:,:,iri)

!---- initialization of local eau_liq variables:

 deaul_tp(:,:)  = c0 ; deaul_iq(:,:)  = c0
 deaul_icw(:,:) = c0 ; deaul_irw(:,:) = c0
 deaul_ici(:,:) = c0 ; deaul_iri(:,:) = c0
 deaul_rwf(:,:) = c0 ; deaul_rif(:,:) = c0
 ddzl_tp(:) = c0  ; ddzl_iq(:) = c0
 dzlsprec(:) = c0 ; dzlrprec(:) = c0
      
!----

 do i = 1, nagrid
    if (do_phys(i)) then

       nlen = nagrid
       len  = 1
	 
!---- eau_liq parameterization of large-scale cloud microphysics:

       call eau_liq &
                (          nlen ,             len, &
                            nlm ,            lpbl, &
                          dtdyn ,      plgl2(i,1), &
                     bpgl2(i,1) ,        plg(i,1), &
                       bpg(i,1) ,       dplg(i,1), &
                       thp(i,1) ,         ql(i,1), &
                        cw(i,1) ,         rw(i,1), &
                        ci(i,1) ,         ri(i,1), &
                 deauc_icw(i,1) ,  deauc_irw(i,1), &
                 deauc_ici(i,1) ,  deauc_iri(i,1), &
                  deaul_tp(i,1) ,   deaul_iq(i,1), &
                 deaul_icw(i,1) ,  deaul_irw(i,1), &
                 deaul_ici(i,1) ,  deaul_iri(i,1), &
                 deaul_rwf(i,1) ,  deaul_rif(i,1), &
                    elqrprec(i) ,     elqsprec(i)  &
                )

             call LSP_pbl(len,                                   &
                  Thp(i,nlm), ql(i,nlm), dplg(i,nlm),            &
                  bpg(i,nlm), bpgl2(i,nlm+1), plgl2(i,nlm+1),    &
                  spec_heat_cp, lsub, hltm, gamfac, tice, grav,  &
                  ddzl_tp(i), ddzl_iq(i),                        &
                  dzlrprec(i), dzlsprec(i) )

            elqrprec(i) = elqrprec(i) + dzlrprec(i) * dtdyni
            elqsprec(i) = elqsprec(i) + dzlsprec(i) * dtdyni
            deaul_tp(i,nlm) = deaul_tp(i,nlm) + ddzl_tp(i) * dtdyni
            deaul_iq(i,nlm) = deaul_iq(i,nlm) + ddzl_iq(i) * dtdyni

    endif ! end(do_phys)
 enddo    ! end nagrid loop
            
!---- eau_liq diagnostics:

#ifndef scm
 call eau_liqqps &
          (     ksdm ,    nagrid ,       nlm ,     pcos1, &
               pcos2 ,     psin1 ,     psin2 ,  elqrprec, &
            elqsprec ,  deaul_tp ,  deaul_iq , deaul_icw, &
           deaul_irw , deaul_ici , deaul_iri , deaul_rwf, &
           deaul_rif,                                     &
           ddzl_tp, ddzl_iq, dzlsprec, dzlrprec, dtdyni   &
          )
#endif

 call eau_liqpbps &
          (     ksdm ,    nagrid ,       nlm ,  elqrprec, &
            elqsprec ,  deaul_tp ,  deaul_iq , deaul_icw, &
           deaul_irw , deaul_ici , deaul_iri , deaul_rwf, &
           deaul_rif                                      &
          )

!---- conservation of total water:

 if (w_diag) then
    do i = 1, nagrid
       if (do_phys(i)) then
          do l = 1, nlm
	         deauc_icwt(i) = deauc_icwt(i)+deauc_icw(i,l)*dplg(i,l)
	         deauc_irwt(i) = deauc_irwt(i)+deauc_irw(i,l)*dplg(i,l)
	         deauc_icit(i) = deauc_icit(i)+deauc_ici(i,l)*dplg(i,l)
	         deauc_irit(i) = deauc_irit(i)+deauc_iri(i,l)*dplg(i,l)
          enddo

          do l = 1, nlm
             deaul_iqt(i) =deaul_iqt(i)+deaul_iq(i,l)*dplg(i,l)
             deaul_icwt(i)=deaul_icwt(i)+deaul_icw(i,l)*dplg(i,l)
             deaul_irwt(i)=deaul_irwt(i)+deaul_irw(i,l)*dplg(i,l)
             deaul_icit(i)=deaul_icit(i)+deaul_ici(i,l)*dplg(i,l)
             deaul_irit(i)=deaul_irit(i)+deaul_iri(i,l)*dplg(i,l)
          enddo
          tot_elq(i)   = deaul_iqt(i)  + deaul_icwt(i) &
                       + deaul_irwt(i) + deaul_icit(i) &
                       + deaul_irit(i) - deauc_icwt(i) &
                       - deauc_irwt(i) - deauc_icit(i) &
                       - deauc_irit(i)
          tot_elq(i)   = tot_elq(i)/grav
          tot_elqpr(i) = elqrprec(i) + elqsprec(i)
!         write(6,102) i,tot_elqpr(i),tot_elq(i), &
!                      tot_elq(i)+tot_elqpr(i)

          deaul_vtot(i,iq)  = deaul_iqt(i)
          deaul_vtot(i,icw) = deaul_icwt(i)
          deaul_vtot(i,irw) = deaul_irwt(i)
          deaul_vtot(i,ici) = deaul_icit(i)
          deaul_vtot(i,iri) = deaul_irit(i)

       endif
    enddo
 endif

!---- large-scale and convective tendencies needed for the geodesic 
!     dynamics:

 dt_thp(:,:)       = deaul_tp(:,:)

 dt_hydro(:,:,iq)  = deaul_iq(:,:)
 dt_hydro(:,:,icw) = deaul_icw(:,:)
 dt_hydro(:,:,irw) = deaul_irw(:,:)
 dt_hydro(:,:,ici) = deaul_ici(:,:)
 dt_hydro(:,:,iri) = deaul_iri(:,:)

!---- formats:
 102 format('--- elq: ',i3,3(1x,e15.8))

! write(6,*) '--- exit subroutine eau_control'

 end subroutine eau_liq_control

!--------------------

      SUBROUTINE LSP_pbl (len, TOL, QOL, dpl, PLK, PLK2, PL2,    &
                         cp, lsub, lcond, gamfac, tice, grav,    &
                         dtdzl, dshdzl, dzlrprec, dzlsprec )

!       Large Scale supersaturation Precipitation routine.
!       This routine creates large-scale supersaturation
!         if surface is supersaturated. Units of output consistent
!         with eauliq.

!     INPUT VARIABLES   TOL     potential temperature profile; K
!                       QOL     water vapor mixing ratio; kg/kg
!                       PL2     surface pressure; Pa
!                       PLK     (PL/Po)**KAPPA
!                       PLK2    (PL2/Po)**KAPPA
!                       dpl     pbl thickness; Pa
!                       cp      specific heat of air; J/kg*K
!                       lsub    latent heat of sublimation; J/kg*K
!                       lcond   latent heat of condensation; J/kg*K
!                       gamfac  gamma=gamfac*qstar/t**2; K**2
!                       tice    freezing temperature K
!                       grav    gravity

!        out VARIABLES  dtdzl   potential temperature increment due to LSP; K
!                       dshdzl  moisture increment due to LSP; kg/kg
!                       dzlrprec rain precipitation produced by LSP; kg/m2
!                       dzlsprec snow precipitation produced by LSP; kg/m2

!--------------------

      use kinds
      use eau_util

!  argument list
      integer (kind=int_kind), intent(in) :: len
      real (kind=dbl_kind), dimension(len), intent(in) ::    &
        TOL, QOL, dpl, PL2, PLK, PLK2
      real (kind=dbl_kind), dimension(len), intent(out) ::   &
        dzlsprec, dzlrprec,dtdzl, dshdzl
      real (kind=dbl_kind), intent(in) ::                    &
        cp, lcond, lsub, gamfac, tice, grav

!   local variables
      integer (kind=int_kind) :: i
      real (kind=dbl_kind) :: gravi, cpinv, to, qo, tem
      real (kind=dbl_kind), dimension(len) :: TL, QOS, PSPMK, TS

!     init vars
      CPINV = 1. / CP
      gravi = 1. / grav

!     COPY INPUT TEMPERATURE AND HUMIDITY PROFILES -> working vars

      tl(:) = tol(:)*plk(:)

      pspmk(:) = plk2(:)/plk(:)
      ts(:) = tl(:)*pspmk(:)

!     find saturation mixing ratio for pbl

      CALL eau_SAT (len, PL2, ts, qsats=QOS)

!     PRECIPITATION FROM THE PBL
!     CHECK FOR SATURATION AT THE SURFACE.
!     IF THE SURFACE AIR IS NOT SUPERSATURATED, KICK OUT.

            DO i=1,len
               IF (QOL(i) .gt. QOS(i)) then
!     REDUCE Q AND INCREASE T TO ELIMINATE SURFACE SUPERSATURATION.
!     DONT BOTHER TO ITERATE.

                  if(ts(i).gt.tice) then  !rain case

                     TEM = (QOL(i)-QOS(i))/          &
                        (1.+GAMFAC*PSPMK(i)*         &
                        QOS(i)/(TS(i)*TS(i)))
                     QO  = QOL(i) - TEM
                     TO  = TL(i) + TEM*lcond*CPINV
!     PRECIPITATION AMOUNT
                     dzlrprec(i) = TEM *dpl(i)*gravi

                  else                       ! snow case

                     TEM = (QOL(i)-QOS(i))/          &
                        (1.+GAMFAC*PSPMK(i)*         &
                        QOS(i)/(TS(i)*TS(i)))
                     QO  = QOL(i) - TEM
                     TO  = TL(i) + TEM*lsub*CPINV
!     PRECIPITATION AMOUNT
                     dzlsprec(i) = TEM*dpl(i)*gravi

                  endif
!     COMP3 INCREMENTS
                  dshdzl(i) = - tem
                  dtdzl(i) = (To/plk(i) - TOL(i))
               endif
            enddo

      END subroutine lsp_pbl
!----------------------------------------------------------------------
