!==============================================================================

 module eau_submicrop
 
 use kinds
 use eau_params

 implicit none
 private
 public:: eau_satlt0, eau_slvlt0, eau_bglt0, eau_slvbglt0

!==============================================================================
!eau_submicrop contains the subroutines that solve the cloud microphysics equa-
!tions for sub-saturated conditions, for single-phase and mixed-phase clouds.
!Laura D. Fowler/mcbug (11-11-03).
!send comments to laura@atmos.colostate.edu

!references:
!Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1996: Liquid and Ice Cloud Mi-
!crophysics in the CSU General Circulation Model. Part I: Model description and
!simulated Microphysical processes. Journal of Climate, 9, 489-529.

!subroutines called:
!   eau_satlt0  : fills the coefficients for the water vapor, cloud water/ice,
!                 rain/snow, and temperature equations for sub-saturated grid-
!                 points.
!   eau_slvlt0  : solves the cloud microphysics equations for water vapor,
!                 cloud water/ice, rain/snow, and temperature for sub-saturated
!                 grid-points. The coefficients of the different equations were
!                 computed in eau_satlt0.

!   eau_bglt0   : fills the coefficients for the water vapor, cloud water/ice,
!                 rain/snow, and temperature equations for sub-saturated grid-
!                 points in the mixed-phase window.
!   eau_slvbglt0: solves the cloud microphysics equations for water vapor,
!                 cloud water/ice, rain/snow, and temperature for sub-saturated
!                 grid-points in the mixed-phase window. The coefficients for
!                 the different equations were computed in eau_bglt0.

!==============================================================================

 contains

!==============================================================================
 subroutine eau_satlt0 &
                (  len ,    nlm ,    nx , innx ,   tt , &
                   sat ,    qwv ,    qc ,   qr , facr , &
                  fevp ,  conv1 , conv2 ,  tau ,  lcs , &
                 dtime , matrix , const                 &
                )
!-----------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.

 integer (kind=int_kind), intent(in), dimension(len*nlm):: &
    innx              !location of grid-points.

 real (kind=dbl_kind), intent(in):: &
    dtime             !time-step                                           (s).

 real (kind=dbl_kind), intent(in), dimension(len*nlm):: &
    tt,              &!temperature                                         (K).
    sat,             &!saturation                                          (-).
    qwv,             &!water vapor mixing ratio                        (kg/kg).
    qc,              &!cloud water/ice mixing ratio                    (kg/kg).
    qr,              &!rain/snow mixing ratio                          (kg/kg).
    facr,            &!f-Array: Collection rate                      (kg/kg/s).
    fevp,            &!f-Array: Evaporation rate                     (kg/kg/s).
    conv1,           &!coefficient related to autoconversion         (kg/kg/s).
    conv2,           &!coefficient related to autoconversion         (kg/kg/s).
    tau,             &!coefficient related to condensation             (per s).
    lcs               !latent heat                                    (J/kg/s).
      
!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,4):: &
    const             !constants of microphysics equation.

 real (kind=dbl_kind), intent(out), dimension(len*nlm,4,4):: &
    matrix            !coefficients of microphysics equation.

!local variables:
 integer (kind=int_kind):: i
 
 real (kind=dbl_kind):: cpi, taui

!------------------------------------------------------------------------------
 cpi = c1/spec_heat_cp

 do i = 1, nx
    taui = c1 / tau(innx(i))

!water vapor equation:
    matrix(i,1,1) = c1
    matrix(i,1,2) = dtime*taui*(sat(innx(i))-c1)
    matrix(i,1,3) = (sat(innx(i))-c1)*fevp(innx(i))*dtime
    matrix(i,1,4) = c0
    const(i,1)    = qwv(innx(i))

!cloud liquid/ice water equation:
    matrix(i,2,1) = c0
    matrix(i,2,2) = c1-(taui*(sat(innx(i))-c1) &
                  - qr(innx(i))*facr(innx(i))-conv1(innx(i)))*dtime
    matrix(i,2,3) = c0
    matrix(i,2,4) = c0
    const(i,2)    = qc(innx(i))+conv2(innx(i))*dtime

!rain/snow equation:
    matrix(i,3,1) = c0
    matrix(i,3,2) = -(conv1(innx(i))+qr(innx(i))*facr(innx(i)))*dtime
    matrix(i,3,3) = c1-(sat(innx(i))-c1)*fevp(innx(i))*dtime
    matrix(i,3,4) = c0
    const(i,3)    = qr(innx(i))-conv2(innx(i))*dtime

!temperature equation:
    matrix(i,4,1) = c0
    matrix(i,4,2) = (-lcs(innx(i))*cpi*(sat(innx(i))-c1)*taui)*dtime
    matrix(i,4,3) = -lcs(innx(i))*cpi*(sat(innx(i))-c1) &
                  * fevp(innx(i))*dtime
    matrix(i,4,4) = c1
    const(i,4)    = tt(innx(i))

 enddo

 end subroutine eau_satlt0
      
!------------------------------------------------------------------------------
 subroutine eau_slvlt0 &
                (len , nlm , nx , matrix , const , vect)
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.
    
 real (kind=dbl_kind), intent(in), dimension(len*nlm,4):: &
    const             !constants of microphysics equation.

 real (kind=dbl_kind), intent(in), dimension(len*nlm,4,4):: &
    matrix            !coefficients of microphysics equation.

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,4):: &
    vect   !Solution vector of cloud microphysics equations.

!local variables:
 integer (kind=int_kind):: i
     
!------------------------------------------------------------------------------
 do i = 1, nx
    vect(i,2) = const(i,2)/matrix(i,2,2)
    vect(i,3) = (const(i,3)-matrix(i,3,2)*vect(i,2))   &
              / matrix(i,3,3)
    vect(i,1) = (const(i,1)-matrix(i,1,2)*vect(i,2)    &
              - matrix(i,1,3)*vect(i,3))/matrix(i,1,1)
    vect(i,4) = (const(i,4)-matrix(i,4,2)*vect(i,2)    &
              - matrix(i,4,3)*vect(i,3))/matrix(i,4,4)
 enddo

 end subroutine eau_slvlt0

!------------------------------------------------------------------------------
 subroutine eau_bglt0 &
                (    len , nlm ,  nx ,  innx ,     wght , &
                      tt , sat , qwv ,   qcw ,      qci , &
                     qri , fcs , fis ,  fevp ,    conv1 , &
                   conv2 , tau , lcs , dtime , matrixbg , &
                 constbg                                  &
                )
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.

 integer (kind=int_kind), intent(in), dimension(len*nlm):: &
    innx              !location of grid-points.

 real (kind=dbl_kind), intent(in):: &
    dtime             !time-step                                           (s).

 real (kind=dbl_kind), intent(in), dimension(len*nlm):: &
    wght,            &!temperature partioning between cloud water/ice      (-).
    tt,              &!temperature                                         (K).
    sat,             &!saturation                                          (-).
    qwv,             &!water vapor mixing ratio                        (kg/kg).
    qcw,             &!cloud water mixing ratio                        (kg/kg).
    qci,             &!cloud ice mixing ratio                          (kg/kg).
    qri,             &!snow mixing ratio                               (kg/kg).
    fcs,             &!f-Array: Collection of qcw by qri             (kg/kg/s).
    fis,             &!f-Array: Collection of qci by qi              (kg/kg/s).
    fevp,            &!f-Array: Evaporation rate                     (kg/kg/s).
    conv1,           &!coefficient related to autoconversion         (kg/kg/s).
    conv2,           &!coefficient related to autoconversion         (kg/kg/s).
    tau,             &!coefficient related to condensation             (per s).
    lcs               !latent heat                                    (J/kg/s).

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,5):: &
    constbg           !constants of microphysics equation.

 real (kind=dbl_kind), intent(out), dimension(len*nlm,5,5):: &
    matrixbg          !coefficients of microphysics equation.

!local variables:
 integer (kind=int_kind):: i

 real (kind=dbl_kind):: cpi, acoef, bcoef, ccoef, dcoef

!------------------------------------------------------------------------------
 cpi = c1/spec_heat_cp

 do i = 1, nx
    acoef = (sat(innx(i))-c1)/tau(innx(i))*dtime
    bcoef = (sat(innx(i))-c1)*fevp(innx(i))*dtime
    ccoef = fcs(innx(i))*qri(innx(i))*dtime
    dcoef = fis(innx(i))*qri(innx(i))*dtime

!water vapor equation:
    matrixbg(i,1,1) = c1
    matrixbg(i,1,2) = acoef
    matrixbg(i,1,3) = acoef
    matrixbg(i,1,4) = bcoef
    matrixbg(i,1,5) = c0
    constbg(i,1)    = qwv(innx(i))

!cloud water equation:
    matrixbg(i,2,1) = c0
    matrixbg(i,2,2) = c1-wght(innx(i))*acoef+ccoef
    matrixbg(i,2,3) = -wght(innx(i))*acoef
    matrixbg(i,2,4) = c0
    matrixbg(i,2,5) = c0
    constbg(i,2)    = qcw(innx(i))

!cloud ice equation:
    matrixbg(i,3,1) = c0
    matrixbg(i,3,2) = -(c1-wght(innx(i)))*acoef
    matrixbg(i,3,3) = c1-(c1-wght(innx(i)))*acoef+dcoef &
                   + conv1(innx(i))* dtime
    matrixbg(i,3,4) = c0
    matrixbg(i,3,5) = c0
    constbg(i,3)    = qci(innx(i))+conv2(innx(i))*dtime

!snow equation:
    matrixbg(i,4,1) = c0
    matrixbg(i,4,2) = -ccoef
    matrixbg(i,4,3) = -dcoef-conv1(innx(i))*dtime
    matrixbg(i,4,4) = c1-bcoef
    matrixbg(i,4,5) = c0
    constbg(i,4)    = qri(innx(i))-conv2(innx(i))*dtime

!temperature equation:
    matrixbg(i,5,1) = c0
    matrixbg(i,5,2) = -(lcs(innx(i))*acoef+lfus*ccoef)*cpi
    matrixbg(i,5,3) = -lcs(innx(i))*cpi*acoef
    matrixbg(i,5,4) = -lsub*cpi*bcoef
    matrixbg(i,5,5) = c1
    constbg(i,5)    = tt(innx(i))

 enddo

 end subroutine eau_bglt0
      
!------------------------------------------------------------------------------
 subroutine eau_slvbglt0 &
                (len , nlm , nx , matrixbg , constbg , vectbg)
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.

 real (kind=dbl_kind), intent(in), dimension(len*nlm,5):: &
    constbg           !constants of microphysics equation.
 
 real (kind=dbl_kind), intent(in), dimension(len*nlm,5,5):: &
    matrixbg          !coefficients of microphysics equation.

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,5):: &
    vectbg            !solution vector of cloud microphysics equations.

!local variables:
 integer (kind=int_kind):: i

 real (kind=dbl_kind), dimension(nx):: alpha1, alpha2, beta1, beta2
     
!------------------------------------------------------------------------------
 do i = 1, nx
    alpha1(i) = matrixbg(i,2,2)*matrixbg(i,3,3) &
              - matrixbg(i,3,2)*matrixbg(i,2,3)
    alpha2(i) = matrixbg(i,2,3)*matrixbg(i,3,2) &
              - matrixbg(i,2,2)*matrixbg(i,3,3)         

    beta1(i)  = constbg(i,2)*matrixbg(i,3,3)    &
              - constbg(i,3)*matrixbg(i,2,3)
    beta2(i)  = constbg(i,2)*matrixbg(i,3,2)    &
              - constbg(i,3)*matrixbg(i,2,2)
          
    vectbg(i,2) = beta1(i)/alpha1(i)
    vectbg(i,3) = beta2(i)/alpha2(i)
    vectbg(i,4) = (constbg(i,4)-matrixbg(i,4,2)*vectbg(i,2)    &
                - matrixbg(i,4,3)*vectbg(i,3))/matrixbg(i,4,4)
    vectbg(i,1) = (constbg(i,1)-matrixbg(i,1,2)*vectbg(i,2)    &
                - matrixbg(i,1,3)*vectbg(i,3)-matrixbg(i,1,4)  &
                * vectbg(i,4))/matrixbg(i,1,1)
    vectbg(i,5) = (constbg(i,5)-matrixbg(i,5,2)*vectbg(i,2)    &
                - matrixbg(i,5,3)*vectbg(i,3)-matrixbg(i,5,4)  &
                * vectbg(i,4))/matrixbg(i,5,5)
 enddo

 end subroutine eau_slvbglt0
       
!==============================================================================

 end module eau_submicrop

!==============================================================================
