!-----------------------------------------------------------------------

 subroutine eau_subsat                                  &
                (    len ,     nlm ,     nn1 ,     nn2, &
                     nn3 ,    inn1 ,    inn2 ,    inn3, &
                   dtelq ,      tl ,    qwvl ,    qcwl, &
                    qrwl ,    qcil ,    qril ,   satsl, &
                 fpracwl , fpsacil , fpsacwl , fprevpl, &
                 fpsevpl ,   wghtl                      &
                )

 use kinds
 use eau_params
 use eau_submicrop

 implicit none

!-----------------------------------------------------------------------

! eau_subsat sets solving the cloud microphysics equations for sub-satu
! rated grid-points. 
! Laura D. Fowler/slikrock (03-07-00).

! send comments to laura@atmos.colostate.edu.

! references:
! Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1996: Liquid and Ice
! Cloud Microphysics in the CSU General Circulation Model.
! Part I: Model description and simulated Microphysical processes.
! Journal of Climate, 9, 489-529.

! Randall, D.A., and L.D. Fowler, 1999: EAUliq: The next generation.
! Dept. of Atmospheric Science Paper 673, Dept. of Atmospheric Science.
! Colorado State University, Fort Collins, Colorado, 65 pp.

! modifications:
!      none.

! subroutines called:
!      eau_satlt0   :computes coefficients of microphysics equations for
!                    sub-saturared grid-points.
!      eau_slvlt0   :solves microphysics equations for sub-saturated 
!                    grid-points.
!      eau_bglt0    :Computes coefficients of microphysics equations for
!                    sub-saturated grid-points in mixed-phase window.
!      eau_slvbglt0 :solves microphysics equations for super-saturated
!                    grid-points in mixed-phase window.

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: & 
   len, &    ! length of sub domain.
   nlm, &    ! number of layers.
   nn1, &    ! number of grid-points for the ice phase.
   nn2, &    ! number of grid-points for the mixed phase.
   nn3       ! number of grid-points for the liquid phase.

 integer (kind=int_kind), intent(in), dimension(len*nlm) :: &
   inn1, &   ! location of nn1 grid-points.
   inn2, &   ! location of nn2 grid-points.
   inn3      ! location of nn3 grid-points.
     
 real (kind=dbl_kind), intent(in) :: &
   dtelq     ! cloud microphysical time-step                       (s).

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
   satsl, &  ! saturation                                          (-).
   fpracwl, &! f-array:Collection of cloud water by rain     (kg/kg/s).
   fpsacil, &! f-array:Collection of cloud ice by snow       (kg/kg/s).
   fpsacwl, &! f-array:Collection of cloud water by snow     (kg/kg/s).
   fprevpl, &! f-array:Evaporation of rain                   (kg/kg/s).
   fpsevpl   ! f-array:Evaporation of snow                   (kg/kg/s).
     
! input/output arguments:
! -----------------------

 real (kind=dbl_kind), intent(inout), dimension(len,nlm) :: &
   tl, &    ! temperature                                          (K).
   qwvl, &  ! water vapor mixing ratio                         (kg/kg).
   qcwl, &  ! cloud water mixing ratio                         (kg/kg).
   qrwl, &  ! rain mixing ratio                                (kg/kg).
   qcil, &  ! cloud ice mixing ratio                           (kg/kg).
   qril, &  ! snow mixing ratio                                (kg/kg).
   wghtl    ! partitioning between cloud water and cloud
!              ice for mixed-phase clouds                          (-).      

! local variables:

 integer (kind=int_kind) :: i
 
 integer (kind=int_kind), dimension(len*nlm) :: & ! for array bounds check
   iinn1, iinn2, iinn3, &  ! horizontal index of inn1, inn2, inn3 resp.
    lnn1, lnn2, lnn3       ! vertical index of inn1, inn2, inn3 resp.

 real (kind=dbl_kind), dimension(len*nlm) :: &
     conv1, conv2, taucst, lcs

 real (kind=dbl_kind), dimension(len*nlm,4) :: const, vect

 real (kind=dbl_kind), dimension(len*nlm,4,4) :: matrix

 real (kind=dbl_kind), dimension(len*nlm,5) :: constbg, vectbg

 real (kind=dbl_kind), dimension(len*nlm,5,5) :: matrixbg  

!-----------------------------------------------------------------------

! compute horizontal and  vertical indices from total index

 do i = 1,nn1
    iinn1(i) = mod(inn1(i)-1,len) + 1 
    lnn1(i) = (inn1(i) - iinn1(i))/len + 1
 enddo
 do i = 1,nn2
    iinn2(i) = mod(inn2(i)-1,len) + 1
    lnn2(i) = (inn2(i) - iinn2(i))/len + 1
 enddo
 do i = 1,nn3
    iinn3(i) = mod(inn3(i)-1,len) + 1
    lnn3(i) = (inn3(i) - iinn3(i))/len + 1
 enddo

 conv1     = c0
 conv2     = c0
 taucst    = c0
 lcs       = c0
 const     = c0
 constbg   = c0
 vect      = c0
 matrix    = c0
 vectbg    = c0
 matrixbg  = c0

!---- ice phase:

 if (nn1 > 0) then

    do i = 1, nn1
       if (qcil(iinn1(i),lnn1(i)) >= qci0) then
          conv1(inn1(i)) = betaelq*exp(0.025*(tl(iinn1(i),lnn1(i))-t00))
          conv2(inn1(i)) = conv1(inn1(i))*qci0
       endif
       taucst(inn1(i))  = tauf
       lcs(inn1(i))     = lsub
    enddo

    call eau_satlt0                               &
             (   len ,     nlm ,     nn1 ,  inn1, &
                  tl ,   satsl ,    qwvl ,  qcil, &
                qril , fpsacil , fpsevpl , conv1, &
               conv2 ,  taucst ,     lcs , dtelq, &
              matrix ,   const                    &
             )

    call eau_slvlt0                        &
             (  len ,  nlm , nn1 , matrix, &
              const , vect                 &
             )

    do i = 1, nn1
       if (vect(i,2) < qci0) then
          conv1(inn1(i)) = c0
          conv2(inn1(i)) = c0
       endif
    enddo

    call eau_satlt0                               &
             (   len ,     nlm ,     nn1 ,  inn1, &
                  tl ,   satsl ,    qwvl ,  qcil, &
                qril , fpsacil , fpsevpl , conv1, &
               conv2 ,  taucst ,     lcs , dtelq, &
              matrix ,   const                    &
             )

    call eau_slvlt0                        &
             (  len ,  nlm , nn1 , matrix, &
              const , vect                 &
             )

    do i = 1, nn1
       qwvl(iinn1(i),lnn1(i)) = vect(i,1)
       qcil(iinn1(i),lnn1(i)) = vect(i,2)
       qril(iinn1(i),lnn1(i)) = vect(i,3)	
       tl(iinn1(i),lnn1(i))   = vect(i,4)
    enddo

 endif

!---- mixed phase:

 if (nn2 > 0) then

    do i = 1, nn2
       conv1(inn2(i))       = c0
       conv2(inn2(i))       = c0
       taucst(inn2(i))      = tauf
       lcs(inn2(i))         = hltm+(1.-wghtl(iinn2(i),lnn2(i)))*lfus
    enddo

    call eau_bglt0                                  &
             (    len ,     nlm ,   nn2 ,     inn2, &
                wghtl ,      tl , satsl ,     qwvl, &
                 qcwl ,    qcil ,  qril ,  fpsacwl, &
              fpsacil , fpsevpl , conv1 ,    conv2, &
               taucst ,     lcs , dtelq , matrixbg, &
              constbg                               &
             )

    call eau_slvbglt0                            &
             (    len ,    nlm , nn2 , matrixbg, &
              constbg , vectbg                   &
             )

    do i = 1, nn2
       if (vectbg(i,2) < c0) then
          wghtl(iinn2(i),lnn2(i)) = c0
          lcs(inn2(i))     = hltm+(1.-wghtl(iinn2(i),lnn2(i)))*lfus
          qcil(iinn2(i),lnn2(i))  = qcil(iinn2(i),lnn2(i)) &
                           + qcwl(iinn2(i),lnn2(i))
          tl(iinn2(i),lnn2(i))    = tl(iinn2(i),lnn2(i))  &
                                +lcs(inn2(i))/spec_heat_cp &
                                * qcwl(iinn2(i),lnn2(i))
          qcwl(iinn2(i),lnn2(i))  = c0
       endif
       if (vectbg(i,3) < c0) then
          wghtl(iinn2(i),lnn2(i)) = 1.
          lcs(inn2(i))     = hltm+(1.-wghtl(iinn2(i),lnn2(i)))*lfus 
          qcwl(iinn2(i),lnn2(i))  = qcwl(iinn2(i),lnn2(i)) &
                           + qcil(iinn2(i),lnn2(i))
          tl(iinn2(i),lnn2(i))    = tl(iinn2(i),lnn2(i)) &
                                -lcs(inn2(i))/spec_heat_cp &
                                * qcil(iinn2(i),lnn2(i))
          qcil(iinn2(i),lnn2(i))  = c0
       endif
    enddo            

    call eau_bglt0                                  &
             (    len ,     nlm ,   nn2 ,     inn2, &
                wghtl ,      tl , satsl ,     qwvl, &
                 qcwl ,    qcil ,  qril ,  fpsacwl, &
              fpsacil , fpsevpl , conv1 ,    conv2, &
               taucst ,     lcs , dtelq , matrixbg, &
              constbg                               &
             )

    call eau_slvbglt0                            &
             (    len ,    nlm , nn2 , matrixbg, &
              constbg , vectbg                   &
             )

    do i = 1, nn2
       if (vectbg(i,2) >= qcw0) then
          vectbg(i,4) = vectbg(i,4) &
                      + alphaelq*(vectbg(i,2)-qcw0)*dtelq
          vectbg(i,5) = vectbg(i,5)+lfus/spec_heat_cp &
                      * alphaelq*(vectbg(i,2)-qcw0)*dtelq
          vectbg(i,2) = vectbg(i,2) &
                      - alphaelq*(vectbg(i,2)-qcw0)*dtelq
       endif
       if (vectbg(i,3) >= qci0) then
          conv1(inn2(i)) = betaelq*exp(0.025*(tl(iinn2(i),lnn2(i))-t00))
          vectbg(i,4) = vectbg(i,4) &
                      + conv1(inn2(i))*(vectbg(i,3)-qci0)*dtelq
          vectbg(i,3) = vectbg(i,3) &
                      - conv1(inn2(i))*(vectbg(i,3)-qci0)*dtelq
       endif
       qwvl(iinn2(i),lnn2(i)) = vectbg(i,1)
       qcwl(iinn2(i),lnn2(i)) = vectbg(i,2)
       qcil(iinn2(i),lnn2(i)) = vectbg(i,3)
       qril(iinn2(i),lnn2(i)) = vectbg(i,4)
       tl(iinn2(i),lnn2(i))   = vectbg(i,5)
    enddo

    do i = 1, nn2
       if (qcil(iinn2(i),lnn2(i)) < c0) &
          write(6,*) 'eaubglt0: i=',i,' qcw=',qcwl(iinn2(i),lnn2(i)), &
             ' qci=',qcil(iinn2(i),lnn2(i)),' temp=', tl(iinn2(i),lnn2(i))
    enddo
    do i = 1, nn2
       if (qcwl(iinn2(i),lnn2(i)) < c0) &
          write(6,*) 'eaubglt0: i=',i,' qcw=',qcwl(iinn2(i),lnn2(i)), &
             ' qci=',qcil(iinn2(i),lnn2(i)),' temp=', tl(iinn2(i),lnn2(i))
    enddo

 endif

!---- liquid phase:

 if (nn3 > 0) then

    do i = 1, nn3 
       if (qcwl(iinn3(i),lnn3(i)) >= qcw0) then
          conv1(inn3(i)) = alphaelq
          conv2(inn3(i)) = conv1(inn3(i))*qcw0
       endif
       taucst(inn3(i))   = taul
       lcs(inn3(i))      = hltm
    enddo

    call eau_satlt0                               &
             (   len ,     nlm ,     nn3 ,  inn3, &
                  tl ,   satsl ,    qwvl ,  qcwl, &
                qrwl , fpracwl , fprevpl , conv1, &
               conv2 ,  taucst ,     lcs , dtelq, &
              matrix ,   const                    &
             )

    call eau_slvlt0                        &
             (  len ,  nlm , nn3 , matrix, &
              const , vect                 &
             )

    do i = 1, nn3
       if (vect(i,2) < qcw0) then
          conv1(inn3(i)) = c0
          conv2(inn3(i)) = c0
       endif
    enddo

    call eau_satlt0                               &
             (   len ,     nlm ,     nn3 ,  inn3, &
                  tl ,   satsl ,    qwvl ,  qcwl, &
                qrwl , fpracwl , fprevpl , conv1, &
               conv2 ,  taucst ,     lcs , dtelq, &
              matrix ,   const                    &
             )

    call eau_slvlt0                        &
             (  len ,  nlm , nn3 , matrix, &
              const , vect                 &
             )

    do i = 1, nn3
       qwvl(iinn3(i),lnn3(i)) = vect(i,1)
       qcwl(iinn3(i),lnn3(i)) = vect(i,2)
       qrwl(iinn3(i),lnn3(i)) = vect(i,3)
       tl(iinn3(i),lnn3(i))   = vect(i,4)
    enddo

 endif

 end subroutine eau_subsat
      
!-----------------------------------------------------------------------



