!==============================================================================

 module eau_supmicrop
 
 use kinds
 use eau_params

 implicit none
 private
 public:: eau_satge0, eau_slvge0, eau_bgge0, eau_slvbgge0

!==============================================================================
!eau_supmicrop contains the subroutines that solve the cloud microphysics equa-
!tions for sup-saturated conditions, for single-phase and mixed-phase clouds.
!Laura D. Fowler/mcbug (11-11-03).
!send comments to laura@atmos.colostate.edu

!references:
!Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1996: Liquid and Ice Cloud Mi-
!crophysics in the CSU General Circulation Model. Part I: Model description and
!simulated Microphysical processes. Journal of Climate, 9, 489-529.

!subroutines called:
!   eau_satge0  :fills the coefficients for the water vapor, cloud water/ice,
!                rain/snow, and temperature equations for super-saturated grid-
!                points.
!   eau_solvge0 :solves the cloud microphysics equations for water vapor, cloud
!                water/ice, rain/snow, and temperature for super-saturated grid
!                points. The coefficients of the different equations were compu
!                ted in eau_satge0.
!   eau_bglt0   :fills the coefficients for the water vapor, cloud water/ice,
!                rain/snow, and temperature equations for super-saturated grid-
!                points in the mixed-phase window.
!   eau_slvbgge0:solves the cloud microphysics equations for water vapor, cloud
!                water/ice, rain/snow, and temperature for super-saturated grid
!                points in the mixed-phase window. The coefficients of the dif-
!                ferent equations were computed in eau_bgge0.

!==============================================================================

 contains

!==============================================================================
 subroutine eau_satge0 &
                ( len ,   nlm ,     nx ,  innx ,  tt , &
                  qwv ,  qsat ,  dqsat ,    qc ,  qr , &
                 facr ,  fevp ,  conv1 , conv2 , tau , &
                  lcs , dtime , matrix , const         &
                )
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nlm,             &!number of layers.
    nx                !number of grid-points.

 integer (kind=int_kind), intent(in), dimension(len*nlm):: &
    innx              !location of grid-points.

 real (kind=dbl_kind), intent(in):: &
    dtime             !time-step                                           (s).

 real (kind=dbl_kind), intent(in), dimension(len*nlm):: &
    tt,              &!temperature                                         (K).
    qsat,            &!saturation mixing ratio                         (kg/kg).
    dqsat,           &!d(qsat)/dT                                    (kg/kg/K).
    qwv,             &!water vapor mixing ratio                        (kg/kg).
    qc,              &!cloud water/ice mixing ratio                    (kg/kg).
    qr,              &!rain/snow mixing ratio                          (kg/kg).
    facr,            &!f-Array: Collection rate                      (kg/kg/s).
    fevp,            &!f-Array: Evaporation rate                     (kg/kg/s).
    conv1,           &!coefficient related to autoconversion         (kg/kg/s).
    conv2,           &!coefficient related to autoconversion         (kg/kg/s).
    tau,             &!coefficient related to condensation             (per s).
    lcs               !latent heat                                    (J/kg/s).

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,4):: &
    const             !constants of microphysics equation.

 real (kind=dbl_kind), intent(out), dimension(len*nlm,4,4):: &
    matrix            !coefficients of microphysics equation.

!local variables:
 integer (kind=int_kind):: i

 real (kind=dbl_kind):: acoef, bcoef, lcscp, taui

!------------------------------------------------------------------------------

 do i = 1, nx

    taui = c1/tau(innx(i))
    lcscp = lcs(innx(i))/spec_heat_cp
!   acoef = (taui+qr(innx(i))/qsat(innx(i))*fevp(innx(i)))*dtime
    bcoef = (qr(innx(i))/qsat(innx(i))*fevp(innx(i)))*dtime
    acoef = bcoef + taui*dtime

!water vapor equation:
    matrix(i,1,1) = c1+acoef
    matrix(i,1,2) = c0
    matrix(i,1,3) = c0
    matrix(i,1,4) = -acoef*dqsat(innx(i))
    const(i,1)    = qwv(innx(i))+acoef*(qsat(innx(i))   &
                  - dqsat(innx(i))*tt(innx(i)))

!cloud liquid/ice water equation:
    matrix(i,2,1) = -dtime*taui
    matrix(i,2,2) = c1+(qr(innx(i))*facr(innx(i))    &
                  + conv1(innx(i)))*dtime
    matrix(i,2,3) = c0
    matrix(i,2,4) = dtime*taui*dqsat(innx(i))
    const(i,2)    = qc(innx(i))                         &
                  + (conv2(innx(i))-taui*(qsat(innx(i)) &
                  -  dqsat(innx(i))*tt(innx(i))))*dtime
      
!rain/snow equation:
    matrix(i,3,1) = -bcoef
    matrix(i,3,2) = -(conv1(innx(i))+qr(innx(i))        &
                  * facr(innx(i)))*dtime
    matrix(i,3,3) = c1
    matrix(i,3,4) = bcoef*dqsat(innx(i))
    const(i,3)    = qr(innx(i))-conv2(innx(i))*dtime    &
                  - bcoef*(qsat(innx(i))                &
                  - dqsat(innx(i))*tt(innx(i)))

!temperature equation:
    matrix(i,4,1) = -lcscp*acoef
    matrix(i,4,2) = c0
    matrix(i,4,3) = c0
    matrix(i,4,4) = c1+lcscp*acoef*dqsat(innx(i))
    const(i,4)    = tt(innx(i))-lcscp*acoef             &
                  * (qsat(innx(i))-dqsat(innx(i))*tt(innx(i)))

 enddo

 end subroutine eau_satge0
      
!------------------------------------------------------------------------------
 subroutine eau_slvge0 &
               (len, nlm , nx , matrix , const, vect)
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.
    
 real (kind=dbl_kind), intent(in), dimension(len*nlm,4):: &
    const             !constants of microphysics equation.

 real (kind=dbl_kind), intent(in), dimension(len*nlm,4,4):: &
    matrix            !coefficients of microphysics equation.

!output arguments:
!----------------- 
 real (kind=dbl_kind), intent(out), dimension(len*nlm,4):: &
    vect              !solution vector of cloud microphysics equations.

!local variables:
 integer (kind=int_kind) :: i

!------------------------------------------------------------------------------
 do i = 1, nx
    vect(i,1) = (matrix(i,1,4)*const(i,4)-matrix(i,4,4)*const(i,1))
    vect(i,1) = vect(i,1)/(matrix(i,1,4)*matrix(i,4,1) &
              - matrix(i,1,1)*matrix(i,4,4))
    vect(i,4) = (const(i,1)*matrix(i,4,1)- const(i,4)*matrix(i,1,1))
    vect(i,4) = vect(i,4)/(matrix(i,1,4)*matrix(i,4,1) &
              - matrix(i,1,1)*matrix(i,4,4))      
    vect(i,2) = (const(i,2)-matrix(i,2,1)*vect(i,1)    &
              - matrix(i,2,4)*vect(i,4))/matrix(i,2,2)
    vect(i,3) = (const(i,3)-matrix(i,3,1)*vect(i,1)    &
                      -  matrix(i,3,2)*vect(i,2)       &
                      -  matrix(i,3,4)*vect(i,4))      &
                      /  matrix(i,3,3)
 enddo
            
 end subroutine eau_slvge0
       
!------------------------------------------------------------------------------
 subroutine eau_bgge0 &
                (     len ,     nlm ,   nx ,  innx ,  wght , &
                       tt ,     qwv , qsat , dqsat ,   qcw , &
                      qci ,     qri ,  fcs ,   fis ,  fevp , &
                    conv1 ,   conv2 ,  tau ,   lcs , dtime , &
                 matrixbg , constbg                          &
                )
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.

 integer (kind=int_kind), intent(in), dimension(len*nlm):: &
    innx              !location of grid-points.

 real (kind=dbl_kind), intent(in):: &
    dtime             !time-step                                           (s).

 real (kind=dbl_kind), intent(in), dimension(len*nlm):: &
    wght,            &!temperature partioning between cloud water/ice      (-).
    tt,              &!temperature                                         (K).
    qsat,            &!saturation mixing ratio                         (kg/kg).
    dqsat,           &!d(qsat)/dT                                    (kg/kg/K).
    qwv,             &!water vapor mixing ratio                        (kg/kg).
    qcw,             &!cloud water mixing ratio                        (kg/kg).
    qci,             &!cloud ice mixing ratio                          (kg/kg).
    qri,             &!snow mixing ratio                               (kg/kg).
    fcs,             &!f-Array: Collection of qcw by qri             (kg/kg/s).
    fis,             &!f-Array: Collection of qci by qi              (kg/kg/s).
    fevp,            &!f-Array: Evaporation rate                     (kg/kg/s).
    conv1,           &!coefficient related to autoconversion         (kg/kg/s).
    conv2,           &!coefficient related to autoconversion         (kg/kg/s).
    tau,             &!coefficient related to condensation             (per s).
    lcs               !latent heat                                    (J/kg/s).

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,5):: &
    constbg           !constants of microphysics equation.

 real (kind=dbl_kind), intent(out), dimension(len*nlm,5,5):: &
    matrixbg          !coefficients of microphysics equation.

!local variables:
 integer (kind=int_kind):: i

 real (kind=dbl_kind):: cpi, taui, acoef, bcoef, ccoef, dcoef, ecoef, fcoef

!------------------------------------------------------------------------------
 cpi = c1/spec_heat_cp

 do i = 1, nx
    taui  = c1 / tau(innx(i))
    acoef = (taui+qri(innx(i))/qsat(innx(i))*fevp(innx(i)))*dtime
    bcoef = wght(innx(i))*taui*dtime 
    ccoef = (c1-wght(innx(i)))*taui*dtime
    dcoef = fcs(innx(i))*qri(innx(i))*dtime
    ecoef = fis(innx(i))*qri(innx(i))*dtime    
    fcoef = qri(innx(i))/qsat(innx(i))*fevp(innx(i))*dtime

!water vapor equation:
    matrixbg(i,1,1) = c1+acoef  
    matrixbg(i,1,2) = c0
    matrixbg(i,1,3) = c0
    matrixbg(i,1,4) = c0
    matrixbg(i,1,5) = -acoef*dqsat(innx(i))
    constbg(i,1)    = qwv(innx(i))+acoef &
                    * (qsat(innx(i))-dqsat(innx(i))*tt(innx(i)))
 
!cloud water equation:
    matrixbg(i,2,1) = -bcoef
    matrixbg(i,2,2) = c1+dcoef
    matrixbg(i,2,3) = c0
    matrixbg(i,2,4) = c0
    matrixbg(i,2,5) = bcoef*dqsat(innx(i))
    constbg(i,2)    = qcw(innx(i))-bcoef &
                    * (qsat(innx(i))-dqsat(innx(i))*tt(innx(i)))

!cloud ice equation:
    matrixbg(i,3,1) = -ccoef
    matrixbg(i,3,2) = c0
    matrixbg(i,3,3) = c1+ecoef+conv1(innx(i))*dtime
    matrixbg(i,3,4) = c0
    matrixbg(i,3,5) = ccoef*dqsat(innx(i))
    constbg(i,3)    = qci(innx(i))-ccoef*(qsat(innx(i)) &
                    - dqsat(innx(i))*tt(innx(i)))+conv2(innx(i))*dtime 

!snow equation:
    matrixbg(i,4,1) = -fcoef
    matrixbg(i,4,2) = -dcoef
    matrixbg(i,4,3) = -ecoef-conv1(innx(i))*dtime
    matrixbg(i,4,4) = c1
    matrixbg(i,4,5) = fcoef*dqsat(innx(i))
    constbg(i,4)    = qri(innx(i))-fcoef*(qsat(innx(i)) &
                    - dqsat(innx(i))*tt(innx(i)))-conv2(innx(i))*dtime

!temperature equation:
    matrixbg(i,5,1) = -(lcs(innx(i))*taui*dtime+lsub*fcoef)*cpi
    matrixbg(i,5,2) = -lfus*cpi*qri(innx(i))*fcs(innx(i))*dtime
    matrixbg(i,5,3) = c0
    matrixbg(i,5,4) = c0
    matrixbg(i,5,5) = c1+(lcs(innx(i))*taui*dtime+lsub*fcoef)         &
                    * dqsat(innx(i))*cpi
    constbg(i,5)    = tt(innx(i)) - (lcs(innx(i))*taui*dtime+lsub*fcoef) &
                    * cpi*(qsat(innx(i))-dqsat(innx(i))*tt(innx(i)))

 enddo
      
 end subroutine eau_bgge0

!------------------------------------------------------------------------------
 subroutine eau_slvbgge0 &
                (len ,  nlm , nx , matrixbg , constbg , vectbg)
!------------------------------------------------------------------------------
!argument list variables:
!input arguments:
!----------------
 integer (kind=int_kind), intent(in):: &
    len,             &!length of sub-domain.
    nx,              &!number of grid-points.
    nlm               !number of layers.

 real (kind=dbl_kind), intent(in), dimension(len*nlm,5):: &
    constbg           !constants of microphysics equation.

 real (kind=dbl_kind), intent(in), dimension(len*nlm,5,5):: &
    matrixbg          !coefficients of microphysics equation.

!output arguments:
!-----------------
 real (kind=dbl_kind), intent(out), dimension(len*nlm,5):: &
    vectbg            !solution vector of cloud microphysics equations.

!local variables:
 integer (kind=int_kind):: i

 real (kind=dbl_kind), dimension(nx):: &
    alpha, alpha1, alpha2, beta, beta1, beta2

!------------------------------------------------------------------------------
 do i = 1, nx
    alpha1(i) = matrixbg(i,2,5)*matrixbg(i,1,1) &
              - matrixbg(i,2,1)*matrixbg(i,1,5)
    alpha2(i) = matrixbg(i,5,5)*matrixbg(i,1,1) &
              - matrixbg(i,5,1)*matrixbg(i,1,5)
    alpha(i)  = matrixbg(i,5,2)*alpha1(i)       &
              - matrixbg(i,2,2)*alpha2(i)

    beta1(i)  = constbg(i,2)*matrixbg(i,1,1)-constbg(i,1)*matrixbg(i,2,1)
    beta2(i)  = constbg(i,5)*matrixbg(i,1,1)-constbg(i,1)*matrixbg(i,5,1)
    beta(i)   = matrixbg(i,5,2)*beta1(i)-matrixbg(i,2,2)*beta2(i)

    vectbg(i,5) = beta(i)/alpha(i)
    vectbg(i,1) = (constbg(i,1)-matrixbg(i,1,5)*vectbg(i,5))/matrixbg(i,1,1)
    vectbg(i,2) = (constbg(i,2)-matrixbg(i,2,1)*vectbg(i,1)    &
                - matrixbg(i,2,5)*vectbg(i,5))/matrixbg(i,2,2)
    vectbg(i,3) = (constbg(i,3)-matrixbg(i,3,1)*vectbg(i,1)    &
                - matrixbg(i,3,5)*vectbg(i,5))/matrixbg(i,3,3)
    vectbg(i,4) = (constbg(i,4)-matrixbg(i,4,1)*vectbg(i,1)    &
                - matrixbg(i,4,2)*vectbg(i,2)                  &
                - matrixbg(i,4,3)*vectbg(i,3)                  &
                - matrixbg(i,4,5)*vectbg(i,5))/matrixbg(i,4,4)
 enddo

 end subroutine eau_slvbgge0

!==============================================================================

 end module eau_supmicrop

!==============================================================================
