!-----------------------------------------------------------------------

 subroutine eau_supsat                                  &
                (    len ,     nlm ,     np1 ,     np2, &
                     np3 ,    inp1 ,    inp2 ,    inp3, &
                   dtelq ,      tl ,    qwvl ,    qcwl, &
                    qrwl ,    qcil ,    qril ,  qsatsl, &
                 dqsatsl , fpracwl , fpsacil , fpsacwl, &
                 fprevpl , fpsevpl ,   wghtl            &
                )
      
 use kinds
 use eau_params
 use eau_supmicrop

 implicit none
      
!-----------------------------------------------------------------------

! eau_supsat sets solving the cloud microphysics equations for super-
! saturated grid-points. 
! Laura D. Fowler/slikrock (03-07-00).

! send comments to laura@atmos.colostate.edu.

! references:
! Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1996: Liquid and Ice
! Cloud Microphysics in the CSU General Circulation Model.
! Part I: Model description and simulated Microphysical processes.
! Journal of Climate, 9, 489-529.

! Randall, D.A., and L.D. Fowler, 1999: EAUliq: The next generation.
! Dept. of Atmospheric Science Paper 673, Dept. of Atmospheric Science.
! Colorado State University, Fort Collins, Colorado, 65 pp.

! modifications:
!      none.

! subroutines called:
!      eau_satge0   :Computes coefficients of microphysics equations for
!                    super-saturared grid-points.
!      eau_slvge0   :Solves microphysics equations for super-saturated
!                    grid points.
!      eau_bgge0    :Computes coefficients of microphysics equations for
!                    super-saturated grid-points in mixed-phase window.
!      eau_slvbgge0 :Solves microphysics equations for super-saturated
!                    grid-points in mixed-phase window.

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: & 
   len, &    ! length of sub domain.
   nlm, &    ! number of layers.
   np1, &    ! number of grid-points for the ice phase.
   np2, &    ! number of grid-points for the mixed phase.
   np3       ! number of grid-points for the liquid phase.

 integer (kind=int_kind), intent(in), dimension(len*nlm) :: &
   inp1, &   ! location of nn1 grid-points.
   inp2, &   ! location of nn2 grid-points.
   inp3      ! location of nn3 grid-points.
     
 real (kind=dbl_kind), intent(in) :: &
   dtelq     ! cloud microphysical time-step                    (s).

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
   qsatsl, & ! saturation mixing ratio                      (kg/kg).
   dqsatsl, &! d(qsatsl) with respect to temperature      (kg/kg/K).
   fpracwl, &! f-array:Collection of cloud water by rain  (kg/kg/s).
   fpsacil, &! f-array:Collection of cloud ice by snow    (kg/kg/s).
   fpsacwl, &! f-array:Collection of cloud water by snow  (kg/kg/s).
   fprevpl, &! f-array:Evaporation of rain                (kg/kg/s).
   fpsevpl, &! f-array:Evaporation of snow                (kg/kg/s).
   wghtl     ! temperature weight in mixed phase window         (-).

! input/output arguments:
! -----------------------

 real (kind=dbl_kind), intent(inout), dimension(len,nlm) :: &
   tl, &    ! temperature                                       (K).
   qwvl, &  ! water vapor mixing ratio                      (kg/kg).
   qcwl, &  ! cloud water mixing ratio                      (kg/kg).
   qrwl, &  ! rain mixing ratio                             (kg/kg).
   qcil, &  ! cloud ice mixing ratio                        (kg/kg).
   qril     ! snow mixing ratio                             (kg/kg).

! local variables:

 integer (kind=int_kind) :: i
 
 integer (kind=int_kind), dimension(len*nlm) :: & ! for array bounds check
   iinp1, iinp2, iinp3, &  ! horizontal index of inn1, inn2, inn3 resp.
    lnp1, lnp2, lnp3       ! vertical index of inn1, inn2, inn3 resp.

 real (kind=dbl_kind), dimension(len,nlm) :: tl0

 real (kind=dbl_kind), dimension(len*nlm) :: &
   conv1, conv2, taucst, lcs

 real (kind=dbl_kind), dimension(len*nlm,4) :: const, vect

 real (kind=dbl_kind), dimension(len*nlm,4,4) :: matrix

 real (kind=dbl_kind), dimension(len*nlm,5) :: constbg, vectbg
 
 real (kind=dbl_kind), dimension(len*nlm,5,5) :: matrixbg  

!-----------------------------------------------------------------------

! compute horizontal and  vertical indices from total index

 do i = 1,np1
    iinp1(i) = mod(inp1(i)-1,len) + 1 
    lnp1(i) = (inp1(i) - iinp1(i))/len + 1
 enddo
 do i = 1,np2
    iinp2(i) = mod(inp2(i)-1,len) + 1
    lnp2(i) = (inp2(i) - iinp2(i))/len + 1
 enddo
 do i = 1,np3
    iinp3(i) = mod(inp3(i)-1,len) + 1
    lnp3(i) = (inp3(i) - iinp3(i))/len + 1
 enddo

 conv1    = c0
 conv2    = c0
 taucst   = c0
 lcs      = c0
 const    = c0
 constbg  = c0
 vect     = c0
 matrix   = c0
 vectbg   = c0
 matrixbg = c0
 tl0      = tl
      
!---- ice phase:

 if (np1 > 0) then

    do i = 1,np1 
       if (qcil(iinp1(i),lnp1(i)) >= qci0) then
          conv1(inp1(i)) = betaelq*exp(0.025*(tl(iinp1(i),lnp1(i))-t00))
          conv2(inp1(i)) = conv1(inp1(i))*qci0
       endif
       taucst(inp1(i)) = tauf
       lcs(inp1(i))    = lsub
    enddo

    call eau_satge0                               &
             (  len ,    nlm ,     np1 ,    inp1, &
                 tl ,   qwvl ,  qsatsl , dqsatsl, &
               qcil ,   qril , fpsacil , fpsevpl, &
              conv1 ,  conv2 ,  taucst ,     lcs, &
              dtelq , matrix ,   const            &
             )

    call eau_slvge0                        &
             (  len ,  nlm , np1 , matrix, &
              const , vect                 &
             )

    do i = 1, np1
       qwvl(iinp1(i),lnp1(i)) = vect(i,1)
       qcil(iinp1(i),lnp1(i)) = vect(i,2)
       qril(iinp1(i),lnp1(i)) = vect(i,3)
       tl(iinp1(i),lnp1(i))   = vect(i,4)
    enddo

 endif

!---- mixed phase:

 if (np2 > 0) then

    do i = 1, np2 
       conv1(inp2(i))  = c0
       conv2(inp2(i))  = c0
       taucst(inp2(i)) = tauf
       lcs(inp2(i))    = hltm+(1.-wghtl(iinp2(i),lnp2(i)))*lfus
    enddo

    call eau_bgge0                                   &
             (     len ,     nlm ,     np2 ,   inp2, &
                 wghtl ,      tl ,    qwvl , qsatsl, &
               dqsatsl ,    qcwl ,    qcil ,   qril, &
               fpsacwl , fpsacil , fpsevpl ,  conv1, &
                 conv2 ,  taucst ,     lcs ,  dtelq, &
              matrixbg , constbg                     &
             )

    call eau_slvbgge0                            &
             (    len ,    nlm , np2 , matrixbg, &
              constbg , vectbg                   &
             ) 

    do i = 1, np2
       if (vectbg(i,2) >= qcw0) then
          vectbg(i,4) = vectbg(i,4)+alphaelq &
                      * (vectbg(i,2)-qcw0)*dtelq
          vectbg(i,5) = vectbg(i,5)+lfus/spec_heat_cp &
                      * alphaelq*(vectbg(i,2)-qcw0)*dtelq
          vectbg(i,2) = vectbg(i,2)-alphaelq &
                      * (vectbg(i,2)-qcw0)*dtelq
       endif
       if (vectbg(i,3) >= qci0) then
          conv1(inp2(i)) = betaelq*exp(0.025*(tl(iinp2(i),lnp2(i))-t00))
          vectbg(i,4) = vectbg(i,4)+conv1(inp2(i)) &
                      * (vectbg(i,3)-qci0)*dtelq
          vectbg(i,3) = vectbg(i,3)-conv1(inp2(i)) &
                      * (vectbg(i,3)-qci0)*dtelq
       endif
       qwvl(iinp2(i),lnp2(i)) = vectbg(i,1)
       qcwl(iinp2(i),lnp2(i)) = vectbg(i,2)
       qcil(iinp2(i),lnp2(i)) = vectbg(i,3)
       qril(iinp2(i),lnp2(i)) = vectbg(i,4)
       tl(iinp2(i),lnp2(i))   = vectbg(i,5)
    enddo

 endif
   
!---- liquid phase:

 if (np3 > 0) then

    do i = 1, np3 
       if (qcwl(iinp3(i),lnp3(i)) >= qcw0) then
          conv1(inp3(i)) = alphaelq
          conv2(inp3(i)) = conv1(inp3(i))*qcw0
       endif
       taucst(inp3(i)) = taul
       lcs(inp3(i))    = hltm
    enddo

    call eau_satge0                               &
             (  len ,    nlm ,     np3 ,    inp3, &
                 tl ,   qwvl ,  qsatsl , dqsatsl, &
               qcwl ,   qrwl , fpracwl , fprevpl, &
              conv1 ,  conv2 ,  taucst ,     lcs, &
              dtelq , matrix ,   const            &
             )

    call eau_slvge0                        &
             (  len ,  nlm , np3 , matrix, &
              const , vect                 &
             )

    do i = 1, np3
       qwvl(iinp3(i),lnp3(i)) = vect(i,1)
       qcwl(iinp3(i),lnp3(i)) = vect(i,2)
       qrwl(iinp3(i),lnp3(i)) = vect(i,3)
       tl(iinp3(i),lnp3(i))   = vect(i,4)
    enddo

 endif

 end subroutine eau_supsat
      
!-----------------------------------------------------------------------
