!------------------------------------------------------------------------------

module eau_util
      
use kinds
use eau_params
      
implicit none
private
save
      
public :: eau_sat

!------------------------------------------------------------------------------
contains
!------------------------------------------------------------------------------
subroutine eau_sat &
               (   len ,  pl ,    tl ,  qwvl ,  qsats , &
                dqsats , ess , dtess ,  sats ,   wght   &                         
               )
!------------------------------------------------------------------------------
!eau_sat computes the saturation mixing ratio, vapor pressure, and saturation,
!and their derivatives with respect to temperature over water, ice and mixed-
!phase clouds. the parameterization is that used in the Community Climate Com-
!munity Climate Model at NCAR.
!Laura D. Fowler /slikrock (07-01-01).

!send comments to laura@atmos.colostate.edu.

!subroutines called:
!none.

!argument list variables:
!input arguments:
!----------------

integer (kind=int_kind), intent(in):: &
   len                !length of vector.

real(kind=dbl_kind), intent(in), dimension(len):: &
   pl,               &!pressure                                           (Pa).
   tl                 !temperature                                         (K).

real(kind=dbl_kind), intent(in), dimension(len), optional :: &
   qwvl               !water vapor mixing ratio                        (kg/kg).

!output arguments:
!-----------------

real (kind=dbl_kind), intent(out), dimension(len), optional :: &
   qsats,            &!saturation mixing ratio                         (kg/kg).
   dqsats,           &!derivative of qsat with respect to temperature(kg/kg/K).
   ess,              &!saturation vapor pressure                          (Pa).
   dtess,            &!derivative of es with respect to temperature     (Pa/K).
   sats,             &!saturation                                          (-).
   wght               !partitioning between cloud water and cloud ice
                      !for mixed-phase clouds                              (-).      

!local variables:

integer (kind=int_kind):: i

real (kind=dbl_kind):: &
   twmin=173.16,     &!lowest allowed temperature boundary for water       (K).
   twmax=373.16,     &!highest allowed temperature boundary for water      (K).     
   timin=173.16,     &!lowest allowed temperature boundary for ice         (K).
   timax=273.16,     &!highest allowed temperature boundary for ice        (K).
   tnull=273.16       !freezing temperature                                (K).

real (kind=dbl_kind) :: tstl , t0tl

real (kind=dbl_kind), dimension(len):: &
      esw ,      esi ,&
      esm ,&
      tl0 ,&
    wghtm , qsat_tem ,&
   es_tem , dtes_tem ,  wght_tem

!ccm parameterization of saturation vapor pressure over water and ice:

real (kind=dbl_kind), parameter:: &
   ps = 1013.246,    &!reference pressure                                (hPa).
   ts = 373.16,      &!reference temperature                               (K).
   t0 = 273.16        !freezing temperature                                (K)
     
real (kind=dbl_kind):: &
       e1 ,   e2 ,     f ,    f1 ,&
       f2 ,   f3 ,    f4 ,    f5 ,&
   lphase , term , term1 , term2 ,&
   term3     

!------------------------------------------------------------------------------

do i = 1, len

   tl0(i)    = max(twmin,tl(i))
   tl0(i)    = min(twmax,tl0(i))
   if(tl0(i).ge.tbgmin) then
      tstl   = ts / tl0(i)
      e1     = 11.344_dbl_kind*(1.0_dbl_kind - tl0(i)/ts)
      e2     = -3.49149_dbl_kind*(tstl - 1.0_dbl_kind)
      f1     = -7.90298_dbl_kind*(tstl - 1.0_dbl_kind)
      f2     = 5.02808_dbl_kind*log10(tstl)
      f3     = -1.3816_dbl_kind*(10.0_dbl_kind**e1-1.0_dbl_kind) * 0.0000001_dbl_kind
      f4     = 8.1328_dbl_kind*(10.0_dbl_kind**e2-1.0_dbl_kind) * 0.001_dbl_kind
      f5     = log10(ps)
      f      = f1 + f2 + f3 + f4 + f5

      esw(i) = (10.0_dbl_kind**f)*1.e+02_dbl_kind
      esw(i) = min(esw(i),pl(i)*0.9_dbl_kind)
      wght_tem(i)  = c1
      es_tem(i)    = esw(i)
   endif

enddo

!saturation over ice:

do i = 1, len
   if(tl0(i).lt.timax) then

      tl0(i)    = max(tl0(i),timin)
      t0tl      = t0 / tl0(i)
      term1     = 2.01889049_dbl_kind/(t0tl)
      term2     = 3.56654_dbl_kind*log(t0tl)
      term3     = 20.947031_dbl_kind*(t0tl)
      term      = term1 + term2 + term3

      esi(i)    = 575.185606e10_dbl_kind*exp(-(term1 + term2 + term3))
      esi(i)    = min(esi(i),pl(i)*0.9_dbl_kind)
      wght_tem(i)  = c0
      es_tem(i)    = esi(i)

   endif
enddo

!interpolated saturation variables:

do i = 1, len
   if(tl0(i).lt.tbgmax .and. tl0(i).ge.tbgmin) then

      wghtm(i)  = (tl0(i)-tbgmin)/(tbgmax-tbgmin)
      esm(i)    = wghtm(i)*esw(i) + (1.-wghtm(i))*esi(i)
      esm(i)    = min(esm(i),pl(i)*0.9)
      es_tem(i)    = esm(i)
      wght_tem(i)  = wghtm(i)
      
   endif
enddo

if (present(ess)) ess = es_tem

if ( present(dtess).or.present(dqsats) )  then
   do i = 1, len
      lphase    = hltm*wght_tem(i)+lsub*(1.-wght_tem(i))
      dtes_tem(i) = lphase*es_tem(i)/(rv*tl0(i)*tl0(i))
   enddo
   if(present(dtess)) dtess = dtes_tem
endif

if (present(qsats) .or. present(dqsats) .or. present(sats) )  then
   qsat_tem = 0.622_dbl_kind * es_tem / (pl - es_tem)
   if(present(qsats) ) qsats = qsat_tem
   if(present(dqsats)) dqsats = qsat_tem*dtes_tem*pl / (es_tem* &
                 (pl-es_tem))
   if(present(sats)) then 
      if(present(qwvl)) then
         sats = qwvl / qsat_tem
      else
         sats = c0
      endif
   endif
endif

if (present(wght))   wght   = wght_tem

end subroutine eau_sat

!------------------------------------------------------------------------------

end module eau_util

!------------------------------------------------------------------------------
      
      
