   MODULE flux_corrected_transport
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  PURPOSE:  Advection of mass and tracers using flux-corrected transport.
!            See Piotr Smolarkiewicz (hereafter (PS)) "Nonoscillatory 
!            Advection Schemes"  ECMF seminar proceedings. 9-13 Sept. 1991.
!            Numerical methods in atmospheric models.  Vol 1.
!
!            sigma coordinate version.
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   USE kinds

   USE model_parameters
   USE domain_decomposition
   USE sigma_coordinate
   USE grid_metrics
   USE gather_scatter
   USE wrap_bugs6_data

   IMPLICIT NONE
   PRIVATE
#ifdef fluxcorrectedtransport
   SAVE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  mass tendency
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   REAL (KIND=dbl_kind), DIMENSION(nwalls,iim,jjm, nlm  ,nttend,nsdm) ::     &
      msh_f
   REAL (KIND=dbl_kind), DIMENSION(       iim,jjm, nlmp1,nttend,nsdm) ::     &
      msv_f
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  tracer tendency
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   REAL (KIND=dbl_kind),PUBLIC,                                              &
                        DIMENSION(iim,jjm, nlm  ,ntracer,nttend,nsdm) ::     &
      trc_f

   REAL (KIND=dbl_kind), DIMENSION(       iim,jjm, nlm  ,       nsdm) ::     &
      d_volume,      &!
      d_volume_inv

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  PUBLIC SUBROUTINES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   PUBLIC ::                                                                 &
      initialize_fct,set_mass_fct,set_tracer_fct,set_mass_flux_horizontal,   &
      set_mass_low

   CONTAINS
!=======================================================================
!  BEGIN initialize_fct
!=======================================================================
   SUBROUTINE initialize_fct ()
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  PURPOSE: Initialize the flux-corrected transport.  This subroutine needs
!           to be called after the metrics and wrap have been initialized.
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~      
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      icorner,ispace,i,j,k,nsd
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
   msh_f(:,:,:,:,:,:) = 0.0_dbl_kind
   msv_f(  :,:,:,:,:) = 0.0_dbl_kind

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  wrap and subzoneshift normal and dc (in grid_metrics) so that the horizontal 
!  mass fluxes are correctly computed in SUBROUTINE set_mass_flux_horizontal
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   DO ispace = 1,3
      CALL wrap_bugs6 ('fct', vrtx_1L=normal(:,ispace,:,:,:,:))
   ENDDO
   CALL wrap_bugs6 ('fct', vrtx_1L=dc(:,:,:,:,:))
   DO nsd = 1,nsdm
      DO j = 1,jjm
         DO i = 1,iim
            DO icorner=1,ncorners
               IF (subzone_shift(icorner,i,j,nsd)/=0) THEN
                  DO ispace = 1,3
                     normal(:,ispace,icorner,i,j,nsd) =                      &
                           CSHIFT (normal(:,ispace,icorner,i,j,nsd),         &   
                                   SHIFT=subzone_shift(icorner,i,j,nsd))
                  ENDDO
                  dc(:,icorner,i,j,nsd) =                                    &
                           CSHIFT (dc    (:,icorner,i,j,nsd),                &   
                                   SHIFT=subzone_shift(icorner,i,j,nsd))
               ENDIF
            ENDDO
         ENDDO
      ENDDO
   ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  compute the volume and inverse volume of each cell
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   DO k = 1,nlm
      d_volume(:,:,k,:) = area(:,:,:)*(sigl2(k+1)-sigl2(k))
   ENDDO

   WHERE (d_volume(:,:,:,:) /= 0.0_dbl_kind) 
      d_volume_inv(:,:,:,:) = 1.0_dbl_kind/d_volume(:,:,:,:)
   ELSEWHERE
      d_volume_inv(:,:,:,:) = 0.0_dbl_kind
   ENDWHERE

   END SUBROUTINE initialize_fct
!=======================================================================
!  BEGIN initialize_fct
!=======================================================================

!=======================================================================
!  BEGIN set_mass_fct
!=======================================================================
#ifdef no_pbl
   SUBROUTINE set_mass_fct (mss,mss_c,mss_edge,                              &
                            u,sigma_dotu,   &
                            mss_low,mss_flux_horz,mss_flux_vert,&
                            mss_fct)
#else
   SUBROUTINE set_mass_fct (mss,mss_c,mss_edge,                              &
                            u,sigma_dotu,sigma_dote,entrainment,cumulusmf,   &
                            mss_low,mss_flux_horz,mss_flux_vert,mss_flxe_vert,&
                            mss_fct)
#endif
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm, nlm  ,ntprog,nsdm) ::     &
      mss            !  mass defined at cell centers
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(       ncorners,iim,jjm, nlm  ,       nsdm) ::     &
      mss_c          !  mass defined at cell corners
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm, nlmp1,       nsdm) ::     &
      mss_edge       !  mass defined at layer edges
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(nspace,ncorners,iim,jjm, nlm  ,ntprog,nsdm) ::     &
      u              ! horizontal velocity
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm, nlmp1,       nsdm) ::     &
      sigma_dot ,      !  "vertical mass velocity due to mass flux divergence"
#ifndef no_pbl
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm, nlmp1,       nsdm) ::     &
      sigma_dote       !  "vertical mass velocity due to pbl top processes"
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm,              nsdm) ::     &
      entrainment,  &!  entrainment at the PBL top
      cumulusmf      !  cumulus mass flux at the PBL top
#endif
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),        DIMENSION(       iim,jjm, nlm  ,nsdm) ::     &
      mss_low
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::     &
      mss_flux_horz
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(       iim,jjm, nlmp1,nsdm) ::     &
      mss_flux_vert
#ifndef no_pbl
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(       iim,jjm, nlmp1,nsdm) ::     &
      mss_flxe_vert
#endif
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(       iim,jjm, nlm  ,nsdm) ::     &
      mss_fct  !  mass modified with the fct algorithm
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) :: nsd
   REAL (KIND=dbl_kind),        DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::     &
      mss_flx1_horz,   &!
      mss_flx2_horz,   &!
      mss_anti_horz     !
   REAL (KIND=dbl_kind),        DIMENSION(       iim,jjm, nlmp1,nsdm) ::     &
      mss_flx1_vert,   &!
      mss_flx2_vert,   &!
      mss_anti_vert     !
   REAL (KIND=dbl_kind),        DIMENSION(       iim,jjm, nlm  ,nsdm) ::     &
      beta_dn,         &!
      beta_up

!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SET THE HORIZONTAL MASS FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_mass_flux_horizontal (         mss          (    :,:,:,:,:),     &
                                           mss_c        (  :,:,:,:,  :),     &
                                           u            (:,:,:,:,:,:,:),     &
                                           msh_f        (  :,:,:,:,:,:),     &
                                           mss_flx1_horz(  :,:,:,:,  :),     &
                                           mss_flx2_horz(  :,:,:,:,  :),     &
                                           mss_anti_horz(  :,:,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SET THE VERTICAL MASS FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef no_pbl
   CALL set_mass_flux_vertical (           mss          (    :,:,:,:,:),     &
                                           mss_edge     (    :,:,:,  :),     &
                                           sigma_dotu    (    :,:,:,  :),    &
                                           msv_f        (    :,:,:,:,:),     &
                                           mss_flx1_vert(    :,:,:,  :),     &
                                           mss_flx2_vert(    :,:,:,  :),     &
                                           mss_anti_vert(    :,:,:,  :))
#else
   CALL set_mass_flux_vertical (           mss          (    :,:,:,:,:),     &
                                           mss_edge     (    :,:,:,  :),     &
                                           sigma_dotu    (    :,:,:,  :),    &
                                           sigma_dote    (    :,:,:,  :),    &
                                           entrainment  (    :,:,    :),     &
                                           cumulusmf    (    :,:,    :),     &
                                           msv_f        (    :,:,:,:,:),     &
                                           mss_flx1_vert(    :,:,:,  :),     &
                                           mss_flx2_vert(    :,:,:,  :),     &
                                           mss_flxe_vert(    :,:,:,  :),     &
                                           mss_anti_vert(    :,:,:,  :))
#endif
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE LOW ORDER SOLUTION  
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_mass_low (                     mss          (    :,:,:,:,:),     &
                                           mss_flx2_horz(  :,:,:,:,  :),     &
                                           mss_flx2_vert(    :,:,:,  :),     &
                                           mss_low      (    :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE SMOLARKIEWICZ BETA_DOWN THING  (PS (16b))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   CALL set_beta_weights_mss (             mss_anti_horz(  :,:,:,:,  :),     &
!                                           mss_anti_vert(    :,:,:,  :),     &
!                                           mss_low      (    :,:,:,  :),     &
!                                           beta_dn      (    :,:,:,  :),     &
!                                           beta_up      (    :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE CORRECTED ANTI-DIFFUSIVE MASS FLUX (PS (15))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   CALL set_corrected_mss_anti (           beta_dn      (    :,:,:,  :),     &
!                                           beta_up      (    :,:,:,  :),     &
!                                           mss_anti_horz(  :,:,:,:,  :),     &
!                                           mss_anti_vert(    :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE FLUX CORRECTED MASS FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   mss_flux_horz(:,:,:,:,:) =                                                &
                       mss_flx2_horz(:,:,:,:,:)
   mss_flux_vert(  :,:,:,:) =                                                &
                       mss_flx2_vert(  :,:,:,:)
!   mss_flux_horz(:,:,:,:,:) =                                                &
!                       mss_flx1_horz(:,:,:,:,:)+mss_anti_horz(:,:,:,:,:)
!   mss_flux_vert(  :,:,:,:) =                                                &
!                       mss_flx1_vert(  :,:,:,:)+mss_anti_vert(  :,:,:,:)
!  COMPUTE FLUX CORRECTED MASS SOLUTION (PS (10))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   CALL set_flux_corrected_mass (          mss_low      (    :,:,:,  :),     &
!                                           mss_anti_horz(  :,:,:,:,  :),     &
!                                           mss_anti_vert(    :,:,:,  :),     &
!                                           mss_fct      (    :,:,:,  :))
!


    mss_fct(:,:,:,:) = mss_low(:,:,:,:)

   END SUBROUTINE set_mass_fct
!=======================================================================
!  END  set_mass_fct
!=======================================================================

!=======================================================================
!  BEGIN set_mass_flux_horizontal
!=======================================================================
   SUBROUTINE set_mass_flux_horizontal (mss,mss_c,u,                         &
                        msh_f,mss_flx1_horz,mss_flx2_horz,mss_anti_horz)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                  DIMENSION(                iim,jjm, nlm,ntprog,nsdm) ::     &
      mss       !  mass defined at cell centers
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                  DIMENSION(       ncorners,iim,jjm, nlm,       nsdm) ::     &
      mss_c     !  mass defined at cell corners
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                  DIMENSION(nspace,ncorners,iim,jjm, nlm,ntprog,nsdm) ::     &
      u         ! horizontal velocity
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                           DIMENSION(nwalls,iim,jjm, nlm,nttend,nsdm) ::     &
      msh_f             !
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                           DIMENSION(nwalls,iim,jjm, nlm,       nsdm) ::     &
      mss_flx1_horz,   &!
      mss_flx2_horz,   &!
      mss_anti_horz     !
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd,ns,nd
   REAL (KIND=dbl_kind) ::                                                   &
      wind1,wind2,wind3
   REAL (KIND=dbl_kind),DIMENSION(      nspace,ncorners,iim,jjm) ::          &
      mssflx        !
   REAL (KIND=dbl_kind),DIMENSION(ndata,       ncorners,iim,jjm) ::          &
      flux_temp,   &!
      wind_temp
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

   CALL wrap_bugs6 ('fct', vrtx=u(:,:,:,:,:,n3,:))

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE FIRST-ORDER HORIZONTAL MASS FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   mss_flx1_horz(:,:,:,:,:) = 0.0_dbl_kind
!   DO nsd = 1,nsdm
!      DO k = 1,nlm
!         DO nd = 1,ndata
!            wind_temp(nd,:,:,:) = dc(nd,:,:,:,nsd)*                          &
!                            (u(1,:,:,:,k,n3,nsd)*normal(nd,1,:,:,:,nsd)+     &
!                             u(2,:,:,:,k,n3,nsd)*normal(nd,2,:,:,:,nsd))
!         ENDDO
!         DO j = 2,jjm-1
!           DO i = 2,iim-1
!               wind1 = -wind_temp(2,2,i,j+1)+wind_temp(1,1,i  ,j)
!               wind2 = -wind_temp(3,1,i,j  )+wind_temp(1,2,i  ,j)
!               wind3 = -wind_temp(3,2,i,j  )+wind_temp(2,1,i+1,j)
!               mss_flx1_horz(1,i,j,k,nsd) =                                  &
!                                 mss(i  ,j  ,k,n3,nsd)*MAX (zero,wind1)+     &
!                                 mss(i-1,j  ,k,n3,nsd)*MIN (zero,wind1)
!               mss_flx1_horz(2,i,j,k,nsd) =                                  &
!                                 mss(i  ,j  ,k,n3,nsd)*MAX (zero,wind2)+     &
!                                 mss(i-1,j-1,k,n3,nsd)*MIN (zero,wind2)
!               mss_flx1_horz(3,i,j,k,nsd) =                                  &
!                                 mss(i  ,j  ,k,n3,nsd)*MAX (zero,wind3)+     &
!                                 mss(i  ,j-1,k,n3,nsd)*MIN (zero,wind3)
!            ENDDO
!         ENDDO
!         mss_flx1_horz(:,:,:,k,nsd) =                                        &
!                        (sigl2(k+1)-sigl2(k))*mss_flx1_horz(:,:,:,k,nsd)
!      ENDDO
!   ENDDO

   DO nsd = 1,nsdm
      DO k = 1,nlm
         CALL upstream_mass_flux (ipad,iim,jjm,                              &
                                        ndata,ncorners,nwalls,nspace,r3,     &
                                      dc           (:,  :,:,:,     nsd),     &
                                      normal       (:,:,:,:,:,     nsd),     &
                                      u            (  :,:,:,:,k,n3,nsd),     &
                                      mss          (      :,:,k,n3,nsd),     &
                                      mss_flx1_horz(    :,:,:,k,   nsd))
         mss_flx1_horz(:,:,:,k,nsd) =                                        &
                        (sigl2(k+1)-sigl2(k))*mss_flx1_horz(:,:,:,k,nsd)
      ENDDO
   ENDDO

   CALL wrap_bugs6 ('fct', edge_scalar=mss_flx1_horz)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE HIGH-ORDER HORIZONTAL MASS FLUXES TENDENCY
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   msh_f     (:,:,:,:,n3_f,:) = 0.0_dbl_kind
   CALL wrap_bugs6 ('fct', vrtx_scalar=mss_c(  :,:,:,:,    :))

!   DO nsd = 1,nsdm
!      DO k = 1,nlm
!         DO ns = 1,nspace
!            mssflx   (ns,:,:,:) = mss_c(:,:,:,k,nsd)*u(ns,:,:,:,k,n3,nsd)
!         ENDDO
!         DO nd = 1,ndata
!            flux_temp(nd,:,:,:) = dc(nd,:,:,:,nsd)*                          &
!                                (mssflx(1,:,:,:)*normal(nd,1,:,:,:,nsd)+     &
!                                 mssflx(2,:,:,:)*normal(nd,2,:,:,:,nsd))
!         ENDDO
!         DO j = 2,jjm-1
!            DO i = 2,iim-1
!               msh_f(1,i,j,k,n3_f,nsd) =                                     &
!                              -flux_temp(2,2,i,j+1)+flux_temp(1,1,i  ,j)
!               msh_f(2,i,j,k,n3_f,nsd) =                                     &
!                              -flux_temp(3,1,i,j  )+flux_temp(1,2,i  ,j)
!               msh_f(3,i,j,k,n3_f,nsd) =                                     &
!                              -flux_temp(3,2,i,j  )+flux_temp(2,1,i+1,j)
!            ENDDO
!         ENDDO
!         msh_f(:,:,:,k,n3_f,nsd) =                                           &
!                           (sigl2(k+1)-sigl2(k))*msh_f(:,:,:,k,n3_f,nsd)
!      ENDDO
!   ENDDO

   DO nsd = 1,nsdm
      DO k = 1,nlm
         CALL centered_mass_flux (ipad,iim,jjm,                              &
                                        ndata,ncorners,nwalls,nspace,r3,     &
                                           dc    (:,  :,:,:,       nsd),     &
                                           normal(:,:,:,:,:,       nsd),     &
                                           mss_c (    :,:,:,k,     nsd),     &
                                           u     (  :,:,:,:,k,n3  ,nsd),     &
                                           mss   (      :,:,k,n3  ,nsd),     &
                                           msh_f (    :,:,:,k,n3_f,nsd))
         msh_f(:,:,:,k,n3_f,nsd) =                                           &
                           (sigl2(k+1)-sigl2(k))*msh_f(:,:,:,k,n3_f,nsd)
      ENDDO
   ENDDO

   CALL wrap_bugs6 ('fct', edge_scalar=msh_f(:,:,:,:,n3_f,:))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE HIGH-ORDER HORIZONTAL MASS FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   mss_flx2_horz(:,:,:,:,:) = w3*msh_f(:,:,:,:,n3_f,:)+                      &
                              w2*msh_f(:,:,:,:,n2_f,:)+                      &
                              w1*msh_f(:,:,:,:,n1_f,:)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE ANTI-DIFFUSIVE HORIZONTAL MASS FLUXES (PS (8))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   mss_anti_horz(:,:,:,:,:) =                                                &
                       mss_flx2_horz(:,:,:,:,:)-mss_flx1_horz(:,:,:,:,:)

  END SUBROUTINE set_mass_flux_horizontal
!=======================================================================
!  END  set_mass_flux_horizontal
!=======================================================================

!=======================================================================
!  BEGIN  set_mass_flux_vertical
!=======================================================================
#ifdef no_pbl
   SUBROUTINE set_mass_flux_vertical (mss,mss_edge,                          &
                        sigma_dotu,         &
                        msv_f,mss_flx1_vert,mss_flx2_vert,     &
                        mss_anti_vert)
#else
   SUBROUTINE set_mass_flux_vertical (mss,mss_edge,                          &
                        sigma_dotu,sigma_dote,entrainment,cumulusmf,         &
                        msv_f,mss_flx1_vert,mss_flx2_vert,mss_flxe_vert,     &
                        mss_anti_vert)
#endif
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                                DIMENSION(iim,jjm, nlm  ,ntprog,nsdm) ::     &
      mss       !  mass defined at layer centers
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                                DIMENSION(iim,jjm, nlmp1,       nsdm) ::     &
      mss_edge  !  mass defined at layer edges
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                                DIMENSION(iim,jjm, nlmp1,       nsdm) ::     &
      sigma_dotu      ! "vertical mass velocity"
#ifndef no_pbl
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                                DIMENSION(iim,jjm, nlmp1,       nsdm) ::     &
      sigma_dote      ! "vertical mass velocity"
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm,              nsdm) ::     &
      entrainment,  &!  entrainment at the PBL top
      cumulusmf      !  cumulus mass flux at the PBL top
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(iim,jjm, nlmp1,       nsdm) ::     &
      mss_flxe_vert
#endif
!.......................................................................
!  INTENT OUT
!.......................................................................

   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(iim,jjm, nlmp1,nttend,nsdm) ::     &
      msv_f             !
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(iim,jjm, nlmp1,       nsdm) ::     &
      mss_flx1_vert,   &!
      mss_flx2_vert,   &!
      mss_anti_vert     !
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                                DIMENSION(iim,jjm, nlmp1,       nsdm) ::     &
      mss_flxe_vert
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      k,nsd
#ifndef no_pbl
   REAL (KIND=dbl_kind),        DIMENSION(iim,jjm                   ) ::     &
      sigma_dot_PBL_top     !
#endif
   REAL (KIND=dbl_kind),        DIMENSION(iim,jjm,nlmp1,nsdm) ::     &
      sigma_dot_tem     !
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! COMPUTE THE FIRST-ORDER VERTICAL MASS FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   mss_flx1_vert(:,:,:,:) = 0.0_dbl_kind
   mss_flx2_vert(:,:,:,:) = 0.0_dbl_kind
#ifdef no_pbl
   sigma_dot_tem = sigma_dotu / mss_edge
#else
   sigma_dot_tem = (sigma_dotu + sigma_dote) / mss_edge
#endif
   DO nsd = 1,nsdm
      DO k = 2,lpbl
         mss_flx1_vert(:,:,k,nsd) = area(:,:,nsd)*                           &
                   (mss(:,:,k-1,n3,nsd)*MAX (zero,sigma_dot_tem(:,:,k,nsd))+ &
                    mss(:,:,k  ,n3,nsd)*MIN (zero,sigma_dot_tem(:,:,k,nsd)))
      ENDDO
!-----------------------------------------------------------------------
!  PBL top
!-----------------------------------------------------------------------
!      sigma_dot_PBL_top(:,:) =                                               &
!                         grav*(entrainment(:,:,nsd)-cumulusmf(:,:,nsd))/     &
!                                                mss_edge(:,:,lpbl+1,nsd)
!      mss_flx1_vert(:,:,lpbl+1,nsd) = area(:,:,nsd)*                         &
!              (mss(:,:,lpbl  ,n3,nsd)*MAX (zero,sigma_dot_PBL_top(:,:))+     &
!               mss(:,:,lpbl+1,n3,nsd)*MIN (zero,sigma_dot_PBL_top(:,:)))
#ifndef no_pbl
      sigma_dot_PBL_top(:,:) =                                               &
                         grav*(entrainment(:,:,nsd)-cumulusmf(:,:,nsd))
      mss_flx1_vert(:,:,lpbl+1,nsd) = area(:,:,nsd)*sigma_dot_PBL_top(:,:)
!-----------------------------------------------------------------------
!  within PBL
!-----------------------------------------------------------------------
      DO k = lpbl+2,nlm
         mss_flx1_vert(:,:,k,nsd) = area(:,:,nsd)*                           &
                   (mss(:,:,k-1,n3,nsd)*MAX (zero,sigma_dot_tem(:,:,k,nsd))+ &
                    mss(:,:,k  ,n3,nsd)*MIN (zero,sigma_dot_tem(:,:,k,nsd)))
      ENDDO
#endif
   ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE HIGH-ORDER VERTICAL MASS FLUXES TENDENCY
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   msv_f(:,:,:,n3_f,:) = 0.0_dbl_kind
   DO nsd = 1,nsdm
      DO k = 2,lpbl
!         msv_f(:,:,k,n3_f,nsd) = area(:,:,nsd)*                              &
!                                mss_edge(:,:,k,nsd)*sigma_dot_tem(:,:,k,nsd)
         msv_f(:,:,k,n3_f,nsd) = area(:,:,nsd)*sigma_dotu(:,:,k,nsd)
!-----------------------------------------------------------------------
!  compute the high-order vertical mass fluxes
!-----------------------------------------------------------------------
#ifdef no_pbl
         mss_flx2_vert(:,:,k,nsd) = w3*msv_f(:,:,k,n3_f,nsd)+                 &
                                  w2*msv_f(:,:,k,n2_f,nsd)+                   &
                                  w1*msv_f(:,:,k,n1_f,nsd)
#else
         mss_flx2_vert(:,:,k,nsd) = w3*msv_f(:,:,k,n3_f,nsd)+                 &
                                  w2*msv_f(:,:,k,n2_f,nsd)+                   &
                                  w1*msv_f(:,:,k,n1_f,nsd)+                   &
                                  area(:,:,nsd)*sigma_dote(:,:,k,nsd)
         mss_flxe_vert(:,:,k,nsd) = area(:,:,nsd)*sigma_dote(:,:,k,nsd)
#endif
      ENDDO
#ifndef no_pbl
!-----------------------------------------------------------------------
!  PBL top
!-----------------------------------------------------------------------
      mss_flx2_vert(:,:,lpbl+1,nsd) = area(:,:,nsd)*                         &
                          grav*(entrainment(:,:,nsd)-cumulusmf(:,:,nsd))
      mss_flxe_vert(:,:,lpbl+1,nsd) = area(:,:,nsd)*                         &
                          grav*(entrainment(:,:,nsd)-cumulusmf(:,:,nsd))
!-----------------------------------------------------------------------
!  within PBL
!-----------------------------------------------------------------------
      DO k = lpbl+2,nlm
!         msv_f(:,:,k,n3_f,nsd) = area(:,:,nsd)*                              &
!                                mss_edge(:,:,k,nsd)*sigma_dot_tem(:,:,k,nsd)
         msv_f(:,:,k,n3_f,nsd) = area(:,:,nsd)*sigma_dotu(:,:,k,nsd)
!-----------------------------------------------------------------------
!  compute the high-order vertical mass fluxes
!-----------------------------------------------------------------------
         mss_flx2_vert(:,:,k,nsd) = w3*msv_f(:,:,k,n3_f,nsd)+                 &
                                  w2*msv_f(:,:,k,n2_f,nsd)+                   &
                                  w1*msv_f(:,:,k,n1_f,nsd)+                   &
                                  area(:,:,nsd)*sigma_dote(:,:,k,nsd)
         mss_flxe_vert(:,:,k,nsd) = area(:,:,nsd)*sigma_dote(:,:,k,nsd)
      ENDDO
#endif
   ENDDO
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE ANTI-DIFFUSIVE VERTICAL MASS FLUXES (PS (8))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   mss_anti_vert(:,:,:,:) = mss_flx2_vert(:,:,:,:)-mss_flx1_vert(:,:,:,:)

   END SUBROUTINE set_mass_flux_vertical
!=======================================================================
!  END set_mass_flux_vertical
!=======================================================================

!=======================================================================
!  BEGIN set_mass_low
!=======================================================================
   SUBROUTINE set_mass_low (mss,mss_flx1_horz,mss_flx1_vert,mss_low)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,ntprog,nsdm) ::      &
      mss
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,       nsdm) ::      &
      mss_flx1_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,       nsdm) ::      &
      mss_flx1_vert
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      mss_low
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
   i,j,k,nsd
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

   mss_low(:,:,:,:) = 0.0_dbl_kind

   DO nsd = 1,nsdm
      DO k = 1,nlm
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  NORTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = 1; j = jjm-1
         mss_low(i,j,k,nsd) = mss(i,j,k,n3,nsd) -                            &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((mss_flx1_vert(  i  ,j  ,k+1,nsd))-     &
                                     (mss_flx1_horz(2,i  ,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(3,i  ,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(1,i+1,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(2,i+1,j+1,k  ,nsd)+      &
                                      mss_flx1_horz(3,i  ,j+1,k  ,nsd)+      &
                                      mss_flx1_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SOUTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = iim-1; j = 1
         mss_low(i,j,k,nsd) = mss(i,j,k,n3,nsd) -                            &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((mss_flx1_vert(  i  ,j  ,k+1,nsd))-     &
                                     (mss_flx1_horz(1,i  ,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(2,i  ,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(1,i+1,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(2,i+1,j+1,k  ,nsd)+      &
                                      mss_flx1_horz(3,i  ,j+1,k  ,nsd)+      &
                                      mss_flx1_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  INTERIOR CELLS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         DO j = 2,jjm-1
            DO i = 2,iim-1
               mss_low(i,j,k,nsd) = mss(i,j,k,n3,nsd) -                      &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((mss_flx1_horz(1,i,  j  ,k  ,nsd)+      &
                                      mss_flx1_horz(2,i  ,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(3,i  ,j  ,k  ,nsd)+      &
                                      mss_flx1_vert(  i  ,j  ,k+1,nsd))-     &
                                     (mss_flx1_horz(1,i+1,j  ,k  ,nsd)+      &
                                      mss_flx1_horz(2,i+1,j+1,k  ,nsd)+      &
                                      mss_flx1_horz(3,i  ,j+1,k  ,nsd)+      &
                                      mss_flx1_vert(  i  ,j  ,k  ,nsd)))              
            ENDDO
         ENDDO
      ENDDO
   ENDDO

   END SUBROUTINE set_mass_low
!=======================================================================
!  END set_mass_low 
!=======================================================================

!=======================================================================
!  BEGIN set_beta_weights_mss 
!=======================================================================
   SUBROUTINE set_beta_weights_mss (mss_anti_horz,mss_anti_vert,mss_low,     &
                                                        beta_dn,beta_up)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::             &
      mss_anti_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,nsdm) ::             &
      mss_anti_vert
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      mss_low
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      beta_dn,    &!
      beta_up      !
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd
   REAL (KIND=dbl_kind) ::                                                   &
      a_out,a_in,mss_min(nlm),mss_max(nlm)
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind,                &!
      small_number   = 1.0E-18_dbl_kind,  &!  a small number
!      dp_PBL_min =  1000.0_dbl_kind,      &!  minimum pressure thickness [Pa]
!                                          &!  of the PBL
!      dp_PBL_max = 16000.0_dbl_kind        !  maximum pressure thickness [Pa]
!                                           !  of the PBL
      dp_PBL_min =     0.0_dbl_kind,      &!  minimum pressure thickness [Pa]
                                           !  of the PBL
      dp_PBL_max = 100000000.0_dbl_kind    !  maximum pressure thickness [Pa]
                                           !  of the PBL
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
   mss_min(:) =         0.0_dbl_kind
   mss_max(:) = 100000000.0_dbl_kind

   DO k = lpbl+1,nlm
      mss_min(k) = (dp_PBL_min/FLOAT (nlm-lpbl))/(sigl2(k+1)-sigl2(k))
      mss_max(k) = (dp_PBL_max/FLOAT (nlm-lpbl))/(sigl2(k+1)-sigl2(k))
   ENDDO

   DO nsd = 1,nsdm
      DO k = 1,nlm

         DO j = 1,jjm-1
            DO i = 1,iim-1
               a_out = (MAX (zero,mss_anti_horz(1,i  ,j  ,k  ,nsd))+         &
                        MAX (zero,mss_anti_horz(2,i  ,j  ,k  ,nsd))+         &
                        MAX (zero,mss_anti_horz(3,i  ,j  ,k  ,nsd))+         &
                        MAX (zero,mss_anti_vert(  i  ,j  ,k+1,nsd)))-        &
                       (MIN (zero,mss_anti_horz(1,i+1,j  ,k  ,nsd))+         &
                        MIN (zero,mss_anti_horz(2,i+1,j+1,k  ,nsd))+         &
                        MIN (zero,mss_anti_horz(3,i  ,j+1,k  ,nsd))+         &
                        MIN (zero,mss_anti_vert(  i  ,j  ,k  ,nsd)))

               beta_dn(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                &
                                        (mss_low(i,j,k,nsd)-mss_min(k))/     &
                                                    (a_out+small_number)

               a_in  = -((MIN (zero,mss_anti_horz(1,i  ,j  ,k  ,nsd))+       &
                          MIN (zero,mss_anti_horz(2,i  ,j  ,k  ,nsd))+       &
                          MIN (zero,mss_anti_horz(3,i  ,j  ,k  ,nsd))+       &
                          MIN (zero,mss_anti_vert(  i  ,j  ,k+1,nsd)))-      &
                         (MAX (zero,mss_anti_horz(1,i+1,j  ,k  ,nsd))+       &
                          MAX (zero,mss_anti_horz(2,i+1,j+1,k  ,nsd))+       &
                          MAX (zero,mss_anti_horz(3,i  ,j+1,k  ,nsd))+       &
                          MAX (zero,mss_anti_vert(  i  ,j  ,k  ,nsd))))
               beta_up(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                &
                                        (mss_max(k)-mss_low(i,j,k,nsd))/     &
                                                    (a_in +small_number)
            ENDDO
         ENDDO

      ENDDO
   ENDDO

   CALL wrap_bugs6 ('fct', face=beta_dn)
   CALL wrap_bugs6 ('fct', face=beta_up)

   END SUBROUTINE set_beta_weights_mss
!=======================================================================
!  END set_beta_weights_mss 
!=======================================================================

!=======================================================================
!  BEGIN set_corrected_mss_anti 
!=======================================================================
   SUBROUTINE set_corrected_mss_anti (beta_dn,beta_up,                       &
                                            mss_anti_horz,mss_anti_vert)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      beta_dn,    &!
      beta_up
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::             &
      mss_anti_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,nsdm) ::             &
      mss_anti_vert
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind,one = 1.0_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! COMPUTE THE HORIZONTAL CORRECTED MASS FLUX WEIGHTS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   DO nsd = 1,nsdm
!      DO k = 1,nlm
!         DO j = 2,jjm-1
!            DO i = 2,iim-1
!               mss_anti_horz(1,i,j,k,nsd) =                                  &
!   MIN (one,beta_dn(i  ,j  ,k,nsd))*MAX (zero,mss_anti_horz(1,i,j,k,nsd))+   &
!   MIN (one,beta_dn(i-1,j  ,k,nsd))*MIN (zero,mss_anti_horz(1,i,j,k,nsd))
!           mss_anti_horz(2,i,j,k,nsd) =                                      &
!   MIN (one,beta_dn(i  ,j  ,k,nsd))*MAX (zero,mss_anti_horz(2,i,j,k,nsd))+   &
!   MIN (one,beta_dn(i-1,j-1,k,nsd))*MIN (zero,mss_anti_horz(2,i,j,k,nsd))
!           mss_anti_horz(3,i,j,k,nsd) =                                      &
!   MIN (one,beta_dn(i  ,j  ,k,nsd))*MAX (zero,mss_anti_horz(3,i,j,k,nsd))+   &
!   MIN (one,beta_dn(i  ,j-1,k,nsd))*MIN (zero,mss_anti_horz(3,i,j,k,nsd))
!            ENDDO   
!         ENDDO
!      ENDDO
!   ENDDO

   DO nsd = 1,nsdm
      DO k = 1,nlm
         DO j = 2,jjm-1
            DO i = 2,iim-1

               mss_anti_horz(1,i,j,k,nsd) =                                  &
                   MIN (one,beta_dn(i,j,k,nsd),beta_up(i-1,j  ,k,nsd))*      &
                                  MAX (zero,mss_anti_horz(1,i,j,k,nsd))+     &
                   MIN (one,beta_up(i,j,k,nsd),beta_dn(i-1,j  ,k,nsd))*      &
                                  MIN (zero,mss_anti_horz(1,i,j,k,nsd))

               mss_anti_horz(2,i,j,k,nsd) =                                  &
                   MIN (one,beta_dn(i,j,k,nsd),beta_up(i-1,j-1,k,nsd))*      &
                                  MAX (zero,mss_anti_horz(2,i,j,k,nsd))+     &
                   MIN (one,beta_up(i,j,k,nsd),beta_dn(i-1,j-1,k,nsd))*      &
                                  MIN (zero,mss_anti_horz(2,i,j,k,nsd))

               mss_anti_horz(3,i,j,k,nsd) =                                  &
                   MIN (one,beta_dn(i,j,k,nsd),beta_up(i  ,j-1,k,nsd))*      &
                                  MAX (zero,mss_anti_horz(3,i,j,k,nsd))+     &
                   MIN (one,beta_up(i,j,k,nsd),beta_dn(i  ,j-1,k,nsd))*      &
                                  MIN (zero,mss_anti_horz(3,i,j,k,nsd))
            ENDDO   
         ENDDO
      ENDDO
   ENDDO


   CALL wrap_bugs6 ('fct', edge_scalar=mss_anti_horz)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! COMPUTE THE VERTICAL CORRECTED MASS FLUX WEIGHTS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   DO nsd = 1,nsdm
!      DO k = 2,nlm
!         DO j = 1,jjm
!            DO i = 1,iim
!               mss_anti_vert(i,j,k,nsd)=                                     &
!     MIN (one,beta_dn(i,j,k-1,nsd))*MAX (zero,mss_anti_vert(i,j,k,nsd))+     &
!     MIN (one,beta_dn(i,j,k  ,nsd))*MIN (zero,mss_anti_vert(i,j,k,nsd))
!            ENDDO   
!         ENDDO
!      ENDDO
!   ENDDO

   DO nsd = 1,nsdm
      DO k = 2,nlm
         DO j = 1,jjm
            DO i = 1,iim
               mss_anti_vert(i,j,k,nsd)=                                     &
                     MIN (one,beta_dn(i,j,k-1,nsd),beta_up(i,j,k,nsd))*      &
                                    MAX (zero,mss_anti_vert(i,j,k,nsd))+     &
                     MIN (one,beta_up(i,j,k-1,nsd),beta_dn(i,j,k,nsd))*      &
                                    MIN (zero,mss_anti_vert(i,j,k,nsd))
            ENDDO   
         ENDDO
      ENDDO
   ENDDO


   END SUBROUTINE set_corrected_mss_anti
!=======================================================================
!  END set_corrected_mss_anti 
!=======================================================================

!=======================================================================
!  BEGIN set_flux_corrected_mass
!=======================================================================
   SUBROUTINE set_flux_corrected_mass (mss_low,                              &
                                    mss_anti_horz,mss_anti_vert,mss_fct)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      mss_low
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::             &
      mss_anti_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,nsdm) ::             &
      mss_anti_vert
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),INTENT( OUT),                                        &
                        DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      mss_fct
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
   i,j,k,nsd
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

   mss_fct(:,:,:,:) = 0.0_dbl_kind

   DO nsd = 1,nsdm
      DO k = 1,nlm
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  NORTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = 1; j = jjm-1
         mss_fct(i,j,k,nsd) = mss_low(i,j,k,nsd) -                           &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((mss_anti_vert(  i  ,j  ,k+1,nsd))-     &
                                     (mss_anti_horz(2,i  ,j  ,k  ,nsd)+      &
                                      mss_anti_horz(3,i  ,j  ,k  ,nsd)+      &
                                      mss_anti_horz(1,i+1,j  ,k  ,nsd)+      &
                                      mss_anti_horz(2,i+1,j+1,k  ,nsd)+      &
                                      mss_anti_horz(3,i  ,j+1,k  ,nsd)+      &
                                      mss_anti_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SOUTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = iim-1; j = 1
         mss_fct(i,j,k,nsd) = mss_low(i,j,k,nsd) -                           &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((mss_anti_vert(  i  ,j  ,k+1,nsd))-     &
                                     (mss_anti_horz(1,i  ,j  ,k  ,nsd)+      &
                                      mss_anti_horz(2,i  ,j  ,k  ,nsd)+      &
                                      mss_anti_horz(1,i+1,j  ,k  ,nsd)+      &
                                      mss_anti_horz(2,i+1,j+1,k  ,nsd)+      &
                                      mss_anti_horz(3,i  ,j+1,k  ,nsd)+      &
                                      mss_anti_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  INTERIOR CELLS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         DO j = 2,jjm-1
            DO i = 2,iim-1
               mss_fct(i,j,k,nsd) = mss_low(i,j,k,nsd) -                     &
                                           (dt*d_volume_inv(i,j,k,nsd))*     &
                                     ((mss_anti_horz(1,i,  j  ,k  ,nsd)+     &
                                       mss_anti_horz(2,i  ,j  ,k  ,nsd)+     &
                                       mss_anti_horz(3,i  ,j  ,k  ,nsd)+     &
                                       mss_anti_vert(  i  ,j  ,k+1,nsd))-    &
                                      (mss_anti_horz(1,i+1,j  ,k  ,nsd)+     &
                                       mss_anti_horz(2,i+1,j+1,k  ,nsd)+     &
                                       mss_anti_horz(3,i  ,j+1,k  ,nsd)+     &
                                       mss_anti_vert(  i  ,j  ,k  ,nsd)))                
            ENDDO
         ENDDO
      ENDDO
   ENDDO

   END SUBROUTINE set_flux_corrected_mass
!=======================================================================
!  END set_flux_corrected_mass 
!=======================================================================

!=======================================================================
!  BEGIN set_tracer_fct
!=======================================================================
#ifdef no_pbl
   SUBROUTINE set_tracer_fct (trc,trc_high,trc_high_edge,mss,mss_low,        &
                              mss_flux_horz,mss_flux_vert,     &
                              trc_fct)
#else
   SUBROUTINE set_tracer_fct (trc,trc_high,trc_high_edge,mss,mss_low,        &
                              entrainment,cumulusmf,                         &
                              mss_flux_horz,mss_flux_vert,mss_flxe_vert,     &
                              trc_fct)
#endif
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                         DIMENSION(       iim,jjm, nlm  ,       nsdm) ::     &
      trc,          &!
      trc_high       !
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                         DIMENSION(       iim,jjm, nlm  ,ntprog,nsdm) ::     &
      mss            !
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                         DIMENSION(       iim,jjm, nlm  ,       nsdm) ::     &
      mss_low        !
#ifndef no_pbl
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm,              nsdm) ::     &
      entrainment,  &!  entrainment at the PBL top
      cumulusmf      !  cumulus mass flux at the PBL top
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                         DIMENSION(       iim,jjm, nlmp1,       nsdm) ::     &
      mss_flxe_vert 
#endif
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                         DIMENSION(nwalls,iim,jjm, nlm  ,       nsdm) ::     &
      mss_flux_horz  !  
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                         DIMENSION(       iim,jjm, nlmp1,       nsdm) ::     &
      mss_flux_vert,&!  
      trc_high_edge  !
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),INTENT(OUT),                                         &
                         DIMENSION(       iim,jjm, nlm  ,       nsdm) ::     &
      trc_fct
!.......................................................................
!  LOCAL
!.......................................................................
   REAL (KIND=dbl_kind) :: thing
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,       nsdm) ::      &
      trc_flx1_horz,   &!
      trc_flx2_horz,   &!
      trc_anti_horz     !
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,       nsdm) ::      &
      trc_flx1_vert,   &!
      trc_flx2_vert,   &!
      trc_anti_vert     !
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      msstrc_low,      &!
      msstrc_fct,      &!
      trc_min,         &!
      trc_max
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      beta_dn,         &!
      beta_up
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.   
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SET THE HORIZONTAL TRACER FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_tracer_flux_horizontal (         trc          (  :,:,:,  :),     &
                                             trc_high     (  :,:,:,  :),     &
                                             mss_flux_horz(:,:,:,:,  :),     &
                                             trc_flx1_horz(:,:,:,:,  :),     &
                                             trc_flx2_horz(:,:,:,:,  :),     &
                                             trc_anti_horz(:,:,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SET THE VERTICALTRACER FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef no_pbl
   CALL set_tracer_flux_vertical (           trc          (  :,:,:,  :),     &
                                             trc_high_edge(  :,:,:,  :),     &
                                             mss_flux_vert(  :,:,:,  :),     &
                                             trc_flx1_vert(  :,:,:,  :),     &
                                             trc_flx2_vert(  :,:,:,  :),     &
                                             trc_anti_vert(  :,:,:,  :))
#else
   CALL set_tracer_flux_vertical (           trc          (  :,:,:,  :),     &
                                             trc_high_edge(  :,:,:,  :),     &
                                             entrainment  (  :,:,    :),     &
                                             cumulusmf    (  :,:,    :),     &
                                             mss_flux_vert(  :,:,:,  :),     &
                                             mss_flxe_vert(  :,:,:,  :),     &
                                             trc_flx1_vert(  :,:,:,  :),     &
                                             trc_flx2_vert(  :,:,:,  :),     &
                                             trc_anti_vert(  :,:,:,  :))
#endif
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE LOW ORDER SOLUTION
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_msstrc_low (                     mss          (  :,:,:,:,:),     &
                                             trc          (  :,:,:,  :),     &
                                             trc_flx1_horz(:,:,:,:,  :),     &
                                             trc_flx1_vert(  :,:,:,  :),     &
                                             msstrc_low   (  :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SET LIMITER.  FIND LOCAL EXTREMA trc_min AND trc_max (PS (17ab))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_limiter (                        trc          (  :,:,:,  :),     &
                                             msstrc_low   (  :,:,:,  :),     &
                                             mss_low      (  :,:,:,  :),     &
                                             trc_min      (  :,:,:,  :),     &
                                             trc_max      (  :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SET BETA WEIGHTS -- beta_dn AND beta_up (PS (16ab))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_beta_weights_trc (               trc_anti_horz(:,:,:,:,  :),     &
                                             trc_anti_vert(  :,:,:,  :),     & 
                                             msstrc_low   (  :,:,:,  :),     &
                                             mss_low      (  :,:,:,  :),     &
                                             trc_min      (  :,:,:,  :),     &
                                             trc_max      (  :,:,:,  :),     &
                                             beta_dn      (  :,:,:,  :),     &
                                             beta_up      (  :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE CORRECTED ANTI-DIFFUSIVE TRACER FLUX (PS (15))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_corrected_trc_anti (             beta_dn      (  :,:,:,  :),     &
                                             beta_up      (  :,:,:,  :),     &
                                             trc_anti_horz(:,:,:,:,  :),     &
                                             trc_anti_vert(  :,:,:,  :))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE FLUX CORRECTED MASS SOLUTION
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL set_msstrc_fct (                     msstrc_low   (  :,:,:,  :),     &
                                             trc_anti_horz(:,:,:,:,  :),     &
                                             trc_anti_vert(  :,:,:,  :),     &
                                             msstrc_fct   (  :,:,:,  :))

   WHERE (mss(:,:,:,n4,:) /= 0.0_dbl_kind)
      trc_fct(:,:,:,:) = msstrc_fct(:,:,:,:)/(mss(:,:,:,n4,:)+0.0000000001_dbl_kind)
!      trc_fct(:,:,:,:) = msstrc_low(:,:,:,:)/(mss(:,:,:,n4,:)+0.0000000001_dbl_kind)
   ELSEWHERE
      trc_fct(:,:,:,:) = 0.0_dbl_kind
   END WHERE

   END SUBROUTINE set_tracer_fct
!=======================================================================
!  END  set_tracer_fct
!=======================================================================

!=======================================================================
!  BEGIN  set_tracer_flux_horizontal
!=======================================================================
   SUBROUTINE set_tracer_flux_horizontal (trc,trc_high,mss_flux_horz,        &
                              trc_flx1_horz,trc_flx2_horz,trc_anti_horz)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm,nsdm) ::               &
      trc,             &!
      trc_high
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm,nsdm) ::               &
      mss_flux_horz     ! 
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm,nsdm) ::               &
      trc_flx1_horz,   &!
      trc_flx2_horz,   &!
      trc_anti_horz     !
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind,half = 0.5_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE FIRST-ORDER HORIZONTAL TRACER FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   trc_flx1_horz(:,:,:,:,:) = 0.0_dbl_kind
   DO nsd = 1,nsdm
      DO k = 1,nlm
         DO j = 2,jjm-1
            DO i = 2,iim-1
               trc_flx1_horz(1,i,j,k,nsd) =                                  &
                trc(i  ,j  ,k,nsd)*MAX (zero,mss_flux_horz(1,i,j,k,nsd))+    &
                trc(i-1,j  ,k,nsd)*MIN (zero,mss_flux_horz(1,i,j,k,nsd))
               trc_flx1_horz(2,i,j,k,nsd) =                                  &
                trc(i  ,j  ,k,nsd)*MAX (zero,mss_flux_horz(2,i,j,k,nsd))+    &
                trc(i-1,j-1,k,nsd)*MIN (zero,mss_flux_horz(2,i,j,k,nsd))
               trc_flx1_horz(3,i,j,k,nsd) =                                  &
                trc(i  ,j  ,k,nsd)*MAX (zero,mss_flux_horz(3,i,j,k,nsd))+    &
                trc(i  ,j-1,k,nsd)*MIN (zero,mss_flux_horz(3,i,j,k,nsd))
            ENDDO
         ENDDO
      ENDDO
   ENDDO
   CALL wrap_bugs6 ('fct', edge_scalar=trc_flx1_horz)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE HIGHER-ORDER HORIZONTAL TRACER FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   trc_flx2_horz(:,:,:,:,:) = 0.0_dbl_kind
   DO nsd = 1,nsdm
      DO k = 1,nlm
         DO j = 2,jjm-1
            DO i = 2,iim-1
               trc_flx2_horz(1,i,j,k,nsd) =                                  &
                     half*(trc_high(i,j,k,nsd)+trc_high(i-1,j  ,k,nsd))*     &
                                               mss_flux_horz(1,i,j,k,nsd)
               trc_flx2_horz(2,i,j,k,nsd) =                                  &
                     half*(trc_high(i,j,k,nsd)+trc_high(i-1,j-1,k,nsd))*     &
                                               mss_flux_horz(2,i,j,k,nsd)
               trc_flx2_horz(3,i,j,k,nsd) =                                  &
                     half*(trc_high(i,j,k,nsd)+trc_high(i  ,j-1,k,nsd))*     &
                                               mss_flux_horz(3,i,j,k,nsd)
            ENDDO
         ENDDO
      ENDDO
   ENDDO
   CALL wrap_bugs6 ('fct', edge_scalar=trc_flx2_horz)

   trc_anti_horz(:,:,:,:,:) =                                                &
                       trc_flx2_horz(:,:,:,:,:)-trc_flx1_horz(:,:,:,:,:)

   END SUBROUTINE set_tracer_flux_horizontal
!=======================================================================
!  END  set_tracer_flux_horizontal
!=======================================================================

!=======================================================================
!  BEGIN  set_tracer_flux_vertical
!=======================================================================
#ifdef no_pbl
   SUBROUTINE set_tracer_flux_vertical (trc,trc_high_edge,                   &
                              mss_flux_vert,           &
                              trc_flx1_vert,trc_flx2_vert,trc_anti_vert)
#else
   SUBROUTINE set_tracer_flux_vertical (trc,trc_high_edge,                   &
                              entrainment,cumulusmf,mss_flux_vert,           &
                              mss_flxe_vert,                                 &
                              trc_flx1_vert,trc_flx2_vert,trc_anti_vert)
#endif
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlm  ,nsdm) ::                    &
      trc
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlmp1,nsdm) ::                    &
      trc_high_edge
#ifndef no_pbl
   REAL (KIND=dbl_kind),INTENT( IN),                                         &
                DIMENSION(                iim,jjm,              nsdm) ::     &
      entrainment,  &!  entrainment at the PBL top
      cumulusmf      !  cumulus mass flux at the PBL top

   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlmp1,nsdm) ::                    &
      mss_flxe_vert     ! 
#endif
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlmp1,nsdm) ::                    &
      mss_flux_vert     ! 
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlmp1,nsdm) ::                    &
      trc_flx1_vert,   &!
      trc_flx2_vert,   &!
      trc_anti_vert     !
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! COMPUTE THE FIRST-ORDER VERTICAL TRACER FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   trc_flx1_vert(:,:,:,:) = 0.0_dbl_kind
   DO nsd = 1,nsdm
      DO k = 2,lpbl
         trc_flx1_vert(:,:,k,nsd) =                                          &
                  (trc(:,:,k-1,nsd)*MAX (zero,mss_flux_vert(:,:,k,nsd))+     &
                   trc(:,:,k  ,nsd)*MIN (zero,mss_flux_vert(:,:,k,nsd)))
      ENDDO
#ifndef no_pbl
!-----------------------------------------------------------------------
!  PBL top
!-----------------------------------------------------------------------
      trc_flx1_vert(:,:,lpbl+1,nsd) = grav*area(:,:,nsd)*                    &
                  ((trc(:,:,lpbl  ,nsd)*MAX (zero,entrainment(:,:,nsd))+     &
                    trc(:,:,lpbl+1,nsd)*MIN (zero,entrainment(:,:,nsd)))-    &
                   (trc(:,:,lpbl+1,nsd)*MAX (zero,cumulusmf  (:,:,nsd))+     &
                    trc(:,:,lpbl  ,nsd)*MIN (zero,cumulusmf  (:,:,nsd))))

!-----------------------------------------------------------------------
!  within PBL
!-----------------------------------------------------------------------
      DO k = lpbl+2,nlm
         trc_flx1_vert(:,:,k,nsd) =                                          &
                  (trc(:,:,k-1,nsd)*MAX (zero,mss_flux_vert(:,:,k,nsd))+     &
                   trc(:,:,k  ,nsd)*MIN (zero,mss_flux_vert(:,:,k,nsd)))
      ENDDO
#endif
   ENDDO
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  COMPUTE THE HIGH-ORDER VERTICAL TRACER FLUXES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   trc_flx2_vert(:,:,:,:) = 0.0_dbl_kind
   DO nsd = 1,nsdm
      DO k = 2,lpbl
         trc_flx2_vert(:,:,k,nsd) =                                          &
                       trc_high_edge(:,:,k,nsd)*mss_flux_vert(:,:,k,nsd)
!         trc_flx2_vert(:,:,k,nsd) =                                          &
!                       trc_high_edge(:,:,k,nsd)*(mss_flux_vert(:,:,k,nsd) -  &
!                                                 mss_flxe_vert(:,:,k,nsd)) + &
!                  (trc(:,:,k-1,nsd)*MAX (zero,mss_flxe_vert(:,:,k,nsd))+     &
!                   trc(:,:,k  ,nsd)*MIN (zero,mss_flxe_vert(:,:,k,nsd)))
                                                 
      ENDDO
#ifndef no_pbl
!-----------------------------------------------------------------------
!  PBL top
!-----------------------------------------------------------------------
      trc_flx2_vert(:,:,lpbl+1,nsd) = trc_flx1_vert(:,:,lpbl+1,nsd)
!-----------------------------------------------------------------------
!  within PBL
!-----------------------------------------------------------------------
      DO k = lpbl+2,nlm
         trc_flx2_vert(:,:,k,nsd) =                                          &
                       trc_high_edge(:,:,k,nsd)*mss_flux_vert(:,:,k,nsd)
!         trc_flx2_vert(:,:,k,nsd) =                                          &
!                      trc_high_edge(:,:,k,nsd)*(mss_flux_vert(:,:,k,nsd) -  &
!                                                mss_flxe_vert(:,:,k,nsd)) + &
!                  (trc(:,:,k-1,nsd)*MAX (zero,mss_flxe_vert(:,:,k,nsd))+     &
!                   trc(:,:,k  ,nsd)*MIN (zero,mss_flxe_vert(:,:,k,nsd)))
                                                 
      ENDDO
#endif
   ENDDO

   trc_anti_vert(:,:,:,:) = trc_flx2_vert(:,:,:,:)-trc_flx1_vert(:,:,:,:)

   END SUBROUTINE set_tracer_flux_vertical
!=======================================================================
!  END set_tracer_flux_vertical
!=======================================================================

!=======================================================================
!  BEGIN set_msstrc_low
!=======================================================================
   SUBROUTINE set_msstrc_low (mss,trc,trc_flx1_horz,trc_flx1_vert,           &
                                                             msstrc_low)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,ntprog,nsdm) ::      &
      mss
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      trc
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,       nsdm) ::      &
      trc_flx1_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,       nsdm) ::      &
      trc_flx1_vert
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      msstrc_low
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
   i,j,k,nsd
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

   msstrc_low(:,:,:,:) = 0.0_dbl_kind

   DO nsd = 1,nsdm
      DO k = 1,nlm
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  NORTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = 1; j = jjm-1
         msstrc_low(i,j,k,nsd) = mss(i,j,k,n3,nsd)*trc(i,j,k,nsd) -          &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((trc_flx1_vert(  i  ,j  ,k+1,nsd))-     &
                                     (trc_flx1_horz(2,i  ,j  ,k  ,nsd)+      &
                                      trc_flx1_horz(3,i  ,j  ,k  ,nsd)+      &
                                      trc_flx1_horz(1,i+1,j  ,k  ,nsd)+      &
                                      trc_flx1_horz(2,i+1,j+1,k  ,nsd)+      &
                                      trc_flx1_horz(3,i  ,j+1,k  ,nsd)+      &
                                      trc_flx1_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SOUTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = iim-1; j = 1
         msstrc_low(i,j,k,nsd) = mss(i,j,k,n3,nsd)*trc(i,j,k,nsd) -          &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((trc_flx1_vert(  i  ,j  ,k+1,nsd))-     &
                                     (trc_flx1_horz(1,i  ,j  ,k  ,nsd)+      &
                                      trc_flx1_horz(2,i  ,j  ,k  ,nsd)+      &
                                      trc_flx1_horz(1,i+1,j  ,k  ,nsd)+      &
                                      trc_flx1_horz(2,i+1,j+1,k  ,nsd)+      &
                                      trc_flx1_horz(3,i  ,j+1,k  ,nsd)+      &
                                      trc_flx1_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  INTERIOR CELLS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         DO j = 2,jjm-1
            DO i = 2,iim-1 
               msstrc_low(i,j,k,nsd) = mss(i,j,k,n3,nsd)*trc(i,j,k,nsd) -    &
                                            (dt*d_volume_inv(i,j,k,nsd))*    &
                                     ((trc_flx1_horz(1,i,  j  ,k  ,nsd)+     &
                                       trc_flx1_horz(2,i  ,j  ,k  ,nsd)+     &
                                       trc_flx1_horz(3,i  ,j  ,k  ,nsd)+     &
                                       trc_flx1_vert(  i  ,j  ,k+1,nsd))-    &
                                      (trc_flx1_horz(1,i+1,j  ,k  ,nsd)+     &
                                       trc_flx1_horz(2,i+1,j+1,k  ,nsd)+     &
                                       trc_flx1_horz(3,i  ,j+1,k  ,nsd)+     &
                                       trc_flx1_vert(  i  ,j  ,k  ,nsd)))              
            ENDDO
         ENDDO
      ENDDO
   ENDDO

   END SUBROUTINE set_msstrc_low
!=======================================================================
!  END set_msstrc_low 
!=======================================================================

!=======================================================================
!  BEGIN set_limiter 
!=======================================================================
   SUBROUTINE set_limiter (trc,msstrc_low,mss_low,trc_min,trc_max)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlm,nsdm) ::                      &
      trc,         &!
      msstrc_low,  &!
      mss_low       !
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlm,nsdm) ::                      &
      trc_min,     &!
      trc_max       !
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd,p,q
   REAL (KIND=dbl_kind),DIMENSION(iim,jjm, nlm,nsdm) ::                      &
      trc_low
   REAL (KIND=dbl_kind) ::                                                   &
      temp12(12),temp14(14)
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
   p = iim-1; q = jjm-1
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  FIND THE STANDARD LIMITER (see Smolarkiewicz,  (17ab))
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   CALL wrap_bugs6 ('fct', face=   trc    )
   CALL wrap_bugs6 ('fct', face=   mss_low)
   CALL wrap_bugs6 ('fct', face=msstrc_low)

   trc_low(:,:,:,:) = msstrc_low(:,:,:,:)/mss_low(:,:,:,:)

   trc_min(:,:,:,:) = 0.0_dbl_kind; trc_max(:,:,:,:) = 0.0_dbl_kind
!-----------------------------------------------------------------------
!  find local minimum and maximum in the horizontal direction
!-----------------------------------------------------------------------
   DO nsd = 1,nsdm
      DO k = 1,nlm

!  north pole
         temp12(:) = (/ trc(1,q,k,nsd),trc_low(1,q,k,nsd),                   &
                            trc    (p,  1,k,nsd),trc    (1,q-1,k,nsd),       &
         trc    (2,q,k,nsd),trc    (2,q+1,k,nsd),trc    (1,q+1,k,nsd),       &
                            trc_low(p,  1,k,nsd),trc_low(1,q-1,k,nsd),       &
         trc_low(2,q,k,nsd),trc_low(2,q+1,k,nsd),trc_low(1,q+1,k,nsd) /) 

         trc_min(1,q,k,nsd) = MINVAL (temp12)
         trc_max(1,q,k,nsd) = MAXVAL (temp12)

!  south pole
         temp12(:) = (/ trc(p,1,k,nsd),trc_low(p,1,k,nsd),                   &
           trc    (p-1,1,k,nsd),trc    (  1,q,k,nsd),                        &
           trc    (p+1,1,k,nsd),trc    (p+1,2,k,nsd),trc    (p,2,k,nsd),     &
           trc_low(p-1,1,k,nsd),trc_low(  1,q,k,nsd),                        &
           trc_low(p+1,1,k,nsd),trc_low(p+1,2,k,nsd),trc_low(p,2,k,nsd) /)

         trc_min(p,1,k,nsd) = MINVAL (temp12)
         trc_max(p,1,k,nsd) = MAXVAL (temp12)

!  interior cells
         DO j = 2,jjm-1
            DO i = 2,iim-1

               temp14(:) = (/ trc(i,j,k,nsd),trc_low(i,j,k,nsd),             &
       trc    (i-1,j,k,nsd),trc    (i-1,j-1,k,nsd),trc    (i,j-1,k,nsd),     &
       trc    (i+1,j,k,nsd),trc    (i+1,j+1,k,nsd),trc    (i,j+1,k,nsd),     &
       trc_low(i-1,j,k,nsd),trc_low(i-1,j-1,k,nsd),trc_low(i,j-1,k,nsd),     &
       trc_low(i+1,j,k,nsd),trc_low(i+1,j+1,k,nsd),trc_low(i,j+1,k,nsd) /)

               trc_min(i,j,k,nsd) = MINVAL  (temp14)
               trc_max(i,j,k,nsd) = MAXVAL  (temp14)

            ENDDO
         ENDDO
      ENDDO
   ENDDO
!-----------------------------------------------------------------------
!  find local minimum and maximum in the vertical direction
!-----------------------------------------------------------------------
   trc_min(:,:,   1,:) = MIN (trc_min(:,:,     1,:),trc_min(:,:,    2,:))
   trc_max(:,:,   1,:) = MAX (trc_max(:,:,     1,:),trc_max(:,:,    2,:))
   DO k = 2,nlm-1
      trc_min(:,:,k,:) =                                                     &
            MIN (trc_min(:,:,k-1,:),trc_min(:,:,k,:),trc_min(:,:,k+1,:))
      trc_max(:,:,k,:) =                                                     &
            MAX (trc_max(:,:,k-1,:),trc_max(:,:,k,:),trc_max(:,:,k+1,:))
   ENDDO
   trc_min(:,:, nlm,:) = MIN (trc_min(:,:,nlm-1,:),trc_min(:,:, nlm,:))
   trc_max(:,:, nlm,:) = MAX (trc_max(:,:,nlm-1,:),trc_max(:,:, nlm,:))

   END SUBROUTINE set_limiter
!=======================================================================
!  END set_limiter 
!=======================================================================

!=======================================================================
!  BEGIN set_beta_weights_trc 
!=======================================================================
   SUBROUTINE set_beta_weights_trc (trc_anti_horz,trc_anti_vert,             &
                     msstrc_low,mss_low,trc_min,trc_max,beta_dn,beta_up)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::             &
      trc_anti_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,nsdm) ::             &
      trc_anti_vert
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      msstrc_low,  &!
      mss_low,     &!
      trc_min,     &!
      trc_max
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      beta_dn,     &!
      beta_up
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd
   REAL (KIND=dbl_kind) ::                                                   &
      a_out,a_in
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind,small_number = 1.0E-18_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.
   DO nsd = 1,nsdm
      DO k = 1,nlm
!-----------------------------------------------------------------------
!  north pole
!-----------------------------------------------------------------------
         i = 1; j = jjm-1
         a_out =  ((MAX (zero,trc_anti_vert(  i  ,j  ,k+1,nsd)))-            &
                   (MIN (zero,trc_anti_horz(2,i  ,j  ,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(3,i  ,j  ,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(1,i+1,j  ,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(2,i+1,j+1,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(3,i  ,j+1,k  ,nsd))+             &
                    MIN (zero,trc_anti_vert(  i  ,j  ,k  ,nsd))))

         a_in  = -((MIN (zero,trc_anti_vert(  i  ,j  ,k+1,nsd)))-            &
                   (MAX (zero,trc_anti_horz(2,i  ,j  ,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(3,i  ,j  ,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(1,i+1,j  ,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(2,i+1,j+1,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(3,i  ,j+1,k  ,nsd))+             &
                    MAX (zero,trc_anti_vert(  i  ,j  ,k  ,nsd))))

         beta_dn(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                      &
          (msstrc_low(i,j,k,nsd)-mss_low(i,j,k,nsd)*trc_min(i,j,k,nsd))/     &
                                                    (a_out+small_number)
         beta_up(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                      &
          (mss_low(i,j,k,nsd)*trc_max(i,j,k,nsd)-msstrc_low(i,j,k,nsd))/     &
                                                    (a_in +small_number)
!-----------------------------------------------------------------------
!  south pole
!-----------------------------------------------------------------------
         i = iim-1; j = 1
         a_out =  ((MAX (zero,trc_anti_vert(  i  ,j  ,k+1,nsd)))-            &
                   (MIN (zero,trc_anti_horz(1,i  ,j  ,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(2,i  ,j  ,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(1,i+1,j  ,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(2,i+1,j+1,k  ,nsd))+             &
                    MIN (zero,trc_anti_horz(3,i  ,j+1,k  ,nsd))+             &
                    MIN (zero,trc_anti_vert(  i  ,j  ,k  ,nsd))))

         a_in  = -((MIN (zero,trc_anti_vert(  i  ,j  ,k+1,nsd)))-            &
                   (MAX (zero,trc_anti_horz(1,i  ,j  ,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(2,i  ,j  ,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(1,i+1,j  ,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(2,i+1,j+1,k  ,nsd))+             &
                    MAX (zero,trc_anti_horz(3,i  ,j+1,k  ,nsd))+             &
                    MAX (zero,trc_anti_vert(  i  ,j  ,k  ,nsd))))

         beta_dn(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                      &
          (msstrc_low(i,j,k,nsd)-mss_low(i,j,k,nsd)*trc_min(i,j,k,nsd))/     &
                                                    (a_out+small_number)
         beta_up(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                      &
          (mss_low(i,j,k,nsd)*trc_max(i,j,k,nsd)-msstrc_low(i,j,k,nsd))/     &
                                                    (a_in +small_number)
!-----------------------------------------------------------------------
!  interior cells
!-----------------------------------------------------------------------
         DO j = 2,jjm-1
            DO i = 2,iim-1
               a_out =  ((MAX (zero,trc_anti_horz(1,i  ,j  ,k  ,nsd))+       &
                          MAX (zero,trc_anti_horz(2,i  ,j  ,k  ,nsd))+       &
                          MAX (zero,trc_anti_horz(3,i  ,j  ,k  ,nsd))+       &
                          MAX (zero,trc_anti_vert(  i  ,j  ,k+1,nsd)))-      &
                         (MIN (zero,trc_anti_horz(1,i+1,j  ,k  ,nsd))+       &
                          MIN (zero,trc_anti_horz(2,i+1,j+1,k  ,nsd))+       &
                          MIN (zero,trc_anti_horz(3,i  ,j+1,k  ,nsd))+       &
                          MIN (zero,trc_anti_vert(  i  ,j  ,k  ,nsd))))

               a_in  = -((MIN (zero,trc_anti_horz(1,i  ,j  ,k  ,nsd))+       &
                          MIN (zero,trc_anti_horz(2,i  ,j  ,k  ,nsd))+       &
                          MIN (zero,trc_anti_horz(3,i  ,j  ,k  ,nsd))+       &
                          MIN (zero,trc_anti_vert(  i  ,j  ,k+1,nsd)))-      &
                         (MAX (zero,trc_anti_horz(1,i+1,j  ,k  ,nsd))+       &
                          MAX (zero,trc_anti_horz(2,i+1,j+1,k  ,nsd))+       &
                          MAX (zero,trc_anti_horz(3,i  ,j+1,k  ,nsd))+       &
                          MAX (zero,trc_anti_vert(  i  ,j  ,k  ,nsd))))

               beta_dn(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                &
          (msstrc_low(i,j,k,nsd)-mss_low(i,j,k,nsd)*trc_min(i,j,k,nsd))/     &
                                                    (a_out+small_number)

               beta_up(i,j,k,nsd) = (d_volume(i,j,k,nsd)/dt)*                &
          (mss_low(i,j,k,nsd)*trc_max(i,j,k,nsd)-msstrc_low(i,j,k,nsd))/     &
                                                    (a_in +small_number)

            ENDDO
         ENDDO
      ENDDO
   ENDDO

   CALL wrap_bugs6 ('fct', face=beta_dn)
   CALL wrap_bugs6 ('fct', face=beta_up)

   END SUBROUTINE set_beta_weights_trc
!=======================================================================
!  END  set_beta_weights_trc 
!=======================================================================

!=======================================================================
!  BEGIN set_corrected_trc_anti 
!=======================================================================
   SUBROUTINE set_corrected_trc_anti (beta_dn,beta_up,                       &
                                            trc_anti_horz,trc_anti_vert)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,nsdm) ::             &
      beta_dn,  &!
      beta_up
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::             &
      trc_anti_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,nsdm) ::             &
      trc_anti_vert
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
      i,j,k,nsd
   REAL (KIND=dbl_kind),PARAMETER ::                                         &
      zero = 0.0_dbl_kind,one = 1.0_dbl_kind
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! COMPUTE THE HORIZONTAL CORRECTED ANTI-DIFFUSIVE TRACER  
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   DO nsd = 1,nsdm
      DO k = 1,nlm
         DO j = 2,jjm-1
            DO i = 2,iim-1

               trc_anti_horz(1,i,j,k,nsd) =                                  &
                   MIN (one,beta_dn(i,j,k,nsd),beta_up(i-1,j  ,k,nsd))*      &
                                  MAX (zero,trc_anti_horz(1,i,j,k,nsd))+     &
                   MIN (one,beta_up(i,j,k,nsd),beta_dn(i-1,j  ,k,nsd))*      &
                                  MIN (zero,trc_anti_horz(1,i,j,k,nsd))

               trc_anti_horz(2,i,j,k,nsd) =                                  &
                   MIN (one,beta_dn(i,j,k,nsd),beta_up(i-1,j-1,k,nsd))*      &
                                  MAX (zero,trc_anti_horz(2,i,j,k,nsd))+     &
                   MIN (one,beta_up(i,j,k,nsd),beta_dn(i-1,j-1,k,nsd))*      &
                                  MIN (zero,trc_anti_horz(2,i,j,k,nsd))

               trc_anti_horz(3,i,j,k,nsd) =                                  &
                   MIN (one,beta_dn(i,j,k,nsd),beta_up(i  ,j-1,k,nsd))*      &
                                  MAX (zero,trc_anti_horz(3,i,j,k,nsd))+     &
                   MIN (one,beta_up(i,j,k,nsd),beta_dn(i  ,j-1,k,nsd))*      &
                                  MIN (zero,trc_anti_horz(3,i,j,k,nsd))
            ENDDO   
         ENDDO
      ENDDO
   ENDDO

   CALL wrap_bugs6 ('fct', edge_scalar=trc_anti_horz)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! COMPUTE THE VERTICAL CORRECTED ANTI-DIFFUSIVE TRACER
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   DO nsd = 1,nsdm
      DO k = 2,nlm
         DO j = 1,jjm
            DO i = 1,iim
               trc_anti_vert(i,j,k,nsd)=                                     &
                     MIN (one,beta_dn(i,j,k-1,nsd),beta_up(i,j,k,nsd))*      &
                                    MAX (zero,trc_anti_vert(i,j,k,nsd))+     &
                     MIN (one,beta_up(i,j,k-1,nsd),beta_dn(i,j,k,nsd))*      &
                                    MIN (zero,trc_anti_vert(i,j,k,nsd))
            ENDDO   
         ENDDO
      ENDDO
   ENDDO

   END SUBROUTINE set_corrected_trc_anti
!=======================================================================
!  END set_corrected_trc_anti 
!=======================================================================

!=======================================================================
!  BEGIN set_msstrc_fct
!=======================================================================
   SUBROUTINE set_msstrc_fct (msstrc_low,trc_anti_horz,trc_anti_vert,        &
                                                             msstrc_fct)
!.......................................................................
!  INTENT IN
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      msstrc_low
   REAL (KIND=dbl_kind),DIMENSION(nwalls,iim,jjm, nlm  ,       nsdm) ::      &
      trc_anti_horz
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlmp1,       nsdm) ::      &
      trc_anti_vert
!.......................................................................
!  INTENT OUT
!.......................................................................
   REAL (KIND=dbl_kind),DIMENSION(       iim,jjm, nlm  ,       nsdm) ::      &
      msstrc_fct
!.......................................................................
!  LOCAL
!.......................................................................
   INTEGER (KIND=int_kind) ::                                                &
   i,j,k,nsd
!.:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:. .:.

   msstrc_fct(:,:,:,:) = 0.0_dbl_kind

   DO nsd = 1,nsdm
      DO k = 1,nlm
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  NORTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = 1; j = jjm-1
         msstrc_fct(i,j,k,nsd) =  msstrc_low(i,j,k,nsd) -                    &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((trc_anti_vert(  i  ,j  ,k+1,nsd))-     &
                                     (trc_anti_horz(2,i  ,j  ,k  ,nsd)+      &
                                      trc_anti_horz(3,i  ,j  ,k  ,nsd)+      &
                                      trc_anti_horz(1,i+1,j  ,k  ,nsd)+      &
                                      trc_anti_horz(2,i+1,j+1,k  ,nsd)+      &
                                      trc_anti_horz(3,i  ,j+1,k  ,nsd)+      &
                                      trc_anti_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  SOUTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         i = iim-1; j = 1
         msstrc_fct(i,j,k,nsd) =  msstrc_low(i,j,k,nsd) -                    &
                                          (dt*d_volume_inv(i,j,k,nsd))*      &
                                    ((trc_anti_vert(  i  ,j  ,k+1,nsd))-     &
                                     (trc_anti_horz(1,i  ,j  ,k  ,nsd)+      &
                                      trc_anti_horz(2,i  ,j  ,k  ,nsd)+      &
                                      trc_anti_horz(1,i+1,j  ,k  ,nsd)+      &
                                      trc_anti_horz(2,i+1,j+1,k  ,nsd)+      &
                                      trc_anti_horz(3,i  ,j+1,k  ,nsd)+      &
                                      trc_anti_vert(  i  ,j  ,k  ,nsd)))              
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  INTERIOR CELLS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

         DO j = 2,jjm-1
            DO i = 2,iim-1
               msstrc_fct(i,j,k,nsd) = msstrc_low(i,j,k,nsd) -               &
                                           (dt*d_volume_inv(i,j,k,nsd))*     &
                                     ((trc_anti_horz(1,i,  j  ,k  ,nsd)+     &
                                       trc_anti_horz(2,i  ,j  ,k  ,nsd)+     &
                                       trc_anti_horz(3,i  ,j  ,k  ,nsd)+     &
                                       trc_anti_vert(  i  ,j  ,k+1,nsd))-    &
                                      (trc_anti_horz(1,i+1,j  ,k  ,nsd)+     &
                                       trc_anti_horz(2,i+1,j+1,k  ,nsd)+     &
                                       trc_anti_horz(3,i  ,j+1,k  ,nsd)+     &
                                       trc_anti_vert(  i  ,j  ,k  ,nsd)))              
            ENDDO
         ENDDO
      ENDDO
   ENDDO

   END SUBROUTINE set_msstrc_fct
!=======================================================================
!  END set_msstrc_fct 
!=======================================================================

!======================================================================
! BEGIN CENTERED_MASS_FLUX
!======================================================================
        subroutine centered_mass_flux(                                &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                ncorners,                                             &
                nedges,                                               &
                nspace,                                               &
                r3,                                                   &
                dc,                                                   &
                normal,                                               &
                mass_c,                                               &
                u,                                                    &
                mass,                                                 &
                flux)

!-----------------------------------------------------------------------
! PURPOSE : compute mass flux based on the mass_c values
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad            : width of 'ghost cell' region
!       nx              : first dimension of xin (integer)
!       ny              : second dimension of xin (integer)
!       ndata           : amount of data involved in the averaging operator
!       ncorners        : number of corners
!       mass_c          : thickness fields averaged to cell corners (must be wrapped)
!       mass            : mass field defined at cell center
!       flux            : mass flux
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                          &
                        npad, nx, ny, ndata, ncorners, r3, nspace, nedges

        real (kind=dbl_kind), intent(in) ::                             &
                        mass_c     (             ncorners,nx,ny),       &
                        u          (      nspace,ncorners,nx,ny),       &
                        dc         (ndata,       ncorners,nx,ny),       &
                        normal     (ndata,r3    ,ncorners,nx,ny),       &
                        mass                             (nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                           &
                        flux                      (nedges,nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j,icorner,ispace,ii,jj,idata
        real (kind=dbl_kind) :: r(2,nedges)

!----------------------------------------------------------------------------
! INITIALIZE
!----------------------------------------------------------------------------
        flux = 0.0_dbl_kind

!----------------------------------------------------------------------------
! compute mass flux based on mass_c
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad

!----------------------------------------------------------------------------
! fill the edge flux by combining the two half edges
!----------------------------------------------------------------------------
          r(1,1) = ( u(1,2,i  ,j+1)*normal(2,1,2,i  ,j+1) + u(2,2,i  ,j+1)*normal(2,2,2,i  ,j+1) ) * mass_c(2,i  ,j+1)
          r(1,1) = r(1,1) *             dc(2,  2,i  ,j+1)
          r(2,1) = ( u(1,1,i  ,j  )*normal(1,1,1,i  ,j  ) + u(2,1,i  ,j  )*normal(1,2,1,i  ,j  ) ) * mass_c(1,i  ,j  )
          r(2,1) = r(2,1) *             dc(1,  1,i  ,j  )
          r(1,2) = ( u(1,1,i  ,j  )*normal(3,1,1,i  ,j  ) + u(2,1,i  ,j  )*normal(3,2,1,i  ,j  ) ) * mass_c(1,i  ,j  )
          r(1,2) = r(1,2) *             dc(3,  1,i  ,j  )
          r(2,2) = ( u(1,2,i  ,j  )*normal(1,1,2,i  ,j  ) + u(2,2,i  ,j  )*normal(1,2,2,i  ,j  ) ) * mass_c(2,i  ,j  )
          r(2,2) = r(2,2) *             dc(1,  2,i  ,j  )
          r(1,3) = ( u(1,2,i  ,j  )*normal(3,1,2,i  ,j  ) + u(2,2,i  ,j  )*normal(3,2,2,i  ,j  ) ) * mass_c(2,i  ,j  )
          r(1,3) = r(1,3) *             dc(3,  2,i  ,j  )
          r(2,3) = ( u(1,1,i+1,j  )*normal(2,1,1,i+1,j  ) + u(2,1,i+1,j  )*normal(2,2,1,i+1,j  ) ) * mass_c(1,i+1,j  )
          r(2,3) = r(2,3) *             dc(2,  1,i+1,j  )
          
!----------------------------------------------------------------------------
! fill in the flux
!----------------------------------------------------------------------------
          flux(1,i,j) = -r(1,1)+r(2,1)
          flux(2,i,j) = -r(1,2)+r(2,2)
          flux(3,i,j) = -r(1,3)+r(2,3)

        enddo
        enddo

        end subroutine centered_mass_flux
!======================================================================
! END CENTERED_MASS_FLUX
!======================================================================

!======================================================================
! BEGIN UPSTREAM_MASS_FLUX
!======================================================================
        subroutine upstream_mass_flux(                                &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                ncorners,                                             &
                nedges,                                               &
                nspace,                                               &
                r3,                                                   &
                dc,                                                   &
                normal,                                               &
                u,                                                    &
                mass,                                                 &
                flux)

!-----------------------------------------------------------------------
! PURPOSE : compute fluxes based on the upstream value of mass
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad            : width of 'ghost cell' region
!       nx              : first dimension of xin (integer)
!       ny              : second dimension of xin (integer)
!       ndata           : amount of data involved in the averaging operator
!       ncorners        : number of corners
!       mass_c          : thickness fields averaged to cell corners (must be wrapped)
!       mass            : scalar field defined at cell center
!       flux            : upstream mass flux 
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                          &
                        npad, nx, ny, ndata, ncorners, r3, nspace, nedges

        real (kind=dbl_kind), intent(in) ::                             &
                        u          (      nspace,ncorners,nx,ny),       &
                        dc         (ndata,       ncorners,nx,ny),       &
                        normal     (ndata,r3    ,ncorners,nx,ny),       &
                        mass                              (nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                           &
                        flux                      (nedges,nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j,icorner,ispace,ii,jj,idata
        real (kind=dbl_kind) :: r(2,nedges)

!----------------------------------------------------------------------------
! INITIALIZE
!----------------------------------------------------------------------------
        flux = 0.0_dbl_kind

!----------------------------------------------------------------------------
! compute upstream flux
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad

!----------------------------------------------------------------------------
! fill the edge flux by combining the two half edges
!----------------------------------------------------------------------------
          r(1,1) = ( u(1,2,i  ,j+1)*normal(2,1,2,i  ,j+1) + u(2,2,i  ,j+1)*normal(2,2,2,i  ,j+1) ) 
          r(1,1) = r(1,1) *             dc(2,  2,i  ,j+1)
          r(2,1) = ( u(1,1,i  ,j  )*normal(1,1,1,i  ,j  ) + u(2,1,i  ,j  )*normal(1,2,1,i  ,j  ) ) 
          r(2,1) = r(2,1) *             dc(1,  1,i  ,j  )
          r(1,2) = ( u(1,1,i  ,j  )*normal(3,1,1,i  ,j  ) + u(2,1,i  ,j  )*normal(3,2,1,i  ,j  ) ) 
          r(1,2) = r(1,2) *             dc(3,  1,i  ,j  )
          r(2,2) = ( u(1,2,i  ,j  )*normal(1,1,2,i  ,j  ) + u(2,2,i  ,j  )*normal(1,2,2,i  ,j  ) )
          r(2,2) = r(2,2) *             dc(1,  2,i  ,j  )
          r(1,3) = ( u(1,2,i  ,j  )*normal(3,1,2,i  ,j  ) + u(2,2,i  ,j  )*normal(3,2,2,i  ,j  ) )
          r(1,3) = r(1,3) *             dc(3,  2,i  ,j  )
          r(2,3) = ( u(1,1,i+1,j  )*normal(2,1,1,i+1,j  ) + u(2,1,i+1,j  )*normal(2,2,1,i+1,j  ) ) 
          r(2,3) = r(2,3) *             dc(2,  1,i+1,j  )

          r(1,1) = r(2,1)-r(1,1)
          r(2,1) = 0.0
          r(1,2) = r(2,2)-r(1,2)
          r(2,2) = 0.0
          r(1,3) = r(2,3)-r(1,3)
          r(2,3) = 0.0
          
!----------------------------------------------------------------------------
! based on the value of r, fill in the flux
!----------------------------------------------------------------------------
          flux(1,i,j) = maxval(r(:,1))*mass(i,j) + minval(r(:,1))*mass(i-1,j  )
          flux(2,i,j) = maxval(r(:,2))*mass(i,j) + minval(r(:,2))*mass(i-1,j-1)
          flux(3,i,j) = maxval(r(:,3))*mass(i,j) + minval(r(:,3))*mass(i  ,j-1)

        enddo
        enddo

        end subroutine upstream_mass_flux
!======================================================================
! END UPSTREAM_MASS_FLUX
!======================================================================
#endif

   END MODULE flux_corrected_transport
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
