      MODULE gather_scatter

!-----------------------------------------------------------------------
! PURPOSE: contains subroutines which transform data from one data
!       structure to another
!
! the "global grid" is (im,jm), the grid that each processors owns is
!       (iim,jjm,nsdm). gather goes from (iim,jjm,nsdm) to (im,jm),
!       scatter is the reverse. scatter and gather have both mpi and
!       serial versions.
!
! reduce_to_vector_global goes from (im,jm) to (max_ig), i.e. it takes
!       all of the "true" cells on the (im,jm) grid and lists them in
!       a vector format of length (max_ig). scatter_to_grid_global is
!       the reverse. both of these subroutines are serial.
!-----------------------------------------------------------------------

      USE kinds
      USE model_parameters
      USE domain_decomposition
#ifndef no_pbl
      USE domain_decomposition_sfc
#endif
      USE assign_owner
      USE wrap_face_data

      IMPLICIT NONE
      SAVE
      CONTAINS



      subroutine scatter_fine(x_glbl,x_sctr)
! input = x_glbl on global (im,jm) grid
! output = x_sctr on scattered (iim,jjm,nsdm) grid
      REAL (KIND=dbl_kind) :: x_glbl(im,jm),
     &                        x_sctr(iim,jjm,nsdm)

      call scatter(max_level,im,jm,iim,jjm,1,nsdm,x_glbl,x_sctr)

      end subroutine scatter_fine


      subroutine gather_fine(x_sctr,x_glbl)
! input = x_sctr on scattered (iim,jjm,nsdm) grid
! output = x_glbl on global (im,jm) grid
      REAL (KIND=dbl_kind) :: x_glbl(im,jm),
     &                        x_sctr(iim,jjm,nsdm),
     &                        x_wrap(iim,jjm,1,nsdm)	

      x_wrap(:,:,1,:) = x_sctr(:,:,:)
      call wrap(max_level,iim,jjm,1,nsdm,x_wrap(:,:,:,:))
      call gather(max_level,im,jm,iim,jjm,1,nsdm,x_wrap,x_glbl)

      end subroutine gather_fine


#ifndef no_pbl

      subroutine scatter_fine_sfc(x_glbl,x_sctr)
! input = x_glbl on global (im,jm) grid
! output = x_sctr on scattered (iim,jjm,nsdm) grid
      REAL (KIND=dbl_kind) :: x_glbl(im_sfc,jm_sfc),
     &                        x_sctr(iim_sfc,jjm_sfc,nsdm_sfc)

      call scatter(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,1,nsdm_sfc,x_glbl,x_sctr)

      end subroutine scatter_fine_sfc



      subroutine gather_fine_sfc(x_sctr,x_glbl)
! input = x_sctr on scattered (iim,jjm,nsdm) grid
! output = x_glbl on global (im,jm) grid
      REAL (KIND=dbl_kind) :: x_glbl(im_sfc,jm_sfc),
     &                        x_sctr(iim_sfc,jjm_sfc,nsdm_sfc),
     &                        x_wrap(iim_sfc,jjm_sfc,1,nsdm_sfc)	

      x_wrap(:,:,1,:) = x_sctr(:,:,:)
      call wrap(sfc_level,iim_sfc,jjm_sfc,1,nsdm_sfc,x_wrap(:,:,:,:))
      call gather(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,1,nsdm_sfc,
     *            x_wrap,x_glbl)

      end subroutine gather_fine_sfc
#endif


!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE scatter (level,im_t,jm_t,iim_t,jjm_t,nvar,nsdm_t,x1,x2)
! 25 SEPTEMBER 2000
      IMPLICIT NONE
# ifdef mpi
      INCLUDE 'mpif.h'
# endif
      INTEGER (KIND=int_kind) :: level,im_t,jm_t,iim_t,jjm_t,nvar,nsdm_t
      INTEGER (KIND=int_kind) :: i,j,panel,isd,jsd,off,
     &                           msg_size,send_total,sbdmn_number,nsd,
     &                           count_send,msg_dest,msg_tag,ierr
# ifdef mpi
      INTEGER (KIND=int_kind), ALLOCATABLE :: recv_req(:),send_req(:),
     &                                 recv_status(:,:),send_status(:,:)
# endif

      REAL (KIND=dbl_kind) :: x1(im_t,jm_t,nvar),
     &                        x2(iim_t,jjm_t,nvar,nsdm_t)
# ifdef mpi
      REAL (KIND=dbl_kind), ALLOCATABLE :: send_buffer(:,:,:,:),
     &                                     recv_buffer(:,:,:,:)
# endif


      msg_size = iim_t*jjm_t*nvar
     
      IF (my_task.EQ.0) THEN
# ifdef mpi
         send_total = 5*isdm_level(level)*jsdm_level(level)-nsdm_t
         ALLOCATE (send_req(send_total))
         send_req(:) = -999
         ALLOCATE (send_status(MPI_STATUS_SIZE,send_total))
         send_status(:,:) = -999
         ALLOCATE (send_buffer(iim_t,jjm_t,nvar,send_total))
         send_buffer(:,:,:,:) = 0.0
# endif
         sbdmn_number = 0
         nsd = 0
         count_send = 0
         DO panel = 0,4
            off = (im_t/5)*panel
            DO jsd = 0,jsdm_level(level)-1
               DO isd = 0,isdm_level(level)-1
                  sbdmn_number = sbdmn_number + 1
                  IF (sbdmn_owner(level)%owner(sbdmn_number).EQ.0) THEN
                     nsd = nsd + 1
                     DO j = 1,jjm_t
                        DO i = 1,iim_t
                           x2(i,j,:,nsd) = 
     &                         x1(off+(iim_t-2)*isd+i,(jjm_t-2)*jsd+j,:)
                        ENDDO
                     ENDDO
                  ELSE
# ifdef mpi
                     count_send = count_send + 1
                     DO j = 1,jjm_t
                        DO i = 1,iim_t
                           send_buffer(i,j,:,count_send) = 
     &                         x1(off+(iim_t-2)*isd+i,(jjm_t-2)*jsd+j,:)
                        ENDDO
                     ENDDO
                     msg_dest = sbdmn_owner(level)%owner(sbdmn_number)
                     CALL MPI_ISEND (send_buffer(1,1,1,count_send),
     &                       msg_size,
     &                       mpi_float_type,msg_dest,sbdmn_number,
     &                       MPI_COMM_ATMOS,send_req(count_send),ierr)
# endif
                  ENDIF
               ENDDO
            ENDDO
         ENDDO
# ifdef mpi
         CALL MPI_WAITALL (send_total,send_req,send_status,ierr)

         DEALLOCATE (send_req,send_buffer,send_status)
# endif
      ENDIF

# ifdef mpi
      IF (my_task.NE.0) THEN
! POST RECEIVES
         ALLOCATE (recv_req(nsdm_t))
         recv_req(:) = -999
         ALLOCATE (recv_status(MPI_STATUS_SIZE,nsdm_t))
         recv_status(:,:) = -999
         ALLOCATE (recv_buffer(iim_t,jjm_t,nvar,nsdm_t))
         recv_buffer(:,:,:,:) = 0.0

         DO nsd = 1,nsdm_t
            msg_tag = sbdmn_assign(my_task+1,level)%sd(nsd)
            CALL MPI_IRECV (recv_buffer(1,1,1,nsd),msg_size,
     &                      mpi_float_type,0,msg_tag,
     &                      MPI_COMM_ATMOS,recv_req(nsd),ierr)
         ENDDO

         CALL MPI_WAITALL (nsdm_t,recv_req,recv_status,ierr)

! UNPACK BUFFERS
         DO nsd = 1,nsdm_t
            DO j = 1,jjm_t
               DO i = 1,iim_t
                  x2(i,j,:,nsd) = recv_buffer(i,j,:,nsd)
               ENDDO
            ENDDO
         ENDDO

         DEALLOCATE (recv_req,recv_buffer,recv_status)
      ENDIF

      CALL MPI_BARRIER (MPI_COMM_ATMOS,ierr)
# endif


      END SUBROUTINE scatter
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE gather (level,im_t,jm_t,iim_t,jjm_t,nvar,nsdm_t,x2,x1)
! 26 SEPTEMBER 2000
      IMPLICIT NONE
# ifdef mpi
      INCLUDE 'mpif.h'
# endif

      INTEGER (KIND=int_kind) :: level,im_t,jm_t,iim_t,jjm_t,nvar,nsdm_t
      INTEGER (KIND=int_kind) :: i,j,panel,isd,jsd,off,
     &                           msg_size,recv_total,sbdmn_number,nsd,
     &                           count_recv,msg_src,msg_tag,ierr
# ifdef mpi
      INTEGER (KIND=int_kind), ALLOCATABLE :: recv_req(:),send_req(:),
     &                                 recv_status(:,:),send_status(:,:)
# endif

      REAL (KIND=dbl_kind) :: x2(iim_t,jjm_t,nvar,nsdm_t),
     &                        x1(im_t,jm_t,nvar)
# ifdef mpi
      REAL (KIND=dbl_kind), ALLOCATABLE :: recv_buffer(:,:,:,:)
# endif

      msg_size = iim_t*jjm_t*nvar

# ifdef mpi
      IF (my_task.NE.0) THEN
         ALLOCATE (send_req(nsdm_t))
         send_req(:) = -999
         ALLOCATE (send_status(MPI_STATUS_SIZE,nsdm_t))
         send_status(:,:) = -999
         DO nsd = 1,nsdm_t
            msg_tag = sbdmn_assign(my_task+1,level)%sd(nsd)
            CALL MPI_ISEND (x2(1,1,1,nsd),msg_size,
     &                      mpi_float_type,0,msg_tag,
     &                      MPI_COMM_ATMOS,send_req(nsd),ierr)
         ENDDO
         CALL MPI_WAITALL (nsdm_t,send_req,send_status,ierr)

         DEALLOCATE (send_req,send_status)
      ENDIF
#endif

      IF (my_task.EQ.0) THEN
# ifdef mpi
         recv_total = 5*isdm_level(level)*jsdm_level(level)-nsdm_t
         ALLOCATE (recv_req(recv_total))
         recv_req(:) = -999
         ALLOCATE (recv_status(MPI_STATUS_SIZE,recv_total))
         recv_status(:,:) = -999
         ALLOCATE (recv_buffer(iim_t,jjm_t,nvar,recv_total))
         recv_buffer(:,:,:,:) = 0.0
# endif
         sbdmn_number = 0
         nsd = 0
         count_recv = 0

         DO panel = 0,4
            off = (im_t/5)*panel
            DO jsd = 0,jsdm_level(level)-1
               DO isd = 0,isdm_level(level)-1
                  sbdmn_number = sbdmn_number + 1
                  IF (sbdmn_owner(level)%owner(sbdmn_number).EQ.0) THEN
                     nsd = nsd + 1
                     DO j = 1,jjm_t
                        DO i = 1,iim_t
                           x1(off+(iim_t-2)*isd+i,(jjm_t-2)*jsd+j,:) =
     &                                                     x2(i,j,:,nsd)
                        ENDDO
                     ENDDO
                  ELSE
# ifdef mpi
                     count_recv = count_recv + 1
                     msg_src = sbdmn_owner(level)%owner(sbdmn_number)
                     CALL MPI_IRECV (recv_buffer(1,1,1,count_recv),
     &                        msg_size,
     &                        mpi_float_type,msg_src,sbdmn_number,
     &                        MPI_COMM_ATMOS,recv_req(count_recv),ierr)
# endif
                  ENDIF
               ENDDO
            ENDDO
         ENDDO

# ifdef mpi
         CALL MPI_WAITALL (recv_total,recv_req,recv_status,ierr)

         sbdmn_number = 0
         count_recv = 0

         DO panel = 0,4
            off = (im_t/5)*panel
            DO jsd = 0,jsdm_level(level)-1
               DO isd = 0,isdm_level(level)-1
                  sbdmn_number = sbdmn_number + 1
                  IF (sbdmn_owner(level)%owner(sbdmn_number).NE.0) THEN
                     count_recv = count_recv + 1
                     DO j = 1,jjm_t
                        DO i = 1,iim_t
                           x1(off+(iim_t-2)*isd+i,(jjm_t-2)*jsd+j,:) =
     &                                     recv_buffer(i,j,:,count_recv)
                        ENDDO
                     ENDDO
                  ENDIF
               ENDDO
            ENDDO
         ENDDO 

         DEALLOCATE (recv_req,recv_buffer,recv_status)
# endif
      ENDIF

# ifdef mpi
      CALL MPI_BARRIER (MPI_COMM_ATMOS,ierr)
# endif

      END SUBROUTINE gather




!======================================================================
! START OF REDUCE_TO_VECTOR_GLOBAL
!======================================================================
      SUBROUTINE reduce_to_vector_global(xin,xout)

      IMPLICIT NONE
      integer (kind = int_kind) :: i,ii,j,panel,icount
      real (kind = dbl_kind) :: xin(im,jm),xout(max_ig)

      xout(1)=xin(1,jm-1)
      xout(2)=xin(im-1,1)
      icount=2
      DO j = 2,jm-1
         DO panel = 0,4
            DO i = 2,im5-1
               icount=icount+1
               ii = im5*panel+i
               xout(icount)=xin(ii,j)
            ENDDO
         ENDDO
      ENDDO

      end subroutine reduce_to_vector_global
!======================================================================
! END OF REDUCE_TO_VECTOR_GLOBAL
!======================================================================

#ifndef no_pbl

!======================================================================
! START OF REDUCE_TO_VECTOR_GLOBAL_SFC
!======================================================================
      SUBROUTINE reduce_to_vector_global_sfc(xin,xout)

      IMPLICIT NONE
      integer (kind = int_kind) :: i,ii,j,panel,icount
      real (kind = dbl_kind) :: xin(im_sfc,jm_sfc),xout(max_ig_sfc)

      xout(1)=xin(1,jm_sfc-1)
      xout(2)=xin(im_sfc-1,1)
      icount=2
      DO j = 2,jm_sfc-1
         DO panel = 0,4
            DO i = 2,im5_sfc-1
               icount=icount+1
               ii = im5_sfc*panel+i
               xout(icount)=xin(ii,j)
            ENDDO
         ENDDO
      ENDDO

      end subroutine reduce_to_vector_global_sfc
!======================================================================
! END OF REDUCE_TO_VECTOR_GLOBAL_SFC
!======================================================================
#endif

!======================================================================
! START OF SCATTER_TO_GRID_GLOBAL
!======================================================================
      SUBROUTINE scatter_to_grid_global(xin,xout)

      IMPLICIT NONE
      integer (kind = int_kind) :: i,ii,j,panel,icount
      real (kind = dbl_kind), dimension(max_ig), intent(in) ::  xin
      real (kind = dbl_kind), dimension(im,jm), intent(out) ::  xout

      xout = 0.0
      xout(1,jm-1) = xin(1)
      xout(im-1,1) = xin(2)
      icount=2
      DO j = 2,jm-1
         DO panel = 0,4
            DO i = 2,im5-1
               icount=icount+1
               ii = im5*panel+i
               xout(ii,j)=xin(icount)
            ENDDO
         ENDDO
      ENDDO

      end subroutine scatter_to_grid_global
!======================================================================
! END OF SCATTER_TO_GRID_GLOBAL
!=====================================================================

#ifndef no_pbl
!======================================================================
! START OF SCATTER_TO_GRID_GLOBAL_SFC
!======================================================================
      SUBROUTINE scatter_to_grid_global_sfc(xin,xout)

      IMPLICIT NONE
      integer (kind = int_kind) :: i,ii,j,panel,icount
      real (kind = dbl_kind), dimension(max_ig_sfc), intent(in) ::  xin
      real (kind = dbl_kind), dimension(im_sfc,jm_sfc), intent(out) ::  xout

      xout = 0.0
      xout(1,jm_sfc-1) = xin(1)
      xout(im_sfc-1,1) = xin(2)
      icount=2
      DO j = 2,jm_sfc-1
         DO panel = 0,4
            DO i = 2,im5_sfc-1
               icount=icount+1
               ii = im5_sfc*panel+i
               xout(ii,j)=xin(icount)
            ENDDO
         ENDDO
      ENDDO

      end subroutine scatter_to_grid_global_sfc
!======================================================================
! END OF SCATTER_TO_GRID_GLOBAL_SFC
!=====================================================================
#endif

!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      END MODULE gather_scatter
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
