!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module grid_metrics

!-----------------------------------------------------------------------
! PURPOSE : find the weights required to average data from cell centers to
!   cell corners (s_weights), to compute gradients of scalar fields (v_weights),
!   and to compute the Laplacian of a scalar field (l_weights)
!
! NOTE : before generate_grid_metrics can be called the following routine must
!   be called :
!	specify_mpi_communicator
!	specify_mpi_precision
!	set_assign_owner
!	initialize_wrap
!	set_grid_center_mask
!	set_grid_corner_mask
!
! NOTE : the X1 coordinate system is global w/ origin at center of earth
!        the X2 coordinate system is local with the coordinate directions
!          pointing the local east, north, and vertical directions
!
! NOTE : public variables are commented when they are initialized
!	 see generate_grid_metrics
!-----------------------------------------------------------------------

	use kinds
	use model_parameters
	use global_reductions
	use physical_parameters
	use grid_mask
	use geodesic_grid_utilities
	use gather_scatter
	use grid_metrics_generator

	implicit none

	real (kind=dbl_kind) ::                                             &
      		p              (           r3,           iim, jjm, nsdm),   &
      		c              (           r3, ncorners, iim, jjm, nsdm),   &
                w              (ndata,     r3, ncorners, iim, jjm, nsdm),   &
      		area           (                         iim, jjm, nsdm),   &
      		area_inv       (                         iim, jjm, nsdm),   &
                corner_area    (               ncorners, iim, jjm, nsdm),   &
      		corner_area_inv(               ncorners, iim, jjm, nsdm),   &
                sub_area       (ndata,         ncorners, iim, jjm, nsdm),   &
                sub_area_inv   (ndata,         ncorners, iim, jjm, nsdm),   &
                dc             (ndata,         ncorners, iim, jjm, nsdm),   &
      		normal         (ndata,     r3, ncorners, iim, jjm, nsdm),   &
      		tangent        (ndata,     r3, ncorners, iim, jjm, nsdm)

        real (kind=dbl_kind) ::                                             &
                v_weights      (ndata, nspace, ncorners, iim, jjm, nsdm),   &
      		s_weights      (ndata,         ncorners, iim, jjm, nsdm),   &
      	        l_weights      (nweights,                iim, jjm, nsdm)

	real (kind=dbl_kind) ::                                             &
                trans_to_X2 (r3, r3, ncorners, iim, jjm, nsdm),             &
                trans_to_X1 (r3, r3, ncorners, iim, jjm, nsdm)

        real (kind=dbl_kind) ::                                             &
                grid_center_lon    (           iim, jjm, nsdm),             &
                grid_center_lat    (           iim, jjm, nsdm),             &
                grid_corner_lon    ( ncorners, iim, jjm, nsdm),             &
                grid_corner_lat    ( ncorners, iim, jjm, nsdm)              

	real (kind=dbl_kind) :: tot_area, inv_tot_area

	public ::  generate_grid_metrics

!*************************************************************************

        contains

!*************************************************************************

!======================================================================
! BEGIN generate_grid_metrics
!======================================================================
      subroutine generate_grid_metrics ()
      implicit none
      integer (kind=int_kind) ::                                             &
         ispace
      real (kind=dbl_kind) ::                                                &
         p_list(r3,max_ig),                                                  &
         p_tmp(       r3, im    ,jm),                                        &
         work (          iim    ,jjm, 1, nsdm)

!-----------------------------------------------------------------------
! read in grid center data
!	the (x,y,z) point is read in
!-----------------------------------------------------------------------
	if(my_task == 0) then
	   p_list = 0.0_dbl_kind
           CALL read_grid_center_fine (p_list)
         endif
!-----------------------------------------------------------------------
! scatter the xyz grid center locations to (im,jm) data structure and wrap
! p_tmp(r3,im,jm) holds the (x,y,z) location of each grid cell center
! measured in the X1 coordinate system
!-----------------------------------------------------------------------
	if(my_task == 0) then
	  p_tmp = 0.0_dbl_kind
          do ispace=1,3
            call scatter_to_grid_global(p_list(ispace,:),p_tmp(ispace,:,:))
          enddo
	endif

!-----------------------------------------------------------------------
! scatter p_tmp across all processes
!-----------------------------------------------------------------------
	do ispace = 1,3
	  call scatter_fine(p_tmp(ispace,:,:), p(ispace,:,:,:))
	enddo

!-----------------------------------------------------------------------
! wrap p
!-----------------------------------------------------------------------
	do ispace = 1,3
	  work(:,:,1,:) = p(ispace,:,:,:)
	  call wrap(max_level, iim, jjm, 1, nsdm, work(:,:,:,:))
	  p(ispace,:,:,:) = work(:,:,1,:)
	enddo

      CALL generate_grid_metrics_level (                                     &
                            r2,r3,ndata,nspace,ncorners,nweights,            &
                            max_level,im,jm,max_ig,iim,jjm,nsdm,             &
                            p,c,w,                                           &
                                   area,       area_inv,                     &
                            corner_area,corner_area_inv,                     &
                               sub_area,   sub_area_inv,                     &
                            dc,normal,tangent,                               &
                            v_weights,s_weights,l_weights,                   &
                            trans_to_X2,trans_to_X1,                         &
                            grid_center_lon,grid_center_lat,                 &
                            grid_corner_lon,grid_corner_lat,                 &
                            tot_area,inv_tot_area)

!-----------------------------------------------------------------------
! wrap c
!-----------------------------------------------------------------------
	call wrap_vertices(r3,1,c(1,1,1,1,1))


      end subroutine generate_grid_metrics
!======================================================================
! END generate_grid_metrics
!======================================================================

        end module grid_metrics
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
