!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module grid_metrics_generator

!-----------------------------------------------------------------------
! PURPOSE : find the weights required to average data from cell centers to
!   cell corners (s_weights), to compute gradients of scalar fields (v_weights),
!   and to compute the Laplacian of a scalar field (l_weights)
!
! NOTE : before generate_grid_metrics can be called the following routine must
!   be called :
!	specify_mpi_communicator
!	specify_mpi_precision
!	set_assign_owner
!	initialize_wrap
!
! NOTE : the X1 coordinate system is global w/ origin at center of earth
!        the X2 coordinate system is local with the coordinate directions
!          pointing the local east, north, and vertical directions
!
! NOTE : public variables are commented when they are initialized
!	 see generate_grid_metrics
!-----------------------------------------------------------------------

	use kinds
 	use global_reductions
	use physical_parameters
	use geodesic_grid_utilities
	use gather_scatter

	implicit none

	logical, parameter :: l_sphere = .false.

	private :: compute_scalar_weights,                                  &
                   compute_gradient_weights,                                &
                   compute_laplacian_weights,                               &
      		   grid_center_area

	public ::  generate_grid_metrics_level,                             &
                   read_grid_center_fine,                                   &
                   compute_polygon_type


!*************************************************************************

        contains

!*************************************************************************

!======================================================================
! BEGIN generate_grid_metrics_level
!======================================================================
      subroutine generate_grid_metrics_level (                               &
                            r2,r3,ndata,nspace,ncorners,nweights,            &
                            level,im,jm,max_ig,iim,jjm,nsdm,                 &
                            p,c,w,                                           &
                                   area,       area_inv,                     &
                            corner_area,corner_area_inv,                     &
                               sub_area,   sub_area_inv,                     &
                            dc,normal,tangent,                               &
                            v_weights,s_weights,l_weights,                   &
                            trans_to_X2,trans_to_X1,                         &
                            grid_center_lon,grid_center_lat,                 &
                            grid_corner_lon,grid_corner_lat,                 &
                            tot_area,inv_tot_area)
      implicit none

      integer (kind=int_kind), intent(in) ::                                 &
         r2,r3,ndata,nspace,ncorners,nweights,                               &
         level,im,jm,max_ig,iim,jjm,nsdm

      integer (kind=int_kind) ::                                             & 
         i,j,k,l,                                                            &
         icorner,ispace,ksdm

      real (kind=dbl_kind),intent( in) ::                                    &
         p              (           r3,           iim, jjm, nsdm)

      real (kind=dbl_kind),intent(out) ::                                    &
         c              (           r3, ncorners, iim, jjm, nsdm),           &
         w              (ndata,     r3, ncorners, iim, jjm, nsdm),           &
      	 area           (                         iim, jjm, nsdm),           &
      	 area_inv       (                         iim, jjm, nsdm),           &
         corner_area    (               ncorners, iim, jjm, nsdm),           &
      	 corner_area_inv(               ncorners, iim, jjm, nsdm),           &
         sub_area       (ndata,         ncorners, iim, jjm, nsdm),           &
         sub_area_inv   (ndata,         ncorners, iim, jjm, nsdm),           &
         dc             (ndata,         ncorners, iim, jjm, nsdm),           &
      	 normal         (ndata,     r3, ncorners, iim, jjm, nsdm),           &
      	 tangent        (ndata,     r3, ncorners, iim, jjm, nsdm),           &
         v_weights      (ndata, nspace, ncorners, iim, jjm, nsdm),           &
      	 s_weights      (ndata,         ncorners, iim, jjm, nsdm),           &
      	 l_weights      (nweights,                iim, jjm, nsdm),           &
         trans_to_X2    (       r3, r3, ncorners, iim, jjm, nsdm),           &
         trans_to_X1    (       r3, r3, ncorners, iim, jjm, nsdm),           &
         grid_center_lon(                         iim, jjm, nsdm),           &
         grid_center_lat(                         iim, jjm, nsdm),           &
         grid_corner_lon(               ncorners, iim, jjm, nsdm),           &
         grid_corner_lat(               ncorners, iim, jjm, nsdm),           &  
         tot_area,inv_tot_area

      real (kind=dbl_kind),allocatable ::                                    &
         X1(:,:),X2(:,:),                                                    &
         polygon_type(:,:,:),metrics_mask(:,:,:,:),                          &
         grid_center_mask(:,:,:),                                            &
         grid_corner_mask(:,:,:,:),                                          &
         t(:,:),r(:,:),h(:,:),g(:),m(:)

      real (kind=dbl_kind) ::                                                &
         f1,f2,s(2), tem_c(r3)

      allocate (                                                             &
         X1(r3,r3),                                                          &
         X2(r3,r3),                                                          &
         polygon_type(iim,jjm,nsdm),                                         &
         grid_center_mask(iim,jjm,nsdm),                                     &
         grid_corner_mask(ncorners,iim,jjm,nsdm),                            &
         metrics_mask(ncorners,iim,jjm,nsdm),                                &
         t(ndata,r3),r(ndata,r3),h(ndata,r3),                                &
         g(r3),m(ndata))

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! initialize all work arrays
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	 X1 = 0.0_dbl_kind
	 X2 = 0.0_dbl_kind
	 polygon_type = 0.0_dbl_kind
         grid_center_mask = 0.0_dbl_kind
         grid_corner_mask = 0.0_dbl_kind
	 metrics_mask = 0.0_dbl_kind
	 h  = 0.0_dbl_kind
	 r  = 0.0_dbl_kind
	 t  = 0.0_dbl_kind
	 g  = 0.0_dbl_kind
	 m  = 0.0_dbl_kind

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! define the polygon type, define grid_center_mask and grid_corner_mask
!   (the mask definitions are local)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      call compute_polygon_type(my_task,level,im,jm,iim,jjm,nsdm,            &
                                                           polygon_type) 
      where(polygon_type.gt.0.0_dbl_kind) grid_center_mask = 1.0_dbl_kind
      grid_corner_mask(:,2:iim-1,2:jjm-1,:) = 1.0_dbl_kind

!-----------------------------------------------------------------------
! set the metrics mask
!   assume all data in (2:iim,2:jjm) exist
!   delete corner 1 of midlatitude pentagon neighbor up and right
!   delete corner 2 at (iim,2) if (iim-1,1,1) is not the SP
!   delete corner 1 at (2,jjm) if (1,jjm-1,1) is not the NP
!   add in both corners for North Pole and South Pole
!-----------------------------------------------------------------------
	metrics_mask(:,2:iim,2:jjm,:) = 1.0_dbl_kind
	metrics_mask(2,iim,2,:) = 0.0_dbl_kind
	metrics_mask(1,2,jjm,:) = 0.0_dbl_kind
	do ksdm=1,nsdm
	do j=1,jjm
	do i=1,iim
	if(polygon_type(i,j,ksdm) == 2.0_dbl_kind) then
	  metrics_mask(1,i+1,j+1,ksdm) = 0.0_dbl_kind
	endif
        if(polygon_type(i,j,ksdm) == 3.0_dbl_kind) then
          metrics_mask(:,i,j,ksdm) = 1.0_dbl_kind
          metrics_mask(1,i+1,j+1,ksdm) = 1.0_dbl_kind
        endif
        if(polygon_type(i,j,ksdm) == 4.0_dbl_kind) then
          metrics_mask(:,i,j,ksdm) = 1.0_dbl_kind
          metrics_mask(2,i+1,j+1,ksdm) = 1.0_dbl_kind
        endif
	enddo
	enddo
	enddo

!----------------------------------------------------------------------------
! keep the user informed
!----------------------------------------------------------------------------
       if(my_task.eq.0) then
          write(6,*)
          write(6,*) '-------------------------------------------------------'
          write(6,*) ' generating operator weights in generate_grid_metrics'
          write(6,*) ' for resolution : ',max_ig
        endif

!-----------------------------------------------------------------------
! initialize
!-----------------------------------------------------------------------
	c               =  0.0_dbl_kind		! (x,y,z) of grid corners (m)
	w               =  0.0_dbl_kind		! (x,y,z) of grid wall midpoint (m)
	area            =  0.0_dbl_kind		! area of grid cells (m^2)
	area_inv        =  0.0_dbl_kind		! 1/area (m^-2)
	corner_area     =  0.0_dbl_kind		! area associated with grid corners (m^2)
	corner_area_inv =  0.0_dbl_kind		! 1/corner_area (m^-2)
	sub_area        =  0.0_dbl_kind		! corner_area separated into 3 parts (m^2)
        sub_area_inv    =  0.0_dbl_kind         ! inverse of sub_area
	dc              =  0.0_dbl_kind		! length of wall segments (m)
	normal          =  0.0_dbl_kind		! unit vectors normal to cell wall segments 
	tangent         =  0.0_dbl_kind		! unit vectors parallel to cell wall segments
	s_weights       =  0.0_dbl_kind		! weights for averaging scalars to corners
	v_weights       =  0.0_dbl_kind		! weights for computing div,curl,grad
	l_weights       =  0.0_dbl_kind		! weights for computing Laplacian at grid centers
	trans_to_X1     =  0.0_dbl_kind		! (3,3) matrix to transform into X1
	trans_to_X2     =  0.0_dbl_kind		! (3,3) matrix to transform into X2
	grid_center_lon =  0.0_dbl_kind         ! longitude (radians) of grid center
	grid_center_lat =  0.0_dbl_kind         ! latitude (radians) of grid center
	grid_corner_lon =  0.0_dbl_kind         ! longitude (radians) of grid corner
	grid_corner_lat =  0.0_dbl_kind         ! latitude (radians) of grid corner

!-----------------------------------------------------------------------
! the X1 coordinate system has its origin defined at the center of the 
! sphere. the e1, e2, and e3 directions do not vary in space, i.e. the 
! X1 coordinate system is global
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! define the e1 direction
!-----------------------------------------------------------------------
	X1(1,1) = 1.0_dbl_kind
	X1(1,2) = 0.0_dbl_kind
	X1(1,3) = 0.0_dbl_kind

!-----------------------------------------------------------------------
! define the e2 direction
!-----------------------------------------------------------------------
	X1(2,1) = 0.0_dbl_kind
	X1(2,2) = 1.0_dbl_kind
	X1(2,3) = 0.0_dbl_kind

!-----------------------------------------------------------------------
! define the e3 direction
!-----------------------------------------------------------------------
	X1(3,1) = 0.0_dbl_kind
	X1(3,2) = 0.0_dbl_kind
	X1(3,3) = 1.0_dbl_kind
	
!-----------------------------------------------------------------------
! loop over every corner and do the following operations
!   1) first, place the surrounding 3 cell centers into a temporary array (t)
!   2) compute the Voronoi corner for the set of 3 points held in array t
!   3) find the (x,y,z) location of the cell wall center (w)
!   4) find the local east, north, and vertical directions (measured in X1)
!   5) do a coordinate transformation into the X2 coordinate system
!		X2 is a local coordinate system which has unit vectors
!		pointing in the east, north, and vertical directions
!   6) find the vectors normal and along the cell walls (measured in X2)
!   4) compute the spherical sub_area
!   5) add these areas together to compute the corner area
!-----------------------------------------------------------------------
	do ksdm = 1,nsdm
	do j=1,jjm
	  do i=1,iim
	    do icorner = 1,ncorners
	      if(metrics_mask(icorner,i,j,ksdm) == 1.0_dbl_kind) then

!-----------------------------------------------------------------------
! get surrounding locations in xyz (measured in X1)
!-----------------------------------------------------------------------
	        do ispace = 1,r3
	          call get_surrounding_centers(      &
                           i,                        &
                           j,                        &
                           icorner,                  &
                           ndata,                    &
      			   iim,                      &
                           jjm,                      &
                           polygon_type(i,j,ksdm),   &
                           p(ispace,:,:,ksdm),       &
                           m(:) )
	          t(:,ispace) = m(:)
	        enddo

!-----------------------------------------------------------------------
! compute the Voronoi corner (measured in X1)
!   if l_sphere = .true., the voronoi corner is normalized to unit sphere
!   if l_sphere = .false., the voronoi corner lies on the plane fitting t
!-----------------------------------------------------------------------
	      call compute_voronoi_corner(          &
                           l_sphere,                &
                           t(1,:),                  &
                           t(2,:),                  &
                           t(3,:),                  &
                           c(:,icorner,i,j,ksdm) )

!-----------------------------------------------------------------------
! find the vectors (measured in X1) that point in the local
!   east (h(1,:)), north (h(2,:)), and vertical (h(3,:)) direction
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! the vector pointing from the origin of X1 (center of earth) to the
! location of the cell corner defines the local vertical direction.
! measured in X1
!-----------------------------------------------------------------------
              h(3,:) = c(:,icorner,i,j,ksdm)
	      call unit_vector_in_3space(h(3,:))

!-----------------------------------------------------------------------
! g(:) is a work array and holds the vector pointing to the North Pole.
! measured in X1
!-----------------------------------------------------------------------
              g(:) = X1(3,:) 

!-----------------------------------------------------------------------
! find the vector that is perpendicular to the local vertical vector
! and points in the direction of of the North pole, this defines the local
! north direction. measured in X1
!-----------------------------------------------------------------------
              call vector_on_tangent_plane ( h(3,:), g(:), h(2,:) )

!-----------------------------------------------------------------------
! take the cross product of the local North direction and the local vertical
! to find the local east vector. still in X1
!-----------------------------------------------------------------------
              call cross_product_in_3space ( h(2,:), h(3,:), h(1,:) )

!-----------------------------------------------------------------------
! put these 3 vectors into a matrix X2
!-----------------------------------------------------------------------
              X2(1,:) = h(1,:)              ! local east     (measured in X1)
              X2(2,:) = h(2,:)              ! local north    (measured in X1)
              X2(3,:) = h(3,:)              ! local vertical (measured in X1)

!-----------------------------------------------------------------------
! compute the transformation matrices
!    trans_to_X2 transforms vector components measured in X1 into X2
!    trans_to_X1 transforms vector components measured in X2 into X1
!-----------------------------------------------------------------------
              trans_to_X2(:,:,icorner,i,j,ksdm) = matmul(X2,transpose(X1))
	      trans_to_X1(:,:,icorner,i,j,ksdm) = matmul(X1,transpose(X2))

!-----------------------------------------------------------------------
! loop over ndata to find the cell wall center, normal vector, and tangent vector
!-----------------------------------------------------------------------
	      do k=1,ndata
	        l = k + 1
	        if(l == ndata+1) l=1

!-----------------------------------------------------------------------
! find the location of the cell wall center. measured in X1
!   if l_sphere, then w will lie on the unit sphere
!   if not l_sphere, the w lies along the line connecting t(k,:) and t(l,:)
!-----------------------------------------------------------------------
	          r(k,:) =  p5 * ( t(k,:) + t(l,:) )
                  if(l_sphere) then
	            call unit_vector_in_3space(r(k,:))
	          endif
                  w(k,:,icorner,i,j,ksdm) = r(k,:)

!-----------------------------------------------------------------------
! find the tangent vector (points from the cell wall center to the cell
! corner. this is measured in X1. the output of vector_on_tangent_plane
! is a vector pointing toward the cell wall center, flip it around so
! that is points to the cell corner
!-----------------------------------------------------------------------
                  if(l_sphere) then
 	            call vector_on_tangent_plane ( h(3,:), r(k,:), g(:) )
                    g(:) = -g(:)
                  else
                    g(:) = c(:,icorner,i,j,ksdm) - w(k,:,icorner,i,j,ksdm)
                    call unit_vector_in_3space(g(:))
                  endif

!-----------------------------------------------------------------------
! transform this vector to the X2 coordinate system
!-----------------------------------------------------------------------
 	          tangent(k,:,icorner,i,j,ksdm) =     &
      		      matmul(trans_to_X2(:,:,icorner,i,j,ksdm),g)
                  call unit_vector_in_3space(tangent(k,:,icorner,i,j,ksdm))

!-----------------------------------------------------------------------
! transform the local vertical into X2
! this is mainly a sanity check, g(1,2,3) should have values 
! of (0,0,1), respectively
!-----------------------------------------------------------------------
 	          g(:) = matmul(trans_to_X2(:,:,icorner,i,j,ksdm),h(3,:))

!-----------------------------------------------------------------------
! take the cross product of the tangent vector and the vertical to
! find the normal vector (measured in X2)
!-----------------------------------------------------------------------
 	          call cross_product_in_3space (        &
      	   	        tangent(k,:,icorner,i,j,ksdm),  &
      		        g(:),                           &
      		        normal(k,:,icorner,i,j,ksdm))
                  call unit_vector_in_3space(normal(k,:,icorner,i,j,ksdm))

	         enddo	! do k=1,ndata

!-----------------------------------------------------------------------
! loop over ndata again, this time to find the sub_areas and 
!    distance between corners and cell wall centers
!    note that the "t" and "r" arrays are re-used from above
!-----------------------------------------------------------------------
!sphere_check (call linear surface counterpart)
	        do k=1,ndata
	          l = k - 1
	          if(l == 0) l=ndata

                  if(l_sphere) then
	            call spherical_area (                     &
      	  	        t(k,:),                               &
      		        r(k,:),                               &
      		        c(:,icorner,i,j,ksdm),                &
      		        f1)

	            call spherical_area (                     &
      		        t(k,:),                               &
      		        c(:,icorner,i,j,ksdm),                &
      		        r(l,:),                               &
      		        f2)

	            sub_area(k,icorner,i,j,ksdm) = f1 + f2
                    sub_area_inv(k,icorner,i,j,ksdm) =        &
                            c1/sub_area(k,icorner,i,j,ksdm)

	            call distance_along_unit_sphere(          &
      		        r(k,:),                               &
      		        c(:,icorner,i,j,ksdm),                &
      		        dc(k,icorner,i,j,ksdm) )

                  else

                    call compute_triangle_area (              &
                        t(k,:),                               &
                        r(k,:),                               &
                        c(:,icorner,i,j,ksdm),                &
                        f1)

                    call compute_triangle_area (              &
                        t(k,:),                               &
                        c(:,icorner,i,j,ksdm),                &
                        r(l,:),                               &
                        f2)

                    sub_area(k,icorner,i,j,ksdm) = f1 + f2
                    sub_area_inv(k,icorner,i,j,ksdm) =        &
                            c1/sub_area(k,icorner,i,j,ksdm)

                    call distance_along_a_line(               &
                        r(k,:),                               &
                        c(:,icorner,i,j,ksdm),                &
                        dc(k,icorner,i,j,ksdm) )
        
                  endif

	        enddo

!-----------------------------------------------------------------------
! sum the 3 sub_areas to compute the total corner area
!-----------------------------------------------------------------------
 	        corner_area(icorner,i,j,ksdm) =                         &
                             sum(sub_area(:,icorner,i,j,ksdm))

	        corner_area_inv(icorner,i,j,ksdm) =                     &
                             1.0_dbl_kind/corner_area(icorner,i,j,ksdm)

	      endif		! if(metrics_mask == 1.0)
	    enddo		! do icorner
	  enddo		! do i
	enddo		! do j
	enddo		! do ksdm

!-----------------------------------------------------------------------
! sum the sub_areas to find the area associated with of the grid cells centers
!-----------------------------------------------------------------------
	call grid_center_area (polygon_type,ndata,ncorners,iim,jjm,nsdm,     &
      		sub_area,area_inv,area)

!-----------------------------------------------------------------------
! convert metrics into dimensions of Earth
!-----------------------------------------------------------------------
 	dc = dc * a
 	sub_area = sub_area * a * a
	sub_area_inv = sub_area_inv / (a * a)
 	corner_area = corner_area * a * a
 	corner_area_inv = corner_area_inv / ( a * a)
 	area = area * a * a
 	area_inv = area_inv / (a * a)

!-----------------------------------------------------------------------
! compute total area
!-----------------------------------------------------------------------
	tot_area = global_real_sum(area(1,1,1),grid_center_mask(1,1,1), &
      			iim*jjm*nsdm)

	inv_tot_area = c1/tot_area
	write(6,*) ' total area at this resolution : ',tot_area

!-----------------------------------------------------------------------
! compute weights to average scalars from cell centers to cell corners
!-----------------------------------------------------------------------
	do ksdm=1,nsdm
	do j=1,jjm
	do i=1,iim
	  call xyz_to_lonlat(p(:,i,j,ksdm),s(:))
	  grid_center_lon(i,j,ksdm) = s(1)
	  grid_center_lat(i,j,ksdm) = s(2)
	  do icorner=1,ncorners
	    tem_c(:) = c(:,icorner,i,j,ksdm)
            if(grid_corner_mask(icorner,i,j,ksdm)==c1) &
                       call unit_vector_in_3space ( tem_c(:) )
	    call xyz_to_lonlat(tem_c(:), s(:))
            grid_corner_lon(icorner,i,j,ksdm) = s(1)
            grid_corner_lat(icorner,i,j,ksdm) = s(2)
	  enddo
	enddo
	enddo
	enddo
	grid_center_lon = grid_center_lon * grid_center_mask
	grid_center_lat = grid_center_lat * grid_center_mask
	grid_corner_lon = grid_corner_lon * grid_corner_mask
	grid_corner_lat = grid_corner_lat * grid_corner_mask

!-----------------------------------------------------------------------
! compute weights to average scalars from cell centers to cell corners
!-----------------------------------------------------------------------
	call compute_scalar_weights (ndata,ncorners,iim,jjm,nsdm,            &
                                     sub_area,corner_area_inv,               &
                                     grid_corner_mask, s_weights)

!-----------------------------------------------------------------------
! compute weights to compute the gradient of scalar fields
!-----------------------------------------------------------------------
	call compute_gradient_weights (                                      &
                                  ndata,r3,nspace,ncorners,iim,jjm,nsdm,     &
                                  dc,normal,v_weights)

!-----------------------------------------------------------------------
! compute weights to compute the Laplacian of a scalar field
!-----------------------------------------------------------------------
	call compute_laplacian_weights (polygon_type,                        &
                           ndata,nspace,ncorners,nweights,iim,jjm,nsdm,      &
                           area_inv,corner_area_inv,v_weights,l_weights)

!-----------------------------------------------------------------------
! compute weights to compute the Laplacian at the corners
!-----------------------------------------------------------------------
!       call compute_zm_laplacian_weights

!----------------------------------------------------------------------------
! keep the user informed
!----------------------------------------------------------------------------
       if(my_task.eq.0) then
          write(6,*) ' finished in generate_grid_metrics at this resolution'
	  write(6,*) '-------------------------------------------------------'
          write(6,*)
	  write(6,*)
        endif

      deallocate (X1,X2,polygon_type,metrics_mask,t,r,h,g,m)     
      deallocate (grid_corner_mask,grid_center_mask)     

	end subroutine generate_grid_metrics_level
!======================================================================
! END GENERATE_GRID_METRICS_LEVEL
!======================================================================

!======================================================================
! BEGIN COMPUTE_SCALAR_WEIGHTS
!======================================================================
	subroutine compute_scalar_weights (                                  &
                                     ndata,ncorners,iim,jjm,nsdm,            &
                                     sub_area,corner_area_inv,               &
                                     grid_corner_mask, s_weights)
	implicit none
        integer (kind=int_kind),intent( in) ::                               & 
           ndata,ncorners,iim,jjm,nsdm

	integer (kind=int_kind) :: i,j,k,l,icorner,ksdm,ierr

	real (kind=dbl_kind),intent( in) ::                                  &
                 sub_area       (ndata,         ncorners, iim, jjm, nsdm),   &
                 grid_corner_mask(              ncorners, iim, jjm, nsdm),   &
      		 corner_area_inv(               ncorners, iim, jjm, nsdm)

	real (kind=dbl_kind),intent(out) ::                                  &
      		 s_weights      (ndata,         ncorners, iim, jjm, nsdm)

!-----------------------------------------------------------------------
! compute scalar averaging weights at every grid corner which will
!   prognose velocity
!-----------------------------------------------------------------------
	do ksdm = 1,nsdm
        do j=1,jjm
           do i=1,iim
	      do icorner = 1,ncorners
! ross	         if(grid_corner_mask(icorner,i,j,ksdm) == 1.0_dbl_kind) then
                    do k=1,ndata
                       l=k-1
                       if(l == 0) l=ndata
                       s_weights(k,icorner,i,j,ksdm) =                  &
                                     sub_area(k,icorner,i,j,ksdm)       &
                                   * corner_area_inv(icorner,i,j,ksdm)
                    enddo
! ross	         endif
              enddo
           enddo
        enddo
	enddo

	if(my_task.eq.0) then
	write(6,*)
	write(6,*) ' Diagnostics for scalar weights (s_weights) for each task'
	write(6,*) ' maxval of s_weights will be 0.333 on a perfect hexagonal grid'
	write(6,*) ' maxval of sum_weights should be 1.000 to round-off'
	write(6,*) ' minval of sum_weights should be 1.000 to round-off'
	endif
	do i=0,npe_atmos-1
	if(my_task.eq.i) then
	write(6,*) '  ******  ',my_task,' maxval s_weights : ',maxval(s_weights)
	write(6,*) '  ******  ',my_task,' maxval sum s_weights : ',                &
             maxval(sum(s_weights,dim=1),mask=grid_corner_mask.eq.c1)
    write(6,*) '  ******  ',my_task,' minval sum s_weights : ',            &
             minval(sum(s_weights,dim=1),mask=grid_corner_mask.eq.c1)
	endif
#if mpi == 1
        CALL MPI_BARRIER (MPI_COMM_ATMOS,ierr)
#endif
	enddo

#if mpi == 1
        CALL MPI_BARRIER (MPI_COMM_ATMOS,ierr)
#endif


	end subroutine compute_scalar_weights
!======================================================================
! END COMPUTE_SCALAR_WEIGHTS
!======================================================================

!======================================================================
! BEGIN COMPUTE_GRADIENT_WEIGHTS
!======================================================================
	subroutine compute_gradient_weights (                                &
                                 ndata,r3,nspace,ncorners,iim,jjm,nsdm,      &
                                 dc,normal,v_weights)

	implicit none
        integer (kind=int_kind),intent( in) ::                               & 
           ndata,r3,nspace,ncorners,iim,jjm,nsdm

	integer (kind=int_kind) :: i,j,k,l,icorner,ispace,ksdm

        real (kind=dbl_kind),intent( in) ::                                  &
         dc             (ndata,         ncorners, iim, jjm, nsdm),           &
      	 normal         (ndata,     r3, ncorners, iim, jjm, nsdm)
        real (kind=dbl_kind),intent(out) ::                                  &
         v_weights      (ndata, nspace, ncorners, iim, jjm, nsdm)

!-----------------------------------------------------------------------
! compute gradient weights at every corner that will prognose velocity
!-----------------------------------------------------------------------
	do ksdm = 1,nsdm
        do j=1,jjm
         do i=1,iim
          do icorner = 1,ncorners
           do k=1,ndata
            l = k-1
            if(l == 0) l=ndata
	     do ispace = 1,nspace
	      v_weights(k,ispace,icorner,i,j,ksdm) =                          &
               ( +dc(l,icorner,i,j,ksdm) * normal(l,ispace,icorner,i,j,ksdm)  &
                 -dc(k,icorner,i,j,ksdm) * normal(k,ispace,icorner,i,j,ksdm))
             enddo
	    enddo
          enddo
         enddo
        enddo
	enddo

	end subroutine compute_gradient_weights
!======================================================================
! END COMPUTE_GRADIENT_WEIGHTS
!======================================================================

!======================================================================
! BEGIN COMPUTE_LAPLACIAN_WEIGHTS
!======================================================================
	subroutine compute_laplacian_weights (polygon_type,                  &
                           ndata,nspace,ncorners,nweights,iim,jjm,nsdm,      &
                           area_inv,corner_area_inv,v_weights,l_weights)

	implicit none

        integer (kind=int_kind),intent( in) ::                               & 
           ndata,nspace,ncorners,nweights,iim,jjm,nsdm

	integer (kind=int_kind) ::                                           &
           i,j,k,l,ispace,icorner,ksdm

        real (kind=dbl_kind),intent( in) ::                                  &
      	 polygon_type   (                         iim, jjm, nsdm),           &
         area_inv       (                         iim, jjm, nsdm),           &
      	 corner_area_inv(               ncorners, iim, jjm, nsdm),           &
         v_weights      (ndata, nspace, ncorners, iim, jjm, nsdm)

        real (kind=dbl_kind),intent(out) ::                                  &
      	 l_weights      (nweights,                iim, jjm, nsdm)

	real (kind=dbl_kind) :: d,e,f,ai,f1,f2
	logical, parameter :: l_eigenvalue = .false.

!-----------------------------------------------------------------------
! compute Laplacian weights
!-----------------------------------------------------------------------
	do ksdm = 1,nsdm
        do l=1,jjm
          do k=1,iim
	    if(polygon_type(k,l,ksdm) > 0.0_dbl_kind) then
              do ispace=1,nspace

!-----------------------------------------------------------------------
! fill metrics (painfully) by looking at each corner
! what we are doing here is taking the divergence of a gradient
!-----------------------------------------------------------------------
                i=k
                j=l
                icorner = 1
                ai = corner_area_inv(icorner,i,j,ksdm)
                d = v_weights(1,ispace,icorner,i,j,ksdm)
                e = v_weights(2,ispace,icorner,i,j,ksdm)
                f = v_weights(3,ispace,icorner,i,j,ksdm)
                l_weights( 7, k, l,ksdm) = l_weights( 7, k, l,ksdm) + ai*d*d
                l_weights( 1, k, l,ksdm) = l_weights( 1, k, l,ksdm) + ai*d*e
                l_weights( 2, k, l,ksdm) = l_weights( 2, k, l,ksdm) + ai*d*f

                i=k
                j=l
                icorner = 2
                ai = corner_area_inv(icorner,i,j,ksdm)
                d = v_weights(1,ispace,icorner,i,j,ksdm)
                e = v_weights(2,ispace,icorner,i,j,ksdm)
                f = v_weights(3,ispace,icorner,i,j,ksdm)
                l_weights( 7, k, l,ksdm) = l_weights( 7, k, l,ksdm) + ai*d*d
                l_weights( 2, k, l,ksdm) = l_weights( 2, k, l,ksdm) + ai*d*e
                l_weights( 3, k, l,ksdm) = l_weights( 3, k, l,ksdm) + ai*d*f
        
                i=k+1
                j=l
                icorner = 1
                ai = corner_area_inv(icorner,i,j,ksdm)
                d = v_weights(2,ispace,icorner,i,j,ksdm)
                e = v_weights(3,ispace,icorner,i,j,ksdm)
                f = v_weights(1,ispace,icorner,i,j,ksdm)
                l_weights( 7, k, l,ksdm) = l_weights( 7, k, l,ksdm) + ai*d*d
                l_weights( 3, k, l,ksdm) = l_weights( 3, k, l,ksdm) + ai*d*e
                l_weights( 4, k, l,ksdm) = l_weights( 4, k, l,ksdm) + ai*d*f
        

                i=k+1
                j=l+1
                icorner = 2
                ai = corner_area_inv(icorner,i,j,ksdm)
                d = v_weights(2,ispace,icorner,i,j,ksdm)
                e = v_weights(3,ispace,icorner,i,j,ksdm)
                f = v_weights(1,ispace,icorner,i,j,ksdm)
                l_weights( 7, k, l,ksdm) = l_weights( 7, k, l,ksdm) + ai*d*d
                l_weights( 4, k, l,ksdm) = l_weights( 4, k, l,ksdm) + ai*d*e
                l_weights( 5, k, l,ksdm) = l_weights( 5, k, l,ksdm) + ai*d*f
        
                i=k+1
                j=l+1
	        icorner = 1
                ai = corner_area_inv(icorner,i,j,ksdm)
                d = v_weights(3,ispace,icorner,i,j,ksdm)
                e = v_weights(1,ispace,icorner,i,j,ksdm)
                f = v_weights(2,ispace,icorner,i,j,ksdm)
                l_weights( 7, k, l,ksdm) = l_weights( 7, k, l,ksdm) + ai*d*d
                l_weights( 5, k, l,ksdm) = l_weights( 5, k, l,ksdm) + ai*d*e
                l_weights( 6, k, l,ksdm) = l_weights( 6, k, l,ksdm) + ai*d*f
        
        
                i=k
                j=l+1
                icorner = 2
                ai = corner_area_inv(icorner,i,j,ksdm)
                d = v_weights(3,ispace,icorner,i,j,ksdm)
                e = v_weights(1,ispace,icorner,i,j,ksdm)
                f = v_weights(2,ispace,icorner,i,j,ksdm)
                l_weights( 7, k, l,ksdm) = l_weights( 7, k, l,ksdm) + ai*d*d
                l_weights( 6, k, l,ksdm) = l_weights( 6, k, l,ksdm) + ai*d*e
                l_weights( 1, k, l,ksdm) = l_weights( 1, k, l,ksdm) + ai*d*f

              enddo
	    endif

            l_weights(:,k,l,ksdm) = -l_weights(:,k,l,ksdm)*area_inv(k,l,ksdm)

          enddo
        enddo
	enddo

!-----------------------------------------------------------------------
! at pentagons, move the l_weight on neighor 5 into neighbor 6
!-----------------------------------------------------------------------
	where(polygon_type(:,:,:) == 2.0_dbl_kind)
	  l_weights(6,:,:,:) = l_weights(6,:,:,:) + l_weights(5,:,:,:)
	  l_weights(5,:,:,:) = 0.0_dbl_kind
	endwhere

!-----------------------------------------------------------------------
! at NP, move the l_weight on neighbor 1 into neighbor 6
! at SP, move the l_weight on neighbor 3 into neighbor 4
!-----------------------------------------------------------------------
	do ksdm=1,nsdm
        do j=1,jjm
          do i=1,iim
	    if(polygon_type(i,j,ksdm) == 3.0_dbl_kind) then
              l_weights(6,i,j,ksdm) = l_weights(6,i,j,ksdm) + l_weights(1,i,j,ksdm)
              l_weights(1,i,j,ksdm) = 0.0_dbl_kind
            endif
            if(polygon_type(i,j,ksdm) == 4.0_dbl_kind) then
              l_weights(4,i,j,ksdm) = l_weights(4,i,j,ksdm) + l_weights(3,i,j,ksdm)
              l_weights(3,i,j,ksdm) = 0.0_dbl_kind
            endif
	  enddo
	enddo
	enddo

!-----------------------------------------------------------------------
! check the Laplacian weights, should sum to zero around each cell
!-----------------------------------------------------------------------
        f2 = 0.0
	do ksdm=1,nsdm
        do j=1,jjm
          do i=1,iim
            if(polygon_type(i,j,ksdm) > 0.0_dbl_kind) then
              f1 = sum(l_weights(:,i,j,ksdm))
              if(abs(f1) > f2) f2=f1
	    endif
          enddo
        enddo
	enddo
#if mpi == 1
        CALL MPI_BARRIER (MPI_COMM_ATMOS,ierr)
#endif
	if(my_task.eq.0) write(6,*)
        write(6,*) my_task, 'max error in Laplacian weights ',f2
#if mpi == 1
        CALL MPI_BARRIER (MPI_COMM_ATMOS,ierr)
#endif
	if(my_task.eq.0) write(6,*)

!       if(l_eigenvalue) call fill_eigenvalue_matrix

	end subroutine compute_laplacian_weights
!======================================================================
! END COMPUTE_LAPLACIAN_WEIGHTS
!======================================================================

!======================================================================
! BEGINNING OF GRID_CENTER_AREA
!======================================================================
        subroutine grid_center_area(                                  &
                polygon_type,                                         &
                ndata,                                                &
                ncorners,                                             &
                iim,                                                  &
                jjm,                                                  &
                nsdm,                                                 &
      		sub_area,                                             &
      		area_inv,                                             &
      		area)
        implicit none
        integer (kind=int_kind), intent(in) ::                        &
                        ncorners,                                     &
                        ndata,                                        &
                        iim,                                          &
                        jjm,                                          &
                        nsdm
        integer (kind=int_kind) :: i,j,ksdm
 
        real (kind=dbl_kind), intent( in) ::                          &
           polygon_type(               iim,jjm,nsdm),                 &
           sub_area    (ndata,ncorners,iim,jjm,nsdm)          

        real (kind=dbl_kind), intent(out) ::                          &
           area_inv(             iim,jjm,nsdm),                       &
           area    (             iim,jjm,nsdm)

!-----------------------------------------------------------------------
! initialize
!-----------------------------------------------------------------------
        area = 0.0_dbl_kind
        area_inv = 0.0_dbl_kind
!-----------------------------------------------------------------------
! loop over all grid cells and if it is a real cell, then sum
! the sub_areas to find the total grid cell area
!-----------------------------------------------------------------------
	do ksdm = 1,nsdm
          do j=1,jjm
            do i = 1,iim
	      if(polygon_type(i,j,ksdm) > 0.0_dbl_kind) then
                area(i,j,ksdm) = + sub_area(1,1,i  ,j  ,ksdm)       &
                                 + sub_area(1,2,i  ,j  ,ksdm)       &
                                 + sub_area(2,1,i+1,j  ,ksdm)       &
                                 + sub_area(2,2,i+1,j+1,ksdm)       &
                                 + sub_area(3,1,i+1,j+1,ksdm)       &
                                 + sub_area(3,2,i  ,j+1,ksdm)
                area_inv(i,j,ksdm) = 1.0_dbl_kind / area(i,j,ksdm)
	      endif
            enddo
          enddo
	enddo

        end subroutine grid_center_area
!======================================================================
! END OF GRID_CENTER_AREA
!======================================================================

!======================================================================
! BEGIN read_grid_center_fine
!======================================================================
      subroutine read_grid_center_fine (p_list)
      implicit none

      integer (kind=int_kind) ::                                             &
         igrid

      real (kind=dbl_kind),intent(out) ::                                    &
         p_list(3,max_ig)

!-----------------------------------------------------------------------
! read in grid center data
!       the (x,y,z) point is read in
!-----------------------------------------------------------------------
        if (my_task == 0) then
          write(6,*) 'opening grid point file'
          if(max_ig.eq.0001442) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0001442',form='formatted',status='old') 
          elseif(max_ig.eq.0002562) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0002562',form='formatted',status='old') 
          elseif(max_ig.eq.0005762) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0005762',form='formatted',status='old') 
          elseif(max_ig.eq.0010242) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0010242',form='formatted',status='old')
          elseif(max_ig.eq.0023042) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0023042',form='formatted',status='old')
          elseif(max_ig.eq.0040962) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0040962',form='formatted',status='old')
          elseif(max_ig.eq.0163842) then
            open(unit=2,file=trim(datapath)//'data/grid_centers_0163842',form='formatted',status='old')
          else
            write(6,*) ' grid_center file not found'
            stop
          endif
          do igrid=1,max_ig
            read(2,20) p_list(1,igrid),p_list(2,igrid),p_list(3,igrid)
          enddo
 20       format(3f24.20)
          close(2)
          write(6,*) 'done reading grid point file'
        endif

      end subroutine read_grid_center_fine
!======================================================================
! END read_grid_center_fine
!======================================================================

!======================================================================
! BEGIN COMPUTE_POLYGON_TYPE
!======================================================================
        subroutine compute_polygon_type(my_task,level,im,jm,iim,jjm,nsdm,polygon_type)

        integer (kind=int_kind), intent(in) :: my_task,level,im,jm,iim,jjm,nsdm
        real    (kind=dbl_kind), intent(out) :: polygon_type(iim,jjm,nsdm)

        integer (kind=int_kind) :: i,ksdm,panel,im5
        real    (kind=dbl_kind) :: work(im,jm), work_1(iim,jjm,nsdm)

!-----------------------------------------------------------------------
! set contants
!-----------------------------------------------------------------------
        im5 = im/5

!-----------------------------------------------------------------------
! initialize to zero
!-----------------------------------------------------------------------
        polygon_type = 0.0_dbl_kind

!-----------------------------------------------------------------------
! initialize interior of each panel
!-----------------------------------------------------------------------
        do ksdm = 1,nsdm
          polygon_type(2:iim-1,2:jjm-1,ksdm) = 1.0_dbl_kind
        enddo

!-----------------------------------------------------------------------
! add 1 to pentagons
!-----------------------------------------------------------------------
        if(my_task == 0) then
          work(:,:) = 0.0_dbl_kind
          do panel = 0,4
            i  = im5*panel + jm - 1
            work(i,jm-1) = 1.0_dbl_kind         ! NH pentagon
            i  = im5*(panel+1) - 1
            work(i,jm-1) = 1.0_dbl_kind         ! SH pentagon
          enddo
        endif

!-----------------------------------------------------------------------
! scatter across sub domains
!-----------------------------------------------------------------------
        call scatter(level,im,jm,iim,jjm,1,nsdm,work(1,1),work_1(1,1,1))
        polygon_type = polygon_type + work_1

!-----------------------------------------------------------------------
! clean up edges
!-----------------------------------------------------------------------
        do ksdm = 1,nsdm
          polygon_type(1:iim,1,ksdm) = 0.0_dbl_kind
          polygon_type(1:iim,jjm,ksdm) = 0.0_dbl_kind
          polygon_type(1,1:jjm,ksdm) = 0.0_dbl_kind
          polygon_type(iim,1:jjm,ksdm) = 0.0_dbl_kind
        enddo

!-----------------------------------------------------------------------
! set poles to a value of 3 for NP and 4 for SP
!-----------------------------------------------------------------------
        if(my_task == 0) then
          work(:,:) = 0.0_dbl_kind
          work(1,jm-1) = 3.0_dbl_kind           ! north pole
          work(im-1,1) = 4.0_dbl_kind           ! south pole
        endif

!-----------------------------------------------------------------------
! scatter across sub domains
!-----------------------------------------------------------------------
        call scatter(level,im,jm,iim,jjm,1,nsdm,work(1,1),work_1(1,1,1))
        polygon_type = polygon_type + work_1

!-----------------------------------------------------------------------
! set all wrapped grid cells to -1
!-----------------------------------------------------------------------
        where(polygon_type.lt.0.5_dbl_kind) polygon_type = -1.0_dbl_kind

        end subroutine compute_polygon_type
!======================================================================
! END COMPUTE_POLYGON_TYPE
!======================================================================


        end module grid_metrics_generator
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
