
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module gravity_wave_drag_module

!-----------------------------------------------------------------------
! PURPOSE: compute the gravity wave drag (acceleration) at the cell
!  corners
!-----------------------------------------------------------------------


!-------------------------------------------------------------------------
!
!     The purpose of this subroutine is to use a gravity wave drag
!     scheme, first proposed by Palmer et al. (1986).
!
!     Parameters:
!       RIMIN - Minimum, wave-modified, Richardson #
!	GWKAP - tunable parameter kappa equal to 2.5*10^(-5) (m**-1)
!       TPGVARMAX - Upper bound on topography variance, set to (400m)^2
!
!     Variables:
!     Defined at layer edges, and cell vertices
!       ril2_c  - Richardson number at a given point (nd)
!       rhol2_c - density at a given point (kg/m3)
!       bvfl2_c - Brunt-Vaisala frequency at a given point (1/s)
!	ul2     - vector velocity (m/s)
!	u_str   - vector stress  (kg/m/s2)
!       DH - displacement amplitude
!
!     Defined at layer center, and cell vertices
!	u_tend_gwd - tendency of vector velocity (m/s2)
!
!     Defined at bottom only:
!       tpgvar_c - variance of topography, obtained from US Navy dataset
!	u_str0   - vector stress at surface
!	magstr0  - mag(u_str0)
!	UMAG     - stress vector dot velocity vector
!-------------------------------------------------------------------------


        use kinds
        use physical_parameters
        use model_parameters
	use model_control
	use time_manager_module
        use prognostics
        use operators
	use sigma_coordinate
	use qp_module

	implicit none
	private

	integer (kind = int_kind) :: i, j, k, ic

#ifdef generalized_sigma
	integer (kind = int_kind), parameter :: nprop = npbl
#else
	integer (kind = int_kind), parameter :: nprop = 1
#endif

	real (kind = dbl_kind), parameter :: 
     *			small = 1.0e-14_dbl_kind,
     *			tpgvarmax = 160000.0_dbl_kind,
     *			gwkap = 2.5e-5_dbl_kind,
     *			rimin = 0.25_dbl_kind

	real (kind = dbl_kind), dimension(ncorners,iim,jjm,nlm) ::
     *			plg_c = 0.0_dbl_kind

	real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm,nlm), public ::
     *			u_tend_gwd = 0.0_dbl_kind

	real (kind = dbl_kind), dimension(iim,jjm,nlmp1) :: 
     *			rhol2 = 0.0_dbl_kind,
     *			dthdp = 0.0_dbl_kind,
     *			bvfl2 = 0.0_dbl_kind

	real (kind = dbl_kind), dimension(ncorners,iim,jjm,nlmp1) :: 
     *                  ril2_c = 0.0_dbl_kind,
     *                  rhol2_c = 0.0_dbl_kind,
     *			bvfl2_c = 0.0_dbl_kind,
     *			shearl2_c = 0.0_dbl_kind,
     *			dthdp_c = 0.0_dbl_kind,
     *			plgl2_c = 0.0_dbl_kind,
     *			thl2_c = 0.0_dbl_kind

	real (kind = dbl_kind), dimension(nspace,ncorners,iim,jjm,nlmp1) :: 
     *			ul2 = 0.0_dbl_kind,
     *                  shearul2 = 0.0_dbl_kind

        real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm,nsdm), public ::
     *			u_forcing_gwd = 0.0_dbl_kind

	public ::       compute_gravity_wave_drag


!*****************************************************************

        contains

!*****************************************************************

!======================================================================
! BEGINNING OF COMPUTE_GRAVITY_WAVE_DRAG
!======================================================================
	subroutine compute_gravity_wave_drag
	implicit none

        logical, parameter :: recompute = .true.
      real(kind=dbl_kind) :: u_work(iim,jjm,nlm)

!----------------------------------------------------------------------------
! recompute gwd forcing based on logical
!----------------------------------------------------------------------------
	if(recompute) then

!----------------------------------------------------------------------------
! loop over sub-domains
!----------------------------------------------------------------------------
	do ksdm = 1,nsdm

!----------------------------------------------------------------------------
! compute state parameters
!----------------------------------------------------------------------------
	  call pre_gwd

!----------------------------------------------------------------------------
! compute the acceleration due to gravity wave drag
!----------------------------------------------------------------------------
	  call gwd
	  
	  ! save corner 1 tendencies
	     u_work(:,:,:) = u_tend_gwd(1,1,:,:,:)
         call qp3sum(u_work(1,1,1), iim*jjm, 1.0_dbl_kind, 
     *               iqp3_tnmmugwd, ksdm, iim*jjm)
	     u_work(:,:,:) = u_tend_gwd(2,1,:,:,:)
         call qp3sum(u_work(1,1,1), iim*jjm, 1.0_dbl_kind, 
     *               iqp3_tnmmvgwd, ksdm, iim*jjm)
        enddo

	endif	! if(recompute)

	end subroutine compute_gravity_wave_drag
!===========================================================================
! END OF COMPUTE_GRAVITY_WAVE_DRAG
!===========================================================================


!===========================================================================
! BEGINNING OF PRE_GWD
!===========================================================================
	subroutine pre_gwd

!-----------------------------------------------------------------------
! PURPOSE: compute state parameters such as: rhol2_c (density at layer
!    edges (l2) at the cell vertices (_c)), bvfl2_c, shearl2_c
!-----------------------------------------------------------------------

	implicit none

!-----------------------------------------------------------------------
! impose maximum variance limit
!-----------------------------------------------------------------------
	where(tpgvar_c.gt.tpgvarmax) tpgvar_c = tpgvarmax

!-----------------------------------------------------------------------
! compute velocity at layer edges  
!-----------------------------------------------------------------------
	do k=2,nlm
	  ul2(:,:,:,:,k) = p5*(u(:,:,:,:,k-1,n3,ksdm) + u(:,:,:,:,k,n3,ksdm))
	enddo
        ul2(:,:,:,:,    1) = u(:,:,:,:,  1,n3,ksdm)
        ul2(:,:,:,:,nlmp1) = u(:,:,:,:,nlm,n3,ksdm)


!-----------------------------------------------------------------------
! average pressure to cell corner
!  add in k to avoid fpe
!-----------------------------------------------------------------------
	do k=1,nlm
        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  plg(:,:,k,ksdm),
     *                  plg_c(:,:,:,k))
!plg_c = plg_c + k
        where(plg_c(:,:,:,k) == c0 ) plg_c(:,:,:,k) = k
	enddo

!-----------------------------------------------------------------------
! compute shear at layer edges at cell centers
!-----------------------------------------------------------------------

	do ic = 1,ncorners
          call vert_deriv_plg(
     *          u(1,ic,:,:,:,n3,ksdm),
     *          plg_c(ic,:,:,:),
     *          shearul2(1,ic,:,:,:))	
          call vert_deriv_plg(
     *          u(2,ic,:,:,:,n3,ksdm),
     *          plg_c(ic,:,:,:),
     *          shearul2(2,ic,:,:,:))	
	enddo
	shearl2_c = sqrt( shearul2(1,:,:,:,:)**2 + shearul2(2,:,:,:,:)**2)
	where(shearl2_c.lt.small) shearl2_c=small
	
!-----------------------------------------------------------------------
! compute vertical derivative of theta wrt pressure
!-----------------------------------------------------------------------
	call vert_deriv_plg(
     *		th(:,:,:,n3,ksdm),
     *		plg(:,:,:,ksdm),
     *		dthdp(:,:,:))

!-----------------------------------------------------------------------
! compute density and Brunt-Vaisala Freqency
!-----------------------------------------------------------------------
	rhol2(:,:,:) = plgl2(:,:,:,ksdm) / 
     *     (gas_const_r * thl2(:,:,:,ksdm) * bpgl2(:,:,:,ksdm))
	bvfl2(:,:,:) = -rhol2(:,:,:) * grav**2 * dthdp(:,:,:) / 
     *	   thl2(:,:,:,ksdm)

!-----------------------------------------------------------------------
! average stuff to cell wall
!-----------------------------------------------------------------------
        do k=1,nlmp1
           call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  rhol2(:,:,k),
     *                  rhol2_c(:,:,:,k))
           call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  bvfl2(:,:,k),
     *                  bvfl2_c(:,:,:,k))
           call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  dthdp(:,:,k),
     *                  dthdp_c(:,:,:,k))
           call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  thl2(:,:,k,ksdm),
     *                  thl2_c(:,:,:,k))
           call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  plgl2(:,:,k,ksdm),
     *                  plgl2_c(:,:,:,k))
        enddo

!-----------------------------------------------------------------------
! compute gradient Richardson number
!-----------------------------------------------------------------------
	ril2_c = -dthdp_c / (rhol2_c * thl2_c * shearl2_c**2 + small)

!-----------------------------------------------------------------------
! put bounds on ril2_c and bvfl2_c
!-----------------------------------------------------------------------
	where (ril2_c.lt.0.25_dbl_kind) ril2_c = 0.25_dbl_kind
	where (bvfl2_c.lt.small) bvfl2_c=small
	
!-----------------------------------------------------------------------
! convert bvfl2_c from units of s**2 to s
!-----------------------------------------------------------------------
	bvfl2_c = sqrt(bvfl2_c)

	end subroutine pre_gwd
!===========================================================================
! END OF PRE_GWD
!===========================================================================


!===========================================================================
! BEGINNING OF GWD
!===========================================================================
	subroutine gwd

	implicit none

	real (kind = dbl_kind), dimension(nlmp1) ::
     *		rho=0.0_dbl_kind,
     *		bvf=0.0_dbl_kind,
     *		magstr=0.0_dbl_kind

	real (kind = dbl_kind), dimension(2,nlmp1) ::
     *		u_str=0.0_dbl_kind
     
	real (kind = dbl_kind), dimension(2) ::
     *		u_str0=0.0_dbl_kind
     
	real (kind = dbl_kind) ::
     *		magstr0=0.0_dbl_kind,
     *		dh=0.0_dbl_kind,
     *		umag=0.0_dbl_kind,
     *		rf=0.0_dbl_kind,
     *		sqri=0.0_dbl_kind,
     *		rimin=0.0_dbl_kind

	logical ::  skipit, do_write 
	logical, parameter :: l_debug = .true.
	integer, parameter :: id = 15, jd = 16, icd = 1, kd = 1

!-------------------------------------------------------------------------
! initialize tendencies
!-------------------------------------------------------------------------
	u_tend_gwd = c0

!-------------------------------------------------------------------------
!  LOOP over each grid cell (iim,jjm)
!-------------------------------------------------------------------------
      DO J = 1,JJM
      DO I = 1,IIM

!-------------------------------------------------------------------------
!  LOOP over each wall
!-------------------------------------------------------------------------
	do ic = 1, ncorners

!-------------------------------------------------------------------------
!  initialize logical for skipping
!-------------------------------------------------------------------------
	  skipit = .false.

!-------------------------------------------------------------------------
!  set logical for debugging
!-------------------------------------------------------------------------
	 do_write = .false.
	 if(my_task.eq.0.and.l_debug.and.l_diagnostics) then
	 if(j.eq.jd.and.i.eq.id.and.ic.eq.icd.and.ksdm.eq.kd) then
	    do_write = .true.
	    write(6,10) ic, i, j, ksdm
	 endif
	 endif

!-------------------------------------------------------------------------
! initialize stresses
!-------------------------------------------------------------------------
	  u_str = c0
	  magstr = c0

!-------------------------------------------------------------------------
! skip dead wall or places where tpgvar_w = 0
!  (note) tpgvar_w was set to zero at dead walls
!-------------------------------------------------------------------------
          if(tpgvar_c(ic,i,j,ksdm).eq.c0) cycle

!-------------------------------------------------------------------------
! copy data into scalar variables for easy reading
!-------------------------------------------------------------------------
	   rho(:) = rhol2_c(ic,i,j,:)
	   bvf(:) = bvfl2_c(ic,i,j,:)

!-------------------------------------------------------------------------
!  compute the source of gravity wave energy from the surface
!    use rho and bvf from layer edge nlm
!-------------------------------------------------------------------------
           u_str0 = GWKAP * rho(nlm) * bvf(nlm) 
     *			* ul2(:,ic,i,j,nlmp1) * tpgvar_c(ic,i,j,ksdm)
	   magstr0 = max(small,sqrt(u_str0(1)**2+u_str0(2)**2))

	   if(do_write) write(6,20) tpgvar_c(ic,i,j,ksdm), 
     *		u_str0(1), u_str0(2), magstr0, ul2(1,ic,i,j,nlmp1),
     *		ul2(2,ic,i,j,nlmp1),rho(nlm), bvf(nlm)

!-------------------------------------------------------------------------
!  check bounds
!-------------------------------------------------------------------------
          if(magstr0.gt.100.0) then
            write(6,*) ' MAGSTR0 out of range'
            stop
          endif

!-------------------------------------------------------------------------
!  put the sfc stresses in the 3-D stress array.
!-------------------------------------------------------------------------
          u_str(:,nlmp1) =  u_str0
          magstr(nlmp1) =  magstr0

!-------------------------------------------------------------------------
!  save the forcing
!-------------------------------------------------------------------------
	  u_forcing_gwd(:,ic,i,j,ksdm) = u_str0

!-------------------------------------------------------------------------
!  allow all gw energy to propagate across the PBL top
!-------------------------------------------------------------------------
          DO k = NLM,NLM-nprop+1,-1
            u_str(:,k) =  u_str(:,nlmp1)
            magstr(k) =  magstr(nlmp1)
          ENDDO

	  if(do_write) write(6,29)

!-------------------------------------------------------------------------
!  Begin loop over vertical level
!-------------------------------------------------------------------------
          DO k = NLM-nprop,2,-1

!-------------------------------------------------------------------------
!  if there is an ocean point, or turbulence is reached, then go to
!  the bottom of the loop.
!-------------------------------------------------------------------------
            IF (SKIPIT) CYCLE

!-------------------------------------------------------------------------
!  seems like UMAG should be non-negative. also, if its magnitude
!  is too small, then rimin will be < 0.
!-------------------------------------------------------------------------
	    umag = (ul2(1,ic,i,j,k)*u_str0(1) + ul2(2,ic,i,j,k)*u_str0(2))
	    umag = max(umag/magstr0,small)

!-------------------------------------------------------------------------
!  calculate estimate of displacement amplitude, minimum Richardson #
!-------------------------------------------------------------------------
            dh = magstr(k+1) / (GWKAP*rho(k)*bvf(k)*umag)
	    dh = sqrt(max(0.0_dbl_kind,dh))

!-------------------------------------------------------------------------
!  estimate the mininimum Richardson number
!-------------------------------------------------------------------------
	    rf    = bvf(k) * dh / umag
	    sqri  = sqrt(ril2_c(ic,i,j,k))
	    rimin = sqri**2 * (c1 - rf) / (c1 + sqri*rf)**2
	
!-------------------------------------------------------------------------
!  Begin IF-THEN logic to split up what happens when the
!  minimum Ri gets below 1/4
!-------------------------------------------------------------------------
               IF (rimin .GE. 0.25_dbl_kind) THEN

!-------------------------------------------------------------------------
!  If minimum Richardson number is greater than or equal to
!  0.25, then there is no wave breaking, and there is no change
!  in the stress of the above layer.
!-------------------------------------------------------------------------
                  u_str(:,k) = u_str(:,k+1)
                  MAGSTR(k) = MAGSTR(k+1)

               ELSE

!-------------------------------------------------------------------------
!  Ri of 1/4 is "saturation" Richardson number, so if it is
!  exceeded on the downside, reset RIMIN to 1/4.
!-------------------------------------------------------------------------
                 RIMIN = 0.25_dbl_kind

!-------------------------------------------------------------------------
!  calculate saturation vertical displacement.
!-------------------------------------------------------------------------
                 DH = 2.0 * (-(0.50/sqri + 1.0)
     &                    + SQRT(2.0 + 1/sqri))

                 if(dh.lt.0.0.or.dh.gt.0.85) then
                    write(6,*) ' dh out of range'
                    stop
                 endif

                 DH = DH*(UMAG/bvf(k))

!-------------------------------------------------------------------------
!  calculate saturation stress
!-------------------------------------------------------------------------
                 u_str(:,k) = gwkap*rho(k)*bvf(k)*umag*dh**2*u_str0(:)/magstr0
	         magstr(k) = sqrt(u_str(1,k)**2 + u_str(2,k)**2)

!-------------------------------------------------------------------------
!  if the displacement height is less than zero, assume turbulence
!  is taking place, and set stresses equal to zero.
!-------------------------------------------------------------------------
                 IF(DH .LE. 0.) THEN
                    SKIPIT  = .TRUE.
                    u_str(:,k) = 0.
                    MAGSTR(k) = 0.
                 ENDIF

!-------------------------------------------------------------------------
!  check that magnitude of stress decreases with height
!-------------------------------------------------------------------------
                 IF (MAGSTR(k) .GT. MAGSTR(k+1) + 1.0e-3) then
                     write(6,*) ' stress increasing w/ height'
                     stop
                 ENDIF

	       ENDIF 	! (if rimin .gt. 0.25)
	     
	       if(do_write) write(6,30) k,u_str(:,k),sqri**2,rimin

             ENDDO	! do k = nlm-nprop,2,-1

!-------------------------------------------------------------------------
!  At top level, the stress is specified as zero by Palmer et al.
!-------------------------------------------------------------------------
            u_str(:,1) = 0.0

!-------------------------------------------------------------------------
!  Calculate velocity tendencies, this is the flux convergence of the stress
!-------------------------------------------------------------------------
	    if(do_write) write(6,39)
	    do k=1,nlm
	      u_tend_gwd(:,ic,i,j,k) = -grav*(u_str(:,k+1) - u_str(:,k))/
     *			     (plgl2_c(:,i,j,k+1) - plgl2_c(:,i,j,k))
	      if(do_write) write(6,40) k,u_tend_gwd(:,ic,i,j,k)
	    enddo
	    u_src(:,ic,i,j,:,ksdm) = u_src(:,ic,i,j,:,ksdm)
     *             + u_tend_gwd(:,ic,i,j,:)

	  enddo		! do ic = 1,ncorners

	enddo		! do i = 1,iim
	enddo		! do j = 1,jjm

 10	format(// ' ======================================================'/
     *            '     GRAVITY WAVE DIAGNOSTICS FOR A SINGLE PROFILE' /
     *            ' ======================================================'/
     *		  '     (iw, i, j, k) = ',4i4 /)
 20	format( ' stresses at surface '/
     *		'   topographic variance : ', e13.3, ' m**2' / 
     *		'   stress in zonal dir  : ', e13.3, ' N/m**2' /
     *		'   stress in merid dir  : ', e13.3, ' N/m**2' /
     *		'   mag of surface stress: ', e13.3, ' N/m**2' /
     *		'   zonal velocity       : ', e13.3, ' m/s '/
     *		'   merid velocity       : ', e13.3, ' m/s '/
     *		'   density              : ', e13.3, ' kg/m**3' /
     *          '   bouyancy             : ', e13.3, ' s**-1 '/ )
 29	format( /  ' STRESS as a function of vertical level k '/
     *		    '    k     un_str       ut_str        ri          rmin ')
 30	format( i4,4e13.3 )
 39 	format( /  ' STRESS convergence as a function of level k'/
     *              '    k    u_tend       v_tend     ')
 40	format( i4,3e13.3 )
 41	format( //)

	end subroutine gwd
!===========================================================================
! END OF GWD
!===========================================================================

	end module gravity_wave_drag_module
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
