!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module hstatc

!-----------------------------------------------------------------------
! PURPOSE: hstatc contains a series of independent subroutines related
!   to hydrostatics, computing vertical velocity, pressure, and layer
!   edge quantities
!-----------------------------------------------------------------------

	use kinds
	use physical_parameters
        use model_parameters
	use domain_decomposition
	use eau_util
	use sigma_coordinate
#ifndef no_pbl
	use pbl_parameters
#endif

	implicit none
        private
	save

	integer (kind=int_kind) :: k

	public ::
     &     get_dsigdt,
     &     get_alphasig,
     &     get_waterl2,
     &     get_tracerl2,
     &     get_geopotential,
     &     press,
     &     get_pthick,
     &     get_h
#ifdef eau_ng
     *,    get_elqngl2
#endif


!-----------------------------------------------------------------------
! VARIABLES COMMONLY USED:
!-----------------------------------------------------------------------
! note: all variables ending with l2 are defined at the layer edges
!-----------------------------------------------------------------------
!	ps	    =  surface pressure @ current time (Pa)
!	pb	    =  pressure @ pbl top @ current time (Pa), only 
!		         applicable to generalized sigma version
!	ps_f	    =  time tendency of ps (Pa/s)
!	pb_f	    =  time tendency of pb (Pa/s)
!       div_mass_flux    =  mass flux divergence 
!	entrainment =  entrainment of air into pbl (kg/s/m2)
!	cumulusmf   =  mass flux out of pbl due to cumulus towers (kg/s/m2)
!	th          =  potential temperature at layer center (K)
!	tracer 	    =  tracer fields @ current time (units vary)
!	phis        =  surface geopotential (m2/s2)
!	pll2        =  pressure at layer edges (Pa)
!	pl	    =  pressure at layer centers (Pa)
!       bpl2        =  Pfunction at layer edges, 
!			 (pressure/reference pressure)**kappa (unitless)
!       bp          =  Pfunction averaged to layer centers, 
!			 (Arakawa Lamb, 1977, eq 250) (unitless)
!	thl2        =  potential temperature averaged to layer edges (K)
!	dsigdt      =  time tendency of sigma surfaces * pthick (Pa/s)
!	alphasig    =  product of specific volume and sigma (m**3/kg)
!	alpha	    =  specific volume (m**3/kg)
!	phi         =  geopotential at layer center (m2/s2)
!	tracerl2    =  tracers averaged to layer edges (units vary)
!	pthick      =  pressure thickness (Pa), (Arakawa's Pi)
#ifdef eau_ng
!     elqngl2 = elqng variables defined at layer edges.
#endif
!-----------------------------------------------------------------------

!*************************************************************************

        contains

!*************************************************************************

!======================================================================
! BEGINNING OF GET_DSIGDT
!======================================================================
        subroutine get_dsigdt(
#ifndef no_pbl
     &                  entrainment,
     &                  cumulusmf,
     &                  pleak,
#endif
     &                  div_mass_flux, 
     &			dsigdt   )

!----------------------------------------------------------------------------
! written for Phillips sigma coordinate 3-15-98, tdr
! option for generalized sigma coordinate added 5-20-98, tdr
! checked, 1-22-98, tdr
! converted to F90 module, 08-09-00 tdr
!----------------------------------------------------------------------------

        implicit none

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
#ifndef no_pbl
        real (kind=dbl_kind), dimension(iim,jjm), intent(in) ::
     &                  entrainment,
     &                  cumulusmf,
     &			pleak
#endif

        real (kind=dbl_kind), dimension(iim,jjm,nlm), intent(in) ::
     &			div_mass_flux

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nlmp1), intent(out) :: 
     &			dsigdt

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm) ::
     &			ps_f
#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm) ::
     &			pb_f
#endif

	real (kind=dbl_kind), dimension(iim,jjm,nlm) :: 
     &			work

	integer (kind=int_kind) :: ipbl_l2

!---------------------------------------------------------------------------
! find the layer edge that corresponds to the top of the pbl
!---------------------------------------------------------------------------
        ipbl_l2 = nstrat + ntrop + 1

!---------------------------------------------------------------------------
! sum up the mass flux divergnce
!---------------------------------------------------------------------------
	work = c0
 	work(:,:,1) = div_mass_flux(:,:,1)*dsig(1)
 	do k=2,nlm
 	  work(:,:,k) = work(:,:,k-1) + div_mass_flux(:,:,k)*dsig(k)
 	enddo
	
!---------------------------------------------------------------------------
! specify boundary conditions (as a part of the initialization)
!---------------------------------------------------------------------------
	dsigdt     = c0

!---------------------------------------------------------------------------
! compute time tendency of ps and pb
!---------------------------------------------------------------------------
	ps_f(:,:) = -work(:,:,nlm)

#ifndef no_pbl

!---------------------------------------------------------------------------
! the vertical velocity at the pbl is specified by parameterized velocities
!---------------------------------------------------------------------------
	dsigdt(:,:,ipbl_l2) = grav *
     &              (entrainment(:,:) - cumulusmf(:,:) + pleak(:,:))

!---------------------------------------------------------------------------
! find the contribution to the pbl top pressure from the convergence of
!   mass integrated from the model top to the pbl top
!   also add in the tendency due to mass flux across the pbl top
!---------------------------------------------------------------------------
	pb_f(:,:) =   - work(:,:,nstrat+ntrop) 
     &                - dsigdt(:,:,ipbl_l2)
#endif

!---------------------------------------------------------------------------
! start at the top and integrate down
!---------------------------------------------------------------------------
#ifndef no_stratosphere
        do k=1,nstrat
          dsigdt(:,:,k+1) =  -work(:,:,k)
        enddo
#endif

#ifndef no_pbl

!---------------------------------------------------------------------------
! integrated down to the sigma level just above the pbl
!---------------------------------------------------------------------------
        do k=nstrat+1,nstrat+ntrop-1
          dsigdt(:,:,k+1) = - work(:,:,k) - pb_f(:,:) * sigl2(k+1)
        enddo

!---------------------------------------------------------------------------
! integrate thru the pbl down to the surface
!---------------------------------------------------------------------------
        do k=nstrat+ntrop+1,nlm-1
          write(6,*) ' check hstatc:dsigdt:inside pbl'
          dsigdt(:,:,k+1) =  -work(:,:,k) - (c2-sigl2(k+1))*pb_f(:,:)
     *                                    - (sigl2(k+1)-c1)*ps_f(:,:)
        enddo

#else

        do k=nstrat+1,nlm
          dsigdt(:,:,k+1) =  -work(:,:,k) - ps_f(:,:) * sigl2(k+1)
        enddo

#endif

        end subroutine get_dsigdt
!===========================================================================
! END OF GET_DSIGDT
!===========================================================================

	
 
!===========================================================================
!  START OF GET_ALPHASIG
!===========================================================================
	subroutine get_alphasig(
     &                 pll2,
     &                 pl,
     &                 bpl2,
     &                 bp,
     &                 th,
     &                 alphasig,
     &                 alpha)
 
!---------------------------------------------------------------------------
! PURPOSE:
! compute the product of specific volume and sigma to be used in
! computing the terms related to the horizontal pressure gradient force
! compute the specific volume
!---------------------------------------------------------------------------
 
        implicit none
 
!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(iim,jjm,nlm),   intent(in) :: 
     &			bp,
     &			th,
     &			pl

	real (kind=dbl_kind), dimension(iim,jjm,nlmp1), intent(in) :: 
     &			pll2,
     &			bpl2
 
!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nlm),  intent(out) :: 
     &			alphasig,
     &			alpha
 
!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm) :: 
     &			work1, 
     &			work2, 
     &			work3

!---------------------------------------------------------------------------
! compute the product of the specific volume * sigma (alphasig)
! this is used in the evaluation of the pressure gradient force
! it is defined at the layer centers
! also compute the specific volume
!---------------------------------------------------------------------------

	alpha(:,:,:) = gas_const_R*th(:,:,:)*bp(:,:,:)/pl(:,:,:)

        do k=1,nlm
          work1(:,:)      = (bpl2(:,:,k+1) - bp(:,:,k))*sigl2(k+1)
          work2(:,:)      = (bp(:,:,k) - bpl2(:,:,k))*sigl2(k)
	  work3(:,:)      = (pll2(:,:,k+1) - pll2(:,:,k))
          alphasig(:,:,k) = spec_heat_cp*th(:,:,k)*(work1+work2)/work3
        enddo

!---------------------------------------------------------------------------
! special consideration must be given to alpha and alphasig in the PBL
! region, see Suarez et al. 1983, MWR
!---------------------------------------------------------------------------
#ifndef no_pbl
!	k=nlm
        do k=nstrat+ntrop+1,nlm
	  work1(:,:)      =          spec_heat_cp*th(:,:,k)/
     &		               (pll2(:,:,k+1) - pll2(:,:,k))
          alphasig(:,:,k) = work1*(bpl2(:,:,k+1)-bp(:,:,k))
          alpha(:,:,k)    = work1*(bpl2(:,:,k+1)-bpl2(:,:,k))

        enddo
#endif

	end subroutine get_alphasig
!===========================================================================
!  END OF GET_ALPHASIG
!===========================================================================


!===========================================================================
!  START OF GET_TRACERL2
!===========================================================================
	subroutine get_tracerl2(
     &                bp, 
     &                bpl2,
     &                tracer,
     &                tracerl2,
     &                isize,jsize,nlev,npblloc,
     &                interp_type)

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------
! average water quantities to layer edges by mdb 9-13-02
! this is needed to compute the vertical advection terms
! modified version of get_avg originally written by tdr 5-20-98
!---------------------------------------------------------------------------
 
        implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: 
     &    isize       ! array size
     &,   jsize       ! array size
     &,   nlev        ! number of vertical levels
     &,   npblloc     ! number of pbl layers
        real (kind=dbl_kind), dimension(isize,jsize,nlev), intent(in) :: 
     &    bp     ! odd-level pressure function                       (-).

	real (kind=dbl_kind), dimension(isize,jsize,nlev+1), intent(in) ::
     &    bpl2   ! even-level pressure function                      (-).

        real (kind=dbl_kind), dimension(isize,jsize,nlev), intent(in) :: 
     &    tracer ! odd-level tracer fields @ current time   (units vary).
        character(*), intent(in) :: interp_type  ! interpolation type

!  valid interpolation types:
!  ARITHMETIC - arithmetic mean
!  GEOMETRIC - geometric mean
!  THETA - interpolation weights from Suarez and Arakawa 1983 eq 58
!          used in the hydrostatic eqn for energy conservation
!  LOGARITHMIC - Conserves distribution of logarithm,
!                from Arakawa and Lamb 1977, eq 327
!  QSAT - weights traditionally used for QSAT in lat-lon model

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(isize,jsize,nlev+1), 
     &         intent(out) :: 
     &    tracerl2 ! even-level tracer fields @ current time   (units vary).

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
	integer (kind=int_kind) :: 
     &    nlevp1  ! number of vertical levels plus one (sfc level)

	real (kind=dbl_kind), dimension(isize,jsize) ::
     &    aile   ! interpolation weights
     &,   bile   ! interpolation weights

	real (kind=dbl_kind), dimension(isize,jsize) :: 
     &    tracerbp  ! work array

        nlevp1 = nlev+1

!---------------------------------------------------------------------------
! arbitrarily set *l2 @ k=1 and k=nlmp1
!---------------------------------------------------------------------------
        tracerl2    (:,:,1)       =  tracer    (:,:,1)
        tracerl2    (:,:,nlev+1)   =  tracer    (:,:,nlev)


!---------------------------------------------------------------------------
! choose the interpolation case
!---------------------------------------------------------------------------
        select case ( trim(interp_type) )

           case ("ARITHMETIC")

              do k=2,nlev
                 tracerl2(:,:,k) = 0.5_dbl_kind*( tracer(:,:,k-1)
     +                                         +  tracer(:,:,k) )
              enddo

           case ("GEOMETRIC")

              do k=2,nlev
                 tracerl2(:,:,k) = sqrt( tracer(:,:,k-1)*tracer(:,:,k) )
              enddo

           case ("LOGARITHMIC")

              do k=2,nlev
                 call logint(tracer, tracerl2,
     +                       isize, jsize, nlev, k )
              enddo

          case ("THETA")

              do k=2,nlev
                 aile(:,:) = (bpl2(:,:,k)-bp(:,:,k-1))
     +                   / (bp(:,:,k)-bp(:,:,k-1))
                 bile(:,:) = c1-aile(:,:)
                 tracerl2(:,:,k)   =  aile(:,:)*tracer(:,:,k-1)
     +                              + bile(:,:)*tracer(:,:,k)
              enddo

           case ("QSAT")

              do k=2,nlev
                 aile(:,:) = (bpl2(:,:,k)-bp(:,:,k-1))
     +                / (bp(:,:,k)-bp(:,:,k-1))
                 bile(:,:) = c1-aile(:,:)
                 bile(:,:) = c1+bp(:,:,k)*aile(:,:)/(bp(:,:,k-1)*bile(:,:))
                 bile(:,:) = c1/bile(:,:)
                 aile(:,:) = c1-bile(:,:)
                 tracerl2(:,:,k)   =  aile(:,:)*tracer(:,:,k-1)
     +                              + bile(:,:)*tracer(:,:,k)
              enddo

           case default

	      print *,'no valid interpolation selected for get_tracerl2'
              stop

        end select
 
#ifdef no_pbl
!---------------------------------------------------------------------------
! at surface get the value 
! by extrapolation from layers above
!---------------------------------------------------------------------------
          call get_bplus(bp, bpl2, tracer, tracerbp, 
     &                   isize, jsize, nlev, npblloc,
     &                   interp_type, c0)
#else
!---------------------------------------------------------------------------
! at pbl top get the value immediately above the pbl top (B+)
! by extrapolation from layers above
!---------------------------------------------------------------------------
        if (trim(interp_type) == "THETA") then
          call get_bplus(bp, bpl2, tracer, tracerbp, 
     &                   isize, jsize, nlev, npblloc,
     &                   interp_type, fac_extrapt)
        else
          call get_bplus(bp, bpl2, tracer, tracerbp, 
     &                   isize, jsize, nlev, npblloc,
     &                   interp_type, c0)
        endif
#endif
        tracerl2(:,:,nlev-npblloc+1) =  tracerbp(:,:)

	end subroutine get_tracerl2
!===========================================================================
!  END OF GET_TRACERL2
!===========================================================================


!===========================================================================
!  START OF GET_WATERL2
!===========================================================================
	subroutine get_waterl2(
     &                pll2, 
     &                bp, 
     &                bpl2,
     &                ssl,  
     &                ssll2, 
     &                qs, 
     &                qsl2,
     &                phi, 
     &                phil2,
     &                th,  
     &                thl2,  
     &                q,
     &                tl2,
     &                ql2,
     &                isize,jsize,nlev,npblloc)

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------
! Use the method of Arakawa and Lamb (1977) to get layer edge temperature,
!   water vapor, qsat and dry static energy such that 'CICK' is inhibited.
!   Above sigma=0.1 interpolation of the logarithm is used - by dd 9-16-02
! average water quantities to layer edges by mdb 9-13-02
! this is needed to compute the vertical advection terms
! modified version of get_avg originally written by tdr 5-20-98
!---------------------------------------------------------------------------
 
        implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: 
     &    isize       ! array size
     &,   jsize       ! array size
     &,   nlev        ! number of vertical levels
     &,   npblloc     ! number of pbl layers
        real (kind=dbl_kind), dimension(isize,jsize,nlev), intent(in) :: 
     &    bp     ! odd-level pressure function                       (-).
     &,   th     ! odd-level potential temperatures                  (K).
     &,   q      ! odd-level water vapor @ current time          (kg/kg).
     &,   phi    ! odd-level geopotential                         (J/kg).
     &,   qs     ! odd-level saturation mixing ratios            (kg/kg).

	real (kind=dbl_kind), dimension(isize,jsize,nlev+1), intent(in) ::
     &    pll2   ! even-level pressures                             (Pa).
     &,   bpl2   ! even-level pressure function                      (-).

        real (kind=dbl_kind), dimension(isize,jsize,nlev+1),intent(in) :: 
     &    thl2   ! even-level potential temperatures                 (K).
     &,   phil2  ! even-level geopotential                        (J/kg).
 
!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(isize,jsize,nlev+1), 
     &         intent(out) :: 
     &    ql2     ! even-level water vapor @ current time        (kg/kg).

	real (kind=dbl_kind), dimension(isize,jsize,nlev),intent(inout) ::
     &    ssl     ! odd-level dry static energy                    (J/kg).

	real (kind=dbl_kind), dimension(isize,jsize,nlev+1),intent(inout) ::
     &    qsl2    ! even-level saturation mixing ratios           (kg/kg).
     &,   ssll2   ! even-level dry static energy                   (J/kg).
     &,   tl2     ! even-level temperature                            (K).

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
	integer (kind=int_kind) :: 
     &    nlevp1  ! number of vertical levels plus one (sfc level)

	real (kind=dbl_kind), dimension(isize,jsize,nlev) :: 
     &    work1,  ! work array
     &    work2,  ! work array
     &    aile,   ! interpolation weights
     &    bile    ! interpolation weights

	real (kind=dbl_kind), dimension(isize,jsize) :: 
     &    qbp  ! work array

        nlevp1 = nlev+1

!---------------------------------------------------------------------------
! compute odd-level dry static energy
!---------------------------------------------------------------------------
	work1(:,:,:) = th(:,:,:) * bp(:,:,:)  ! odd-level temperature
        ssl(:,:,:) = spec_heat_cp * work1(:,:,:) + phi(:,:,:)

!---------------------------------------------------------------------------
! arbitrarily set *l2 @ k=1 and k=nlmp1
!---------------------------------------------------------------------------
	ql2(:,:,1)     =  q(:,:,1)
        ql2(:,:,nlev+1) =  q(:,:,nlev)
	qsl2(:,:,1)     =  qs(:,:,1)
	ssll2(:,:,1)     =  ssl(:,:,1)
	tl2(:,:,1)     =  th(:,:,1) * bpl2(:,:,1)
	tl2(:,:,nlev+1)     =  th(:,:,nlev) * bpl2(:,:,nlev+1)
        call eau_sat(isize*jsize, pll2(:,:,nlev+1), tl2(:,:,nlev+1),
     &               qsats=qsl2(:,:,nlev+1))
 
!---------------------------------------------------------------------------
! compute even-level temperature (SA83 eqn 42)
!---------------------------------------------------------------------------
        do k = 2,nlev
          tl2(:,:,k) = (phi(:,:,k-1)-phil2(:,:,k))/spec_heat_cp
     &                      + bp(:,:,k-1)*thl2(:,:,k)
        enddo
!---------------------------------------------------------------------------
! compute even-level dry static energy (AL77, eqn after 356)
!---------------------------------------------------------------------------
        do k = 2,nlev+1
          ssll2(:,:,k) = ssl(:,:,k-1) + spec_heat_cp*bp(:,:,k-1)
     &                       * (thl2(:,:,k) - th(:,:,k-1))
        enddo

!---------------------------------------------------------------------------
! interpolate even-level saturation water vapor mixing ratio
!---------------------------------------------------------------------------
        do k = 2,nlev
           aile(:,:,k) = (bpl2(:,:,k)-bp(:,:,k-1))
     +             / (bp(:,:,k)-bp(:,:,k-1))
           bile(:,:,k) = c1-aile(:,:,k)
           bile(:,:,k) = c1+bp(:,:,k)*aile(:,:,k)/(bp(:,:,k-1)*bile(:,:,k))
           bile(:,:,k) = c1/bile(:,:,k)
           aile(:,:,k) = c1-bile(:,:,k)
           qsl2(:,:,k) = aile(:,:,k)*qs(:,:,k-1)+bile(:,:,k)*qs(:,:,k)
        enddo

!---------------------------------------------------------------------------
! compute odd-level relative humidity
!---------------------------------------------------------------------------
        work1(:,:,:) = q(:,:,:) / qs(:,:,:) 
	where(work1.gt.c1) work1 = c1
	where(work1.lt.c0) work1 = c0

!---------------------------------------------------------------------------
! compute even-level relative humidity (AL77 eqn 354)
!  and from that, the even-level water vapor mixing ratio
!---------------------------------------------------------------------------
        do k = 2,nlev 
!         work2(:,:,k) = 2.0_dbl_kind-work1(:,:,k)-work1(:,:,k-1)
!         where(work2(:,:,k) .ne. c0) 
!           work2(:,:,k) =
!    &                   (work1(:,:,k)+work1(:,:,k-1)-
!    &                   2._dbl_kind*work1(:,:,k)*work1(:,:,k-1))
!    &                   / work2(:,:,k)
!         elsewhere
!           work2(:,:,k) = c1
!         endwhere
          work2(:,:,k) = 0.5 * (work1(:,:,k) + work1(:,:,k-1))
          ql2(:,:,k) = work2(:,:,k) * qsl2(:,:,k)
!          ql2(:,:,k) = sqrt( q(:,:,k-1)*q(:,:,k) )
!          ql2(:,:,k) = 0.5*(q(:,:,k-1)+q(:,:,k))
!          call logint(q(1,1,1), work3(1,1,1), isize, jsize, nlev, k )
! AL77 eqn 358
!          ql2(:,:,k) = work2(:,:,k)*ql2(:,:,k)+(c1-work2(:,:,k))*work3(:,:,k)
        enddo

!---------------------------------------------------------------------------
! use interpolation of the logarithm to get water vapor and qsat at layer
!   edges above sigma = 0.1
!---------------------------------------------------------------------------
         do k = 2,nltp1-(nlm-nlev)
            call logint(qs(1,1,1), qsl2(1,1,1), isize, jsize, nlev, k )
            call logint(q(1,1,1), ql2(1,1,1), isize, jsize, nlev, k )
         enddo


!!---------------------------------------------------------------------------
!! compute pbl top values of the output variables
!!---------------------------------------------------------------------------
!        ssll2(:,:,nlev-npblloc+1) =  ssl(:,:,nlev-npblloc)
!        tl2(:,:,nlev-npblloc+1) = 
!     &  (ssll2(:,:,nlev-npblloc+1) - phil2(:,:,nlev-npblloc+1)) / spec_heat_cp

#ifdef no_pbl
        call get_bplus_q(bp, bpl2, pll2, q,
     &                 qbp, qs, tl2(:,:,nlev-npblloc+1),
     &                 isize, jsize, nlev, npblloc,
     &                 c0)
#else
        call get_bplus_q(bp, bpl2, pll2, q,
     &                 qbp, qs, tl2(:,:,nlev-npblloc+1),
     &                 isize, jsize, nlev, npblloc,
     &                 fac_extrapq)
#endif
        ql2(:,:,nlev-npblloc+1) =  qbp(:,:)

	end subroutine get_waterl2
!===========================================================================
!  END OF GET_WATERL2
!===========================================================================

        subroutine logint(x, xl2, isize, jsize, nlev, k )
!---------------------------------------------------------------------------
! This subroutine computes a logarithmic interpolation according to AL77 eqn 327
!---------------------------------------------------------------------------

! argument list declarations
        integer (kind=int_kind), intent(in) ::
     &       isize, jsize, nlev, k
        real (kind=dbl_kind), dimension(isize,jsize,nlev), intent(in) ::
     &                              x ! input mid-layer variable
        real (kind=dbl_kind), dimension(isize,jsize, nlev+1), intent(out) ::
     &                              xl2 ! output mid-layer variable

! local variables
        real (kind=dbl_kind), dimension(isize,jsize) :: diffx  !work variable

        where(x(:,:,k-1).ne.0.0_dbl_kind. and .x(:,:,k).ne.0.0_dbl_kind)
           xl2(:,:,k) = log( x(:,:,k-1)/x(:,:,k) )
        elsewhere
           xl2(:,:,k) = 0.0_dbl_kind
        endwhere
        diffx(:,:) = x(:,:,k-1) - x(:,:,k)
        where(abs(diffx(:,:)).gt.1.e-15_dbl_kind)
           xl2(:,:,k) = xl2(:,:,k)*x(:,:,k-1)*x(:,:,k)/diffx(:,:)
        elsewhere
           xl2(:,:,k) = x(:,:,k-1)
        endwhere


        end subroutine logint

#ifdef eau_ng
!===========================================================================
!  START OF GET_ELQNGL2
!===========================================================================
      subroutine get_elqngl2(
     *                   dsigdt,
     *                   elqng,
     *                   elqngl2,
     *                   isize,jsize,nlev)

      implicit none

!---------------------------------------------------------------------------
! PURPOSE : compute the elqng variables at layer edges.
!           Laura D. Fowler/mcbug (12-01-01)
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: 
     &    isize       ! array size
     &,   jsize       ! array size
     &,   nlev        ! number of vertical levels

      real (kind=dbl_kind), dimension(isize,jsize,nlev,nelqng), intent(in)::
     &  elqng

      real (kind=dbl_kind), dimension(isize,jsize,nlev+1), intent(in)::
     &  dsigdt

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------

      real (kind=dbl_kind), dimension(isize,jsize,nlev+1,nelqng)
     *,    intent(out) ::
     *  elqngl2

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------

      integer (kind=int_kind)::
     * i, ijm, ielq

      ijm = isize*jsize

!---------------------------------------------------------------------------
! arbitrarily set *l2 @ k=1 and k=nlmp1
!---------------------------------------------------------------------------

      elqngl2(:,:,1,:)     = elqng(:,:,1,:)
      elqngl2(:,:,nlmp1,:) = elqng(:,:,nlm,:)

!----------------------------------------------------------------------
! use the upstream value of elqng for the cloud fraction
!----------------------------------------------------------------------

      do k = 2, nlm
         do i = 1, ijm
            if(dsigdt(i,1,k) .gt. c0) then
               elqngl2(i,1,k,1) = elqng(i,1,k-1,1)
            else
               elqngl2(i,1,k,1) = elqng(i,1,k,1)
            endif
         enddo
      enddo

!----------------------------------------------------------------------
! other elqng variables 
!----------------------------------------------------------------------

      do ielq = 2, nelqng
         do k = 2, nlm
            do i = 1, ijm
               elqngl2(:,:,k,ielq) =  p5*(elqng(:,:,k-1,ielq)
     &                              +  elqng(:,:,k,ielq))
            enddo
         enddo
      enddo

      end subroutine get_elqngl2
      
!===========================================================================
!  END OF GET_ELQNGL2
!===========================================================================
#endif

!===========================================================================
!  START OF GET_GEOPOTENTIAL
!===========================================================================
	subroutine get_geopotential(
     &                   bpl2,
     &                   bp,
     &                   phis,
     &                   th,
     &                   thl2,
     &                   sigl2loc,
     &                   phi,
     &                   phil2,
     &                   isize,jsize,nlev)

	implicit none

!---------------------------------------------------------------------------
! PURPOSE : compute the geopential, tdr 5-20-98
!---------------------------------------------------------------------------
 
!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: isize,jsize,nlev
        real (kind=dbl_kind), dimension(isize,jsize),      intent(in) :: 
     &    phis   ! surface geopotential                        (m^2/s^2).

        real (kind=dbl_kind), dimension(isize,jsize,nlev),  intent(in) :: 
     &    bp     ! odd-level pressure function                       (-).
     &,   th     ! odd-level potential temperatures                  (K).

	real (kind=dbl_kind), dimension(isize,jsize,nlev+1),intent(in) :: 
     &    bpl2   ! even-level pressure function                      (-).
     &,   thl2   ! even-level potential temperatures                 (K).
 
	real (kind=dbl_kind), dimension(nlev+1),intent(in) :: 
     &    sigl2loc  ! sigma values at layer edges                    (-).

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(isize,jsize,nlm) :: 
     &    phi    ! odd-level geopotentials                     (m^2/s^2). 

        real (kind=dbl_kind), dimension(isize,jsize,nlev+1) :: 
     &    phil2  ! even-level geopotentials                    (m^2/s^2). 

!---------------------------------------------------------------------------
! LOCAL VARIABLES
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(isize,jsize) :: 
     &    work1   ! work array
     &,   aile    ! interpolation weights
     &,   bile    ! interpolation weights

	real (kind=dbl_kind) :: 
     &    isige   ! inverse of sigmas 

	real (kind=dbl_kind), dimension(nlev) :: 
     &    dsigloc ! sigma value differences

	integer (kind=int_kind) ::
     &    ntop    ! top-level of column to integrate to

	integer (kind=int_kind) :: 
     &    nlevp1  ! number of vertical levels plus one (sfc level)

        nlevp1 = nlev+1
        do k=1,nlev
          dsigloc(k) = sigl2loc(k+1) - sigl2loc(k)
        enddo

!---------------------------------------------------------------------------
! integrate from bottom up
!---------------------------------------------------------------------------

! mid layer
        work1(:,:) = (bpl2(:,:,nlevp1)-bp(:,:,nlev))
        phi(:,:,nlev) = phis(:,:) + spec_heat_cp*th(:,:,nlev)*work1(:,:)
#ifdef no_pbl
        do k=nlev-1,1,-1
         work1(:,:) = (bp(:,:,k+1) - bp(:,:,k))
         phi(:,:,k) = phi(:,:,k+1) + spec_heat_cp*thl2(:,:,k+1)*work1(:,:)
        enddo

#else
! pbl top, must get appropriate thl2 first
        aile(:,:) = (bpl2(:,:,nlev)-bp(:,:,nlev-1))
     +             / (bp(:,:,nlev)-bp(:,:,nlev-1))
        bile(:,:) = c1-aile(:,:)

! interpolated th
        work1(:,:) =aile(:,:)*th(:,:,nlev-1) + bile(:,:)*th(:,:,nlev)
        work1(:,:) = (bp(:,:,nlev) - bp(:,:,nlev-1))*work1(:,:)
        phi(:,:,nlev-1) = phi(:,:,nlev) + spec_heat_cp*work1(:,:)
        do k=nlev-2,1,-1
         work1(:,:) = (bp(:,:,k+1) - bp(:,:,k))
         phi(:,:,k) = phi(:,:,k+1) + spec_heat_cp*thl2(:,:,k+1)*work1(:,:)
        enddo

#endif

! layer edges
        phil2(:,:,nlevp1) = phis(:,:)
	ntop = nstrat
#ifdef no_pbl
        do k=nlev,ntop+2,-1
         isige = 1.0 / sigl2loc(k)
         work1(:,:) = spec_heat_cp * th(:,:,k) 
     &        *(sigl2loc(k)*(bp(:,:,k)-bpl2(:,:,k))
     &        +sigl2loc(k+1)*(bpl2(:,:,k+1)-bp(:,:,k)))
         phil2(:,:,k) = (phil2(:,:,k+1)*sigl2loc(k+1) -
     &       dsigloc(k)*phi(:,:,k) + work1(:,:))*isige
        enddo
#else
        work1(:,:  ) = (bpl2(:,:,nlevp1)-bpl2(:,:,nlev))
        phil2(:,:,nlev)   = phis(:,:) + spec_heat_cp*th(:,:,nlev)*work1(:,:)


!-- if called by sfc_control, reset ntop (because only doing for the
!-- lowest npbl+2 layers)
        if (nlm /= nlev) ntop = 0

        do k=nlev-1,ntop+2,-1
         isige = 1.0 / sigl2loc(k)
         work1(:,:) = spec_heat_cp * th(:,:,k) 
     &        *(sigl2loc(k)*(bp(:,:,k)-bpl2(:,:,k))
     &        +sigl2loc(k+1)*(bpl2(:,:,k+1)-bp(:,:,k)))
         phil2(:,:,k) = (phil2(:,:,k+1)*sigl2loc(k+1) -
     &       dsigloc(k)*phi(:,:,k) + work1(:,:))*isige
        enddo
#endif

        do k=ntop+1,2,-1
         work1(:,:) = (bp(:,:,k) - bpl2(:,:,k))
         phil2(:,:,k) = phi(:,:,k) 
     &           + spec_heat_cp*thl2(:,:,k)*work1(:,:)
        enddo

	end subroutine get_geopotential
!===========================================================================
!  END OF GET_GEOPOTENTIAL
!===========================================================================


!===========================================================================
!  START OF PRESS
!===========================================================================
	subroutine press(
     &               pll2,
     &               pl,
     &               bpl2,
     &               bp,
     &               ps,
#ifndef no_pbl
     &               pb,
#endif
     &               isize,jsize)

!---------------------------------------------------------------------------
! PURPOSE:
! compute the pressure at layer edges and layer center
! compute the nondimensional Pfnc at layer egdes and layer center
! tdr 5-20-98
!---------------------------------------------------------------------------

        implicit none
 
!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: isize,jsize
        real (kind=dbl_kind), dimension(isize,jsize), intent(in) :: 
     &    ps        ! surface pressure                               (Pa).
#ifndef no_pbl
        real (kind=dbl_kind), dimension(isize,jsize), intent(in) :: 
     &    pb        ! PBL-top pressure                               (Pa).
#endif

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(isize,jsize,nlmp1), intent(out) :: 
     &    pll2   ! even-level pressures                             (Pa).
     &,   bpl2   ! even-level pressure function                      (-).

	real (kind=dbl_kind), dimension(isize,jsize,nlm),   intent(out) :: 
     &    pl     ! odd-level pressures                                (Pa).
     &,   bp     ! odd-level pressure function                         (-).
 
!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(isize,jsize,nlmp1) :: 
     &    work   ! work array
	real (kind=dbl_kind)  :: 
     &    f1, f2, f4, f5    ! mathematical quantities
 
!---------------------------------------------------------------------------
! compute some constants
!---------------------------------------------------------------------------
	f1 = c1/(c1+kappa)
	f2 = c1/(p0_sfc**kappa)
	f4 = c1/kappa
	f5 = c1/f2
	work = c0
 
!-----------------------------------------------------------------------
! compute the pressure at the layer edges, pll2 (Pa)
!---------------------------------------------------------------------------
#ifndef no_pbl

#ifdef no_stratosphere
        do k=1,ntrop+1
          pll2(:,:,k) = (pb(:,:) - p_top) * sigl2(k) + p_top
        enddo
        do k=ntrop+2,nlmp1
          pll2(:,:,k) = (ps(:,:) - pb(:,:)) * (sigl2(k)-c1) + pb(:,:)
        enddo
#else
	do k=1,nstrat+1
	  pll2(:,:,k) = (p_interface - p_top) * sigl2(k) + p_interface
	enddo
	do k=nstrat+2,nstrat+ntrop+1
	  pll2(:,:,k) = (pb(:,:) - p_interface) * sigl2(k) + p_interface
	enddo
	do k=nstrat+ntrop+2,nlmp1
	  pll2(:,:,k) = (ps(:,:) - pb(:,:)) * (sigl2(k)-c1) + pb(:,:)
	enddo
#endif

#else

#ifdef no_stratosphere
        do k=1,ntrop+1
          pll2(:,:,k) = (ps(:,:) - p_top) * sigl2(k) + p_top
        enddo
#else
	do k=1,nstrat+1
	  pll2(:,:,k) = (p_interface - p_top) * sigl2(k) + p_interface
	enddo
	do k=nstrat+2,nstrat+ntrop+1
	  pll2(:,:,k) = (ps(:,:) - p_interface) * sigl2(k) + p_interface
	enddo
#endif
#endif
 
!--------------------------------------------------------------------------
! compute the Pfunction at the layer edges and at layer center, bpl2, bp
! compute the pressure at the layer center, pl (Pa)
!---------------------------------------------------------------------------
	do k=1,nlmp1
 	  bpl2(:,:,k) =  f2 * exp(kappa*log(pll2(:,:,k)))
          work (:,:,k) =  f5 * bpl2(:,:,k) * pll2(:,:,k)
	enddo
	do k=1,nlm
	  bp(:,:,k) = f1 * f2 * (work(:,:,k+1)-work(:,:,k)) / 
     &                         (pll2(:,:,k+1) - pll2(:,:,k))
 	  pl(:,:,k) = p0_sfc * exp(f4*log(bp(:,:,k)))
	enddo
 
	end subroutine press
!===========================================================================
!  END OF PRESS
!===========================================================================


!===========================================================================
!  START OF GET_PTHICK
!===========================================================================
	subroutine get_pthick(
     &                  ps,
#ifndef no_pbl
     &                  pb,
#endif
     &                  pthick,
     &                  invpthick)

!---------------------------------------------------------------------------
! PURPOSE
!  compute the pressure thickness of various layers, Arakawa's PI
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm),     intent(in)  :: 
     &				pb
#endif
	real (kind=dbl_kind), dimension(iim,jjm),     intent(in)  :: 
     &				ps 
 
!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nlm), intent(out) :: 
     &				pthick,
     &				invpthick
 
!---------------------------------------------------------------------------
! if pure sigma, pthick == surface pressure
! if generalized sigma, pthick = {stratosphere,troposphere,pbl} thickness
!---------------------------------------------------------------------------

#ifdef no_pbl

#ifdef no_stratosphere
        do k=1,ntrop
          pthick(:,:,k) = ps(:,:) - p_top
        enddo
#else
        do k=1,nstrat
          pthick(:,:,k) = p_interface - p_top
        enddo
        do k=nstrat+1,nstrat+ntrop
          pthick(:,:,k) = ps(:,:) - p_interface
        enddo
#endif

#else

#ifdef no_stratosphere
        do k=1,ntrop
          pthick(:,:,k) = pb(:,:) - p_top
        enddo
        do k=ntrop+1,nlm
          pthick(:,:,k) = ps(:,:) - pb(:,:)
        enddo
#else
        do k=1,nstrat
          pthick(:,:,k) = p_interface - p_top
        enddo
        do k=nstrat+1,nstrat+ntrop
          pthick(:,:,k) = pb(:,:) - p_interface
        enddo
        do k=nstrat+ntrop+1,nlm
          pthick(:,:,k) = ps(:,:) - pb(:,:)
        enddo
#endif

#endif


	invpthick = c1 / pthick

	end subroutine get_pthick
!===========================================================================
!  END OF GET_PTHICK
!===========================================================================

!===========================================================================
!  START OF GET_BPLUS
!===========================================================================
      subroutine get_bplus(
     &                bp, 
     &                bpl2,
     &                x,
     &                xbp,
     &                isize, jsize, nlev, npblloc,
     &                interp_type, fac_extrap)

!---------------------------------------------------------------------------
! PURPOSE:
! compute the value of a variable at the PBL top (level B+)
!---------------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: 
     &    isize       ! array size
     &,   jsize       ! array size
     &,   nlev        ! number of vertical levels
     &,   npblloc     ! number of pbl layers

        real (kind=dbl_kind), dimension(isize,jsize,nlev), intent(in) :: 
     &    bp     ! odd-level pressure function                       (-).
     &,   x      ! odd-level field @ current time   (units vary).

	real (kind=dbl_kind), dimension(isize,jsize,nlev+1), intent(in) ::
     &    bpl2   ! even-level pressure function                      (-).

        character(*), intent(in) :: interp_type  ! interpolation type
        real(kind=dbl_kind), intent(in) :: fac_extrap    ! degree of extrapolation 0-1

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(isize,jsize), intent(out) :: 
     &   xbp     ! field at level B+ @ current time   (units vary).

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(isize,jsize) :: 
     &    exfac

      if (trim(interp_type) == "THETA") then
        exfac(:,:) = fac_extrap*(bpl2(:,:,nlev-npblloc+1) - bp(:,:,nlev-npblloc)) /  
     &                (bp(:,:,nlev-npblloc) - bp(:,:,nlev-npblloc-1)) 
        xbp(:,:) = x(:,:,nlev-npblloc) +  
     &             exfac*(x(:,:,nlev-npblloc)-x(:,:,nlev-npblloc-1))
!-- theta must increase upwards
        if (trim(interp_type) == "THETA") 
     &    xbp(:,:) = max(xbp(:,:),x(:,:,nlev-npblloc+1))
      else
        xbp(:,:) =  x(:,:,nlev-npblloc)
      endif

      end subroutine get_bplus
!===========================================================================
!  END OF GET_BPLUS
!===========================================================================

!===========================================================================
!  START OF GET_BPLUS_Q
!===========================================================================
      subroutine get_bplus_q(
     &                bp, 
     &                bpl2,
     &                pll2,
     &                q,
     &                qbp,
     &                qs,
     &                tbp,
     &                isize, jsize, nlev, npblloc,
     &                fac_extrap)

!---------------------------------------------------------------------------
! PURPOSE:
! compute the value of water vapor mixing ratio at the PBL top (level B+)
!---------------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) :: 
     &    isize       ! array size
     &,   jsize       ! array size
     &,   nlev        ! number of vertical levels
     &,   npblloc     ! number of pbl layers

        real (kind=dbl_kind), dimension(isize,jsize,nlev), intent(in) :: 
     &    bp     ! odd-level pressure function                       (-).
     &,   qs     ! odd-level saturation mixing ratios            (kg/kg).
     &,   q      ! odd-level q @ current time                    (kg/kg).

	real (kind=dbl_kind), dimension(isize,jsize,nlev+1), intent(in) ::
     &    bpl2   ! even-level pressure function                      (-).
     &,   pll2   ! even-level pressures                             (Pa).

        real (kind=dbl_kind), dimension(isize,jsize), intent(in) :: 
     &    tbp    ! temperature at level B+ @ current time           (K).

        real(kind=dbl_kind), intent(in) :: fac_extrap    ! degree of extrapolation, 0-1

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(isize,jsize), intent(out) :: 
     &   qbp     ! field at level B+ @ current time   (units vary).

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(isize,jsize,nlev) :: 
     &    rhtem
      real (kind=dbl_kind), dimension(isize,jsize) :: 
     &    exfac
     &,   qsatb
     &,   pb

        rhtem = q/qs
        exfac(:,:) = (bpl2(:,:,nlev-npblloc+1) - bp(:,:,nlev-npblloc)) /  
     &                (bp(:,:,nlev-npblloc) - bp(:,:,nlev-npblloc-1)) 
        qbp(:,:) = rhtem(:,:,nlev-npblloc) +  
     &             exfac*(rhtem(:,:,nlev-npblloc)-rhtem(:,:,nlev-npblloc-1))
        pb = pll2(:,:,nlev-npblloc+1)
        call eau_sat(isize*jsize, pb, tbp, qsats=qsatb)
!        write(*,*) rhtem(2,2,:)
!        write(*,*) tbp(2,2),qbp(2,2),qsatb(2,2)*1000.

!-- saturation ratio must be between 0 and 1
        qbp = min(qbp,1.0_dbl_kind)
        qbp = max(qbp,0.0_dbl_kind)
        qbp = qbp * qsatb

        qbp(:,:) =  (c1-fac_extrap)*q(:,:,nlev-npblloc) + fac_extrap*qbp(:,:)

      end subroutine get_bplus_q
!===========================================================================
!  END OF GET_BPLUS_Q
!===========================================================================

!===========================================================================
!  START OF GET_H
!===========================================================================
	subroutine get_h(
     &                   phi,
     &                   bp,
     &                   th,
     &                   q,
     &                   qsl2,
     &                   h)

	implicit none

!---------------------------------------------------------------------------
! PURPOSE : compute the moist static energy, mdb 1-25-01
!---------------------------------------------------------------------------
 
!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(iim,jjm,nlm),  intent(in) :: 
     &                            phi,
     &                            bp,
     &                            th,
     &                            q,
     &                            qsl2
 
!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(iim,jjm,nlm) :: h

!---------------------------------------------------------------------------
! LOCAL VARIABLES
!---------------------------------------------------------------------------
	integer (kind=int_kind) :: i, j, k

        do k = 1,nlm
          do i = 1,iim
            do j = 1,jjm
              if (q(i,j,k) > qsl2(i,j,k)) then
                h(i,j,k) = spec_heat_cp*bp(i,j,k)*th(i,j,k) + 
     &                      phi(i,j,k) + hltm*qsl2(i,j,k)
              else
                h(i,j,k) = spec_heat_cp*bp(i,j,k)*th(i,j,k) + 
     &                      phi(i,j,k) + hltm*q(i,j,k)
              endif
            enddo
          enddo
        enddo

	end subroutine get_h
!===========================================================================
!  END OF GET_H
!===========================================================================

	end module hstatc

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
