!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module initialize

!-------------------------------------------------------------------------
!  PURPOSE:
!	1) initialize data to zero
!	2) read in data defining the grid structure
!	3) read in data to interpolate from geodesic grid to lat/lon
!	4) read in run-time variables
!	5) read in boundary conditions
!	6) read in or generate initial conditions
!	7) dump initial conditions to a data file

!---- modifications:
!     added initialization related to the eau_ng parameterization of frac
!     tional cloudiness.
!     Laura D. Fowler/mcbug (09-06-02).

!-------------------------------------------------------------------------

	use physical_parameters
	use model_parameters
	use domain_decomposition
	use model_control
	use global_reductions
	use prognostics
	use interpolate_to_latlon
	use timer
	use wrap_face_data
	use wrap_vertex_data
	use grid_mask
	use grid_metrics
	use operators_hl
	use dissipation_operator
        use gather_scatter
	use hstatc
	use sigma_coordinate
#ifndef no_pbl
	use fluxcoupler_fields
#endif
	use time_manager_module
	use elliptic_solver
	use multigrid_solver
#ifdef fluxcorrectedtransport
        use component_intra
        use assign_owner_bugs6
        use flux_corrected_transport
        use wrap_bugs6_data
#endif
	use tracer_names
	use adjustments
	use post_step_module
#ifndef no_pbl
#ifdef eau_ng
      use eau_names
#endif      
#endif      

	implicit none
	save

	private :: 
     *    generate_initial_conditions


!*************************************************************************

	contains

!*************************************************************************


!======================================================================
! BEGINNING OF INITIALIZE_MODEL
!======================================================================

	subroutine initialize_model
	logical (kind=log_kind) :: l_error
#ifdef fluxcorrectedtransport
      integer (kind=int_kind) :: even_odd_factor
#endif

!-------------------------------------------------------------------------
! INITIALIZATION
!-------------------------------------------------------------------------
	call init_model_parameters
	call init_model_control
	call init_prognostics
	call init_adjustments
	call init_vertical_coordinate
	call init_tracer_names
	call init_time_manager
#ifndef no_pbl
#ifdef eau_ng
      call init_eau_names
#endif
#endif

!-----------------------------------------------------------------------
! assign subdomains to each process
!  (when message passing is turned off, all subdomains are owned by one process)
!-----------------------------------------------------------------------
        call set_assign_owner (l_error)

!-----------------------------------------------------------------------
! initialize the utility that updates the face data along subdomain boundary
!-----------------------------------------------------------------------
        call initialize_wrap

!-----------------------------------------------------------------------
! initialize the utility that updates the corner data along subdomain boundary
!----------------------------------------------------------------------- 
        call initialize_wrap_vertices (l_error)

!-----------------------------------------------------------------------
! determine which grid cells are "real," and which are "ghost" cell
!-----------------------------------------------------------------------
        call set_grid_center_mask

!-----------------------------------------------------------------------
! based on the grid_center_mask, compute mask for the grid corners
!-----------------------------------------------------------------------
        call set_grid_corner_mask

!-----------------------------------------------------------------------
! generate weights to compute divergence, curl, gradient, and Laplacian
!-----------------------------------------------------------------------
        call generate_grid_metrics
        call wrap_vertices(r3,1,c(1,1,1,1,1))

!-----------------------------------------------------------------------
! initialize the dissipation operator metrics
!-----------------------------------------------------------------------
        do ksdm = 1,nsdm
          call init_diss_op(ksdm,trans_to_X2(:,:,:,:,:,ksdm),c(:,:,:,:,ksdm))
        enddo

#ifdef fluxcorrectedtransport
! register this component for intra-component communications
            call initialize_component('fct')
            if(l_grid_even) then
               even_odd_factor=2
            else
               even_odd_factor=3
            endif
            call set_component_horizontal('fct',                           
     *                                     max_level, even_odd_factor, max_ig,
     *                                     im, jm, iim, jjm, isdm, jsdm )
            call set_component_parallel('fct',                           
     *                                    npe_atmos, my_task, MPI_int_type,  
     *                                    MPI_float_type, MPI_comm_atmos )
         call set_assign_owner_bugs6 (isdm, jsdm, max_level, max_level,    
     *                           partition_factor, 'fct',               
     *                           './assign_file' )
        call initialize_wrap_bugs6('fct')
        call initialize_fct
#endif

!-----------------------------------------------------------------------  
! initialize the multigrid solver
!-----------------------------------------------------------------------
        call initialize_multigrid ()

!-------------------------------------------------------------------------
! open files which stay open the entire run
!-------------------------------------------------------------------------
        call open_files

!-------------------------------------------------------------------------
! call init_interp_to_lat_lon which initializes the interpolation routine
!-------------------------------------------------------------------------
        call init_interp_to_lat_lon

!-------------------------------------------------------------------------
! get runtime parameters
!-------------------------------------------------------------------------
        call input

!-------------------------------------------------------------------------
! read in boundary data (32 bit)
!-------------------------------------------------------------------------
        call read_atm_bc(3)

!-------------------------------------------------------------------------
! read in restart if desired, l_restart set in namelist file
!-------------------------------------------------------------------------
        if(l_restart) then
          call read_restart
	  tau_end = tau_end + tau
          write(6,*) my_task, tau_end, tau
	else
	  call generate_initial_conditions
        endif

!-------------------------------------------------------------------------
! get pointers based on step_count
!-------------------------------------------------------------------------
	call get_pointers

!---------------------------------------------------------------------------
! ALL INITIAL DATA HAS BEEN READ IN OR GENERATED
!---------------------------------------------------------------------------

!-----------------------------------------------------------------------
! DO HYDROSTATICS and derive diagnostic variables
!-----------------------------------------------------------------------
	call post_step

#ifndef no_pbl
!-----------------------------------------------------------------------
! initialize fields that may be needed in the flux coupler
!-----------------------------------------------------------------------
	spdm_atm(:,:,:) = spdm(:,:,:)
	shear1_atm(:,:,:) = shear1(:,:,:)
        tpgvar_atm(:,:,:) = tpgvar(:,:,:)
#endif

!-----------------------------------------------------------------------
! write initial conditions and boundary conditions
!-----------------------------------------------------------------------
        call write_bc
        call write_prognostics

	end subroutine initialize_model
!======================================================================
! END OF INITIALIZE_MODEL
!======================================================================


!======================================================================
! BEGINNING OF GENERATE_INITIAL_CONDITIONS
!======================================================================
	subroutine generate_initial_conditions

!---------------------------------------------------------------------------
! PURPOSE: just what the title says
!---------------------------------------------------------------------------

        implicit none
        real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: work
	real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) :: buffer
        integer (kind=int_kind) :: i,k

!---------------------------------------------------------------------------
! set absolute vorticity to f
!     divergence to zero
!     surface pressure to 984.00 mb
!     pbl top pressure to 90% of the surface pressure
!     theta to an isothermal atmosphere
!     tracers to zero
!     ifdef eau_ng, set elnqg variables to zero
!---------------------------------------------------------------------------
        do i=1,ntprog
        do k=1,nlm
         eta(:,:,k,i,:) = f(:,:,:)
        enddo
        enddo

        relative(:,:,:,:)     = c0
        div(:,:,:,:,:)        = c0
        ps(:,:,:,:)           = 98400.0
#ifndef no_pbl
	pb(:,:,:,:)           = 0.9*ps
#endif
        psi(:,:,:,:)          = c0
        chi(:,:,:,:)          = c0
	tracer(:,:,:,:,:,:)   = c0
#ifndef no_pbl
#ifdef eau_ng
        elqng(:,:,:,:,:,:)    = c0
#endif
#endif

	do ksdm = 1,nsdm
#ifdef no_pbl
        call press(
     *          plgl2 (:,:,:,ksdm),
     *          plg   (:,:,:,ksdm),
     *          bpgl2 (:,:,:,ksdm),
     *          bpg   (:,:,:,ksdm),
     *          ps    (:,:,1,ksdm),
     *          iim, jjm)
#else
        call press(
     *          plgl2 (:,:,:,ksdm),
     *          plg   (:,:,:,ksdm),
     *          bpgl2 (:,:,:,ksdm),
     *          bpg   (:,:,:,ksdm),
     *          ps    (:,:,1,ksdm),
     *          pb    (:,:,1,ksdm),
     *          iim, jjm)
#endif
	enddo

        do i=1,ntprog
          th(:,:,:,i,:) = 280.0/bpg(:,:,:,:)
        enddo

!---------------------------------------------------------------------------
! add random noise to theta field
! random numbers are distributed between 0 and 1
! subtract 0.5 to keep avg(th) nearly the same
!---------------------------------------------------------------------------
        do k=1,nlm
          do i=1,ntprog
            call random_number(work)
            work = work - 0.5
            th(:,:,k,i,:) = th(:,:,k,i,:) + work(:,:,:)
          enddo
        enddo

!---------------------------------------------------------------------------
! wrap theta 
!---------------------------------------------------------------------------
	do i=1,ntprog
	  buffer(:,:,:,:) = th(:,:,:,i,:)
	  call wrap (max_level,
     *             iim,
     *             jjm,
     *             nlm,
     *             nsdm,
     *             buffer(1,1,1,1))
	  th(:,:,:,i,:) = buffer(:,:,:,:)
	enddo
	
!---------------------------------------------------------------------------
! compute velocity and ke
!---------------------------------------------------------------------------
	call calculate_u_and_ke(nlm,psi(:,:,:,:),chi(:,:,:,:),
     *       u(:,:,:,:,:,n4,:),ke(:,:,:,:))

	end subroutine generate_initial_conditions
!======================================================================
! END OF GENERATE_INITIAL_CONDITIONS
!======================================================================


	end module initialize

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
