!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module interpolate_to_latlon

!--------------------------------------------------------------------------
!
!	this module allows data on the geodesic grid to be interpolated
!	  to a regular lat/lon grid. the interpolation is bilinear, it is
!	  locally accurate, but not conservative. the interpolation data
! 	  is read in from a text data file in ./data
!
!--------------------------------------------------------------------------

	use kinds
	use physical_parameters
	use model_parameters
	use domain_decomposition
	use operators
        use gather_scatter

	implicit none
	save

#if resolution == 00642
        integer (kind=int_kind), parameter :: ims= 40,jms=20
#endif

#if resolution == 02562
        integer (kind=int_kind), parameter :: ims= 80,jms=40
#endif

#if resolution == 10242
        integer (kind=int_kind), parameter :: ims=160,jms=80
#endif

#if resolution == 40962
        integer (kind=int_kind), parameter :: ims=320,jms=160
#endif

	integer (kind=int_kind), parameter :: maxs = ims*jms

	real (kind=dbl_kind), dimension(maxs,3) :: wght
	integer (kind=int_kind), dimension(maxs,3) :: indx


!**************************************************************************

	contains

!**************************************************************************


!======================================================================
! BEGINNING OF INIT_INTERP_TO_LAT_LON
!======================================================================

        subroutine init_interp_to_lat_lon

!--------------------------------------------------------------------------
!	initialize the interpolation scheme by reading
!	  in the weights and the inde
!--------------------------------------------------------------------------

	implicit none

        integer (kind=int_kind) :: i,j,ii
	integer (kind=int_kind), parameter :: iunit = 2

!--------------------------------------------------------------------------
!  based on the grid size, open the correct file
!--------------------------------------------------------------------------
	if(my_task.eq.0) then

        if(max_ig.eq.162) then
                open(iunit,file=trim(datapath)//'data/interp_sgg_00162',
     *               form='formatted',status='old')
        elseif(max_ig.eq.642) then
                open(iunit,file=trim(datapath)//'data/interp_sgg_00642',
     *               form='formatted',status='old')
        elseif(max_ig.eq.2562) then
                open(iunit,file=trim(datapath)//'data/interp_sgg_02562',
     *               form='formatted',status='old')
        elseif(max_ig.eq.10242) then
                open(iunit,file=trim(datapath)//'data/interp_sgg_10242',
     *               form='formatted',status='old')
        elseif(max_ig.eq.40962) then
                open(iunit,file=trim(datapath)//'data/interp_sgg_40962',
     *               form='formatted',status='old')
        else
                stop
        endif


!--------------------------------------------------------------------------
!  read in data
!--------------------------------------------------------------------------
        do j=1,jms
        do i=1,ims
          ii=(j-1)*ims+i
          read(iunit,*) indx(ii,1),wght(ii,1),
     *               indx(ii,2),wght(ii,2),
     *               indx(ii,3),wght(ii,3)
        enddo
        enddo
        close(iunit)

	endif 	! if(my_task.eq.0)
#ifdef mpi
	call MPI_BARRIER(MPI_COMM_ATMOS, ierr)
#endif

	end subroutine init_interp_to_lat_lon

!======================================================================
! END OF INIT_INTERP_TO_LAT_LON
!======================================================================



!======================================================================
! BEGINNING OF INTERP_TO_LAT_LON
!======================================================================

	subroutine interp_to_lat_lon(xin,xout)

!--------------------------------------------------------------------------
! 	interpolation from geodesic grid to lat/lon grid
! 	for every lat/lon grid point, the closest 3 geodesic grid
! 	points were found and the weight solved from.  This is done
! 	in a pre-processing mode and the weights and index mapping
! 	is read in at runtime.
!--------------------------------------------------------------------------

	implicit none

	integer (kind=int_kind) ::  i,j,k,ii
	real (kind=dbl_kind), dimension(max_ig) , intent(in) :: xin(max_ig)
	real (kind=dbl_kind), dimension(ims,jms), intent(out):: xout(ims,jms)

	ii=0
	do j=1,jms
	do i=1,ims
	  ii=ii+1
	  xout(i,j)=0.0
	  do k=1,3
	    xout(i,j)=xout(i,j)+wght(ii,k)*xin(indx(ii,k))
	  enddo
	enddo
	enddo

	end subroutine interp_to_lat_lon

!======================================================================
! END OF INTERP_TO_LAT_LON
!======================================================================
 
 

!======================================================================
! BEGINNING OF INTERP_DRIVER_2D
!======================================================================

	subroutine interp_driver_2d(xin,iunit)

!--------------------------------------------------------------------------
! interplote and write out interpolated data to unit=iunit
! reduces an array from (im,jm) to (max_ig), then interpolates that array
!  to (ims,jms), data is dumped to iunit as a sequential unformatted file
!--------------------------------------------------------------------------

	implicit none

	real (kind=dbl_kind), dimension(im,jm), intent(in) :: xin

	real (kind=dbl_kind), dimension(max_ig) ::   igvec
	real (kind=dbl_kind), dimension(ims,jms) ::  interp_igvec
	real (kind=real_kind), dimension(ims,jms) :: interp_tmp
	integer (kind=int_kind) :: iunit

	call reduce_to_vector_global(xin,igvec)
 	call interp_to_lat_lon(igvec,interp_igvec)
 	interp_tmp=interp_igvec
 	write(iunit) interp_tmp

	end subroutine interp_driver_2d
!======================================================================
! END OF INTERP_DRIVER_2D
!======================================================================



!======================================================================
! BEGINNING OF INTERP_DRIVER_3D
!======================================================================
        subroutine interp_driver_3d(xin, ncalls, iunit)

!--------------------------------------------------------------------------
! recursive calls to interp_driver_2d
!--------------------------------------------------------------------------

        implicit none
	integer (kind=int_kind) :: iunit, ncalls, k
        real (kind=dbl_kind), dimension(im,jm,ncalls), intent(in) :: xin

	do k=ncalls,1,-1
	  call interp_driver_2d(xin(:,:,k), iunit)
	enddo

        end subroutine interp_driver_3d
!======================================================================
! END OF INTERP_DRIVER_3D
!======================================================================




!======================================================================
! BEGINNING OF INTERP_DRIVER_mjo
!======================================================================

	subroutine interp_driver_mjo(xin,iunit)

!--------------------------------------------------------------------------
! interplote and write out interpolated data to unit=iunit
! reduces an array from (im,jm) to (max_ig), then interpolates that array
!  to (ims,jms), data is dumped to iunit as a sequential unformatted file
!--------------------------------------------------------------------------

	implicit none

	real (kind=dbl_kind), dimension(im,jm), intent(in) :: xin
	integer (kind=int_kind), intent(in) :: iunit

	real (kind=dbl_kind), dimension(max_ig) ::   igvec
	real (kind=dbl_kind), dimension(ims,jms) ::  interp_igvec
	real (kind=real_kind), dimension(ims,jms) :: interp_tmp
        integer (kind=int_kind) :: jbeg, jend

	call reduce_to_vector_global(xin,igvec)
 	call interp_to_lat_lon(igvec,interp_igvec)
 	interp_tmp=interp_igvec
        jbeg = jms/2 - 4
        jend = jms/2 + 5
 	write(iunit) interp_tmp(:,jbeg:jend)

	end subroutine interp_driver_mjo
!======================================================================
! END OF INTERP_DRIVER_mjo
!======================================================================



!======================================================================
! BEGINNING OF INTERP_DRIVER_mjo3
!======================================================================

	subroutine interp_driver_mjo3(xin,nvar,iunit)

!--------------------------------------------------------------------------
! interplote and write out interpolated data to unit=iunit
! reduces an array from (im,jm) to (max_ig), then interpolates that array
!  to (ims,jms), data is dumped to iunit as a sequential unformatted file
!--------------------------------------------------------------------------

	implicit none

	integer (kind=int_kind), intent(in) :: iunit, nvar
	real (kind=dbl_kind), dimension(im,jm,nvar), intent(in) :: xin

        integer (kind=int_kind) :: k

	do k=1,nvar
	  call interp_driver_mjo(xin(:,:,k), iunit)
	enddo

	end subroutine interp_driver_mjo3
!======================================================================
! END OF INTERP_DRIVER_mjo3
!======================================================================



	end module interpolate_to_latlon

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
