!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module operators

        use kinds
        use model_parameters
        use geodesic_grid_utilities

        implicit none

!*************************************************************************

        contains

!*************************************************************************


!======================================================================
! BEGIN CURL_OF_A_VECTOR
!======================================================================
        subroutine curl_of_a_vector(                                  &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                nspace,                                               &
                ncorners,                                             &
                area_inv,                                             &
                v_weights,                                            &
                xin,                                                  &
                xout)

!-----------------------------------------------------------------------
! PURPOSE : given a vector defined at cell corners, find the curl
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad      : width of 'ghost cell' region
!       nx        : first dimension of xin (integer)
!       ny        : second dimension of xin (integer)
!       ndata     : amount of data involved in curl operator
!       nspace    : number of spatial dimensions
!       ncorners  : number of corners
!       area_inv  : inverse of grid cell area (m^-2)
!       v_weights : weights to compute curl (m)
!       xin       : vector (x)
!       xout      : curl of xin (x*m^-1)
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                             &
                        npad, nx, ny, ndata, nspace, ncorners

        real (kind=dbl_kind), intent(in) ::                                &
                        area_inv                        (nx,ny),           &
                        v_weights (ndata,nspace,ncorners,nx,ny),           &
                        xin             (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                               &
                        xout                            (nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j

!----------------------------------------------------------------------------
! INITIALIZE parts of xout not filled by the curl operator
!----------------------------------------------------------------------------
        do j = 1,npad
        do i = 1,nx
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do j=ny-npad+1,ny
        do i = 1,nx
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=1,npad
        do j=1,ny
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=nx-npad+1,nx
        do j=1,ny
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo

!----------------------------------------------------------------------------
! COMPUTE CURL AT INTERIOR POINTS
!----------------------------------------------------------------------------
        do j=1,ny-npad
        do i=1,nx-npad

        xout(i,j) =                                                       &
                  (  + v_weights(1,2,1,i  ,j  ) * xin(1,1,i  ,j  )        &
                     - v_weights(1,1,1,i  ,j  ) * xin(2,1,i  ,j  )        &
                     + v_weights(1,2,2,i  ,j  ) * xin(1,2,i  ,j  )        &
                     - v_weights(1,1,2,i  ,j  ) * xin(2,2,i  ,j  )        &
                     + v_weights(2,2,1,i+1,j  ) * xin(1,1,i+1,j  )        &
                     - v_weights(2,1,1,i+1,j  ) * xin(2,1,i+1,j  )        &
                     + v_weights(2,2,2,i+1,j+1) * xin(1,2,i+1,j+1)        &
                     - v_weights(2,1,2,i+1,j+1) * xin(2,2,i+1,j+1)        &
                     + v_weights(3,2,1,i+1,j+1) * xin(1,1,i+1,j+1)        &
                     - v_weights(3,1,1,i+1,j+1) * xin(2,1,i+1,j+1)        &
                     + v_weights(3,2,2,i  ,j+1) * xin(1,2,i  ,j+1)        &
                     - v_weights(3,1,2,i  ,j+1) * xin(2,2,i  ,j+1) )      &
                     * area_inv(i,j)

        enddo           ! do i
        enddo           ! do j

        end subroutine curl_of_a_vector
!======================================================================
! END CURL_OF_A_VECTOR
!======================================================================


!======================================================================
! BEGIN DIV_OF_A_VECTOR
!====================================================================== 
        subroutine div_of_a_vector(                                   &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                nspace,                                               &
                ncorners,                                             &
                area_inv,                                             &
                v_weights,                                            &
                xin,                                                  &
                xout )

!-----------------------------------------------------------------------
! PURPOSE : given a vector defined at cell corners, find the divergence
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad      : width of 'ghost cell' region
!       nx        : first dimension of xin (integer)
!       ny        : second dimension of xin (integer)
!       ndata     : amount of data involved in div operator
!       nspace    : number of spatial dimensions
!       ncorners  : number of corners
!       area_inv  : inverse of grid cell area (m^-2)
!       v_weights : weights to compute div (m)
!       xin       : vector(x)
!       xout      : divergence of xin (x*m^-1)
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                          &
                        npad, nx, ny, ndata, nspace, ncorners

        real (kind=dbl_kind), intent(in) ::                             &
                        area_inv                        (nx,ny),        &
                        v_weights (ndata,nspace,ncorners,nx,ny),        &
                        xin             (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                            &
                        xout                            (nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j


!----------------------------------------------------------------------------
! INITIALIZE parts of xout not filled by the curl operator
!----------------------------------------------------------------------------
        do j = 1,npad
        do i = 1,nx
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do j=ny-npad+1,ny
        do i = 1,nx
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=1,npad
        do j=1,ny
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=nx-npad+1,nx
        do j=1,ny
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo

!----------------------------------------------------------------------------
! COMPUTE DIVERGENCE AT INTERIOR POINTS
!----------------------------------------------------------------------------
        do j=1,ny-npad
        do i=1,nx-npad

        xout(i,j) =                                                        &
                   ( - v_weights(1,1,1,i  ,j  ) * xin(1,1,i  ,j  )         &
                     - v_weights(1,2,1,i  ,j  ) * xin(2,1,i  ,j  )         &
                     - v_weights(1,1,2,i  ,j  ) * xin(1,2,i  ,j  )         &
                     - v_weights(1,2,2,i  ,j  ) * xin(2,2,i  ,j  )         &
                     - v_weights(2,1,1,i+1,j  ) * xin(1,1,i+1,j  )         &
                     - v_weights(2,2,1,i+1,j  ) * xin(2,1,i+1,j  )         &
                     - v_weights(2,1,2,i+1,j+1) * xin(1,2,i+1,j+1)         &
                     - v_weights(2,2,2,i+1,j+1) * xin(2,2,i+1,j+1)         &
                     - v_weights(3,1,1,i+1,j+1) * xin(1,1,i+1,j+1)         &
                     - v_weights(3,2,1,i+1,j+1) * xin(2,1,i+1,j+1)         &
                     - v_weights(3,1,2,i  ,j+1) * xin(1,2,i  ,j+1)         &
                     - v_weights(3,2,2,i  ,j+1) * xin(2,2,i  ,j+1) )       &
                     * area_inv(i,j)

        enddo           ! do i
        enddo           ! do j

        end subroutine div_of_a_vector
!======================================================================
! END DIV_OF_A_VECTOR
!======================================================================



!======================================================================
! BEGINNING OF LAPLACIAN_OF_A_SCALAR
!======================================================================
        subroutine laplacian_of_a_scalar(                              &
                npad,                                                  &
                nx,                                                    &
                ny,                                                    &
                nweights,                                              &
                l_weights,                                             &
                xin,                                                   &
                xout)

!-----------------------------------------------------------------------
! PURPOSE : given a scalar defined at cell centers, computed the Laplacian
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad      : width of 'ghost cell' region
!       nx        : first dimension of xin (integer)
!       ny        : second dimension of xin (integer)
!       nweights  : amount of data incorporated in Laplacian operator
!       area_inv  : inverse of grid cell area (m^-2)
!       v_weights : weights to compute Laplacian (m^-2)
!       xin       : scalar (x)
!       xout      : Laplacian of xin (x*m^-2)
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                           &
                        npad, nx, ny, nweights            

        real (kind=dbl_kind), intent(in) ::                              &
                        l_weights      (nweights,nx,ny),                 &
                        xin                     (nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                             &
                        xout                    (nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j

!----------------------------------------------------------------------------
! INITIALIZE parts of xout that are not filled by the Laplacian operator
!----------------------------------------------------------------------------
        do j = 1,npad
        do i = 1,nx
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do j=ny-npad+1,ny
        do i = 1,nx
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=1,npad
        do j=1,ny
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=nx-npad+1,nx
        do j=1,ny
          xout(i,j) = 0.0_dbl_kind
        enddo
        enddo

!----------------------------------------------------------------------------
! COMPUTE LAPLACIAN AT INTERIOR POINTS
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad

          xout(i,j) =                                                &
                         + l_weights( 7, i, j) * xin(i  ,j  )        &
                         + l_weights( 1, i, j) * xin(i-1,j  )        &
                         + l_weights( 2, i, j) * xin(i-1,j-1)        &
                         + l_weights( 3, i, j) * xin(i  ,j-1)        &
                         + l_weights( 4, i, j) * xin(i+1,j  )        &
                         + l_weights( 5, i, j) * xin(i+1,j+1)        &
                         + l_weights( 6, i, j) * xin(i  ,j+1) 

        enddo           ! do i
        enddo           ! do j

!----------------------------------------------------------------------------
! COMPUTE LAPLACIAN AT NORTH POLE (position 1 does not exist)
!----------------------------------------------------------------------------
        i = 1
        j = ny-1
        xout(i,j) =                                                  &
                         + l_weights( 7, i, j) * xin(i  ,j  )        &
                         + l_weights( 2, i, j) * xin(nx ,1  )        &
                         + l_weights( 3, i, j) * xin(i  ,j-1)        &
                         + l_weights( 4, i, j) * xin(i+1,j  )        &
                         + l_weights( 5, i, j) * xin(i+1,j+1)        &
                         + l_weights( 6, i, j) * xin(i  ,j+1)

!----------------------------------------------------------------------------
! COMPUTE LAPLACIAN AT SOUTH POLE (position 3 does not exist)
!----------------------------------------------------------------------------
        i = nx-1
        j = 1
        xout(i,j) =                                                  &
                         + l_weights( 7, i, j) * xin(i  ,j  )        &
                         + l_weights( 1, i, j) * xin(i-1,j  )        &
                         + l_weights( 2, i, j) * xin(1  ,ny )        &
                         + l_weights( 4, i, j) * xin(i+1,j  )        &
                         + l_weights( 5, i, j) * xin(i+1,j+1)        &
                         + l_weights( 6, i, j) * xin(i  ,j+1)         

        end subroutine laplacian_of_a_scalar
!======================================================================
! END OF LAPLACIAN_OF_A_SCALAR
!======================================================================



!======================================================================
! BEGIN K_CROSS_V
!======================================================================
        subroutine k_cross_v(                                        &
                nx,                                                  &
                ny,                                                  &
                nspace,                                              &
                ncorners,                                            &
                xin,                                                 &
                xout )

!-----------------------------------------------------------------------
! PURPOSE : take the k vector and cross it with xin
! NOTE : assumes an orthogonal (and right-handed) coordinate system
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       nx        : first dimension of xin (integer)
!       ny        : second dimension of xin (integer)
!       nspace    : number of spatial dimensions (integer)
!       ncorners  : number of corners (integer)
!       xin       : a vector defined at cell corners
!       xout      : k cross xin
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                       &
                        nx, ny, nspace, ncorners

        real (kind=dbl_kind),  intent(in) ::                         &
                        xin      (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind),  intent(out) ::                        & 
                        xout     (nspace,ncorners,nx,ny)

        integer (kind=int_kind) :: i,j,icorner

!----------------------------------------------------------------------------
! COMPUTE the CROSS PRODUCT
!----------------------------------------------------------------------------
        do j=1,ny
        do i=1,nx
        do icorner=1,ncorners
          xout(1,icorner,i,j) = -xin(2,icorner,i,j)
          xout(2,icorner,i,j) = +xin(1,icorner,i,j)
        enddo
        enddo
        enddo

        end subroutine k_cross_v
!======================================================================
! END K_CROSS_V
!=====================================================================



!======================================================================
! BEGIN GRADIENT
!======================================================================
        subroutine gradient(                                          &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                nspace,                                               &
                ncorners,                                             &
                corner_area_inv,                                      &
                v_weights,                                            &
                xin,                                                  &
                xout)

!-----------------------------------------------------------------------
! PURPOSE : take the gradient of a scalar fields
!       gradient is defined at corners
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad      : width of 'ghost cell' region
!       nx        : first dimension of xin (integer)
!       ny        : second dimension of xin (integer)
!       ndata     : amount of data involved in curl operator
!       nspace    : number of spatial dimensions
!       ncorners  : number of corners
!       area_inv  : inverse of grid cell area (m^-2)
!       v_weights : weights to compute gradient (m)
!       xin       : scalar field defined at cell center
!       xout      : gradient of scalar field defined at corners
!
! NOTE : xin is a scalar, xout is a vector
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                          &
                        npad, nx, ny, ndata, nspace, ncorners

        real (kind=dbl_kind), intent(in) ::                             &
                        corner_area_inv        (ncorners,nx,ny),        &
                        v_weights (ndata,nspace,ncorners,nx,ny),        &
                        xin                             (nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                            &
                        xout            (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j,ispace

!----------------------------------------------------------------------------
! INITIALIZE parts of xout that are not filled by the gradient operator
!----------------------------------------------------------------------------
!       xout(:,:,:,1:npad) = 0.0_dbl_kind
!       xout(:,:,:,ny-npad+1:ny) = 0.0_dbl_kind
!       xout(:,:,1:npad,:) = 0.0_dbl_kind
!       xout(:,:,nx-npad+1:nx,:) = 0.0_dbl_kind
        xout = 0.0_dbl_kind

!----------------------------------------------------------------------------
! COMPUTE GRADIENT
!   gradient is computed for corners 1 and 2 for each interior cell
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad
        do ispace=1,nspace

          xout(ispace,1,i,j) =                                         &
            ( + v_weights(1,ispace,1,i,j) * xin(i  ,j  )               &
              + v_weights(2,ispace,1,i,j) * xin(i-1,j  )               &
              + v_weights(3,ispace,1,i,j) * xin(i-1,j-1) )             &
                 * corner_area_inv(1,i,j)       

          xout(ispace,2,i,j) =                                         &
            ( + v_weights(1,ispace,2,i,j) * xin(i  ,j  )               &
              + v_weights(2,ispace,2,i,j) * xin(i-1,j-1)               &
              + v_weights(3,ispace,2,i,j) * xin(i  ,j-1) )             &
                 * corner_area_inv(2,i,j)

        enddo
        enddo
        enddo

       end subroutine gradient
!======================================================================
! END GRADIENT
!======================================================================



!======================================================================
! BEGIN AVERAGE_TO_CELL_CORNERS
!======================================================================
        subroutine average_to_cell_corners(                           &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                ncorners,                                             &
                s_weights,                                            &
                xin,                                                  &
                xout)

!-----------------------------------------------------------------------
! PURPOSE : average scalar quanties from cell centers to cell corners
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       npad            : width of 'ghost cell' region
!       nx              : first dimension of xin (integer)
!       ny              : second dimension of xin (integer)
!       ndata           : amount of data involved in the averaging operator
!       ncorners        : number of corners
!       s_weights       : weights
!       xin             : scalar field defined at cell center
!       xout            : scalar field averaged to cell corners
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                          &
                        npad, nx, ny, ndata, ncorners

        real (kind=dbl_kind), intent(in) ::                             &
                        s_weights  (ndata,ncorners,nx,ny),              &
                        xin                       (nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                           &
                        xout             (ncorners,nx,ny)               

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j
        real (kind=dbl_kind) :: factor=0.3333333_dbl_kind

!----------------------------------------------------------------------------
! INITIALIZE
!----------------------------------------------------------------------------
        do j = 1,npad-1
        do i = 1,nx
          xout(:,i,j) = 0.0_dbl_kind
        enddo
        enddo
        do j=ny-npad+1,ny
        do i = 1,nx
          xout(:,i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=1,npad-1
        do j=1,ny
          xout(:,i,j) = 0.0_dbl_kind
        enddo
        enddo
        do i=nx-npad+1,nx
        do j=1,ny
          xout(:,i,j) = 0.0_dbl_kind
        enddo
        enddo

!----------------------------------------------------------------------------
! AVERAGE TO CELL CORNERS
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad

          xout(1,i,j) =                                                 &
              + s_weights(1,1,i,j) * xin(i  ,j  )                       &
              + s_weights(2,1,i,j) * xin(i-1,j  )                       &
              + s_weights(3,1,i,j) * xin(i-1,j-1)                        

         xout(2,i,j) =                                                  &
              + s_weights(1,2,i,j) * xin(i  ,j  )                       &
              + s_weights(2,2,i,j) * xin(i-1,j-1)                       &
              + s_weights(3,2,i,j) * xin(i  ,j-1)

        enddo           ! do i
        enddo           ! do j


        end subroutine average_to_cell_corners
!======================================================================
! END AVERAGE_TO_CELL_CORNERS
!======================================================================



!======================================================================
! BEGIN CALCULATE_CORNER_KINETIC_ENERGY
!======================================================================
        subroutine calculate_corner_kinetic_energy(                   &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                nspace,                                               &
                ncorners,                                             &
                u,                                                    &
                ke_c  )

!-----------------------------------------------------------------------
! PURPOSE : compute kinetic energy at cell corners
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                              &
                        npad, nx, ny, ndata, nspace, ncorners

        real (kind=dbl_kind), intent(in) ::                                 &
                        u                             (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                                &
                        ke_c                                (ncorners,nx,ny)
!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        real (kind=dbl_kind), parameter :: factor=0.5_dbl_kind
        integer (kind=int_kind) :: i,j,icorner


!----------------------------------------------------------------------------
! initialize
!----------------------------------------------------------------------------
        ke_c(:,:,:) = 0.0_dbl_kind

!----------------------------------------------------------------------------
! compute kinetic energy at each interior cell center
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad
        do icorner=1,ncorners
          ke_c(icorner,i,j) = factor * (u(1,icorner,i,j)*u(1,icorner,i,j)   &
                                      + u(2,icorner,i,j)*u(2,icorner,i,j))
        enddo
        enddo
        enddo

        end subroutine calculate_corner_kinetic_energy
!======================================================================
! BEGIN CALCULATE_CORNER_KINETIC_ENERGY
!======================================================================


!======================================================================
! BEGIN CALCULATE_CENTER_KINETIC_ENERGY
!======================================================================
        subroutine calculate_center_kinetic_energy(                   &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                nspace,                                               &
                ncorners,                                             &
                sub_area,                                             &
                area_inv,                                             &
                ke_c,                                                 &
                ke)

!-----------------------------------------------------------------------
! PURPOSE : compute kinetic energy at cell corners and cell centers
!
! NOTE : IMPORTANT : ke_c must be wrapped before calling this subroutine
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       nx              : first dimension of xin (integer)
!       ny              : second dimension of xin (integer)
!       ndata           : amount of data involved in the averaging operator
!       nspace          : number of spatial dimenions
!       ncorners        : number of corners
!       sub_area        : partial corner areas "owned" by cell centers
!       ke              : u dot u, averaged to cell center
!       ke_c            : u dot u, at cell corner
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                              &
                        npad, nx, ny, ndata, nspace, ncorners

        real (kind=dbl_kind), intent(in) ::                                    &
                        sub_area                (ndata,       ncorners,nx,ny), &
                        area_inv                                      (nx,ny), &
                        ke_c                                 (ncorners,nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                                   &
                        ke                                           (nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j

!----------------------------------------------------------------------------
! initialize
!----------------------------------------------------------------------------
        ke(:,:) = 0.0_dbl_kind

!----------------------------------------------------------------------------
! sum over the corners to get KE at cell center
!----------------------------------------------------------------------------
        do j=1,ny-npad
        do i=1,nx-npad

          ke(i,j) =                                                        &
                    ( + sub_area(1,1,i  ,j  ) * ke_c(1,i  ,j  )            &
                      + sub_area(1,2,i  ,j  ) * ke_c(2,i  ,j  )            &
                      + sub_area(2,1,i+1,j  ) * ke_c(1,i+1,j  )            &
                      + sub_area(2,2,i+1,j+1) * ke_c(2,i+1,j+1)            &
                      + sub_area(3,1,i+1,j+1) * ke_c(1,i+1,j+1)            &
                      + sub_area(3,2,i  ,j+1) * ke_c(2,i  ,j+1) )          &
                      * area_inv(i,j)

        enddo
        enddo

        end subroutine calculate_center_kinetic_energy
!======================================================================
! END CALCULATE_CENTER_KINETIC_ENERGY
!======================================================================


!======================================================================
! BEGIN CALCULATE_VELOCITY
!======================================================================
        subroutine calculate_velocity(                                &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ndata,                                                &
                nspace,                                               &
                ncorners,                                             &
                corner_area_inv,                                      &
                v_weights,                                            &
                psi,                                                  &
                chi,                                                  &
                xout )

!-----------------------------------------------------------------------
! PURPOSE : given the stream function and velocity potential, compute
!       the velocity. psi and chi are defined at cell center, u at corners
! NOTE : it is assumed that psi and chi have been wrapped before
!       calling this subroutine.
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       nx              : first dimension of xin (integer)
!       ny              : second dimension of xin (integer)
!       ndata           : amount of data involved in curl operator
!       nspace          : number of spatial dimensions
!       ncorners        : number of corners
!       corner_area_inv : inverse of grid cell area (m^-2)
!       v_weights       : weights to compute gradient (m)
!       psi             : streamfunction (m2/s)
!       chi             : velocity potential (m2/s)
!       xout            : velocity vector (m/s)
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                          &
                        npad, nx, ny, ndata, nspace, ncorners

        real (kind=dbl_kind), intent(in) ::                             &
                        corner_area_inv        (ncorners,nx,ny),        &
                        v_weights (ndata,nspace,ncorners,nx,ny),        &
                        psi                             (nx,ny),        &
                        chi                             (nx,ny)         

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(inout) ::                          &
                        xout               (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        real (kind = dbl_kind) ::                                       &
                        work1           (nspace,ncorners,nx,ny),        &
                        work2           (nspace,ncorners,nx,ny)

        integer (kind=int_kind) :: i,j,ispace,icorner

!----------------------------------------------------------------------------
! FIND GRADIENT OF PSI AND CHI
!----------------------------------------------------------------------------
        call gradient(                  &
                npad,                   &
                nx,                     &
                ny,                     &
                ndata,                  &
                nspace,                 &
                ncorners,               &
                corner_area_inv,        &
                v_weights,              &
                chi,                    &
                xout )        

        call gradient(                  &
                npad,                   &
                nx,                     &
                ny,                     &
                ndata,                  &
                nspace,                 &
                ncorners,               &
                corner_area_inv,        &
                v_weights,              &
                psi,                    &
                work1 )

        call k_cross_v(                 &
                nx,                     &
                ny,                     &
                nspace,                 &
                ncorners,               &
                work1,                  &
                work2 )

!----------------------------------------------------------------------------
! COMPUTE VELOCITY
!----------------------------------------------------------------------------
        do j=1,ny
        do i=1,nx
        do icorner=1,ncorners
        do ispace = 1,nspace
          xout(ispace,icorner,i,j) =    xout(ispace,icorner,i,j)        &
                                     + work2(ispace,icorner,i,j)
        enddo
        enddo
        enddo
        enddo

        end subroutine calculate_velocity
!======================================================================
! END CALCULATE_VELOCITY
!======================================================================



!======================================================================
! BEGIN AVERAGE_PV_TO_CELL_CORNERS
!======================================================================
        subroutine average_pv_to_cell_corners(                        &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                ncorners,                                             &
                nspace,                                               &
                ndata,                                                &
                xin,                                                  &
                h,                                                    &
                inv_h,                                                &
                xout )

!-----------------------------------------------------------------------
! PURPOSE : average absolute vorticity from cell centers to cell corners
!       form should conserve potential enstrophy under the process of
!       horizontal advection
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       nx              : first dimension of xin (integer)
!       ny              : second dimension of xin (integer)
!       ncorners        : number of corners
!       h               : mass field
!       inv_h           : inverse of mass field
!       xin             : vorticity field defined at cell center
!       xout            : vorticity field averaged to cell corners
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                        &
                        npad, nx, ny, ncorners, nspace, ndata

        real (kind=dbl_kind), intent(in) ::                           &
                        h                                  (nx,ny),   &
                        inv_h                              (nx,ny),   &
                        xin                                (nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                          &
                        xout            (ncorners,nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        real (kind=dbl_kind) ::                                       &
                        pv                       (nx,ny)

        real (kind=dbl_kind)  ::                                            &
                        factor

        integer (kind=int_kind) :: i,j


!----------------------------------------------------------------------------
! INITIALIZE parts of array that are not filled
!----------------------------------------------------------------------------
        xout = 0.0_dbl_kind
        factor = 1.0_dbl_kind/9.0_dbl_kind

!----------------------------------------------------------------------------
! compute potential vorticity
!----------------------------------------------------------------------------
        pv = xin*inv_h

!----------------------------------------------------------------------------
! average to cell corners for all interior points
!----------------------------------------------------------------------------
        do j=npad+1,ny-npad
        do i=npad+1,nx-npad
        xout(1,i,j) = factor *                                        &
                (  h(i  ,j  )  +  h(i-1,j  )  +  h(i-1,j-1) )         &
              * ( pv(i  ,j  )  + pv(i-1,j  )  + pv(i-1,j-1) )
        xout(2,i,j) = factor *                                        &
                (  h(i  ,j  )  +  h(i-1,j-1)  +  h(i  ,j-1) )         &
              * ( pv(i  ,j  )  + pv(i-1,j-1)  + pv(i  ,j-1) )
        enddo
        enddo

        end subroutine average_pv_to_cell_corners
!======================================================================
! END AVERAGE_PV_TO_CELL_CORNERS
!======================================================================



!======================================================================
! BEGIN AVERAGE_VECTOR_TO_CENTERS
!======================================================================
        subroutine average_vector_to_centers(                         &
                npad,                                                 &
                nx,                                                   &
                ny,                                                   &
                nspace,                                               &
                r3,                                                   &
                ncorners,                                             &
                ndata,                                                &
                area_inv,                                             &
                sub_area,                                             &
                trans_to_X1,                                          &
                xin,                                                  &
                xout)

!-----------------------------------------------------------------------
! PURPOSE : given vector data defined at cell corners, average that data
! to the cell centers. The averaging is sub_area weighted to make sure
! that dead corners are masked out in the averaging.
!
! NOTE: since this averaged velocity will be transformed into numerous
! coordinate systems, the xout is left in the global (x,y,z) system.
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                             &
                        npad, nx, ny, ndata, nspace, ncorners,r3

        real (kind=dbl_kind), intent(in) ::                                &
                        area_inv                        (nx,ny),           &
                        sub_area        (ndata ,ncorners,nx,ny),           &
                        trans_to_X1      (r3,r3,ncorners,nx,ny),           &
                        xin             (nspace,ncorners,nx,ny)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                               &
                        xout                         (r3,nx,ny)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: i,j
        real (kind=dbl_kind) :: h(6,r3), r(6,r3)

        xout = c0
        do j=1,ny-1
        do i=1,nx-1
          h = c0
          r = c0
          h(1,1:2) = xin(1:2,1,i  ,j  )*sub_area(1,1,i  ,j  )
          h(2,1:2) = xin(1:2,2,i  ,j  )*sub_area(1,2,i  ,j  )
          h(3,1:2) = xin(1:2,1,i+1,j  )*sub_area(2,1,i+1,j  )
          h(4,1:2) = xin(1:2,2,i+1,j+1)*sub_area(2,2,i+1,j+1)
          h(5,1:2) = xin(1:2,1,i+1,j+1)*sub_area(3,1,i+1,j+1)
          h(6,1:2) = xin(1:2,2,i  ,j+1)*sub_area(3,2,i  ,j+1)
          h = h * area_inv(i,j)
          r(1,:) = matmul(trans_to_X1(:,:,1,i  ,j  ),h(1,:))
          r(2,:) = matmul(trans_to_X1(:,:,2,i  ,j  ),h(2,:))
          r(3,:) = matmul(trans_to_X1(:,:,1,i+1,j  ),h(3,:))
          r(4,:) = matmul(trans_to_X1(:,:,2,i+1,j+1),h(4,:))
          r(5,:) = matmul(trans_to_X1(:,:,1,i+1,j+1),h(5,:))
          r(6,:) = matmul(trans_to_X1(:,:,2,i  ,j+1),h(6,:))
          xout(:,i,j) = r(1,:)+r(2,:)+r(3,:)+r(4,:)+r(5,:)+r(6,:)
        enddo
        enddo

        end subroutine average_vector_to_centers
!======================================================================
! END AVERAGE_VECTOR_TO_CENTERS
!======================================================================


        end module operators

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
