!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module operators_hl

	use grid_metrics
	use operators
	use wrap_face_data
	use wrap_vertex_data
	use grid_mask
	use elliptic_solver

!======================================================================
! PURPOSE : this module is a collection of HIGHER LEVEL operators.
! these operators are built off of the "operators" module. They are 
! distinguished from the "operators" modules by the use of wrap_face_data
! and wrap_vertex_data, i.e. the need to do boundary updates in the process
! of computing the necessary quanities.
!
! NOTE : calls to wrap and wrap_vertices lead to message passing. These
! are effectively syncronization points.
!
! NOTE : since calls to wrap* are allowed, no subroutine in this module
! can be called inside a "do ksdm=1,nsdm" loop
!
! NOTE : the convention here is to wrap all intent(out) variables when
! wrapping is applicable.
!======================================================================

	implicit none

!*************************************************************************

        contains

!*************************************************************************


!======================================================================
! BEGIN CALCULATE_U_AND_KE
!======================================================================
        subroutine calculate_u_and_ke(                          &
                    kkm,                                        &
                    psi,                                        &
                    chi,                                        &
                    u,                                          &
                    ke   )

!-----------------------------------------------------------------------
! PURPOSE :  given psi and chi, find the velocity (at corners)
!  can kinetic energy (at cell centers)
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!                 : 
!                 : 
!                 : 
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                             &
                        kkm

        real (kind=dbl_kind), intent(in) ::                                &
                        psi             (iim,jjm,kkm,nsdm),                &
                        chi             (iim,jjm,kkm,nsdm)


!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(inout) ::                                  &
                        u               (nspace,ncorners,iim,jjm,kkm,nsdm),   &
                        ke              (                iim,jjm,kkm,nsdm)  


!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: k,ksdm
 	real (kind=dbl_kind) :: work_c(ncorners,iim,jjm,kkm,nsdm)
	real (kind=dbl_kind) :: work1_c(nspace,ncorners,iim,jjm,kkm,nsdm)

!----------------------------------------------------------------------------
! loop over subdomains and over vertical levels
!----------------------------------------------------------------------------
	work1_c = c0
	do ksdm = 1,nsdm
 	do k=1,kkm
	  call calculate_velocity(                                &
                ipad,                                             &
                iim,                                              &
                jjm,                                              &
                ndata,                                            &
                nspace,                                           &
                ncorners,                                         &
                corner_area_inv(:,:,:,ksdm),                      &
                v_weights(:,:,:,:,:,ksdm),                        &
                psi(:,:,k,ksdm),                                  &
                chi(:,:,k,ksdm),                                  &
                work1_c(:,:,:,:,k,ksdm) )
 	enddo
	enddo

 	u = work1_c

	call wrap_vertices(nspace,kkm,u(1,1,1,1,1,1))
	work_c(:,:,:,:,:) = p5 * (   u(1,:,:,:,:,:) * u(1,:,:,:,:,:)     &
                                   + u(2,:,:,:,:,:) * u(2,:,:,:,:,:)  )

	do ksdm=1,nsdm
 	do k=1,kkm

	  call calculate_center_kinetic_energy(                   &
                ipad,                                             &
                iim,                                              &
                jjm,                                              &
                ndata,                                            &
                nspace,                                           &
                ncorners,                                         &
                sub_area(:,:,:,:,ksdm),                           &
                area_inv(:,:,ksdm),                           &
                work_c(:,:,:,k,ksdm),                             &
                ke(:,:,k,ksdm)          )

        enddo
	enddo

!----------------------------------------------------------------------------
! wrap ke
!----------------------------------------------------------------------------
        call wrap (max_level, iim , jjm, kkm, nsdm, ke(1,1,1,1))

        end subroutine calculate_u_and_ke
!======================================================================
! END CALCULATE_U_AND_KE
!======================================================================


!======================================================================
! BEGIN CALCULATE_REL_AND_DIV
!======================================================================
        subroutine calculate_rel_and_div(                         &
                 kkm,                                             &
                 u,                                               &
                 relative,                                        &
                 divergence )
                                                                
!-----------------------------------------------------------------------
! PURPOSE : given an unwrapped velocity field, compute the relative
!  vorticity, divergence, and kinetic energy
!
! NOTE : this routine is generalized, u can be an arbitrary vector field
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!                 : 
!                 : 
!                 : 
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                             &
                        kkm

        real (kind=dbl_kind), intent(in) ::                                &
                        u             (nspace,ncorners,iim,jjm,kkm,nsdm)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                               &
                        relative                      (iim,jjm,kkm,nsdm),  &
                        divergence                    (iim,jjm,kkm,nsdm)
                        
!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: k,ksdm

!----------------------------------------------------------------------------
! initialize intent(out)
!----------------------------------------------------------------------------
	relative = 0.0_dbl_kind
	divergence = 0.0_dbl_kind

!----------------------------------------------------------------------------
! wrap the vector field
!----------------------------------------------------------------------------
	call wrap_vertices(nspace,kkm,u(1,1,1,1,1,1))

!----------------------------------------------------------------------------
! loop over subdomains and over vertical levels
!----------------------------------------------------------------------------
        do ksdm = 1,nsdm
        do k=1,kkm

	  call curl_of_a_vector(                                        &
                  ipad,                                                 &
                  iim,                                                  &
                  jjm,                                                  &
                  ndata,                                                &
                  nspace,                                               &
                  ncorners,                                             &
                  area_inv(:,:,ksdm),                               &
                  v_weights(:,:,:,:,:,ksdm),                            &
                  u(:,:,:,:,k,ksdm),                                    &
                  relative(:,:,k,ksdm)  )

          call div_of_a_vector(                                         &
                  ipad,                                                 &
                  iim,                                                  &
                  jjm,                                                  &
                  ndata,                                                &
                  nspace,                                               &
                  ncorners,                                             &
                  area_inv(:,:,ksdm),                               &
                  v_weights(:,:,:,:,:,ksdm),                            &
                  u(:,:,:,:,k,ksdm),                                    &
                  divergence(:,:,k,ksdm)  )
 	enddo
	enddo

!----------------------------------------------------------------------------
! wrap relative, divergence, and ke
!----------------------------------------------------------------------------
	call wrap (max_level, iim , jjm, kkm, nsdm, relative(:,:,:,:))
        call wrap (max_level, iim , jjm, kkm, nsdm, divergence(:,:,:,:))

        end subroutine calculate_rel_and_div
!======================================================================
! END CALCULATE_REL_AND_DIV
!======================================================================


!======================================================================
! BEGIN SCALAR_LAPLACIAN_TO_THE_N
!======================================================================
        subroutine scalar_laplacian_to_the_n(                        &
                          norder,                                    &
                          kkm,                                       &
                          xin,                                       &
                          xout   ) 

!-----------------------------------------------------------------------
! PURPOSE : given a scalar field (xin), compute del squre n time
!   i.e. if n = 3, then xout is del sixth of xin
!-----------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!                 : 
!                 : 
!                 : 
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
	integer (kind=int_kind), intent(in) ::                             &
                        norder,                                            &
                        kkm

        real (kind=dbl_kind), intent(in) ::                                &
                        xin             (iim,jjm,kkm,nsdm)

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind=dbl_kind), intent(out) ::                               &
                        xout            (iim,jjm,kkm,nsdm)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        integer (kind=int_kind) :: k,ksdm,n
	real (kind=dbl_kind) :: work(iim,jjm,kkm,nsdm)

!----------------------------------------------------------------------------
! copy xin into work
!----------------------------------------------------------------------------
	work = xin

!----------------------------------------------------------------------------
! loop over norder
!----------------------------------------------------------------------------
	do n = 1,norder

!----------------------------------------------------------------------------
! loop over nsdm and kkm
!----------------------------------------------------------------------------
	do ksdm = 1,nsdm
	do k=1,kkm

        call laplacian_of_a_scalar(                                    &
                ipad,                                                  &
                iim,                                                   &
                jjm,                                                   &
                nweights,                                              &
                l_weights(:,:,:,ksdm),                                 &
                work(:,:,k,ksdm),                                      &
                xout(:,:,k,ksdm)   )
	enddo
	enddo

!----------------------------------------------------------------------------
! wrap xout
!----------------------------------------------------------------------------
 	call wrap (max_level, iim , jjm, kkm, nsdm, xout(1,1,1,1))

!----------------------------------------------------------------------------
! copy xout into work
!----------------------------------------------------------------------------
        work = xout

	enddo

        end subroutine scalar_laplacian_to_the_n
!======================================================================
! END SCALAR_LAPLACIAN_TO_THE_N
!======================================================================


!======================================================================
! BEGIN LAPLACIAN_OF_A_VECTOR
!======================================================================
        subroutine laplacian_of_a_vector(      &
                kkm,      &
                norder,      &
                xin,      &
                xout)

!-----------------------------------------------------------------------
! PURPOSE : given a vector defined at cell corners, find the Laplacian.
!       this formulation uses the vector identity
!       del dot del (V) = grad (del dot V) - curl (curl (V) )
!----------------------------------------------------------------------------

        implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       xin             : vector (x)
!       xout            : Laplacian of xin (x*m^-1)
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                                   &
                        norder, kkm

        real (kind=dbl_kind), intent(in) ::                                      &
                        xin                   (nspace,ncorners,iim,jjm,kkm,nsdm)

        real (kind=dbl_kind), intent(inout)::                                    &
                        xout                  (nspace,ncorners,iim,jjm,kkm,nsdm)

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
        real (kind=dbl_kind) ::      &
                        work                                  (iim,jjm,nsdm),      &
                        work1_c               (nspace,ncorners,iim,jjm,nsdm),      &
                        work2_c               (nspace,ncorners,iim,jjm,nsdm)

        integer (kind=int_kind) :: i,j,k,ispace,icorner,iorder,ksdm

!----------------------------------------------------------------------------
! initialize, use xout as workspace
!----------------------------------------------------------------------------
	do ksdm=1,nsdm
        do k=1,kkm
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        do ispace=1,nspace
          xout(ispace,icorner,i,j,k,ksdm) = xin(ispace,icorner,i,j,k,ksdm)
        enddo
        enddo
        enddo
        enddo
        enddo
	enddo

!----------------------------------------------------------------------------
! loop over del2 operator norder times
!----------------------------------------------------------------------------
        do iorder = 1,norder

!----------------------------------------------------------------------------
! wrap data
!----------------------------------------------------------------------------
         call wrap_vertices(nspace,kkm,xout)

!----------------------------------------------------------------------------
! loop over k
!----------------------------------------------------------------------------
        do k = 1,kkm

!----------------------------------------------------------------------------
! compute grad (del dot V) and put in intent(out)
!----------------------------------------------------------------------------
	do ksdm = 1,nsdm
        call div_of_a_vector(      &
                ipad,      &
                iim,      &
                jjm,      &
                ndata,      &
                nspace,      &
                ncorners,      &
                area_inv(:,:,ksdm),      &
                v_weights(:,:,:,:,:,ksdm),      &
                xout(:,:,:,:,k,ksdm),      &
                work(:,:,ksdm) )
	enddo

        call wrap ( max_level,iim,jjm,1,nsdm,work(1,1,1))

	do ksdm =1,nsdm
        call gradient(      &
                ipad,      &
                iim,      &
                jjm,      &
                ndata,      &
                nspace,      &
                ncorners,      &
                corner_area_inv(:,:,:,ksdm),      &
                v_weights(:,:,:,:,:,ksdm),      &
                work(:,:,ksdm),      &
                work1_c(:,:,:,:,ksdm) )
	enddo

!----------------------------------------------------------------------------
! take the gradient of the curl of the velocity
!----------------------------------------------------------------------------
	do ksdm = 1,nsdm
        call curl_of_a_vector(      &
                ipad,      &
                iim,      &
                jjm,      &
                ndata,      &
                nspace,      &
                ncorners,      &
                area_inv(:,:,ksdm),      &
                v_weights(:,:,:,:,:,ksdm),      &
                xout(:,:,:,:,k,ksdm),      &
                work(:,:,ksdm) )
	enddo

	call wrap ( max_level,iim,jjm,1,nsdm,work(1,1,1))

	do ksdm = 1,nsdm
        call gradient(      &
                ipad,      &
                iim,      &
                jjm,      &
                ndata,      &
                nspace,      &
                ncorners,      &
                corner_area_inv(:,:,:,ksdm),      &
                v_weights(:,:,:,:,:,ksdm),      &
                work(:,:,ksdm),      &
                work2_c(:,:,:,:,ksdm) )       
	enddo

!----------------------------------------------------------------------------
! take cross product of this vector with k
!----------------------------------------------------------------------------
	do ksdm = 1,nsdm
        call k_cross_v(      &
                iim,      &
                jjm,      &
                nspace,      &
                ncorners,      &
                work2_c(:,:,:,:,ksdm),      &
                xout(:,:,:,:,k,ksdm))
	enddo


!----------------------------------------------------------------------------
! add this term to the intent(out) array
!----------------------------------------------------------------------------
	do ksdm=1,nsdm
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        do ispace = 1,nspace
          xout(ispace,icorner,i,j,k,ksdm) = +    xout(ispace,icorner,i,j,k,ksdm)      &
                                            + work1_c(ispace,icorner,i,j,  ksdm)
        enddo
        enddo
        enddo
        enddo
	enddo

        enddo           ! do k = 1,kkm

        enddo           ! do iorder=1,norder

        end subroutine laplacian_of_a_vector
!======================================================================
! END LAPLACIAN_OF_A_VECTOR
!======================================================================


        end module operators_hl

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
