      module gmeans_module

!  This file initializes, stores, computes and writes the diagnostic output
!     that is globally and daily averaged. This module was designed to 
!     provide the AMIP 2 required diagnostics, but others may be added
!     as well.

      use kinds
      use model_parameters
      use physical_parameters
      use time_manager_module

      implicit none
      save
      private

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!	PROCEDURE TO ADD A GLOBAL MEAN VARIABLE TO THE MODEL
!
!   	In this file:
!	1. Increase Nglob by 1
!	2. Add new entries in the tables for
!          -a variable named iglob_xxx, where xxx is the name in glob_name.
!               Add this to the list of iglob variables and assign it a value
!               1 larger than the previous one.
!       3. Go to subroutine initialize_globinfo further down in this file.
!          Add an assignment statement for each of these variables for
!          index iglob_xxx (these are down in a block):
!  	   -glob_name (the name of the field conforming, if possible, to 
!          	amip conventions
!          -glob_units ( the units of the field)
!          -glob_descr ( A longer description of the field)
!	In the file where the variable resides
!	4. Be sure that a "use gmeans_module" statement is in the subroutine
!       5. Add a gmean_sum statement for the field you wish to store.
!	The description of the gmean_sum argument list is in the file with 
!       subroutine gmean_sum
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!  tables of global mean information

!   number of installed available global mean, 
!     change these as more global means are added
      integer (kind=int_kind), parameter :: 
     *                                      nglob = 11

!   names are the variable names and conform to amip standards where possible
!   these are assigned in subroutine initialize_globinfo further down
      character (len=16), dimension(nglob) :: glob_name

!  units are the units the fields are stored in and are mks where possible
!   these are assigned in subroutine initialize_globinfo further down
      character (len=16), dimension(nglob) :: glob_units 

!  descr is a description of the field
!   these are assigned in subroutine initialize_globinfo further down
      character (len=80), dimension(nglob) :: glob_descr

      integer (kind=int_kind), dimension(nglob) :: glob_indx
      logical (kind=log_kind), dimension(nglob) :: glob_log = .false.

! these will point to  locations in the global mean information tables
! their values are assigned sequentially, and the last value is used
!   as the dimension of the above tables
      integer (kind=int_kind), parameter, public ::
     *     iglob_rmt	= 1
     *,    iglob_hfns	= 2
     *,    iglob_enek	= 3
     *,    iglob_moa	= 4
     *,    iglob_torts	= 5
     *,    iglob_ta	= 6
     *,    iglob_ps	= 7
     *,    iglob_evsps	= 8
     *,    iglob_snc	= 9
     *,    iglob_snd	= 10
     *,    iglob_tso	= 11

!   number of saved fields
      integer (kind=int_kind) :: iglob = 0

!   number of samples
      real (kind=dbl_kind), dimension(nglob) :: iglob_sample
      character (len=4), dimension(nglob) :: glob_sys

!   global mean data slots
      real (kind=dbl_kind), dimension(nglob) :: global_means
      real (kind=dbl_kind) :: ice_free_area

      public ::
     *   gmean_sum_atm
     *,	 initialize_globinfo ,  write_gmean_diagnostics
#ifndef no_pbl
     *,  gmean_sum_sfc
#endif
    

      contains

      subroutine initialize_globinfo

!  this subroutine assigns all the global mean information and opens the file
!    and initializes the data

      use gather_scatter

      implicit none

      character*8, time_label
      integer (kind=int_kind) :: monthofyear
      integer (kind=int_kind) :: n

! initialize
	glob_name(:) = " no name gmean"
	glob_units(:) = "no units gmean"
	glob_descr(:) = "no description gmean"
	glob_sys(:) = "atm"
      
!  do assignments for variable name, units and description 
        glob_name (iglob_rmt ) = "rmt"
        glob_units(iglob_rmt ) = "W/m^2"
        glob_descr(iglob_rmt ) = "Net radiation at model top (positive down)"
        glob_log  (iglob_rmt ) = .true.
	glob_sys  (iglob_rmt) = "atm"

        glob_name (iglob_hfns ) = "hfns"
        glob_units(iglob_hfns ) = "W/m^2"
        glob_descr(iglob_hfns ) = "Net downward energy flux at surfac3"
        glob_log  (iglob_hfns ) = .true.
	glob_sys  (iglob_hfns) = "atm"

        glob_name (iglob_enek ) = "enek"
        glob_units(iglob_enek ) = "W/m^2"
        glob_descr(iglob_enek ) = "Total kinetic energy (per unit area)"
        glob_log  (iglob_enek ) = .true.
	glob_sys  (iglob_enek) = "atm"

        glob_name (iglob_moa ) = "moa"
        glob_units(iglob_moa ) = "W/m^2"
        glob_descr(iglob_moa ) = "Total relative ang. momentum (per unit area)"
        glob_log  (iglob_moa ) = .true.
	glob_sys  (iglob_moa) = "atm"

        glob_name (iglob_torts ) = "torts"
        glob_units(iglob_torts ) = "W/m^2"
        glob_descr(iglob_torts ) = "Total surface torque (inc. mountain torque)"
        glob_log  (iglob_torts ) = .true.
	glob_sys  (iglob_torts) = "atm"

        glob_name (iglob_ta ) = "ta"
        glob_units(iglob_ta ) = "W/m^2"
        glob_descr(iglob_ta ) = "Temperature (mass-weighted vertically avgd)"
        glob_log  (iglob_ta ) = .true.
	glob_sys  (iglob_ta) = "atm"

        glob_name (iglob_ps ) = "ps"
        glob_units(iglob_ps ) = "W/m^2"
        glob_descr(iglob_ps ) = "Surface pressure"
        glob_log  (iglob_ps ) = .true.
	glob_sys  (iglob_ps) = "atm"

        glob_name (iglob_evsps ) = "evsps"
        glob_units(iglob_evsps ) = "W/m^2"
        glob_descr(iglob_evsps ) = "Evaporation and sublimation (per unit area)"
        glob_log  (iglob_evsps ) = .true.
	glob_sys  (iglob_evsps) = "atm"

        glob_name (iglob_snc ) = "snc"
        glob_units(iglob_snc ) = "W/m^2"
        glob_descr(iglob_snc ) = "Snow-covered area"
        glob_log  (iglob_snc ) = .true.
	glob_sys  (iglob_snc) = "sfc"

        glob_name (iglob_snd ) = "snd"
        glob_units(iglob_snd ) = "W/m^2"
        glob_descr(iglob_snd ) = "Snow depth (water equivalent)"
        glob_log  (iglob_snd ) = .true.
	glob_sys  (iglob_snd) = "sfc"

        glob_name (iglob_tso ) = "tso"
        glob_units(iglob_tso ) = "W/m^2"
        glob_descr(iglob_tso ) = "SST over open (ice-free) ocean"
        glob_log  (iglob_tso ) = .true.
	glob_sys  (iglob_tso) = "sfc"

	do n=1,nglob
	glob_name(n) = trim(glob_name(n))
	glob_units(n) = trim(glob_units(n))
	glob_descr(n) = trim(glob_descr(n))
	glob_sys(n) = trim(glob_sys(n))
	enddo

!  initialize some qp tables
        glob_indx = 0

        do n = 1,nglob
           if(glob_log(n)) then
             iglob = iglob + 1
             glob_indx(n) = iglob
           endif
         enddo

         if(my_task.eq.0) then
         write(6,*)
         do n = 1,nglob
           if(glob_log(n)) then
             write(6,11) glob_name(n), glob_descr(n),glob_indx(n)
 11          format('Will save gmean : ',a16,5x,a20, '   in gmean slot   ', i8)
           endif
         enddo
	 write(6,*)
         endif



!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

      iglob_sample = 0
      global_means = c0
      ice_free_area = c0

      if(my_task.eq.0)  then
        open(unit=93,
     *          file=trim(datapath)//'gmean_output/gmeans_'//trim(time_label),
     *		form='formatted')

      write(93,fmt='("            nday",20a16)')(trim(glob_name(n)),n=1,nglob)
      endif

      end subroutine initialize_globinfo





      subroutine gmean_sum_atm
     *  (xin, len, x_const, index, ksdm, ioffset, step_sample)

!  this subroutine accumulates a field in a section in the global means array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a global mean
!  x_const - a weight to scale xin
!  index - the global mean array index specified by iglob_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ksdm - the subdomain index
!  ioffset - (optional) the offset to the starting location in the 
!            qp2 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      use grid_metrics

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset
      integer (kind=int_kind), intent(in), optional ::
     *     step_sample

      real (kind=dbl_kind), dimension(len), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: n, offset, jbeg, jend, ibeg, iend, j, i
      real (kind=dbl_kind), dimension(len) :: area_mask_loc
      real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: area_mask
      

#ifdef scm
      return
#endif

      area_mask = area * grid_center_mask

      if(glob_log(index)) then
! update the sample size
         if(.not.present(step_sample)) then
           if(ksdm.eq.1) iglob_sample(index) = iglob_sample(index) + 1
         endif

         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1
         do j = jbeg,jend
           ibeg = 1 + max( 0, offset-(j-1)*iim )
           iend = min( iim, offset+len-(j-1)*iim )
           do i = ibeg,iend
             n = (j-1)*iim + i
             area_mask_loc(n-offset) = area_mask(i,j,ksdm)
           enddo
         enddo

! save the global mean array
         do n = 1,len
           global_means(index) = global_means(index)
     *             + x_const * xin(n) * area_mask_loc(n)
         enddo

      endif !  if(glob_log(index))

      end subroutine gmean_sum_atm

#ifndef no_pbl
      subroutine gmean_sum_sfc
     *  (xin, len, x_const, index, ksdm, ioffset, iz, step_sample)

!  this subroutine accumulates a field in a section in the global means array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a global mean
!  x_const - a weight to scale xin
!  index - the global mean array index specified by iglob_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ksdm - the subdomain index
!  ioffset - (optional) the offset to the starting location in the 
!            global_means field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      use domain_decomposition_sfc

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset
      integer (kind=int_kind), intent(in), optional ::
     *     step_sample

      real (kind=dbl_kind), dimension(len), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), dimension(len), intent(in), optional ::
     *      iz  !  seaice field
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: n, offset, jbeg, jend, ibeg, iend, j, i
      real (kind=dbl_kind), dimension(len) :: area_mask_loc

#ifdef scm
      return
#endif

      if(glob_log(index)) then
! update the sample size
         if(.not.present(step_sample)) then
           if(ksdm.eq.1) iglob_sample(index) = iglob_sample(index) + 1
         endif

         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         jbeg = offset/iim_sfc + 1
         jend = (offset+len-1)/iim_sfc+1
         do j = jbeg,jend
           ibeg = 1 + max( 0, offset-(j-1)*iim_sfc )
           iend = min( iim_sfc, offset+len-(j-1)*iim_sfc )
           do i = ibeg,iend
             n = (j-1)*iim_sfc + i
             if(grid_mask_sfc(i,j,ksdm)) then
                area_mask_loc(n-offset) = area_sfc(i,j,ksdm)
             else
                area_mask_loc(n-offset) = 0.0
             endif
           enddo
         enddo
! save the global mean array
         if(index.eq.iglob_tso) then
! ice free sst special case
           if(iz(n).lt.1.e-4_dbl_kind) then
           do n = 1,len
             global_means(index) = global_means(index)
     *             + x_const * xin(n) * area_mask_loc(n)
             ice_free_area = ice_free_area + area_mask_loc(n) * x_const
           enddo
           endif
         else
           do n = 1,len
             global_means(index) = global_means(index)
     *             + x_const * xin(n) * area_mask_loc(n)
           enddo
         endif


      endif !  if(glob_log(index))

      end subroutine gmean_sum_sfc

#endif


      subroutine write_gmean_diagnostics

        use domain_decomposition
        use grid_metrics

      implicit none

      real (kind=dbl_kind) :: nday, area_mask(iim,jjm,nsdm)
      integer (kind=int_kind) :: n, i

! compute the mask
      area_mask = area * grid_center_mask

! sum up means over processors
      i = 0
      do n = 1,nglob
         if(glob_log(n)) then
            if(glob_sys(n).eq.'atm') then
               call gsum_atm(global_means(n))
            else if(glob_sys(n).eq.'sfc') then
#ifdef no_pbl
               call gsum_atm(global_means(n))
#else
               call gsum_sfc(global_means(n))
#endif
            endif
            if(iglob_sample(n).gt.0.5_dbl_kind) then
              iglob_sample(n) = 1.0 / iglob_sample(n)
            else
              iglob_sample(n) = 0.0
            endif
         else
           iglob_sample(n) = 0.0
         endif
      enddo
#ifdef no_pbl
      if(glob_log(iglob_tso)) call gsum_atm(ice_free_area)
#else
      if(glob_log(iglob_tso)) call gsum_sfc(ice_free_area)
#endif
! normalize by sample
      global_means = global_means * iglob_sample
      ice_free_area = ice_free_area * iglob_sample(iglob_tso)

! get global area and normalize to get global mean
      tot_area = sum(area_mask)
      call gsum_atm(tot_area)
      global_means = global_means / tot_area
#ifdef link_to_gcm_physics
      global_means(iglob_tso) = global_means(iglob_tso)
     *		 * tot_area / ice_free_area
#endif

! write to the file
      nday = float(nsecond_dyn) / 86400.
      if(my_task.eq.0)
     * write(93,'(f16.5,20e16.8)')nday, global_means

!   zero out for next period of summation

      global_means = c0
      ice_free_area = c0
      iglob_sample = 0

      contains

        subroutine gsum_atm(global_sum)


        real (kind=dbl_kind), intent(inout) :: global_sum

	real (kind=dbl_kind) :: local_sum

#ifdef mpi
        local_sum = global_sum
        call MPI_ALLREDUCE(local_sum, global_sum, 1,
     &                   mpi_float_type, MPI_SUM, MPI_COMM_ATMOS, ierr)
#endif
        end subroutine gsum_atm

#ifndef no_pbl
        subroutine gsum_sfc(global_sum)

        use domain_decomposition_sfc

        real (kind=dbl_kind), intent(inout) :: global_sum

	real (kind=dbl_kind) :: local_sum

#ifdef mpi
        local_sum = global_sum
        call MPI_ALLREDUCE(local_sum, global_sum, 1,
     &                   mpi_float_type, MPI_SUM, MPI_COMM_SFC, ierr)
#endif
        end subroutine gsum_sfc
#endif

      end subroutine write_gmean_diagnostics



      end module gmeans_module
