!-----------------------------------------------------------------------

	MODULE netcdf_general_tools
	
!-----------------------------------------------------------------------
!  Purpose: 
!   contains parameters and procedures used by both qp and pbp 
!   procedures:
!   	module parameters: 
!	   maximum length of character variables,
!	   constant character names, 
!	   missing value
!	   return code and counter
!  	module procedures:
!	   get_units - extracts a units substring 
!	   set_time - converts time to NetCDF time, writes to file
!	   set_frac - writes frac values
!	   error_handler - deals with netcdf errors
!	   current_time - returns the current time and date as a string
!	netcdf parameters and functions - contained via include
!
!  Scope:
!	all parameters, variables and procedures are public
!
!-----------------------------------------------------------------------

        use kinds
        use netcdf_util

	implicit none
	
!   module parameters (character variable limits) also public
	integer, parameter :: MAXFILE=128, 
     *			      MAXUNITS=40, 
     *			      MAXLONGN=80,
     *			      MAXNAME=16, 
     *			      STRLEN = 10

!   missing value 
	real (kind=real_kind), parameter :: MISSING=1e36


!   derived type to hold metadata info:
!   ni, nj are only meaningful on rectangular (lat/lon) grid structures
	type meta_type
	   character(len=MAXFILE) :: filename
	   character(len=4) :: datatype
	   integer (kind=int_kind) :: ni
	   integer (kind=int_kind) :: nj
	   integer (kind=int_kind) :: npoints
	   integer (kind=int_kind) :: timerec
	   integer (kind=int_kind) :: nlevels
	end type meta_type

!        public :: meta_type, parameters


	CONTAINS

!-----------------------------------------------------------------------
!
	FUNCTION set_meta (fname, dtype, ncells, i, j, nlevels)
!	
!-----------------------------------------------------------------------

        type(meta_type) :: set_meta
        character(len=*), intent(in) :: fname
        character(len=4), intent(in) :: dtype
	integer (kind=int_kind), intent(in) :: ncells
	integer (kind=int_kind), intent(in) :: i, j
	integer (kind=int_kind), intent(in), optional :: nlevels

        set_meta%filename = fname
        set_meta%datatype = dtype
	set_meta%npoints = ncells
	set_meta%ni = i
	set_meta%nj = j
	set_meta%timerec = 0

        if (present(nlevels)) then
	  set_meta%nlevels = nlevels
        else
          set_meta%nlevels = 0
        end if

	END FUNCTION set_meta



!-----------------------------------------------------------------------

        SUBROUTINE next_time_step (geo_meta)

!-----------------------------------------------------------------------

        type(meta_type), intent(inout) :: geo_meta

        geo_meta%timerec = geo_meta%timerec + 1

	END SUBROUTINE next_time_step



!-----------------------------------------------------------------------

	FUNCTION get_time_step(geo_meta)

!-----------------------------------------------------------------------

	integer (kind=int_kind) :: get_time_step
	type(meta_type), intent(in) :: geo_meta

	get_time_step = geo_meta%timerec

	END FUNCTION get_time_step



!-----------------------------------------------------------------------

	FUNCTION file_name(geo_meta)

!-----------------------------------------------------------------------

	character(len=MAXFILE) :: file_name
	type(meta_type), intent(in) :: geo_meta

	file_name = geo_meta%filename

	END FUNCTION file_name



!-----------------------------------------------------------------------

	SUBROUTINE get_units(description, units, long_len, 
     *			     unit_len)
	
!-----------------------------------------------------------------------
!  Purpose:
!	extracts a string enclosed within parentheses - used for 
!	units which are contained in a general description string.
!	returns the units string (units) and its length (unit_len),
!	the description string with the units removed (longname),
!	and its length (long_len).
!	note: embedded parentheses are ok
!
!  Variables:
!   Module parameters used:
!	MAXUNITS, MAXLONGN
!	
!  Bugs:
!	1) if the rightmost parenthesis is unmatched, units will be 
!	   set to " " (one space) - this is to be interpreted as "none"
!	2) if a "(" is unmatched, it will be part of the returned
!	   longname string
!       3) strings of only units (i.e. entire string enclosed in 
!	   parentheses) do not work.
!-----------------------------------------------------------------------
	
	character(len=*), intent(in) :: description
	character(len=*), intent(in) :: units
	integer (kind=int_kind), intent(out) :: unit_len, long_len
	

	   long_len = len_trim(description)
	   unit_len = len_trim(units)

	END SUBROUTINE get_units



!-----------------------------------------------------------------------

	SUBROUTINE define_time (fileID, timeID, doperiod)

!-----------------------------------------------------------------------
! Purpose:
!   defines (unlimited) time dimension and various time related variables
!   returns the netcdf id of the time dimension
!   dimension defined:
!       time
!       char_len
!   variables defined:
!       time
!       char_time
!       start_period
!       end_period
!       period_length
!
! Scope: 
!   Module variables used:
!      netcdf routines and constants
!
! FIXME - pbps do not need period_start, period_length, etc.
!-----------------------------------------------------------------------

	integer (kind=int_kind), intent(in) :: fileID
	integer (kind=int_kind), intent(out) :: timeID
	logical (kind=log_kind), intent(in) :: doperiod

        integer (kind=int_kind) :: tmpid, chardimID
        integer (kind=int_kind), dimension(1) :: tmparr
        integer (kind=int_kind), dimension(2) :: char_dims

        integer (kind=int_kind) :: status

	status = nf_def_dim (fileID, "time", NF_UNLIMITED, timeID)
	status = nf_def_dim (fileID, "char_len", STRLEN, chardimID)

        tmparr = (/timeID/)
	char_dims = (/chardimID, timeID/)
	   
	status = nf_def_var (fileID, "time", NF_DOUBLE, 1, tmparr, tmpid)
	status = nf_put_att_text (fileID, tmpid, "quantity", 4, "time")
	status = nf_put_att_text (fileID, tmpid, "units", 16,
     *			"days since 1-1-1")
	status = nf_put_att_text (fileID, tmpid, "calendar", 6, "noleap")
	
	status = nf_def_var (fileID, "char_time", NF_CHAR, 2,
     *				char_dims, tmpid)
	status = nf_put_att_text (fileID, tmpid, "format", 10,
     *				 "mm/dd/yyyy")

        if (doperiod) then
	  status = nf_def_var (fileID, "start_period", NF_DOUBLE, 1, tmparr, 
     *			       tmpid)
	  status = nf_put_att_text (fileID, tmpid, "long_name", 24, 
     *			  	   "start of averaged period")
	  status = nf_put_att_text (fileID, tmpid, "units", 16,
     *			            "days since 1-1-1")

	  status = nf_def_var (fileID, "end_period", NF_DOUBLE, 1, tmparr,
     *			       tmpid)
	  status = nf_put_att_text (fileID, tmpid, "long_name", 22,
     *				    "end of averaged period")
	  status = nf_put_att_text (fileID, tmpid, "units", 16,
     *			            "days since 1-1-1")
	
	  status = nf_def_var (fileID, "period_length", NF_DOUBLE, 1, tmparr,
     *			       tmpid)
	  status = nf_put_att_text (fileID, tmpid, "long_name", 25,
     *				    "length of averaged period")
	  status = nf_put_att_text (fileID, tmpid, "units", 4,
     *			            "days")
        end if


	END SUBROUTINE define_time




!-----------------------------------------------------------------------

	SUBROUTINE set_time(qpfid, nsecond, timerec, nyear0, nsec_start)
	
!-----------------------------------------------------------------------
!  Purpose: 
!	sets the time in a netcdf file as a string and an 8 byte float
!
!  Variables:
!   Module parameters used:
!	STRLEN
!
!  Assumptions:
!	1) qpfid is the id number of a netcdf file that is open in data mode
!	2) the netcdf file has time variables named "time" and "char_time"
!-----------------------------------------------------------------------
	
	integer (kind=int_kind), intent(in) :: qpfid, timerec, nsecond
	integer (kind=int_kind), intent(in) :: nyear0
	integer (kind=int_kind), intent(in), optional :: nsec_start
	
	integer (kind=int_kind) :: nseconds_coord
	real (kind=selected_real_kind(13)) :: nctime
	character(len=STRLEN) :: character_time	
	integer (kind=int_kind) :: cstart(2), ccount(2), timevid, chartimevid
	integer (kind=int_kind) :: rcode

	cstart = (/1, timerec/)   
	ccount = (/STRLEN, 1/)

	nseconds_coord = nsecond

!   if necessary, store time start and time end, calculate average
	if (present(nsec_start)) then
	    rcode = nf_inq_varid (qpfid, "start_period", timevid)
	    call convert_time(nsec_start)
	    rcode = nf_put_var1_double (qpfid, timevid, timerec, nctime)

	    rcode = nf_inq_varid (qpfid, "end_period", timevid)
	    call convert_time(nsecond)
	    rcode = nf_put_var1_double (qpfid, timevid, timerec, nctime)
	
	    rcode = nf_inq_varid (qpfid, "period_length", timevid)
	    call convert_time(nsecond-nsec_start)
	    rcode = nf_put_var1_double (qpfid, timevid, timerec, nctime)
	
	    nseconds_coord = (nsec_start + nsecond)/2
	endif

!   translate time into standard netcdf time and character string date
	call convert_time(nseconds_coord) 
	
!   get time variable id's
	rcode = nf_inq_varid (qpfid, "time", timevid)
	rcode = nf_inq_varid (qpfid, "char_time", chartimevid)
	
!   store converted time in appropriate variables
	rcode = nf_put_var1_double (qpfid, timevid, timerec,
     *		  		    nctime)
	rcode = nf_put_vara_text (qpfid, chartimevid, cstart, ccount,
     *		  		  character_time)



	CONTAINS	

!---------------------------------------------------------------

	SUBROUTINE convert_time(secs)   !!! contained in set_time !!!
	
!---------------------------------------------------------------
! Purpose:
!     This subroutine converts the time stored in nsecond to standard
!   netcdf time and to a character date string of the format "mm/dd/yyyy"
!
! Scope:
!    Variables from containing subroutine scope:
!	nctime, 
!       character_time, 
!       amip
!
! Assumptions:  
!	1) nsecond is a four-byte integer number of seconds
!	2) no leap years
!----------------------------------------------------------------

	integer (kind=int_kind), intent(in) :: secs
 
	integer (kind=int_kind) :: day, year, seconds_rem
	character (len=2) :: month, cday
	character (len=4) :: cyear
	character (len=6) :: line   ! internal file 

! amip code modified for amip2, for amip1 add 1977 instead of 1976
! instead of testing for amip, nyear0 has amip info within it
!  nctime section 
	nctime = dble(secs)/86400.0
!  if start date = 1/1/1977, add 1976 years	
cif (amip) then
#ifndef scm
!	   nctime = nctime + 365*(nyear0-1)
#endif
cendif

!  character time section	
	seconds_rem = secs
!  calculate years
	year = seconds_rem/(365*86400) + 1
cif (amip) then
	   year = year+nyear0-1
cendif
	
	seconds_rem = mod(seconds_rem, 365*86400)   ! seconds remaining 
	day = seconds_rem/86400 + 1
	
	if (day<=31) then
	   month = "01"
	elseif (day<=59) then  ! no leap years (feb. always 28 days)
	   month = "02"
	   day = day - 31
	elseif (day<=90) then
	   month = "03"
	   day = day - 59
	elseif (day<=120) then
	   month = "04"
	   day = day - 90
	elseif (day<=151) then
	   month = "05"
	   day = day - 120
	elseif (day<=181) then
	   month = "06"
	   day = day - 151
	elseif (day<=212) then
	   month = "07"
	   day = day - 181
	elseif (day<=243) then
	   month = "08"
	   day = day - 212
	elseif (day<=273) then
	   month = "09"
	   day = day - 243
	elseif (day<=304) then
	   month = "10"
	   day = day - 273
	elseif (day<=335) then
	   month = "11"
	   day = day - 304
	else
	   month = "12"
	   day = day - 334
	endif
	
!  internal file used to convert integer data to character string
        write (unit=line, fmt='(I2.2, I4.4)') day, year
        read (unit=line, fmt='(A2, A4)') cday, cyear
    	
	character_time = month//"/"//cday//"/"//cyear
     	

	END SUBROUTINE convert_time
	
	END SUBROUTINE set_time   !!! back to module scope !!!



!-----------------------------------------------------------------------

	FUNCTION current_time ( )
	
!-----------------------------------------------------------------------
!   Purpose:
!	returns a 30 character string containing the current time and date
!	in the form "mm/dd/yyyy at hh:mm zzzzz GMT"
!
!   Bugs:
!	1) date_and_time intrinsic f90 routine is not supported on
!	   all f90 compilers
!-----------------------------------------------------------------------

	character(len=30) :: current_time
	
	character(len=8) :: date
	character(len=10) :: time
	character(len=5) :: zone
	integer (kind=int_kind), dimension(8) :: values

	call date_and_time(date, time, zone, values)

	current_time = date(5:6) // "/" // date(7:8) // "/" // date(1:4)
     *			//" at "// time(1:2) // ":" // time(3:4) // " "
     *			// zone // " GMT "

	END FUNCTION current_time



!-----------------------------------------------------------------------
	
	SUBROUTINE add_global_atts (fileID, runname, grid, version, max_ig)
	
!-----------------------------------------------------------------------
! Purpose:
!   sets global attributes 
!
! Scope: 
!   Module variables used:
!
! Bugs:
!   netcdf file (fileID) must be in define mode.
!
!FIXME - geodesic specific stuff needs to go...
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: fileID
        integer (kind=int_kind), intent(in), optional :: max_ig
        character(len=*), intent(in) :: runname
        character(len=*), intent(in) :: grid
        character(len=*), intent(in) :: version

        integer (kind=int_kind) :: status

!   add standard global attributes
	status = nf_put_att_text (fileID, NF_GLOBAL, "calendar", 6,
     *			"noleap")
        call handle_error (status, "calendar")

	status = nf_put_att_text (fileID, NF_GLOBAL, "institution", 25,
     *			"Colorado State University")
        call handle_error (status, "institution")

	status = nf_put_att_text (fileID, NF_GLOBAL, "history", 39,
     *			         "Created: "//current_time())
        call handle_error (status, "history")

	status = nf_put_att_text(fileID, NF_GLOBAL, "run", len(runname),
     *			         runname)
        call handle_error (status, "run")

        status = nf_put_att_text (fileID, NF_GLOBAL, "grid", len(grid), 
     *                            grid)
        call handle_error (status, "grid")

        status = nf_put_att_text(fileID, NF_GLOBAL, "version", 
     *                           len(version), version)
        call handle_error (status, "version")

        status = nf_put_att_text(fileID, NF_GLOBAL, "extravaratts", 3, "yes")
        call handle_error (status, "extravaratts")

        if(present(max_ig))then
           status = nf_put_att_int(fileID, NF_GLOBAL, "total_grid_size", 
     *                           nf_int, 1, max_ig)
           call handle_error (status, "total_grid_size")
        endif

#ifdef scm
	status = nf_put_att_text (fileID, NF_GLOBAL, "scm", 3,
     *			"SCM")
        call handle_error (status, "scm")
#endif


	END SUBROUTINE add_global_atts



	END MODULE netcdf_general_tools	

