      module pbp_module

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!   This file stores pbp variables and utility routines for 
!     initializing and storing them.
!   PBP variables are time-series diagnostics saved at user selected
!     points.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      use kinds
      use model_parameters
      use domain_decomposition
      use sigma_coordinate
#ifdef scm
      use scm_module
#else
      use grid_metrics
      use gather_scatter
#endif
      use netcdf_pbp
      use time_manager_module

      implicit none 
      save
      private

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!	INPUT FILES FOR PBP DATA
!          data/atm_pbpopts - to specify which fields are saved
!          data/pbp_locs - lon,lat pairs to specify which grid points
!              data are saved at. Units are degrees E, N respectively.
!              one pair per record
!
!	PROCEDURE TO ADD A PBP VARIABLE TO THE MODEL
!
!       Adding a pbp2 (2-dimensional field)
!   	In this file:
!	1. Increase Npbp2 by 1
!	2. Add new entries in the tables for
!          -a variable named ipbp2_xxx, where xxx is the name in pbp2_name.
!               Add this to the list of ipbp2 variables and assign it a value
!               1 larger than the previous one.
!       3. Go to subroutine initialize_pbpinfo further down in this file.
!          Add an assignment statement for each of these variables for
!          index ipbp2_xxx (these are down in a block):
!  	   -pbp2_name (the name of the field conforming, if possible, to 
!          	amip conventions
!          -pbp2_units ( the units of the field)
!          -pbp2_descr ( A longer description of the field)
!	In the file where the variable resides
!	4. Be sure that a "use pbp_module" statement is in the subroutine
!       5. Add a pbp2sum statement for the field you wish to store.
!	The description of the qp3sum argument list is in the file with 
!       subroutine pbp2sum
!
!       Adding a pbp1 (horizontal only field)
!	Use the instructions above, replacing pbp2 with pbp1
!
!       Hints:
!       -If the field you want to save is a linear combination of model
!        variables you can use multiple pbpp*sum calls with the appropriate 
!        weights.
!        Example - reflected TOA SW is swinc - dswtop. Use a pbp1sum call
!        for swinc with a weight of 1.0 and a call for dswtop with a weight
!        of -1.0
!       NOTE - some routines are not called every timestep. Zeros will be 
!        in the pbp slots called from such routines in the timesteps
!        they are not called. These time steps can be filtered out in 
!        post-processing (e.g. using NCO operators)
!       ADDITIONAL NOTE - if a pbp call is made from physics or radgcm,
!        pbp*_physics must be set true to account for the fact that the 
!        summation may not be done avery time step. In addition, the scaling
!        factor passed into the call must be multipled by the
!        min(n_pbp,n_physics).
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!  tables of pbp information

!   number of installed available pbps, change these as more pbps are added
      integer (kind=int_kind), parameter :: &
                                            npbp2 = 69, &
                                            npbp1 = 82

!   number of samples
      integer (kind=int_kind), public :: ipbp_sample

!   names are the variable names and conform to amip standards where possible
!   these are assigned in subroutine initialize_pbpinfo further down
      character (len=16), dimension(npbp1) :: pbp1_name
      character (len=16), dimension(npbp2) :: pbp2_name

!  units are the units the fields are stored in and are mks where possible
!   these are assigned in subroutine initialize_pbpinfo further down
      character (len=16), dimension(npbp1) :: pbp1_units 
      character (len=16), dimension(npbp2) :: pbp2_units

!  descr is a description of the field
!   these are assigned in subroutine initialize_pbpinfo further down
      character (len=80), dimension(npbp1) :: pbp1_descr
      character (len=80), dimension(npbp2) :: pbp2_descr

!  type denotes which type of variable this is (for ezplot)
      character (len=3), dimension(npbp1), public :: pbp1_type
      character (len=3), dimension(npbp2), public :: pbp2_type

!  physics is true for fields accumulated on the physics time step
!   these are assigned in subroutine initialize_pbpinfo further down

      integer (kind=int_kind), dimension(npbp1) :: pbp1_indx
      integer (kind=int_kind), dimension(npbp2) :: pbp2_indx
      logical (kind=log_kind), dimension(npbp1) :: pbp1_log
      logical (kind=log_kind), dimension(npbp2) :: pbp2_log
      logical (kind=log_kind), dimension(npbp1) :: pbp1_physics
      logical (kind=log_kind), dimension(npbp2) :: pbp2_physics

! these will point to  locations in the pbp information tables
! their values are assigned sequentially, and the last value is used
!   as the dimension of the above tables
      integer (kind=int_kind), parameter, public :: &
           ipbp1_ps         = 1, &
           ipbp1_clblp      = 2, &
           ipbp1_blp        = 3, &
           ipbp1_rls        = 4, &
           ipbp1_rss        = 5, &
           ipbp1_rst	    = 6, &
           ipbp1_rlt	    = 7, &
           ipbp1_rsdt	    = 8, &
           ipbp1_spdm	    = 9, &
           ipbp1_rlds	    = 10, &
           ipbp1_blent	    = 11, &
           ipbp1_huss	    = 12, &
           ipbp1_clbl	    = 13, &
           ipbp1_rsds	    = 14, &
           ipbp1_dryconvinc = 15, &
           ipbp1_hfss	    = 16, &
           ipbp1_tradgrdeff = 17, &
           ipbp1_psl        = 18, &
           ipbp1_tas	    = 19, &
           ipbp1_hfls	    = 20, &
           ipbp1_rsus	    = 21, &
           ipbp1_rlus	    = 22, &
           ipbp1_rsut	    = 23, &
           ipbp1_rsdscs	    = 24, &
           ipbp1_rsuscs	    = 25, &
           ipbp1_rldscs	    = 26, &
           ipbp1_rlutcs	    = 27, &
           ipbp1_rsutcs	    = 28, &
           ipbp1_clt	    = 29, &
           ipbp1_leakp	    = 30, &
           ipbp1_leakpinc   = 31, &
           ipbp1_leakn	    = 32, &
           ipbp1_leakninc   = 33, &
           ipbp1_pbldmf	    = 34, &
           ipbp1_fillvq     = 35, &
           ipbp1_fillvcw    = 36, &
           ipbp1_fillvci    = 37, &
           ipbp1_fillvrw    = 38, &
           ipbp1_fillvri    = 39, &
           ipbp1_filliq     = 40, &
           ipbp1_fillicw    = 41, &
           ipbp1_fillici    = 42, &
           ipbp1_fillirw    = 43, &
           ipbp1_filliri    = 44, &
           ipbp1_filllq     = 45, &
           ipbp1_filllcw    = 46, &
           ipbp1_filllci    = 47, &
           ipbp1_filllrw    = 48, &
           ipbp1_filllri    = 49, &
           ipbp1_cuppr      = 50, &
           ipbp1_cupprpbl   = 51, &
           ipbp1_cupmb      = 52, &
           ipbp1_cape       = 53, &
           ipbp1_elqwvt     = 54, &
           ipbp1_elqcwt     = 55, &
           ipbp1_elqrwt     = 56, &
           ipbp1_elqcit     = 57, &
           ipbp1_elqrit     = 58, &
           ipbp1_elqprr     = 59, &
           ipbp1_elqprs     = 60, &
           ipbp1_elqpr      = 61, &
           ipbp1_totpr      = 62, &
           ipbp1_sumelq     = 63, &
           ipbp1_qadvtint   = 64, &
           ipbp1_tskin      = 65, &
           ipbp1_th1        = 66, &
           ipbp1_cupinc1    = 67, &
           ipbp1_cupinc2    = 68, &
           ipbp1_cupinc3    = 69, &
           ipbp1_cupinc4    = 70, &
           ipbp1_cupinc5    = 71, &
           ipbp1_cupinc6    = 72, &
           ipbp1_cupinc7    = 73, &
           ipbp1_cupinc8    = 74, &
           ipbp1_cupinc9    = 75, &
           ipbp1_cupinc10   = 76, &
           ipbp1_cupinc11   = 77, &
           ipbp1_cupinc12   = 78, &
           ipbp1_cupinc13   = 79, &
           ipbp1_cupinc14   = 80, &
           ipbp1_cupinc15   = 81, &
           ipbp1_cupinc16   = 82

      integer (kind=int_kind), parameter, public :: &
           ipbp2_pl         = 1, &
           ipbp2_pl2        = 2, &
           ipbp2_zg         = 3, &
           ipbp2_ta         = 4, &
           ipbp2_vorticity  = 5, &
           ipbp2_divergence = 6, &
           ipbp2_tntsw      = 7, &
           ipbp2_tntlw      = 8, &
           ipbp2_hur        = 9, &
           ipbp2_tntdc      = 10, &
           ipbp2_qsat       = 11, &
           ipbp2_o3         = 12, &
           ipbp2_tntswcs    = 13, &
           ipbp2_tntlwcs    = 14, &
           ipbp2_cl         = 15, &
           ipbp2_ua         = 16, &
           ipbp2_va         = 17, &
           ipbp2_wap        = 18, &
           ipbp2_tnqdc      = 19, &
           ipbp2_fill3q     = 20, &
           ipbp2_fill3cw    = 21, &
           ipbp2_fill3ci    = 22, &
           ipbp2_fill3rw    = 23, &
           ipbp2_fill3ri    = 24, &
           ipbp2_hhl        = 25, &
           ipbp2_hhls       = 26, &
           ipbp2_ssl        = 27, &
           ipbp2_ssvl       = 28, &
           ipbp2_cke        = 29, &
           ipbp2_cdwork     = 30, &
           ipbp2_masflx     = 31, &
           ipbp2_dtcup      = 32, &
           ipbp2_dqcup      = 33, &
           ipbp2_dhcup      = 34, &
           ipbp2_dvcup      = 35, &
           ipbp2_dlcup      = 36, &
           ipbp2_drcup      = 37, &
           ipbp2_dicup      = 38, &
           ipbp2_dscup      = 39, &
           ipbp2_tcuph      = 40, &
           ipbp2_tcupv      = 41, &
           ipbp2_tcupl      = 42, &
           ipbp2_tcupr      = 43, &
           ipbp2_tcupi      = 44, &
           ipbp2_tcups      = 45, &
           ipbp2_cwelq      = 46, &
           ipbp2_rwelq      = 47, &
           ipbp2_cielq      = 48, &
           ipbp2_rielq      = 49, &
           ipbp2_wvelq      = 50, &
           ipbp2_dtelq      = 51, &
           ipbp2_dqelq      = 52, &
           ipbp2_dvelq      = 53, &
           ipbp2_dlelq      = 54, &
           ipbp2_drelq      = 55, &
           ipbp2_dielq      = 56, &
           ipbp2_dselq      = 57, &
           ipbp2_drwfelq    = 58, &
           ipbp2_drifelq    = 59, &
           ipbp2_scmvdelt   = 60, &
           ipbp2_scmvdelq   = 61, &
           ipbp2_q1         = 62, &
           ipbp2_q2         = 63, &
           ipbp2_tadiff     = 64, &
           ipbp2_trlxterm   = 65, &
           ipbp2_qrlxterm   = 66, &
           ipbp2_trevterm   = 67, &
           ipbp2_qrevterm   = 68, &
           ipbp2_qdiff      = 69

!   number of saved fields
      integer (kind=int_kind) :: ipbp2 = 0, ipbp1 = 0

!   number of saved gridpoints (per processor)
      integer (kind=int_kind) :: ijlen = 0
!   number of saved gridpoints (per subdomain)
      integer (kind=int_kind), dimension(nsdm) :: kijlen = 0
!   offset to first saved gridpoint in subdomain
      integer (kind=int_kind), dimension(nsdm) :: koffsetpbp = 0

!   pbp data slots
      real (kind=real_kind), allocatable, dimension(:,:,:) :: pbp2
      real (kind=real_kind), allocatable, dimension(:,:)   :: pbp1

!   pbp locations
      real (kind=real_kind), allocatable, dimension(:)   :: &
                  lonpbp, latpbp
      integer (kind=int_kind), allocatable, dimension(:) :: &
                  ipbploc, jpbploc, ksdmpbploc

      public :: &
          initialize_pbpinfo, &
          initialize_pbpfiles, &
          write_pbp_diagnostics, &
          pbp1sum, &
          pbp2sum

      contains


      subroutine initialize_pbpinfo

!  this subroutine assigns all the pbp information from the input file atm_pbpopts

      implicit none

      integer (kind=int_kind) :: ldummy, ierr, n, i, item, j, ksdm, ireturn=0
      logical (kind=log_kind) :: dopbptem
      character (len=16) :: nametem
      real (kind=dbl_kind) :: lattem, lontem, xtem, ytem, ztem, dist, xdist
      real (kind=dbl_kind), dimension(iim, jjm, nsdm) :: pbp_loc
      real (kind=dbl_kind), dimension(im,jm) :: pbp_global, lat_global, lon_global
      real (kind=dbl_kind), dimension(max_ig) :: &
            pbp_g, lat_g, lon_g, x_phys, y_phys, z_phys

! read pbp location data

#ifdef scm
! always save pbp for scm
        ijlen = 1
        kijlen = 1
#else
!-----------------------------------------------------------------------
! open file, check error status
!-----------------------------------------------------------------------
        call gather_fine(grid_center_lat(:,:,:), lat_global)
        call gather_fine(grid_center_lon(:,:,:), lon_global)
        if (my_task == 0) then
          pbp_global = c0
          pbp_g = c0
          lat_g = c0
          lon_g = c0
          call reduce_to_vector_global(lat_global(:,:), lat_g(:))
          call reduce_to_vector_global(lon_global(:,:), lon_g(:))
!-- get x,y,z for atm grid points
          do i = 1,max_ig
             z_phys(i) = cos(lat_g(i))
             x_phys(i) = cos(lon_g(i)) * z_phys(i)
             y_phys(i) = sin(lon_g(i)) * z_phys(i)
             z_phys(i) = sin(lat_g(i))
          enddo
          lon_g = lon_g * 180. / pi
          lat_g = lat_g * 180. / pi

          open(unit=2,file=trim(datapath)//'data/pbp_locs',form='formatted',  &
               status='old',iostat=ierr)
          if (ierr.ne.0) then
            write(6,*) &
            ' pbp_locs file does not exist, no pbp data will be written'
            ireturn = 1 
            go to 99
          endif
          do
            read(2,*,iostat=ierr)lontem, lattem
            if (ierr.ne.0) exit
!-- find nearest atm grid point for each specified pbp lat,lon
!-- dist and xdist are dot products, greatest dot product is the point we want
            dist = -1.
            ztem = cos(lattem*pi/180.)
            xtem = cos(lontem*pi/180.) * ztem
            ytem = sin(lontem*pi/180.) * ztem
            ztem = sin(lattem*pi/180.)
            do i = 1,max_ig
              xdist = xtem*x_phys(i) + ytem*y_phys(i) + ztem*z_phys(i)
              if(xdist.gt.dist) then
                dist = xdist
                item = i
              endif
            enddo
            if (pbp_g(item).gt.0.5) then
              print*,'gridpoint nearest pbp lon lat ',lontem, lattem,  &
                     'is already selected'
            else
              pbp_g(item) = c1
              print*,' for pbp lon,lat ',lontem, lattem, &
              ' the nearest grid cell is at ' , lon_g(item), lat_g(item)
            endif
          enddo
          call scatter_to_grid_global(pbp_g(:),pbp_global(:,:))
 99     continue
          close(2)
        endif
#ifdef mpi
        call MPI_BCAST(           &
             ireturn,             &
             1,                   &
             mpi_int_type,        &
             0,                   &
             MPI_COMM_ATMOS,      &
             ierr)
#endif
        if(ireturn.eq.1) return
	call scatter_fine(pbp_global(:,:),pbp_loc(:,:,:))
#endif



! initialize
	pbp1_name(:) = " no name pbp1"
	pbp2_name(:) = " no name pbp2"
	pbp1_units(:) = "no units pbp1"
	pbp2_units(:) = "no units pbp2"
	pbp1_descr(:) = "no description pbp1"
	pbp2_descr(:) = "no description pbp2"
	pbp1_physics(:) = .false.
	pbp2_physics(:) = .false.
      
!  do assignments for variable name, units and description 
        pbp1_name (ipbp1_blent) = "blent"
        pbp1_units(ipbp1_blent) = "kg/m^2/s"
        pbp1_descr(ipbp1_blent) = "PBL top entrainment rate"
        pbp1_type (ipbp1_blent) = "pbl"

        pbp1_name (ipbp1_clblp) = "clblp"
        pbp1_units(ipbp1_clblp) = "Pa"
        pbp1_descr(ipbp1_clblp) = "Boundary layer cloud depth"
        pbp1_type (ipbp1_clblp) = "pbl"

        pbp1_name (ipbp1_blp) = "blp"
        pbp1_units(ipbp1_blp) = "Pa"
        pbp1_descr(ipbp1_blp) = "PBL depth"
        pbp1_type (ipbp1_blp) = "pbl"

        pbp1_name (ipbp1_rls) = "rls"
        pbp1_units(ipbp1_rls) = "W/m^2"
        pbp1_descr(ipbp1_rls) = "SFC all-sky net longwave radiation"
        pbp1_physics(ipbp1_rls) = .true.
        pbp1_type (ipbp1_rls) = "rad"

        pbp1_name (ipbp1_rss) = "rss"
        pbp1_units(ipbp1_rss) = "W/m^2"
        pbp1_descr(ipbp1_rss) = "SFC all-sky net shortwave radiation"
        pbp1_physics(ipbp1_rss) = .true.
        pbp1_type (ipbp1_rss) = "rad"

        pbp1_name (ipbp1_rst) = "rst"
        pbp1_units(ipbp1_rst) = "W/m^2"
        pbp1_descr(ipbp1_rst) = "TOA all-sky net shortwave radiation"
        pbp1_physics(ipbp1_rst) = .true.
        pbp1_type (ipbp1_rst) = "rad"

        pbp1_name (ipbp1_rlt) = "rlt"
        pbp1_units(ipbp1_rlt) = "W/m^2"
        pbp1_descr(ipbp1_rlt) = "TOA all-sky outgoing longwave radiation"
        pbp1_physics(ipbp1_rlt) = .true.
        pbp1_type (ipbp1_rlt) = "rad"

        pbp1_name (ipbp1_rsdt) = "rsdt"
        pbp1_units(ipbp1_rsdt) = "W/m^2"
        pbp1_descr(ipbp1_rsdt) = "TOA incident shortwave radiation"
        pbp1_physics(ipbp1_rsdt) = .true.
        pbp1_type (ipbp1_rsdt) = "rad"

        pbp1_name (ipbp1_spdm) = "spdm"
        pbp1_units(ipbp1_spdm) = "m/s"
        pbp1_descr(ipbp1_spdm) = "PBL wind speed"
        pbp1_type (ipbp1_spdm) = "pbl"

        pbp1_name (ipbp1_rlds) = "rlds"
        pbp1_units(ipbp1_rlds) = "W/m^2"
        pbp1_descr(ipbp1_rlds) = "SFC all-sky downward longwave radiation"
        pbp1_physics(ipbp1_rlds) = .true.
        pbp1_type (ipbp1_rlds) = "rad"

        pbp1_name (ipbp1_ps) = "ps"
        pbp1_units(ipbp1_ps) = "Pa"
        pbp1_descr(ipbp1_ps) = "Surface pressure"
        pbp1_type (ipbp1_ps) = "pbl"

        pbp1_name (ipbp1_huss) = "huss"
        pbp1_units(ipbp1_huss) = "kg/kg"
        pbp1_descr(ipbp1_huss) = "Surface (2m) specific humidity"
        pbp1_type (ipbp1_huss) = "pbl"

        pbp1_name (ipbp1_clbl) = "clbl"
        pbp1_units(ipbp1_clbl) = "-"
        pbp1_descr(ipbp1_clbl) = "Incidence of PBL clouds"
        pbp1_type (ipbp1_clbl) = "pbl"

        pbp1_name (ipbp1_rsds) = "rsds"
        pbp1_units(ipbp1_rsds) = "W/m^2"
        pbp1_descr(ipbp1_rsds) = "SFC all-sky downward shortwave radiation"
        pbp1_physics(ipbp1_rsds) = .true.
        pbp1_type (ipbp1_rsds) = "rad"

        pbp1_name (ipbp1_dryconvinc) = "dryconvinc"
        pbp1_units(ipbp1_dryconvinc) = "-"
        pbp1_descr(ipbp1_dryconvinc) = "Dry Convective Adjustment Incidence"
        pbp1_type (ipbp1_dryconvinc) = "dyn"

        pbp1_name (ipbp1_hfss) = "hfss"
        pbp1_units(ipbp1_hfss) = "W/m^2"
        pbp1_descr(ipbp1_hfss) = "Surface Sensible Heat Flux"
        pbp1_type (ipbp1_hfss) = "pbl"

        pbp1_name (ipbp1_tradgrdeff) = "tradgrdeff "
        pbp1_units(ipbp1_tradgrdeff) = "K"
        pbp1_descr(ipbp1_tradgrdeff) = "Effective radiative ground temperature"
        pbp1_type (ipbp1_tradgrdeff) = "pbl"

        pbp1_name (ipbp1_psl) = "psl"
        pbp1_units(ipbp1_psl) = "Pa"
        pbp1_descr(ipbp1_psl) = "sealevel pressure"
        pbp1_type (ipbp1_psl) = "pbl"

        pbp1_name (ipbp1_tas) = "tas"
        pbp1_units(ipbp1_tas) = "K"
        pbp1_descr(ipbp1_tas) = "sfc air temperature"
        pbp1_type (ipbp1_tas) = "pbl"

        pbp1_name (ipbp1_hfls) = "hfls"
        pbp1_units(ipbp1_hfls) = "W/m^2"
        pbp1_descr(ipbp1_hfls) = "Surface latent heat flux"
        pbp1_type (ipbp1_hfls) = "pbl"

        pbp1_name (ipbp1_rsus) = "rsus"
        pbp1_units(ipbp1_rsus) = "W/m^2"
        pbp1_descr(ipbp1_rsus) = "SFC reflected shortwave radiation"
        pbp1_physics(ipbp1_rsus) = .true.
        pbp1_type (ipbp1_rsus) = "rad"

        pbp1_name (ipbp1_rlus) = "rlus"
        pbp1_units(ipbp1_rlus) = "W/m^2"
        pbp1_descr(ipbp1_rlus) = "SFC upwelling longwave radiation"
        pbp1_physics(ipbp1_rlus) = .true.
        pbp1_type (ipbp1_rlus) = "rad"

        pbp1_name (ipbp1_rsut) = "rsut"
        pbp1_units(ipbp1_rsut) = "W/m^2"
        pbp1_descr(ipbp1_rsut) = "TOA reflected shortwave radiation"
        pbp1_physics(ipbp1_rsut) = .true.
        pbp1_type (ipbp1_rsut) = "rad"

        pbp1_name (ipbp1_rsdscs) = "rsdscs"
        pbp1_units(ipbp1_rsdscs) = "W/m^2"
        pbp1_descr(ipbp1_rsdscs) = "SFC clear-sky downward shortwave radiation"
        pbp1_physics(ipbp1_rsdscs) = .true.
        pbp1_type (ipbp1_rsdscs) = "rad"

        pbp1_name (ipbp1_rsuscs) = "rsuscs"
        pbp1_units(ipbp1_rsuscs) = "W/m^2"
        pbp1_descr(ipbp1_rsuscs) = "SFC clear-sky reflected shortwave radiation"
        pbp1_physics(ipbp1_rsuscs) = .true.
        pbp1_type (ipbp1_rsuscs) = "rad"

        pbp1_name (ipbp1_rldscs) = "rldscs"
        pbp1_units(ipbp1_rldscs) = "W/m^2"
        pbp1_descr(ipbp1_rldscs) = "SFC clear-sky downward longwave radiation"
        pbp1_physics(ipbp1_rldscs) = .true.
        pbp1_type (ipbp1_rldscs) = "rad"

        pbp1_name (ipbp1_rlutcs) = "rlutcs"
        pbp1_units(ipbp1_rlutcs) = "W/m^2"
        pbp1_descr(ipbp1_rlutcs) = "TOA clear-sky outgoing longwave radiation"
        pbp1_physics(ipbp1_rlutcs) = .true.
        pbp1_type (ipbp1_rlutcs) = "rad"

        pbp1_name (ipbp1_rsutcs) = "rsutcs"
        pbp1_units(ipbp1_rsutcs) = "W/m^2"
        pbp1_descr(ipbp1_rsutcs) = "TOA clear-sky reflected shortwave radiation"
        pbp1_physics(ipbp1_rsutcs) = .true.
        pbp1_type (ipbp1_rsutcs) = "rad"

        pbp1_name (ipbp1_clt) = "clt"
        pbp1_units(ipbp1_clt) = "%"
        pbp1_descr(ipbp1_clt) = "Total cloud amount"
        pbp1_physics(ipbp1_clt) = .true.
        pbp1_type (ipbp1_clt) = "rad"

        pbp1_name (ipbp1_leakp) = "leakp"
        pbp1_units(ipbp1_leakp) = "kg/m2/s"
        pbp1_descr(ipbp1_leakp) = "positive pleak"
        pbp1_type (ipbp1_leakp) = "pbl"

        pbp1_name (ipbp1_leakpinc) = "leakpinc"
        pbp1_units(ipbp1_leakpinc) = "-"
        pbp1_descr(ipbp1_leakpinc) = "positive pleak incidence"
        pbp1_type (ipbp1_leakpinc) = "pbl"

        pbp1_name (ipbp1_leakn) = "leakn"
        pbp1_units(ipbp1_leakn) = "kg/m2/s"
        pbp1_descr(ipbp1_leakn) = "negative pleak"
        pbp1_type (ipbp1_leakn) = "pbl"

        pbp1_name (ipbp1_leakninc) = "leakninc"
        pbp1_units(ipbp1_leakninc) = "-"
        pbp1_descr(ipbp1_leakninc) = "negative pleak incidence"
        pbp1_type (ipbp1_leakninc) = "pbl"

        pbp1_name (ipbp1_pbldmf) = "pbldmf"
        pbp1_units(ipbp1_pbldmf) = "kg/m2/s"
        pbp1_descr(ipbp1_pbldmf) = "pbl mass flux divergence"
        pbp1_type (ipbp1_pbldmf) = "pbl"

        pbp1_name (ipbp1_fillvq) = "fillvq"
        pbp1_units(ipbp1_fillvq) = "kg/m2/s"
        pbp1_descr(ipbp1_fillvq) = "vert. int. water vapor hole fill rate"
        pbp1_type (ipbp1_fillvq) = "dyn"

        pbp1_name (ipbp1_fillvcw) = "fillvcw"
        pbp1_units(ipbp1_fillvcw) = "kg/m2/s"
        pbp1_descr(ipbp1_fillvcw) = "vert. int. cloud water hole fill rate"
        pbp1_type (ipbp1_fillvcw) = "dyn"

        pbp1_name (ipbp1_fillvci) = "fillvci"
        pbp1_units(ipbp1_fillvci) = "kg/m2/s"
        pbp1_descr(ipbp1_fillvci) = "vert. int. cloud ice hole fill rate"
        pbp1_type (ipbp1_fillvci) = "dyn"

        pbp1_name (ipbp1_fillvrw) = "fillvrw"
        pbp1_units(ipbp1_fillvrw) = "kg/m2/s"
        pbp1_descr(ipbp1_fillvrw) = "vert. int. rain hole fill rate"
        pbp1_type (ipbp1_fillvrw) = "dyn"

        pbp1_name (ipbp1_fillvri) = "fillvri"
        pbp1_units(ipbp1_fillvri) = "kg/m2/s"
        pbp1_descr(ipbp1_fillvri) = "vert. int. snow hole fill rate"
        pbp1_type (ipbp1_fillvri) = "dyn"

        pbp1_name (ipbp1_filliq) = "filliq"
        pbp1_units(ipbp1_filliq) = "-"
        pbp1_descr(ipbp1_filliq) = "water vapor hole fill incidence"
        pbp1_type (ipbp1_filliq) = "dyn"

        pbp1_name (ipbp1_fillicw) = "fillicw"
        pbp1_units(ipbp1_fillicw) = "-"
        pbp1_descr(ipbp1_fillicw) = "cloud water hole fill incidence"
        pbp1_type (ipbp1_fillicw) = "dyn"

        pbp1_name (ipbp1_fillici) = "fillici"
        pbp1_units(ipbp1_fillici) = "-"
        pbp1_descr(ipbp1_fillici) = "cloud ice hole fill incidence"
        pbp1_type (ipbp1_fillici) = "dyn"

        pbp1_name (ipbp1_fillirw) = "fillirw"
        pbp1_units(ipbp1_fillirw) = "-"
        pbp1_descr(ipbp1_fillirw) = "rain hole fill incidence"
        pbp1_type (ipbp1_fillirw) = "dyn"

        pbp1_name (ipbp1_filliri) = "filliri"
        pbp1_units(ipbp1_filliri) = "-"
        pbp1_descr(ipbp1_filliri) = "snow hole fill incidence"
        pbp1_type (ipbp1_filliri) = "dyn"

        pbp1_name (ipbp1_filllq) = "filllq"
        pbp1_units(ipbp1_filllq) = "-"
        pbp1_descr(ipbp1_filllq) = "water vapor hole fill incidence*layers filled"
        pbp1_type (ipbp1_filllq) = "dyn"

        pbp1_name (ipbp1_filllcw) = "filllcw"
        pbp1_units(ipbp1_filllcw) = "-"
        pbp1_descr(ipbp1_filllcw) = "cloud water hole fill incidence*layers filled"
        pbp1_type (ipbp1_filllcw) = "dyn"

        pbp1_name (ipbp1_filllci) = "filllci"
        pbp1_units(ipbp1_filllci) = "-"
        pbp1_descr(ipbp1_filllci) = "cloud ice hole fill incidence*layers filled"
        pbp1_type (ipbp1_filllci) = "dyn"

        pbp1_name (ipbp1_filllrw) = "filllrw"
        pbp1_units(ipbp1_filllrw) = "-"
        pbp1_descr(ipbp1_filllrw) = "rain hole fill incidence*layers filled"
        pbp1_type (ipbp1_filllrw) = "dyn"

        pbp1_name (ipbp1_filllri) = "filllri"
        pbp1_units(ipbp1_filllri) = "-"
        pbp1_descr(ipbp1_filllri) = "snow hole fill incidence*layers filled"
        pbp1_type (ipbp1_filllri) = "dyn"

        pbp1_name (ipbp1_cuppr) = "cuppr"
        pbp1_units(ipbp1_cuppr) = "mm per s"
        pbp1_descr(ipbp1_cuppr) = "Convective precipitation rate"
        pbp1_type (ipbp1_cuppr) = "cup"
     
        pbp1_name (ipbp1_cupprpbl) = "cupprpbl"
        pbp1_units(ipbp1_cupprpbl) = "mm per s"
        pbp1_descr(ipbp1_cupprpbl) = "Convective precipitation rate for PBL cloud base"
        pbp1_type (ipbp1_cupprpbl) = "cup"

        pbp1_name (ipbp1_cape) = "cape"
        pbp1_units(ipbp1_cape) = "J/kg"
        pbp1_descr(ipbp1_cape) = "Conditional available potential energy"
        pbp1_type (ipbp1_cape) = "cup"

        pbp1_name (ipbp1_cupmb) = "cupmb"
        pbp1_units(ipbp1_cupmb) = "hPa per hour"
        pbp1_descr(ipbp1_cupmb) = "Convective mass flux at PBL top"
        pbp1_type (ipbp1_cupmb) = "cup"

        pbp1_name (ipbp1_elqwvt) = "elqwvt"
        pbp1_units(ipbp1_elqwvt) = "kg/m2"
        pbp1_descr(ipbp1_elqwvt) = "Vertically-integrated water vapor"
        pbp1_type (ipbp1_elqwvt) = "elq"

        pbp1_name (ipbp1_elqcwt) = "elqcwt"
        pbp1_units(ipbp1_elqcwt) = "kg/m2"
        pbp1_descr(ipbp1_elqcwt) = "Vertically-integrated cloud water"
        pbp1_type (ipbp1_elqcwt) = "elq"
     
        pbp1_name (ipbp1_elqrwt) = "elqrwt"
        pbp1_units(ipbp1_elqrwt) = "kg/m2"
        pbp1_descr(ipbp1_elqrwt) = "Vertically-integrated rain"
        pbp1_type (ipbp1_elqrwt) = "elq"

        pbp1_name (ipbp1_elqcit) = "elqcit"
        pbp1_units(ipbp1_elqcit) = "kg/m2"
        pbp1_descr(ipbp1_elqcit) = "Vertically-integrated cloud ice"
        pbp1_type (ipbp1_elqcit) = "elq"

        pbp1_name (ipbp1_elqrit) = "elqrit"
        pbp1_units(ipbp1_elqrit) = "kg/m2"
        pbp1_descr(ipbp1_elqrit) = "Vertically-integrated snow"
        pbp1_type (ipbp1_elqrit) = "elq"

        pbp1_name (ipbp1_elqprr) = "elqprr"
        pbp1_units(ipbp1_elqprr) = "kg/m2/s"
        pbp1_descr(ipbp1_elqprr) = "Stratiform precipitation rate of rain"
        pbp1_type (ipbp1_elqprr) = "elq"

        pbp1_name (ipbp1_elqprs) = "elqprs"
        pbp1_units(ipbp1_elqprs) = "kg/m2/s"
        pbp1_descr(ipbp1_elqprs) = "Stratiform precipitation rate of snow"
        pbp1_type (ipbp1_elqprs) = "elq"

        pbp1_name (ipbp1_elqpr) = "elqpr"
        pbp1_units(ipbp1_elqpr) = "kg/m2/s"
        pbp1_descr(ipbp1_elqpr) = "Stratiform precipitation rate"
        pbp1_type (ipbp1_elqpr) = "elq"
        
        pbp1_name (ipbp1_totpr) = "totpr"
        pbp1_units(ipbp1_totpr) = "kg/m2/s"
        pbp1_descr(ipbp1_totpr) = "Total precipitation rate"
        pbp1_type (ipbp1_totpr) = "elq"

        pbp1_name (ipbp1_sumelq) = "sumelq"
        pbp1_units(ipbp1_sumelq) = "kg/m2"
        pbp1_descr(ipbp1_sumelq) = "Vertically-integrated wv+cw+rain+ci+snow"
        pbp1_type (ipbp1_sumelq) = "elq"

        pbp1_name (ipbp1_qadvtint) = "qadvtint"
        pbp1_units(ipbp1_qadvtint) = "kg/m2"
        pbp1_descr(ipbp1_qadvtint) = "Vertically-integrated moisture advection"
        pbp1_type (ipbp1_qadvtint) = "dyn"
      
        pbp1_name (ipbp1_tskin) = "tskin"
        pbp1_units(ipbp1_tskin) = "K"
        pbp1_descr(ipbp1_tskin) = "Computed skin temperature"
        pbp1_type (ipbp1_tskin) = "pbl"
  
        pbp1_name (ipbp1_th1) = "th1"
        pbp1_units(ipbp1_th1) = "K"
        pbp1_descr(ipbp1_th1) = "Potential temperature of layer 1"
        pbp1_type (ipbp1_th1) = "dyn"
  
        pbp1_name (ipbp1_cupinc1) = "cupinc1"
        pbp1_units(ipbp1_cupinc1) = "-"
        pbp1_descr(ipbp1_cupinc1) = "Cloud test - convectively unstable PBL cloud base"
        pbp1_type (ipbp1_cupinc1) = "cup"

        pbp1_name (ipbp1_cupinc2) = "cupinc2"
        pbp1_units(ipbp1_cupinc2) = "-"
        pbp1_descr(ipbp1_cupinc2) = "Cloud test - convective unstable cloud bases"
        pbp1_type (ipbp1_cupinc2) = "cup"
  
        pbp1_name (ipbp1_cupinc3) = "cupinc3"
        pbp1_units(ipbp1_cupinc3) = "-"
        pbp1_descr(ipbp1_cupinc3) = "Cloud test - after entrainment rate (PBL)"
        pbp1_type (ipbp1_cupinc3) = "cup"
  
        pbp1_name (ipbp1_cupinc4) = "cupinc4"
        pbp1_units(ipbp1_cupinc4) = "-"
        pbp1_descr(ipbp1_cupinc4) = "Cloud test - after entrainment rate (ALL)"
        pbp1_type (ipbp1_cupinc4) = "cup"
  
        pbp1_name (ipbp1_cupinc5) = "cupinc5"
        pbp1_units(ipbp1_cupinc5) = "-"
        pbp1_descr(ipbp1_cupinc5) = "Cloud test - after cloud tests (PBL)"
        pbp1_type (ipbp1_cupinc5) = "cup"
  
        pbp1_name (ipbp1_cupinc6) = "cupinc6"
        pbp1_units(ipbp1_cupinc6)  = "-"
        pbp1_descr(ipbp1_cupinc6) = "Cloud test - after cloud tests (ALL)"
        pbp1_type (ipbp1_cupinc6) = "cup"
  
        pbp1_name (ipbp1_cupinc7) = "cupinc7"
        pbp1_units(ipbp1_cupinc7) = "-"
        pbp1_descr(ipbp1_cupinc7) = "Cloud test - after moist static energy (PBL)"
        pbp1_type (ipbp1_cupinc7) = "cup"
  
        pbp1_name (ipbp1_cupinc8) = "cupinc8"
        pbp1_units(ipbp1_cupinc8) = "-"
        pbp1_descr(ipbp1_cupinc8) = "Cloud test - after moist static energy (ALL)"
        pbp1_type (ipbp1_cupinc8) = "cup"
  
        pbp1_name (ipbp1_cupinc9) = "cupinc9"
        pbp1_units(ipbp1_cupinc9) = "-"
        pbp1_descr(ipbp1_cupinc9) = "Cloud test - after water budget (PBL)"
        pbp1_type (ipbp1_cupinc9) = "cup"
  
        pbp1_name (ipbp1_cupinc10) = "cupinc10"
        pbp1_units(ipbp1_cupinc10) = "-"
        pbp1_descr(ipbp1_cupinc10) = "Cloud test - after water budget (ALL)"
        pbp1_type (ipbp1_cupinc10) = "cup"
  
        pbp1_name (ipbp1_cupinc11) = "cupinc11"
        pbp1_units(ipbp1_cupinc11) = "-"
        pbp1_descr(ipbp1_cupinc11) = "Cloud test - after normalized mass flux (PBL)"
        pbp1_type (ipbp1_cupinc11) = "cup"
  
        pbp1_name (ipbp1_cupinc12) = "cupinc12"
        pbp1_units(ipbp1_cupinc12) = "-"
        pbp1_descr(ipbp1_cupinc12) = "Cloud test - after normalized mass flux (ALL)"
        pbp1_type (ipbp1_cupinc12) = "cup"
  
        pbp1_name (ipbp1_cupinc13) = "cupinc13"
        pbp1_units(ipbp1_cupinc13) = "-"
        pbp1_descr(ipbp1_cupinc13) = "Cloud test - after cloud work function (PBL)"
        pbp1_type (ipbp1_cupinc13) = "cup"
  
        pbp1_name (ipbp1_cupinc14) = "cupinc14"
        pbp1_units(ipbp1_cupinc14) = "-"
        pbp1_descr(ipbp1_cupinc14) = "Cloud test - after cloud work function (ALL)"
        pbp1_type (ipbp1_cupinc14) = "cup"
  
        pbp1_name (ipbp1_cupinc15) = "cupinc15"
        pbp1_units(ipbp1_cupinc15) = "-"
        pbp1_descr(ipbp1_cupinc15) = "Cloud test - after eau cupcld (PBL)"
        pbp1_type (ipbp1_cupinc15) = "cup"
  
        pbp1_name (ipbp1_cupinc16) = "cupinc16"
        pbp1_units(ipbp1_cupinc16) = "-"
        pbp1_descr(ipbp1_cupinc16) = "Cloud test - after eau cupcld (ALL)"
        pbp1_type (ipbp1_cupinc16) = "cup"
  
!--------------------------------------------------------------------------
! start pbp2s
!--------------------------------------------------------------------------
        pbp2_name (ipbp2_pl) = "pl"
        pbp2_units(ipbp2_pl) = "hPa"
        pbp2_descr(ipbp2_pl) = "Mid-layer Pressure"
        pbp2_type (ipbp2_pl) = "dyn"

        pbp2_name (ipbp2_pl2) = "pl2"
        pbp2_units(ipbp2_pl2) = "hPa"
        pbp2_descr(ipbp2_pl2) = "Lower Layer Edge Pressure"
        pbp2_type (ipbp2_pl2) = "dyn"

        pbp2_name (ipbp2_zg) = "zg"
        pbp2_units(ipbp2_zg) = "m"
        pbp2_descr(ipbp2_zg) = "Geopotential height"
        pbp2_type (ipbp2_zg) = "dyn"

        pbp2_name (ipbp2_ta) = "ta"
        pbp2_units(ipbp2_ta) = "K"
        pbp2_descr(ipbp2_ta) = "Air Temperature"
        pbp2_type (ipbp2_ta) = "dyn"

        pbp2_name (ipbp2_vorticity) = "vorticity"
        pbp2_units(ipbp2_vorticity) = "s^-1"
        pbp2_descr(ipbp2_vorticity) = "Relative vorticity"
        pbp2_type (ipbp2_vorticity) = "dyn"

        pbp2_name (ipbp2_divergence) = "divergence"
        pbp2_units(ipbp2_divergence) = "s^-1"
        pbp2_descr(ipbp2_divergence) = "Divergence"
        pbp2_type (ipbp2_divergence) = "dyn"

        pbp2_name (ipbp2_tntsw) = "tntsw"
        pbp2_units(ipbp2_tntsw) = "K/s "
        pbp2_physics(ipbp2_tntsw) = .true.
        pbp2_descr(ipbp2_tntsw) = "Atmospheric all-sky SW heating rate"
        pbp2_type (ipbp2_tntsw) = "rad"

        pbp2_name (ipbp2_tntlw) = "tntlw"
        pbp2_units(ipbp2_tntlw) = "K/s"
        pbp2_descr(ipbp2_tntlw) = "Atmospheric all-sky LW heating rate"
        pbp2_physics(ipbp2_tntlw) = .true.
        pbp2_type (ipbp2_tntlw) = "rad"

        pbp2_name (ipbp2_hur) = "hur"
        pbp2_units(ipbp2_hur) = "-"
        pbp2_descr(ipbp2_hur) = "Relative Humidity"
        pbp2_type (ipbp2_hur) = "dyn"

        pbp2_name (ipbp2_tntdc) = "tntdc"
        pbp2_units(ipbp2_tntdc) = "K/s"
        pbp2_descr(ipbp2_tntdc) = "Temperature tendency, dry convective adjustment                "
        pbp2_type (ipbp2_tntdc) = "dyn"

        pbp2_name (ipbp2_tnqdc) = "tnqdc"
        pbp2_units(ipbp2_tnqdc) = "kg/kg/s"
        pbp2_descr(ipbp2_tnqdc) = "Water vapor tendency, dry convective adjustment                "
        pbp2_type (ipbp2_tnqdc) = "dyn"

        pbp2_name (ipbp2_qsat) = "qsat"
        pbp2_units(ipbp2_qsat) = "kg/kg"
        pbp2_descr(ipbp2_qsat) = "Saturation mixing ratio"
        pbp2_type (ipbp2_qsat) = "dyn"

        pbp2_name (ipbp2_cl) = "cl"
        pbp2_units(ipbp2_cl) = "%"
        pbp2_descr(ipbp2_cl) = "Cloud amount"
        pbp2_physics(ipbp2_cl) = .true.
        pbp2_type (ipbp2_cl) = "dyn"

        pbp2_name (ipbp2_o3) = "o3"
        pbp2_units(ipbp2_o3) = "kg/kg"
        pbp2_descr(ipbp2_o3) = "Ozone mixing ratio"
        pbp2_physics(ipbp2_o3) = .true.
        pbp2_type (ipbp2_o3) = "dyn"

        pbp2_name (ipbp2_ua) = "ua"
        pbp2_units(ipbp2_ua) = "m/s"
        pbp2_descr(ipbp2_ua) = "Zonal wind"
        pbp2_type (ipbp2_ua) = "dyn"

        pbp2_name (ipbp2_va) = "va"
        pbp2_units(ipbp2_va) = "m/s"
        pbp2_descr(ipbp2_va) = "Meridional wind"
        pbp2_type (ipbp2_va) = "dyn"

        pbp2_name (ipbp2_wap) = "wap"
        pbp2_units(ipbp2_wap) = "Pa/s"
        pbp2_descr(ipbp2_wap) = "Vertical motion"
        pbp2_type (ipbp2_wap) = "dyn"

        pbp2_name (ipbp2_tntswcs) = "tntswcs"
        pbp2_units(ipbp2_tntswcs) = "K/s"
        pbp2_descr(ipbp2_tntswcs) = "Atmospheric clear-sky SW heating rate"
        pbp2_physics(ipbp2_tntswcs) = .true.
        pbp2_type (ipbp2_tntswcs) = "rad"

        pbp2_name (ipbp2_tntlwcs) = "tntlwcs"
        pbp2_units(ipbp2_tntlwcs) = "K/s"
        pbp2_descr(ipbp2_tntlwcs) = "Atmospheric clear-sky LW heating rate"
        pbp2_physics(ipbp2_tntlwcs) = .true.
        pbp2_type (ipbp2_tntlwcs) = "rad"

        pbp2_name (ipbp2_fill3q) = "fill3q"
        pbp2_units(ipbp2_fill3q) = "kg/kg/s"
        pbp2_descr(ipbp2_fill3q) = "Water vapor hole fill rate"
        pbp2_type (ipbp2_fill3q) = "dyn"

        pbp2_name (ipbp2_fill3cw) = "fill3cw"
        pbp2_units(ipbp2_fill3cw) = "kg/kg/s"
        pbp2_descr(ipbp2_fill3cw) = "Cloud water hole fill rate"
        pbp2_type (ipbp2_fill3cw) = "dyn"

        pbp2_name (ipbp2_fill3ci) = "fill3ci"
        pbp2_units(ipbp2_fill3ci) = "kg/kg/s"
        pbp2_descr(ipbp2_fill3ci) = "Cloud ice hole fill rate"
        pbp2_type (ipbp2_fill3ci) = "dyn"

        pbp2_name (ipbp2_fill3rw) = "fill3rw"
        pbp2_units(ipbp2_fill3rw) = "kg/kg/s"
        pbp2_descr(ipbp2_fill3rw) = "Rain hole fill rate"
        pbp2_type (ipbp2_fill3rw) = "dyn"

        pbp2_name (ipbp2_fill3ri) = "fill3ri"
        pbp2_units(ipbp2_fill3ri) = "kg/kg/s"
        pbp2_descr(ipbp2_fill3ri) = "Snow hole fill rate"
        pbp2_type (ipbp2_fill3ri) = "dyn"

        pbp2_name (ipbp2_hhl) = "hhl"
        pbp2_units(ipbp2_hhl) = "J/kg"
        pbp2_descr(ipbp2_hhl) = "Moist static energy"
        pbp2_physics(ipbp2_hhl) = .true.
        pbp2_type (ipbp2_hhl) = "cup"
        
        pbp2_name (ipbp2_hhls) = "hhls"
        pbp2_units(ipbp2_hhls) = "J/kg"
        pbp2_descr(ipbp2_hhls) = "Saturation moist static energy"
        pbp2_physics(ipbp2_hhls) = .true.
        pbp2_type (ipbp2_hhls) = "cup"

        pbp2_name (ipbp2_ssl) = "ssl"
        pbp2_units(ipbp2_ssl) = "J/kg"
        pbp2_descr(ipbp2_ssl) = "Dry static energy"
        pbp2_physics(ipbp2_ssl) = .true.
        pbp2_type (ipbp2_ssl) = "cup"
        
        pbp2_name (ipbp2_ssvl) = "ssvl"
        pbp2_units(ipbp2_ssvl) = "J/kg"
        pbp2_descr(ipbp2_ssvl) = "Virtual dry static energy"
        pbp2_physics(ipbp2_ssvl) = .true.
        pbp2_type (ipbp2_ssvl) = "cup"

        pbp2_name (ipbp2_cke) = "cke"
        pbp2_units(ipbp2_cke) = "J/m^2"
        pbp2_descr(ipbp2_cke) = "Convective kinetic energy"
        pbp2_physics(ipbp2_cke) = .true.
        pbp2_type (ipbp2_cke) = "cup"
        
        pbp2_name (ipbp2_cdwork) = "cdwork"
        pbp2_units(ipbp2_cdwork) = "J/kg"
        pbp2_descr(ipbp2_cdwork) = "Convective cloud work function"
        pbp2_physics(ipbp2_cdwork) = .true.
        pbp2_type (ipbp2_cdwork) = "cup"

        pbp2_name (ipbp2_masflx) = "masflx"
        pbp2_units(ipbp2_masflx) = "kg/kg/s"
        pbp2_descr(ipbp2_masflx) = "Convective mass flux"
        pbp2_physics(ipbp2_masflx) = .true.
        pbp2_type (ipbp2_masflx) = "cup"

        pbp2_name (ipbp2_dtcup) = "dtcup"
        pbp2_units(ipbp2_dtcup) = "K/s"
        pbp2_descr(ipbp2_dtcup) = "Convective heating rate"
        pbp2_physics(ipbp2_dtcup) = .true.
        pbp2_type (ipbp2_dtcup) = "cup"
     
        pbp2_name (ipbp2_dqcup) = "dqcup"
        pbp2_units(ipbp2_dqcup) = "kg/kg/s"
        pbp2_descr(ipbp2_dqcup) = "Convective tendency of total water"
        pbp2_physics(ipbp2_dqcup) = .true.
        pbp2_type (ipbp2_dqcup) = "cup"

        pbp2_name (ipbp2_dhcup) = "dhcup"
        pbp2_units(ipbp2_dhcup) = "J/kg/s"
        pbp2_descr(ipbp2_dhcup) = "Tendency of moist static energy due to cup microp"
        pbp2_physics(ipbp2_dhcup) = .true.
        pbp2_type (ipbp2_dhcup) = "cup"

        pbp2_name (ipbp2_dvcup) = "dvcup"
        pbp2_units(ipbp2_dvcup) = "kg/kg/s"
        pbp2_descr(ipbp2_dvcup) = "Tendency of water vapor due to cup microp"
        pbp2_physics(ipbp2_dvcup) = .true.
        pbp2_type (ipbp2_dvcup) = "cup"

        pbp2_name (ipbp2_dlcup) = "dlcup"
        pbp2_units(ipbp2_dlcup) = "kg/kg/s"
        pbp2_descr(ipbp2_dlcup) = "Tendency of cloud water due to cup microp"
        pbp2_physics(ipbp2_dlcup) = .true.
        pbp2_type (ipbp2_dlcup) = "cup"

        pbp2_name (ipbp2_drcup) = "drcup"
        pbp2_units(ipbp2_drcup) = "kg/kg/s"
        pbp2_descr(ipbp2_drcup) = "Tendency of rain due to cup microp"
        pbp2_physics(ipbp2_drcup) = .true.
        pbp2_type (ipbp2_drcup) = "cup"

        pbp2_name (ipbp2_dicup) = "dicup"
        pbp2_units(ipbp2_dicup) = "kg/kg/s"
        pbp2_descr(ipbp2_dicup) = "Tendency of cloud ice due to cup microp"
        pbp2_physics(ipbp2_dicup) = .true.
        pbp2_type (ipbp2_dicup) = "cup"

        pbp2_name (ipbp2_dscup) = "dscup"
        pbp2_units(ipbp2_dscup) = "kg/kg/s"
        pbp2_descr(ipbp2_dscup) = "Tendency of snow due to cup microp"
        pbp2_physics(ipbp2_dscup) = .true.
        pbp2_type (ipbp2_dscup) = "cup"

        pbp2_name (ipbp2_tcuph) = "tcuph"
        pbp2_units(ipbp2_tcuph) = "J/kg/s"
        pbp2_descr(ipbp2_tcuph) = "Tendency of moist static energy due to cup transport"
        pbp2_physics(ipbp2_tcuph) = .true.
        pbp2_type (ipbp2_tcuph) = "cup"

        pbp2_name (ipbp2_tcupv) = "tcupv"
        pbp2_units(ipbp2_tcupv) = "kg/kg/s"
        pbp2_descr(ipbp2_tcupv) = "Tendency of water vapor due to cup transport"
        pbp2_physics(ipbp2_tcupv) = .true.
        pbp2_type (ipbp2_tcupv) = "cup"

        pbp2_name (ipbp2_tcupl) = "tcupl"
        pbp2_units(ipbp2_tcupl) = "kg/kg/s"
        pbp2_descr(ipbp2_tcupl) = "Tendency of cloud water due to cup transport"
        pbp2_physics(ipbp2_tcupl) = .true.
        pbp2_type (ipbp2_tcupl) = "cup"

        pbp2_name (ipbp2_tcupr) = "tcupr"
        pbp2_units(ipbp2_tcupr) = "kg/kg/s"
        pbp2_descr(ipbp2_tcupr) = "Tendency of rain due to cup transport"
        pbp2_physics(ipbp2_tcupr) = .true.
        pbp2_type (ipbp2_tcupr) = "cup"

        pbp2_name (ipbp2_tcupi) = "tcupi"
        pbp2_units(ipbp2_tcupi) = "kg/kg/s"
        pbp2_descr(ipbp2_tcupi) = "Tendency of cloud ice due to cup transport"
        pbp2_physics(ipbp2_tcupi) = .true.
        pbp2_type (ipbp2_tcupi) = "cup"

        pbp2_name (ipbp2_tcups) = "tcups"
        pbp2_units(ipbp2_tcups) = "kg/kg/s"
        pbp2_descr(ipbp2_tcups) = "Tendency of snow due to cup transport"
        pbp2_physics(ipbp2_tcups) = .true.
        pbp2_type (ipbp2_tcups) = "cup"

        pbp2_name (ipbp2_cwelq) = "cwelq"
        pbp2_units(ipbp2_cwelq) = "kg/kg"
        pbp2_descr(ipbp2_cwelq) = "Cloud water mixing ratio"
        pbp2_type (ipbp2_cwelq) = "elq"

        pbp2_name (ipbp2_rwelq) = "rwelq"
        pbp2_units(ipbp2_rwelq) = "kg/kg"
        pbp2_descr(ipbp2_rwelq) = "Rain mixing ratio"
        pbp2_type (ipbp2_rwelq) = "elq"

        pbp2_name (ipbp2_cielq) = "cielq"
        pbp2_units(ipbp2_cielq) = "kg/kg"
        pbp2_descr(ipbp2_cielq) = "Cloud ice mixing ratio"
        pbp2_type (ipbp2_cielq) = "elq"

        pbp2_name (ipbp2_rielq) = "rielq"
        pbp2_units(ipbp2_rielq) = "kg/kg"
        pbp2_descr(ipbp2_rielq) = "Snow mixing ratio"
        pbp2_type (ipbp2_rielq) = "elq"

        pbp2_name (ipbp2_wvelq) = "wvelq"
        pbp2_units(ipbp2_wvelq) = "kg/kg"
        pbp2_descr(ipbp2_wvelq) = "Water vapor mixing ratio"
        pbp2_type (ipbp2_wvelq) = "elq"

        pbp2_name (ipbp2_dtelq) = "dtelq"
        pbp2_units(ipbp2_dtelq) = "K/s"
        pbp2_descr(ipbp2_dtelq) = "Large-scale heating rate"
        pbp2_type (ipbp2_dtelq) = "elq"

        pbp2_name (ipbp2_dqelq) = "dqelq"
        pbp2_units(ipbp2_dqelq) = "kg/kg/s"
        pbp2_descr(ipbp2_dqelq) = "Large-scale tendency of total water"
        pbp2_type (ipbp2_dqelq) = "elq"

        pbp2_name (ipbp2_dvelq) = "dvelq"
        pbp2_units(ipbp2_dvelq) = "kg/kg/s"
        pbp2_descr(ipbp2_dvelq) = "Large-scale tendency of water vapor"
        pbp2_type (ipbp2_dvelq) = "elq"

        pbp2_name (ipbp2_dlelq) = "dlelq"
        pbp2_units(ipbp2_dlelq) = "kg/kg/s"
        pbp2_descr(ipbp2_dlelq) = "Large-scale tendency of cloud water"
        pbp2_type (ipbp2_dlelq) = "elq"

        pbp2_name (ipbp2_drelq) = "drelq"
        pbp2_units(ipbp2_drelq) = "kg/kg/s"
        pbp2_descr(ipbp2_drelq) = "Large-scale tendency of rain"
        pbp2_type (ipbp2_drelq) = "elq"

        pbp2_name (ipbp2_dielq) = "dielq"
        pbp2_units(ipbp2_dielq) = "kg/kg/s"
        pbp2_descr(ipbp2_dielq) ="Large-scale tendency of cloud ice"
        pbp2_type (ipbp2_dielq) = "elq"

        pbp2_name (ipbp2_dselq) = "dselq"
        pbp2_units(ipbp2_dselq) = "kg/kg/s"
        pbp2_descr(ipbp2_dselq) = "Large-scale tendency of snow"
        pbp2_type (ipbp2_dselq) = "elq"
     
        pbp2_name (ipbp2_drwfelq) = "drwfelq"
        pbp2_units(ipbp2_drwfelq) = "kg/kg/s"
        pbp2_descr(ipbp2_drwfelq) = "Tendency of rain due to precipitation"
        pbp2_type (ipbp2_drwfelq) = "elq"

        pbp2_name (ipbp2_drifelq) = "drifelq"
        pbp2_units(ipbp2_drifelq) = "kg/kg/s"
        pbp2_descr(ipbp2_drifelq) = "Tendency of snow due to precipitation"
        pbp2_type (ipbp2_drifelq) = "elq"

        pbp2_name (ipbp2_scmvdelt) = "scmvdelt"
        pbp2_units(ipbp2_scmvdelt) = "K/day"
        pbp2_descr(ipbp2_scmvdelt) = "SCM total temperature advection"
        pbp2_type (ipbp2_scmvdelt) = "dyn"

        pbp2_name (ipbp2_scmvdelq) = "scmvdelq"
        pbp2_units(ipbp2_scmvdelq) = "g/kg/day"
        pbp2_descr(ipbp2_scmvdelq) = "SCM total moisture advection"
        pbp2_type (ipbp2_scmvdelq) = "dyn"

        pbp2_name (ipbp2_q1) = "q1"
        pbp2_units(ipbp2_q1) = "K/day"
        pbp2_descr(ipbp2_q1) = "Apparent heat source"
        pbp2_type (ipbp2_q1) = "dyn"

        pbp2_name (ipbp2_q2) = "q2"
        pbp2_units(ipbp2_q2) = "K/day"
        pbp2_descr(ipbp2_q2) = "Apparent moisture sink"
        pbp2_type (ipbp2_q2) = "dyn"

        pbp2_name (ipbp2_tadiff) = "tadiff"
        pbp2_units(ipbp2_tadiff) = "K"
        pbp2_descr(ipbp2_tadiff) = "Model minus observed temperature"
        pbp2_type (ipbp2_tadiff) = "dyn"

        pbp2_name (ipbp2_qdiff) = "qdiff"
        pbp2_units(ipbp2_qdiff) = "kg/kg"
        pbp2_descr(ipbp2_qdiff) = "Model minus observed water vapor mixing ratio"
        pbp2_type (ipbp2_qdiff) = "dyn"

        pbp2_name (ipbp2_trlxterm) = "trlxterm"
        pbp2_units(ipbp2_trlxterm) = "K/day"
        pbp2_descr(ipbp2_trlxterm) = "SCM T relaxation term"
        pbp2_type (ipbp2_trlxterm) = "dyn"

        pbp2_name (ipbp2_qrlxterm) = "qrlxterm"
        pbp2_units(ipbp2_qrlxterm) = "g/kg/day"
        pbp2_descr(ipbp2_qrlxterm) = "SCM Q relaxation term"
        pbp2_type (ipbp2_qrlxterm) = "dyn"

        pbp2_name (ipbp2_trevterm) = "trevterm"
        pbp2_units(ipbp2_trevterm) = "K/day"
        pbp2_descr(ipbp2_trevterm) = "SCM T revealed term"
        pbp2_type (ipbp2_trevterm) = "dyn"

        pbp2_name (ipbp2_qrevterm) = "qrevterm"
        pbp2_units(ipbp2_qrevterm) = "g/kg/day"
        pbp2_descr(ipbp2_qrevterm) = "SCM Q revealed term"
        pbp2_type (ipbp2_qrevterm) = "dyn"

 	do n=1,npbp1
	  pbp1_name(n) = trim(pbp1_name(n))
	  pbp1_units(n) = trim(pbp1_units(n))
	  pbp1_descr(n) = trim(pbp1_descr(n))
	enddo

	do n=1,npbp2
      pbp2_name(n) = trim(pbp2_name(n))
      pbp2_units(n) = trim(pbp2_units(n))
      pbp2_descr(n) = trim(pbp2_descr(n))
	enddo

!  initialize some pbp tables
      pbp1_indx = 0
      pbp2_indx = 0
      pbp1_log = .false.
      pbp2_log = .false.

!  always save 3d pressures and sfc pressure
      pbp2_log(ipbp2_pl) = .true.
      pbp2_indx(ipbp2_pl) = 1
      pbp2_log(ipbp2_pl2) = .true.
      pbp2_indx(ipbp2_pl2) = 2
      ipbp2 = 2
      pbp1_log(ipbp1_ps) = .true.
      pbp1_indx(ipbp1_ps) = 1
      ipbp1 = 1

!  read input data
      if(my_task.eq.0) then
         open(unit=2,file=trim(datapath)//'data/atm_pbpopts', form='formatted')

         do
           read(2,*, IOSTAT=ierr)dopbptem,ldummy,nametem
	       nametem = trim(nametem)
           if (ierr .ne. 0) exit
           if (dopbptem) then
             if (ldummy == 1) then
               do n = 1,npbp2
                 if (nametem.eq.pbp2_name(n)) then

!   test if this is an automatically saved field
                   if (nametem.eq.pbp2_name(ipbp2_pl) .or.  &
                       nametem.eq.pbp2_name(ipbp2_pl2)) exit

                   ipbp2 = ipbp2 + 1
                   pbp2_log(n) = .true.
                   pbp2_indx(n) = ipbp2
                   exit
                 endif
                 if (n.eq.npbp2)  &
                   print *,nametem,' is not an available pbp2 diagnostic'
               enddo
             else if (ldummy.eq.0) then
               do n = 1,npbp1
                 if (nametem.eq.pbp1_name(n)) then

!   test if this is an automatically saved field
                   if (nametem.eq.pbp1_name(ipbp1_ps)) exit

                   ipbp1 = ipbp1 + 1
                   pbp1_log(n) = .true.
                   pbp1_indx(n) = ipbp1
                   exit
                 endif
                 if (n.eq.npbp1)  &
                   print *,nametem,' is not an available pbp1 diagnostic'
               enddo
             endif
           endif
         enddo

         close(2)

	 write(6,*)
         do n = 1,npbp2
           if(pbp2_log(n)) then
	     write(6,10) pbp2_name(n), pbp2_descr(n),pbp2_indx(n)
 10          format('Will save pbp2: ',a12,a68, '  in pbp2 slot  ', i4)
	   endif
         enddo
         write(6,*)
         do n = 1,npbp1
           if(pbp1_log(n)) then
             write(6,11) pbp1_name(n), pbp1_descr(n),pbp1_indx(n)
 11          format('Will save pbp1: ',a12,a68, '  in pbp1 slot  ', i4)
           endif
         enddo
	 write(6,*)
      endif  ! if my_task.eq.0

!  broadcast information

#ifdef mpi
        call MPI_BCAST(pbp1_indx,npbp1,mpi_int_type,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp2_indx,npbp2,mpi_int_type,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp1_log,npbp1,MPI_LOGICAL,0,MPI_COMM_ATMOS,ierr)
        
        call MPI_BCAST(pbp2_log,npbp2,MPI_LOGICAL,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp1_name,npbp1*16,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp2_name,npbp2*16,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp1_units,npbp1*16,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp2_units,npbp2*16,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp1_descr,npbp1*80,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp2_descr,npbp2*80,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(ipbp1,1,mpi_int_type,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(ipbp2,1,mpi_int_type,0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(pbp1_type,npbp1*3,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)
        
        call MPI_BCAST(pbp2_type,npbp2*3,MPI_CHARACTER,0,MPI_COMM_ATMOS,ierr)
#endif


! no pbp saved at any grid point on this processor
#ifdef scm
      ijlen = 1
      kijlen = 1
#else
      do ksdm = 1,nsdm
        do j = 1,jjm
        do i = 1,iim
           if (pbp_loc(i,j,ksdm)*grid_center_mask(i,j,ksdm).gt.0.5) then
              ijlen = ijlen + 1
           endif
        enddo
        enddo
      enddo
#endif
        if (ijlen == 0) return 

!  allocate the pbp arrays

      allocate(pbp1(0:ipbp1, ijlen))
      allocate(pbp2(nlm, 0:ipbp2, ijlen))
      allocate(lonpbp(ijlen))
      allocate(latpbp(ijlen))
      allocate(ipbploc(ijlen))
      allocate(jpbploc(ijlen))
      allocate(ksdmpbploc(ijlen))

#ifdef scm
      lonpbp(1) = lon_scm * 180. / pi
      latpbp(1) = lat_scm * 180. / pi
      ipbploc(1) = 1
      jpbploc(1) = 1
      ksdmpbploc(1) = 1
#else
      ijlen = 0
      do ksdm = 1,nsdm
        koffsetpbp(ksdm) = ijlen
        do j = 1,jjm
        do i = 1,iim
           if (pbp_loc(i,j,ksdm)*grid_center_mask(i,j,ksdm).gt.0.5) then
              kijlen(ksdm) = kijlen(ksdm) + 1
              ijlen = ijlen + 1
              lonpbp(ijlen) = grid_center_lon(i,j,ksdm) * 180. / pi
              latpbp(ijlen) = grid_center_lat(i,j,ksdm) * 180. / pi
              ipbploc(ijlen) = i
              jpbploc(ijlen) = j
              ksdmpbploc(ijlen) = ksdm
           endif
        enddo
        enddo
      enddo
#endif

      pbp1 = 0.0
      pbp2 = 0.0

	end subroutine initialize_pbpinfo


        subroutine initialize_pbpfiles( simulation_tag )

!  this subroutine assigns initializes the pbp output files

#ifdef scm
        use scm_module
#endif

        implicit none

        character*40, intent(in) :: simulation_tag

        character*8, time_label
        character*4, proc_label

        integer (kind=int_kind) :: monthofyear

        if (ijlen == 0) return

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if (n_restart > 0) then
          write(unit=time_label,fmt='(i7)') 1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)') 1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

        write(unit=proc_label,fmt='(i4)') 1000+my_task
        proc_label(1:1) = 'p'

        ipbp_sample = 0
        pbp1 = 0.0
        pbp2 = 0.0

#ifdef scm
        call pinit_atm_nc(simulation_tag, &
                          'pbp_output/patm_'//trim(time_label) &
                          //trim(proc_label)//'.pbp1.nc', &
                          pbp1_descr, pbp1_name, pbp1_units, pbp1_type, pbp1_log, &
                          nlm, p_top, max_ig, &
                          lonpbp, latpbp, &
                          'pbp_output/patm_'//trim(time_label) &
                          //trim(proc_label)//'.pbp2.nc', &
                          pbp2_descr, pbp2_name, pbp2_units, pbp2_type, pbp2_log, &
                          nlev_scm, plev_scminit)
#else
        call pinit_atm_nc(simulation_tag, &
                          'pbp_output/patm_'//trim(time_label) &
                          //trim(proc_label)//'.pbp1.nc', &
                          pbp1_descr, pbp1_name, pbp1_units, pbp1_type, pbp1_log, &
                          nlm, p_top, max_ig, &
                          lonpbp, latpbp, &
                          'pbp_output/patm_'//trim(time_label) &
                          //trim(proc_label)//'.pbp2.nc', &
                          pbp2_descr, pbp2_name, pbp2_units, pbp2_type, pbp2_log)
#endif


	end subroutine initialize_pbpfiles


        subroutine write_pbp_diagnostics

        implicit none

        real (kind=dbl_kind) :: pbp1_sample_inv
        integer (kind=int_kind) :: n

        if(ijlen.eq.0) return

        pbp1_sample_inv = 1. / ipbp_sample

        pbp1(1:ipbp1,:) = pbp1(1:ipbp1,:) * pbp1_sample_inv
        pbp2(:,1:ipbp2,:) = pbp2(:,1:ipbp2,:) * pbp1_sample_inv

!        write(*,*) 'write_pbp_diags:  nsecond_dyn = ',nsecond_dyn
!        write(*,*) 'write_pbp_diags:  nyear0 = ',nyear0
!        read(*,*)
        call pwrt_atm_nc (nsecond_dyn, nyear0,  &
                          pbp1(1:ipbp1,:), pbp1_name, pbp1_log, pbp1_indx,  &
                          pbp2(:,1:ipbp2,:), pbp2_name, pbp2_log, pbp2_indx)

! initialize pbp1 and pbp2
!  if pbp1 or pbp2 field is accumulated on physics time step, old value 
!    must be retained
        ipbp_sample = 0
        do n = 1,npbp1
          if (pbp1_log(n)) then
            if (pbp1_physics(n)) then
              if (mod(step_count,n_physics) == 0) then
                pbp1(pbp1_indx(n),:) = 0.0
              else
                pbp1(pbp1_indx(n),:) = pbp1(pbp1_indx(n),:) * n_pbp
              endif
            else
              pbp1(pbp1_indx(n),:) = 0.0
            endif
          endif
        enddo

        do n = 1,npbp2
          if (pbp2_log(n)) then
            if (pbp2_physics(n)) then
              if (mod(step_count,n_physics) == 0) then
                pbp2(:,pbp2_indx(n),:) = 0.0
              else
                pbp2(:,pbp2_indx(n),:) = pbp2(:,pbp2_indx(n),:) * n_pbp
              endif
            else
              pbp2(:,pbp2_indx(n),:) = 0.0
            endif
          endif
        enddo

        end subroutine write_pbp_diagnostics


      subroutine pbp1sum(xin, x_const, index, ksdm, len, ioffset)

!  this subroutine accumulates a field in a section in the pbp1 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a pbp1
!  x_const - a weight to scale xin
!  index - the pbp1 array index specified by ipbp1_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            pbp1 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::  &
           index, &   ! index of the pbp slot to save field to
           ksdm, &    ! subdomain to fill
           len        ! length of xin array section
      integer (kind=int_kind), intent(in), optional :: ioffset

      real (kind=dbl_kind), dimension(len), intent(in) :: &
           xin  !  input field to accumulate
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

!    local variables
      integer (kind=int_kind) :: offset, n, item

!  no points to save in this subdomain
      if (kijlen(ksdm) == 0) return 

      if (pbp1_log(index)) then
         
         if (present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         
         do n = 1+koffsetpbp(ksdm), kijlen(ksdm)+koffsetpbp(ksdm)
            item = ipbploc(n) + (jpbploc(n)-1)*iim - offset
            if (item.ge.1.and.item.le.len)  &
               pbp1(pbp1_indx(index), n) = pbp1(pbp1_indx(index),n)  & 
                                         + x_const * xin(item)
         enddo

      endif !  if(pbp1_log(index))

      end subroutine pbp1sum



      subroutine pbp2sum(xin, lenx, x_const, index, ksdm, len, ioffset)

!  this subroutine accumulates a field in a section in the pbp2 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a pbp2
!  lenx - horizontal dimension of xin
!  x_const - a weight to scale xin
!  index - the pbp2 array index specified by ipbp2_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            pbp2 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::  &
           index, &   ! index of the pbp slot to save field to
           ksdm, &    ! subdomain to fill
           len, &     ! length of xin array section
           lenx    ! horizontal dimension of xin
      integer (kind=int_kind), intent(in), optional :: ioffset

      real (kind=dbl_kind), dimension(len, nlm), intent(in) ::  &
            xin  !  input field to accumulate
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

!    local variables
      integer (kind=int_kind) :: offset, n, item

!  no points to save in this subdomain
      if (kijlen(ksdm) == 0) return 

      if (pbp2_log(index)) then
         
         if (present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         
         do n = 1+koffsetpbp(ksdm), kijlen(ksdm)+koffsetpbp(ksdm)
            item = ipbploc(n) + (jpbploc(n)-1)*iim - offset
            if(item.ge.1.and.item.le.len)  &
               pbp2(:,pbp2_indx(index), n) = pbp2(:,pbp2_indx(index),n)  & 
                                         + x_const * xin(item,:)
         enddo

      endif !  if(pbp2_log(index))

      end subroutine pbp2sum

      end module pbp_module
