module pbp_module_sfc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!   This file stores pbp variables and utility routines for 
!     initializing and storing them.
!   PBP variables are time-series diagnostics saved at user selected
!     points.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  use kinds
  use model_parameters
  use domain_decomposition_sfc
  use netcdf_pbp
  use time_manager_module
#ifdef scm
  use scm_module
#endif

  implicit none 
  save
  private

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!	INPUT FILES FOR PBP DATA
!          data/sfc_pbpopts - to specify which fields are saved
!          data/pbp_locs - lon,lat pairs to specify which grid points
!              data are saved at. Units are degrees E, N respectively.
!              one pair per record
!
!	PROCEDURE TO ADD A PBP VARIABLE TO THE MODEL
!
!       Adding a pbp1 (2-dimensional field)
!   	In this file:
!	1. Increase Npbp1_sfc by 1
!	2. Add new entries in the tables for
!          -a variable named ipbp1sfc_xxx, where xxx is the name in pbp1_name.
!               Add this to the list of ipbp1sfc variables and assign it a value
!               1 larger than the previous one.
!       3. Go to subroutine initialize_pbpinfo_sfc further down in this file.
!          Add an assignment statement for each of these variables for
!          index ipbp1sfc_xxx (these are down in a block):
!  	   -pbp1sfc_name (the name of the field conforming, if possible, to 
!          	amip conventions
!          -pbp1sfc_units ( the units of the field)
!          -pbp1sfc_descr ( A longer description of the field)
!	In the file where the variable resides
!	4. Be sure that a "use pbp_module_sfc" statement is in the subroutine
!       5. Add a pbp1sum_sfc statement for the field you wish to store.
!	The description of the pbp1sum_sfc argument list is in the file with 
!       subroutine pbp1sum_sfc
!
!
!       Hints:
!       -If the field you want to save is a linear combination of model
!        variables you can use multiple pbpp*sum calls with the appropriate 
!        weights.
!        Example - reflected TOA SW is swinc - dswtop. Use a pbp1sum call
!        for swinc with a weight of 1.0 and a call for dswtop with a weight
!        of -1.0
!       NOTE - some routines are not called every timestep. Zeros will be 
!        in the pbp slots called from such routines in the timesteps
!        they are not called. These time steps can be filtered out in 
!        post-processing (e.g. using NCO operators)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!  tables of pbp information

!-- number of installed available pbps, change these as more pbps are added
  integer (kind=int_kind), parameter :: npbp1_sfc = 81

!-- number of samples
  integer (kind=int_kind), public :: ipbp_sample_sfc

!-- names are the variable names and conform to amip standards where possible
!-- these are assigned in subroutine initialize_pbpinfo further down
  character (len=16), dimension(npbp1_sfc) :: pbp1sfc_name

!-- units are the units the fields are stored in and are mks where possible
!-- these are assigned in subroutine initialize_pbpinfo further down
  character (len=16), dimension(npbp1_sfc) :: pbp1sfc_units 

!-- descr is a description of the field
!-- these are assigned in subroutine initialize_pbpinfo further down
  character (len=80), dimension(npbp1_sfc) :: pbp1sfc_descr

  integer (kind=int_kind), dimension(npbp1_sfc) :: pbp1sfc_indx
  logical (kind=log_kind), dimension(npbp1_sfc) :: pbp1sfc_log

!-- type denotes which surface component the field exists on
!-- sib, ice, sea, ( or sfc if common to all components)
!-- these are assigned in subroutine initialize_pbpinfo_sfc further down
  character (len=3), dimension(npbp1_sfc), public :: pbp1sfc_type

! these will point to  locations in the pbp information tables
! their values are assigned sequentially, and the last value is used
!   as the dimension of the above tables
  integer (kind=int_kind), parameter, public ::  &
    ipbp1_sfc_clblp      =   1,    &
    ipbp1_sfc_blent      =   2,    &
    ipbp1_sfc_clbl       =   3,    &
    ipbp1_sfc_hfss       =   4,    &
    ipbp1_sfc_tradgrdeff =   5,    &
    ipbp1_sfc_hfls       =   6,    &
    ipbp1_sfc_ventmf	 =   7,    &
    ipbp1_sfc_tke	     =   8,    &
    ipbp1_sfc_ps	     =   9,    &
    ipbp1_sfc_psb        =  10,    &
    ipbp1_sfc_dlwbot     =  11,    &
    ipbp1_sfc_dcupr      =  12,    &
    ipbp1_sfc_dlspr      =  13,    &
    ipbp1_sfc_dlsps      =  14,    &
    ipbp1_sfc_spdm       =  15,    &
    ipbp1_sfc_shear1     =  16,    &
    ipbp1_sfc_tpblp2     =  17,    &
    ipbp1_sfc_tpblp1     =  18,    &
    ipbp1_sfc_tpbl       =  19,    &
    ipbp1_sfc_shpblp2    =  20,    &
    ipbp1_sfc_shpblp1    =  21,    &
    ipbp1_sfc_shpbl      =  22,    &
    ipbp1_sfc_zs         =  23,    &
    ipbp1_sfc_www1       =  24,    &
    ipbp1_sfc_www2       =  25,    &
    ipbp1_sfc_www3       =  26,    &
    ipbp1_sfc_assimn     =  27,    &
    ipbp1_sfc_respg      =  28,    &
    ipbp1_sfc_co2flx     =  29,    &
    ipbp1_sfc_gprod      =  30,    &
    ipbp1_sfc_cons       =  31,    &
    ipbp1_sfc_bync       =  32,    &
    ipbp1_sfc_totshear   =  33,    &
    ipbp1_sfc_eci        =  34,    &
    ipbp1_sfc_ect        =  35,    &
    ipbp1_sfc_egi        =  36,    &
    ipbp1_sfc_egs        =  37,    &
    ipbp1_sfc_exo        =  38,    &
    ipbp1_sfc_q3o        =  39,    &
    ipbp1_sfc_q3l        =  40,    &
    ipbp1_sfc_tcas       =  41,    &
    ipbp1_sfc_qcas       =  42,    &
    ipbp1_sfc_rhcas      =  43,    &
    ipbp1_sfc_capac1     =  44,    &
    ipbp1_sfc_capac2     =  45,    &
    ipbp1_sfc_tcan       =  46,    &
    ipbp1_sfc_tg         =  47,    &
    ipbp1_sfc_rstfac1    =  48,    &
    ipbp1_sfc_rstfac2    =  49,    &
    ipbp1_sfc_rstfac3    =  50,    &
    ipbp1_sfc_rstfac4    =  51,    &
    ipbp1_sfc_zlt        =  52,    &
    ipbp1_sfc_green      =  53,    &
    ipbp1_sfc_vcover     =  54,    &
    ipbp1_sfc_ra         =  55,    &
    ipbp1_sfc_rb         =  56,    &
    ipbp1_sfc_rd         =  57,    &
    ipbp1_sfc_rst        =  58,    &
    ipbp1_sfc_chf        =  59,    &
    ipbp1_sfc_shf        =  60,    &
    ipbp1_sfc_radc31     =  61,    &
    ipbp1_sfc_radc32     =  62,    &
    ipbp1_sfc_mrro       =  63,    &
    ipbp1_sfc_mrso       =  64,    &
    ipbp1_sfc_lciinc     =  65,    &
    ipbp1_sfc_safe       =  66,    &
    ipbp1_sfc_snw        =  67,    &
    ipbp1_sfc_ustar      =  68,    &
    ipbp1_sfc_z0         =  69,    &
    ipbp1_sfc_buoyf 	 =  70,    &
    ipbp1_sfc_mol        =  71,    &
    ipbp1_sfc_spd10      =  72,    &
    ipbp1_sfc_t2         =  73,    &
    ipbp1_sfc_dtlci1     =  74,    &
    ipbp1_sfc_dtlci2     =  75,    &
    ipbp1_sfc_dqlci1     =  76,    &
    ipbp1_sfc_dqlci2     =  77,    &
    ipbp1_sfc_dzm        =  78,    &
    ipbp1_sfc_tbp        =  79,    &
    ipbp1_sfc_wbp        =  80,    &
    ipbp1_sfc_thvgm      =  81

!-- number of saved fields
  integer (kind=int_kind) :: ipbp1_sfc = 0

!-- number of saved gridpoints (per processor)
  integer (kind=int_kind) :: ijlen_sfc = 0

!-- number of saved gridpoints (per subdomain)
  integer (kind=int_kind), dimension(nsdm) :: kijlen_sfc = 0

!-- offset to first saved gridpoint in subdomain
  integer (kind=int_kind), dimension(nsdm) :: koffsetpbp_sfc = 0

!-- pbp data slots
  real (kind=real_kind), allocatable, dimension(:,:) :: pbp1_sfc

!-- pbp locations
  real (kind=real_kind), allocatable, dimension(:) :: lonpbp_sfc, latpbp_sfc
  integer (kind=int_kind), allocatable, dimension(:) ::  &
    ipbploc_sfc, jpbploc_sfc, ksdmpbploc_sfc
  character (len=3), allocatable, dimension(:) :: surface_type

  public :: initialize_pbpinfo_sfc, initialize_pbpfiles_sfc, &
            write_pbp_diagnostics_sfc, pbp1sum_sfc

  contains


  subroutine initialize_pbpinfo_sfc

!-- this subroutine assigns all the pbp information from the input file sfc_pbpopts

    use fluxcoupler_fields_sfc
    use sib_parameters
    use ocean_parameters
    use landice_parameters
    implicit none

    integer (kind=int_kind) :: ldummy, ioerr, n, i, item, j, ksdm, &
                               ireturn=0, jtem, ktem
    logical (kind=log_kind) :: dopbptem
    character (len=16) :: nametem
    real (kind=dbl_kind) :: lattem, lontem, xtem, ytem, ztem, dist, xdist, xdistmax
    real (kind=dbl_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) :: &
      pbp_g, x_phys, y_phys, z_phys, lon_g, lat_g

!-- read pbp location data

#ifdef scm
! always save pbp for scm
    ijlen_sfc = 1
    kijlen_sfc = 1
#else
!-----------------------------------------------------------------------
! open file, check error status
!-----------------------------------------------------------------------
    pbp_g = c0

!-- get x,y,z for atm grid points
    do ksdm = 1,nsdm_sfc
      do j = 1,jjm_sfc
        do i = 1,iim_sfc
          z_phys(i,j,ksdm) = cos(lat_sfc(i,j,ksdm))
          x_phys(i,j,ksdm) = cos(lon_sfc(i,j,ksdm)) * z_phys(i,j,ksdm)
          y_phys(i,j,ksdm) = sin(lon_sfc(i,j,ksdm)) * z_phys(i,j,ksdm)
          z_phys(i,j,ksdm) = sin(lat_sfc(i,j,ksdm))
        enddo
      enddo
    enddo
    lon_g = lon_sfc * 180. / pi
    lat_g = lat_sfc * 180. / pi

    open(unit=2,file=trim(datapath)//'data/pbp_locs',form='formatted',  &
         iostat=ioerr)
    if (ioerr /= 0) then
      write(6,*) ' pbp_locs file does not exist, no pbp data will be written'
      ireturn = 1 
    else
      do
        read(2,*,iostat=ioerr)lontem, lattem
        if (ioerr /= 0) exit
!-- find nearest atm grid point for each specified pbp lat,lon
!-- dist and xdist are dot products, greatest dot product is the point we want
        dist = -1.
        item = 1; jtem = 1; ktem = 1
        xdistmax = -1.
        ztem = cos(lattem*pi/180.)
        xtem = cos(lontem*pi/180.) * ztem
        ytem = sin(lontem*pi/180.) * ztem
        ztem = sin(lattem*pi/180.)
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if (grid_mask_sfc(i,j,ksdm)) then
                xdist = xtem*x_phys(i,j,ksdm) + ytem*y_phys(i,j,ksdm) &
                                              + ztem*z_phys(i,j,ksdm)
                if (xdist > dist) then
                  dist = xdist
                  item = i
                  jtem = j
                  ktem = ksdm
                endif
              endif
            enddo
          enddo
        enddo

#ifdef mpi
!  is this the processor with the max dot product?
!  if not, reset item, jtem and ktem to 1,1,1 where grid_mask_sfc is false
        call MPI_ALLREDUCE(dist, xdistmax, 1,  &
                           mpi_float_type, MPI_MAX, MPI_COMM_SFC, ierr)
        if (dist < xdistmax) then
          item = 1
          jtem = 1
          ktem = 1
        endif

#endif

        if (grid_mask_sfc(item,jtem,ktem)) then
          if (pbp_g(item,jtem,ktem) > 0.5) then
             print*,'gridpoint nearest pbp lon lat ',lontem, lattem, &
                   'is already selected'
          else
            pbp_g(item,jtem,ktem) = c1
            print*,' for pbp lon,lat ',lontem, lattem, &
                   ' the nearest grid cell is at ' ,   &
                   lon_g(item,jtem,ktem),lat_g(item,jtem,ktem)
          endif
        endif
      enddo
    endif   ! if pbp_locs file is found
    close(2)
#ifdef mpi
    call MPI_BCAST(       &
         ireturn,         &
         1,               &
         mpi_int_type,    &
         0,               &
         MPI_COMM_SFC,    &
         ierr)
#endif
    if (ireturn == 1) return
#endif

!-- initialize
	pbp1sfc_name(:) = " no name pbp1"
	pbp1sfc_units(:) = "no units pbp1"
	pbp1sfc_descr(:) = "no description pbp1"
	pbp1sfc_type(:) = "sfc"
      
!  do assignments for variable name, units and description 
    pbp1sfc_name (ipbp1_sfc_blent ) = "blent"
    pbp1sfc_units(ipbp1_sfc_blent ) = "kg/m^2/s"
    pbp1sfc_descr(ipbp1_sfc_blent ) = "PBL top entrainment rate"

    pbp1sfc_name (ipbp1_sfc_clblp ) = "clblp"
    pbp1sfc_units(ipbp1_sfc_clblp ) = "Pa"
    pbp1sfc_descr(ipbp1_sfc_clblp ) = "Boundary layer cloud depth"

    pbp1sfc_name (ipbp1_sfc_spdm  ) = "spdm"
    pbp1sfc_units(ipbp1_sfc_spdm  ) = "m/s"
    pbp1sfc_descr(ipbp1_sfc_spdm  ) = "PBL wind speed"

    pbp1sfc_name (ipbp1_sfc_ps    ) = "ps"
    pbp1sfc_units(ipbp1_sfc_ps    ) = "Pa"
    pbp1sfc_descr(ipbp1_sfc_ps    ) = "Surface pressure"
 
    pbp1sfc_name (ipbp1_sfc_clbl  ) = "clbl"
    pbp1sfc_units(ipbp1_sfc_clbl  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_clbl  ) = "EAUcf: Incidence of PBL clouds"

    pbp1sfc_name (ipbp1_sfc_hfss  ) = "hfss"
    pbp1sfc_units(ipbp1_sfc_hfss  ) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_hfss  ) = "SURFACE SENSIBLE HEAT FLUX"

    pbp1sfc_name (ipbp1_sfc_tradgrdeff) = "tradgrdeff "
    pbp1sfc_units(ipbp1_sfc_tradgrdeff) = "K"
    pbp1sfc_descr(ipbp1_sfc_tradgrdeff) = "Effective radiative ground temperature"

    pbp1sfc_name (ipbp1_sfc_hfls  ) = "hfls"
    pbp1sfc_units(ipbp1_sfc_hfls  ) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_hfls  ) = "Surface latent heat flux"

    pbp1sfc_name (ipbp1_sfc_ventmf  ) = "ventmf"
    pbp1sfc_units(ipbp1_sfc_ventmf  ) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_ventmf  ) = "ventilation mass flux"

    pbp1sfc_name (ipbp1_sfc_tke  ) = "tke"
    pbp1sfc_units(ipbp1_sfc_tke  ) = "m2/s2"
    pbp1sfc_descr(ipbp1_sfc_tke  ) = "pbl turbulent kinetic energy"

    pbp1sfc_name (ipbp1_sfc_psb  ) = "psb" 
    pbp1sfc_units(ipbp1_sfc_psb  ) = "mb"
    pbp1sfc_descr(ipbp1_sfc_psb  ) = "PBL depth"

    pbp1sfc_name (ipbp1_sfc_dlwbot  ) = "dlwbot" 
    pbp1sfc_units(ipbp1_sfc_dlwbot  ) = "W/m2"
    pbp1sfc_descr(ipbp1_sfc_dlwbot  ) = "down-welling longwave radiation"

    pbp1sfc_name (ipbp1_sfc_dcupr  ) = "dcupr" 
    pbp1sfc_units(ipbp1_sfc_dcupr  ) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_dcupr  ) = "convective precipitation"

    pbp1sfc_name (ipbp1_sfc_dlspr  ) = "dlspr" 
    pbp1sfc_units(ipbp1_sfc_dlspr  ) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_dlspr  ) = "large-scale stratiform rain"

    pbp1sfc_name (ipbp1_sfc_dlsps  ) = "dlsps" 
    pbp1sfc_units(ipbp1_sfc_dlsps  ) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_dlsps  ) = "large-scale stratiform snow"

    pbp1sfc_name (ipbp1_sfc_shear1  ) = "shear1" 
    pbp1sfc_units(ipbp1_sfc_shear1  ) = "m2/s2"
    pbp1sfc_descr(ipbp1_sfc_shear1  ) = "delta u across pbl squared"

    pbp1sfc_name (ipbp1_sfc_tpblp2) = "tpblp2"
    pbp1sfc_units(ipbp1_sfc_tpblp2) = "K"
    pbp1sfc_descr(ipbp1_sfc_tpblp2) = "potential temp 2 layers above pbl"

    pbp1sfc_name (ipbp1_sfc_tpblp1) = "tpblp1" 
    pbp1sfc_units(ipbp1_sfc_tpblp1) = "K"
    pbp1sfc_descr(ipbp1_sfc_tpblp1) = "potential temp 1 layer above pbl"

    pbp1sfc_name (ipbp1_sfc_tpbl) = "tpbl" 
    pbp1sfc_units(ipbp1_sfc_tpbl) = "K"
    pbp1sfc_descr(ipbp1_sfc_tpbl) = "potential temp of pbl"

    pbp1sfc_name (ipbp1_sfc_shpblp2) = "shpblp2" 
    pbp1sfc_units(ipbp1_sfc_shpblp2) = "kg of water /kg of air"
    pbp1sfc_descr(ipbp1_sfc_shpblp2) = "specific humidity 2 layers above pbl"

    pbp1sfc_name (ipbp1_sfc_shpblp1) = "shpblp1" 
    pbp1sfc_units(ipbp1_sfc_shpblp1) = "kg of water /kg of air"
    pbp1sfc_descr(ipbp1_sfc_shpblp1) = "specific humidity 1 layer above pbl"

    pbp1sfc_name (ipbp1_sfc_shpbl) = "shpbl" 
    pbp1sfc_units(ipbp1_sfc_shpbl) = "kg of water /kg of air"
    pbp1sfc_descr(ipbp1_sfc_shpbl) = "specific humidity of pbl"

    pbp1sfc_name (ipbp1_sfc_zs) = "zs"
    pbp1sfc_units(ipbp1_sfc_zs) = "m^2/s^2"
    pbp1sfc_descr(ipbp1_sfc_zs) = "surface geopotential"

    pbp1sfc_name (ipbp1_sfc_co2flx) = "co2flx"
    pbp1sfc_units(ipbp1_sfc_co2flx) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_co2flx) = "sib surface carbon flux"

    pbp1sfc_name (ipbp1_sfc_www1  ) = "www1"
    pbp1sfc_units(ipbp1_sfc_www1  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_www1  ) = "soil moisture layer 1"
	pbp1sfc_type (ipbp1_sfc_www1)   = "sib"

    pbp1sfc_name (ipbp1_sfc_www2  ) = "www2"
    pbp1sfc_units(ipbp1_sfc_www2  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_www2  ) = "soil moisture layer 2"
	pbp1sfc_type (ipbp1_sfc_www2)   = "sib"

    pbp1sfc_name (ipbp1_sfc_www3  ) = "www3"
    pbp1sfc_units(ipbp1_sfc_www3  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_www3  ) = "soil moisture layer 3"
	pbp1sfc_type (ipbp1_sfc_www3)   = "sib"

    pbp1sfc_name (ipbp1_sfc_assimn  ) = "assimn"
    pbp1sfc_units(ipbp1_sfc_assimn  ) = "moles/m2/s"
    pbp1sfc_descr(ipbp1_sfc_assimn  ) = "net carbon assimilation"
	pbp1sfc_type (ipbp1_sfc_assimn)   = "sib"

    pbp1sfc_name (ipbp1_sfc_respg  ) = "respg"
    pbp1sfc_units(ipbp1_sfc_respg  ) = "moles/m2/s"
    pbp1sfc_descr(ipbp1_sfc_respg  ) = "carbon respiration"
	pbp1sfc_type (ipbp1_sfc_respg)   = "sib"

!-- added by mb
    pbp1sfc_name (ipbp1_sfc_gprod  ) = "gprod"
    pbp1sfc_units(ipbp1_sfc_gprod  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_gprod  ) = "PBL Gross TKE Production"

    pbp1sfc_name (ipbp1_sfc_cons  ) = "cons"
    pbp1sfc_units(ipbp1_sfc_cons  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_cons  ) = "PBL TKE Consumption"

    pbp1sfc_name (ipbp1_sfc_bync  ) = "bync"
    pbp1sfc_units(ipbp1_sfc_bync  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_bync  ) = "PBL Buoyancy"

    pbp1sfc_name (ipbp1_sfc_totshear  ) = "shear"
    pbp1sfc_units(ipbp1_sfc_totshear  ) = "-"
    pbp1sfc_descr(ipbp1_sfc_totshear  ) = "PBL Total Shear"

    pbp1sfc_name (ipbp1_sfc_eci) = "eci"
    pbp1sfc_units(ipbp1_sfc_eci) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_eci) = "Evaporation of canopy interception"
    pbp1sfc_type (ipbp1_sfc_eci) = "sib"

    pbp1sfc_name (ipbp1_sfc_ect) = "ect"
    pbp1sfc_units(ipbp1_sfc_ect) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_ect) = "Canopy transpiration"
    pbp1sfc_type (ipbp1_sfc_ect) = "sib"

    pbp1sfc_name (ipbp1_sfc_egi) = "egi"
    pbp1sfc_units(ipbp1_sfc_egi) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_egi) = "Evaporation of surface interception"
    pbp1sfc_type (ipbp1_sfc_egi) = "sib"

    pbp1sfc_name (ipbp1_sfc_egs) = "egs"
    pbp1sfc_units(ipbp1_sfc_egs) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_egs) = "Evaporation of surface soil moisture"
    pbp1sfc_type (ipbp1_sfc_egs) = "sib"

    pbp1sfc_name (ipbp1_sfc_exo) = "exo"
    pbp1sfc_units(ipbp1_sfc_exo) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_exo) = "Runoff due to excess soil moisture"
    pbp1sfc_type (ipbp1_sfc_exo) = "sib"

    pbp1sfc_name (ipbp1_sfc_q3o) = "q3o"
    pbp1sfc_units(ipbp1_sfc_q3o) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_q3o) = "Runoff due to gravitational drainage"
    pbp1sfc_type (ipbp1_sfc_q3o) = "sib"

    pbp1sfc_name (ipbp1_sfc_q3l) = "q3l"
    pbp1sfc_units(ipbp1_sfc_q3l) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_q3l) = "Liston runoff term"
    pbp1sfc_type (ipbp1_sfc_q3l) = "sib"

    pbp1sfc_name (ipbp1_sfc_tcas) = "tcas"
    pbp1sfc_units(ipbp1_sfc_tcas) = "K"
    pbp1sfc_descr(ipbp1_sfc_tcas) = "Canopy air space temperature"
    pbp1sfc_type (ipbp1_sfc_tcas) = "sib"

    pbp1sfc_name (ipbp1_sfc_qcas) = "qcas"
    pbp1sfc_units(ipbp1_sfc_qcas) = "kg/kg"
    pbp1sfc_descr(ipbp1_sfc_qcas) = "Canopy air space water vapor"
    pbp1sfc_type (ipbp1_sfc_qcas) = "sib"

    pbp1sfc_name (ipbp1_sfc_rhcas) = "rhcas"
    pbp1sfc_units(ipbp1_sfc_rhcas) = "-"
    pbp1sfc_descr(ipbp1_sfc_rhcas) = "Canopy air space relative humidity"
    pbp1sfc_type (ipbp1_sfc_rhcas) = "sib"

    pbp1sfc_name (ipbp1_sfc_capac1) = "capac1"
    pbp1sfc_units(ipbp1_sfc_capac1) = "kg/m2"
    pbp1sfc_descr(ipbp1_sfc_capac1) = "Canopy intercepted liquid water"
    pbp1sfc_type (ipbp1_sfc_capac1) = "sib"

    pbp1sfc_name (ipbp1_sfc_capac2) = "capac2"
    pbp1sfc_units(ipbp1_sfc_capac2) = "kg/m2"
    pbp1sfc_descr(ipbp1_sfc_capac2) = "Surface intercepted liquid water"
    pbp1sfc_type (ipbp1_sfc_capac2) = "sib"

    pbp1sfc_name (ipbp1_sfc_tcan) = "tcan"
    pbp1sfc_units(ipbp1_sfc_tcan) = "K"
    pbp1sfc_descr(ipbp1_sfc_tcan) = "Canopy temperature"
    pbp1sfc_type (ipbp1_sfc_tcan) = "sib"

    pbp1sfc_name (ipbp1_sfc_tg) = "tg"
    pbp1sfc_units(ipbp1_sfc_tg) = "K"
    pbp1sfc_descr(ipbp1_sfc_tg) = "Surface soil temperature"
    pbp1sfc_type (ipbp1_sfc_tg) = "sib"

    pbp1sfc_name (ipbp1_sfc_rstfac1) = "rstfac1"
    pbp1sfc_units(ipbp1_sfc_rstfac1) = "-"
    pbp1sfc_descr(ipbp1_sfc_rstfac1) = "Humidity stress factor"
    pbp1sfc_type (ipbp1_sfc_rstfac1) = "sib"

    pbp1sfc_name (ipbp1_sfc_rstfac2) = "rstfac2"
    pbp1sfc_units(ipbp1_sfc_rstfac2) = "-"
    pbp1sfc_descr(ipbp1_sfc_rstfac2) = "Soil moisture stress factor"
    pbp1sfc_type (ipbp1_sfc_rstfac2) = "sib"

    pbp1sfc_name (ipbp1_sfc_rstfac3) = "rstfac3"
    pbp1sfc_units(ipbp1_sfc_rstfac3) = "-"
    pbp1sfc_descr(ipbp1_sfc_rstfac3) = "Temperature stress factor"
    pbp1sfc_type (ipbp1_sfc_rstfac3) = "sib"

    pbp1sfc_name (ipbp1_sfc_rstfac4) = "rstfac4"
    pbp1sfc_units(ipbp1_sfc_rstfac4) = "-"
    pbp1sfc_descr(ipbp1_sfc_rstfac4) = "Product of stress factors 1-3"
    pbp1sfc_type (ipbp1_sfc_rstfac4) = "sib"

    pbp1sfc_name (ipbp1_sfc_zlt) = "zlt"
    pbp1sfc_units(ipbp1_sfc_zlt) = "m^2/m^2"
    pbp1sfc_descr(ipbp1_sfc_zlt) = "Total leaf area index"
    pbp1sfc_type (ipbp1_sfc_zlt) = "sib"

    pbp1sfc_name (ipbp1_sfc_green) = "green"
    pbp1sfc_units(ipbp1_sfc_green) = "-"
    pbp1sfc_descr(ipbp1_sfc_green) = "Fraction of zlt that is green"
    pbp1sfc_type (ipbp1_sfc_green) = "sib"

    pbp1sfc_name (ipbp1_sfc_vcover) = "vcover"
    pbp1sfc_units(ipbp1_sfc_vcover) = "-"
    pbp1sfc_descr(ipbp1_sfc_vcover) = "Horizontal vegetation cover"
    pbp1sfc_type (ipbp1_sfc_vcover) = "sib"

    pbp1sfc_name (ipbp1_sfc_ra) = "ra"
    pbp1sfc_units(ipbp1_sfc_ra) = "s/m"
    pbp1sfc_descr(ipbp1_sfc_ra) = "Aerodynamic resistance"
    pbp1sfc_type (ipbp1_sfc_ra) = "sib"

    pbp1sfc_name (ipbp1_sfc_rb) = "rb"
    pbp1sfc_units(ipbp1_sfc_rb) = "s/m"
    pbp1sfc_descr(ipbp1_sfc_rb) = "Bulk canopy boundary layer resistance"
    pbp1sfc_type (ipbp1_sfc_rb) = "sib"

    pbp1sfc_name (ipbp1_sfc_rd) = "rd"
    pbp1sfc_units(ipbp1_sfc_rd) = "s/m"
    pbp1sfc_descr(ipbp1_sfc_rd) = "Ground-canopy air space resistance"
    pbp1sfc_type (ipbp1_sfc_rd) = "sib"

    pbp1sfc_name (ipbp1_sfc_rst) = "rst"
    pbp1sfc_units(ipbp1_sfc_rst) = "s/m"
    pbp1sfc_descr(ipbp1_sfc_rst) = "Stomatal resistance"
    pbp1sfc_type (ipbp1_sfc_rst) = "sib"

    pbp1sfc_name (ipbp1_sfc_chf) = "chf"
    pbp1sfc_units(ipbp1_sfc_chf) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_chf) = "Canopy heat storage flux"
    pbp1sfc_type (ipbp1_sfc_chf) = "sib"

    pbp1sfc_name (ipbp1_sfc_shf) = "shf"
    pbp1sfc_units(ipbp1_sfc_shf) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_shf) = "Soil heat storage flux"
    pbp1sfc_type (ipbp1_sfc_shf) = "sib"

    pbp1sfc_name (ipbp1_sfc_radc31) = "radc31"
    pbp1sfc_units(ipbp1_sfc_radc31) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_radc31) = "Radiation intercepted by canopy"
    pbp1sfc_type (ipbp1_sfc_radc31) = "sib"

    pbp1sfc_name (ipbp1_sfc_radc32) = "radc32"
    pbp1sfc_units(ipbp1_sfc_radc32) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_radc32) = "Radiation intercepted by ground"
    pbp1sfc_type (ipbp1_sfc_radc32) = "sib"

    pbp1sfc_name (ipbp1_sfc_mrro  ) = "mrro"
    pbp1sfc_units(ipbp1_sfc_mrro  ) = "kg/m2/s"
    pbp1sfc_descr(ipbp1_sfc_mrro  ) = "Total Runoff"
    pbp1sfc_type (ipbp1_sfc_mrro)   = "sib"

    pbp1sfc_name (ipbp1_sfc_mrso  ) = "mrso"
    pbp1sfc_units(ipbp1_sfc_mrso  ) = "kg/m2"
    pbp1sfc_descr(ipbp1_sfc_mrso  ) = "Total Soil Water Content"
    pbp1sfc_type (ipbp1_sfc_mrso)   = "sib"

    pbp1sfc_name (ipbp1_sfc_lciinc) = "lciinc" 
    pbp1sfc_units(ipbp1_sfc_lciinc) = "-"
    pbp1sfc_descr(ipbp1_sfc_lciinc) = "Layer Cloud Instability Incidence"

    pbp1sfc_name (ipbp1_sfc_safe) = "safe" 
    pbp1sfc_units(ipbp1_sfc_safe) = "J/kg"
    pbp1sfc_descr(ipbp1_sfc_safe) = "SAFE"

    pbp1sfc_name (ipbp1_sfc_snw  ) = "snw"
    pbp1sfc_units(ipbp1_sfc_snw  ) = "kg/m2"
    pbp1sfc_descr(ipbp1_sfc_snw  ) = "Snow depth (water equivalent)"
    pbp1sfc_type (ipbp1_sfc_snw  ) = "sib"

    pbp1sfc_name (ipbp1_sfc_ustar) = "ustar"
    pbp1sfc_units(ipbp1_sfc_ustar) = "m/s"
    pbp1sfc_descr(ipbp1_sfc_ustar) = "Friction velocity"

    pbp1sfc_name (ipbp1_sfc_z0  ) = "z0"
    pbp1sfc_units(ipbp1_sfc_z0  ) = "m"
    pbp1sfc_descr(ipbp1_sfc_z0  ) = "Surface Roughness Height"
    pbp1sfc_type (ipbp1_sfc_z0  )   = "sib"

    pbp1sfc_name (ipbp1_sfc_buoyf) = "buoyf"
    pbp1sfc_units(ipbp1_sfc_buoyf) = "W/m^2"
    pbp1sfc_descr(ipbp1_sfc_buoyf) = "Surface Buoyancy Flux"

    pbp1sfc_name (ipbp1_sfc_mol) = "L"
    pbp1sfc_units(ipbp1_sfc_mol) = "m"
    pbp1sfc_descr(ipbp1_sfc_mol) = "Monin-Obukhov length"

    pbp1sfc_name (ipbp1_sfc_spd10) = "spd10"
    pbp1sfc_units(ipbp1_sfc_spd10) = "m/s"
    pbp1sfc_descr(ipbp1_sfc_spd10) = "Wind speed at 10m"

    pbp1sfc_name (ipbp1_sfc_t2) = "t2"
    pbp1sfc_units(ipbp1_sfc_t2) = "K"
    pbp1sfc_descr(ipbp1_sfc_t2) = "Temperature at 2m"

    pbp1sfc_name (ipbp1_sfc_dtlci1) = "dtlci1"
    pbp1sfc_units(ipbp1_sfc_dtlci1) = "K/day"
    pbp1sfc_descr(ipbp1_sfc_dtlci1) = "LCI heating above PBL"

    pbp1sfc_name (ipbp1_sfc_dtlci2) = "dtlci2"
    pbp1sfc_units(ipbp1_sfc_dtlci2) = "K/day"
    pbp1sfc_descr(ipbp1_sfc_dtlci2) = "LCI heating in PBL"

    pbp1sfc_name (ipbp1_sfc_dqlci1) = "dqlci1"
    pbp1sfc_units(ipbp1_sfc_dqlci1) = "g/kg/day"
    pbp1sfc_descr(ipbp1_sfc_dqlci1) = "LCI moistening above PBL"

    pbp1sfc_name (ipbp1_sfc_dqlci2) = "dqlci2"
    pbp1sfc_units(ipbp1_sfc_dqlci2) = "g/kg/day"
    pbp1sfc_descr(ipbp1_sfc_dqlci2) = "LCI moistening in PBL"

    pbp1sfc_name (ipbp1_sfc_dzm  ) = "dzm" 
    pbp1sfc_units(ipbp1_sfc_dzm  ) = "m"
    pbp1sfc_descr(ipbp1_sfc_dzm  ) = "Boundary layer depth"

    pbp1sfc_name (ipbp1_sfc_tbp) = "tbp"
    pbp1sfc_units(ipbp1_sfc_tbp) = "K"
    pbp1sfc_descr(ipbp1_sfc_tbp) = "Level B+ Temperature"

    pbp1sfc_name (ipbp1_sfc_wbp) = "wbp"
    pbp1sfc_units(ipbp1_sfc_wbp) = "kg/kg"
    pbp1sfc_descr(ipbp1_sfc_wbp) = "Level B+ Water Vapor Mixing Ratio"

    pbp1sfc_name (ipbp1_sfc_thvgm) = "thvgm"
    pbp1sfc_units(ipbp1_sfc_thvgm) = "K"
    pbp1sfc_descr(ipbp1_sfc_thvgm) = "Virtual Temperature Difference"

	do n=1,npbp1_sfc
      pbp1sfc_name(n) = trim(pbp1sfc_name(n))
      pbp1sfc_units(n) = trim(pbp1sfc_units(n))
      pbp1sfc_descr(n) = trim(pbp1sfc_descr(n))
	enddo

!-- initialize some pbp tables
    pbp1sfc_indx = 0
    pbp1sfc_log = .false.

    pbp1sfc_log(ipbp1_sfc_ps) = .true.
    pbp1sfc_indx(ipbp1_sfc_ps) = 1
    ipbp1_sfc = 1

!-- read input data
    if (my_task == 0) then
      open(unit=2,file=trim(datapath)//'data/sfc_pbpopts', form='formatted')
      do
        read(2,*,IOSTAT=ioerr) dopbptem,ldummy,nametem
	    nametem = trim(nametem)
        if (ioerr /= 0) exit
        if (dopbptem) then
          do n = 1,npbp1_sfc
            if (nametem == pbp1sfc_name(n)) then

!   test if this is an automatically saved field
              if (nametem == pbp1sfc_name(ipbp1_sfc_ps)) exit

              ipbp1_sfc = ipbp1_sfc + 1
              pbp1sfc_log(n) = .true.
              pbp1sfc_indx(n) = ipbp1_sfc
              exit
            endif
            if (n == npbp1_sfc)  &
              print *,nametem,' is not an available pbp1 diagnostic'
          enddo
        endif
      enddo

      close(2)

      write(6,*)
      do n = 1,npbp1_sfc
        if (pbp1sfc_log(n)) then
          write(6,11) pbp1sfc_name(n), pbp1sfc_descr(n),pbp1sfc_indx(n)
 11       format('Will save sfc pbp1 : ',a16,5x,a20, ' in pbp1 slot ', i8)
        endif
      enddo
	  write(6,*)
    endif  ! if my_task.eq.0

!-- broadcast information

#ifdef mpi
    call MPI_BCAST(            &
            pbp1sfc_indx,      &
            npbp1_sfc,         &
            mpi_int_type,      &
            0,                 &
            MPI_COMM_SFC,      &
            ierr)

    call MPI_BCAST(            &
            pbp1sfc_log,       &
            npbp1_sfc,         &
            MPI_LOGICAL,       &
            0,                 &
            MPI_COMM_SFC,      &
            ierr)
        
    call MPI_BCAST(            &
            ipbp1_sfc,         &
            1,                 &
            mpi_int_type,      &
            0,                 &
            MPI_COMM_SFC,      &
            ierr)

#endif


!-- no pbp saved at any grid point on this processor
#ifdef scm
    ijlen_sfc = 1
    kijlen_sfc = 1
#else
    do ksdm = 1,nsdm_sfc
      do j = 1,jjm_sfc
        do i = 1,iim_sfc
          if (pbp_g(i,j,ksdm) > 0.5_dbl_kind .and. grid_mask_sfc(i,j,ksdm)) then
            ijlen_sfc = ijlen_sfc + 1
          endif
        enddo
      enddo
    enddo
#endif
    if (ijlen_sfc == 0) return 

!  allocate the pbp arrays

    allocate(pbp1_sfc(0:ipbp1_sfc, ijlen_sfc))
    allocate(lonpbp_sfc(ijlen_sfc))
    allocate(latpbp_sfc(ijlen_sfc))
    allocate(ipbploc_sfc(ijlen_sfc))
    allocate(jpbploc_sfc(ijlen_sfc))
    allocate(ksdmpbploc_sfc(ijlen_sfc))
    allocate(surface_type(ijlen_sfc))

#ifdef scm
    !lonpbp_sfc(1) = lon_sfc(1,1,1) * 180. / pi
    !latpbp_sfc(1) = lat_sfc(1,1,1) * 180. / pi
    lonpbp_sfc(1) = lon_scm * 180. / pi
    latpbp_sfc(1) = lat_scm * 180. / pi
    ipbploc_sfc(1) = 1
    jpbploc_sfc(1) = 1
    ksdmpbploc_sfc(1) = 1
#else
    ijlen_sfc = 0
    do ksdm = 1,nsdm_sfc
      koffsetpbp_sfc(ksdm) = ijlen_sfc
      do j = 1,jjm_sfc
      do i = 1,iim_sfc
        if (pbp_g(i,j,ksdm) > 0.5_dbl_kind .and. grid_mask_sfc(i,j,ksdm)) then
          kijlen_sfc(ksdm) = kijlen_sfc(ksdm) + 1
          ijlen_sfc = ijlen_sfc + 1
          lonpbp_sfc(ijlen_sfc) = lon_sfc(i,j,ksdm) * 180. / pi
          latpbp_sfc(ijlen_sfc) = lat_sfc(i,j,ksdm) * 180. / pi
          ipbploc_sfc(ijlen_sfc) = i
          jpbploc_sfc(ijlen_sfc) = j
          ksdmpbploc_sfc(ijlen_sfc) = ksdm
        endif
      enddo
      enddo
    enddo
#endif

    do i = 1,ijlen_sfc
      if (hr_sea(ipbploc_sfc(i),jpbploc_sfc(i),ksdmpbploc_sfc(i))) then
        surface_type(i) = 'ocn'
      else if (hr_sib(ipbploc_sfc(i),jpbploc_sfc(i),ksdmpbploc_sfc(i))) then
        surface_type(i) = 'sib'
      else if (hr_ice(ipbploc_sfc(i),jpbploc_sfc(i),ksdmpbploc_sfc(i))) then
        surface_type(i) = 'ice'
      endif
    enddo

    pbp1_sfc = 0.0

  end subroutine initialize_pbpinfo_sfc


  subroutine initialize_pbpfiles_sfc( simulation_tag )

!-- this subroutine assigns initializes the pbp output files

    implicit none

    character (len=16), intent(in) :: simulation_tag

    character (len=8) :: time_label
    character (len=4) :: proc_label
    integer (kind=int_kind) :: monthofyear

    if (ijlen_sfc == 0) return

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
    if (n_restart > 0) then
      write(unit=time_label,fmt='(i7)') 1000000+nsecond_dyn/86400
      time_label(1:1) = 'd'
    else
      monthofyear = mod(nmonth-1,12)+1
      write(unit=time_label,fmt='(i7)') 1000000+100*(nyear+nyear0)+monthofyear 
      time_label(1:1) = 'y'
    endif

    write(unit=proc_label,fmt='(i4)') 1000+my_task
    proc_label(1:1) = 'p'

    ipbp_sample_sfc = 0
    pbp1_sfc = 0.0

    call pinit_sfc_nc(simulation_tag,                                     &
                      'pbp_output/psfc_'//trim(time_label)                &
                      //trim(proc_label)//'.pbp1.nc',                     &
                      pbp1sfc_descr, pbp1sfc_name, pbp1sfc_units,         &
                      pbp1sfc_type, pbp1sfc_log, lonpbp_sfc, latpbp_sfc,  &
                      max_ig_sfc, surface_type)

  end subroutine initialize_pbpfiles_sfc


  subroutine write_pbp_diagnostics_sfc

    implicit none

    real (kind=dbl_kind) :: pbp1_sample_inv

    if (ijlen_sfc == 0) return

    pbp1_sample_inv = 1. / ipbp_sample_sfc

    pbp1_sfc(1:ipbp1_sfc,:) = pbp1_sfc(1:ipbp1_sfc,:) * pbp1_sample_inv

    call pwrt_sfc_nc(nsecond_dyn, nyear0, pbp1_sfc(1:ipbp1_sfc,:), &
                     pbp1sfc_name, pbp1sfc_log, pbp1sfc_indx)

    ipbp_sample_sfc = 0
    pbp1_sfc = 0.0

  end subroutine write_pbp_diagnostics_sfc


  subroutine pbp1sum_sfc(xin, x_const, index, ksdm, len, ioffset)

!-- this subroutine accumulates a field in a section in the pbp1_sfc array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a pbp1
!  x_const - a weight to scale xin
!  index - the pbp1 array index specified by ipbp1_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            pbp1 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    implicit none

    integer (kind=int_kind), intent(in) ::                &
      index, &   ! index of the pbp slot to save field to
      ksdm, &    ! subdomain to fill
      len        ! length of xin array section
    integer (kind=int_kind), intent(in), optional :: ioffset

    real (kind=dbl_kind), dimension(len), intent(in) :: xin  !  input field to accumulate
    real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

!-- local variables
    integer (kind=int_kind) :: offset, n, item

!-- no points to save in this subdomain
    if (kijlen_sfc(ksdm) == 0) return 

    if (pbp1sfc_log(index)) then
         
      if (present(ioffset)) then
        offset = ioffset
      else
        offset = 0
      endif
         
      do n = 1+koffsetpbp_sfc(ksdm), kijlen_sfc(ksdm)+koffsetpbp_sfc(ksdm)
        item = ipbploc_sfc(n) + (jpbploc_sfc(n)-1)*iim_sfc - offset
        if (item >= 1 .and. item <= len)      &
          pbp1_sfc(pbp1sfc_indx(index), n) =  &
            pbp1_sfc(pbp1sfc_indx(index), n) + x_const * xin(item)
      enddo

    endif !  if(pbp1sfc_log(index))

  end subroutine pbp1sum_sfc
  
end module pbp_module_sfc
