      module qp_ml

!-- modified version of qp_module to write out special 3d fields on model levels
!-- (NOT interpolated pressure levels)

!  This file initializes, stores, computes and writes the diagnostic output
!     that is global and time averaged. The short-hand term for these
!     diagnostics is 'qp'. QP3 are 3 dimensional diagnostics, QP2 are
!     2 dimensional (horizontal) only.

      use kinds
      use model_parameters
      use domain_decomposition
      use sigma_coordinate
      use tracer_names
      use netcdf_geodesic
      use time_manager_module
      use gather_scatter
      use grid_metrics
      use wrap_vertex_data

      implicit none
      save
      private

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!	PROCEDURE TO ADD A QP VARIABLE TO THE MODEL
!
!       Adding a QP3 (3-dimensional field)
!   	In this file:
!	1. Increase Nqp3 by 1
!	2. Add new entries in the tables for
!          -a variable named iqp3_xxx, where xxx is the name in qp3_name.
!               Add this to the list of iqp3 variables and assign it a value
!               1 larger than the previous one.
!       3. Go to subroutine initialize_qpinfo further down in this file.
!          Add an assignment statement for each of these variables for
!          index iqp3_xxx (these are down in a block):
!  	   -qp3_name (the name of the field conforming, if possible, to 
!          	amip conventions
!          -qp3_units ( the units of the field)
!          -qp3_descr ( A longer description of the field)
!	In the file where the variable resides
!	4. Be sure that a "use qp_module" statement is in the subroutine
!       5. Add a qp3sum statement for the field you wish to store.
!	The description of the qp3sum argument list is in the file with 
!       subroutine qp3sum
!	6. If the added field is a product for which primes are desired
!          a product for the bars must be added also. The names should bear
!          the tags _prime and _bar. Look for the ua_ta_prime example for
!          the other fields that must be added with this one.
!
!       Adding a QP2 (or hf2) (horizontal only field)
!	Use the instructions above, replacing qp3 with qp2 (hf2)
!
!       Hints:
!       -If the field you want to save is a linear combination of model
!        variables you can use multiple qp*sum calls with the appropriate 
!        weights.
!        Example - reflected TOA SW is swinc - dswtop. Use a qp2sum call
!        for swinc with a weight of 1.0 and a call for dswtop with a weight
!        of -1.0
!        NOTE: if you use multiple qp2sum calls to save a diagnostics, all
!          but one call must include the step_sample argument.
!       -If the field you want to save is in a subroutine that is not called
!        every timestep, the weight should include a factor to compensate for
!        that.
!        Example - radiation is called every physics timestep, once every
!        n_physics timesteps. Include a factor of n_physics in the weight
!       -hardwired constants in the argument list must be of the right kind
!        Example - a weight of 1.0 should be written 1.0_dbl_kind
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!  tables of qp information

!   number of installed available qps, change these as more qps are added
      integer (kind=int_kind), parameter, public :: 
     *                                      nqp3ml  = 17

!   names are the variable names and conform to amip standards where possible
!   these are assigned in subroutine initialize_qpinfo further down
      character (len=16), dimension(nqp3ml), public :: qp3ml_name

!  units are the units the fields are stored in and are mks where possible
!   these are assigned in subroutine initialize_qpinfo further down
      character (len=16), dimension(nqp3ml), public :: qp3ml_units

!  descr is a description of the field
!   these are assigned in subroutine initialize_qpinfo further down
      character (len=80), dimension(nqp3ml), public :: qp3ml_descr

!  type denotes which type of variable this is (for ezplot)
      character (len=3), dimension(nqp3ml), public :: qp3ml_type

      integer (kind=int_kind), dimension(nqp3ml), public :: qp3ml_indx
      logical (kind=log_kind), dimension(nqp3ml), public :: qp3ml_log = .false.

! these will point to locations in the qp information tables
! their values are assigned sequentially, and the last value is used
!   as the dimension of the above tables
      integer (kind=int_kind) :: i
      
      integer (kind=int_kind), parameter, public ::
     *     iqp3ml_tnmrc	     = 1
     *,    iqp3ml_dqelq	     = 2
     *,    iqp3ml_fill3q     = 3
     *,    iqp3ml_fill3cw    = 4
     *,    iqp3ml_fill3ci    = 5
     *,    iqp3ml_fill3rw    = 6
     *,    iqp3ml_fill3ri    = 7
     *,    iqp3ml_tnqdc      = 8
     *,    iqp3ml_dsigdt     = 9
     *,    iqp3ml_tnmrd      = 10
     *,    iqp3ml_tnqdc_pw   = 11
     *,    iqp3ml_ta         = 12
     *,    iqp3ml_th         = 13
     *,    iqp3ml_hus        = 14
     *,    iqp3ml_hur        = 15
     *,    iqp3ml_hhl        = 16
     *,    iqp3ml_masflx     = 17


!   number of saved fields
      integer (kind=int_kind), public :: iqp3ml = 0

!   number of samples
      integer (kind=int_kind), dimension(nqp3ml), public :: iqp3ml_sample

!   time at start of summation
      integer (kind=int_kind), public :: nsecondinit

!   qp data slots
      real (kind=real_kind), allocatable, dimension(:,:,:,:,:), public :: 
     *                 qp3ml

!  underlying surface type
       real (kind=real_kind), dimension(iim,jjm,nsdm), public ::
     *           landfrac, seafrac, icefrac

!  indices for latlon interpolation
       integer (kind=int_kind), dimension(iim,jjm,nsdm), public ::
     *           interp_indx_ml

!  unique points in domain
      integer (kind=int_kind), public :: qpml_points

!  *** switch to turn the use of these on and off
      logical (kind=log_kind), public :: doml=.false.
      
      public ::
     *  qp3mlsum, initialize_qpmlinfo 
     *, initialize_qpmlfiles,  write_qpml_diagnostics


    

      contains

      subroutine initialize_qpmlinfo

!  this subroutine assigns all the qp information from the input file atm_qpopts


      implicit none

      integer (kind=int_kind) :: n, m
      real (kind=dbl_kind), dimension(max_ig) :: indx_max_ig = 0.0
      real (kind=dbl_kind), dimension(im,jm) :: indx_imjm = 0.0
      real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: rinterp_indx = 0.0

      if (.not. doml) return
      
! initialize
	qp3ml_name(:) = " no name qp3"
	qp3ml_units(:) = "no units qp3"
	qp3ml_descr(:) = "no description qp3"
        qp3ml_type(:) = "no type qp3"
      
!  do assignments for variable name, units, description and type
!        qp2ml_name (iqp2ml_blent ) = "blent"
!        qp2ml_units(iqp2ml_blent ) = "kg/m^2/s"
!        qp2ml_descr(iqp2ml_blent ) = "PBL top entrainment rate"
!        qp2ml_type(iqp2ml_blent ) = "pbl"

!--------------------------------------------------------------------------
! start qp3s
!--------------------------------------------------------------------------
        qp3ml_name (iqp3ml_tnmrc      ) = "tnmrc"
        qp3ml_units(iqp3ml_tnmrc      ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_tnmrc      ) =
     *   ":cup: moisture tendency due to convective processes"
        qp3ml_type(iqp3ml_tnmrc ) = "cup"
        qp3ml_indx(iqp3ml_tnmrc ) = 1

        qp3ml_name (iqp3ml_dqelq      ) = "dqelq"
        qp3ml_units(iqp3ml_dqelq      ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_dqelq      ) =
     *   ":elq: large-scale tendency of total water"
        qp3ml_type(iqp3ml_dqelq ) = "elq"
        qp3ml_indx(iqp3ml_dqelq ) = 2

        qp3ml_name (iqp3ml_fill3q  ) = "fill3q"
        qp3ml_units(iqp3ml_fill3q  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_fill3q  ) = "water vapor hole fill rate"
        qp3ml_type(iqp3ml_fill3q ) = "dyn"
        qp3ml_indx(iqp3ml_fill3q ) = 3

        qp3ml_name (iqp3ml_fill3cw  ) = "fill3cw"
        qp3ml_units(iqp3ml_fill3cw  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_fill3cw  ) = "cloud water hole fill rate"
        qp3ml_type(iqp3ml_fill3cw ) = "dyn"
        qp3ml_indx(iqp3ml_fill3cw ) = 4

        qp3ml_name (iqp3ml_fill3ci  ) = "fill3ci"
        qp3ml_units(iqp3ml_fill3ci  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_fill3ci  ) = "cloud ice hole fill rate"
        qp3ml_type(iqp3ml_fill3ci ) = "dyn"
        qp3ml_indx(iqp3ml_fill3ci ) = 5

        qp3ml_name (iqp3ml_fill3rw  ) = "fill3rw"
        qp3ml_units(iqp3ml_fill3rw  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_fill3rw  ) = "rain hole fill rate"
        qp3ml_type(iqp3ml_fill3rw ) = "dyn"
        qp3ml_indx(iqp3ml_fill3rw ) = 6

        qp3ml_name (iqp3ml_fill3ri  ) = "fill3ri"
        qp3ml_units(iqp3ml_fill3ri  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_fill3ri  ) = "snow hole fill rate"
        qp3ml_type(iqp3ml_fill3ri ) = "dyn"
        qp3ml_indx(iqp3ml_fill3ri ) = 7

        qp3ml_name (iqp3ml_tnqdc  ) = "tnqdc"
        qp3ml_units(iqp3ml_tnqdc  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_tnqdc  ) = "Water vapor tendency, dcadj"
        qp3ml_type(iqp3ml_tnqdc ) = "dyn"
        qp3ml_indx(iqp3ml_tnqdc ) = 8

        qp3ml_name (iqp3ml_dsigdt  ) = "dsigdt"
        qp3ml_units(iqp3ml_dsigdt  ) = "Pa/s"
        qp3ml_descr(iqp3ml_dsigdt  ) = "Vert motion"
        qp3ml_type(iqp3ml_dsigdt ) = "dyn"
        qp3ml_indx(iqp3ml_dsigdt ) = 9

        qp3ml_name (iqp3ml_tnmrd  ) = "tnmrd"
        qp3ml_units(iqp3ml_tnmrd  ) = "kg/kg/s"
        qp3ml_descr(iqp3ml_tnmrd  ) =
     *        "Moisture tendency due to diabatic processes"
        qp3ml_type(iqp3ml_tnmrd ) = "dyn"
        qp3ml_indx(iqp3ml_tnmrd ) = 10

        qp3ml_name (iqp3ml_tnqdc_pw) = "tnqdc_pw"
        qp3ml_units(iqp3ml_tnqdc_pw) = "kg/kg/s"
        qp3ml_descr(iqp3ml_tnqdc_pw) = "Water vapor tendency, dcadj"
        qp3ml_type(iqp3ml_tnqdc_pw) = "dyn"
        qp3ml_indx(iqp3ml_tnqdc_pw) = 11

        qp3ml_name (iqp3ml_ta) = "ta"
        qp3ml_units(iqp3ml_ta) = "K"
        qp3ml_descr(iqp3ml_ta) = "Air temperature"
        qp3ml_type(iqp3ml_ta) = "dyn"
        qp3ml_indx(iqp3ml_ta) = 12

        qp3ml_name (iqp3ml_th) = "th"
        qp3ml_units(iqp3ml_th) = "K"
        qp3ml_descr(iqp3ml_th) = "Potential temperature"
        qp3ml_type(iqp3ml_th) = "dyn"
        qp3ml_indx(iqp3ml_th) = 13

        qp3ml_name (iqp3ml_hus) = "hus"
        qp3ml_units(iqp3ml_hus) = "kg/kg"
        qp3ml_descr(iqp3ml_hus) = "Specific humidity"
        qp3ml_type(iqp3ml_hus) = "dyn"
        qp3ml_indx(iqp3ml_hus) = 14

        qp3ml_name (iqp3ml_hur) = "hur"
        qp3ml_units(iqp3ml_hur) = "-"
        qp3ml_descr(iqp3ml_hur) = "Relative humidity"
        qp3ml_type(iqp3ml_hur) = "dyn"
        qp3ml_indx(iqp3ml_hur) = 15

        qp3ml_name (iqp3ml_hhl) = "hhl"
        qp3ml_units(iqp3ml_hhl) = "J/kg"
        qp3ml_descr(iqp3ml_hhl) = "Moist static energy"
        qp3ml_type(iqp3ml_hhl) = "dyn"
        qp3ml_indx(iqp3ml_hhl) = 16

        qp3ml_name (iqp3ml_masflx) = "masflx"
        qp3ml_units(iqp3ml_masflx) = "kg/kg/s"
        qp3ml_descr(iqp3ml_masflx) = "Convective mass flux"
        qp3ml_type(iqp3ml_masflx) = "dyn"
        qp3ml_indx(iqp3ml_masflx) = 17


	do n=1,nqp3ml
          qp3ml_name(n) = trim(qp3ml_name(n))
          qp3ml_units(n) = trim(qp3ml_units(n))
          qp3ml_descr(n) = trim(qp3ml_descr(n))
	  qp3ml_type(n) = trim(qp3ml_type(n))
          qp3ml_log(n) = .true.
	enddo

!  always save mask, pressure, geopotential and air temperature
      iqp3ml = nqp3ml

      if(my_task.eq.0) then
	 write(6,*)
         do n = 1,nqp3ml
           if(qp3ml_log(n)) then
	     write(6,10) qp3ml_name(n), qp3ml_descr(n),qp3ml_indx(n)
 10          format('Will save qp3ml: ',a12,a68, '  in qp3ml slot  ', i4)
	   endif
         enddo
         write(6,*)
      endif  ! if my_task.eq.0


!  broadcast information

#ifdef mpi
        call MPI_BCAST(
     *       qp3ml_indx,
     *       nqp3ml,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)
        
        call MPI_BCAST(
     *       qp3ml_log,
     *       nqp3ml,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3ml_name,
     *       nqp3ml*16,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3ml_units,
     *       nqp3ml*16,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3ml_descr,
     *       nqp3ml*80,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       iqp3ml,
     *       1,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3ml_type,
     *       nqp3ml*3,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

#endif


!  allocate the qp arrays

      allocate(qp3ml(iim, jjm, nlm, iqp3ml, nsdm))

      interp_indx_ml = 0

!   assign a sequential index to each grid point
      if (my_task.eq.0) then
         do m = 1,max_ig
            indx_max_ig(m) = m
         enddo
         call scatter_to_grid_global(indx_max_ig(:),indx_imjm(:,:))
      endif
      call scatter_fine(indx_imjm(:,:),rinterp_indx(:,:,:))
      interp_indx_ml(:,:,:) = nint(rinterp_indx(:,:,:))       
      
!----------------------------------------------------------------------

      end subroutine initialize_qpmlinfo



      subroutine initialize_qpmlfiles(simulation_tag, grid_mask)

!  this subroutine assigns all the qpml information from the input file atm_qpopts

      use netcdf_geodesic

      implicit none

      character*40, intent(in) :: simulation_tag
      real (kind=dbl_kind), intent(in), dimension(iim,jjm,nsdm) ::
     *     grid_mask

      real (kind=dbl_kind), dimension(2,2,iim,jjm,nsdm) :: grid_corner
      real (kind=selected_real_kind(13)), allocatable, dimension(:) ::
     *        qpcenterlat, qpcenterlon, qpcenterarea
      real (kind=selected_real_kind(13)), allocatable, dimension(:,:) :: 
     *        qpcornerlat, qpcornerlon

      character*8, time_label
      character*4, proc_label
      integer (kind=int_kind) :: j, ksdm, n_points
      integer (kind=int_kind) :: monthofyear

      integer (kind=int_kind), parameter :: corners=6
      real (kind=dbl_kind), dimension(corners,iim,jjm,nsdm) :: 
     *        cornerlat, cornerlon

      if (.not. doml) return
      
!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

      write(unit=proc_label,fmt='(i4)')
     *                1000+my_task
      proc_label(1:1) = 'p'

!-- process grid corner data
      grid_corner(1,:,:,:,:) = grid_corner_lon(:,:,:,:)
      grid_corner(2,:,:,:,:) = grid_corner_lat(:,:,:,:)
      call wrap_vertices(2,1, grid_corner)
      cornerlon = 0.0
      cornerlon(1:2,:,:,:) = grid_corner(1,1:2,:,:,:)
      cornerlon(3,1:iim-1,:,:) = grid_corner(1,1,2:iim,:,:)
      cornerlon(4,1:iim-1,1:jjm-1,:) = grid_corner(1,2,2:iim,2:jjm,:)
      cornerlon(5,1:iim-1,1:jjm-1,:) = grid_corner(1,1,2:iim,2:jjm,:)
      cornerlon(6,:,1:jjm-1,:) = grid_corner(1,2,:,2:jjm,:)
      cornerlat = 0.0
      cornerlat(1:2,:,:,:) = grid_corner(2,1:2,:,:,:)
      cornerlat(3,1:iim-1,:,:) = grid_corner(2,1,2:iim,:,:)
      cornerlat(4,1:iim-1,1:jjm-1,:) = grid_corner(2,2,2:iim,2:jjm,:)
      cornerlat(5,1:iim-1,1:jjm-1,:) = grid_corner(2,1,2:iim,2:jjm,:)
      cornerlat(6,:,1:jjm-1,:) = grid_corner(2,2,:,2:jjm,:)

!-- fill the zeroes in the np, sp pentagons 
      cornerlon(6,1,jjm-1,:) = cornerlon(5,1,jjm-1,:)
      cornerlat(6,1,jjm-1,:) = cornerlat(5,1,jjm-1,:)
      cornerlon(3,iim-1,1,:) = cornerlon(2,iim-1,1,:)
      cornerlat(3,iim-1,1,:) = cornerlat(2,iim-1,1,:)

      qpml_points = 0
      do ksdm = 1,nsdm
        do j = 1,jjm
          do i = 1,iim
            if(grid_mask(i,j,ksdm).gt.0.5_dbl_kind) 
     *        qpml_points = qpml_points + 1
          enddo
        enddo
      enddo

      allocate (qpcornerlat(corners,qpml_points))
      allocate (qpcenterlat(qpml_points))
      allocate (qpcornerlon(corners,qpml_points))
      allocate (qpcenterlon(qpml_points))
      allocate (qpcenterarea(qpml_points))

!  gather unique data
      n_points = 0
      do ksdm = 1,nsdm
        do j = 1,jjm
          do i = 1,iim
             if(grid_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                n_points = n_points + 1
                qpcenterlat(n_points) = grid_center_lat(i,j,ksdm)
                qpcenterlon(n_points) = grid_center_lon(i,j,ksdm)
                qpcenterarea(n_points) = area(i,j,ksdm)
                qpcornerlat(:,n_points) = cornerlat(:,i,j,ksdm)
                qpcornerlon(:,n_points) = cornerlon(:,i,j,ksdm)
             endif
          enddo
        enddo
      enddo

      call ginit_atm_nc_2 (simulation_tag, qpml_points, nlm, 
     *                 "qp_output/gatm_"//trim(time_label)
!     *                 //trim(proc_label)//".g3ml.nc",
     *                 //trim(proc_label)//".g4.nc",
     *                 qp3ml_descr, qp3ml_name, qp3ml_units, 
     *                 qp3ml_type, qp3ml_log,
     *                 qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                 corners, max_ig, qpcenterarea)

      deallocate (qpcornerlat)
      deallocate (qpcenterlat)
      deallocate (qpcornerlon)
      deallocate (qpcenterlon)
      deallocate (qpcenterarea)

      nsecondinit = nsecond_dyn - dt
      iqp3ml_sample = 0
      qp3ml(:,:,:,:,:) = c0

      end subroutine initialize_qpmlfiles


      subroutine qp3mlsum(xin, lenx, x_const, index, ksdm, len, ioffset,
     *                    step_sample)

!  this subroutine vertically interpolates a field to constant pressure
!      or theta surfaces and accumulates it in a section in the qp3 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a qp3
!  lenx - horizontal dimension of xin
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by iqp3_xxx (where xxx is the 
!          field name) in the calling program
!  ksdm - the subdomain index
!  len - the length of the data domain to store (can't be large than lenx)
!  ioffset - (optional) the offset to the starting location in the 
!            qp3 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section to fill
     *,    lenx    ! horizontal dimension of xin
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset, step_sample

      real (kind=dbl_kind), dimension(lenx,nlm), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend

      real (kind=dbl_kind), dimension(iimjjm,nlm) :: x1

#ifdef scm
      return
#endif

      if (.not. doml) return

      if (qp3ml_log(index)) then

!   test for compatible length arguments
         if (lenx.lt.len) then
            print *,'qp3mlsum called with incompatible len, len arguments'
            print *,' for qp3ml index ',index
            return
         endif
         
         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         x1 = 0.0
         x1(offset+1:offset+len,:) = xin(1:len,:)

         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1

         if (qp3ml_log(index)) then
! update the sample size
           if (.not.present(step_sample)) then
             if (ksdm.eq.1.and.offset.eq.0)
     *             iqp3ml_sample(index) = iqp3ml_sample(index) + 1
           endif
! save the qp3 array
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             qp3ml(ibeg:iend,j,:,qp3ml_indx(index),ksdm) =
     *            qp3ml(ibeg:iend,j,:,qp3ml_indx(index),ksdm)
     *            + x_const * x1(item+ibeg:item+iend,:)
           enddo
         endif

      endif !  if(qp3_log(index).ne.0)

      end subroutine qp3mlsum




      subroutine write_qpml_diagnostics(grid_mask)

      use netcdf_general_tools

      implicit none

      real (kind=dbl_kind), dimension(iim,jjm,nsdm), intent(in) :: grid_mask

      integer (kind=int_kind) :: n
      real (kind=real_kind) :: qp3ml_samples_inv
      logical (kind=log_kind), dimension(iim,jjm,nsdm) :: l_grid_mask

      if (.not. doml) return

      l_grid_mask = .false.
      where(grid_mask(:,:,:).gt.0.5_dbl_kind) l_grid_mask(:,:,:) = .true.

! normalize
!      write(*,*) 'write_qpml: iqp3ml_sample = ',iqp3ml_sample
      do n = 1,nqp3ml
         if(qp3ml_log(n)) then
            if(iqp3ml_sample(n).eq.0) then
               qp3ml_samples_inv = 0.0
            else
               qp3ml_samples_inv = 1. / float(iqp3ml_sample(n))
            endif
            qp3ml(:,:,:,qp3ml_indx(n),:) = qp3ml(:,:,:,qp3ml_indx(n),:) 
     +                                    * qp3ml_samples_inv
         endif
      enddo
!      write(*,*) 'write_qpml: ',maxval(qp3ml(:,:,nlm,10,:))*86400.

!  write the data to a netcdf file
      call gwrt_atm_nc_2(nsecond_dyn, nsecondinit, nyear0, qpml_points, 
     *                 l_grid_mask,
     *                 landfrac, seafrac, icefrac, interp_indx_ml,
     *                 qp3ml(:,:,1:nlm,1:iqp3ml,:), qp3ml_name, qp3ml_log, 
     *                 qp3ml_indx )

!   zero out for next period of summation

      nsecondinit = nsecond_dyn
      qp3ml = c0
      iqp3ml_sample = 0

      end subroutine write_qpml_diagnostics

      end module qp_ml
