module qp_module_sfc

!  This file initializes, stores, computes and writes the diagnostic output
!     that is global and time averaged. The short-hand term for these
!     diagnostics is 'qp'. QP2 are2 dimensional (horizontal) only.

!  This module is for storing fields that exist on all points of the 
!     surface grid

  use kinds
  use model_parameters
  use domain_decomposition_sfc
  use sigma_coordinate
  use netcdf_geodesic
  use time_manager_module
  use sib_parameters

  implicit none
  save
  private

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!	PROCEDURE TO ADD A QP VARIABLE TO THE MODEL
!
!       Adding a QP2 (or hf2) (2-dimensional field)
!   	In this file:
!	1. Increase Nqp2_sfc by 1
!	2. Add new entries in the tables for
!          -a variable named iqp2_sfc_xxx, where xxx is the name in
!               qp2_name_sfc.
!               Add this to the list of iqp3_sfc variables and assign it a value
!               1 larger than the previous one.
!       3. Go to subroutine initialize_qpinfo_sfc further down in this file.
!          Add an assignment statement for each of these variables for
!          index iqp2_sfc_xxx (these are down in a block):
!  	   -qp2_name_sfc (the name of the field conforming, if possible, to 
!          	amip conventions
!          -qp2_units_sfc ( the units of the field)
!          -qp2_descr_sfc ( A longer description of the field)
!          -qp2_mask_sfc (which surfaces the field exists on)
!	In the file where the variable resides
!	4. Be sure that a "use qp_module_sfc" statement is in the subroutine
!       5. Add a qp2sum_sfc statement for the field you wish to store.
!	The description of the qp2sum_sfc argument list is in the file with 
!       subroutine qp2sum_sfc
!
!       Hints:
!       -If the field you want to save is a linear combination of model
!        variables you can use multiple qp2sum_sfc calls with the appropriate 
!        weights.
!       -If the field you want to save is in a subroutine that is not called
!        every timestep, the weight should include a factor to compensate for
!        that.
!        Example - radiation is called every physics timestep, once every
!        n_physics timesteps. Include a factor of n_physics in the weight
!       -hardwired constants in the argument list must be of the right kind
!        Example - a weight of 1.0 should be written 1.0_dbl_kind
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!  tables of qp information

!   number of installed available qps, change these as more qps are added
  integer (kind=int_kind), parameter, public :: &
    nqp2_sfc = 84 + nsoil+1, &
    nhf2_sfc = 3

!   names are the variable names and conform to amip standards where possible
!   these are assigned in subroutine initialize_qpinfo further down
  character (len=16), dimension(nqp2_sfc), public :: qp2_name_sfc
  character (len=16), dimension(nhf2_sfc), public :: hf2_name_sfc

!  units are the units the fields are stored in and are mks where possible
!   these are assigned in subroutine initialize_qpinfo further down
  character (len=16), dimension(nqp2_sfc), public :: qp2_units_sfc
  character (len=16), dimension(nhf2_sfc), public :: hf2_units_sfc

!  descr is a description of the field
!   these are assigned in subroutine initialize_qpinfo further down
  character (len=80), dimension(nqp2_sfc), public :: qp2_descr_sfc
  character (len=80), dimension(nhf2_sfc), public :: hf2_descr_sfc

!  type denotes which type of variable this is (for ezplot)
  character (len=3), dimension(nqp2_sfc), public :: qp2_type_sfc
  character (len=3), dimension(nhf2_sfc), public :: hf2_type_sfc

!  mask denotes which surface component the field exist on
!   sib, ice, sea, ( or sfc if common to all components)
!   these are assigned in subroutine initialize_qpinfo further down
  character (len=3), dimension(nqp2_sfc), public :: qp2_mask_sfc
  character (len=3), dimension(nhf2_sfc), public :: hf2_mask_sfc

  integer (kind=int_kind), dimension(nqp2_sfc), public :: qp2_indx_sfc
  integer (kind=int_kind), dimension(nhf2_sfc), public :: hf2_indx_sfc
  logical (kind=log_kind), dimension(nqp2_sfc), public :: qp2_log_sfc = .false.
  logical (kind=log_kind), dimension(nhf2_sfc), public :: hf2_log_sfc = .false.

! these will point to  locations in the qp information tables
! their values are assigned sequentially, and the last value is used
!   as the dimension of the above tables
  integer (kind=int_kind) :: i
  integer (kind=int_kind), parameter, public :: &
    iqp2_sfc_blent = 1, &
    iqp2_sfc_clblp	= 2, &
    iqp2_sfc_clbl	= 3, &
    iqp2_sfc_hfss	= 4, &
    iqp2_sfc_tradgrdeff	= 5, &
    iqp2_sfc_hfls	= 6, &
    iqp2_sfc_tke    	= 7, &
    iqp2_sfc_ventmf  	= 8, &
    iqp2_sfc_www1    	= 9, &
    iqp2_sfc_www2    	= 10, &
    iqp2_sfc_www3    	= 11, &
    iqp2_sfc_mrro   	= 12, &
    iqp2_sfc_mrfso   	= 13, &
    iqp2_sfc_mrsos   	= 14, &
    iqp2_sfc_mrso   	= 15, &
    iqp2_sfc_mrros   	= 16, &
    iqp2_sfc_snw   	= 17, &
    iqp2_sfc_snc   	= 18, &
    iqp2_sfc_snm   	= 19, &
    iqp2_sfc_sic   	= 20, &
    iqp2_sfc_ps		= 21, &
    iqp2_sfc_psb         = 22, &
    iqp2_sfc_dlwbot      = 23, &
    iqp2_sfc_dcupr       = 24, &
    iqp2_sfc_dlspr       = 25, &
    iqp2_sfc_dlsps       = 26, &
    iqp2_sfc_spdm        = 27, &
    iqp2_sfc_shear1      = 28, &
    iqp2_sfc_tpblp2      = 29, &
    iqp2_sfc_tpblp1      = 30, &
    iqp2_sfc_tpbl        = 31, &
    iqp2_sfc_shpblp2     = 32, &
    iqp2_sfc_shpblp1     = 33, &
    iqp2_sfc_shpbl       = 34, &
    iqp2_sfc_zs          = 35, &
    iqp2_sfc_assimn      = 36, &
    iqp2_sfc_respg       = 37, &
    iqp2_sfc_co2flx      = 38, &
    iqp2_sfc_shacu       = 39, &
    iqp2_sfc_eci         = 40, &
    iqp2_sfc_ect         = 41, &
    iqp2_sfc_egi         = 42, &
    iqp2_sfc_egs         = 43, &
    iqp2_sfc_exo         = 44, &
    iqp2_sfc_q3o         = 45, &
    iqp2_sfc_q3l         = 46, &
    iqp2_sfc_tcas        = 47, &
    iqp2_sfc_qcas        = 48, &
    iqp2_sfc_rhcas       = 49, &
    iqp2_sfc_capac1      = 50, &
    iqp2_sfc_capac2      = 51, &
    iqp2_sfc_tcan        = 52, &
    iqp2_sfc_tg          = 53, &
    iqp2_sfc_rstfac1     = 54, &
    iqp2_sfc_rstfac2     = 55, &
    iqp2_sfc_rstfac3     = 56, &
    iqp2_sfc_rstfac4     = 57, &
    iqp2_sfc_eta         = 58, &
    iqp2_sfc_negeinc     = 59, &
    iqp2_sfc_zlt         = 60, &
    iqp2_sfc_green       = 61, &
    iqp2_sfc_vcover      = 62, &
    iqp2_sfc_ra          = 63, &
    iqp2_sfc_rb          = 64, &
    iqp2_sfc_rd          = 65, &
    iqp2_sfc_rst         = 66, &
    iqp2_sfc_chf         = 67, &
    iqp2_sfc_shf         = 68, &
    iqp2_sfc_radc31      = 69, &
    iqp2_sfc_radc32      = 70, &
    iqp2_sfc_lciinc      = 71, &
    iqp2_sfc_safe        = 72, &
    iqp2_sfc_ustar       = 73, &
    iqp2_sfc_mol         = 74, &
    iqp2_sfc_buoyf       = 75, &
    iqp2_sfc_spd10       = 76, &
    iqp2_sfc_z0          = 77, &
    iqp2_sfc_t2          = 78, &
    iqp2_sfc_dtlci1      = 79, &
    iqp2_sfc_dtlci2      = 80, &
    iqp2_sfc_dqlci1      = 81, &
    iqp2_sfc_dqlci2      = 82, &
    iqp2_sfc_tbp         = 83, &
    iqp2_sfc_wbp         = 84

  integer (kind=int_kind), parameter, dimension(nsoil+1), public :: &
    iqp2_sfc_soilscale = (/ (84+i, i=1,nsoil+1) /)

  integer (kind=int_kind), parameter, public :: &
    ihf2_sfc_mrso	= 1, &
    ihf2_sfc_mrro	= 2, &
    ihf2_sfc_snw 	= 3


!   number of saved fields
  integer (kind=int_kind), public :: iqp2_sfc = 0, ihf2_sfc=0

!   number of samples
  integer (kind=int_kind), public :: iqp_sample_sfc
  integer (kind=int_kind), public :: ihf_sample_sfc

!   time at start of summation
  integer (kind=int_kind), public :: nsecondinit_sfc

!   qp data slots
  real (kind=real_kind), allocatable, dimension(:,:,:,:), public :: &
    qp2_sfc, &
    hf2_sfc

!  indices for latlon interpolation
   integer (kind=int_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc), public :: &
     interp_indx_sfc

!  unique points in domain
  integer (kind=int_kind), public :: qp_points_sfc

  public :: &
    qp2sum_sfc, initialize_qpinfo_sfc, qp_sfc_increment, &
    initialize_qpfiles_sfc,  write_qp_diagnostics_sfc, &
    hf2sum_sfc, write_hf_diagnostics_sfc


    

  contains

  subroutine initialize_qpinfo_sfc

!  this subroutine assigns all the qp information from the input file atm_qpopts

  implicit none

  integer (kind=int_kind) :: ldummy, n, ioerr
  logical (kind=log_kind) :: doqptem
  character (len=16) :: nametem
  character (len=2) :: tnumber

! initialize
    qp2_name_sfc(:) = " no name qp2"
    hf2_name_sfc(:) = " no name hf2"
	qp2_units_sfc(:) = "no units qp2"
	hf2_units_sfc(:) = "no units hf2"
	qp2_descr_sfc(:) = "no description qp2"
	hf2_descr_sfc(:) = "no description hf2"
	qp2_mask_sfc(:) = "sfc"  ! assumed to exist for all surfaces
	hf2_mask_sfc(:) = "sfc"  ! assumed to exist for all surfaces
      
!  do assignments for variable name, units and description 
    qp2_name_sfc (iqp2_sfc_blent ) = "blent"
    qp2_units_sfc(iqp2_sfc_blent ) = "kg/m^2/s"
    qp2_descr_sfc(iqp2_sfc_blent ) = "PBL top entrainment rate"
    qp2_mask_sfc (iqp2_sfc_blent)  = "sfc"
    qp2_type_sfc (iqp2_sfc_blent)  = "sfc"

    qp2_name_sfc (iqp2_sfc_clblp ) = "clblp"
    qp2_units_sfc(iqp2_sfc_clblp ) = "Pa"
    qp2_descr_sfc(iqp2_sfc_clblp ) = "Boundary layer cloud depth"
    qp2_mask_sfc (iqp2_sfc_clblp)  = "sfc"
    qp2_type_sfc (iqp2_sfc_clblp)  = "sfc"

    qp2_name_sfc (iqp2_sfc_clbl  ) = "clbl"
    qp2_units_sfc(iqp2_sfc_clbl  ) = "-"
    qp2_descr_sfc(iqp2_sfc_clbl  ) = "EAUcf: Incidence of PBL clouds"
    qp2_mask_sfc (iqp2_sfc_clbl)   = "sfc"
    qp2_type_sfc (iqp2_sfc_clbl)   = "sfc"

    qp2_name_sfc (iqp2_sfc_hfss  ) = "hfss"
    qp2_units_sfc(iqp2_sfc_hfss  ) = "W/m^2"
    qp2_descr_sfc(iqp2_sfc_hfss  ) = "SURFACE SENSIBLE HEAT FLUX"
    qp2_mask_sfc (iqp2_sfc_hfss)   = "sfc"
    qp2_type_sfc (iqp2_sfc_hfss)   = "sfc"

    qp2_name_sfc (iqp2_sfc_tradgrdeff) = "tradgrdeff "
    qp2_units_sfc(iqp2_sfc_tradgrdeff) = "K"
    qp2_descr_sfc(iqp2_sfc_tradgrdeff) = "Effective radiative ground temperature"
	qp2_mask_sfc (iqp2_sfc_tradgrdeff)   = "sfc"
	qp2_type_sfc (iqp2_sfc_tradgrdeff)   = "sfc"

    qp2_name_sfc (iqp2_sfc_hfls  ) = "hfls"
    qp2_units_sfc(iqp2_sfc_hfls  ) = "W/m^2"
    qp2_descr_sfc(iqp2_sfc_hfls  ) = "Surface latent heat flux"
    qp2_mask_sfc (iqp2_sfc_hfls)   = "sfc"
    qp2_type_sfc (iqp2_sfc_hfls)   = "sfc"

    qp2_name_sfc (iqp2_sfc_tke  ) = "tke"
    qp2_units_sfc(iqp2_sfc_tke  ) = "m2/s2"
    qp2_descr_sfc(iqp2_sfc_tke  ) = "pbl turbulent kinetic energy"
    qp2_mask_sfc (iqp2_sfc_tke)   = "sfc"
    qp2_type_sfc (iqp2_sfc_tke)   = "sfc"

    qp2_name_sfc (iqp2_sfc_ventmf  ) = "ventmf"
    qp2_units_sfc(iqp2_sfc_ventmf  ) = "kg/m2/s"
    qp2_descr_sfc(iqp2_sfc_ventmf  ) = "ventilation mass flux"
    qp2_mask_sfc (iqp2_sfc_ventmf)   = "sfc"
    qp2_type_sfc (iqp2_sfc_ventmf)   = "sfc"

    qp2_name_sfc (iqp2_sfc_www1  ) = "www1"
    qp2_units_sfc(iqp2_sfc_www1  ) = "-"
    qp2_descr_sfc(iqp2_sfc_www1  ) = "soil moisture layer 1"
    qp2_mask_sfc (iqp2_sfc_www1)   = "sib"
    qp2_type_sfc (iqp2_sfc_www1)   = "sib"

	qp2_name_sfc (iqp2_sfc_www2  ) = "www2"
	qp2_units_sfc(iqp2_sfc_www2  ) = "-"
	qp2_descr_sfc(iqp2_sfc_www2  ) = "soil moisture layer 2"
	qp2_mask_sfc (iqp2_sfc_www2)   = "sib"
	qp2_type_sfc (iqp2_sfc_www2)   = "sib"

	qp2_name_sfc (iqp2_sfc_www3  ) = "www3"
	qp2_units_sfc(iqp2_sfc_www3  ) = "-"
	qp2_descr_sfc(iqp2_sfc_www3  ) = "soil moisture layer 3"
	qp2_mask_sfc (iqp2_sfc_www3)   = "sib"
	qp2_type_sfc (iqp2_sfc_www3)   = "sib"

	qp2_name_sfc (iqp2_sfc_mrro ) = "mrro"
	qp2_units_sfc(iqp2_sfc_mrro ) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_mrro ) = "Total runoff (including drainage)"
	qp2_mask_sfc (iqp2_sfc_mrro)   = "sib"
	qp2_type_sfc (iqp2_sfc_mrro)   = "sib"

	qp2_name_sfc (iqp2_sfc_mrfso ) = "mrfso"
	qp2_units_sfc(iqp2_sfc_mrfso ) = "kg/m2"
	qp2_descr_sfc(iqp2_sfc_mrfso ) = "Total soil frozen water content"
	qp2_mask_sfc (iqp2_sfc_mrfso)   = "sib"
	qp2_type_sfc (iqp2_sfc_mrfso)   = "sib"

	qp2_name_sfc (iqp2_sfc_mrsos ) = "mrsos"
	qp2_units_sfc(iqp2_sfc_mrsos ) = "kg/m2"
	qp2_descr_sfc(iqp2_sfc_mrsos ) = "Surface soil water content (upper 0.1m)"
	qp2_mask_sfc (iqp2_sfc_mrsos)   = "sib"
	qp2_type_sfc (iqp2_sfc_mrsos)   = "sib"

	qp2_name_sfc (iqp2_sfc_mrso ) = "mrso"
	qp2_units_sfc(iqp2_sfc_mrso ) = "kg/m2"
	qp2_descr_sfc(iqp2_sfc_mrso ) = "Total soil water content"
	qp2_mask_sfc (iqp2_sfc_mrso)   = "sib"
	qp2_type_sfc (iqp2_sfc_mrso)   = "sib"

	qp2_name_sfc (iqp2_sfc_mrros ) = "mrros"
	qp2_units_sfc(iqp2_sfc_mrros ) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_mrros ) = "Surface runoff"
	qp2_mask_sfc (iqp2_sfc_mrros)   = "sib"
	qp2_type_sfc (iqp2_sfc_mrros)   = "sib"

	qp2_name_sfc (iqp2_sfc_snw ) = "snw"
	qp2_units_sfc(iqp2_sfc_snw ) = "kg/m2"
	qp2_descr_sfc(iqp2_sfc_snw ) = "Snow depth (water equivalent)"
	qp2_mask_sfc (iqp2_sfc_snw )   = "sib"
	qp2_type_sfc (iqp2_sfc_snw )   = "sib"

	qp2_name_sfc (iqp2_sfc_snc ) = "snc"
	qp2_units_sfc(iqp2_sfc_snc ) = "%"
	qp2_descr_sfc(iqp2_sfc_snc ) = "Snow cover"
	qp2_mask_sfc (iqp2_sfc_snc )   = "sib"
	qp2_type_sfc (iqp2_sfc_snc )   = "sib"

	qp2_name_sfc (iqp2_sfc_snm ) = "snm"
	qp2_units_sfc(iqp2_sfc_snm ) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_snm ) = "Snow melt"
	qp2_mask_sfc (iqp2_sfc_snm )   = "sib"
	qp2_type_sfc (iqp2_sfc_snm )   = "sib"

	qp2_name_sfc (iqp2_sfc_sic ) = "sic"
	qp2_units_sfc(iqp2_sfc_sic ) = "%"
	qp2_descr_sfc(iqp2_sfc_sic ) = "Sea-ice concentration"
	qp2_mask_sfc (iqp2_sfc_sic )   = "ocean"
	qp2_type_sfc (iqp2_sfc_sic )   = "sfc"

	qp2_name_sfc (iqp2_sfc_ps  ) = "ps"
	qp2_units_sfc(iqp2_sfc_ps  ) = "mb"
	qp2_descr_sfc(iqp2_sfc_ps  ) = "surface pressure"
	qp2_mask_sfc (iqp2_sfc_ps  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_ps  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_psb  ) = "psb" 
	qp2_units_sfc(iqp2_sfc_psb  ) = "mb"
	qp2_descr_sfc(iqp2_sfc_psb  ) = "PBL depth"
	qp2_mask_sfc (iqp2_sfc_psb  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_psb  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_dlwbot  ) = "dlwbot" 
	qp2_units_sfc(iqp2_sfc_dlwbot  ) = "W/m2"
	qp2_descr_sfc(iqp2_sfc_dlwbot  ) = "down-welling longwave radiation"
	qp2_mask_sfc (iqp2_sfc_dlwbot  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_dlwbot  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_dcupr  ) = "dcupr" 
	qp2_units_sfc(iqp2_sfc_dcupr  ) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_dcupr  ) = "convective precipitation"
	qp2_mask_sfc (iqp2_sfc_dcupr  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_dcupr  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_dlspr  ) = "dlspr" 
	qp2_units_sfc(iqp2_sfc_dlspr  ) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_dlspr  ) = "large-scale stratiform rain"
	qp2_mask_sfc (iqp2_sfc_dlspr  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_dlspr  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_dlsps  ) = "dlsps" 
	qp2_units_sfc(iqp2_sfc_dlsps  ) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_dlsps  ) = "large-scale stratiform snow"
	qp2_mask_sfc (iqp2_sfc_dlsps  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_dlsps  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_spdm  ) = "spdm" 
	qp2_units_sfc(iqp2_sfc_spdm  ) = "m/s"
	qp2_descr_sfc(iqp2_sfc_spdm  ) = "surface wind speed"
	qp2_mask_sfc (iqp2_sfc_spdm  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_spdm  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_shear1  ) = "shear1" 
	qp2_units_sfc(iqp2_sfc_shear1  ) = "m2/s2"
	qp2_descr_sfc(iqp2_sfc_shear1  ) = "delta u across pbl squared"
	qp2_mask_sfc (iqp2_sfc_shear1  ) = "sfc"
	qp2_type_sfc (iqp2_sfc_shear1  ) = "sfc"

	qp2_name_sfc (iqp2_sfc_tpblp2) = "tpblp2"
	qp2_units_sfc(iqp2_sfc_tpblp2) = "K"
	qp2_descr_sfc(iqp2_sfc_tpblp2) = "potential temp 2 layers above pbl"
	qp2_mask_sfc (iqp2_sfc_tpblp2) = "sfc"
	qp2_type_sfc (iqp2_sfc_tpblp2) = "sfc"

	qp2_name_sfc (iqp2_sfc_tpblp1) = "tpblp1" 
	qp2_units_sfc(iqp2_sfc_tpblp1) = "K"
	qp2_descr_sfc(iqp2_sfc_tpblp1) = "potential temp 1 layer above pbl"
	qp2_mask_sfc (iqp2_sfc_tpblp1) = "sfc"
	qp2_type_sfc (iqp2_sfc_tpblp1) = "sfc"

	qp2_name_sfc (iqp2_sfc_tpbl) = "tpbl" 
	qp2_units_sfc(iqp2_sfc_tpbl) = "K"
	qp2_descr_sfc(iqp2_sfc_tpbl) = "potential temp of pbl"
	qp2_mask_sfc (iqp2_sfc_tpbl) = "sfc"
	qp2_type_sfc (iqp2_sfc_tpbl) = "sfc"

	qp2_name_sfc (iqp2_sfc_shpblp2) = "shpblp2" 
	qp2_units_sfc(iqp2_sfc_shpblp2) = "kg of water /kg of air"
	qp2_descr_sfc(iqp2_sfc_shpblp2) = "specific humidity 2 layers above pbl"
	qp2_mask_sfc (iqp2_sfc_shpblp2) = "sfc"
	qp2_type_sfc (iqp2_sfc_shpblp2) = "sfc"

	qp2_name_sfc (iqp2_sfc_shpblp1) = "shpblp1" 
	qp2_units_sfc(iqp2_sfc_shpblp1) = "kg of water /kg of air"
	qp2_descr_sfc(iqp2_sfc_shpblp1) = "specific humidity 1 layer above pbl"
	qp2_mask_sfc (iqp2_sfc_shpblp1) = "sfc"
	qp2_type_sfc (iqp2_sfc_shpblp1) = "sfc"

	qp2_name_sfc (iqp2_sfc_shpbl) = "shpbl" 
	qp2_units_sfc(iqp2_sfc_shpbl) = "kg of water /kg of air"
	qp2_descr_sfc(iqp2_sfc_shpbl) = "specific humidity of pbl"
	qp2_mask_sfc (iqp2_sfc_shpbl) = "sfc"
	qp2_type_sfc (iqp2_sfc_shpbl) = "sfc"

	qp2_name_sfc (iqp2_sfc_zs) = "zs"
	qp2_units_sfc(iqp2_sfc_zs) = "m^2/s^2"
	qp2_descr_sfc(iqp2_sfc_zs) = "surface geopotential"
	qp2_mask_sfc (iqp2_sfc_zs) = "sfc"
	qp2_type_sfc (iqp2_sfc_zs) = "sfc"

	qp2_name_sfc (iqp2_sfc_assimn) = "assimn"
	qp2_units_sfc(iqp2_sfc_assimn) = "moles/m2/s"
	qp2_descr_sfc(iqp2_sfc_assimn) = "Net carbon assimilation"
	qp2_mask_sfc (iqp2_sfc_assimn) = "sib"
	qp2_type_sfc (iqp2_sfc_assimn) = "sib"

	qp2_name_sfc (iqp2_sfc_respg) = "respg"
	qp2_units_sfc(iqp2_sfc_respg) = "moles/m2/s"
	qp2_descr_sfc(iqp2_sfc_respg) = "Carbon respiration"
	qp2_mask_sfc (iqp2_sfc_respg) = "sib"
	qp2_type_sfc (iqp2_sfc_respg) = "sib"

	qp2_name_sfc (iqp2_sfc_co2flx) = "co2flx"
	qp2_units_sfc(iqp2_sfc_co2flx) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_co2flx) = "sib surface carbon flux"
	qp2_mask_sfc (iqp2_sfc_co2flx) = "sib"
	qp2_type_sfc (iqp2_sfc_co2flx) = "sib"

	qp2_name_sfc (iqp2_sfc_shacu) = "shacu"
	qp2_units_sfc(iqp2_sfc_shacu) = "-"
	qp2_descr_sfc(iqp2_sfc_shacu) = "Shallow Cu criterion incidence"
	qp2_mask_sfc (iqp2_sfc_shacu) = "sfc"
	qp2_type_sfc (iqp2_sfc_shacu) = "sfc"

	qp2_name_sfc (iqp2_sfc_eci) = "eci"
	qp2_units_sfc(iqp2_sfc_eci) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_eci) = "Evaporation of canopy interception"
	qp2_mask_sfc (iqp2_sfc_eci) = "sib"
	qp2_type_sfc (iqp2_sfc_eci) = "sib"

	qp2_name_sfc (iqp2_sfc_ect) = "ect"
	qp2_units_sfc(iqp2_sfc_ect) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_ect) = "Canopy transpiration"
	qp2_mask_sfc (iqp2_sfc_ect) = "sib"
	qp2_type_sfc (iqp2_sfc_ect) = "sib"

	qp2_name_sfc (iqp2_sfc_egi) = "egi"
	qp2_units_sfc(iqp2_sfc_egi) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_egi) = "Evaporation of surface interception"
	qp2_mask_sfc (iqp2_sfc_egi) = "sib"
	qp2_type_sfc (iqp2_sfc_egi) = "sib"

	qp2_name_sfc (iqp2_sfc_egs) = "egs"
	qp2_units_sfc(iqp2_sfc_egs) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_egs) = "Evaporation of surface soil moisture"
	qp2_mask_sfc (iqp2_sfc_egs) = "sib"
	qp2_type_sfc (iqp2_sfc_egs) = "sib"

	qp2_name_sfc (iqp2_sfc_exo) = "exo"
	qp2_units_sfc(iqp2_sfc_exo) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_exo) = "Runoff due to excess soil moisture"
	qp2_mask_sfc (iqp2_sfc_exo) = "sib"
	qp2_type_sfc (iqp2_sfc_exo) = "sib"

	qp2_name_sfc (iqp2_sfc_q3o) = "q3o"
	qp2_units_sfc(iqp2_sfc_q3o) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_q3o) = "Runoff due to gravitational drainage"
	qp2_mask_sfc (iqp2_sfc_q3o) = "sib"
	qp2_type_sfc (iqp2_sfc_q3o) = "sib"

	qp2_name_sfc (iqp2_sfc_q3l) = "q3l"
	qp2_units_sfc(iqp2_sfc_q3l) = "kg/m2/s"
	qp2_descr_sfc(iqp2_sfc_q3l) = "Liston runoff term"
	qp2_mask_sfc (iqp2_sfc_q3l) = "sib"
	qp2_type_sfc (iqp2_sfc_q3l) = "sib"

	qp2_name_sfc (iqp2_sfc_tcas) = "tcas"
	qp2_units_sfc(iqp2_sfc_tcas) = "K"
	qp2_descr_sfc(iqp2_sfc_tcas) = "Canopy air space temperature"
	qp2_mask_sfc (iqp2_sfc_tcas) = "sib"
	qp2_type_sfc (iqp2_sfc_tcas) = "sib"

	qp2_name_sfc (iqp2_sfc_qcas) = "qcas"
	qp2_units_sfc(iqp2_sfc_qcas) = "kg/kg"
	qp2_descr_sfc(iqp2_sfc_qcas) = "Canopy air space water vapor"
	qp2_mask_sfc (iqp2_sfc_qcas) = "sib"
	qp2_type_sfc (iqp2_sfc_qcas) = "sib"

	qp2_name_sfc (iqp2_sfc_rhcas) = "rhcas"
	qp2_units_sfc(iqp2_sfc_rhcas) = "-"
	qp2_descr_sfc(iqp2_sfc_rhcas) = "Canopy air space relative humidity"
	qp2_mask_sfc (iqp2_sfc_rhcas) = "sib"
	qp2_type_sfc (iqp2_sfc_rhcas) = "sib"

	qp2_name_sfc (iqp2_sfc_capac1) = "capac1"
	qp2_units_sfc(iqp2_sfc_capac1) = "kg/m2"
	qp2_descr_sfc(iqp2_sfc_capac1) = "Canopy intercepted liquid water"
	qp2_mask_sfc (iqp2_sfc_capac1) = "sib"
	qp2_type_sfc (iqp2_sfc_capac1) = "sib"

	qp2_name_sfc (iqp2_sfc_capac2) = "capac2"
	qp2_units_sfc(iqp2_sfc_capac2) = "kg/m2"
	qp2_descr_sfc(iqp2_sfc_capac2) = "Surface intercepted liquid water"
	qp2_mask_sfc (iqp2_sfc_capac2) = "sib"
	qp2_type_sfc (iqp2_sfc_capac2) = "sib"

	qp2_name_sfc (iqp2_sfc_tcan) = "tcan"
	qp2_units_sfc(iqp2_sfc_tcan) = "K"
	qp2_descr_sfc(iqp2_sfc_tcan) = "Canopy temperature"
	qp2_mask_sfc (iqp2_sfc_tcan) = "sib"
	qp2_type_sfc (iqp2_sfc_tcan) = "sib"

	qp2_name_sfc (iqp2_sfc_tg) = "tg"
	qp2_units_sfc(iqp2_sfc_tg) = "K"
	qp2_descr_sfc(iqp2_sfc_tg) = "Surface soil temperature"
	qp2_mask_sfc (iqp2_sfc_tg) = "sib"
	qp2_type_sfc (iqp2_sfc_tg) = "sib"

	qp2_name_sfc (iqp2_sfc_rstfac1) = "rstfac1"
	qp2_units_sfc(iqp2_sfc_rstfac1) = "-"
	qp2_descr_sfc(iqp2_sfc_rstfac1) = "Humidity stress factor"
	qp2_mask_sfc (iqp2_sfc_rstfac1) = "sib"
	qp2_type_sfc (iqp2_sfc_rstfac1) = "sib"

	qp2_name_sfc (iqp2_sfc_rstfac2) = "rstfac2"
	qp2_units_sfc(iqp2_sfc_rstfac2) = "-"
	qp2_descr_sfc(iqp2_sfc_rstfac2) = "Soil moisture stress factor"
	qp2_mask_sfc (iqp2_sfc_rstfac2) = "sib"
	qp2_type_sfc (iqp2_sfc_rstfac2) = "sib"

	qp2_name_sfc (iqp2_sfc_rstfac3) = "rstfac3"
	qp2_units_sfc(iqp2_sfc_rstfac3) = "-"
	qp2_descr_sfc(iqp2_sfc_rstfac3) = "Temperature stress factor"
	qp2_mask_sfc (iqp2_sfc_rstfac3) = "sib"
	qp2_type_sfc (iqp2_sfc_rstfac3) = "sib"

	qp2_name_sfc (iqp2_sfc_rstfac4) = "rstfac4"
	qp2_units_sfc(iqp2_sfc_rstfac4) = "-"
	qp2_descr_sfc(iqp2_sfc_rstfac4) = "Product of stress factors 1-3"
	qp2_mask_sfc (iqp2_sfc_rstfac4) = "sib"
	qp2_type_sfc (iqp2_sfc_rstfac4) = "sib"

	qp2_name_sfc (iqp2_sfc_eta) = "eta"
	qp2_units_sfc(iqp2_sfc_eta) = "-"
	qp2_descr_sfc(iqp2_sfc_eta) = "PBL cld topo variance factor"
	qp2_mask_sfc (iqp2_sfc_eta) = "sfc"
	qp2_type_sfc (iqp2_sfc_eta) = "sfc"

	qp2_name_sfc (iqp2_sfc_negeinc) = "negeinc"
	qp2_units_sfc(iqp2_sfc_negeinc) = "-"
	qp2_descr_sfc(iqp2_sfc_negeinc) = "Negative entrainment incidence"
	qp2_mask_sfc (iqp2_sfc_negeinc) = "sfc"
	qp2_type_sfc (iqp2_sfc_negeinc) = "sfc"

	qp2_name_sfc (iqp2_sfc_zlt) = "zlt"
	qp2_units_sfc(iqp2_sfc_zlt) = "m^2/m^2"
	qp2_descr_sfc(iqp2_sfc_zlt) = "Total leaf area index"
	qp2_mask_sfc (iqp2_sfc_zlt) = "sib"
	qp2_type_sfc (iqp2_sfc_zlt) = "sib"

	qp2_name_sfc (iqp2_sfc_green) = "green"
	qp2_units_sfc(iqp2_sfc_green) = "-"
	qp2_descr_sfc(iqp2_sfc_green) = "Fraction of zlt that is green"
	qp2_mask_sfc (iqp2_sfc_green) = "sib"
	qp2_type_sfc (iqp2_sfc_green) = "sib"

	qp2_name_sfc (iqp2_sfc_vcover) = "vcover"
	qp2_units_sfc(iqp2_sfc_vcover) = "-"
	qp2_descr_sfc(iqp2_sfc_vcover) = "Horizontal vegetation cover"
	qp2_mask_sfc (iqp2_sfc_vcover) = "sib"
	qp2_type_sfc (iqp2_sfc_vcover) = "sib"

	qp2_name_sfc (iqp2_sfc_ra) = "ra"
	qp2_units_sfc(iqp2_sfc_ra) = "s/m"
	qp2_descr_sfc(iqp2_sfc_ra) = "Aerodynamic resistance"
	qp2_mask_sfc (iqp2_sfc_ra) = "sib"
	qp2_type_sfc (iqp2_sfc_ra) = "sib"

	qp2_name_sfc (iqp2_sfc_rb) = "rb"
	qp2_units_sfc(iqp2_sfc_rb) = "s/m"
	qp2_descr_sfc(iqp2_sfc_rb) = "Bulk canopy boundary layer resistance"
	qp2_mask_sfc (iqp2_sfc_rb) = "sib"
	qp2_type_sfc (iqp2_sfc_rb) = "sib"

	qp2_name_sfc (iqp2_sfc_rd) = "rd"
	qp2_units_sfc(iqp2_sfc_rd) = "s/m"
	qp2_descr_sfc(iqp2_sfc_rd) = "Ground-canopy air space resistance"
	qp2_mask_sfc (iqp2_sfc_rd) = "sib"
	qp2_type_sfc (iqp2_sfc_rd) = "sib"

	qp2_name_sfc (iqp2_sfc_rst) = "rst"
	qp2_units_sfc(iqp2_sfc_rst) = "s/m"
	qp2_descr_sfc(iqp2_sfc_rst) = "Stomatal resistance"
	qp2_mask_sfc (iqp2_sfc_rst) = "sib"
	qp2_type_sfc (iqp2_sfc_rst) = "sib"

	qp2_name_sfc (iqp2_sfc_chf) = "chf"
	qp2_units_sfc(iqp2_sfc_chf) = "W/m^2"
	qp2_descr_sfc(iqp2_sfc_chf) = "Canopy heat storage flux"
	qp2_mask_sfc (iqp2_sfc_chf) = "sib"
	qp2_type_sfc (iqp2_sfc_chf) = "sib"

	qp2_name_sfc (iqp2_sfc_shf) = "shf"
	qp2_units_sfc(iqp2_sfc_shf) = "W/m^2"
	qp2_descr_sfc(iqp2_sfc_shf) = "Soil heat storage flux"
	qp2_mask_sfc (iqp2_sfc_shf) = "sib"
	qp2_type_sfc (iqp2_sfc_shf) = "sib"

	qp2_name_sfc (iqp2_sfc_radc31) = "radc31"
	qp2_units_sfc(iqp2_sfc_radc31) = "W/m^2"
	qp2_descr_sfc(iqp2_sfc_radc31) = "Radiation intercepted by canopy"
	qp2_mask_sfc (iqp2_sfc_radc31) = "sib"
	qp2_type_sfc (iqp2_sfc_radc31) = "sib"

	qp2_name_sfc (iqp2_sfc_radc32) = "radc32"
	qp2_units_sfc(iqp2_sfc_radc32) = "W/m^2"
	qp2_descr_sfc(iqp2_sfc_radc32) = "Radiation intercepted by ground"
	qp2_mask_sfc (iqp2_sfc_radc32) = "sib"
	qp2_type_sfc (iqp2_sfc_radc32) = "sib"

	qp2_name_sfc (iqp2_sfc_lciinc) = "lciinc"
	qp2_units_sfc(iqp2_sfc_lciinc) = "-"
	qp2_descr_sfc(iqp2_sfc_lciinc) = "Layer Cloud Instability Incidence"
	qp2_mask_sfc (iqp2_sfc_lciinc) = "sfc"
	qp2_type_sfc (iqp2_sfc_lciinc) = "sfc"

	qp2_name_sfc (iqp2_sfc_safe) = "safe"
	qp2_units_sfc(iqp2_sfc_safe) = "J/kg"
	qp2_descr_sfc(iqp2_sfc_safe) = "SAFE"
	qp2_mask_sfc (iqp2_sfc_safe) = "sfc"
	qp2_type_sfc (iqp2_sfc_safe) = "sfc"

	qp2_name_sfc (iqp2_sfc_ustar) = "ustar"
	qp2_units_sfc(iqp2_sfc_ustar) = "m/s"
	qp2_descr_sfc(iqp2_sfc_ustar) = "Friction velocity"
	qp2_mask_sfc (iqp2_sfc_ustar) = "sfc"
	qp2_type_sfc (iqp2_sfc_ustar) = "sfc"

	qp2_name_sfc (iqp2_sfc_mol) = "L"
	qp2_units_sfc(iqp2_sfc_mol) = "m"
	qp2_descr_sfc(iqp2_sfc_mol) = "Monin-Obukhov length"
	qp2_mask_sfc (iqp2_sfc_mol) = "sfc"
	qp2_type_sfc (iqp2_sfc_mol) = "sfc"

	qp2_name_sfc (iqp2_sfc_buoyf) = "buoyf"
	qp2_units_sfc(iqp2_sfc_buoyf) = "W/m^2"
	qp2_descr_sfc(iqp2_sfc_buoyf) = "Surface buoyancy flux"
	qp2_mask_sfc (iqp2_sfc_buoyf) = "sfc"
	qp2_type_sfc (iqp2_sfc_buoyf) = "sfc"

	qp2_name_sfc (iqp2_sfc_spd10) = "spd10"
	qp2_units_sfc(iqp2_sfc_spd10) = "m/s"
	qp2_descr_sfc(iqp2_sfc_spd10) = "Wind Speed at 10m"
	qp2_mask_sfc (iqp2_sfc_spd10) = "sfc"
	qp2_type_sfc (iqp2_sfc_spd10) = "sfc"

	qp2_name_sfc (iqp2_sfc_t2) = "t2"
	qp2_units_sfc(iqp2_sfc_t2) = "K"
	qp2_descr_sfc(iqp2_sfc_t2) = "Temperature at 2m"
	qp2_mask_sfc (iqp2_sfc_t2) = "sfc"
	qp2_type_sfc (iqp2_sfc_t2) = "sfc"

	qp2_name_sfc (iqp2_sfc_z0) = "z0"
	qp2_units_sfc(iqp2_sfc_z0) = "m"
	qp2_descr_sfc(iqp2_sfc_z0) = "Surface Roughness Height"
	qp2_mask_sfc (iqp2_sfc_z0) = "sfc"
	qp2_type_sfc (iqp2_sfc_z0) = "sfc"

	qp2_name_sfc (iqp2_sfc_dtlci1) = "dtlci1"
	qp2_units_sfc(iqp2_sfc_dtlci1) = "K/day"
	qp2_descr_sfc(iqp2_sfc_dtlci1) = "LCI heating above PBL"
	qp2_mask_sfc (iqp2_sfc_dtlci1) = "sfc"
	qp2_type_sfc (iqp2_sfc_dtlci1) = "sfc"

	qp2_name_sfc (iqp2_sfc_dtlci2) = "dtlci2"
	qp2_units_sfc(iqp2_sfc_dtlci2) = "K/day"
	qp2_descr_sfc(iqp2_sfc_dtlci2) = "LCI heating in PBL"
	qp2_type_sfc (iqp2_sfc_dtlci2) = "sfc"

	qp2_name_sfc (iqp2_sfc_dqlci1) = "dqlci1"
	qp2_units_sfc(iqp2_sfc_dqlci1) = "g/kg/day"
	qp2_descr_sfc(iqp2_sfc_dqlci1) = "LCI moistening above PBL"
	qp2_mask_sfc (iqp2_sfc_dqlci1) = "sfc"
	qp2_type_sfc (iqp2_sfc_dqlci1) = "sfc"

	qp2_name_sfc (iqp2_sfc_dqlci2) = "dqlci2"
	qp2_units_sfc(iqp2_sfc_dqlci2) = "g/kg/day"
	qp2_descr_sfc(iqp2_sfc_dqlci2) = "LCI moistening in PBL"
	qp2_mask_sfc (iqp2_sfc_dqlci2) = "sfc"
	qp2_type_sfc (iqp2_sfc_dqlci2) = "sfc"

	qp2_name_sfc (iqp2_sfc_tbp) = "tbp"
	qp2_units_sfc(iqp2_sfc_tbp) = "K"
	qp2_descr_sfc(iqp2_sfc_tbp) = "Level B+ Temperature"
	qp2_mask_sfc (iqp2_sfc_tbp) = "sfc"
	qp2_type_sfc (iqp2_sfc_tbp) = "sfc"

	qp2_name_sfc (iqp2_sfc_wbp) = "wbp"
	qp2_units_sfc(iqp2_sfc_wbp) = "kg/kg"
	qp2_descr_sfc(iqp2_sfc_wbp) = "Level B+ Water Vapor Mixing Ratio"
	qp2_mask_sfc (iqp2_sfc_wbp) = "sfc"
	qp2_type_sfc (iqp2_sfc_wbp) = "sfc"

  do n = 1,nsoil+1
	if (n <= 9) then
	  write(unit=tnumber,fmt='(i1)')n
	else
	  write(unit=tnumber,fmt='(i2)')n
	endif
	qp2_name_sfc (iqp2_sfc_soilscale(n)) = "soilscale"//trim(tnumber)
	qp2_units_sfc(iqp2_sfc_soilscale(n)) = "-"
	qp2_descr_sfc(iqp2_sfc_soilscale(n)) = &
      "Temperature-moisture soil respiration scaling factor "// trim(tnumber)
    qp2_mask_sfc (iqp2_sfc_soilscale(n)) = "sib"
    qp2_type_sfc (iqp2_sfc_soilscale(n)) = "sib"
  enddo

!--------------------------------------------------------------------------
! start hf2s
!--------------------------------------------------------------------------
	hf2_name_sfc (ihf2_sfc_mrso ) = "mrso"
	hf2_units_sfc(ihf2_sfc_mrso  ) = "kg/m2"
	hf2_descr_sfc(ihf2_sfc_mrso  ) = "Total soil water content - 6 hourly"
	hf2_mask_sfc (ihf2_sfc_mrso)   = "sib"

	hf2_name_sfc (ihf2_sfc_mrro ) = "mrro"
	hf2_units_sfc(ihf2_sfc_mrro  ) = "kg/(m2s)"
	hf2_descr_sfc(ihf2_sfc_mrro  ) = "Total runoff (including drainage) - 6 hourly"
	hf2_mask_sfc (ihf2_sfc_mrro)   = "sib"

	hf2_name_sfc (ihf2_sfc_snw  ) = "snw"
	hf2_units_sfc(ihf2_sfc_snw  ) = "kg/m2"
	hf2_descr_sfc(ihf2_sfc_snw  ) = "Snow depth (water equivalent) - 6 hourly"
	hf2_mask_sfc (ihf2_sfc_snw)   = "sib"

	do n=1,nqp2_sfc
	  qp2_name_sfc(n)  = trim(qp2_name_sfc(n))
	  qp2_units_sfc(n) = trim(qp2_units_sfc(n))
	  qp2_descr_sfc(n) = trim(qp2_descr_sfc(n))
	enddo
	do n=1,nhf2_sfc
	  hf2_name_sfc(n) = trim(hf2_name_sfc(n))
	  hf2_units_sfc(n) = trim(hf2_units_sfc(n))
	  hf2_descr_sfc(n) = trim(hf2_descr_sfc(n))
	enddo

!  initialize some qp tables
    qp2_indx_sfc = 0
    hf2_indx_sfc = 0
    qp2_log_sfc = .false.
    hf2_log_sfc = .true.

!  read input data
    if (my_task_sfc == 0) then
      open(unit=2,file=trim(datapath)//'data/sfc_qpopts', form='formatted')

      do
        read(2,*, IOSTAT=ioerr)doqptem,ldummy,nametem
	    nametem = trim(nametem)
        if (ioerr /= 0) exit
        if (doqptem) then
          do n = 1,nqp2_sfc
            if (nametem == qp2_name_sfc(n)) then
              iqp2_sfc = iqp2_sfc + 1
              qp2_log_sfc(n) = .true.
              qp2_indx_sfc(n) = iqp2_sfc
              exit
            endif
            if(n.eq.nqp2_sfc) &
              print *,nametem,' is not an available qp2 diagnostic'
          enddo
        endif
      enddo

      close(2)

	  write(6,*)
      do n = 1,nqp2_sfc
        if (qp2_log_sfc(n)) then
          write(6,11) qp2_name_sfc(n), qp2_descr_sfc(n),qp2_indx_sfc(n)
 11       format('Will save qp2_sfc : ',a16,5x,a20, '   in qp2 slot   ', i8)
        endif
      enddo
	  write(6,*)
    endif  ! if my_task_sfc.eq.0

    do n = 1,nhf2_sfc
       ihf2_sfc = ihf2_sfc + 1
       hf2_indx_sfc(n) = ihf2_sfc
    enddo

!  broadcast information

#ifdef mpi
    call MPI_BCAST(        &
            qp2_indx_sfc,  &
            nqp2_sfc,      &
            mpi_int_type,  &
            0,             &
            MPI_COMM_SFC,  &
            ierr)

    call MPI_BCAST(        &
            qp2_log_sfc,   &
            nqp2_sfc,      &
            MPI_LOGICAL,   &
            0,             &
            MPI_COMM_SFC,  &
            ierr)
        
    call MPI_BCAST(        &
            qp2_name_sfc,  &
            nqp2_sfc*16,   &
            MPI_CHARACTER, &
            0,             &
            MPI_COMM_SFC,  &
            ierr)

    call MPI_BCAST(        &
            qp2_units_sfc, &
            nqp2_sfc*16,   &
            MPI_CHARACTER, &
            0,             &
            MPI_COMM_SFC,  &
            ierr)

    call MPI_BCAST(        &
            qp2_descr_sfc, &
            nqp2_sfc*80,   &
            MPI_CHARACTER, &
            0,             &
            MPI_COMM_SFC,  &
            ierr)

    call MPI_BCAST(        &
            iqp2_sfc,      &
            1,             &
            mpi_int_type,  &
            0,             &
            MPI_COMM_SFC,  &
            ierr)

#endif


  end subroutine initialize_qpinfo_sfc



  subroutine initialize_qpfiles_sfc(simulation_tag)

!  this subroutine assigns all the qp information from the input file atm_qpopts

  use gather_scatter
  include 'netcdf.inc'

  character*40, intent(in) :: simulation_tag

!  variables for reading the netcdf file
  character (len=80) :: ncgeogrid
  integer (kind=int_kind) :: gridID
  integer (kind=int_kind) :: gridvarID
  integer (kind=int_kind) :: gridsize
!  global arrays for grid locations
  real (kind=selected_real_kind(13)), dimension(:), allocatable :: &
    grid_center_lat, grid_center_lon
  real (kind=selected_real_kind(13)), dimension(:,:), allocatable :: &
    grid_corner_lat, grid_corner_lon

!   work variable to scatter the data (global array)
  real (kind=dbl_kind), dimension(:), allocatable :: grid_tem

!  global arrays on geodesic grid
  real (kind=dbl_kind), dimension(im_sfc, jm_sfc) :: centerlatg, centerlong
  real (kind=dbl_kind), dimension(:,:,:), allocatable :: &
    cornerlatg, cornerlong

!  arrays on each processor
  real (kind=dbl_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) :: &
    centerlat, centerlon
  real (kind=dbl_kind), dimension(:,:,:,:), allocatable :: &
    cornerlat, cornerlon
  real (kind=selected_real_kind(13)), allocatable, dimension(:) :: &
    qpcenterlat, qpcenterlon
  real (kind=selected_real_kind(13)), allocatable, dimension(:,:) :: &
    qpcornerlat, qpcornerlon

! indices to map to global vector
  real (kind=dbl_kind), dimension(max_ig_sfc) :: indx_max_ig = c0
  real (kind=dbl_kind), dimension(im_sfc,jm_sfc) :: indx_imjm = c0
  real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) :: rinterp_indx = c0

  character(len=8) :: time_label
  character(len=4) :: proc_label
  integer (kind=int_kind) :: i, j, ksdm, n, n_points, corners, m, istatus, monthofyear

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
  if (n_restart > 0) then
    write(unit=time_label,fmt='(i7)') 1000000+nsecond_dyn/86400
    time_label(1:1) = 'd'
  else
    monthofyear = mod(nmonth-1,12)+1
    write(unit=time_label,fmt='(i7)') 1000000+100*(nyear+nyear0)+monthofyear 
    time_label(1:1) = 'y'
  endif

  write(unit=proc_label,fmt='(i4)') 1000+my_task_sfc
  proc_label(1:1) = 'p'

!  allocate the qp arrays

  allocate(qp2_sfc(iim_sfc, jjm_sfc, 0:iqp2_sfc, nsdm_sfc))
  allocate(hf2_sfc(iim_sfc, jjm_sfc, 0:ihf2_sfc, nsdm_sfc))

  interp_indx_sfc = 0

!   assign a sequential index to each grid point
  if (my_task_sfc == 0) then
    do m = 1,max_ig_sfc
      indx_max_ig(m) = m
    enddo
    call scatter_to_grid_global_sfc(indx_max_ig(:),indx_imjm(:,:))
  endif
  call scatter(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,1,nsdm_sfc, &
               indx_imjm,rinterp_indx)
  interp_indx_sfc(:,:,:) = nint(rinterp_indx(:,:,:))

  if (my_task_sfc == 0) then
!  read the grid coordinates
    select case (max_ig_sfc)
      case (2562)
        ncgeogrid = trim(datapath)//"data/C02562.global.nc"
      case (10242)
        ncgeogrid = trim(datapath)//"data/C10242.global.nc"
      case (40962)
        ncgeogrid = trim(datapath)//"data/C40962.global.nc"
      case default
        print *, "NetCDF warning: invalid grid size specified in"
        print *, "                module ncgeowrt: define_geo_grid."
        print *, "                grid size = ", max_ig_sfc
        print *, "Reading from default grid size = 10242 cells."
        ncgeogrid = trim(datapath)//"data/C10242.global.nc"
      end select

! open grid file, get dimension sizes
	istatus = nf_open (ncgeogrid, NF_NOWRITE, gridID)

	istatus = nf_inq_dimid (gridID, "grid_size", gridvarID)
	istatus = nf_inq_dimlen (gridID, gridvarID, gridsize)

	istatus = nf_inq_dimid (gridID, "grid_corners", gridvarID)
	istatus = nf_inq_dimlen (gridID, gridvarID, corners)

	allocate (grid_tem(gridsize))
	allocate (grid_center_lat(gridsize))
	allocate (grid_center_lon(gridsize))
	allocate (grid_corner_lat(corners,gridsize))
	allocate (grid_corner_lon(corners,gridsize))

	istatus = nf_inq_varid (gridID, "grid_center_lat", gridvarID)
	istatus = nf_get_var_double (gridID, gridvarID, grid_center_lat)

	istatus = nf_inq_varid (gridID, "grid_center_lon", gridvarID)
	istatus = nf_get_var_double (gridID, gridvarID, grid_center_lon)

	istatus = nf_inq_varid (gridID, "grid_corner_lon", gridvarID)
	istatus = nf_get_var_double (gridID, gridvarID, grid_corner_lon)

	istatus = nf_inq_varid (gridID, "grid_corner_lat", gridvarID)
	istatus = nf_get_var_double (gridID, gridvarID, grid_corner_lat)

	istatus = nf_close (gridID)

#ifdef mpi
  endif  ! if(my_task.eq.0)

  call MPI_BCAST(           &
            corners,        &
            1,              &
            mpi_int_type,   &
            0,              &
            MPI_COMM_SFC,   &
            ierr)

  call MPI_BARRIER(MPI_COMM_SFC,ierr)

  if (my_task_sfc == 0) then
#endif

    grid_tem(:) = grid_center_lat(:)
    call scatter_to_grid_global_sfc(grid_tem(:),centerlatg(:,:))
    grid_tem(:) = grid_center_lon(:)
    call scatter_to_grid_global_sfc(grid_tem(:),centerlong(:,:))
    allocate (cornerlatg(im_sfc,jm_sfc,corners))
    allocate (cornerlong(im_sfc,jm_sfc,corners))
    do n = 1,corners
      grid_tem(:) = grid_corner_lat(n,:)
      call scatter_to_grid_global_sfc(grid_tem(:),cornerlatg(:,:,n))
      grid_tem(:) = grid_corner_lon(n,:)
      call scatter_to_grid_global_sfc(grid_tem(:),cornerlong(:,:,n))
    enddo

    deallocate (grid_center_lat) 
    deallocate (grid_center_lon)
    deallocate (grid_corner_lat)
    deallocate (grid_corner_lon)
    deallocate (grid_tem)
  endif ! if(my_task_sfc.eq.0)

!-------------------------------------------------------------------------
! scatter variables from (im_sfc,jm_sfc) to (iim_sfc,jjm_sfc,nsdm_sfc) grid
!-------------------------------------------------------------------------
  allocate (cornerlat(iim_sfc,jjm_sfc,corners,nsdm_sfc))
  allocate (cornerlon(iim_sfc,jjm_sfc,corners,nsdm_sfc))
  call scatter(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,1,nsdm_sfc, &
               centerlatg,centerlat)
  call scatter(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,1,nsdm_sfc, &
               centerlong,centerlon)
  call scatter(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,corners,nsdm_sfc, &
               cornerlatg,cornerlat)
  call scatter(sfc_level,im_sfc,jm_sfc,iim_sfc,jjm_sfc,corners,nsdm_sfc, &
               cornerlong,cornerlon)

  qp_points_sfc = 0
  do ksdm = 1,nsdm_sfc
	do j = 1,jjm_sfc
	  do i = 1,iim_sfc
		if (grid_mask_sfc(i,j,ksdm)) qp_points_sfc = qp_points_sfc + 1
	  enddo
	enddo
  enddo

  allocate (qpcornerlat(corners,qp_points_sfc))
  allocate (qpcenterlat(qp_points_sfc))
  allocate (qpcornerlon(corners,qp_points_sfc))
  allocate (qpcenterlon(qp_points_sfc))
#if resolution != sfc_resolution
  allocate (qpgridarea(qp_points_sfc))
#endif

!  gather unique data
  n_points = 0
  do ksdm = 1,nsdm_sfc
	do j = 1,jjm_sfc
	  do i = 1,iim_sfc
		 if (grid_mask_sfc(i,j,ksdm)) then
			n_points = n_points + 1
			qpcenterlat(n_points) = centerlat(i,j,ksdm)
			qpcenterlon(n_points) = centerlon(i,j,ksdm)
#if resolution != sfc_resolution
			qpgridarea(n_points) = gridarea(i,j,ksdm)
#endif
			qpcornerlat(:,n_points) = cornerlat(i,j,:,ksdm)
			qpcornerlon(:,n_points) = cornerlon(i,j,:,ksdm)
		 endif
	  enddo
	enddo
  enddo

#if resolution == sfc_resolution
  call ginit_sfc_nc (simulation_tag, "qp_output/gsfc_"//trim(time_label) &
                      //trim(proc_label)//".g2.nc", &
                      qp2_descr_sfc, qp2_name_sfc, qp2_units_sfc, qp2_type_sfc, &
                      qp2_log_sfc, qp_points_sfc, &
     		       "hf_output/gsfc_"//trim(time_label) &
                      //trim(proc_label)//".g2.nc", &
                      hf2_descr_sfc, hf2_name_sfc, hf2_units_sfc, &
                      hf2_type_sfc, hf2_log_sfc, &
                      qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon, &
                      corners, max_ig_sfc)
#else
  call ginit_sfc_nc (simulation_tag, "qp_output/gsfc_"//trim(time_label) &
                      //trim(proc_label)//".g2.nc", &
                      qp2_descr_sfc, qp2_name_sfc, qp2_units_sfc, qp2_type_sfc, &
                      qp2_log_sfc, qp_points_sfc, &
     		       "hf_output/gsfc_"//trim(time_label) &
                      //trim(proc_label)//".g2.nc", &
                      hf2_descr_sfc, hf2_name_sfc, hf2_units_sfc, &
                      hf2_type_sfc, hf2_log_sfc, &
                      qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon, &
                      corners, nsfc_tot, qpgridarea)
#endif

  deallocate (qpcornerlat)
  deallocate (qpcenterlat)
  deallocate (qpcornerlon)
  deallocate (qpcenterlon)
  deallocate (cornerlat)
  deallocate (cornerlon)
#if resolution == sfc_resolution
  if (my_task_sfc == 0) then
	deallocate (cornerlatg)
	deallocate (cornerlong)
  endif
#else
  deallocate (qpgridarea)
#endif

  nsecondinit_sfc = nsecond_dyn - dt
  iqp_sample_sfc = 0
  qp2_sfc(:,:,:,:) = c0
  ihf_sample_sfc = 0
  hf2_sfc(:,:,:,:) = c0

  end subroutine initialize_qpfiles_sfc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine qp_sfc_increment

  IMPLICIT none

  iqp_sample_sfc = iqp_sample_sfc + 1
  ihf_sample_sfc = ihf_sample_sfc + 1


  end subroutine qp_sfc_increment

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine qp2sum_sfc(xin, x_const, index, ksdm, len, ioffset)

!  this subroutine accumulates a field in a section in the qp2 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a qp2
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by iqp2_sfc_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            qp2 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  implicit none

  integer (kind=int_kind), intent(in) :: &
    index, &   ! index of the qp slot to save field to
    ksdm, &    ! subdomain to fill
    len        ! length of xin array section
  integer (kind=int_kind), intent(in), optional :: ioffset

  real (kind=dbl_kind), dimension(len), intent(in) :: &
    xin  !  input field to accumulate
  real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
  integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend

  real (kind=dbl_kind), dimension(iimjjm_sfc) :: x1

#ifdef scm
  return
#endif

  if (qp2_log_sfc(index)) then
         
	 if (present(ioffset)) then
	   offset = ioffset
	 else
	   offset = 0
	 endif
	 x1 = c0
	 x1(offset+1:offset+len) = xin(1:len)

	 jbeg = offset/iim_sfc + 1
	 jend = (offset+len-1)/iim_sfc+1

! save the qp2 array
	 do j = jbeg,jend
	   item = (j-1)*iim_sfc
	   ibeg = 1 + max( 0, offset-(j-1)*iim_sfc )
	   iend = min( iim_sfc, offset+len-(j-1)*iim_sfc )
	   qp2_sfc(ibeg:iend,j,qp2_indx_sfc(index),ksdm) =  &
         qp2_sfc(ibeg:iend,j,qp2_indx_sfc(index),ksdm)  &
         + x_const * x1(ibeg+item:iend+item)
	 enddo


  endif !  if(qp2_log_sfc(index))

  end subroutine qp2sum_sfc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine hf2sum_sfc(xin, x_const, index, ksdm, len, ioffset)

!  this subroutine accumulates a field in a section in the hf2 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a hf2
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by ihf2_sfc_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            hf2 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  implicit none

  integer (kind=int_kind), intent(in) :: &
    index, &   ! index of the qp slot to save field to
    ksdm, &    ! subdomain to fill
    len     ! length of xin array section
  integer (kind=int_kind), intent(in), optional :: ioffset

  real (kind=dbl_kind), dimension(len), intent(in) :: &
    xin  !  input field to accumulate
   real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
  integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend

  real (kind=dbl_kind), dimension(iimjjm_sfc) :: x1

#ifdef scm
  return
#endif

  if (hf2_log_sfc(index)) then
         
	 if (present(ioffset)) then
	   offset = ioffset
	 else
	   offset = 0
	 endif
	 x1 = c0
	 x1(offset+1:offset+len) = xin(1:len)

	 jbeg = offset/iim_sfc + 1
	 jend = (offset+len-1)/iim_sfc+1

! save the hf2 array
	 do j = jbeg,jend
	   item = (j-1)*iim_sfc
	   ibeg = 1 + max( 0, offset-(j-1)*iim_sfc )
	   iend = min( iim_sfc, offset+len-(j-1)*iim_sfc )
	   hf2_sfc(ibeg:iend,j,hf2_indx_sfc(index),ksdm) =  &
         hf2_sfc(ibeg:iend,j,hf2_indx_sfc(index),ksdm)  &
         + x_const * x1(ibeg+item:iend+item)
	 enddo


  endif !  if(hf2_log_sfc(index))

  end subroutine hf2sum_sfc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine write_qp_diagnostics_sfc

  use landice_parameters
  use sib_parameters
  use ocean_parameters
  use netcdf_general_tools


  integer (kind=int_kind) :: n
  real (kind=real_kind) :: qp_samples_inv

! normalize
  qp_samples_inv = 1. / float(iqp_sample_sfc)
  qp2_sfc = qp2_sfc * qp_samples_inv

! mask out sections of fields that don't exist over all surfaces with 
!   special value
  do n = 1,nqp2_sfc
	 if (qp2_log_sfc(n)) then
		if (qp2_mask_sfc(n).eq.'sib') &
          where(.not.hr_sib(:,:,:))   &
            qp2_sfc(:,:,qp2_indx_sfc(n),:) = MISSING
        if (qp2_mask_sfc(n).eq.'ice') &
          where(.not.hr_ice(:,:,:))   &
            qp2_sfc(:,:,qp2_indx_sfc(n),:) = MISSING
        if (qp2_mask_sfc(n).eq.'sea') &
          where(.not.hr_sea(:,:,:))   &
            qp2_sfc(:,:,qp2_indx_sfc(n),:) = MISSING
     endif
  enddo
      
!  write the data to a netcdf file
  call gwrt_sfc_nc(nsecond_dyn, nsecondinit_sfc, nyear0,     &
                   qp_points_sfc, grid_mask_sfc,             &
                   qp2_sfc(:,:,1:iqp2_sfc,:), qp2_name_sfc,  &
                   qp2_log_sfc, qp2_indx_sfc, interp_indx_sfc  )

!   zero out for next period of summation
  nsecondinit_sfc = nsecond_dyn
  qp2_sfc = c0
  iqp_sample_sfc = 0

  end subroutine write_qp_diagnostics_sfc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine write_hf_diagnostics_sfc

  use landice_parameters
  use sib_parameters
  use ocean_parameters
  use netcdf_general_tools


  integer (kind=int_kind) :: n
  real (kind=real_kind) :: hf_samples_inv

! normalize
  hf_samples_inv = c1 / float(ihf_sample_sfc)
  hf2_sfc = hf2_sfc * hf_samples_inv

! mask out sections of fields that don't exist over all surfaces with 
!   special value
  do n = 1,nhf2_sfc
	 if (hf2_log_sfc(n)) then
		if (hf2_mask_sfc(n).eq.'sib') &
          where(.not.hr_sib(:,:,:))   &
            hf2_sfc(:,:,qp2_indx_sfc(n),:) = MISSING
        if (hf2_mask_sfc(n).eq.'ice') &
          where(.not.hr_ice(:,:,:))   &
            hf2_sfc(:,:,qp2_indx_sfc(n),:) = MISSING
        if (hf2_mask_sfc(n).eq.'sea') &
          where(.not.hr_sea(:,:,:))   &
            hf2_sfc(:,:,hf2_indx_sfc(n),:) = MISSING
     endif
  enddo
      
!  write the data to a netcdf file
  call gwrt_hf_sfc_nc(nsecond_dyn, nsecondinit_sfc, nyear0,      &
                      qp_points_sfc, grid_mask_sfc,              &
                      hf2_sfc(:,:,1:iqp2_sfc,:), hf2_name_sfc,   &
                      hf2_log_sfc, hf2_indx_sfc, interp_indx_sfc  )

!   zero out for next period of summation
  nsecondinit_sfc = nsecond_dyn
  hf2_sfc = c0
  ihf_sample_sfc = 0

  end subroutine write_hf_diagnostics_sfc

end module qp_module_sfc
