module output_utilities

!-----------------------------------------------------------------------
! PURPOSE : provide a means to do various output functions in a 
!           parallel environment
!     output_process0 - writes a text message and one scalar( optional)
!        for local process 0
!     output_vector_form - writes a geodesic data vector of size max_ig
!        for local process 0, using formatted 5e15.8 ascii format
!     output_vector_bin - writes a geodesic data vector of size max_ig 
!        for local process 0, using unformatted real*4 binary output
!     output_all_in_order - writes a text message and one scalar (optional)
!        for all local processes, ordered from process zero through npe-1
!-----------------------------------------------------------------------

   use kinds
   use component_intra
   private

#ifdef fluxcorrectedtransport
#ifdef mpi
      	 include 'mpif.h'
#endif

! public member functions
   public ::                                                           &
      output_process0,                                                 &
      output_vector_form,                                              &
      output_vector_bin,                                               &
      output_all_in_order

   contains

   subroutine output_process0(component_name, msg_text, msg_int, msg_dbl)

! argument list
   character(*), intent(in) ::                                         &
      component_name   ! handle to get necessary parallel environment variables
   character(*), intent(in) ::                                         &
      msg_text         ! text message to write
   integer (kind=int_kind), intent(in), optional ::                    &
      msg_int
   real (kind=dbl_kind), intent(in), optional ::                       &
      msg_dbl

! local variables
   integer (kind=int_kind) ::                                          &
      my_task

   my_task = get_my_task(component_name)

   if(my_task == 0) then

! write the messages
      if(present(msg_int) .or. present(msg_dbl)) then
         if(present(msg_int)) write(6,*)msg_text,msg_int
         if(present(msg_dbl)) write(6,*)msg_text,msg_dbl
      else
         write(6,*) msg_text
      endif

   endif

   end subroutine output_process0


   subroutine output_vector_form(component_name, file_name, x)

! argument list
   character(*), intent(in) ::                                         &
      component_name   ! handle to get necessary parallel environment variables
   character(*), intent(in) ::                                         &
      file_name        ! name of output file to write to
   real(kind=dbl_kind), intent(in), dimension(:) :: x  ! max_ig vector

! local variables
   integer (kind=int_kind) ::                                          &
      my_task

   my_task = get_my_task(component_name)

   if(my_task == 0) then
      open(unit=39,file=trim(datapath)//file_name,form='formatted')
      write(39,fmt='(5e15.8)')x
      close(39)
   endif

   end subroutine output_vector_form


   subroutine output_vector_bin(component_name, file_name, x)

! argument list
   character(*), intent(in) ::                                         &
      component_name   ! handle to get necessary parallel environment variables
   character(*), intent(in) ::                                         &
      file_name        ! name of output file to write to
   real(kind=dbl_kind), intent(in), dimension(:) :: x  ! max_ig vector

! local variables
   integer (kind=int_kind) ::                                          &
      my_task, max_ig 
   real (kind=real_kind), allocatable, dimension(:) :: y

   my_task = get_my_task(component_name)

   if(my_task == 0) then
      max_ig = size(x)
      allocate(y(max_ig))
      y = x
      open(unit=39,file=trim(datapath)//file_name,form='unformatted')
      write(39)y
      close(39)
      deallocate(y)
   endif

   end subroutine output_vector_bin


   subroutine output_all_in_order(component_name, msg_text, msg_int, msg_dbl)

! argument list
   character(*), intent(in) ::                                         &
      component_name   ! handle to get necessary parallel environment variables
   character(*), intent(in) ::                                         &
      msg_text         ! text message to write
   integer (kind=int_kind), intent(in), optional ::                    &
      msg_int
   real (kind=dbl_kind), intent(in), optional ::                       &
      msg_dbl

! local variables
   integer (kind=int_kind) ::                                          &
      my_task, i, npe, communicator, ierr

   my_task = get_my_task(component_name)
   communicator = get_my_communicator(component_name)
   npe = get_my_npe(component_name)

! this loop insures that the output from each process will be ordered
   do i = 0,npe-1
      if(my_task == i) then

! write the messages
         if(present(msg_int) .or. present(msg_dbl)) then
            if(present(msg_int)) write(6,*)' *** process ',my_task,msg_text,msg_int
            if(present(msg_dbl)) write(6,*)' *** process ',my_task,msg_text,msg_dbl
         else
            write(6,*) msg_text
         endif

      endif
#ifdef mpi
      CALL MPI_BARRIER (communicator,ierr)
#endif
   enddo
#ifdef mpi
   CALL MPI_BARRIER (communicator,ierr)
#endif

   end subroutine output_all_in_order

#endif


end module output_utilities
