module atmos_ozone_utilities

! this module has routines that provide climatological ozone mixing ratios
!   on the geodesic grid and interpolated to model levels. These routines:
!     1. read the monthly data and interpolate to the geodesic grid (initialize_ozone).
!        this is done once
!     2. get a daily value of the data by time interpolation (time_update_ozone). This
!        is done daily.
!     3. vertically interpolate the ozone data to model levels (set_current_ozone). This
!        is done every time the radiation is called due to the possibility of large
!        heating rates in the stratosphere moving the zeta surfaces substantially
!        from time step to time step.

use kinds
use model_parameters
use sigma_coordinate
use domain_decomposition
USE physical_parameters
#ifndef scm
        use grid_metrics
#else
        use scm_module
#endif

implicit none
save
private

! module scope variables:
! parameters for input files
   integer (kind=int_kind), parameter ::   &
      ix=72,           & ! zonal dimension
      iy=44,           & ! meridional dimension
      lo3i=60,         & ! number of levels
      lo3e=100,        & ! number of extended levels
      lo3d=lo3i-1,     & ! number of levels
      lo3f=lo3e-lo3i+1,& ! number of levels
      lrec=(iy*lo3d + ix*iy)*4       ! file record length

   integer (kind=int_kind) ::                    &
      i, j, k, l, m, ksdm                     ! loop indices

! input variables needed for vertical interpolation down the road
   real (kind=4), dimension(lo3e) ::             &
      plol,             &! layer pressure (mb)
      zlol               ! layer height (km)
   real (kind=4), dimension(lo3e+1), save ::           &
      ploi,             &! interface pressure (mb)
      dploi,            &! layer thickness (dploi(3) has ploi(k) as base) (mb)
      zloi               ! interface height (km)
      
! variables interpolated to geodesic grid, still on input levels and by month
real (kind=dbl_kind), dimension(iim,jjm,lo3i,12,nsdm), save ::   &
   o3g                   ! ozone profile (ppmv)
real (kind=dbl_kind), dimension(iim,jjm,12,nsdm), save ::        &
   tcog                  ! total column ozone (cm-stp)

! public member functions
public ::              &
   initialize_ozone,   &
   set_current_ozone
   
contains

subroutine initialize_ozone(o3dat, pl2)

!monthly values of ozone mixing ratio interpolated horizontally and vertically
!  are generated
   real(kind=dbl_kind), intent(inout) :: o3dat(iim,jjm,nlm+1,12,nsdm)
   real(kind=dbl_kind), intent(in) :: pl2(iim,jjm,nlm+1,nsdm)

!!!!!!!!!!!!!!!
! Read in the 4x5 data produced by Liang and Wang - SUNYA
!
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  http://www-pcmdi.llnl.gov/amip/AMIP2EXPDSN/OZONE/wang.html
!
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     data basis: ozonesonds, sage nd toms data
!
!    archived by xin-zhong liang and wei-chyung wang
!
!    date of release: september 16, 1996
!
!-----references:
!     liang, x.-z., and w.-c. wang, 1996: atmospheric ozone climatology
!            for use in gcms, (in preparation)
!     wang, w.-c., x.-z. liang, m.p. dudek, d. pollard, and
!            s.l. thompson, 1995: atmospheric ozone as a climate gas.
!            atmos. res., 37, 247-256.
!
! This code is adapted from subroutine radopl provided by Liang. Some of the
!   things specific to the CSU data set they created have been hard-wired in.
!
! The data consists of two parts: lat-z profiles at 44 latitudes, 60 pressures
!   and 72x44 maps of total column ozone. There are 12 climatological monthly means
!   of each. The latter is used to scale the former after vertical interpolation to
!   insure that the vertical integral is conserved.

!  input data arrays:
   real (kind=4), dimension(ix,iy) ::            &
      tcoi               ! total column ozone (cm-stp)
   real (kind=4), dimension(iy,lo3i) ::          &
      o3i                ! ozone concentration (ppmv)
   real (kind=4), dimension(ix) ::               &
      rlon               ! longitudes (deg E)
   real (kind=4), dimension(iy) ::               &
      rlat               ! latitudes (deg N)
   real (kind=4) ::                              &
      o3vubc             ! ozone upper boundary condition (ppmv)
      

! work variables
   integer (kind=int_kind) ::                    &
      lono3d, lato3d, layo3d, layo3e,   & ! header variables
      js, jn, iw, ie,                   & ! indices of bounding lats, lons
      m                              ! subdomain and indices
   real (kind=4) ::                              &
      sl, of, zlolk, dplol, o3pub, plo3, r2d,    &
      hgtub, hgtsb, hgtus, ologu
   real (kind=dbl_kind)  ::                      &
      w_sw, w_se, w_ne, w_nw, w_s, w_n    ! interpolation weights
      

   open(3 ,file=trim(datapath)//'data/GAS.IEEE-CSU',status='old',         &
               form='unformatted'   &
              ,access='direct',recl=LREC)
!
!-----Read in the header information (input data grid mesh)
!
   read(3 ,rec=1)                     &
             LONO3D                   &   ! # longitudes carrying data
           , LATO3D                   &   ! # latitudes  carrying data
           , LAYO3D                   &   ! # layers     carrying data
           , LAYO3E                   &   ! # layers with extended top
           , O3VUBC                   &   ! the u.b.c. ozone   in ppmv
           ,(RLON(I),I=1,IX)          &   ! longitudes      in degrees
           ,(RLAT(J),J=1,IY)          &   ! latitudes       in degrees
           ,(PLOL(K),K=1,LO3E)        &   ! layer pressure       in mb
           ,(ZLOL(K),K=1,LO3E)        &   ! layer height         in km
           ,(PLOI(K),K=1,LO3E+1)      &   ! interface pressure   in mb
           ,(ZLOI(K),K=1,LO3E+1)          ! interface height     in km
   PLO3   = PLOI(1+LO3F)
   O3PUB       = O3VUBC* PLOI(2)    / PLO3
   OLOGU       = LOG(O3VUBC)
   HGTUB       = ZLOL(1)
   HGTSB       = ZLOL(2+LO3F-1)
   HGTUS       = HGTUB - HGTSB

      DPLOi(1)  = ploi(1+LO3F)
      DO  L    = 2 ,LO3I
        DPLOi(L)  = ploi(L+LO3F)-ploi(L+LO3F-1)
      enddo

! read monthly data
   do m = 1,12
      read(3 ,rec=m+1)                       &
              ((O3I(J,L),J=1,IY),L=2,LO3I)   &   ! scale in ppmv
             ,((TCOI(I,j),I=1,IX),j=1,iy)        ! tca in cm-STP

!-----Extrapolation by Log-linear in height above LO3B
      DO J    = 1 ,IY
         O3I(J,1) = O3PUB
         SL       =(OLOGU - LOG(O3I(J,2)))/HGTUS        ! slope
         OF       = OLOGU - SL*HGTUB                    ! offset
         DO K    = 2 ,LO3F
            ZLOLK       = ZLOL(K)
            DPLOL       =(PLOI(K+1)-PLOI(K)) / PLO3
            O3I(J,K) = EXP(OF+SL*ZLOLK)*DPLOL+ O3I(J,K)
         enddo
      enddo
      
! interpolate horizontally to model geodesic grid (linear for o3i
!    bilinear for tcoi)
      r2d = 180. / pi
      do ksdm = 1,nsdm
         do j = 1,jjm
            do i   = 1,iim
#ifndef scm
               js = (r2d*grid_center_lat(i,j,ksdm)+90.0) * 0.25  ! lat-lon lat index s of gridpoint
               jn = js + 1                                       ! lat-lon lat index s of gridpoint
               iw = (r2d*grid_center_lon(i,j,ksdm) + 185.) * 0.2 ! lat-lon lat index s of gridpoint
               ie = iw + 1                                       ! lat-lon lat index s of gridpoint
!  weights for bilinear interpolation
               w_sw = ((5.*ie-185.) - r2d*grid_center_lon(i,j,ksdm)) *  &
                      ((4.*jn-90.) - r2d*grid_center_lat(i,j,ksdm)) 
               w_se = (r2d*grid_center_lon(i,j,ksdm) - (5.*iw-185.)) *  &
                      ((4.*jn-90.) - r2d*grid_center_lat(i,j,ksdm))
               w_ne = (r2d*grid_center_lon(i,j,ksdm) - (5.*iw-185.)) *  &
                      (r2d*grid_center_lat(i,j,ksdm) - (4.*js-90.))
               w_nw = ((5.*ie-185.) - r2d*grid_center_lon(i,j,ksdm)) *  &
                      (r2d*grid_center_lat(i,j,ksdm) - (4.*js-90.))
! weights for linear latitude interpolation
               w_s = (4.*jn-90.) - r2d*grid_center_lat(i,j,ksdm)
               w_n = r2d*grid_center_lat(i,j,ksdm) - (4.*js-90.)
#else
               js = (r2d*lat_scm+90.0) * 0.25  ! lat-lon lat index s of gridpoint
               jn = js + 1                                       ! lat-lon lat index s of gridpoint
               iw = (r2d*lon_scm + 185.) * 0.2 ! lat-lon lat index s of gridpoint
               ie = iw + 1                                       ! lat-lon lat index s of gridpoint
!  weights for bilinear interpolation
               w_sw = ((5.*ie-185.) - r2d*lon_scm) *  &
                      ((4.*jn-90.) - r2d*lat_scm) 
               w_se = (r2d*lon_scm  - (5.*iw-185.)) *  &
                      ((4.*jn-90.) - r2d*lat_scm)
               w_ne = (r2d*lon_scm - (5.*iw-185.)) *  &
                      (r2d*lat_scm - (4.*js-90.))
               w_nw = ((5.*ie-185.) - r2d*lon_scm) *  &
                      (r2d*lat_scm - (4.*js-90.))
! weights for linear latitude interpolation
               w_s = (4.*jn-90.) - r2d*lat_scm
               w_n = r2d*lat_scm - (4.*js-90.)
#endif
!  put lat lon indices within bounds
               js = max(js,1)
               jn = min(jn,44)
               jn = max(jn,1)
               js = min(js,44)
               if(iw.le.0) iw = iw + 72
               if(iw.gt.72) iw = iw - 72
               if(ie.le.0) ie = ie + 72
               if(ie.gt.72) ie = ie - 72
!  interpolate (bilinearly)
               tcog(i,j,m,ksdm) = ( tcoi(iw,jn) * w_nw             &
                                +    tcoi(iw,js) * w_sw            &
                                +    tcoi(ie,js) * w_se            &
                                +    tcoi(ie,jn) * w_ne ) * 0.05
               o3g(i,j,:,m,ksdm) = ( o3i(jn,:) * w_n           &
                                +    o3i(js,:) * w_s ) * 0.25
            enddo
         enddo
      enddo
   enddo   ! m loop
   
! close the data file
   close(3)

end subroutine initialize_ozone


subroutine set_current_ozone(ksdm, m, prs_phy, climo3)

! This routine vertical interpolates climatological ozone from standard pressure levels
!   to model pressures, while conserving the column integral. Some code is adapted from
!   Liang's subroutine radopl.

! Argument list
   integer (kind=int_kind), intent(in) ::   &
      ksdm,               & ! subdomain
      m                     ! month index
   real (kind=dbl_kind), dimension(iim,jjm,0:nlm), intent(in) ::  &
      prs_phy       ! model's physics grid pressure levels at layer edges
   real (kind=dbl_kind), dimension(iim,jjm,nlm+1), intent(out) ::  &
      climo3        ! climatological ozone mixing ratio (kg/kg)
      
! local variables
   real (kind=dbl_kind), dimension(lo3i) ::   &
      plo3      ! pressure work array(mb)
   real (kind=dbl_kind), dimension(nlm+1) ::   &
      o3pl      ! ozone path from toa (cm-stp)
! variables interpolated to geodesic grid and to a specific day
!   still on input levels
real (kind=dbl_kind), dimension(iim,jjm,lo3i) ::   &
   opl                   ! pressure weighted ozone profile

   real (kind=dbl_kind) ::                    &
      ojt, opljt, plo3t, ojs, opljs,          &
      plo3s, plo3u, plo3d, pik, wk, fac

! fac converts from cm-stp to kg/kg
   fac = grav * 101325. / (100. * 173.22 * 273.15) 
!               std p     cks->mks  rgas o3   std t
!
!-----Set interface pressures for all data carrying layers
!
! from the monthly values, compute the pressure weighted vertical integrations
!    (from Liang's code)
      DO l    = 1 ,lo3i
         k = l+lo3f
         DO J    = 1 ,jjm
            DO i    = 1 ,iim
               Opl(i,J,L) = O3g(i,J,L,m,ksdm) * dploi(l)
            enddo
         enddo
      enddo
      DO L    = 2 ,LO3I
         K           = L - 1
         DO J    = 1 ,jjm
            DO i    = 1 ,iim
               Opl(i,j,L) = Opl(i,J,K) + Opl(i,J,L)
            enddo
         enddo
      enddo
   DO L    = 1 ,LO3I
      PLO3(L)   = PLOI(L+LO3F)
   enddo
   do j = 1,jjm       ! loop over grid cells
      do i = 1,iim    ! loop over grid cells
         OJT         = O3g(i,J,1,m,ksdm)
         OPLJT       = OPL(i,J,1)
         PLO3T       = PLO3(1)
         OJS         = O3g(i,J,LO3I,m,ksdm)
         OPLJS       = OPL(i,J,LO3I)
         PLO3S       = PLO3(LO3I)
         DO  L    = 2 ,LO3I
            PLO3U       = PLO3(L-1)
            PLO3D       = PLO3(L)
            DO k    = 1 ,nlm+1
               PIK         = Prs_phy(I,j,K-1) * 0.01
               IF( PIK.LT.PLO3T ) THEN
!.....Assume homogeneous Ozone layer above PLO3T
                  O3PL(k) = OPLJT  + OJT*(PIK-PLO3T)
               ELSE IF( PIK.EQ.PLO3T ) THEN
!.....Just on the Ozone input top PLO3T
                  O3PL(k) = OPLJT
               ELSE IF( PIK.GT.PLO3U.AND.PIK.LT.PLO3D ) THEN
!.....Interpolate between [PLO3U,PLO3D]
                  O3PL(k) = OPL(i,J,L)  + O3g(i,J,L,m,ksdm)*(PIK-PLO3D)
               ELSE IF( PIK.EQ.PLO3D ) THEN
!.....Just on the Ozone input-level PLO3D
                  O3PL(k) = OPL(i,J,L)
               ELSE IF( PIK.GT.PLO3S ) THEN
!.....Assume homogeneous Ozone layer below PLO3S
                  O3PL(k) = OPLJS  + OJS*(PIK-PLO3S)
               ENDIF
            enddo
         enddo

! normalize by the total column amount         
         WK   = TCOg(I,J,m,ksdm)/O3PL(nlm+1)
!
!-----Calculate the radiative path length from the toa
!
         DO l    = 1 ,nlm + 1
            O3PL(l) = WK*O3PL(l)
         enddo
         
! currently the ozone is in cm-stp, convert to layer mixing ratio
         climo3(i,j,1) = fac * o3pl(1) / prs_phy(i,j,0)
         do l = 1,nlm
            climo3(i,j,l+1) = fac*(o3pl(l+1)-o3pl(l)) /             &
                            (prs_phy(i,j,l)-prs_phy(i,j,l-1))
         enddo
      enddo   ! i loop
   enddo      ! j loop

end subroutine set_current_ozone

end module atmos_ozone_utilities
