      module bc_update

!  This module gets controls the updating of time varying boundary conditions
!  It is assumed that monthly values are provided for the boundary conditions
!     and that the value of the boundary condition is an interpolation
!     of the monthly values at the precise midpoints of the months.

      use kinds
      use domain_decomposition
      use domain_decomposition_sfc
      use physical_parameters
      use time_manager_module
      implicit none

      logical (kind=log_kind), save :: 
     *     fixday = .false. 

      integer (kind=int_kind), save ::
     *     sdedyfix = 80

      logical (kind=log_kind), save ::
     *     init_bc = .true.

      integer (kind=int_kind), save, dimension(12) :: month
      DATA MONTH/31,28,31,30,31,30,31,31,30,31,30,31/

      integer (kind=int_kind), save ::
     *    k1  ! index of month with midmonth point prior to interpolation time
     *,   k2  ! index of month with midmonth point after interpolation time
     *,   k1_sea  ! analogous to k1, but for amip bc
     *,   k2_sea  ! analogous to k2, but for amip bc

!  k1 and k2 cycle at the end of the year
!  k1_sea and k2_sea continue until data runs out

      real (kind=dbl_kind), save ::
     *    tpgf1 ! interpolation weight for monthly array index k1
     *,   tpgf2 ! interpolation weight for monthly array index k2
     *,   dtpg  ! inverse interpolation timestep
      real (kind=dbl_kind), save :: nday

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!
!	fixday = if true, use perpetual conditions for day sdedyfix
!       sdedyfix = day of year to apply perpetual boundary conditions
!       month = array of days in each calendar month
!
!-------------------------------------------------------------------

      contains



 	subroutine bc_update_manager(sst_prog)

!  This module computes which months contribute to the temporal interpolation
!     of the boundary conditions and the interpolation weights.

      implicit none

      real (kind=dbl_kind), intent(in), optional :: 
     &   sst_prog(iim,jjm,nsdm)

      real (kind=dbl_kind) :: midmonth, mnthdy
      integer (kind=int_kind) :: jday, jmonth, ksdm
      
      if(fixday) then
         nday = sdedyfix
      else
         nday = ndayofyear -1
      endif

      jday = month(1)
      jmonth = 1
      do while(jday.le.int(nday))
         jmonth = jmonth + 1
         jday = jday + month(jmonth)
      enddo
      mnthdy = month(jmonth) + nday - jday

      midmonth = month(jmonth) * 0.5
      if (mnthdy .lt. midmonth) then
         k1 = jmonth -1
         k2 = jmonth
         k1_sea = k1 + nyear*12
         k2_sea = k2 + nyear*12
         if (k1 .lt. 1) k1=12
         if(k1_sea.lt.1)k1_sea = 1
         TPGF1 = (midmonth-MNTHDY)*2.0/(MONTH(K1)+month(k2))
         tpgf2 = 1.0 - tpgf1
      else
         k1 = jmonth
         k2 = jmonth + 1
         k1_sea = k1 + nyear*12
         k2_sea = k2 + nyear*12
         if (k2 .gt. 12) k2 = 1
         TPGF2 = (MNTHDY-midmonth)*2.0/(MONTH(K1)+month(k2))
         TPGF1 = 1.0 - TPGF2
      endif

      dtpg = 2. / ( 86400.*float(month(k1)+month(k2)) )

!--------------------------------------------------------------------------
! loop over the atmos subdomains
!--------------------------------------------------------------------------
      do ksdm = 1,nsdm

!--------------------------------------------------------------------------
!  update the atm bc every time step
!--------------------------------------------------------------------------

         call bc_update_atm(ksdm)

      enddo
!--------------------------------------------------------------------------
! end loop over the atmos subdomains
!--------------------------------------------------------------------------

!--------------------------------------------------------------------------
! loop over the surface subdomains
!--------------------------------------------------------------------------
        do ksdm = 1,nsdm_sfc

!--------------------------------------------------------------------------
!  update other boundary conditions if required
!--------------------------------------------------------------------------
       if(l_bc_update.or.init_bc) then
          if(present(sst_prog)) then
 	     call bc_update_ocean(ksdm, sst_prog(1,1,ksdm))
          else
 	     call bc_update_ocean(ksdm)
          endif
          call bc_update_sib(ksdm)
!         no landice boundary conditions to update
       endif

      enddo
!--------------------------------------------------------------------------
! end loop over the surface subdomains
!--------------------------------------------------------------------------

!--------------------------------------------------------------------------
! set flag to false to indicate that initialization is complete
!--------------------------------------------------------------------------
      init_bc = .false.


      end subroutine bc_update_manager





      subroutine bc_update_atm(ksdm)

!  This subroutine computes the orbital position and the prescribed
!     climatological ozone.

      use prognostics
      use atmos_ozone_utilities
#ifdef bugs_rad
      use rad_params
      use rad_o3params
#else
      use radgcm_parameters
#endif


      implicit none

      integer (kind=int_kind), intent(in) :: ksdm

      logical (kind=log_kind), save :: first=.true.
      real (kind=dbl_kind) :: t1, t2, t3, t4
      integer (kind=int_kind) :: sdedy
      integer (kind=int_kind), save :: iiday
      real (kind=dbl_kind), save :: y
      integer (kind=int_kind) :: i, j, l


      if(ksdm.eq.1) then
C     INTEGRATE FOR THE EARTHS POSITION
C     FIRST IS NEEDED LOWER DOWN IN THE CODE, AND IS SET FALSE THERE

      sdedy = int(nday)
      IF (FIRST .OR. .NOT. FIXDAY) THEN
         IF (.NOT. FIRST .AND. SDEDY .NE. EQNX) then
            T1 = FFF(Y)*H
            T2 = FFF(Y+T1*.5)*H
            T3 = FFF(Y+T2*.5)*H
            T4 = FFF(Y+T3)*H
            Y = Y + (T1+2.*(T2+T3)+T4)/6.
         else
            IIDAY  = SDEDY - EQNX
            y = 0.0
            IF (IIDAY .ne. 0) then
               IF (IIDAY .LT. 0) IIDAY = IIDAY + 365
               do while (iiday .gt. 0)
                  iiday = iiday -1
                  T1 = FFF(Y)*H
                  T2 = FFF(Y+T1*.5)*H
                  T3 = FFF(Y+T2*.5)*H
                  T4 = FFF(Y+T3)*H
                  Y = Y + (T1+2.*(T2+T3)+T4)/6.
               enddo
            endif
         endif

C     COMPUTE DECLINATION AND EARTH-SUN DISTANCE

         SIND   = sin(decmax*(pi/180.)) * SIN(Y)
         COSD   = SQRT(1.-SIND*SIND)
         RSDIST = ((1.-ECCN*ECCN)/(1.-ECCN*COS(Y-PERHLR)))**2
      ENDIF

      endif  ! if ksdm.eq.1

      if(first.or..not.fixday) then
!---- update climatological ozone:
         CALL set_current_ozone (ksdm, k1, plgl2(:,:,1:nlm+1,    ksdm), 
     +                           o3dat (:,:,:,k1,ksdm) )
         CALL set_current_ozone (ksdm, k2, plgl2(:,:,1:nlm+1,    ksdm), 
     +                           o3dat (:,:,:,k2,ksdm) )
   
         do l = 1, nlm+1
            do j = 1, jjm
               do i = 1, iim
                  climo3(i,j,l,ksdm) = tpgf1 * o3dat(i,j,l,k1,ksdm)
     ^                               + tpgf2 * o3dat(i,j,l,k2,ksdm)
               enddo
            enddo
         enddo

!---- surface fluxes for passive tracers:
#ifdef passive_tracers
         do l = 1, npassive
            do j = 1, jjm
               do i = 1, iim
                  tracer_flux(i,j,l,ksdm) = 
     +                            tpgf1 * tracer_dat(i,j,l,k1,ksdm)
     +                          + tpgf2 * tracer_dat(i,j,l,k2,ksdm)
               enddo
            enddo
         enddo
#endif
      endif !(end first.or..not.fixday)

      if(ksdm.eq.nsdm) first = .false.

      contains

      function fff(x) result(f_result)
      implicit none

      real (kind=dbl_kind), intent(in) :: x
      real (kind=dbl_kind) :: f_result

c    Function definition for earth's orbit
      f_result = sqrt(RECCN)*(1.-ECCN*COS(X-PERHLR))**2

      end function fff


      end subroutine bc_update_atm



      subroutine bc_update_ocean(ksdm,sst_prog)

      use ocean_parameters

      implicit none

      integer (kind=int_kind), intent(in) :: ksdm
      real (kind=dbl_kind), intent(in), optional ::
     &   sst_prog(iim_sfc,jjm_sfc)

      logical (kind=log_kind), save :: first=.true.

      integer (kind=int_kind) :: i, j

C     UPDATE SEA SURFACE TEMPERATURES, ice thickness, qflux parameters

      if(nocean.eq.0) return

      if(nmonth_bc.le.12) then
        k1_sea=k1
        k2_sea=k2
      else
        if(k2_sea.gt.nmonth_bc) k2_sea=nmonth_bc
        if(k1_sea.gt.nmonth_bc) stop 'no more bc_ocean data - stopping'
      endif

      if(first.or..not.fixday) then
         DO j=1,jjm_sfc
            DO i=1,iim_sfc
               if(lqflux) then
                  qf(I,j,ksdm) = TPGF1*qfm(I,j,k1_sea,ksdm)
     *                                + TPGF2*qfm(I,j,k2_sea,ksdm)
                  if(.not.e2prog)
     *                e2(I,j,ksdm) = TPGF1*e2m(I,j,k1_sea,ksdm)
     *                                + TPGF2*e2m(I,j,k2_sea,ksdm)
               else
                  iz(I,j,ksdm) = TPGF1*izm(I,j,k1_sea,ksdm)
     *                                    + TPGF2*izm(I,j,k2_sea,ksdm)
                  iz(i,j,ksdm) = 
     *                max( min(iz(i,j,ksdm),1.0_dbl_kind), 0.0_dbl_kind)
               endif

               if(iz(i,j,ksdm) .le. 0.000001) then
                  sn(i,j,ksdm) = 0.0
                if(present(sst_prog) )then
                  sst(i,j,ksdm) = sst_prog(i,j)
                else
                  IF (.not.  lqflux ) then
                     sst(I,j,ksdm) = TPGF1*sstm(I,j,k1_sea,ksdm)
     *                           + TPGF2*sstm(I,j,k2_sea,ksdm) + sstanom
                     sst(i,j,ksdm) = max(sst(i,j,ksdm), 271.38_dbl_kind)
                  endif
                endif
               endif
            enddo
         enddo
      endif

      if(ksdm.eq.nsdm_sfc) first = .false.


      end subroutine bc_update_ocean



      subroutine bc_update_sib(ksdm)

      use sib_parameters

      implicit none

      integer (kind=int_kind), intent(in) :: ksdm

      logical (kind=log_kind), save :: first=.true.

      integer (kind=int_kind) :: i, j, l

C     UPDATE SEA SURFACE TEMPERATURES, ice thickness, qflux parameters

      if(nsib.eq.0)return

      if(first.or..not.fixday) then
         DO j=1,jjm_sfc
            DO i=1,iim_sfc
               aparc(I,j,ksdm) = TPGF1*aparc_m(I,j,k1,ksdm)
     *                             + TPGF2*aparc_m(I,j,k2,ksdm)
               zlt(I,j,ksdm) = TPGF1*zlt_m(I,j,k1,ksdm)
     *                             + TPGF2*zlt_m(I,j,k2,ksdm)
               green(I,j,ksdm) = TPGF1*green_m(I,j,k1,ksdm)
     *                             + TPGF2*green_m(I,j,k2,ksdm)
               z0d(I,j,ksdm) = TPGF1*z0d_m(I,j,k1,ksdm)
     *                             + TPGF2*z0d_m(I,j,k2,ksdm)
               dd(I,j,ksdm) = TPGF1*dd_m(I,j,k1,ksdm)
     *                             + TPGF2*dd_m(I,j,k2,ksdm)
               rbc(I,j,ksdm) = TPGF1*rbc_m(I,j,k1,ksdm)
     *                             + TPGF2*rbc_m(I,j,k2,ksdm)
               rdc(I,j,ksdm) = TPGF1*rdc_m(I,j,k1,ksdm)
     *                             + TPGF2*rdc_m(I,j,k2,ksdm)
               gmudmu(I,j,ksdm) = TPGF1*gmudmu_m(I,j,k1,ksdm)
     *                             + TPGF2*gmudmu_m(I,j,k2,ksdm)
            enddo
         enddo

         if(prescribed_gw) then
            DO j=1,jjm_sfc
               DO i=1,iim_sfc
                  DO l=1,3
                     www(I,j,l,ksdm) = TPGF1*www_m(I,j,l,k1,ksdm)
     *                                 + TPGF2*www_m(I,j,l,k2,ksdm)
                  enddo
               enddo
            enddo
         endif
      endif

      if(first) then
	do j = 1,jjm_sfc
	  do i = 1,iim_sfc
           if(hr_sib(i,j,ksdm)) then
             rootd(i,j,ksdm) =
     *            min(rootd(i,j,ksdm),(sodep(i,j,ksdm)*0.75))
c soil moisture layers
             zdepth(i,j,1,ksdm) = 0.02 * poros(i,j,ksdm)
             zdepth(i,j,2,ksdm) = 
     *            (rootd(i,j,ksdm) - 0.02)*poros(i,j,ksdm)
             zdepth(i,j,3,ksdm) = poros(i,j,ksdm)*sodep(i,j,ksdm) 
     *             - ( zdepth(i,j,1,ksdm)+zdepth(i,j,2,ksdm) )
            endif
           enddo
        enddo
c soil thermal layers
        if(.not.forcerestore) then
	  do j = 1,jjm_sfc
	    do i = 1,iim_sfc
             if(hr_sib(i,j,ksdm)) then
              ztdep(i,j,1,ksdm) = 6.0 - sodep(i,j,ksdm)
              ztdep(i,j,2,ksdm) = sodep(i,j,ksdm) - rootd(i,j,ksdm)
              ztdep(i,j,3,ksdm) = 8. * (rootd(i,j,ksdm)-0.02) / 15.
              ztdep(i,j,4,ksdm) = 4. * (rootd(i,j,ksdm)-0.02) / 15.
              ztdep(i,j,5,ksdm) = 2. * (rootd(i,j,ksdm)-0.02) / 15.
              ztdep(i,j,6,ksdm) = 1. * (rootd(i,j,ksdm)-0.02) / 15.
             endif
            enddo
          enddo
        endif
c       if(ksdm.eq.nsdm_sfc)call write_sib_prognostics
      endif

      if(ksdm.eq.nsdm_sfc) first = .false.


      end subroutine bc_update_sib

      end module bc_update

