!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

         module flux_coupler_module

!-----------------------------------------------------------------------
! PURPOSE: provide a means of allowing the different model components
!	interact.
!-----------------------------------------------------------------------

        use kinds
        use physical_parameters
        use model_control
        use prognostics
	use sfc_control_module
	use sfc_init
#ifndef scm
        use assign_owner
        use gather_scatter
#endif

#if resolution != sfc_resolution 
	use remapping
#endif

        use tracer_names
	use fluxcoupler_fields
	use fluxcoupler_fields_sfc
#ifndef scm
        use qp_module
        use gmeans_module
#endif
        use pbp_module
        use eau_util
        use timer

#ifdef scm
        use scm_module
#endif
        implicit none
        save

	integer (kind=int_kind) :: i,j,icount,ii,panel, n

	integer (kind=int_kind), parameter :: 
     *		nvar_state = 14+2*(npbl+2),	! number of state variables to interpolate
     *		nvar_flux  = 17+3*npbl		! number of flux variables to average

	real (kind=dbl_kind), dimension(iim,jjm,nvar_state,nsdm) :: 
     *		atmos_state, ! atmos state variables packed into a single array
     *		atmos_gradx, ! atmos state variables x gradient
     *		atmos_grady ! atmos state variables y gradient
	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nvar_state,nsdm_sfc) :: 
     *		sfc_state    ! state variables interpolated to sfc grid

	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nvar_flux,nsdm_sfc) ::
     *		sfc_flux     ! flux values on sfc grid packed into single array
	real (kind=dbl_kind), dimension(iim,jjm,nvar_flux,nsdm) ::
     *		atmos_flux   ! flux valies interpolated onto atmos grid

        logical (kind=log_kind), dimension(nvar_state) ::
     *          lgrad_state,            ! compute a gradient for this state variable
     *          lminlim_terp,           ! impose a minimum on interpolation of this state variable
     *          lmaxlim_terp            ! impose a maximum on interpolation of this state variable

        real (kind=dbl_kind), dimension(nvar_state) ::
     *          minlim_terp,            ! minimum limit to impose for state var
     *          maxlim_terp             ! maximum limit to impose for state var

      real (kind = dbl_kind), dimension(iim,jjm)::
     *          dplg,                   ! pressure thickness  (hPa).
     *          work_cw,                ! cloud water path (kg/m^2).
     *          work_ci                 ! cloud ice path   (kg/m^2).

	real (kind = dbl_kind), dimension(iim,jjm,nsdm) ::
     *          sea_level_pressure

	integer (kind = int_kind) ::   iradvdc, iradndc, iradvbc, iradnbc,
     *		idlwbot,
     *		ifluxd, icupr, ilspr, ilsps, ispdm, ishear, ips, ipsb,
     *          itpgvar

	integer (kind = int_kind), dimension(npbl+2) :: itheta, ish

        integer (kind = int_kind) ::   ialvdf, ialndf, ialvdr, ialndr, itgeff4,
     *		ipc, icldpc, ifws, ifss, idpent, ico2flx

        integer (kind = int_kind), dimension(2) :: idtlci, idqlci, idrag

        integer (kind = int_kind), dimension(npbl) :: ilwp, ipblcw, ipblci

	private 

	public :: flux_coupler, init_flux_coupler

!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF FLUX_COUPLER
!======================================================================
        subroutine flux_coupler(init_fc)

        implicit none

        logical (kind=log_kind), intent(inout) :: init_fc

	logical (kind=log_kind) :: l_flux_coupler = .true.
	integer (kind=int_kind), parameter :: n_flux_coupler = 1
	integer (kind=int_kind) :: ksdm, i, j


#if resolution == sfc_resolution 

        call timer_start('surface')
	call sfc_control_driver(n_flux_coupler, init_fc)
        call timer_stop('surface')

#else

#ifndef scm
!---------------------------------------------------------------------------
! determine whether or not to do this computation
!---------------------------------------------------------------------------
        if(mod(step_count,n_flux_coupler).eq.0.or.init_fc) then
           l_flux_coupler = .true.
           if(my_task.eq.0) write(6,*) ' doing flux coupler calculations'
        else
           l_flux_coupler = .false.
        endif

!---------------------------------------------------------------------------
! only do calculation when l_flux_coupler is true
!---------------------------------------------------------------------------
	if(l_flux_coupler) then
        call timer_start('flux_coupler')

!---------------------------------------------------------------------------
! pack atmos variables into a single global array
! this entails a global_gather
! pack_atmos_buffer fills the array atmos_state_buffer
!---------------------------------------------------------------------------
	call pack_atmos_buffer

!---------------------------------------------------------------------------
! interpolate state vector to surface grid
! interpolate_to_sfc_grid fills the array sfc_state_buffer
!---------------------------------------------------------------------------
	call a_to_s_remapping(
     *		atmos_state(1,1,1,1),
     *		atmos_gradx(1,1,1,1),
     *		atmos_grady(1,1,1,1),
     *          lgrad_state(1),
     *		nvar_state,
     *		sfc_state(1,1,1,1) )

!---------------------------------------------------------------------------
! put representative values in sfc ghost cells
!---------------------------------------------------------------------------
	do ksdm = 1, nsdm_sfc
	   do j = 1, jjm_sfc
	      do i = 1, iim_sfc
                 if(.not.grid_mask_sfc(i,j,ksdm))
     *                sfc_state(i,j,:,ksdm) = sfc_state(2,2,:,ksdm)
              enddo
           enddo
        enddo
        call timer_stop('flux_coupler')

!---------------------------------------------------------------------------
! all processes execute the surface parameterization
! sfc_control_driver has the following tasks
!	1)  unpacks the data in sfc_state
!	2)  calls the surface parameterizations
!	3)  packs output into sfc_flux
!---------------------------------------------------------------------------
        call timer_start('surface')
	call sfc_control_driver(n_flux_coupler, init_fc)
        call timer_stop('surface')

!---------------------------------------------------------------------------
! interpolate flux vector to surface grid
! interpolate_to_atmos_grid fills the array atmos_flux_buffer
!---------------------------------------------------------------------------
        call timer_start('flux_coupler')
        call s_to_a_remapping(
     *		sfc_flux(1,1,1,1),
     *		nvar_flux,
     *		atmos_flux(1,1,1,1) )

!---------------------------------------------------------------------------
! unpack the atmos_flux_buffer into the appropriate arrays
! and scatter across the MPI_COMM_ATMOS communicator
!---------------------------------------------------------------------------
	call scatter_atmos_flux

!---------------------------------------------------------------------------
! end the if test for l_flux_coupler
!---------------------------------------------------------------------------
        call timer_stop('flux_coupler')
	endif
#endif
#endif		! resolution == sfc_resolution

!---------------------------------------------------------------------------
! add in the surface forcing to the source arrays
!---------------------------------------------------------------------------
        call timer_start('surface')
	call compute_surface_forcing

!---------------------------------------------------------------------------
! fill the qp data using _atm variables
!---------------------------------------------------------------------------
	if(.not.init_fc) call fill_qp(n_flux_coupler)

!---------------------------------------------------------------------------
! ad hoc sfc output
!---------------------------------------------------------------------------
!	if(my_task.eq.0.and.l_output.and..not.init_fc) then
!	  call sfc_fields
!	endif

!---------------------------------------------------------------------------
! set flag to indicate that the initialization is complete
!---------------------------------------------------------------------------
         init_fc = .false.
        call timer_stop('surface')

	end subroutine flux_coupler
!===========================================================================
! END FLUX_COUPLER
!===========================================================================
	

!======================================================================
! BEGINNING OF INIT_FLUX_COUPLER
!======================================================================
        subroutine init_flux_coupler

  	implicit none

	integer (kind=int_kind) :: sd_np, sd_sp, n
	real (kind=real_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) ::
     *        landfrac_sfc, icefrac_sfc, seafrac_sfc

!--------------------------------------------------------------------------
! do special initialization when atmosphere and surface grids differ
!--------------------------------------------------------------------------
#if resolution != sfc_resolution

!--------------------------------------------------------------------------
! allocate global arrays for just a single processor
!-------------------------------------------------------------------------
        atmos_state = c0
        sfc_state = c0 
        atmos_flux = c0
        sfc_flux = c0

!--------------------------------------------------------------------------
! initialize the variable tags for the flux coupler
!--------------------------------------------------------------------------
	call init_variable_names

!--------------------------------------------------------------------------
! read in remapping weights
!--------------------------------------------------------------------------
        call init_remapping

#endif	! if resolution != sfc_resolution

#if resolution == sfc_resolution
#ifdef scm
      grid_mask_sfc = .true.
      grid_indx_sfc = 1.0_dbl_kind
#else
      grid_mask_sfc = grid_center_mask==1.0_dbl_kind
      grid_indx_sfc = grid_center_index
      grid_center_lon_sfc = grid_center_lon
      grid_center_lat_sfc = grid_center_lat
      grid_corner_lon_sfc = grid_corner_lon
      grid_corner_lat_sfc = grid_corner_lat
#endif
#else
      grid_mask_sfc = .false.
      sd_np = isdm*(  jsdm-1)+1 ! SUBDOMAIN NUMBER OF THE NORTH POLE
      sd_sp = isdm*(4*jsdm+1)   ! SUBDOMAIN NUMBER OF THE SOUTH POLE
      grid_mask_sfc(2:iim_sfc-1,2:jjm_sfc-1,:) = .true.
      DO n = 1,nsdm_sfc
         IF (sbdmn_assign(my_task+1,sfc_level)%sd(n).EQ.sd_np) THEN
           grid_mask_sfc(1,jjm_sfc-1,n) = .true.
         ENDIF
         IF (sbdmn_assign(my_task+1,sfc_level)%sd(n).EQ.sd_sp) THEN
           grid_mask_sfc(iim_sfc-1,1,n) = .true.
         ENDIF
      ENDDO
#endif


!--------------------------------------------------------------------------
! get initial conditions for "surface" components
!--------------------------------------------------------------------------
	call init_sfc_variables
#ifdef scm
	  select case (scm_gridpt_type)
        case (1)
          call init_sib
        case (2)
          call init_ocnflx
        case (3)
          call init_landice
      end select
#else
        call init_ocnflx(seafrac_sfc)
        call init_landice(icefrac_sfc)
        call init_sib(landfrac_sfc)
#endif

        call init_coupler
    
#if resolution == sfc_resolution
#ifndef scm
        landfrac(:,:,:) = landfrac_sfc(:,:,:)
        seafrac(:,:,:) = seafrac_sfc(:,:,:)
        icefrac(:,:,:) = icefrac_sfc(:,:,:)
#endif
#else
! dummy values until implemented properly for the atmos ne sfc case
      seafrac = 1.0
      landfrac = 0.0
      icefrac = 0.0
#endif

      end subroutine init_flux_coupler
!======================================================================
! END OF INIT_FLUX_COUPLER
!======================================================================



!======================================================================
! BEGINNING OF SFC_CONTROL_DRIVER
!======================================================================
      subroutine sfc_control_driver(n_flux_coupler, init_fc)

!-----------------------------------------------------------------------
! PURPOSE: wrapper to drive sfc_control. the two cases: 
!	atmos_eq_sfc = .true. and atmos_eq_sfc = .false. are accounted for
!	explicitly
!-----------------------------------------------------------------------
      
      implicit none

      integer (kind=int_kind), intent(in) :: n_flux_coupler
      logical (kind=log_kind), intent(in) :: init_fc

      real (kind = dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) :: tmp

!--------------------------------------------------------------------------
! depending on whether or not atmos_eq_sfc is true or not, fill the _sfc
!	arrays differently
!--------------------------------------------------------------------------
#if resolution == sfc_resolution

!--------------------------------------------------------------------------
!  derive some variables required to drive the surface parameterizations
!  note the conversion to MB
!--------------------------------------------------------------------------
      psb_atm(:,:,:)  = ( ps(:,:,n3,:) - pb(:,:,n3,:) ) * 0.01
      t_atm(:,:,:,:)  = th(:,:,nlm-(npbl+1):nlm,n3,:)
      sh_atm(:,:,:,:) = tracer(:,:,nlm-(npbl+1):nlm,iq,n3,:)

!--------------------------------------------------------------------------
!   copy from atm to sfc
!--------------------------------------------------------------------------
       ps_sfc(:,:,:)     = plgl2(:,:,nlm+1,:) * 0.01    ! note conversion to MB
       psb_sfc(:,:,:)    = psb_atm(:,:,:)
       radvdc_sfc(:,:,:) = radvdc_atm(:,:,:)
       radndc_sfc(:,:,:) = radndc_atm(:,:,:)
       radvbc_sfc(:,:,:) = radvbc_atm(:,:,:)
       radnbc_sfc(:,:,:) = radnbc_atm(:,:,:)
       dlwbot_sfc(:,:,:) = dlwbot_atm(:,:,:)
       fluxd_sfc(:,:,:)  = fluxd_atm(:,:,:)
       dcupr_sfc(:,:,:)  = dcupr_atm(:,:,:)
       dlspr_sfc(:,:,:)  = dlspr_atm(:,:,:)
       dlsps_sfc(:,:,:)  = dlsps_atm(:,:,:)
       spdm_sfc(:,:,:)   = spdm_atm(:,:,:)
       shear1_sfc(:,:,:) = shear1_atm(:,:,:)
       tpgvar_sfc(:,:,:) = tpgvar_atm(:,:,:)
       t_sfc(:,:,:,:)    = t_atm(:,:,:,:)
       sh_sfc(:,:,:,:)   = sh_atm(:,:,:,:)

#else

!--------------------------------------------------------------------------
!   fill from buffer 
!--------------------------------------------------------------------------
	 ps_sfc     (:,:,:)    = sfc_state (:,:, ips     , :)
         psb_sfc    (:,:,:)    = sfc_state (:,:, ipsb    , :)
         radvdc_sfc (:,:,:)    = sfc_state (:,:, iradvdc , :)
         radndc_sfc (:,:,:)    = sfc_state (:,:, iradndc , :)
         radvbc_sfc (:,:,:)    = sfc_state (:,:, iradvbc , :)
         radnbc_sfc (:,:,:)    = sfc_state (:,:, iradnbc , :)
         dlwbot_sfc (:,:,:)    = sfc_state (:,:, idlwbot , :)
         fluxd_sfc  (:,:,:)    = sfc_state (:,:, ifluxd  , :)
         dcupr_sfc  (:,:,:)    = sfc_state (:,:, icupr   , :)
         dlspr_sfc  (:,:,:)    = sfc_state (:,:, ilspr   , :)
         dlsps_sfc  (:,:,:)    = sfc_state (:,:, ilsps   , :)
         spdm_sfc   (:,:,:)    = sfc_state (:,:, ispdm   , :)
         shear1_sfc (:,:,:)    = sfc_state (:,:, ishear  , :)
         tpgvar_sfc (:,:,:)    = sfc_state (:,:, itpgvar , :)
	 
         do j=1,npbl+2
           t_sfc(:,:,j,:)  = sfc_state (:,:, itheta(j), :)
           sh_sfc(:,:,j,:) = sfc_state (:,:, ish(j), :)
         enddo
	

!--------------------------------------------------------------------------
!  the variable, ps_sfc, presently holds the sea level pressure
!	compute the surface pressure, note converstion to MB
!--------------------------------------------------------------------------
 	do ksdm=1,nsdm_sfc
           call invCSLP(ps_sfc(1,1,ksdm), tmp(1,1,ksdm)
     *,                 t_sfc(1,1,2+npbl,ksdm), zs_sfc(1,1,ksdm), iimjjm_sfc )
           ps_sfc(:,:,ksdm) = tmp(:,:,ksdm)*0.01
 	enddo

#endif

!--------------------------------------------------------------------------
!  loop over each sub-domain and call the surface parameterization
!--------------------------------------------------------------------------
      do ksdm = 1,nsdm_sfc
       call sfc_control( init_fc, dt, ksdm, n_flux_coupler )
      enddo

!--------------------------------------------------------------------------
! depending on whether or not atmos_eq_sfc is true , fill the _atm
!	arrays differently
!--------------------------------------------------------------------------
#if resolution == sfc_resolution

!--------------------------------------------------------------------------
!   copy from sfc to atm
!--------------------------------------------------------------------------
        alvdf_atm  (:,:,:)   = alvdf_sfc  (:,:,:)
        alndf_atm  (:,:,:)   = alndf_sfc  (:,:,:)
        alvdr_atm  (:,:,:)   = alvdr_sfc  (:,:,:)
        alndr_atm  (:,:,:)   = alndr_sfc  (:,:,:)
        tgeff4_atm (:,:,:)   = tgeff4_sfc (:,:,:)
        pc_atm     (:,:,:)   = pc_sfc     (:,:,:)
        cldpc_atm  (:,:,:)   = cldpc_sfc  (:,:,:)
        fws_atm    (:,:,:)   = fws_sfc    (:,:,:)
        fss_atm    (:,:,:)   = fss_sfc    (:,:,:) 
        dpent_atm  (:,:,:)   = dpent_sfc  (:,:,:)
        co2flx_atm (:,:,:)   = co2flx_sfc (:,:,:)
        lwp_atm    (:,:,:,:) = lwp_sfc    (:,:,:,:)
        pblcw_atm  (:,:,:,:) = pblcw_sfc  (:,:,:,:)
        pblci_atm  (:,:,:,:) = pblci_sfc  (:,:,:,:)
        dtlci_atm  (:,:,:,:) = dtlci_sfc  (:,:,:,:)
        dqlci_atm  (:,:,:,:) = dqlci_sfc  (:,:,:,:)
        drag_atm   (:,:,:,:) = drag_sfc   (:,:,:,:)
	
#else

!--------------------------------------------------------------------------
!   fill the sfc_flux buffer
!--------------------------------------------------------------------------
       
	  sfc_flux (:,:,ialvdf  ,:) = alvdf_sfc  (:,:,:)
          sfc_flux (:,:,ialndf  ,:) = alndf_sfc  (:,:,:)
          sfc_flux (:,:,ialvdr  ,:) = alvdr_sfc  (:,:,:)
          sfc_flux (:,:,ialndr  ,:) = alndr_sfc  (:,:,:)
          sfc_flux (:,:,itgeff4 ,:) = tgeff4_sfc (:,:,:)
	  sfc_flux (:,:,ipc     ,:) = pc_sfc     (:,:,:)
          sfc_flux (:,:,icldpc  ,:) = cldpc_sfc  (:,:,:)
          sfc_flux (:,:,ifws    ,:) = fws_sfc    (:,:,:)
          sfc_flux (:,:,ifss    ,:) = fss_sfc    (:,:,:)
          sfc_flux (:,:,idpent  ,:) = dpent_sfc  (:,:,:)
          sfc_flux (:,:,ico2flx ,:) = co2flx_sfc (:,:,:)
	  do j=1,npbl
            sfc_flux (:,:,ilwp(j) ,:) = lwp_sfc  (:,:,j,:)
            sfc_flux (:,:,ipblcw(j) ,:) = pblcw_sfc(:,:,j,:)
            sfc_flux (:,:,ipblci(j) ,:) = pblcw_sfc(:,:,j,:)
	  enddo
	  do j=1,2
            sfc_flux (:,:,idtlci(j) ,:) = dtlci_sfc  (:,:,j,:)
            sfc_flux (:,:,idqlci(j) ,:) = dqlci_sfc  (:,:,j,:)
            sfc_flux (:,:,idrag(j)  ,:) = drag_sfc   (:,:,j,:)
	  enddo



#endif

      end subroutine sfc_control_driver
!======================================================================
! END OF SFC_CONTROL_DRIVER
!======================================================================


!======================================================================
! BEGINNING OF INIT_VARIABLE_NAMES
!======================================================================
	subroutine init_variable_names

	implicit none
	integer (kind=int_kind) :: ivar

!-------------------------------------------------------------------------
! PURPOSE : assign an integer value to each variable in order to make the
!	mapping more robust
!-------------------------------------------------------------------------

!-------------------------------------------------------------------------
! 2D state variables
!-------------------------------------------------------------------------
	iradvdc = 1	! shortwave rad (???)
	iradndc = 2	! shortwave rad (???)
	iradvbc = 3	! shortwave rad (???)
	iradnbc = 4	! shortwave rad (???)
	idlwbot = 5	! downward long wave @ surface (???)
	ifluxd  = 6	! downward long wave @ pbl top (???)
	icupr   = 7	! convective precip (???)
	ilspr   = 8	! eauliq rain (???)
	ilsps   = 9	! eauliq snow (???)
	ispdm   = 10	! wind speed (m/s)
	ishear  = 11    ! pbl top wind shear (???)
	ips     = 12	! surfuce pressure (Pa)
	ipsb	= 13	! pbl depth (Pa)
	itpgvar = 14    ! topo variance (???)

!-------------------------------------------------------------------------
! 3D state variables
!-------------------------------------------------------------------------
	ivar = 14
	do i = 1,npbl+2
	  ivar = ivar + 1
	  itheta(i) = ivar 	! temperature (???)
	enddo
	do i = 1,npbl+2
	  ivar = ivar + 1
	  ish(i) = ivar			! specific humidity (???)
	enddo

!-------------------------------------------------------------------------
! sanity check
!-------------------------------------------------------------------------
	if(ivar.ne.nvar_state) then
	  write(6,*) ' state variable are not matching nvar_state'
	  stop
	endif
	
!-------------------------------------------------------------------------
! 2D flux variables
!-------------------------------------------------------------------------
	ialvdf  = 1    ! albedo (???)
	ialndf  = 2    ! albedo (???)
	ialvdr  = 3    ! albedo (???)
	ialndr  = 4    ! albedo (???)
	itgeff4 = 5    ! emitting temperature (???)
	ipc     = 6    ! stratus thickness (???)
	icldpc  = 7    ! stratus fraction (???)
	ifws    = 8    ! latent heat (???)
	ifss    = 9    ! sensible heat (???)
	idpent  = 10   ! entrainment (???)
	ico2flx = 11   ! co2 flux (???)

!-------------------------------------------------------------------------
! 3D flux variables
!-------------------------------------------------------------------------
	ivar = 11
	do i = 1,2
	  ivar = ivar + 1
	  idqlci(i) = ivar	! lci moistening (???)
	enddo
	do i = 1,2
          ivar = ivar + 1
          idtlci(i) = ivar	! lci heating (???)
        enddo
	do i=1,2
          ivar = ivar + 1
	  idrag(i) = ivar	! sfc drag coefficient (???)
	enddo
	do i=1,npbl
	  ivar = ivar + 1
	  ilwp(i) = ivar	! liquid water path
	enddo
	do i=1,npbl
	   ivar = ivar + 1
	   ipblcw(i) = ivar ! pbl cloud liquid water mixing ratio
	enddo
	do i=1,npbl
	   ivar = ivar + 1
	   ipblci(i) = ivar ! pbl cloud ice water mixing ratio
	enddo
	

!-------------------------------------------------------------------------
! sanity check
!-------------------------------------------------------------------------
        if(ivar.ne.nvar_flux) then
          write(6,*) ' flux variable are not matching nvar_flux'
          stop
        endif

!-------------------------------------------------------------------------
! determine which variables will be 2nd order interpolated
!-------------------------------------------------------------------------
        lgrad_state(:) = .true.
        lminlim_terp(:) = .true.
        lmaxlim_terp(:) = .false.
        minlim_terp(:) = c0
        maxlim_terp(:) = c0

! non-default values set here, for psb, compare with values in step.F
        minlim_terp(ipsb) = 10.0
        lmaxlim_terp(ipsb) = .true.
        maxlim_terp(ipsb) = 160.0
        lminlim_terp(ips) = .false.
        do i = 1,npbl+2
          lminlim_terp(itheta(i)) = .false.
        enddo

	end subroutine init_variable_names
!======================================================================
! END OF INIT_VARIABLE_NAMES
!======================================================================

#if resolution != sfc_resolution 
!======================================================================
! BEGINNING OF PACK_ATMOS_BUFFER
!======================================================================
	subroutine pack_atmos_buffer

!-------------------------------------------------------------------------
! PURPOSE : back atmos data into a single buffer for interpolation
!-------------------------------------------------------------------------

	implicit none
	integer (kind=int_kind) :: ivar
        real (kind=dbl_kind), dimension(iim,jjm,2*nvar_state,nsdm) :: tmp_grad

!--------------------------------------------------------------------------
!  derive some variables required to drive the surface parameterizations
!  note the conversion to MB
!--------------------------------------------------------------------------
      psb_atm(:,:,:) = ( ps(:,:,n3,:) - pb(:,:,n3,:) ) * 0.01
      t_atm(:,:,:,:) = th(:,:,nlm-(npbl+1):nlm,n3,:)
      sh_atm(:,:,:,:) = tracer(:,:,nlm-(npbl+1):nlm,iq,n3,:)

!--------------------------------------------------------------------------
! compute sea level pressure on the atmosphere grid
!--------------------------------------------------------------------------
        do ksdm = 1,nsdm
           call cslp( sea_level_pressure(1,1,ksdm), plgl2(1,1,nlm+1,ksdm),
     *                th(1,1,nlm,n3,ksdm), phis(1,1,ksdm), iim*jjm )
        enddo


	atmos_state(:,:,iradvdc,:) = radvdc_atm(:,:,:)
        atmos_state(:,:,iradndc,:) = radndc_atm(:,:,:)
        atmos_state(:,:,iradvbc,:) = radvbc_atm(:,:,:)
        atmos_state(:,:,iradnbc,:) = radnbc_atm(:,:,:)
        atmos_state(:,:,idlwbot,:) = dlwbot_atm(:,:,:)
        atmos_state(:,:,ifluxd ,:) = fluxd_atm (:,:,:)
        atmos_state(:,:,icupr  ,:) = dcupr_atm (:,:,:)
        atmos_state(:,:,ilspr  ,:) = dlspr_atm (:,:,:)
	atmos_state(:,:,ilsps  ,:) = dlsps_atm (:,:,:)
        atmos_state(:,:,ispdm  ,:) = spdm_atm  (:,:,:)
        atmos_state(:,:,ishear ,:) = shear1_atm(:,:,:)
        atmos_state(:,:,ips    ,:) = sea_level_pressure(:,:,:)
	atmos_state(:,:,ipsb   ,:) = psb_atm   (:,:,:)
	atmos_state(:,:,itpgvar,:) = tpgvar_atm(:,:,:)
        
	do i = 1,npbl+2
	  atmos_state(:,:,itheta(i),:) = t_atm (:,:,i,:)
	  atmos_state(:,:,ish(i)   ,:) = sh_atm(:,:,i,:)
	enddo

	call wrap(
     *		max_level,
     *		iim,
     *		jjm,
     *		nvar_state,
     *		nsdm,
     *		atmos_state)

        atmos_gradx(:,:,:,:) = c0
        atmos_grady(:,:,:,:) = c0
        ivar = 0
        do n = 1,nvar_state
           if(lgrad_state(n)) then
              ivar = ivar + 2
              tmp_grad(:,:,ivar-1:ivar,:) = 0.0
              do ksdm = 1,nsdm
                 call compute_interp_gradient(
     *                   atmos_state(:,:,n,ksdm),
     *                   tmp_grad(:,:,ivar-1,ksdm),
     *                   tmp_grad(:,:,ivar,ksdm),
     *                   lminlim_terp(n),
     *                   lmaxlim_terp(n),
     *                   minlim_terp(n),
     *                   maxlim_terp(n),
     *                   ksdm   )
              enddo
           endif
        enddo

	if(ivar.ne.0) then
!  zero gradients at poles
           tmp_grad(1,jjm-1,:,:) = c0
           tmp_grad(iim-1,1,:,:) = c0
      	   call wrap(
     *		max_level,
     *		iim,
     *		jjm,
     *		ivar,
     *		nsdm,
     *		tmp_grad)

        ivar = 0
           do n = 1,nvar_state
              if(lgrad_state(n)) then
                 ivar = ivar + 2
                 atmos_gradx(:,:,n,:) = tmp_grad(:,:,ivar-1,:)
                 atmos_grady(:,:,n,:) = tmp_grad(:,:,ivar,:)
              endif
           enddo
        endif

	end subroutine pack_atmos_buffer
!======================================================================
! END OF PACK_ATMOS_BUFFER
!======================================================================

     
!======================================================================
! BEGINNING OF SCATTER_ATMOS_FLUX
!======================================================================
        subroutine scatter_atmos_flux

!-------------------------------------------------------------------------
! PURPOSE : scatter atmos_flux_buffer across all processes and
!	unpack into individual arrays
!-------------------------------------------------------------------------

	implicit none
        integer (kind=int_kind) :: ivar


        alvdf_atm  (:,:,:)   = atmos_flux(:,:,ialvdf,:)
        alndf_atm  (:,:,:)   = atmos_flux(:,:,ialndf,:)
        alvdr_atm  (:,:,:)   = atmos_flux(:,:,ialvdr,:)
        alndr_atm  (:,:,:)   = atmos_flux(:,:,ialndr,:)
        tgeff4_atm (:,:,:)   = atmos_flux(:,:,itgeff4,:)
        pc_atm     (:,:,:)   = atmos_flux(:,:,ipc,:)
        cldpc_atm  (:,:,:)   = atmos_flux(:,:,icldpc,:)
        fws_atm    (:,:,:)   = atmos_flux(:,:,ifws,:)
        fss_atm    (:,:,:)   = atmos_flux(:,:,ifss,:)
        dpent_atm  (:,:,:)   = atmos_flux(:,:,idpent,:)
	co2flx_atm (:,:,:)   = atmos_flux(:,:,ico2flx,:)
        do i = 1,2
	  dqlci_atm(:,:,i,:) = atmos_flux (:,:, idqlci(i) ,:)	
	  dtlci_atm(:,:,i,:) = atmos_flux (:,:, idtlci(i) ,:)
	  drag_atm (:,:,i,:) = atmos_flux (:,:, idrag(i)  ,:)
	enddo
	do i=1,npbl
	  lwp_atm  (:,:,i,:) = atmos_flux (:,:, ilwp(i)   ,:)
	  pblcw_atm(:,:,i,:) = atmos_flux (:,:, ipblcw(i) ,:)
	  pblci_atm(:,:,i,:) = atmos_flux (:,:, ipblci(i) ,:)
	enddo

	end subroutine scatter_atmos_flux
!======================================================================
! END OF SCATTER_ATMOS_FLUX
!======================================================================
#endif

!======================================================================
! BEGINNING OF FILL_QP
!======================================================================
	subroutine fill_qp(n_flux_coupler)

	implicit none
        integer (kind=int_kind), intent(in) :: n_flux_coupler
        integer (kind=int_kind):: k
        real (kind=dbl_kind), dimension(iim,jjm) :: qtem
        real (kind=dbl_kind) :: x_flux_coupler

        x_flux_coupler = float(n_flux_coupler)

!--------------------------------------------------------------------------
!  loop over atmosphere sub-domains
!--------------------------------------------------------------------------
        do ksdm = 1,nsdm

        qtem(:,:) = tgeff4_atm(:,:,ksdm) **0.25
#ifndef scm
        call qp2sum(qtem(1,1), c1, iqp2_ts, ksdm, iim*jjm)
        call qp2sum(fss_atm(1,1,ksdm), c1, iqp2_hfss, ksdm, iim*jjm)
        call qp2sum(fws_atm(1,1,ksdm), hltm, iqp2_hfls, ksdm, iim*jjm)
        call qp2sum(fws_atm(1,1,ksdm), c1, iqp2_evspsbl, ksdm, iim*jjm)
        call qp2sum(dpent_atm(1,1,ksdm), c1, iqp2_blent, ksdm, iim*jjm)
        call qp2sum(pc_atm(1,1,ksdm),100.0_dbl_kind,iqp2_clblp,ksdm,iim*jjm)
        call qp2sum(cldpc_atm(1,1,ksdm), c1, iqp2_clbl, ksdm, iim*jjm)

        call hf2sum(fss_atm(1,1,ksdm), c1, ihf2_hfss, ksdm, iim*jjm)
        call hf2sum(fws_atm(1,1,ksdm), hltm, ihf2_hfls, ksdm, iim*jjm)


        work_cw(:,:) = 0.0_dbl_kind
        work_ci(:,:) = 0.0_dbl_kind
        do k = 1, npbl
           dplg(:,:) = plgl2(:,:,lpbl+k+1,ksdm)-plgl2(:,:,lpbl+k,ksdm)
           work_cw(:,:) = work_cw(:,:)
     *                  + pblcw_atm(:,:,k,ksdm)*dplg(:,:)*gravi
           work_ci(:,:) = work_ci(:,:)
     *                  + pblci_atm(:,:,k,ksdm)*dplg(:,:)*gravi
        enddo
        call qp2sum(work_cw(1,1),c1,iqp2_pblcw,ksdm,iim*jjm)
        call qp2sum(work_ci(1,1),c1,iqp2_pblci,ksdm,iim*jjm)

        dplg(:,:) = plgl2(:,:,nlm+1,ksdm)-plgl2(:,:,nlm,ksdm)
        work_cw(:,:) = grav*fws_atm(:,:,ksdm)/dplg(:,:)
        call qp2sum(work_cw(1,1), c1, iqp2_evaptend, ksdm, iim*jjm)

        call gmean_sum_atm(fss_atm(1,1,ksdm), iim*jjm, -x_flux_coupler,
     *		 iglob_hfns, ksdm, step_sample=0 )
        call gmean_sum_atm(fws_atm(1,1,ksdm), iim*jjm, -x_flux_coupler*hltm,
     *		 iglob_hfns, ksdm, step_sample=0)

        call gmean_sum_atm(fws_atm(1,1,ksdm), iim*jjm, c1,
     *		 iglob_evsps, ksdm)
#endif

        call pbp1sum(qtem(1,1), x_flux_coupler, ipbp1_tradgrdeff, ksdm, iim*jjm)
        call pbp1sum(fss_atm(1,1,ksdm), x_flux_coupler, ipbp1_hfss, 
     *		ksdm, iim*jjm)
        call pbp1sum(fws_atm(1,1,ksdm), hltm*x_flux_coupler, ipbp1_hfls, 
     *		ksdm, iim*jjm)
        call pbp1sum(dpent_atm(1,1,ksdm), x_flux_coupler, ipbp1_blent, 
     *		ksdm, iim*jjm)
        call pbp1sum(pc_atm(1,1,ksdm),100.0_dbl_kind*x_flux_coupler,
     *		ipbp1_clblp,ksdm,iim*jjm)
        call pbp1sum(cldpc_atm(1,1,ksdm), x_flux_coupler, ipbp1_clbl,
     *		 ksdm, iim*jjm)
      
      enddo
!--------------------------------------------------------------------------
!  end loop over atmosphere sub-domains
!--------------------------------------------------------------------------

	end subroutine fill_qp
!======================================================================
! END OF FILL_QP
!======================================================================


!======================================================================
! BEGINNING OF COMPUTE_SURFACE_FORCING
!======================================================================
	subroutine compute_surface_forcing

        use sib_parameters
	implicit none
!--------------------------------------------------------------------------
!  loop over sub-domains (on atmospheric grid) and compute
! 	the source of theta and water.
!--------------------------------------------------------------------------
        do ksdm = 1,nsdm
          th_src(:,:,nlm,ksdm) = th_src(:,:,nlm,ksdm) + fss_atm(:,:,ksdm)
     *       * grav / ( spec_heat_cp * bpgl2(:,:,nlm+1,ksdm) )
          tracer_src(:,:,nlm,iq,ksdm) = tracer_src(:,:,nlm,iq,ksdm)
     *       + fws_atm(:,:,ksdm)*grav 
          drag(:,:,ksdm) = drag_atm(:,:,1,ksdm)
          entrainment(:,:,ksdm) = dpent_atm(:,:,ksdm)
#ifdef passive_tracers
          if(doSibCO2)
     +       tracer_flux(:,:,npassive,ksdm) = co2flx_atm(:,:,ksdm)
#endif
        enddo
!--------------------------------------------------------------------------
!  end loop over atmosphere sub-domains
!--------------------------------------------------------------------------

	end subroutine compute_surface_forcing
!======================================================================
! END OF COMPUTE_SURFACE_FORCING
!======================================================================


! this routine needs to be modified for the parallel flux coupler
!	subroutine sfc_fields
!
!	real (kind=real_kind) :: tmp(nsfc_tot)
!	real (kind=dbl_kind) :: ps(nsfc_tot), t(nsfc_tot), qs(nsfc_tot)
!
!	write(6,*) ' dumping data to unit 57',nsfc_tot
!	do i=1,nvar_state
!	  tmp(1:nsfc_tot) = sfc_state_buffer(i,1:nsfc_tot)
!	  write(57) tmp
!	enddo
!	do i=1,nvar_flux
!	  tmp(1:nsfc_tot) = sfc_flux_buffer(i,1:nsfc_tot)
!	  write(57) tmp
!	enddo
!
!	ps(1:nsfc_tot) = sfc_state_buffer(ips,1:nsfc_tot)
!	t(1:nsfc_tot) = (sfc_flux_buffer(itgeff4,1:nsfc_tot))**0.25
!	write(6,*) minval(ps),maxval(ps),minval(t),maxval(t)
!	call eau_sat(nsfc_tot,ps,t,qsats=qs)
!	tmp = qs
!	write(6,*) minval(qs),maxval(qs)
!	write(57) tmp
!
!	end subroutine sfc_fields

#if resolution != sfc_resolution 
!======================================================================
! BEGINNING OF COMPUTE_INTERP_GRADIENT
!======================================================================
        subroutine compute_interp_gradient(
     *                   x,
     *                   gradx,
     *                   grady,
     *                   lminlim,
     *                   lmaxlim,
     *                   xminlim,
     *                   xmaxlim,
     *                   ksdm  )

!----------------------------------------------------------------------------
! This subroutine computes the gradient of a state variable for the 
!   purpose of second-order accurate interpolation in the flux_coupler.
!   Limits to the value of the variable on the surface grid may be imposed.
!----------------------------------------------------------------------------

	use grid_metrics
	implicit none

!-----------------------------------------------------------------------
! VARIABLE DESCRIPTION
!       x             : input field
!       gradx         : zonal derivative 
!       grady         : meridional derivative 
!       lminlim       : if true, adjust gradient to impose a minimum limit
!       lmaxlim       : if true, adjust gradient to impose a maximum limit
!       xminlim       : minimum limit to be imposed
!       xmaxlim       : maximum limit to be imposed
!-----------------------------------------------------------------------

!  argument list
	real (kind=dbl_kind), dimension(iim,jjm), intent(in) ::
     *                        x
	real (kind=dbl_kind), dimension(iim,jjm), intent(out) :: 
     *                        gradx,
     *                        grady
        logical (kind=log_kind), intent(in) :: lminlim, lmaxlim
	real (kind=dbl_kind), intent(in) :: xminlim, xmaxlim
	integer (kind=int_kind), intent(in) :: ksdm

!  local variables
	real (kind=dbl_kind), dimension(6,iim,jjm) :: work_n, work_t
        integer (kind=int_kind) :: i, j, l
	real (kind=dbl_kind) :: distmax, gmax, gdif, fac

!----------------------------------------------------------------------------
! compute derivatives normal and tangential to cell walls
!----------------------------------------------------------------------------

!tdrmod
!	call NORMAL_DERIVATIVE(
!    *			x,
!    *			inv_p_dist,
!    *			inv_c_dist,
!    *			work_n)
!call TANGENTIAL_DERIVATIVE(
!    *			x,
!    *			inv_p_dist,
!    *			inv_c_dist,
!    *			work_t)
!       call vect_rot_and_avg_to_cell_center
!    *        ( work_n(:,:,:), work_t(:,:,:),
!    *          gradx(:,:), grady(:,:),
!    *          c_dist(:,:,:,ksdm),
!    *          cos_rot_angle(:,:,:,ksdm), sin_rot_angle(:,:,:,ksdm) )
!endtdrmod

!----------------------------------------------------------------------------
! impose limits
!----------------------------------------------------------------------------
        if(lminlim.or.lmaxlim) then
           do j = 1,jjm
              do i = 1,iim
                 distmax = 0.0
                 do l = 1,6
!tdrmod
!                   distmax = max(distmax,
!    *                  p_dist(l,i,j,ksdm)**2+0.25*c_dist(l,i,j,ksdm)**2) 
!endtdrmod
                 enddo
                  gmax = sqrt( distmax*(gradx(i,j)**2+grady(i,j)**2) )
                 if(lminlim) then
                    gdif = max(0._dbl_kind, x(i,j)-xminlim )
                    if(gmax.gt.gdif) then
                       fac = gdif/gmax
                       gradx(i,j) = gradx(i,j) * fac
                       grady(i,j) = grady(i,j) * fac
                    endif
                 endif
                 if(lmaxlim) then
                    gdif = max(0._dbl_kind, xmaxlim-x(i,j) )
                    if(gmax.gt.gdif) then
                       fac = gdif/gmax
                       gradx(i,j) = gradx(i,j) * fac
                       grady(i,j) = grady(i,j) * fac
                    endif
                 endif
              enddo
           enddo
	endif

!----------------------------------------------------------------------------
! convert to per radian
!----------------------------------------------------------------------------
        grady(:,:) = grady(:,:) * a
        gradx(:,:) = gradx(:,:) * a * cos(grid_center_lat(:,:,ksdm))

        end subroutine compute_interp_gradient
!======================================================================
! END OF COMPUTE_INTERP_GRADIENT
!======================================================================
#endif
	
        end module flux_coupler_module
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
