c-------------------------------------------------------------------

      SUBROUTINE LWRAD (grav2, cp, NOZ, CLIZON, PL, PL2, TL, TG, TSA,
     *   TBMN, QL, O3L, O3TOP, CLD, CLU, ATL, ATLCLR, RS,
     *   fluxd_pb, ULWBOT,
     *   DLWBOT, ULWTOP, RSCLR, ULTCLR, co2fac, emitmax, emitrnd,
     *   olrtom, olrctom, nlen, nlm, lpbl, nls, len, nnp, nocld )

      use kinds

      IMPLICIT none

c-------------------------------------------------------------------
C     LONG-WAVE RADIATION PARAMETERIZATION DEVELOPED BY HARSHVARDHAN,
C     1983.

c     REFERENCES: Harshvardhan, R. Davies, D. A. Randall and T. G. Corsetti
c        (1987) "A fast radiation parameterization for general circulation
c        models", J. Geophys. Res., 92, 1009-1016.

C        "BROADBAND WATER VAPOR TRANSMISSION FUNCTIONS FOR ATMOSPHERIC
C        I.R. FLUX COMPUTATIONS", MING-DAH CHOU ; SUBMITTED TO JAS
C        (1983).

C        "A PARAMETERIZATION OF THE ABSORPTION IN THE 15 MICRON CO2
C        SPECTRAL REGION WITH APPLICATION TO CLIMATE SENSITIVITY
C        STUDIES", MING-DAH CHOU AND LI PENG ; TO APPEAR IN JAS (1983).

C        "SOME EXTENSIONS AND APPLICATIONS OF THE NEW RANDOM MODEL FOR
C        MOLECULAR BAND TRANSMISSION", C.D. ROGERS ; QJRMS, 94, 99-102
C        (1968).

c     MODIFICATIONS:
c		- took out real decl of CON1, never referenced.  kwitt 7/3/91
c		- inlined gcmlib calls.  kwitt 1/27/92
c		- following vars have been replaced by their parameter:
c		  IM LM LMP1 NP NPP1 LS IMP2 IMNP IMNPP1 IMLM LS1 IMT2
c		  kwitt 1/30/92
c		- replaced 'half precision' variables with their value.
c		  kwitt 1/30/92

c     SUBROUTINES CALLED:
c		lwflux, cldslw
c     FUNCTIONS CALLED:
c		none

C     ARGUMENT LIST VARIABLES
C        Input variables: 
C		grav2	grav / 100
C		cp	dry air specific heat
c		noz	no ozone logical
c               nocld   clouds are invisible
c		pl	mid-layer pressures (mb)
c		pl2	layer edge pressures (mb)
c		tl	mid-layer temperatures (K)
c		tg	surface boundary temperature (K)
c		tsa	surface air temperature (K)
c		tbmn	temperature at the top of the PBL stratus clouds (K)
c		ql	mid-layer specific humidity (Kg/Kg)
c		o3l	mid-layer ozone mixing ratios (Kg/Kg)
c		cld	supersaturation cloud amount
c		clu	convective anvil cloud amount
C        Output variables: 
C		atl	atmospheric heating rate (K/time-step)
c		atlclr	clear sky atmospheric heating rate (K/time-step)
c		rs	net upward surface longwave radiative flux(w/m**2)
c		fluxd_pb	downward radiative flux at the
c			top of a PBL stratus layer (w/m**2)
c		ulwtop	top of atmosphere outgoing longwave radiative
c			flux (w/m**2)
c		dlwbot	downward component of longwave radiative flux
c			at the surface (w/m**2)
c		rsclr	clear sky net upward surface longwave
c			radiative flux (w/m**2)
c		ultclr	clear sky ulwtop (w/m**2)
      integer (kind=int_kind) ::
     &   nlen, nlm, lpbl, len, nls, nnp
      REAL (kind=dbl_kind) ::
     &   PL(nlen,NLM), PL2(nlen,NLM+1), TL(nlen,NLM),
     *   TG(nlen), QL(nlen,NLM), O3L(nlen,NLM), CLD(nlen,NLM),
     *   TBMN(nlen), ATL(nlen,NLM), ULWTOP(nlen), DLWBOT(nlen),
     *   ULWBOT(nlen), CLU(nlen,NLM), ATLCLR(nlen,NLM),
     *   TSA(nlen), RSCLR(nlen), grav2, cp, RS(nlen),
     *   fluxd_pb(nlen), ULTCLR(nlen), O3TOP(nlen),
     *   emitrnd(nlen,nlm+2), emitmax(nlen,nlm+2), olrtom(nlen),
     *   olrctom(nlen),co2fac
      LOGICAL (kind=log_kind) ::
     &   NOZ,CLIZON, nocld

c     LOCAL VARIABLES:
C	B250 - PLANCK FUNCTION TABLE FOR WATER VAPOR BANDS; CENTER AND
C		WING.
C	NTM1 - NUMBER OF ROWS IN B250 - 1.
C	TEMP1 - LOWEST TEMPERATURE FOR WHICH B250 IS TABULATED.
C	DTB - TEMPERATURE INCREMENT IN B250.
C	NUP1 - NUMBER OF COLUMNS IN GL AND COEFF.
C	UBAR - SCALED WATER VAPOR PATH LENGTH IN WINDOW.
C	VBAR - SCALED WATER VAPOR PATH LENGTH IN CENTER.
C	WBAR - SCALED WATER VAPOR PATH LENGTH IN WING.
C	RAWI - WATER VAPOR AMOUNT IN LAYER.
C	OZAI - OZONE AMOUNT IN LAYER.
C	SHH - PLANCK FUNCTION AT LEVEL TEMPERATURE FOR WATER VAPOR
C		BANDS.
C	SHI - G-FUNCTION FOR A PATH FROM LEVEL TO TOP OF ATMOSPHERE.
C	TXUF - 1.USED AS MATRIX OF G-FUNCTIONS FOR PATHS FROM EACH LEVEL
C		TO ALL OTHER LAYERS.
C		2.USED FOR TRANSMISSION IN CO2 BAND.
C		3.USED FOR TRANSMISSION IN OZONE BAND.
C		4.IN CLDSLW USED FOR PROBABILITY OF CLEAR LINE-OF-SIGHT
C		FROM EACH LEVEL TO ALL OTHER LAYERS FOR MAX OVERLAP.
C	WV,WW - INTERPOLATED VALUE OF GL FOR BAND CENTER AND WING
C		RESPECTIVELY.
C	YV,YW - LINEAR TERM IN TEMPERATURE CORRECTION FOR BAND CENTER
C		AND WING RESPECTIVELY.
C	ZV,ZW - QUADRATIC TERM IN TEMPERATURE CORRECTION FOR BAND CENTER
C		AND WING RESPECTIVELY.
C	BLKCO2 - PLANCK FUNCTION TABLE FOR CO2 BANDS.
C	BLKWIN - PLANCK FUNCTION TABLE FOR OZONE BAND.
C	CC - PLANCK FUNCTION AT LEVEL TEMPERATURE FOR CO2 BANDS.
C	VBARM - PLANCK FUNCTION AT LEVEL TEMPERATURE FOR OZONE BAND.
C	PP - DOPPLER BROADENING CUT-OFF.
C	PSCALV - SCALED CO2 AMOUNT IN BAND CENTER.
C	PSCALW - SCALED CO2 AMOUNT IN BAND WING.
C	DX - PARAMETERIZED OPTICAL DEPTH OF WATER VAPOR LINE IN CO2
C		BAND WING.
C	DY - PARAMETERIZED OPTICAL DEPTH OF WATER VAPOR CONTINUUM IN
C		CO2 BAND WING, ALSO USED FOR OZONE BAND WING.
C	SV,SW - MINIMUM IN TABLE OF LOG WATER VAPOR AMOUNT IN BAND CENTER
C		AND WING RESPECTIVELY.
C	CS - PROBABILITY OF CLEAR LINE-OF-SIGHT FROM EACH LEVEL TO
C		ALL OTHER LAYERS.
C	CH - PROBABILITY OF CLEAR LINE-OF-SIGHT FROM LEVEL TO TOP OF
C		THE ATMOSPHERE.
C	CC - PROBABILITY OF CLEAR LINE-OF-SIGHT FROM LEVEL TO SURFACE.
C	CT - PROBABILITY OF CLEAR LINE-OF-SIGHT FROM LEVEL TO TOP OF
C		ATMOSPHERE FOR MAXIMUN OVERLAP.
C	CU - PROBABILITY OF CLEAR LINE-OF-SIGHT FROM LEVEL TO SURFACE
C		FOR MAXIMUM OVERLAP.
C	emitrnd - emissivity of random overlap clouds
C	emitmax - emissivity of maximum overlap clouds
      REAL (kind=dbl_kind) ::
     &   TXUF(len,nnp,nnp), CS(len,nlm+2,nlm+2), X1(len,nlm+3),
     *   X2(len,nlm+3), TV1(len,nlm+3), CSS(len,nlm+2),
     *   CCU(len,nlm+2), CH(len,nlm+2), X4(len,nlm+2),
     *   X3(len,nlm+2), CC(len,nlm+3), TV2(len,nlm+3),
     *   TUI2(len,nlm+3), RAWI(len,nlm+3), TUI(len,nlm+3),
     *   SHU(len,NLM+1), SHH(len,nlm+2), SUM(len),
     *   SUMSAV(len), DTB,DTBINV,PR(2), PRE(2), DP(len,nlm+2),
     *   SHI(len,nlm+2), PAI(len,nlm+2), FLUXU(len,nlm+2),
     *   UBAR(len,nlm+2), WBAR(len,nlm+2), UBARM(len,nlm+2),
     *   OZAI(len,nlm+2), TAI(len,nlm+2), FLUXD(len,nlm+2),
     *   VBARM(len,nlm+2), VBAR(len,nlm+2), WBARM(len,nlm+2)
      REAL (kind=dbl_kind) ::
     &   fac, d642
      LOGICAL (kind=log_kind) ::
     &   ozone
c       Array indices and counters
      integer (kind=int_kind) ::
     &        i, j, nnpp1, nlmp1, nlsp1
     *,       ls2, ls2p2, ip, ipm1, l, ix, ipp1

c----------------------------------------------------------------

      nlmp1 = nlm+1
      nnpp1 = nnp+1
      nlsp1 = nls+1
      DTB = 5.
      DTBINV = 1. / DTB
      PR(1) = 1. / 3.0 e 01
      PR(2) = 1. / 3.0 e 02

C     SET OZONE LOGICAL VARIABLE

      OZONE = .FALSE.
      IF ((.NOT. NOZ) .OR. CLIZON) OZONE = .TRUE.

C     COPY PARAMETERS INTO LOCAL VARIABLES
                                                                       
      LS2 = NLS+2
      LS2P2  = LS2+2

C     FAC CONVERTS TO DEGREES / TIME STEP

      FAC = grav2 / cp

      PRE(1) = 1. / 2.75 e 02
      PRE(2) = 1. / 5.5 e 02
      D642 = 1. / 6.426 e 02

C     PTOP AND DP AT TOP DONT CHANGE

      DO I=1,len
         pai(i,1) = pl2(i,1)
         DP(I,1) = 0.
         DP(I,2) = PAI(I,1)
         PAI(I,1) = PAI(I,1) * .5
      ENDDO

      DO L=nls+1,nLM
         do i=1,len
	    rawi(i,l+2) = ql(i,l)
         enddo
      ENDDO

      DO l=1,nls
         DO I=1,len
            RAWI(I,l+2) = 3. e -06
         ENDDO
      ENDDO

      DO J=1,nLM
         do i=1,len
	    pai(i,j+1) = pl(i,j)
	    tai(i,j+2) = tl(i,j)
	    x1(i,j+1) = pl2(i,j+1)
         enddo

         if (clizon .or. (.not. noz)) then
            do i=1,len
               ozai(i,j+2) = o3l(i,j)
            enddo
         endif
      ENDDO

      DO I=1,len
         X1(I,1) = DP(I,2)
      ENDDO

      DO l=1,nlm
         DO I=1,len
            DP(I,l+2) = X1(I,l+1) - X1(I,l)
         ENDDO
      ENDDO

C     TEMPERATURE AND HUMIDITY INTERPOLATIONS

      do l=1,nlm
         do i=1,len

! tdrmod - ibm does not like the max function
   	    if(.1e-022_dbl_kind.gt.rawi(i,l+2)) rawi(i,l+2) = .1e-022_dbl_kind
!           rawi(i,l+2) = max(.1e-022_dbl_kind, rawi(i,l+2))
! tdrmodend

         enddo
      enddo

      DO I=1,len
         tui(i,nnp) = tsa(i)
         tui(i,nnpp1) = tg(i)
         PAI(I,nNP) = X1(I,nLMP1)
      ENDDO

      IF (CLIZON.or..not.noz) THEN
         DO I=1,len
            OZAI(I,1) = 0.
            OZAI(I,2) = O3TOP(i) * DP(I,2) * 1.02
         ENDDO

         DO l=1,nlm
            DO I=1,len
               OZAI(I,l+2) = OZAI(I,l+2) * DP(I,l+2) * 1.02

! tdr - ibm does not like the max function
	       if(.1e-09_dbl_kind.gt.ozai(i,l+2)) ozai(i,l+2)=.1e-09_dbl_kind
!              ozai(i,l+2) = max(.1-09_dbl_kind, ozai(i,l+2))
! tdrmodend

            ENDDO
         ENDDO
      ENDIF

C     DO TEMPERATURE INTERPOLATION

      DO I=1,len
         RAWI(I,1) = 0.
         RAWI(I,2) = 3. e -06 * DP(I,2) * 1.02
      ENDDO

CDIR$ IVDEP
      DO l=1,nlm
         DO I=1,len
            RAWI(I,l+2) = RAWI(I,l+2) * DP(I,l+2) * 1.02
            X1(I,l) = X1(I,l+1) / PAI(I,l+1)            ! no dependency

            x2(i,l) = log(x1(i,l))
            X1(I,l) = PAI(I,l+2) / PAI(I,l+1)
            x4(i,l) = log(x1(i,l))
         ENDDO
      ENDDO

      DO l=1,nlm-2
         DO I=1,len
            TUI(I,l+2) = TAI(I,l+2) + X2(I,l) 
     *            / X4(I,l) * (TAI(I,l+3) - TAI(I,l+2))
         ENDDO
      ENDDO

      DO I=1,len
         tui(i,nlmp1) = tbmn(i)
         TAI(I,1) = TAI(I,3)
         TAI(I,2) = TAI(I,3)
      ENDDO

      DO I=1,len
         TUI(I,1) = TAI(I,1)
         TUI(I,2) = TAI(I,2)
      ENDDO

C     COMPUTE SCALED WATER VAPOR AMOUNTS

      DO I=1,len
         UBAR(I,1) = 0.
         VBAR(I,1) = 0.
         WBAR(I,1) = 0.
      ENDDO

      DO l=1,nlmp1
         DO I=1,len
            x2(i,l) = exp((0.5 e -02 * (TAI(I,l+1) - 225.)))
            VBAR(I,l+1) = RAWI(I,l+1) * X2(I,l) * (PAI(I,l) * PRE(1))
            X3(I,l) = 1.6 e -02 * (TAI(I,l+1) - 256.)

            x2(i,l) = exp(x3(i,l))
            WBAR(I,l+1) = RAWI(I,l+1) * X2(I,l) * (PAI(I,l) * PRE(2))

            x2(i,l) = exp((1.8 e 03 / TAI(I,l+1) - 6.0811))
            X2(I,l) = X2(I,l) * PAI(I,l) * D642
            X2(I,l) = X2(I,l) * RAWI(I,l+1) * RAWI(I,l+1) / DP(I,l+1)
         ENDDO
      ENDDO

      DO IP=2,nNP
         DO I=1,len
            UBAR(I,IP) = UBAR(I,IP-1) + X2(I,IP-1)
            VBAR(I,IP) = VBAR(I,IP-1) + VBAR(I,IP)
            WBAR(I,IP) = WBAR(I,IP-1) + WBAR(I,IP)
         ENDDO
      ENDDO

      DO l=1,nlmp1
         DO I=1,len
            UBARM(I,l+1) = (UBAR(I,l+1) + UBAR(I,l)) * .5
            VBARM(I,l+1) = (VBAR(I,l+1) + VBAR(I,l)) * .5
            WBARM(I,l+1) = (WBAR(I,l+1) + WBAR(I,l)) * .5
         ENDDO
      ENDDO

      CALL LWFLUX (co2fac, TXUF, TV1, DP, CSS, CCU, CH, CC, TV2, TUI2,
     *   SHI, RAWI, TUI, SHU, SHH, SUMSAV, DTB, DTBINV, PR, PAI, FLUXU,
     *   UBAR, VBAR, WBAR, UBARM, OZAI, WBARM, TAI, FLUXD, VBARM, OZONE,
     *   len, nlm, nnp )

C     COMPUTE CLEAR SKY FLUXES ONLY FOR CLOUD RADIATIVE FORCING
C     ZERO OUT DOWNWARD FLUX ACCUMULATORS AT L=1

      DO I=1,len
         FLUXD(I,1) = 0.

C     UPWARD FLUX AT THE SURFACE

         FLUXU(I,nNP) = SUMSAV(I)
      ENDDO

C     COMPUTE DOWNWARD FLUXES

      DO IP=2,nNP
         DO I=1,len
            SUM(I) = 0.
         ENDDO

         IPM1 = IP - 1

         DO IX=1,IPM1
            DO I=1,len
               SUM(I) = SUM(I) + TXUF(I,IX,IP)
            ENDDO
         ENDDO

         DO I=1,len
            FLUXD(I,IP) = SUM(I) - SHI(I,IP) + SHH(I,IP)
         ENDDO
      ENDDO

C     COMPUTE UPWARD FLUXES

      DO IP=1,nLMP1
         IPP1 = IP + 1

         DO I=1,len
            SUM(I) = 0.
         ENDDO

         DO IX=IPP1,nNP
            DO I=1,len
               SUM(I) = SUM(I) + TXUF(I,IX,IP)
            ENDDO
         ENDDO

         DO I=1,len
            FLUXU(I,IP) = (SUM(I) + SHU(I,IP)) + SHH(I,IP)
         ENDDO
      ENDDO

      DO l=1,nnp
         DO I=1,len
            X1(I,l) = FLUXD(I,l) - FLUXU(I,l)
         ENDDO
      ENDDO

      DO I=1,len
         ULTCLR(I) = -X1(I,1)
         olrCtom(I) = -X1(I,2)
         RSCLR(I) = -X1(I,nNP)
      ENDDO

      DO l=1,nlm
         DO I=1,len
            X2(I,l) = (X1(I,l+1) - X1(I,l+2)) * FAC / DP(I,l+2)
         ENDDO
      ENDDO

      DO l = 1,nlm
         DO I = 1,len
            ATLCLR(I,l) = X2(I,l)
         ENDDO
      ENDDO

C     WE DONT ALLOW CLOUDS IN THE STRATOSPHERE

      DO l=1,nlsp1
         DO I=1,len
            CSS(I,l) = emitrnd(i,l)
            CCU(I,l) = emitmax(i,l)
         ENDDO
      ENDDO

      if(nocld) then
         do l = nlsp1,nlm  
            DO I=1,len
               CCU(I,l+1) = 0.0
               CSS(I,l+1) = 0.0
            ENDDO
         ENDDO
      else
         do l = nlsp1,nlm  
            DO I=1,len
               CCU(I,l+1) = emitmax(i,l+1) * Clu(I,l)
               CSS(I,l+1) = emitrnd(i,l+1) * Cld(I,l)
            ENDDO
         ENDDO
      endif

C     GET THE CONTRIBUTION FROM CLOUDINESS

      CALL CLDSLW( CSS, CCU, CH, CC, CS, len, nlm )

      DO l=1,nnp
         DO j=1,nnp
            DO I=1,len
               TXUF(I,j,l) = TXUF(I,j,l) * CS(I,j,l)
            ENDDO
         ENDDO
      ENDDO

      DO l=1,nnp
         DO I=1,len
            SHI(I,l) = SHI(I,l) * CH(I,l)
         ENDDO
      ENDDO

      DO l=1,nlmp1
         DO I=1,len
            shu(i,l) = shu(i,l) * cc(i,l) + shh(i,l)
         ENDDO
      ENDDO

C     ZERO OUT DOWNWARD FLUX ACCUMULATORS AT L=1

      DO I=1,len
         FLUXD(I,1) = 0.

C     UPWARD FLUX AT THE SURFACE

         FLUXU(I,nNP) = SUMSAV(I)
      ENDDO

C     UPWARD FLUX AT THE SURFACE WAS COMPUTED BEFORE CALL TO CLDSLW
C     DOWNWARD FLUXES

      DO IP=2,nNP
         DO I=1,len
            SUM(I) = 0.
         ENDDO

         IPM1 = IP - 1

         DO IX=1,IPM1
            DO I=1,len
               SUM(I) = SUM(I) + TXUF(I,IX,IP)
            ENDDO
         ENDDO

         DO I=1,len
            FLUXD(I,IP) = SUM(I) - SHI(I,IP) + SHH(I,IP)
         ENDDO
      ENDDO

C     UPWARD FLUXES

      DO IP=1,nLMP1
         IPP1 = IP + 1

         DO I=1,len
            SUM(I) = 0.
         ENDDO

         DO IX=IPP1,nNP
            DO I=1,len
               SUM(I) = SUM(I) + TXUF(I,IX,IP)
            ENDDO
         ENDDO

         DO I=1,len
            FLUXU(I,IP) = SUM(I) + SHU(I,IP)
         ENDDO
      ENDDO

      DO l=1,nnp
         DO I=1,len
            X1(I,l) = FLUXD(I,l) - FLUXU(I,l)
         ENDDO
      ENDDO

      DO I=1,len
         ULWTOP(I) = -X1(I,1)
         olrTOm(I) = -X1(I,2)
         RS(I) = -X1(I,nNP)
         DLWBOT(I) = FLUXD(I,nNP)
         ULWBOT(I) = FLUXU(I,nNP)
      ENDDO

      DO l=1,nlm
         DO I=1,len
            X2(I,l) = (X1(I,l+1) - X1(I,l+2)) * FAC / DP(I,l+2)
         ENDDO
      ENDDO

      DO l = 1,nlm
         DO I = 1,len
            ATL(I,l) = X2(I,l)
         ENDDO
      ENDDO

c    drb moved to surface grid, downward flux at pbl top passed out

      DO I=1,len
!         fluxd_pb(i) = fluxd(i,nlmp1)
         fluxd_pb(i) = fluxd(i,lpbl)
      ENDDO

      RETURN
      END
