c-------------------------------------------------------------------

      SUBROUTINE oceanflx( len,
     *     grav, dt, cp, hltm, delta, stefan,
     *     pr0, ribc, vkrmn, grav2, gamfac, tice, 
     *     tg, bps, thm, ps, sh, 
     *     spdm, drag, ventmf, cu, ct, ustar, zmelt, flxice,
     *     psb, fws, fss, smr, ros, ts, zb,
     *     ticm, iz, sn, lspr, cupr, ss, lqflux, day, dtpg,
     *     e2prog, tofday, hmix1, hmix2, hmax, e21, e22, e2, qf,   
     *     tke, dotkef, tpgf1, thvgm, louis, zwind, ztemp,
     *     z0 )

c     subroutine ocean computes the surface fluxes for an ocean grid point 

      use kinds
      use eau_util

      IMPLICIT none

c-------------------------------------------------------------------

c     SUBROUTINES CALLED: 

c     FUNCTIONS CALLED:
c          none
                       
c     argument list variables
c        len             gridpoints in this subdomain
      integer (kind=int_kind) ::
     *   len 

  
c        lqflux          if true, do prognistic mixed layer ocean and sea-ice
c        e2prog          if true, predict e2 rather than prescribe it
c        dotkef          if true, use changan formulation rather than deardorff
      logical (kind=log_kind) ::
     *   lqflux, e2prog, dotkef, louis

c        grav            gravitational acceleration (m/s**2)
c        dt              time step (s)
c        cp              specific heat of dry air (J/kg)
c        delta
c        ribc            critical richardson number
c        pr0
c        vkrmn           von karman's constant
c        grav2           grav *0.01 mb/Pa
c        gamfac
c        tice            freezing temperature (K)
c        stefan          stefan-boltzman constant
c        tofday          time of day (hours)
c        day             length of day (s)
c        dtpg            time interpolation fractional time step
c        BPS            (surface pressure/p00)**kapa
c        PSB             PBL depth (hPa)
c        PS              surface presure (hPa)  
c        SMR             net surface radiation (W/m**2)
c        SPDM            PBL wind speed (m/s) 
c        ros             surface air density (kg/m**3)
c        ts              surface air temperature (K)
c        zb              boundary layer geometric depth (m)
c        lspr            stratiform precip rate (mm/s)
c        cupr            convective precip rate (mm/s)
c        ss              surface net absorbed solar radiation (W/m**2)
c        hmix1           ocean mixed layer depth, previous 15th of month (m)
c        hmix2           ocean mixed layer depth, next 15th of month (m)
c        e21             ocean deep layer energy, previous 15th of month(J/m^2)
c        e22             ocean deep layer energy, next 15th of month (J/m^2)
      real (kind=dbl_kind), intent(in) ::
     *     grav, dt, cp, hltm, delta, ribc, day, dtpg, zwind, ztemp,
     *     pr0, vkrmn, grav2, gamfac, tice, stefan, tofday, tpgf1
c     input arrays
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     bps, ps, 
     *     spdm, psb,
     *     smr, ros, ts, zb,
     *     lspr, cupr,
     *     ss, hmix1, hmix2, hmax, qf,
     *     e21, e22,
     *     tke
c     output arrays
C        DRAG            SHEAR STRESS (PA)
c        FWS             surface evaporation (MM) 
c        FSS             surface sensible heat flux (w/m**2) 
c        cu
c        ct
c        ventmf
c        ustar
c        zmelt
c        flxice
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     ventmf, fws, fss, cu
     *,    ct, ustar, zmelt, flxice, thvgm
      real (kind=dbl_kind), intent(out), dimension(len,2) ::
     *     drag

c     in/out arrays (modified)
c        tg             sea surface or ice surface temperature (K)
C        THM            MIXED LAYER POTENTIAL TEMPERATURE (K)
c        SH             PBL water vapor mixing ratio (kg/kg)
c        e2             ocean deep layer energy (J/m^2)
c        ticm           ice interior temperature (K)
c        iz             ice thickness (m)
c        sn             snow depth (m water equivalent)
      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     tg, thm, sh, e2, sn, iz,
     *     ticm, z0

c     local variables
      real (kind=dbl_kind), dimension(len) ::
     *     tha, sha, www,
     *     cni, cuni, ctni,
     *     ctni3, cti, cui, 
     *     rib,
     *     thgeff, shgeff,
     *     thmtem, shtem,
     *     cun, ctn,
     *     z1z0Urt, z1z0Trt

      integer (kind=int_kind) :: i

c     local variables (from flxslv and flxsice)
      real (kind=dbl_kind) ::
     *     io,ki,ks, cigma, rci, tb, hicei, qgi, hlti 

      real (kind=dbl_kind) ::
     *     ig2
      real (kind=dbl_kind), dimension(len) ::
     *     GTR, QG, GAM, xo  
     *,    TEMV       
     *,    VDT, CPGBL, tem1, tflux        
     *,    qtem, rtem, rc, stem 
     *,    psbg2, dgtc, rs, zzwind, zztemp, ptem
      real (kind=dbl_kind), dimension(len,3) ::
     *     b
      real (kind=dbl_kind), dimension(len,7) ::
     *     a

             
c-------------------------------------------------------------------

c     initialize some arrays
      DO I=1,len
         www(i) = 1.0     !  wet surface
         THA(I) = Tg(I) / BPS(I)
         if(iz(i).lt.1.e-8) then
           z0(i) = 0.0002
         else
           z0(i) = 0.0001
         endif
      enddo
      ptem(1:len) = ps(1:len) * 100.
      call eau_sat(len, ptem, tg, qsats=sha )
      do i = 1,len
         thgeff(i) = tha(i)
         shgeff(i) = sha(i)
      enddo

c     get wind stress (drag) and ventilation mass flux (ventmf)
      if(louis) then
          do i = 1,len
             zzwind(i) = zwind
             zztemp(i) = ztemp
          enddo
         call VMFCALZ(PR0,RIBC,VKRMN,DELTA,GRAV
     *,               SH,PS,z0           
     *,               SPDM,SHA,ROS,CU,ct,THVGM,RIB     
     *,               USTAR,VENTMF,THM, tha, zzwind, zztemp
     *,               cuni, cun, ctn, z1z0Urt, z1z0Trt, len ) 
      else
         CALL VMFCAL(PR0,RIBC,VKRMN,DELTA,GRAV
     *,              SH,PS,Z0,SPDM   
     *,              SHA,ZB,ROS,CU,THVGM,RIB,USTAR     
     *,              VENTMF,THM, tha, cni, cuni, ctni
     *,              ctni3, cti, cui, len
     *,              thgeff, shgeff, tke, ct, dotkef ) 
      endif                                    
      DO I=1,len
         DRAG(I,1) = ROS(I) * USTAR(I)**2/spdm(i)
         DRAG(I,2) = DRAG(I,1)
      enddo

c     get latent and sensible heat fluxes, new sea or ice surface temperature
c      if(lqflux) then     !  mixed layer ocean with prescribed heat fluxes
c
c         htpg = dt * dtpg
c         hpgf1 = tpgf1 - (3600.*tofday*dtpg -htpg)
c         hpgf2 = 1. - hpgf1
c
cc          mixed layer depths for this and next timestep
c         do i = 1,len
c            hmix(I) = hPGF1*hmix1(i) + hPGF2*hmix2(i)
c            hmixn(I) = (hPGF1-htpg)*hmix1(i) +
c     *         (hPGF2+htpg)*hmix2(i)
c         enddo
c
c         if(.not.e2prog)then
c          prescribed deep mixed layer energy
c            do i = 1,len      
c               e2(i) = hPGF1*e21(i) +
c     *                      hPGF2*e22(i)
c            enddo
c         endif
c
c         CALL qflux( len, nocean, grav2, hltm, dt, tice, stefan, ps
c     *,              psb, bps, ss, rs, tg, iz, sn, e2, hmix, hmixn
c     *,              hmax, ticm, thm, sh, ventmf, cupr, lspr, fss
c     *,              fws, flxice, roff, zmelt, e2prog, gamfac, cp, qf )
c
c      else   ! if (lqflux)

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     flxslv and flxsice inlined for prescribed SST and ice cover
c
c      implicit solution for boundary temperature, pbl temperature and 
c           pbl moisture
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     cigma = 0.0 for no longwave feedback
c     rci = 0.0 for no ice heat capacity (0 layer ice instead of 1 layer ice)
c
      cigma = stefan
c      cigma = 0.0
      rci = 1.8828 e6
c      rci = 0.0
      tb = tice-1.9
      ki=2.0334
      ks=1.032
      io=0.17
      IG2   =1.0/GRAV2                                               
      HLTI  =1.0/HLTM                                                
      do i = 1,len 
         rc(i) = rci
         tem1(i) = 0.0
c        if(iz(i).lt.0.003) rc(i) = 0.0
         if(sn(i).le.0.0000001) tem1(i) = 1.0
      enddo
      call eau_sat(len, ptem, tg, qsats=qg)
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C    MATRIX SYSTEM TO SOLVE IS:                                         
C                                                                       
C              (  A1   A2    0  )      (  T  )       (  B1  )           
C              (  A3   A4   A5  )  *   ( tg  )   =   (  B2  )           
C              (   0   A6   A7  )      ( SH  )       (  B3  )           
C                                                                       
C    FOR SEAICE POINTS                                    
C    FOR OCEAN POINTS tg IS FIXED AND THE ABOVE SYSTEM SIMPLIFIES 
c      (terms multiplied by xo vanish)       
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      do i = 1,len
         if(iz(i).gt.0.0) then
            xo(i) = 1.0
         dgtc(i) =  4. * cigma * tg(i)**3                                         
         stem(i) = 2. * ks * ki /
     *           (2. * ki * sn(i) + iz(i)*ks)
	 rtem(i) = 2. * ki / iz(i)
	 qtem(i) = 1./(rc(i) * iz(i) + dt * (rtem(i) + stem(i)) )
         else
            xo(i) = 0.0
            stem(i) = 0.0
            rtem(i) = 0.0
            qtem(i) = 0.0
            dgtc(i) = 0.0
         endif
      enddo

      do i = 1,len
         rs(i) = ss(i) - smr(i)                                           
         psbg2 (I)=PSB(i)*IG2                                               
         GAM  (I)=QG(I)*GAMFAC/(tg(i)*tg(i))                               
         VDT  (I)=Ventmf(i)*dt                                                 
         CPGBL(I)=CP*GAM(I)*HLTI

         A(I,1)=cp *bps(i)*(psbg2(I)+VDT(I))                                   
         A(I,2)=-dt*(dgtc(i)*xo(i)+cp * Ventmf(i))      
c  A3, A5 zero for ice free ocean, a4=1  
         A(I,3)=-Ventmf(i)*CP*bps(i)*xo(i)                                     
         A(I,4)= xo(i) * ( stem(i)*(1. - dt*stem(i)*qtem(i))     
     ^         + Ventmf(i)*(cp+hltm*CPGBL(I)) +dgtc(i)  ) 
     ^           +(1.-xo(i))                                   
         A(I,5)=-Ventmf(i)*HLTM*xo(i)                                      
         A(I,6)=-VDT(I)*CPGBL(I)                                     
         A(I,7)=psbg2(I)+VDT(I)                                       
C                                                                       
         B(I,1)=cp *psbg2(I)*Thm(i)*bps(i) -
     *               dt* dgtc(i) * tg(i)*xo(i)                                       
c  B2 = tg for ice free ocean                                              
         B(I,2)= xo(i)*(ss(i)*(1.-io*tem1(i)) - rs(i) +
     1          dgtc(i) * tg(i)
     1         + qtem(i) * stem(i) * (dt*tb*rtem(i) +
     1           ticm(i)*rc(i)*iz(i))                                 
     1          +Ventmf(i)*HLTM*(CPGBL(I)*tg(i)-QG(I)) ) 
     1         +(1.0-xo(i))*tg(i)               
         B(I,3)=psbg2(I)*SH(i)+
     *          VDT(I)*(QG(I)-CPGBL(I)*tg(i))          
C                                                                       
C     SOLVE SYSTEM BY ELIMINATION AND BACK SUBSTITUTION                 
C                                                                       
         TEMV(I  )=A(I,3)/A(I,1)                                           
         A   (I,4)=A(I,4)-A(I,2)*TEMV(I)                                   
         B   (I,2)=B(I,2)-B(I,1)*TEMV(I)                                   
         TEMV(I  )=A(I,6)/A(I,4)                                           
         A   (I,7)=A(I,7)-A(I,5)*TEMV(I)                                   
         B   (I,3)=B(I,3)-B(I,2)*TEMV(I)                                   
         B   (I,3)=B(I,3)/A(I,7)                                           
         B   (I,2)=(B(I,2)-A(I,5)*B(I,3))/A(I,4)                           
         B   (I,1)=(B(I,1)-A(I,2)*B(I,2))/A(I,1)                           
      enddo
C                                                                       
C     CHECK ICE/SEAICE POINTS WITH GT > TICE AND CORRECT                
C 
      do i = 1,len       
         GTR(I) = tice 
         if(b(i,2).gt.GTR(I).and.iz(i).gt.0.0) then
            QGI   =QG(I)+CPGBL(I)*(GTR(I)-tg(i))                        
C                                                                       
            A(I,1)=cp*bps(i)*(PSBG2(i)+VDT(I))                               
            A(I,7)=PSBG2(i)+VDT(I)                                   
C                                                                       
            B(I,1)=(cp*(PSBG2(i)*Thm(i)*bps(i)+VDT(I)*GTR(I))
     ^          - dt * dgtc(i) * (tg(i) - gtr(i)) )/A(I,1)            
            B(I,2)=gtr(i)                                                 
            B(I,3)=(PSBG2(i)*SH(i)+VDT(I)*QGI)/A(I,7)                
	 endif
      enddo
C                                                                       
C     UPDATE PROGNOSTIC VARIABLES                                       
C                                                                       
      DO I=1,LEN                                                   
         Thmtem(i)=B(I,1)                                                     
         GTR(I)=B(I,2)                                                     
         SHtem (i)=B(I,3)                                                     
         QG (I)=QG(I)+CPGBL(I)*(GTR(I)-tg(i))                              
         tg (i)=GTR(I)                                                     
C                                                                       
C     SURFACE SENSIBLE AND LATENT HEAT FLUXES                           
C                                                                       
         FSS(i)=Ventmf(i)*CP*(tg(i)-bps(i) * Thmtem(i))                             
         FWS(i)=Ventmf(i)*(QG(I)-SHtem(i))    !  kg/m2/s (evaporation rate)                            
      enddo

c     ice interior temperature and net flux, snow evaporation, snow melt

      HICEi = dt/(3.*0.9917037e2)                                 
      DO I=1,LEN     
         if(iz(i).gt.0.0) then
            ticm(i)=qtem(i)* (dt*(tg(i)*stem(i)+
     ^        tb*rtem(i)+io*ss(i)*tem1(i))
     ^        +ticm(i)*rc(i)*iz(i)) 
            ticm(i) = min(ticm(i),tice)                                                    
            tflux(i) = ss(i)*(1.-io*tem1(i))-
     *              rs(i) - fss(i) - fws(i)*hltm
     *             + (ticm(i)-tg(i))*stem(i)  
            flxice(i) = smr(i) - (fss(i) +fws(i)*hltm)                                    
c           flxice(i) = stem(i)*(tg(i)-ticm(i)) +
c     *                     ss(i) * io * tem1(i)                         
c
c  snow evaporation and snowmelt
c
c           hsinc1 = max(0.0_dbl_kind,fws(i) * dt)
c           hsinc1 = min(hsinc1,sn(i))
c           SN(i) = SN(i) - hsinc1*0.001
c           hsinc2 = max(0.0_dbl_kind,tflux(i)*hicei)
c           hsinc2 = min(hsinc2,sn(i))
c           flxice(i) = flxice(i) - hsinc2/hicei
c           SN(i) = SN(i) - hsinc2
c           zmelt(i) = hsinc2 
            zmelt(i) = 0.0
c
c  snow accumulation
c
c          if(tg(i).lt.tice) then
c             SN(i) = SN(i) +
c     ^                 (LSPR(i) + CUPR(i)) * dt*0.001
c          endif
         else
            ticm(i) = tg(i)
            sn(i) = 0.0
            flxice(i) = smr(i) - (fss(i)+fws(i)*hltm)
            zmelt(i) = 0.0
            sn(i) = 0.0
         endif                                              
      enddo

c      endif   !  if(lqflux)           

      RETURN
      END
