      module cpl_pbl

!-----------------------------------------------------------------------
! PURPOSE: 
!   1) Determine if stratus clouds are present in the PBL mixed layer, and if so
!      compute the PBL cloud base pressure (pc).
!   2) Determine if PBL layer cloud instability exists (subroutine cloud).
!   3) Compute the PBL top entrainment rate and the PBL turbulent kinetic energy.
!-----------------------------------------------------------------------

      use kinds
      use physical_parameters
      use pbl_parameters
      use eau_util, ONLY: eau_sat
      use cpl_cubic_module
      use comp_tke_module
#ifdef scm
      use scm_module
#endif

      implicit none
      private
      public :: cpl_pbl_state

      contains

!======================================================================
! BEGINNING OF cpl_PBL_STATE
!======================================================================
      Subroutine cpl_PBL_STATE(nlen, len, bpe, pl2, &
        theta, q, zs, ts, psb, &
        dhb, dwb, dpl, &
        tb, qsatb, tbp, wbp, dtt, &
        dtlci, dqlci, safe, pc, pcbnolci, &
        dsvcrit, zb, toposd, eta, cld, &
        lwp, pblcw, liqb, tbmns, shallowcu, lciinc, &
!-- optional arguments for entrainment calculation
        fss, fws, ventmf, thvgm, deltasv, &
        fhs, fsvs, tke, spdm, shear1, &
        ros, cu, fluxd, drb, dpent, &
        diss, ribn, altent1, altent2, sumcoeff, &
        negprod, bync, shear, gprod, &
        cons, negeinc)

!-- originally written by Mark Branson, 12/98
    
!-- qsats instead of qsatw is used in all the calls to eau_sat.
!     Laura D. Fowler, 10/01.

!     SUBROUTINES CALLED:
!        compe, compri, cloud, comp_tke, eau_sat, iterbs

!     FUNCTIONS CALLED:
!        none

!-- vertical index key
!-- a) layer center variables: 
!--    1 = nlmm2, 2 = nlmm1, 3 = nlm (or pbl)
!-- b) layer edge variables:
!--    1 = top of nlmm2, 2 = top of nlmm1, 3 = top of nlm, 4 = sfc

      implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) :: &
         nlen, &       ! total array size
         len           ! size of chunk being passed in
      real (kind=dbl_kind), dimension(nlen,3), intent(in) :: &
         bpe, &        ! (p/po)^kappa at layer edges                     (-)
         pl2           ! pressure at layer edges                        (mb)
      real (kind=dbl_kind), dimension(nlen,2), intent(in) :: &
         theta, &      ! potential temperature                           (K)
         q             ! water vapor mixing ratio                     (g/kg)

      real (kind=dbl_kind), dimension(len), intent(in) :: &
         zs, &         ! surface geopotential                      (m^2/s^2)
         psb, &        ! PBL pressure depth                             (mb)
         zb, &         ! PBL depth                                       (m)
         toposd, &     ! sub-grid scale topography standard deviation    (m)
         wbp, &        ! Level B+ water vapor mixing ratio           (kg/kg)
         tbp           ! Level B+ temperature                            (K)
      real (kind=dbl_kind), dimension(len), intent(inout) :: &
         ts            ! surface temperature                             (K)
      real (kind=dbl_kind), intent(in) :: &
         dtt           ! time step                                       (s)

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(nlen,2), intent(out) :: &
         dpl, &        ! layer pressure thicknesses                     (mb)
         dtlci, &      ! layer nlm and nlmm1 temperature change 
!                        due to LCI mixing                               (K)
         dqlci         ! layer nlm and nlmm1 mixing ratio change
!                        due to LCI mixing                           (kg/kg)

      real (kind=dbl_kind), dimension(len), intent(inout) :: &
         qsatb         ! PBL top saturation mixing ratio             (kg/kg)
      real (kind=dbl_kind), dimension(len), intent(out) :: &
         lwp, &        ! PBL liquid water path                           (?)
         pblcw, &      ! PBL cloud liquid water mixing ratio         (kg/kg) 
         liqb, &       ! PBL top liquid water mixing ratio           (kg/kg)
         tbmns, &      ! ???                                             (K)
         tb, &         ! PBL top air temperature                         (K)
         dhb, &        ! moist static energy change across PBL top (m^2/s^2)            
         dwb, &        ! mixing ratio change across PBL top          (kg/kg)
         safe, &       ! LCI criterion (>0 = stable)                  (J/kg)
         pc, &         ! PBL cloud base pressure                        (mb)
         dsvcrit, &    ! critical virtual dry static energy difference(J/kg)
         eta, &        ! toposd/zb                                       (-)
         cld, &        ! fractional PBL cloud amount                     (-)
         pcbnolci, &   ! PBL stratus pressure depth (no LCI effect)     (mb)
         shallowcu, &  ! shallow cumulus incidence (test diag)           (-)
         lciinc        ! LCI incidence                                   (-)

!---------------------------------------------------------------------------
! OPTIONAL INTENT IN
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(len), intent(in), optional :: &
         fss, &        ! surface sensible heat flux                  (W/m^2)
         fws, &        ! surface latent heat flux                  (kg/m2/s)
         ventmf, &     ! ventilation mass flux                     (kg/m2/s)
         thvgm, &      ! virtual potential temperature difference
!                       between the middle of the PBL and the sfc        (K)
         spdm, &       ! PBL wind speed                                (m/s)
         shear1, &     ! wind shear at the PBL top                      (/s)
         ros, &        ! surface air density                        (kg/m^3)
         cu, &         ! surface momentum transfer coefficient           (?)
         fluxd         ! downard LW radiative flux at  PBL top       (W/m^2)

!---------------------------------------------------------------------------
! OPTIONAL INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(len), intent(out), optional :: &
         drb, &        ! LW radiative flux jump across PBL top       (W/m^2)
         ribn, &       ! bulk Richardson number                          (-)
         deltasv, &    ! virtual dry static energy jump at PBL top (m^2/s^2)
         fhs, &        ! surface moist static energy flux            (W/m^2)
         fsvs, &       ! surface virtual dry static energy flux      (W/m^2)
         dpent, &      ! PBL entrainment rate (E)                  (kg/m2/s)
         diss, &       ! TKE dissipation rate                       (kg/s^3)
         altent1, &    ! alternative E value                      (kg/m/s^2)
         altent2, &    ! alternative E value                      (kg/m/s^2)
         sumcoeff, &   ! TKE cubic solver output term                    (?)
         bync, &       ! PBL buoyancy term                               (?)
         shear, &      ! PBL total shear                                 (?)
         gprod, &      ! PBL "production" term                           (?)
         cons, &       ! PBL "consumption" term                          (?)
         negeinc       ! negative entrainment incidence                  (?)

      real (kind=dbl_kind), dimension(len), intent(inout), optional :: &
         tke           ! PBL turbulent kinetic energy              (m^2/s^2)
      logical (kind=log_kind), dimension(len), intent(out), optional :: &
         negprod       ! PBL negative production term                    (?)

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
      integer (kind=int_kind) :: &
         i, l
      real (kind=dbl_kind) :: &
         temp, &
         rgfac, &
         bone, &       ! entrainment constant (func. of diss. const. C)  (-)
         btwo, &       ! entrainment constant (func. of diss. const. C)  (-)
         hm, &         ! mixed layer moist static energy           (m^2/s^2)
         hs, &         ! surface moist static energy               (m^2/s^2)
         rhom, &       ! PBL air density                            (kg/m^3)
         drbfac, &
         pcblk         ! 1/12.5mb                                     (mb-1)
      real (kind=dbl_kind), dimension(len) :: &
         hbp, &        ! moist static energy at level B+           (m^2/s^2)
         spanew, &     ! thermodynamic factor used to compute phi
         phib, &       ! geopotential at PBL top                   (m^2/s^2)
         wm, &         ! mixed layer total water mixing ratio        (kg/kg)
         qvs, &        ! surface water vapor mixing ratio            (kg/kg)
         qvm, &        ! PBL water vapor mixing ratio                (kg/kg)
         qsats, &      ! surface saturation water vapor mixing ratio (kg/kg)
         qvb, &        ! PBL top water vapor mixing ratio            (kg/kg)
         qsbp, &       ! Level B+ saturation water vapor mixing ratio(kg/kg)
         epslnb, &     ! (Cp/L)*tb (Equiv. to epsilon term in 
!                        Randall 1980a)                                  (-)
         alphab, &     ! (Equiv. to beta term in Randall 1980a)          (-) 
         gamb, &       ! (L/Cp)*(qsatb/tb**2)                            (-)
         svb, &        ! PBL top virtual dry static energy            (J/kg)
         svbp, &       ! Level B+ virtual dry static energy           (J/kg)
         rob, &        ! PBL top air density                        (kg/m^3)
         tbclr, &      ! "dry" PBL top temperature                       (K)
         hb, &         ! PBL top moist static energy               (m^2/s^2)
         hstarbp       ! PBL top saturation moist static energy    (m^2/s^2)
      real (kind=dbl_kind), dimension(nlen,2) :: &
         imassl, &     ! (grav*0.01)/(layer pressure thickness)     (m^2/kg)
         phil          ! odd-level geopotentials                   (m^2/s^2)
      real (kind=dbl_kind), dimension(nlen,2) :: &
         qls, &        ! layer saturation mixing ratios              (kg/kg)
         bp, &         ! odd-level (p/pref)**kappa                       (-)
         pl, &         ! odd-level pressures (layer centers)            (mb)
         tl            ! odd-level temperatures (layer centers)         (mb)
      real (kind=dbl_kind), dimension(nlen,3) :: &
         pl2pa         ! even-level pressures                           (Pa)
      real (kind=dbl_kind), dimension(nlen,2) :: &
         plpa          ! odd-level pressures                            (Pa)
      logical (kind=log_kind), dimension(len) :: &
         strts, &      ! logical to indicate presence of stratus in PBL
         nentr         ! logical to flag cases where E will be set
!                        equal to zero because there is cloud detected
!                        in both the PBL and the layer above

!-- set needed constants
      pcblk = 0.08

      do l = 1,3
        do i = 1,len
          pl2pa(i,l) = pl2(i,l)*100.
        enddo
      enddo

      do l = 1,2
        do i = 1,len
          bp(i,l) = pl2(i,l+1)*bpe(i,l+1)
          bp(i,l) = bp(i,l)-pl2(i,l)*bpe(i,l)
          temp = 1.0/(pl2(i,l+1) - pl2(i,l))
          temp = temp/kappap1
          bp(i,l) = bp(i,l)*temp
          plpa(i,l) = (bp(i,l)**inv_kappa) * p0_sfc
          pl(i,l) = plpa(i,l)*0.01
          tl(i,l) = theta(i,l) * bp(i,l)
        enddo
        call eau_sat(len, plpa(1,l), tl(1,l), qsats=qls(1,l) )               
      enddo
      do l=1,2
        do i = 1,len
          dpl(i,l) = pl2(i,l+1) - pl2(i,l)
          imassl(i,l) = grav2/dpl(i,l)
        enddo
      enddo

!-- these geopotentials are relative to the sfc
      do i = 1,len
        phil(i,2) = spec_heat_cp*theta(i,2)*(bpe(i,3)-bp(i,2))+zs(i)                                    
        temp = (bpe(i,2)-bp(i,1)) / (bp(i,2)-bp(i,1))
        temp = spec_heat_cp*(bp(i,2)-bp(i,1)) *  &
               (temp*theta(i,1)+(1.-temp)*theta(i,2))
        phil(i,1) = phil(i,2)+temp   
                                 
        spanew(i) = spec_heat_cp*theta(i,2)*(bpe(i,3)-bpe(i,2))         
        phib(i) = phil(i,1) + spec_heat_cp*theta(i,1)*(bp(i,1)-bpe(i,2))
        tb(i) = theta(i,2)*bpe(i,2) 
        tbclr(i) = tb(i)    ! save this for cases where LCI = T
        wm(i) = q(i,2) 
      enddo
      call eau_sat(len, pl2pa(1,2), tb(1), qsats=qsatb(1))
      call eau_sat(len, pl2pa(1,3), ts(1), qsats=qsats(1))

!-- iterate to get better estimates for TB and TS when PBL is saturated
      call ITERBS

      do i = 1,len
        hs = spec_heat_cp*ts(i)   + zs(i)     + hltm*qvs(i)
        hm = spec_heat_cp*tl(i,2) + phil(i,2) + hltm*qvm(i)
        hb(i) = spec_heat_cp*tb(i)   + phib(i)   + hltm*qvb(i)
        hbp(i) = spec_heat_cp * tbp(i) + phib(i) + hltm * wbp(i)
        dhb(i) = hbp(i) - hb(i)
        dwb(i) = wbp(i) - wm(i)
      enddo

!-- we also need the saturation mixing ratio at level B+ to compute dsvcrit
!-- call this qsbp
      call eau_sat(len, pl2pa(1,2), tbp(1), qsats=qsbp(1))

!-- new 8/2001
      do i = 1,len
        hstarbp(i) = spec_heat_cp * tbp(i) + phib(i) + hltm * qsbp(i)
        if ((hb(i) > hstarbp(i)).and.(qvb(i) >= qsatb(i))) then
          shallowcu(i) = 1.0
        else
          shallowcu(i) = 0.0
        endif
      enddo

      do i = 1,len
        gamb(i) = gamfac*qsatb(i) / (tb(i) * tb(i))

        tbmns(i) = (q(i,1) - qsatb(i)) * hltm
        tbmns(i) = tbmns(i) / ((1.+gamb(i))*spec_heat_cp)
        tbmns(i) = max(0.0_dbl_kind, tbmns(i))
        tbmns(i) = tbmns(i) + tb(i)

        epslnb(i) = cpbl*tb(i)
        dsvcrit(i) = (1.-deltap1*epslnb(i))/(1.+gamb(i))
        dsvcrit(i) = dsvcrit(i)*hltm*(qsbp(i)-wbp(i))

        alphab(i) = (1.+deltap1*gamb(i)*epslnb(i))/(1.+gamb(i))
        safe(i) = alphab(i) * dhb(i) - epslnb(i) * hltm * dwb(i)

      enddo

!-- Determine if PBL cloud layer is experiencing layer cloud instability
      call cloud (tl, q, pl,                &
            pl2(1,2), bp(1,2), bpe(1,2),    &
            phil, imassl, tb,               &
            qsatb, gamb,                    &
            dtlci, dqlci, ts, pc,           &
            nentr, qls,                     &
            nlen, len, dtt, wbp, hbp, safe)                        

      do i = 1,len
        if (pc(i)-pl2(i,2).gt.0.1_dbl_kind)  then
          cld(i) = 1.0
        else
          lwp(i) = 0.0
          pblcw(i) = 0.0
          cld(i) = 0.0
        endif
        if (dtlci(i,1) .ne. 0.0) then
          lciinc(i) = 1.0
        else
          lciinc(i) = 0.0
        endif
      enddo
      if (frac_strat) then
        do i = 1,len
!          eta(i) = toposd(i)/zb(i)
          eta(i) = (toposd(i)/zb(i))**2.
          cld(i) = max(cld(i)*(1.-50.*eta(i)),0.0_dbl_kind)
        enddo
      else
        do i = 1,len
          eta(i) = 0.0
        enddo
      endif    

      if (present(dpent)) then     

        bone = 0.252 * (c**(1./3.))
        btwo = 0.504 * (c**(-2./3.))
        rgfac  = 100.0 / gas_const_R

!-- now that pc values have been computed in cloud, calc STRTS values
!--   adjust the 100% stratus drb for the actual stratus fraction
        do i = 1,len
          strts(i) = pc(i).gt.pl2(i,2)

          drb(i) = stefan * tbmns(i)**4 - fluxd(i)
          drbfac = min(1.0_dbl_kind, (pc(i)-pl2(i,2))*pcblk )
          drb(i) = max( 0.0_dbl_kind, drb(i) * drbfac )
          drb(i) = drb(i)*cld(i)

!-- fix 2/99: for lci=true case, use "dry" pbl thermo variables to compute
!-- the pbl top richardson number and the entrainment rate
          if (.not.strts(i)) then
            tb(i) = tbclr(i)
            qvb(i) = wm(i)
            qvm(i) = wm(i)
            qvs(i) = wm(i)
            liqb(i) = 0.
          endif

          svbp(i) = spec_heat_cp*tbp(i)*(1.+delta*wbp(i)) + phib(i)
          svb(i) = spec_heat_cp*tb(i)*(1.+delta*qvb(i)-liqb(i))+phib(i)
          deltasv(i) = svbp(i)-svb(i)
          fhs(i)  = fss(i) + hltm*fws(i)                            
#ifdef scm
          if (scm_presfluxes) then
            fsvs(i) = fss(i) + (delta*cpbl*ts(i)*hltm*fws(i))
          else
            fsvs(i) = spec_heat_cp * bpe(i,3)*ventmf(i)*thvgm(i)
          endif
#else
          fsvs(i) = spec_heat_cp * bpe(i,3)*ventmf(i)*thvgm(i)
#endif
          rob(i) = rgfac*pl2(i,2)/tb(i)          
          rhom = 100. * psb(i)/spanew(i)    
          diss(i) = rhom*tke(i)*sqrt(tke(i))*c

        enddo

!-- NEW 6/2000: will use "classic" E/tke closure if pblclassic=T
        if (pblclassic) then

          call COMP_TKE_CLASSIC(len, pl2(1,3), pl2(1,2), pc, dhb, &
             dwb, drb, q(1,2), nentr, spdm, &
             shear1, psb, tke, epslnb, &
             alphab, dpent, fws, fhs, fsvs,  &
             ros, zb, cu, ts, tb, &
             strts, rob, dtt, &
             shear, gprod, cons)

!-- zero out variables not used in classic scheme

          negprod(:) = .false.
          sumcoeff(:) = 0.
          altent2(:) = 0.
          altent1(:) = 0.
          ribn(:) = 0.
          diss(:) = 0.
          bync(:) = 0.
        
        else

          call COMPRI(len,zb,tke,deltasv,safe, &
                      strts,tb,ribn)

          call COMPE(len,bone,btwo,rob,tke, &
                     ribn,zb,alphab,drb,tb, &
                     strts,dpent)

          call COMP_TKE(len, pl2(1,3), pl2(1,2), pc, dhb, &
             dwb, drb, q(1,2), nentr, spdm, &
             shear1, psb, tke, epslnb, alphab, &
             dpent, altent1, altent2, fws, fhs, &
             fsvs, ros, zb, cu, ts, &
             tb, strts, rob, ribn, diss, &
             fss, safe, sumcoeff, &
             dtt, negprod, bync, &
             shear )

!-- zero out variables not used in the "new" scheme
          gprod(:) = 0.
          cons(:) = 0.
          if (konor) negprod(:) = .false.

        endif  ! pblclassic

        do i = 1,len
          if (dpent(i) < 0.0_dbl_kind) then
            negeinc(i) = 1.0
          else
            negeinc(i) = 0.0
          endif
        enddo

      endif

      contains

!======================================================================
! BEGINNING OF ITERBS
!======================================================================
      Subroutine Iterbs
!-- for saturated PBL, iterate to get better estimate of TB.
!-- if entire PBL is saturated, also do this for TS.

!-- local variables
      real (kind=dbl_kind) :: &
         lovcp, &
         told, &
         gammab, &
         qsold, &
         dtemp, &
         qold, &
         qnew, &
         qsatbsav, &
         qsatssav
      real (kind=dbl_kind), dimension(1) :: &
         tnew, &
         qsnew
      integer (kind=int_kind) :: &
         i,j

      lovcp = c1/cpbl                                             

      qsatbsav = qsatb(1)
      qsatssav = qsats(1)

!-- if PBL is saturated, iterate to get better estimates of tb and qvb      
      DO i = 1,len
        if (wm(i).gt.qsatb(i)) then
          told = tb(i)
          qsold = qsatb(i)
          qold = wm(i)

          j = 1
          dtemp = 1000.
          do while (abs(dtemp).gt.0.001)
            gammab = gamfac*qsold / (told*told)
            dtemp = lovcp*(qold-qsold)/(1.+gammab)
            tnew(1) = told + lovcp*(qold-qsold)/(1.+gammab)
            call eau_sat(1, pl2pa(i,2), tnew(1), qsats=qsnew(1))
            qnew = qold - (qold-qsold)/(1.+gammab)
            told = tnew(1)
            qsold = qsnew(1)
            qold = qnew
            if (j.ge.50) then
              write(*,*) 'Tb iteration not converging at ',i,j,dtemp
              write(*,*) 'Init t,q,p ',tb(i),wm(i),pl2pa(i,3)
              exit
            endif
            j = j+1
          enddo
          tb(i) = tnew(1)
          qsatb(i) = qsnew(1)
          liqb(i) = wm(i)-qsatb(i)
          qvb(i) = qsatb(i)

!-- compute the cloud base pressure
          pc(i) = pl2(i,2)+(pl2(i,3)-pl2(i,2))*  &
               (wm(i)-qsatb(i)) / (max(1.e-10_dbl_kind,qsats(i)-qsatb(i)))
!-- ensure that cloud base pressure is not greater than the surface pressure
          pc(i) = min(pl2(i,3),pc(i))

!-- also check that qsats is not less than qsatb.  from the above formula
!-- for pc, this would cause pc < 0 --> not good. (a similar check is
!-- performed in cloud.F)
          if (qsats(i) .lt. qsatb(i)) pc(i) = pl2(i,3)

          if (pc(i) .eq. pl2(i,3)) then
!-- new 12/98: if surface is saturated, iterate to get better estimates
!-- of ts and qsats
            told = ts(i)
            qsold = qsats(i)
            qold = wm(i)
            j = 1
            dtemp = 1000.
            do while (abs(dtemp).gt.0.001)
              gammab = gamfac*qsold / (told*told)
              dtemp = lovcp*(qold-qsold)/(1.+gammab)
              tnew(1) = told + lovcp*(qold-qsold)/(1.+gammab)
              call eau_sat(1, pl2pa(i,3), tnew(1), qsats=qsnew(1) )
              qnew = qold - (qold-qsold)/(1.+gammab)
              told = tnew(1)
              qsold = qsnew(1)
              qold = qnew
              if (j.ge.50) then
                write(*,*) 'Ts iteration not converging at ',i,j, &
                  dtemp, told, qold
                exit
              endif
              j = j+1
            enddo
            ts(i) = tnew(1)
            qsats(i) = qsnew(1)
            qvs(i) = qsats(i)
!-- recompute the cloud base pressure
            pc(i) = pl2(i,2)+(pl2(i,3)-pl2(i,2))*  &
                    (wm(i)-qsatb(i)) / (qsats(i)-qsatb(i))
          else
            qvs(i) = wm(i)
          endif

!-- ensure that cloud base pressure is not greater than the surface pressure
!          pc(i) = min(pl2(i,3),pc(i))

          qvm(i) = (qvs(i)*(pl2(i,3)-pc(i)) + 0.5*(qvs(i)+qvb(i))*  &
                   (pc(i)-pl2(i,2))) / psb(i)

          pblcw(i) = max(0.0_dbl_kind,0.5*(wm(i)-qsatb(i))  &
                      *(pc(i)-pl2(i,2))/psb(i))
          lwp(i) = pblcw(i)*psb(i)*1.e02/grav

!-- check that cloud base pressure is in fact a larger number than pb
          if (pc(i)-pl2(i,2).lt.-0.001_dbl_kind) then
            write(*,*) 'pc<pb: ',i
            write(*,*) '  pc = ',pc(i)
            write(*,*) '  pb = ',pl2(i,2)
            write(*,*) '  ps = ',pl2(i,3)
            write(*,*) '  qvb = ',qvb(i)*1000.
            write(*,*) '  wm = ',wm(i)*1000.
            write(*,*) '  qvs = ',qvs(i)*1000.
            write(*,*) '  qsatb = ',qsatb(i)*1000.
            write(*,*) '  qsats = ',qsats(i)*1000.
            write(*,*) '  tb = ',tb(i)
            write(*,*) '  ts = ',ts(i)
            write(*,*) '  orig qsatb = ',qsatbsav*1000.
            write(*,*) '  orig qsats = ',qsatssav*1000.
!           read(*,*)

            pc(i) = pl2(i,2)
          endif
!-- use this block of code for non-interp B/S scheme
!        if (wm(i).gt.qsatb(i)) then
!          qvb(i) = qsatb(i)
!          liqb(i) = wm(i) - qsatb(i)
!          pc(i) = pl2(i,2)+(pl2(i,3)-pl2(i,2))*
!     +            (wm(i)-qsatb(i)) / (qsats(i)-qsatb(i))
!          pc(i) = min(pl2(i,3),pc(i)) 

        else  ! wm(i)< qsatb(i) --> pbl is subsaturated
          lwp(i) = 0.0
          pblcw(i) = 0.0
          qvb(i) = wm(i)
          qvm(i) = wm(i)
          qvs(i) = wm(i)
          liqb(i) = 0.
          pc(i) = pl2(i,2)
        endif

!-- new: save pc value for all point in lcl variable (some pc values
!-- get reset to pb in cloud.F)
        pcbnolci(i) = pc(i)-pl2(i,2)
      enddo

      end subroutine iterbs
!======================================================================
! END OF ITERBS
!======================================================================

      end subroutine cpl_PBL_STATE
!======================================================================
! END OF cpl_PBL_STATE
!======================================================================

!======================================================================
! BEGINNING OF CLOUD
!======================================================================

      subroutine cloud (tl, ql, pl, pl2, plk, &
                        plk2, phil, imassl, tb, qob, &
                        gammab, dtlci, dqlci, ts, pc, &
                        nentr, qls, nlen, len, dt, &
                        wbp, hbp, safe)        

      IMPLICIT none

!----------------------------------------------------------------------------
!     Subroutine cloud determines if PBL clouds are present and if layer
!     cloud instability (LCI) is required.  Recalculates temperature and
!     specific humidity in nlm & nlm-1 layers after LCI mixing.

!     REFERENCES: Suarez, Arakawa & Randall (1983) "The Parameterisation
!        of the Planetary Boundary layer in the UCLA General Circulation
!        Model: Formulation & Results", Mon. Wea. Rev. V111, 2224-2243.

!        Randall (1980a) "Conditional Instability of the first kind
!        upside-down", JAS V37, 125-130.

!        Randall (1980b) "Entrainment into a stratocumulus layer with
!        distributed radiative heating", JAS V37 148-159

!     MODIFICATIONS:
!		- changed VQSAT call to VNQSAT.  kwitt 10/23/91
!		- inlined gcmlib.F sub calls.  kwitt 10/25/91
!                 include "bitwrk.com" removed
!               - no common blocks, everything goes through argument
!                 list.  modified for multitasking physics
!                 dd 95.10.20
!               - rewritten to remove gathers/scatters, which made
!                 code hard to decipher.  M. Branson 10/98
!               - qsats instead of qsatw is used in all the calls to eau_sat.
!                 Laura D. Fowler, 10/01.

!     SUBROUTINES CALLED:
!		 - eau_sat: calculation of saturation variables.

!     FUNCTIONS CALLED:
!		AMIN

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) :: &
         nlen, len
      real (kind=dbl_kind), intent(in) :: &
         dt

      real (kind=dbl_kind), intent(in), dimension(nlen,2) ::  &
         tl,  &     ! odd level potential temperature (nlm & nlm-1 layer)
         ql,  &     ! odd level specific humidity (nlm)
         pl2, &     ! even level pressure (top & base of nlm)
         plk2, &    ! even level (P/Pref)**kapa (top & base nlm)
         phil, &    ! odd level geopotential (nlm & nlm-1)
         imassl, &  ! (grav*0.01)/(layer pressure thickness) (nlm & nlm-1)
         qls        ! odd level saturation specific humidity (nlm)

      real (kind=dbl_kind), intent(in), dimension(len) :: &
         pl, &      ! odd level pressure (nlm)
         plk, &     ! odd level (P/Pref)**kapa (nlm)
         wbp        ! water vapor mixing ratio at level B+

!---------------------------------------------------------------------------
! INTENT INOUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), intent(inout), dimension(len) :: &
         tb, &      ! Temp at top of nlm (if nlm unsaturated and dry adiabatic)
         qob, &     ! Saturation spec. hum. at top of nlm using TB
         gammab     ! (L/Cp)*(qob/tb**2)

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), intent(out), dimension(nlen,2) :: &
         dtlci, &   ! temp increment due to LCI mixing (nlm & nlm-1)
         dqlci      ! spec. hum. increment due to LCI mixing (nlm & nlm-1)
      real (kind=dbl_kind), intent(out), dimension(len) :: &
         pc         ! cloud base pressure

      logical (kind=log_kind), intent(out), dimension(len) :: &
         nentr(len) ! Logical to indicate where nlm & nlm-1 both saturated

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(len) :: &
         wbploc, &  ! water vapor mixing ratio at level B+
         wm, &      ! PBL water vapor mixing ratio
         tll, &     ! temperature at nlm-1 after LCI mixing
         qol, &     ! saturation wvmr at nlm-1
         hm, &      ! PBL moist static energy
         hbp, &     ! moist static energy at nlm-1
         safe, &    ! LCI condition
         ts, &      ! surface temperature
         qos, &     ! saturation wvmr using tm(nlm) (after LCI mixing)
         am, &      ! time constant used to control LCI mixing
         tem, &     ! local work array (analogous to mass exchange between
!                     nlm & nlm-1 during LCI mixing)
         tm, &      ! PBL temperature
         pltem, &   ! odd level pressures in Pa (for eau_sat)
         wbpinit

      logical (kind=log_kind), dimension(len) :: &
         bitx, bity, bitz, &  ! various tests
         satpbl, &  ! is PBL saturated
         stable

      integer (kind=int_kind) :: &
         i, l      ! do loop indices
      real (kind=dbl_kind) ::  &
         taulci, &  ! LCI time scale
         dtdtau     ! dt/taulci

!----------------------------------------------------------------------------


      data taulci/3600./

      dtdtau = dt / taulci            

!-- Initialize output variables
      do i = 1,len
         qol(i) = qls(i,1)
         wm(i) = ql(i,2)
         wbploc(i) = wbp(i)
         wbpinit(i) = wbp(i)
         nentr(i) = .false.
         stable(i) = .false.
         do l = 1,2
           dtlci(i,l) = 0.
           dqlci(i,l) = 0.
         enddo
      enddo

!-- TEST FOR SATURATION IN THE PBL
!-- [pbl mixing ratio is greater than the saturation mixing ratio at the
!-- pbl top]
!-- NEW 11/98: initial SAFE value now computed outside and passed in
      do i=1,len
         satpbl(i) = qob(i) .lt. ql(i,2)
         if (satpbl(i)) then
           hm(i)   = spec_heat_cp * tl(i,2) + phil(i,2) + hltm * ql(i,2)
           if (safe(i).ge.0.) stable(i) = .true.
         endif
      enddo

!-- For LCI stable grid points, compute the cloud base pressure (pc)
!-- Also, if layer above (nlmm1) is also saturated, set logical nentr
!-- which means the entrainment rate will be zero.
!-- ** NEW 12/98 ** : cloud base pressure (pc) now compute in pbl_state
!--   and passed in.  can be modified below if LCI mixing occurs.

      do i=1,len
        if (satpbl(i)) then
!-- ** LCI stable case **

        if (stable(i)) then
!-- Used to compute the cloud base pressure here, but that is no longer
!-- necessary as it is computed in pbl_state.F before cloud.F is called.
!          ts(i) = tb(i)*plk2(i,2) / plk2(i,1)
!          pltem(i) = pl2(i,2)*100.
!          call eau_sat(1, pltem(i), ts(i), qsats=qos(i) )
!          pc(i) = pl2(i,1)+(pl2(i,2)-pl2(i,1))*(wm(i)-qob(i))
!     *             / (qos(i)-qob(i))
          
!-- ensure that cloud base pressure is not greater than the surface pressure
!          pc(i) = min(pl2(i,2),pc(i))

!-- also check that the saturation mixing ratio at the surface is not
!-- less than the saturation mixing ratio at the pbl top.  this can also
!-- cause pc > ps.  ISN'T THIS SUPERFLUOUS?
!          bity(i) = qos(i) .le. qob(i)
!          if (bity(i)) pc(i) = pl2(i,2)

!-- if layer nlmm1 is saturated also, set logical nentr = true which
!-- means that the entrainment rate will get set to zero in entrain.F
          if (qol(i) .lt. ql(i,1)) nentr(i) = .true.

        else
!-- ** LCI unstable case **
          tem(i) = (gammab(i) * (plk2(i,1) / plk(i))  &
              * ((hbp(i)-hm(i)) / hltm  &
              - (ql(i,1)-wm(i))) - (ql(i,1)-wm(i)))*imassl(i,2)  &
              - (wm(i)-qob(i)) * (imassl(i,2) + imassl(i,1))
          am(i)  = dtdtau / (imassl(i,2) + imassl(i,1))
          if (tem(i).gt.0.) then
            tem(i) = (wm(i) - qob(i)) / tem(i)
            if (am(i) .gt. tem(i)) am(i) = tem(i)
          endif
          tem(i) = am(i)*(hbp(i)-hm(i))/  &
                   (1.+(imassl(i,2)+imassl(i,1)) * am(i))
          hm(i)  = hm(i) + imassl(i,2)*tem(i)
          hbp(i) = hbp(i) - imassl(i,1)*tem(i)

          tem(i) = am(i)*(wbploc(i)-wm(i))/  &
                   (1.+(imassl(i,2)+imassl(i,1)) * am(i))
          wm(i)  = wm(i) + imassl(i,2)*tem(i)
          wbploc(i) = wbploc(i) - imassl(i,1)*tem(i)

!-- define temperature and water increments
          tm(i)       = (hm(i)  - phil(i,2) - hltm*wm(i)) /spec_heat_cp
          tll(i)      = (hbp(i) - phil(i,1) - hltm*wbploc(i))/spec_heat_cp
          dtlci(i,1) = tll(i) - tl(i,1)                                  
          dtlci(i,2) = tm(i)  - tl(i,2)                               
          dqlci(i,1) = wbploc(i) - wbpinit(i)
          dqlci(i,2) = wm(i)  - ql(i,2)                               

!-- check for special cases when clouds are left
          tb(i) = tm(i)*plk2(i,1)/plk(i)
!          write(*,*) i,tm(i),plk2(i,1),plk(i),tb(i)
          pltem(i) = pl2(i,1) * 100.
          call eau_sat(1, pltem(i), tb(i), qsats=qob(i))
          gammab(i) = gamfac*qob(i)/(tb(i)*tb(i))
          call eau_sat(1, pltem(i), tll(i), qsats=qol(i))

!          bity(i) = wbploc(i) .gt. qol(i)
          bitz(i) = wm(i) .gt. qob(i)

!-- check for nentr condition again now that mixing is done
          if ((wbploc(i).gt.qol(i)).and.(wm(i).gt.qob(i)))  &
             nentr(i) = .true.

!-- this is a really strange test, because am will ALWAYS be > 0 given
!-- how it is defined above (or possibly modified if tem > 0).  so bity
!-- should always be false
          bity(i) = am(i) .lt. 0.

!-- this should always be false since bity is always false
          bitx(i) = bity(i) .and. bitz(i)

!-- recompute the post-mixing PBL cloud base pressure
          ts(i)  = tm(i)*plk2(i,2)/plk(i)
          pltem(i) = pl2(i,2) * 100.
          call eau_sat(1,pltem(i),ts(i),qsats=qos(i))
          pc(i) = pl2(i,1)+(pl2(i,2)-pl2(i,1))*(wm(i)-qob(i))  &
                  / (qos(i)-qob(i))
          pc(i) = min(pl2(i,2),pc(i))

!-- this will always be true, which means that pc will always get set equal
!-- to the pbl top pressure -> we're pretending there's no cloud in the pbl
!-- when LCI is detected for a given grid point
          bitz(i) = .not. bitx(i)
          if (bitz(i)) pc(i) = pl2(i,1)
          bity(i) = qos(i) .le. qob(i)
          bitx(i) = bitx(i) .and. bity(i)
          if (bitx(i)) pc(i) = pl2(i,2)

        endif    ! LCI unstable points

        endif    ! saturated points
      enddo

      end subroutine cloud
!======================================================================
! END OF CLOUD
!======================================================================






      end module cpl_pbl

