!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module remapping

!-----------------------------------------------------------------------
! PURPOSE: read in data to perform interpolations to and from an 
!	atmosphere and surface grid 
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! COMMENTS: the module is written to accommodate an arbitrary number of
!	remappings (n_remap), but in general we will be interested in
!	only two remappings: atmos_to_sfc and sfc_to_atmos
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! DEFINITION OF N_REMAP
!
!   n_remap = 1 is the atmosphere to surface remapping
!   n_remap = 2 is the surface to atmosphere remapping
!
!   prec_remap is the precision of floating points in the remapping file
!	should be set to 'single' or 'double'
!-----------------------------------------------------------------------

	use kinds
        use model_parameters
	use domain_decomposition_sfc
	use netcdf_util
	use wrap_face_data
	use grid_metrics
	use grid_mask

        implicit none
	private
	save

	integer, parameter :: n_remap = 2

	integer (kind=int_kind), dimension(n_remap) :: 
     *		num_links(nsdm_sfc,n_remap),
     *		num_wgts(n_remap)

	real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     *		remap_matrix

	integer (kind=int_kind), allocatable, dimension(:,:,:) ::
     *          src_address_i, src_address_j,
     *          dst_address_i, dst_address_j

        public :: init_remapping, a_to_s_remapping, s_to_a_remapping


!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF INIT_REMAPPING
!======================================================================
	subroutine init_remapping

!-----------------------------------------------------------------------
! PURPOSE: read in info required to do remappings
!-----------------------------------------------------------------------


	integer :: i, j, indxd, ksdm, num_links_max, num_wgts_max, nl, l, ierr
	integer, parameter :: single = selected_real_kind(6),
     *			      double = selected_real_kind(13)

        integer (kind=int_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *         a2s_dst_beg, a2s_dst_len, s2a_src_beg, s2a_src_len
        integer (kind=int_kind), dimension(n_remap) :: lrecl
	character(len=8) :: prec_remap(n_remap)
        real (kind=single), dimension(3) :: tmp_s
        real (kind=double), dimension(3) :: tmp_d
        logical (kind=log_kind), dimension(1) :: lgrad=(/.false./)
        real (kind=dbl_kind), dimension(iim,jjm,1,nsdm) :: 
     *		work_atm, work_atm_gradx, work_atm_grady
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,1,nsdm_sfc) :: 
     *		work_sfc

        real (kind=dbl_kind) :: min_atm, min_sfc, max_atm, max_sfc,
     *      	sum_atm, sum_sfc, xmax, xmin, xsum

	if(my_task_sfc.eq.0)write(6,9)

        a2s_dst_beg = 0
        a2s_dst_len = 0
        s2a_src_beg = 0
        s2a_src_len = 0

!  open and read from the file with the mapping locations: a to s mapping
	open(unit=3,file=trim(datapath)//'data/a_to_s_remap_loc_sdm',
     *       form='unformatted',
     *       access='DIRECT',recl=8)
	do ksdm = 1,nsdm_sfc
          num_links(ksdm,1) = 0
	  do j = 1,jjm_sfc
	    do i = 1,iim_sfc
	      indxd = grid_indx_sfc(i,j,ksdm)
              if(grid_mask_sfc(i,j,ksdm)) then
                read(3,rec=indxd)
     *               a2s_dst_beg(i,j,ksdm), a2s_dst_len(i,j,ksdm)
                num_links(ksdm,1) = num_links(ksdm,1) + a2s_dst_len(i,j,ksdm)
              endif
	    enddo
	  enddo
	enddo
        close(3)
       
!  open and read from the file with the mapping locations: s to a mapping
	open(unit=3,file=trim(datapath)//'data/s_to_a_remap_loc_sdm',
     *       form='unformatted',
     *       access='DIRECT',recl=8)
	do ksdm = 1,nsdm_sfc
          num_links(ksdm,2) = 0
	  do j = 1,jjm_sfc
	    do i = 1,iim_sfc
	      indxd = grid_indx_sfc(i,j,ksdm)
              if(grid_mask_sfc(i,j,ksdm)) then
                read(3,rec=indxd)
     *               s2a_src_beg(i,j,ksdm), s2a_src_len(i,j,ksdm)
                num_links(ksdm,2) = num_links(ksdm,2) + s2a_src_len(i,j,ksdm)
              endif
	    enddo
	  enddo
	enddo
        close(3)

	num_links_max = maxval(num_links(:,:))

!  open the files with the remap weights, read headers
	open(unit=3,file=trim(datapath)//'data/a_to_s_remap_sdm',
     *       form='unformatted',
     *       access='DIRECT',recl=16)
        read(unit=3,rec=1)num_wgts(1),prec_remap(1),lrecl(1)
        close(3)

	open(unit=3,file=trim(datapath)//'data/s_to_a_remap_sdm',
     *       form='unformatted',
     *       access='DIRECT',recl=16)
        read(unit=3,rec=1)num_wgts(2),prec_remap(2),lrecl(2)
        close(3)

        num_wgts_max = maxval(num_wgts(:))

        allocate (src_address_i(num_links_max,2,nsdm_sfc))
        allocate (src_address_j(num_links_max,2,nsdm_sfc))
        allocate (dst_address_i(num_links_max,2,nsdm_sfc))
        allocate (dst_address_j(num_links_max,2,nsdm_sfc))
        allocate (remap_matrix(num_links_max,num_wgts_max,2,nsdm_sfc))
        remap_matrix = 0.0
        src_address_i = -9
        src_address_j = -9
        dst_address_i = -9
        dst_address_j = -9

!  read the remap weights: a to s
	open(unit=3,file=trim(datapath)//'data/a_to_s_remap_sdm',
     *       form='unformatted',
     *       access='DIRECT',recl=lrecl(1))
        do ksdm = 1,nsdm_sfc
          nl = 0
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              indxd = grid_indx_sfc(i,j,ksdm)
              if(grid_mask_sfc(i,j,ksdm)) then
                do l = a2s_dst_beg(i,j,ksdm),
     *                      a2s_dst_beg(i,j,ksdm)+a2s_dst_len(i,j,ksdm)-1
                  nl = nl + 1
                  if(trim(prec_remap(1)).eq.'single') then
                    read(3,rec=l+1)    
     *                   src_address_i(nl,1,ksdm), src_address_j(nl,1,ksdm),
     *                   dst_address_i(nl,1,ksdm), dst_address_j(nl,1,ksdm), 
     *                   tmp_s(1:num_wgts(1))
                         remap_matrix(nl,1:num_wgts(1),1,ksdm) = 
     *			    tmp_s(1:num_wgts(1))
                  else
                    read(3,rec=l+1)    
     *                   src_address_i(nl,1,ksdm), src_address_j(nl,1,ksdm),
     *                   dst_address_i(nl,1,ksdm), dst_address_j(nl,1,ksdm),
     *                   tmp_d(1:num_wgts(1))
                         remap_matrix(nl,1:num_wgts(1),1,ksdm) = 
     *			    tmp_d(1:num_wgts(1))
		  endif
                enddo
              endif
            enddo
          enddo
        enddo
        close(3)

!  read the remap weights: s to a
	open(unit=3,file=trim(datapath)//'data/s_to_a_remap_sdm',
     *       form='unformatted',
     *       access='DIRECT',recl=lrecl(2))
        do ksdm = 1,nsdm_sfc
          nl = 0
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              indxd = grid_indx_sfc(i,j,ksdm)
              if(grid_mask_sfc(i,j,ksdm)) then
                do l = s2a_src_beg(i,j,ksdm),
     *                      s2a_src_beg(i,j,ksdm)+s2a_src_len(i,j,ksdm)-1
                  nl = nl + 1
                  if(trim(prec_remap(2)).eq.'single') then
                    read(3,rec=l+1)    
     *                   src_address_i(nl,2,ksdm), src_address_j(nl,2,ksdm),
     *                   dst_address_i(nl,2,ksdm), dst_address_j(nl,2,ksdm), 
     *                   tmp_s(1:num_wgts(2))
                         remap_matrix(nl,1:num_wgts(2),1,ksdm) = 
     *			    tmp_s(1:num_wgts(2))
                  else
                    read(3,rec=l+1)    
     *                   src_address_i(nl,2,ksdm), src_address_j(nl,2,ksdm),
     *                   dst_address_i(nl,2,ksdm), dst_address_j(nl,2,ksdm),
     *                   tmp_d(1:num_wgts(2))
                         remap_matrix(nl,1:num_wgts(1),2,ksdm) = 
     *			    tmp_d(1:num_wgts(2))
		  endif
                enddo
              endif
            enddo
          enddo
        enddo
        close(3)

       

  9     format(//,' INITIALIZING THE ATMOS TO SURFACE REMAPPING MODULE',/)
 17	format('    min/max of src data     : ',2f15.9)
 18	format('    min/max/sum of dst data : ',3f15.9)  


!-----------------------------------------------------------------------
! test for conservation
!-----------------------------------------------------------------------
!  a to s test
	  work_atm = 1.0_dbl_kind
	  work_atm_gradx = 0.0_dbl_kind
	  work_atm_grady = 0.0_dbl_kind
	  work_sfc = 0.0_dbl_kind
	  call a_to_s_remapping(work_atm, work_atm_gradx, work_atm_grady, 
     &                          lgrad, 1, work_sfc)
          xmax = -1.e36
          xmin = 1.e36
          xsum = 0.0
          do ksdm = 1,nsdm
           do j = 1,jjm
            do i = 1,iim
              if(grid_center_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                xmax = max(xmax, work_atm(i,j,1,ksdm))
                xmin = min(xmin, work_atm(i,j,1,ksdm))
                xsum = xsum + work_atm(i,j,1,ksdm)
              endif
	    enddo
	   enddo
	  enddo
#ifdef mpi
        call MPI_ALLREDUCE(xmax, max_atm, 1,
     &                     mpi_float_type, MPI_MAX, MPI_COMM_ATMOS, ierr)
        call MPI_ALLREDUCE(xmin, min_atm, 1,
     &                     mpi_float_type, MPI_MIN, MPI_COMM_ATMOS, ierr)
        call MPI_ALLREDUCE(xsum, sum_atm, 1,
     &                     mpi_float_type, MPI_SUM, MPI_COMM_ATMOS, ierr)
#else
        max_atm = xmax
        min_atm = xmin
        sum_atm = xsum
#endif
          xmax = -1.e36
          xmin = 1.e36
          xsum = 0.0
          do ksdm = 1,nsdm_sfc
           do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(grid_mask_sfc(i,j,ksdm)) then
                xmax = max(xmax, work_sfc(i,j,1,ksdm))
                xmin = min(xmin, work_sfc(i,j,1,ksdm))
                xsum = xsum + work_sfc(i,j,1,ksdm)
              endif
	    enddo
	   enddo
	  enddo
#ifdef mpi
        call MPI_ALLREDUCE(xmax, max_sfc, 1,
     &                     mpi_float_type, MPI_MAX, MPI_COMM_SFC, ierr)
        call MPI_ALLREDUCE(xmin, min_sfc, 1,
     &                     mpi_float_type, MPI_MIN, MPI_COMM_SFC, ierr)
        call MPI_ALLREDUCE(xsum, sum_sfc, 1,
     &                     mpi_float_type, MPI_SUM, MPI_COMM_SFC, ierr)
#else
        max_sfc = xmax
        min_sfc = xmin
        sum_sfc = xsum
#endif

        if(my_task.eq.0) then
	  write(6,*)' TESTING A2S REMAPPING FOR CONSERVATION AND ACCURACY'
	  write(6,17) min_atm, max_atm
	  write(6,18) min_sfc, max_sfc, sum_sfc
        endif
          
!  s to a test
	  work_atm = 0.0_dbl_kind
	  work_sfc = 0.0_dbl_kind
          where(grid_mask_sfc(:,:,:)) work_sfc(:,:,1,:) = 1.0_dbl_kind
	  call s_to_a_remapping(work_sfc, 1, work_atm)

          xmax = -1.e36
          xmin = 1.e36
          xsum = 0.0
          do ksdm = 1,nsdm
           do j = 1,jjm
            do i = 1,iim
              if(grid_center_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                xmax = max(xmax, work_atm(i,j,1,ksdm))
                xmin = min(xmin, work_atm(i,j,1,ksdm))
                xsum = xsum + work_atm(i,j,1,ksdm)
              endif
	    enddo
	   enddo
	  enddo
#ifdef mpi
        call MPI_ALLREDUCE(xmax, max_atm, 1,
     &                     mpi_float_type, MPI_MAX, MPI_COMM_ATMOS, ierr)
        call MPI_ALLREDUCE(xmin, min_atm, 1,
     &                     mpi_float_type, MPI_MIN, MPI_COMM_ATMOS, ierr)
        call MPI_ALLREDUCE(xsum, sum_atm, 1,
     &                     mpi_float_type, MPI_SUM, MPI_COMM_ATMOS, ierr)
#else
        max_atm = xmax
        min_atm = xmin
        sum_atm = xsum
#endif
          xmax = -1.e36
          xmin = 1.e36
          xsum = 0.0
          do ksdm = 1,nsdm_sfc
           do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(grid_mask_sfc(i,j,ksdm)) then
                xmax = max(xmax, work_sfc(i,j,1,ksdm))
                xmin = min(xmin, work_sfc(i,j,1,ksdm))
                xsum = xsum + work_sfc(i,j,1,ksdm)
              endif
	    enddo
	   enddo
	  enddo
#ifdef mpi
        call MPI_ALLREDUCE(xmax, max_sfc, 1,
     &                     mpi_float_type, MPI_MAX, MPI_COMM_SFC, ierr)
        call MPI_ALLREDUCE(xmin, min_sfc, 1,
     &                     mpi_float_type, MPI_MIN, MPI_COMM_SFC, ierr)
        call MPI_ALLREDUCE(xsum, sum_sfc, 1,
     &                     mpi_float_type, MPI_SUM, MPI_COMM_SFC, ierr)
#else
        max_sfc = xmax
        min_sfc = xmin
        sum_sfc = xsum
#endif
          
        if(my_task.eq.0) then
	  write(6,*)' TESTING S2A REMAPPING FOR CONSERVATION AND ACCURACY'
	  write(6,17) min_sfc, max_sfc
	  write(6,18) min_atm, max_atm, sum_atm
        endif


!-----------------------------------------------------------------------
! everyone wait
!-----------------------------------------------------------------------
#ifdef mpi
 	CALL MPI_BARRIER (MPI_COMM_SFC,ierr)
#endif

	END SUBROUTINE init_remapping
!===========================================================================
! END OF INIT_REMAPPING
!===========================================================================


!===========================================================================
! BEGINNING OF A_TO_S_REMAPPING
!===========================================================================
	subroutine a_to_s_remapping(xin, xgrad, ygrad, lgrad, nvar, xout)
	implicit none

	integer (kind=int_kind), intent(in) :: nvar
	real (kind=dbl_kind), dimension(iim,jjm,nvar,nsdm), intent(in) ::
     &		 xin, xgrad, ygrad
	real (kind=dbl_kind), 
     *        dimension(iim_sfc,jjm_sfc,nvar,nsdm_sfc), intent(out) :: xout
	logical (kind=log_kind), dimension(nvar), intent(in) :: lgrad
	integer (kind=int_kind):: iremap, i, ksdm, j

!-----------------------------------------------------------------------
! do remapping
!-----------------------------------------------------------------------
	iremap = 1
	xout(:,:,:,:) = 0.0

        do ksdm = 1,nsdm
 	  do i=1,num_links(ksdm,iremap)
        xout(dst_address_i(i,iremap,ksdm),dst_address_j(i,iremap,ksdm),:,ksdm) =
     *  xout(dst_address_i(i,iremap,ksdm),dst_address_j(i,iremap,ksdm),:,ksdm) +
     *  xin(src_address_i(i,iremap,ksdm),src_address_j(i,iremap,ksdm),:,ksdm) *
     *          remap_matrix(i,1,iremap,ksdm)
 	  enddo

          if(num_wgts(iremap).ge.3) then
 	    do j=1,nvar
              if(lgrad(j)) then
 	         do i=1,num_links(ksdm,iremap)
        xout(dst_address_i(i,iremap,ksdm),dst_address_j(i,iremap,ksdm),j,ksdm) =
     *  xout(dst_address_i(i,iremap,ksdm),dst_address_j(i,iremap,ksdm),j,ksdm) +
     *  xgrad(src_address_i(i,iremap,ksdm),src_address_j(i,iremap,ksdm),j,ksdm)
     *           *remap_matrix(i,2,iremap,ksdm) + 
     *  ygrad(src_address_i(i,iremap,ksdm),src_address_j(i,iremap,ksdm),j,ksdm)
     *           *remap_matrix(i,3,iremap,ksdm)

 	         enddo
              endif
 	    enddo
          endif
 	enddo

	end subroutine a_to_s_remapping
!===========================================================================
! END OF A_TO_S_REMAPPING
!===========================================================================


!===========================================================================
! BEGINNING OF S_TO_A_REMAPPING
!===========================================================================
        subroutine s_to_a_remapping(xin, nvar, xout)
        implicit none

        integer (kind=int_kind), intent(in) :: nvar
	real (kind=dbl_kind), dimension(iim,jjm,nvar,nsdm), intent(out) :: xout
	real (kind=dbl_kind), 
     *        dimension(iim_sfc,jjm_sfc,nvar,nsdm_sfc), intent(in) :: xin
        integer (kind=int_kind) :: iremap, i, ksdm

!-----------------------------------------------------------------------
! do remapping
!-----------------------------------------------------------------------
	iremap = 2
	xout(:,:,:,:) = 0.0

        do ksdm = 1,nsdm
 	  do i=1,num_links(ksdm,iremap)
        xout(dst_address_i(i,iremap,ksdm),dst_address_j(i,iremap,ksdm),:,ksdm) =
     *  xout(dst_address_i(i,iremap,ksdm),dst_address_j(i,iremap,ksdm),:,ksdm) +
     *  xin(src_address_i(i,iremap,ksdm),src_address_j(i,iremap,ksdm),:,ksdm) *
     *          remap_matrix(i,1,iremap,ksdm)
 	  enddo
 	enddo

        call wrap_fc( max_level,
     *                iim,
     *                jjm,
     *                nvar,
     *                nsdm,
     *                xout(:,:,:,:))

        end subroutine s_to_a_remapping
!===========================================================================
! END OF S_TO_A_REMAPPING
!===========================================================================

        end module remapping

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
