c-------------------------------------------------------------------

      SUBROUTINE SWRAD (grav2, cp, 
     *   NOZ, CLIZON, S0, CODanv, codnoanv, OM0,
     *   GWIP, ALVDF, ALNDF, ALVDR, ALNDR, COSZ, PL20, DPL, TL, QL, O3L,
     *   O3TOP, CLD, CLU, SWINC, DSWCLR, DSWTOP, SSCLR, SS, DSWBOT, ASL,
     *   ASLCLR, RADVBC, RADVDC, RADNBC, RADNDC, sfac, dswbcl, fracanv,
     *   radvbcc, radvdcc, radnbcc, radndcc, dswtom, dswctom,
     *   nlen, nlm, nls, len, nocld )

      use kinds

      IMPLICIT none

c-------------------------------------------------------------------
c	Calculates heating rates in the atmosphere due to solar
c	radiation.  Calculates amount of solar radiation reaching
c	the earth.

c     REFERENCES: Harshvardhan, R. Davies, D. A. Randall and T. G. Corsetti
c        (1987) "A fast radiation parameterization for general circulation
c        models", J. Geophys. Res., 92, 1009-1016.

c     MODIFICATIONS:
c	- bitwrk.com include removed.  sequential array listim
c	  done in routine with i8vint.  kwitt 7/5/91
c	- inlined gcmlib.F subroutines.  kwitt 1/22/92
c	- renamed following local variables to parameter names IM NP NPP1 LMP1
c	  LS1 LS LM IMP2 IMP2LM   kwitt 1/31/92
c	- replace common /work/ with argument list.   dd 11/09/92
c	- cloud optical depth, single scatter albedo, and asymmetry constant now
c	  passed in as arguments instead of being prescribed from within the
c	  radiation code.   dd 02/24/92
c       - clear sky components of surface insolation now passed out 
c         argument list.  dd 02/20/95
c       - top of model fluxes added. dd 08/21/95

c     SUBROUTINES CALLED:
c		setsw
c     FUNCTIONS CALLED:
c		none

C     ARGUMENT LIST VARIABLES
c		grav2	grav / 100
c		cp	dray air specific heat
c		noz	no ozone logical
c		clizon	use climatological ozone logical
c               nocld   clouds are invisible
c		s0	solar constant, adjusted for earth sun distance (w/m**2)
c		alXYY	surface albedos
c		  	X=v	visible
c			X=n	near IR
c			YY=df	diffuse beam
c			YY=dr	direct beam
c		cosz	cosine (zenith angle) of the sun
c		pl20	layer edge pressures (mb)
c		dpl	layer pressure thicknesses (mb)
c		tl	mid-layer temperatures (K)
c		ql	mid-layer specific humidity (Kg/Kg)
c		o3l	mid-layer ozone mixing ratios (Kg/Kg)
c		o3top	ozone concentration above model top (Kg/Kg)
c		cld	supersaturation cloud amount
c		clu	convective anvil cloud amount
c		sfac	a factor less than 1 to multiply cosz for
c			the no-diurnal cycle case
c		fracanv	fractional anvils logical
c		codanv	  cloud optical depth (total anvil coverage)
c		codnoanv  cloud optical depth (no anvils)
c		om0	single scatter albedo
c		gwip	asymmetry constant

C        Output variables: 
c		swinc	solar radiative flux incident on the top of
c			the atmosphere (w/m**2)
c		dswclr	clear sky dswtop
c		dswtop	net solar radiative flux absorbed at the top
c			of the atmosphere (w/m**2)
c		ss	net solar radiative flux absorbed by the
c			earths surface (w/m**2)
c		ssclr	clear sky ss
c		dswbot	solar radiative flux incident on the earths
c			surface (w/m**2)
c		dswbcl	clear sky solar radiative flux incident
c			on the earths surface (w/m**2)
c		asl	atmospheric heating rates (K/time-step)
c		aslclr	clear sky asl
c		radXYc	net radiative flux at the earths sruface for
c			the xy part of the beam
c			X=v	visible
c			X=n	near IR
c			Y=d	diffuse beam
c			Y=b	direct beam
      integer (kind=int_kind) ::
     *   nlen, len, nlm, nls
      real (kind=dbl_kind) ::
     *   grav2, cp,
     *   PL20(nlen,nlm+1), TL(nlen,NLM), QL(nlen,NLM),
     *   CLD(nlen,NLM), CLU(nlen,NLM), DPL(nlen,NLM),
     *   DSWCLR(nlen), DSWTOP(nlen), SSCLR(nlen), COSZ(nlen),
     *   ASLCLR(nlen,NLM), RADVBC(nlen), RADVDC(nlen),
     *   RADNBC(nlen), RADNDC(nlen), DSWBOT(nlen), dswbcl(len),
     *   sfac(nlen), O3L(nlen,NLM), SWINC(nlen), SS(nlen),
     *   ASL(nlen,NLM), ALVDR(nlen), ALNDR(nlen), ALVDF(nlen),
     *   ALNDF(nlen), codanv(nlen,nlm+1), om0(nlen,nlm+1),
     *   gwip(nlen,nlm+1), codnoanv(nlen,nlm+1), o3top(nlen),
     *   radvbcc(nlen), radvdcc(nlen), radnbcc(nlen),
     *   radndcc(nlen), dswtom(nlen), dswctom(nlen)
      LOGICAL NOZ, CLIZON, fracanv, nocld

c     LOCAL VARIABLES:

      logical (kind=log_kind) ::
     *         BITX(len*(nlm+1)), BITD(len*(nlm+2)),
     *         BITN(len*(nlm+2)), DOOZ
      real (kind=dbl_kind) ::
     *   PU(len*(nlm+2)), TA(len*(nlm+1)), WA(len*(nlm+1)),
     *   OA(len*(nlm+1)), DP(len*(nlm+1)),
     *   DPC(len*(nlm+1)), SCOSZ(len), CMU(len), COSMAG(len),
     *   CSS(len*(nlm+1)), CCU(len*(nlm+1)), SL(len),
     *   ACLR(len*(nlm+1)),
     *   DSCLR(len), SC(len), RVBL(len), RVDL(len),
     *   RNBL(len), RNDL(len), RVDC(len), RNBC(len),
     *   RNDC(len), AGV(len), AGN(len), RSURFN(len),
     *   DSCLD(len), ACLD(len*(nlm+1)), RVBC(len), RSURFV(len),
     *   gsfac(len), eoanv(len*(nlm+1)), om(len*(nlm+1)),
     *   gwi(len*(nlm+1)),
     *   eonoanv(len*(nlm+1)), S0,
     *   dscldtom(len), dsclrtom(len)
      real (kind=dbl_kind) :: fac
      integer (kind=int_kind) ::
     *   NSOL, NSOLP1, NSLMP1, NSOLNP, NCLD, LITX(len*(nlm+1))
     *,  nlmp1, nnp, nnpp1, imlmp1, imnnp, nlsp1, i, il
     *,  nst2p1, nsollm, nzercd, nlimwa, nrstwa, nwa1, l, j

c----------------------------------------------------------------

      nlmp1 = nlm+1
      nlsp1 = nls+1
      nnp = nlm+2
      nnpp1 = nnp+1
      imlmp1 = len*nlmp1
      imnnp = len*nnp
c     fill sequential array of indices

      DOOZ   = ((.NOT. NOZ) .OR. CLIZON)
      FAC    = grav2 / cp

C     SET BITS FOR DAYTIME GRID POINTS

      do i=1,len
         bitx(i) = cosz(i) .ge. .01
      enddo

      nsol = 0
      DO I=1,len
         if(bitx(i)) nsol = nsol + 1
         SS(I)     = 0.
         SWINC(I)  = 0.
         SSCLR(I)  = 0.
         DSWTOP(I) = 0.
         DSWTOm(I) = 0.
         DSWcTOm(I) = 0.
         DSWBOT(I) = 0.
         DSWCLR(I) = 0.
         DSWBCL(I) = 0.
         RADVBC(I) = 0.
         RADVDC(I) = 0.
         RADNBC(I) = 0.
         RADNDC(I) = 0.
         RADVBCc(I) = 0.
         RADVDCc(I) = 0.
         RADNBCc(I) = 0.
         RADNDCc(I) = 0.
         RVBC(I)   = 0.
         RVDC(I)   = 0.
         RNBC(I)   = 0.
         RNDC(I)   = 0.
         SC(I)     = 0.
      ENDDO

      DO IL=1,IMLMP1
         ACLD(IL)  = 0.
      ENDDO

      DO l=1,nlm
         DO IL=1,len
            ASL(IL,l)    = 0.
            ASLCLR(IL,l) = 0.
         ENDDO
      ENDDO

      IF (NSOL .EQ. 0) RETURN

      NSOLP1 = NSOL + 1
      NST2P1 = NSOL + NSOLP1
      NSLMP1 = NSOL * nLMP1
      NSOLLM = NSOL * nLM
      NSOLNP = NSOL * nNP
      NZERCD = NSOL * NLSP1
      NLIMWA = NSOL * nLS
      NRSTWA = NSLMP1 - NLIMWA
      NWA1 = NLIMWA + 1

      DO L=1,nLM
         do i=1,len
            bitx(i+l*len) = bitx(i)
         enddo
      ENDDO

      DO I=1,NSOL
         PU(I) = 0.
      ENDDO

      j = 1
      do l = 1,nlmp1
         do i=1,len
            if (bitx(i)) then
               litx(j) = i
               j = j+1
            endif
         enddo
      enddo

      do l = 1,nlmp1
         do i = 1,nsol
            il = i + nsol*(l-1) 
            pu(nsol+il) = pl20(litx(i),l)
            om(il) = om0(litx(i),l)
            eoanv(il) = codanv(litx(i),l)
            eonoanv(il) = codnoanv(litx(i),l)
            gwi(il) = gwip(litx(i),l)
         enddo
      enddo
      do l = 1,nlm
         do i = 1,nsol
            il = i + nsol*(l-1) 
            dp(nsol+il) = dpl(litx(i),l)
            ta(nsol+il) = tl(litx(i),l)
            wa(nsol+il) = ql(litx(i),l)
            css(nsol+il) = cld(litx(i),l)
            ccu(nsol+il) = clu(litx(i),l)
         enddo
      enddo
      if(nocld) then
         do l = 1,nlm
            do i = 1,nsol
               il = i + nsol*(l-1) 
               css(nsol+il) = 0.0
               ccu(nsol+il) = 0.0
            enddo
         enddo
      endif


      IF (CLIZON.or..not.noz) THEN
         do l = 1,nlm
            do i = 1,nsol
               il = i + nsol*(l-1) 
               oa(nsol+il) = o3l(litx(i),l)
            enddo
         enddo
         do i = 1,nsol
            oa(i) = o3top(litx(i))
         enddo
      ENDIF
      IF (.NOT. NOZ) THEN
         do i = 1,nslmp1
            oa(i) = max(.1e-9_dbl_kind, oa(i))
         enddo
      ENDIF

      do i = 1,nsol
         agv(i) = alvdf(litx(i))
         agn(i) = alndf(litx(i))
         rsurfv(i) = alvdr(litx(i))
         rsurfn(i) = alndr(litx(i))
         cmu(i) = cosz(litx(i))
         gsfac(i) = sfac(litx(i))
      enddo

      DO I=1,NSOL
         DP(I) = PU(NSOL+I)
         TA(I) = TA(NSOL+I)
         WA(I) = WA(NSOL+I)
      ENDDO

      DO I=1,NSLMP1
         DPC(I) = DP(I)
      ENDDO

      DO I=1,NZERCD
         CSS(I) = 0.
         CCU(I) = 0.
      ENDDO

      DO I=1,NLIMWA
         WA(I) = .000003
      ENDDO

      do i=1,nrstwa
         wa(nwa1+i-1) = max(.1e-22_dbl_kind, wa(nwa1+i-1))
      enddo

      DO I=1,NSOL
         COSMAG(I)  = 35. / (sqrt (1224. * CMU(I) * CMU(I) + 1.))
         SCOSZ(I)   = S0 * gsfac(i) * CMU(I)
      ENDDO

      do i = 1,nsol 
         swinc(litx(i)) = scosz(i)
      enddo

      CALL SETSW (PU,TA,WA,OA
     *,           Dp,DPC,SCOSZ,CMU,COSMAG
     *,           CSS,CCU,SL,ACLR,DSCLR,SC,ACLD
     *,           RVBL,RVDL,RNBL,RNDL,RVBC
     *,           RVDC,RNBC,RNDC,AGV,RSURFV
     *,           AGN,RSURFN,DSCLD,DOOZ,BITD
     *,           NSOL,NSOLP1,NSLMP1,NSOLNP,NCLD,fracanv
     *,           eoanv,eonoanv,om,gwi,dsclrtom,dscldtom
     *,           len, imnnp, imlmp1, nlmp1 )

      do i = 1,nsol 
         ssclr(litx(i)) = sl(i)
         dswclr(litx(i)) = dsclr(i)
         dswctom(litx(i)) = dsclrtom(i)
         radvbcc(litx(i)) = rvbl(i)
         radvdcc(litx(i)) = rvdl(i)
         radnbcc(litx(i)) = rnbl(i)
         radndcc(litx(i)) = rndl(i)
      enddo
      DO I=1,len
         DSWBcl(I) = RADVBcC(I) + RADVDcC(I) + RADNBcC(I) + RADNDcC(I)
      ENDDO

      if (ncld .ne. 0) then
         do i=1,nslmp1
            bitn(i) = .not. (bitd(i))
            if (bitn(i)) acld(i) = aclr(i)
         enddo

         do i=1,nsol
            if (bitn(i)) then
               rvbc(i) = rvbl(i)
               rvdc(i) = rvdl(i)
               rnbc(i) = rnbl(i)
               rndc(i) = rndl(i)
               sc(i) = sl(i)
               dscld(i) = dsclr(i)
               dscldtom(i) = dsclrtom(i)
            endif
         enddo
      endif

      do i = 1,nsol 
         ss(litx(i)) = sc(i)
         dswtop(litx(i)) = dscld(i)
         dswtom(litx(i)) = dscldtom(i)
         radvbc(litx(i)) = rvbc(i)
         radvdc(litx(i)) = rvdc(i)
         radnbc(litx(i)) = rnbc(i)
         radndc(litx(i)) = rndc(i)
      enddo
      do l = 1,nlm 
         do i = 1,nsol 
            il = i + nsol*(l-1) 
            aslclr(litx(i),l) = aclr(nsol+il)
            asl(litx(i),l) = acld(nsol+il)
         enddo
      enddo

      DO I=1, len
         DSWBOT(I) = RADVBC(I) + RADVDC(I) + RADNBC(I) + RADNDC(I)
      ENDDO

      DO l=1,nlm
         DO I=1, len
            ASLCLR(I,l) = ASLCLR(I,l) * FAC / DPL(I,l)
            ASL(I,l) = ASL(I,l) * FAC / DPL(I,l)
         ENDDO
      ENDDO

      RETURN
      END
