!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module post_step_module

!-----------------------------------------------------------------------
! PURPOSE: update the diagnostic variables of the atmosphere based on
!	the new time step (n4) prognostic variables
!-----------------------------------------------------------------------

        use kinds
        use physical_parameters
        use model_parameters
        use grid_mask
        use grid_metrics
        use operators
        use prognostics
        use timer
        use hstatc
	  use sigma_coordinate
	  use utility
	  use eau_util
#ifndef no_pbl
#ifdef link_to_gcm_physics
	use pbl_parameters
#endif
#endif

        implicit none
	private

	integer (kind=int_kind) :: k,icorner,ksdm
	
	real (kind=dbl_kind) :: work_c(ncorners,iim,jjm)

	public :: post_step


!**********************************************************************

        contains

!**********************************************************************

!======================================================================
! BEGINNING OF POST_STEP
!======================================================================
        subroutine post_step

        implicit none

!-----------------------------------------------------------------------
! PURPOSE: compute diagnostically determined variables
!-----------------------------------------------------------------------

        real (kind=dbl_kind), dimension(iim,jjm,nlm) :: temperature

!------------------------------------------------------------------
! begin loop over ksdm
!------------------------------------------------------------------
        do ksdm=1,nsdm

#ifdef no_pbl
!-----------------------------------------------------------------------
! get pressure thickness (Arakawa's Pi)
!-----------------------------------------------------------------------
        call timer_start('hstatc')
        call get_pthick(
     *          ps        (:,:,n4,ksdm),
     *          pthick    (:,:,:,n4, ksdm),
     *          invpthick (:,:,:,n4, ksdm) )

!-----------------------------------------------------------------------
! compute pressure that layer edges and layer centers
!-----------------------------------------------------------------------
        call press(
     *          plgl2 (:,:,: ,ksdm),
     *          plg   (:,:,: ,ksdm),
     *          bpgl2 (:,:,: ,ksdm),
     *          bpg   (:,:,: ,ksdm),
     *          ps    (:,:,n4,ksdm),
     *          iim, jjm)

#else
!-----------------------------------------------------------------------
! get pressure thickness (Arakawa's Pi)
!-----------------------------------------------------------------------
        call timer_start('hstatc')
        call get_pthick(
     *          ps        (:,:,n4,ksdm),
     *          pb        (:,:,n4,ksdm),
     *          pthick    (:,:,:,n4, ksdm),
     *          invpthick (:,:,:,n4, ksdm) )

!-----------------------------------------------------------------------
! compute pressure that layer edges and layer centers
!-----------------------------------------------------------------------
        call press(
     *          plgl2 (:,:,: ,ksdm),
     *          plg   (:,:,: ,ksdm),
     *          bpgl2 (:,:,: ,ksdm),
     *          bpg   (:,:,: ,ksdm),
     *          ps    (:,:,n4,ksdm),
     *          pb    (:,:,n4,ksdm),
     *          iim, jjm)

#endif
!-----------------------------------------------------------------------
! average theta to layer edges
!-----------------------------------------------------------------------
        call get_tracerl2(
     *          bpg      (:,:,:,  ksdm       ),
     *          bpgl2    (:,:,:,  ksdm       ),
     *          th       (:,:,:, n4,  ksdm   ),
     *          thl2     (:,:,:,  ksdm       ),
     *          iim, jjm, nlm, npbl, 'THETA'  )

#ifdef eau_ng
!-----------------------------------------------------------------------
! calculate elqng variables to layer edges
!-----------------------------------------------------------------------
        call get_elqngl2(
     *          dsigdt (:,:,:,ksdm),
     *          elqng  (:,:,:,:,n4,ksdm),
     *          elqngl2(:,:,:,:,ksdm),
     *          iim, jjm, nlm )
#endif

!-----------------------------------------------------------------------
! compute geopotential height
!-----------------------------------------------------------------------
        call get_geopotential(
     *          bpgl2 (:,:,:,ksdm   ),
     *          bpg   (:,:,:,ksdm   ),
     *          phis  (:,:,ksdm     ),
     *          th    (:,:,:,n4,ksdm),
     *          thl2  (:,:,:,ksdm   ),
     *          sigl2                ,
     *          phi   (:,:,:,ksdm   ),
     *          phil2 (:,:,:,ksdm   ), 
     *          iim, jjm, nlm  )
   
!---------------------------------------------------------------------------
! compute water vapor saturation mixing ratio
!---------------------------------------------------------------------------
        temperature(:,:,:) = th(:,:,:,n4,ksdm) * bpg(:,:,:,ksdm) ! temperature
        call eau_sat(iim*jjm*nlm, plg(:,:,:,ksdm),
     *               temperature, qsats=qs(:,:,:,ksdm) )

!-----------------------------------------------------------------------
! average non water vapor tracers to layer edge
!-----------------------------------------------------------------------

        do k = 2,5
           call get_tracerl2(
     *          bpg      (:,:,:,  ksdm       ),
     *          bpgl2    (:,:,:,  ksdm       ),
     *          tracer   (:,:,:,  k, n4, ksdm),
     *          tracerl2 (:,:,:,  k,  ksdm   ),
     *          iim, jjm, nlm, npbl, 'ARITHMETIC'  )
        enddo

        do k = 6,ntracer
           call get_tracerl2(
     *          bpg      (:,:,:,  ksdm       ),
     *          bpgl2    (:,:,:,  ksdm       ),
     *          tracer   (:,:,:,  k, n4, ksdm),
     *          tracerl2 (:,:,:,  k,  ksdm   ),
     *          iim, jjm, nlm, npbl, 'ARITHMETIC'  )
        enddo

!-----------------------------------------------------------------------
! compute layer edge values of temperature, water vapor and qsat and
!   dry static energy
!----------------------------------------------------------------------- 
        call get_waterl2(
     *          plgl2    (:,:,:,  ksdm       ),
     *          bpg      (:,:,:,  ksdm       ),
     *          bpgl2    (:,:,:,  ksdm       ),
     *          ssl      (:,:,:,  ksdm       ),
     *          ssll2    (:,:,:,  ksdm       ),
     *          qs       (:,:,:,  ksdm       ),
     *          qsl2     (:,:,:,  ksdm       ),
     *          phi      (:,:,:,  ksdm       ),
     *          phil2    (:,:,:,  ksdm       ),
     *          th       (:,:,:,n4,  ksdm       ),
     *          thl2     (:,:,:,  ksdm),
     *          tracer   (:,:,:,  iq, n4, ksdm),
     *          tl2      (:,:,:,  ksdm),
     *          tracerl2 (:,:,:,  iq,  ksdm   ),
     *          iim, jjm, nlm, npbl  )

!-----------------------------------------------------------------------
! compute specific volume
!-----------------------------------------------------------------------
        call get_alphasig(
     *          plgl2    (:,:,:,ksdm   ),
     *          plg      (:,:,:,ksdm   ),
     *          bpgl2    (:,:,:,ksdm   ),
     *          bpg      (:,:,:,ksdm   ),
     *          th       (:,:,:,n4,ksdm),
     *          alphasig (:,:,:,ksdm   ),
     *          alpha    (:,:,:,ksdm   )  )
        call timer_stop('hstatc')

#ifndef no_pbl
!-------------------------------------------------------------------------
! compute spdm and shear1
!-------------------------------------------------------------------------
	call timer_start('velocity')

!-------------------------------------------------------------------------
! initialize spdm and average over the pbl layers
!-------------------------------------------------------------------------
	spdm(:,:,ksdm) = c0
	do k=nlm-npbl+1,nlm
          spdm(:,:,ksdm) = spdm(:,:,ksdm) + 
     *	   sqrt(max( 0.0_dbl_kind, 2.0_dbl_kind*ke(:,:,k,ksdm)) )*dsig(k)
	enddo
	spdm(:,:,ksdm) = spdm(:,:,ksdm)*grid_center_mask(:,:,ksdm)

!-------------------------------------------------------------------------
! compute shear across pbl top at each cell wall
! average shear from cell walls to cell center
!-------------------------------------------------------------------------
	shear1(:,:,ksdm) = c0
	work_c = c0
	k = nlm - npbl
	do icorner=1,ncorners
	  work_c(:,:,:) = (u(1,:,:,:,k,n4,ksdm)-u(1,:,:,:,k+1,n4,ksdm))**2 +
     *                    (u(2,:,:,:,k,n4,ksdm)-u(2,:,:,:,k+1,n4,ksdm))**2
	enddo
        call calculate_center_kinetic_energy(
     *          ipad,
     *          iim,
     *          jjm,
     *          ndata,
     *          nspace,
     *          ncorners,
     *          sub_area(:,:,:,:,ksdm),
     *          area_inv(:,:,ksdm),
     *          work_c(:,:,:),
     *          shear1(:,:,ksdm)  )

	shear1(:,:,ksdm) = 0.5 * shear1(:,:,ksdm)*grid_center_mask(:,:,ksdm)
	call timer_stop('velocity')
#endif

        enddo   ! do ksdm = 1,nsdm

	end subroutine post_step
!======================================================================
! END OF POST_STEP
!======================================================================
 
        end module post_step_module

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
