!-----------------------------------------------------------------------

      subroutine rad_control
     +               (   ksdm , n_physics, n_pbp
     +,               do_phys ,    tofday
     +,                 dtdyn ,      lon0
     +,                  lat0 ,      pl2g
     +,                 bpgl2 ,       tl2
     +,                   plg ,       bpg
     +,                    th ,    tracer
     +,                 pblcw ,     pblci
     +,                climo3 ,     dlcup
     +,                 dicup ,     alvdr
     +,                 alvdf ,     alndr
     +,                 alndf ,    tgeff4
     +,                    pc
     +,                 cldpc ,   log_sea
     +,               log_sib ,   log_ice
     +,                radvbc ,    radvdc
     +,                radnbc ,    radndc
     +,               pblfulw ,   pblfdlw
     +,                  delt
     +               ) 
      
      use kinds
      use tracer_names
      use eau_params
      use eau_cldproperties
      use rad_params
      use rad_subdomain
#ifdef rad_isccp
      use isccp_tauinit
#endif

#ifdef scm
      use scm_module
#endif

      implicit none
      
!-----------------------------------------------------------------------

! references:

! rad_control is the driver that controls the call to parameterizations
! of long- and short-wave radiation.
! Laura D. Fowler/slikrock (02-01-01).

! send comments to laura@atmos.colostate.edu.

! subroutines called:

!     eau_cf       : calculates the horizontal fraction of clouds.
!     eau_ncw      : calculates the number concentration of cloud water
!                    droplets over land and oceans.
!     eau_effr     : calculates the effective radii of cloud water
!                    droplets and cloud ice crystals.
!     rad_cld      : sets all the variables and arrays related to the
!                    cloudiness and its optical properties.
!     rad_sfc      : sets all the variables and arrays related to the
!                    surface (surface temperature,surface albedos,...).
!     rad_sounding : sets all the variables and arrays related to the
!                    atmosphere (temperature, water vapor,...).
!     rad_driver   : calls the long- and short-wave radiation codes.
!     rad_qps      : calculates the qp2s and qp3s diagnostics.

! argument list variables:
! ------------------------
!     input arguments:
!     ----------------
      logical (kind=log_kind), intent(in), dimension(nagrid)::
     &  do_phys      !select the non-ghost cells                   (-).
     &, log_sea      !true if it is an ocean point                 (-).
     &, log_ice      !true if it is an ice point                   (-).
     &, log_sib      !true if it is a sib point                    (-).
     
      integer (kind=int_kind), intent(in)::
     &  ksdm         !loop index.
     &, n_physics    !nb of short-time steps per hour. 
     &, n_pbp        !number of time-steps between pbp outputs.

      real (kind=dbl_kind), intent(in)::
     &  tofday       !time of day.
     &, dtdyn        !dynamics time-step                           (s).

      real (kind=dbl_kind), intent(in), dimension(nagrid)::
     &  lon0         !longitude of grid cells                (radians).
     &, lat0         !latitude of grid cells                 (radians).
     &, tgeff4       !surface temperature^4                      (K^4).
     &, cldpc        !PBL cloud fraction                           (-).
     &, pc           !PBL cloud thickness                         (mb).
     &, alvdr        !visible direct surface albedo                (-).
     &, alndr        !near-IR direct surface albedo                (-).
     &, alvdf        !visible diffuse surface albedo               (-).
     &, alndf        !near-IR diffuse surface albedo               (-).

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm)::
     &  plg          !pressure                                   (hPa).
     &, bpg          !(pl/p0)**kapa                                (-).
     &, th           !potential temperature                        (K).
     &, dlcup        !detrainment rate of cloud water        (kg/kg/s).
     &, dicup        !detrainment rate of cloud ice          (kg/kg/s).
     
      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1)::
     &  pl2g         !level pressure                             (hPa).
     &, bpgl2        !(pl/p0)**kapa at layer edges                 (-).
     &, tl2          ! temperature at layer edges         (K).
     &, climo3       !climatological ozone mixing ratio        (kg/kg).

      real (kind=dbl_kind), dimension(nagrid,nlm,ntracer)::
     &  tracer       !array of non-passive tracers             (kg/kg).

      real (kind=dbl_kind), dimension(nagrid,npbl)::
     &   pblcw       !pbl cloud water mixing ratio             (kg/kg).
     &,  pblci       !pbl cloud ice mixing ratio               (kg/kg).
 
!     output arguments:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(nagrid)::
     &  radvbc       !sfc visible direct net SW radiation      (W/m^2).
     &, radvdc       !sfc visible diffuse net SW radiation     (W/m^2).
     &, radnbc       !sfc near-ir direct net SW radiation      (W/m^2).
     &, radndc       !sfc near-ir diffuse net SW radiation     (W/m^2).

      real (kind=dbl_kind), intent(out), dimension(nagrid,nlm)::
     &  delt         !radiative tendency of temperature            (K).

      real (kind=dbl_kind), intent(out), dimension(nagrid,nlm-lpbl+1)::
     &  pblfulw      !lw upward flux inside the PBL             (W/m2).
     &, pblfdlw      !lw downward flux inside the PBL           (W/m2).

! local variables:

      integer (kind=int_kind)::
     &  i, l

      real (kind=dbl_kind)::
     &  dtrad        !physical time-step                           (s).   
     &, heat_fac     !mutiplying factor.
     &, s0           !solar constant                            (W/m2).
     &, pcos1        !cosine of first diurnal harmonic             (-).
     &, pcos2        !cosine of first diurnal harmonic             (-).
     &, psin1        !sine of second diurnal harmonic              (-).
     &, psin2        !sine of second diurnal harmonic              (-).      

      real (kind=dbl_kind), dimension(nagrid)::
     &  cldt2d       !vertically integrated cloud fraction         (-).
     &, ncw          !number concentration of water droplets   (g/m^3).
     &, delf         !net flux                      
     &, delfcl       !net flux (clear)

      real (kind=dbl_kind), dimension(nagrid,nlm)::
     &  tl           !temperature                                  (K).
     &, ql           !water vapor mixing ratio                 (kg/kg).
     &, cw           !cloud water mixing ratio                 (kg/kg).
     &, ci           !cloud ice mixing ratio                   (kg/kg).
     &, ri           !snow mixing ratio                        (kg/kg).
     &, clu          !convective cloud fraction                    (-).
     &, cld          !large-scale saturation cloud fraction        (-).
     &, cldt         !total cloud fraction                         (-).
     &, cldw         !fraction of water clouds                     (-).
     &, cldi         !fraction of ice and snow clouds              (-).
     &, cldm         !fration of mixed-phase clouds                (-).
     &, effrw        !effective radius of water droplets          (mu).
     &, effri        !effective radius of ice droplets            (mu).
     
! local input arrays:

      real (kind=dbl_kind), dimension(len)::
     &  sfc_t        !surface temperature                          (K).
     &, cldmx_rad    !maximum cloud fraction                       (-).
     &, amu0         !cosine of solar zenith angle                 (-).
     &, slr          !length of day                                (-).
     &, lon          !longitude of grid cells                (radians).
     &, lat          !latitude of grid cells                 (radians).
 
      real (kind=dbl_kind), dimension(len,mbs)::
     &  sfc_asdir    !spectral surface albedo (direct beam)        (-).
     &, sfc_asdif    !spectral surface albedo (diffuse beam)       (-).

      real (kind=dbl_kind), dimension(len,mbir)::
     &  sfc_emis     !spectral surface emissivity                  (-).

      real (kind=dbl_kind), dimension(len,nlm_rad)::
     &  cld_rad      !local "radiative" cloud fraction                   (-).
     &, effrw_rad    !local effective radius of water droplets    (mu).
     &, effri_rad    !local effective radius of ice crystals      (mu).
     &, pl_rad       !local layer pressure                       (hPa).
     &, dpl_rad      !local layer pressure thickness             (hPa).
     &, tl_rad       !local layer temperature                            (K).
     &, ql_rad       !local water vapor mixing ratio           (kg/kg).
     &, cw_rad       !local cloud water mixing ratio           (kg/kg).
     &, ci_rad       !local cloud ice mixing ratio             (kg/kg).
     &, o3_rad       !local ozone mixing ratio                 (kg/kg).

      real (kind=dbl_kind), dimension(len,nlm_rad+1)::
     &  pl2_rad      !Pressure at layer edges                    (hPa).
     &, tl2_rad      !Temperature at layer edges                   (K).
     
! local output arrays:

      real (kind=dbl_kind), dimension(len)::
     &  alvdrc_l     !sfc visible direct albedo                    (-).
     &, alvdfc_l     !sfc visible diffuse albedo                   (-).
     &, alndrc_l     !sfc near-ir direct albedo                    (-).
     &, alndfc_l     !sfc near-ir diffuse albedo                   (-).
     &, radvbc_l     !local radvbc                             (W/m^2).
     &, radvdc_l     !local radvdc                             (W/m^2).
     &, radnbc_l     !local radnbc                             (W/m^2).
     &, radndc_l     !local radndc                             (W/m^2).
     &, radvbcc_l    !local radvbcc                            (W/m^2).
     &, radvdcc_l    !local radvdcc                            (W/m^2).
     &, radnbcc_l    !local radnbcc                            (W/m^2).
     &, radndcc_l    !local radndcc                            (W/m^2).
     &, swinc_l      !local swinc                              (W/m^2).

      real (kind=dbl_kind), dimension(len,nlm_rad)::
     &  tcldwl_vis   !local optical depth of water clouds (sw)     (-).
     &, tcldil_vis   !local optical depth of ice clouds (sw)       (-).
     &, tcldwl_irw   !local optical depth of water clouds (lw)     (-).
     &, tcldil_irw   !local optical depth of ice clouds (lw)       (-).
     &, atl_tem      !all-sky lw heating rate                    (K/s).
     &, atlcl_tem    !clear-sky lw heating rate                  (K/s).
     &, asl_tem      !all-sky sw heating rate                    (K/s).
     &, aslcl_tem    !clear-sky sw heating rate                  (K/s).

      real (kind=dbl_kind), dimension(len,nlm_rad+1)::
     &  fdsw         !downward SW flux                         (W/m^2).
     &, fusw         !upward SW flux                           (W/m^2).
     &, fdlw         !downward LW flux                         (W/m^2).
     &, fulw         !upward LW flux                           (W/m^2).
     &, fdswcl       !downward clear-sky SW flux               (W/m^2).
     &, fuswcl       !upward clear-sky SW flux                 (W/m^2).
     &, fdlwcl       !downward clear-sky LW flux               (W/m^2).
     &, fulwcl       !upward clear-sky LW flux                 (W/m^2).

! local variables used as diagnostics only:

      real (kind=dbl_kind), dimension(nagrid)::
     &  alvdrc       !sfc visible direct albedo                            (-).
     &, alvdfc       !sfc visible diffuse albedo                           (-).
     &, alndrc       !sfc near-ir direct albedo                            (-).
     &, alndfc       !sfc near-ir diffuse albedo                           (-).
     &, radvbcc      !sfc visible direct clear net sw                  (W/m^2).
     &, radvdcc      !sfc visible diffuse clear net sw                 (W/m^2).
     &, radnbcc      !sfc near-ir direct clear net sw                  (W/m^2).
     &, radndcc      !sfc near-ir diffuse clear net sw                 (W/m^2).
     &, swinc        !toa incident solar radiation                     (W/m^2).
     &, swtoa        !toa all-sky reflected sw radiation               (W/m^2).
     &, swtoacl      !toa clear-sky reflected sw radiation             (W/m^2).
     &, swnettoa     !toa net sw radiation                             (W/m^2).
     &, swnettoacl   !toa clear-sky net sw radiation                   (W/m^2).
     &, swdnsfc      !sfc downward sw radiation                        (W/m^2).
     &, swdnsfccl    !sfc clear-sky net sw radiation                   (W/m^2).
     &, swnetsfc     !sfc net sw radiation                             (W/m^2).
     &, swnetsfccl   !sfc clear-sky net sw radiation                   (W/m^2).
     &, swtom        !tom all-sky net sw radiation                     (W/m^2).
     &, swtomcl      !tom clear-sky net sw radiation                   (W/m^2).
     &, lwtoa        !toa outgoing lw radiation                        (W/m^2).
     &, lwtoacl      !toa clear-sky outgoing lw radiation              (W/m^2).
     &, lwupsfc      !sfc upward lw radiation                          (W/m^2).
     &, lwdnsfc      !sfc downward lw radiation                        (W/m^2).
     &, lwdnsfccl    !sfc clear-sky downward lw radiation              (W/m^2).
     &, lwnetsfc     !sfc net lw radiation                             (W/m^2).
     &, lwnetsfccl   !sfc clear-sky net lw radiation                   (W/m^2).
     &, lwtom        !tom all-sky net lw radiation                     (W/m^2).
     &, lwtomcl      !tom clear-sky net lw radiation                   (W/m^2).

      real (kind=dbl_kind), dimension(nagrid)::
     &  scosz        !cosine of solar zenith angle.
     &, sfac         !correction for length of day.

      real (kind=dbl_kind), dimension(nagrid,nlm)::
     &  atlcl        !clear-sky lw radiative heating rate                (K/s).
     &, aslcl        !clear-sky sw radiative heating rate                (K/s).
     &, atl          !long-wave radiative heating rate                   (K/s).
     &, asl          !short-wave radiative heating rate                  (K/s).
     &, tauw_vis     !optical depth of water clouds (sw:vis)               (-).
     &, taui_vis     !optical depth of ice clouds (sw:vis)                 (-).
     &, taum_vis     !optical depth of mixed-phase clouds (sw:vis)         (-).
     &, taut_vis     !optical depth of total clouds (vis)                  (-).
     &, tauw_irw     !optical depth of water clouds (lw:window)            (-).
     &, taui_irw     !optical depth of ice clouds (lw:window)              (-).
     &, taum_irw     !optical depth of mixed-phase clouds (lw:win)         (-).
     &, taut_irw     !optical depth of total clouds (lw:window)            (-).

#ifdef rad_isccp
!---- isccp cloud types:

!---- input arguments:
      integer (kind=int_kind), parameter::
     &  ncol       =  1 !number of subcolumns                              (-).
     &, top_height =  2 !set the adjustment of the cloud-top height        (-).
                        !adjust top-height=1 ; do no adjust top-height=2
     &, overlap    =  1 !set the cloud overlap                             (-).
                        !maximum=1 ; random=2 ; maximum/random=3

      integer (kind=int_kind)::
     &  seed            !seed value                                        (-).

      real (kind=dbl_kind), dimension(nagrid)::
     &  amu0_all        !cosine of the solar zenith angle for the whole
                        !grid                                              (-).

      real (kind=dbl_kind), dimension(:), allocatable::
     &  skt             !skin temperature                                  (K).
     &, emsfc_lw        !surface longwave emissivity                       (-).

      real (kind=dbl_kind), dimension(:,:), allocatable::
     &  pfull           !layer pressure                                   (Pa).
     &, qv              !layer specific humidity                       (kg/kg).
     &, at              !layer temperature                                 (K).
     &, cc              !layer total cloud fraction                        (-).
     &, conv            !layer convective cloud fraction                   (-).
     &, dtau_s          !layer optical depth of stratiform clouds          (-).
     &, dtau_c          !layer optical depth of convective clouds          (-).
     &, dem_s           !layer infrared emissivity of stratiform clouds    (-).
     &, dem_c           !layer infrared emissivity of convective clouds    (-).
     &, phalf           !level pressure                                   (Pa).

!---- output arguments:
      real (kind=dbl_kind), dimension(:), allocatable::
     &  totalcldarea    !the fraction of model grid-box columns with cloud
                        !somewhere in them.                                (-).
     &, meanptop        !mean cloud-top pressure                         (hPa).
     &, meantaucld      !mean optical thickness                            (-).

      real (kind=dbl_kind), dimension(:,:), allocatable::
     &  boxtau          !optical thickness in each column                  (-).
     &, boxptop         !cloud-top pressure in each column               (hPa).

      real (kind=dbl_kind), dimension(:,:,:), allocatable::
     &  fq_isccp        !the fraction of the model grid box covered by each
                        !type of the 49 isccp-d level cloud types          (-).     

!---- local variables:
      logical (kind=log_kind), dimension(nagrid)::
     &  do_isccp

      integer (kind=int_kind)::
     &  ipres,itau,npts
      integer (kind=int_kind), dimension(nagrid)::
     &  inpts

!---- qps diagnostics:
      real (kind=dbl_kind), dimension(nagrid,5,3)::
     &  isccp
#endif

!-----------------------------------------------------------------------
!     write(6,*) '--- enter rad_control: ksdm=', ksdm, ' nagrid='
!    +,          nagrid,' nlm=', nlm,' npbl=', npbl,' lpbl=', lpbl

      dtrad   = dtdyn * n_physics
      
!---- for now:

      pcos1 = radc1 ; pcos2 = radc1
      psin1 = radc1 ; psin2 = radc1

!---- initialization of output variables:

      radvbc(:) = radc0 ; radvdc(:) = radc0
      radnbc(:) = radc0 ; radndc(:) = radc0
      
      pblfulw(:,:) = radc0 ; pblfdlw(:,:)  = radc0
      
      delt(:,:)    = radc0

!---- initialization of local input variables:
      
      ql(:,:) = tracer(:,:,iq)   ; cw(:,:)  = tracer(:,:,icw) 
      ci(:,:) = tracer(:,:,ici)  ; ri(:,:)  = tracer(:,:,iri)
      tl(:,:) = th(:,:)*bpg(:,:)

!---- initialization of local cloud related variables:

      ncw(:)     = radc0 ; cldt2d(:)  = radc0

      clu(:,:)   = radc0 ; cld(:,:)   = radc0
      cldt(:,:)  = radc0 ; cldw(:,:)  = radc0
      cldi(:,:)  = radc0 ; cldm(:,:)  = radc0
      effrw(:,:) = radc0 ; effri(:,:) = radc0

!---- initialization of local radiation variables:

      alvdrc(:)     = radc0 ; alvdfc(:)    = radc0
      alndrc(:)     = radc0 ; alndfc(:)    = radc0
      radvbcc(:)    = radc0 ; radvdcc(:)   = radc0
      radnbcc(:)    = radc0 ; radndcc(:)   = radc0

      scosz(:)      = radc0 ; sfac(:)      = radc0
      swtoa(:)      = radc0 ; swtoacl(:)   = radc0
      swnettoacl(:) = radc0 ; swdnsfccl(:) = radc0
      swnetsfccl(:) = radc0 ; swtom(:)     = radc0
      swtomcl(:)    = radc0 ; swinc(:)     = radc0
      swnettoa(:)   = radc0 ; swdnsfc(:)   = radc0
      swnetsfc(:)   = radc0

      lwtoa(:)      = radc0 ; lwtoacl(:)   = radc0
      lwupsfc(:)    = radc0 ; lwdnsfc(:)   = radc0
      lwdnsfccl(:)  = radc0 ; lwnetsfc(:)  = radc0
      lwnetsfccl(:) = radc0 ; lwtom(:)     = radc0
      lwtomcl(:)    = radc0

      asl(:,:)      = radc0 ; atl(:,:)      = radc0
      aslcl(:,:)    = radc0 ; atlcl(:,:)    = radc0
      tauw_vis(:,:) = radc0 ; taui_vis(:,:) = radc0
      taum_vis(:,:) = radc0 ; taut_vis(:,:) = radc0
      tauw_irw(:,:) = radc0 ; taui_irw(:,:) = radc0
      taum_irw(:,:) = radc0 ; taut_irw(:,:) = radc0

#ifdef rad_isccp
      amu0_all(:) = radc0
#endif
       
!---- loop over the nagrid grid-points starts here:

      do i = 1, nagrid
         if(do_phys(i)) then
         
!---- cloud fraction: this subroutine will go away when eau_liqng is
!     finished.

!           write(6,*) '--- enter eau_cf:'
            call eau_cf
     +               (    nagrid ,        len ,       nlm
     +,                     npbl ,   cldpc(i) ,  plg(i,1)
     +,                  tl(i,1) ,    cw(i,1) ,   ci(i,1)
     +,               dlcup(i,1) , dicup(i,1) ,  clu(i,1)
     +,                 cld(i,1) ,  cldt(i,1) , cldw(i,1)
     +,                cldi(i,1) ,  cldm(i,1) , cldt2d(i)
     +            )
!           write(6,*) '--- exit eau_cf:'

!---- effective radii of cloud water droplets and ice crystals: this
!     subroutine will go away when eau_liqng is finished.

!           write(6,*) '--- enter eau_ncw:'
            call eau_ncw
     +               (    nagrid ,        len , log_sea(i)
     +,               log_ice(i) , log_sib(i) ,     ncw(i)
     +               )
!           write(6,*) '--- exit eau_ncw:'
     
!           write(6,*) '--- enter eau_effr:'
            call eau_effr
     +               (    nagrid ,        len ,      nlm
     +,                     npbl ,     ncw(i) ,  plg(i,1)
     +,                  tl(i,1) ,    cw(i,1) ,   ci(i,1)
     +,               pblcw(i,1) , pblci(i,1) , cldt(i,1)
     +,               effrw(i,1) , effri(i,1)
     +               )
!           write(6,*) '--- exit eau_effr:'
           
!---- initialization of surface variables:

!           write(6,*) '--- enter rad_sfc:'
            call rad_sfc
     +               (  nagrid ,       len ,       mbs
     +,                   mbir , tgeff4(i) ,  alvdr(i)
     +,               alndr(i) ,  alvdf(i) ,  alndf(i)
     +,                  sfc_t , sfc_asdir , sfc_asdif
     +,               sfc_emis
     +         )
!           write(6,*) '--- exit rad_sfc:'

!---- initialization of variables related to cloudiness and its
!     optical properties:

            call rad_cld
     +               (    nagrid ,       len ,        nlm
     +,                 cld(i,1) ,  clu(i,1) , effrw(i,1)
     +,               effri(i,1) , cldmx_rad ,    cld_rad
     +,                effrw_rad , effri_rad 
     +               )
!           write(6,*) '--- end rad_cld: i=',i
!           do l = 2, nlm_rad
!              ll = l-1
!              write(6,300) l,effrw(i,ll),effrw_rad(1,l)
!    +,                       effri(i,ll),effri_rad(1,l)
!           enddo
  300 format(i3,5(1x,f8.3))

!---- initialization of atmospheric sounding:

            call rad_sounding
     +               (     nagrid ,         len ,         nlm
     +,                      npbl ,   pl2g(i,1) ,  bpgl2(i,1)
     +,                  tl2(i,1) ,    plg(i,1) ,    bpg(i,1)
     +,                   th(i,1) ,     ql(i,1) ,     cw(i,1)
     +,                   ci(i,1) ,  pblcw(i,1) ,  pblci(i,1)
     +,               climo3(i,1) ,     pl2_rad ,     tl2_rad
     +,                    pl_rad ,     dpl_rad ,      tl_rad
     +,                    ql_rad ,      cw_rad ,      ci_rad
     +,                    o3_rad
     +               )

!---- call to calculation of the solar zenith angle and length of day:

            lon(len) = lon0(i)
            lat(len) = lat0(i)
      
            call rad_coszen
     +               (   len ,   cosd ,   sind
     +,               tofday , solcon , rsdist
     +,               rotper ,     pi ,    lat
     +,                  lon ,     s0 ,   amu0
     +,                  slr
     +               )

#ifdef rad_isccp
            amu0_all(i) = amu0(len)
#endif

!---- call the parameterization of long- and short-wave radiation:

            alvdrc_l(:)  = radc0 ; alvdfc_l(:)  = radc0
	    alndrc_l(:)  = radc0 ; alndfc_l(:)  = radc0
            radvbc_l(:)  = radc0 ; radvdc_l(:)  = radc0
            radnbc_l(:)  = radc0 ; radndc_l(:)  = radc0
            radvbcc_l(:) = radc0 ; radvdcc_l(:) = radc0
            radnbcc_l(:) = radc0 ; radndcc_l(:) = radc0
            swinc_l(:)   = radc0

	    tcldwl_vis(:,:) = radc0 ; tcldil_vis(:,:) = radc0
	    tcldwl_irw(:,:) = radc0 ; tcldil_irw(:,:) = radc0
	    atl_tem(:,:)    = radc0 ; atlcl_tem(:,:)  = radc0
	    asl_tem(:,:)    = radc0 ; aslcl_tem(:,:)  = radc0

	    fdsw(:,:)    = radc0 ; fusw(:,:)    = radc0
            fdlw(:,:)    = radc0 ; fulw(:,:)    = radc0
            fdswcl(:,:)  = radc0 ; fuswcl(:,:)  = radc0
            fdlwcl(:,:)  = radc0 ; fulwcl(:,:)  = radc0

            call rad_driver
     +               (       len ,    nlm_rad ,    pl2_rad
     +,                  tl2_rad ,     pl_rad ,    dpl_rad
     +,                   tl_rad ,     ql_rad ,     cw_rad
     +,                   ci_rad ,     o3_rad ,    cld_rad
     +,                effrw_rad ,  effri_rad ,      sfc_t
     +,                     amu0 ,        slr ,  sfc_asdir
     +,                sfc_asdif ,   sfc_emis ,  cldmx_rad
     +,                       s0 , tcldwl_vis , tcldil_vis
     +,               tcldwl_irw , tcldil_irw ,       fdsw
     +,                     fusw ,       fdlw ,       fulw
     +,                   fdswcl ,     fuswcl ,     fdlwcl
     +,                   fulwcl ,    swinc_l ,   radvbc_l
     +,                 radvdc_l ,   radnbc_l ,   radndc_l
     +,                radvbcc_l ,  radvdcc_l ,  radnbcc_l
     +,                radndcc_l ,   alvdrc_l ,   alvdfc_l
     +,                 alndrc_l ,   alndfc_l
     +               )

!---- computation of long and short wave radiative heating rates:

!           heat_fac = 9.80*0.01/cpair
            heat_fac = 9.80*0.01/spec_heat_cp
            do l = 1, nlm_rad
               delf(1:len)    = fulw(1:len,l)-fdlw(1:len,l)
     +                        - fulw(1:len,l+1)+fdlw(1:len,l+1)
               delfcl(1:len)  = fulwcl(1:len,l)-fdlwcl(1:len,l)
     +                        - fulwcl(1:len,l+1)+ fdlwcl(1:len,l+1)
               atl_tem(1:len,l)   = -heat_fac*delf(1:len)
     +                            / dpl_rad(1:len,l)
               atlcl_tem(1:len,l) = -heat_fac*delfcl(1:len)
     +                            / dpl_rad(1:len,l) 

               delf(1:len)    = fusw(1:len,l)-fdsw(1:len,l)
     +                        - fusw(1:len,l+1)+fdsw(1:len,l+1)
               delfcl(1:len)  = fuswcl(1:len,l)-fdswcl(1:len,l)
     +                        - fuswcl(1:len,l+1) + fdswcl(1:len,l+1)
               asl_tem(1:len,l)   = -heat_fac*delf(1:len)
     +                            / dpl_rad(1:len,l)
               aslcl_tem(1:len,l) = -heat_fac*delfcl(1:len)
     +                            / dpl_rad(1:len,l)             
            enddo 

!---- fills in top-of-the-atmosphere, top-of-model, and surface lw and
!     sw diagnostics:

            lwtoa(i)      = fulw(1,1)
            lwtoacl(i)    = fulwcl(1,1)        
            lwupsfc(i)    = fulw(1,nlm_rad+1)
            lwdnsfc(i)    = fdlw(1,nlm_rad+1)
            lwdnsfccl(i)  = fdlwcl(1,nlm_rad+1)
            lwnetsfc(i)   = fulw(1,nlm_rad+1)-fdlw(1,nlm_rad+1)
            lwnetsfccl(i) = fulwcl(1,nlm_rad+1)-fdlwcl(1,nlm_rad+1)
            lwtom(i)      = fulw(1,2)-fdlw(1,2)
            lwtomcl(i)    = fulwcl(1,2)-fdlwcl(1,2)

            swinc(i)      = swinc_l(1)
            swtoa(i)      = fusw(1,1)
            swtoacl(i)    = fuswcl(1,1) 
            swnettoa(i)   = fdsw(1,1)-fusw(1,1)
            swnettoacl(i) = fdswcl(1,1)-fuswcl(1,1)
            swdnsfc(i)    = fdsw(1,nlm_rad+1) 
            swdnsfccl(i)  = fdswcl(1,nlm_rad+1)
            swnetsfc(i)   = fdsw(1,nlm_rad+1)-fusw(1,nlm_rad+1) 
            swnetsfccl(i) = fdswcl(1,nlm_rad+1)-fuswcl(1,nlm_rad+1)  
            swtom(i)      = fdsw(1,2)-fusw(1,2)
            swtomcl(i)    = fdswcl(1,2)-fuswcl(1,2)

            scosz(i)      = amu0(1)
	      sfac(i)       = slr(1) 

!---- fills in sw all-sky and clear-sky radiances:

            alvdrc(i)     = alvdrc_l(1)
	      alvdfc(i)     = alvdfc_l(1)
	      alndrc(i)     = alndrc_l(1)
	      alndfc(i)     = alndfc_l(1)

            radvbc(i)     = radvbc_l(1)
            radvdc(i)     = radvdc_l(1)
            radnbc(i)     = radnbc_l(1)
            radndc(i)     = radndc_l(1)
            radvbcc(i)    = radvbcc_l(1)
            radvdcc(i)    = radvdcc_l(1)
            radnbcc(i)    = radnbcc_l(1)
            radndcc(i)    = radndcc_l(1)

!---- fills in lw and sw all-sky and clear-sky heating rates:
            
            do l = 1, nlm
               atl(i,l)   = atl_tem(1,l+1)
               asl(i,l)   = asl_tem(1,l+1)
               atlcl(i,l) = atlcl_tem(1,l+1)
               aslcl(i,l) = aslcl_tem(1,l+1)
            enddo               

!---- fills in the optical properties of water and ice clouds: will need
!     to be corrected when the cloud fraction is less than 1.:

            do l = 1, nlm
!              tauw_vis(i,l) = tcldwl_vis(1,l+1)
!              taui_vis(i,l) = tcldil_vis(1,l+1)
!              tauw_irw(i,l) = tcldwl_irw(1,l+1)
!              taui_irw(i,l) = tcldil_irw(1,l+1)
               tauw_vis(i,l) = (tcldwl_vis(1,l+1)+tcldil_vis(1,l+1))
     +                       * cldw(i,l)
               taui_vis(i,l) = (tcldwl_vis(1,l+1)+tcldil_vis(1,l+1))
     +                       * cldi(i,l)
               taum_vis(i,l) = (tcldwl_vis(1,l+1)+tcldil_vis(1,l+1))
     +                       * cldm(i,l)
               taut_vis(i,l) = (tcldwl_vis(1,l+1)+tcldil_vis(1,l+1))
     +                       * cldt(i,l)
               tauw_irw(i,l) = (tcldwl_irw(1,l+1)+tcldil_irw(1,l+1))
     +                       * cldw(i,l)
               taui_irw(i,l) = (tcldwl_irw(1,l+1)+tcldil_irw(1,l+1))
     +                       * cldi(i,l)
               taum_irw(i,l) = (tcldwl_irw(1,l+1)+tcldil_irw(1,l+1))
     +                       * cldm(i,l)
               taut_irw(i,l) = (tcldwl_irw(1,l+1)+tcldil_irw(1,l+1))
     +                       * cldt(i,l)
            enddo

!---- fills in the lw upward and downward fluxes in the PBL:

            pblfulw(i,1:npblp1) = fulw(1,nlm_rad-npbl+1:nlm_rad+1)
            pblfdlw(i,1:npblp1) = fdlw(1,nlm_rad-npbl+1:nlm_rad+1)

         endif

      enddo

!---- radiative tendency:

      !delt(:,:) = atl(:,:) + asl(:,:)
#ifdef scm
      if(scm_rad) then
         delt(1,:) = delt(1,:) + rad_terp(1,1,:,ksdm)
      else
         delt(:,:) = delt(:,:) + atl(:,:) + asl(:,:)
      endif
#else
      delt(:,:) = delt(:,:) + atl(:,:) + asl(:,:)
#endif

#ifdef rad_isccp
!---- isccp cloud types:

!---- select non-ghost day-time grid-points:
      do_isccp(:) = .false.
      where(do_phys(:) .and. amu0_all(:).ge.0.01_dbl_kind) 
     +      do_isccp(:) = .true.
      npts     = 0
      inpts(:) = 0
      do i = 1, nagrid
         if(do_isccp(i)) then
            npts = npts + 1
            inpts(npts) = i
         endif
      enddo
!     write(6,*) '--- isccp_cloudtypes: npts=', npts

      day_time: select case(npts)
      case (0)
         isccp(:,:,:) = 0.0_dbl_kind

      case default
         isccp(:,:,:) = 0.0_dbl_kind
!---- allocate and fill input arrays to isccp_cloud_types:
         allocate(skt(npts))
         allocate(emsfc_lw(npts))
         allocate(pfull(npts,nlm))
         allocate(qv(npts,nlm))
         allocate(at(npts,nlm))
         allocate(cc(npts,nlm))
         allocate(conv(npts,nlm))
         allocate(dtau_s(npts,nlm))
         allocate(dtau_c(npts,nlm))
         allocate(dem_s(npts,nlm))
         allocate(dem_c(npts,nlm))
         allocate(phalf(npts,nlm+1))
      
         skt(:)      = tgeff4(inpts(1:npts))**0.25
         emsfc_lw(:) = 1.0_dbl_kind
         pfull(:,:)  = plg(inpts(1:npts),:)
         qv(:,:)     = ql(inpts(1:npts),:)
         at(:,:)     = tl(inpts(1:npts),:)

         cc(:,:)     = cld(inpts(1:npts),:)+clu(inpts(1:npts),:)
         conv(:,:)   = clu(inpts(1:npts),:)

         dtau_s(:,:) = (tauw_vis(inpts(1:npts),:)+taui_vis(inpts(1:npts),:)
     +               +  taum_vis(inpts(1:npts),:)) * cld(inpts(1:npts),:)
         dtau_c(:,:) = (tauw_vis(inpts(1:npts),:)+taui_vis(inpts(1:npts),:)
     +               +  taum_vis(inpts(1:npts),:)) * clu(inpts(1:npts),:)         
         dem_s(:,:)  = 0.5_dbl_kind*cld(inpts(1:npts),:)
         dem_c(:,:)  = 0.5_dbl_kind*clu(inpts(1:npts),:)
         phalf(:,:)  = pl2g(inpts(1:npts),:)

!---- allocate and fill output arrays to isccp_cloud_types:
         allocate(totalcldarea(npts))
         allocate(meanptop(npts))
         allocate(meantaucld(npts))
         allocate(boxtau(npts,ncol))
         allocate(boxptop(npts,ncol))
         allocate(fq_isccp(npts,5,3))

         totalcldarea(:) = 0.0_dbl_kind
         meanptop(:)     = 0.0_dbl_kind
         meantaucld(:)   = 0.0_dbl_kind
         boxtau(:,:)     = 0.0_dbl_kind
         boxptop(:,:)    = 0.0_dbl_kind
         fq_isccp(:,:,:) = 0.0_dbl_kind

!---- run the isccp algorithm:
         do i = 1, npts
            seed = (pfull(i,nlm)-int(pfull(i,nlm)))*100+1
            call isccp_cloud_types
     +                 (            nlm ,              ncol
     +,                            seed ,        pfull(i,:)
     +,                      phalf(i,:) ,           qv(i,:)
     +,                         cc(i,:) ,         conv(i,:)
     +,                     dtau_s(i,:) ,       dtau_c(i,:)
     +,                      top_height ,           overlap
     +,                   tautab(0:255) , invtau(-20:45000)
     +,                          skt(:) ,       emsfc_lw(:)
     +,                         at(i,:) ,        dem_s(i,:)
     +,                      dem_c(i,:) ,   fq_isccp(i,:,:)
     +,                 totalcldarea(i) ,       meanptop(i)
     +,                   meantaucld(i) ,       boxtau(i,:)
     +,                    boxptop(i,:)
     +                 )

            isccp(inpts(i),1,1) = fq_isccp(i,1,1)
            isccp(inpts(i),2,1) = fq_isccp(i,2,1)
            isccp(inpts(i),3,1) = fq_isccp(i,3,1)
            isccp(inpts(i),4,1) = fq_isccp(i,4,1)
            isccp(inpts(i),5,1) = fq_isccp(i,5,1)

            isccp(inpts(i),1,2) = fq_isccp(i,1,2)
            isccp(inpts(i),2,2) = fq_isccp(i,2,2)
            isccp(inpts(i),3,2) = fq_isccp(i,3,2)
            isccp(inpts(i),4,2) = fq_isccp(i,4,2)
            isccp(inpts(i),5,2) = fq_isccp(i,5,2)

            isccp(inpts(i),1,3) = fq_isccp(i,1,3)
            isccp(inpts(i),2,3) = fq_isccp(i,2,3)
            isccp(inpts(i),3,3) = fq_isccp(i,3,3)
            isccp(inpts(i),4,3) = fq_isccp(i,4,3)
            isccp(inpts(i),5,3) = fq_isccp(i,5,3)

         enddo
         
!----
         where(isccp(:,:,:) > 0.999) isccp(:,:,:) = 1.0_dbl_kind
!        do i = 1, nagrid
!           do ipres = 1, 3
!              write(6,101) i,ipres,(isccp(i,itau,ipres),itau=1,5)
!           enddo
!           write(6,*)         
!        enddo

!---- deallocate all arrays:
         deallocate(skt)
         deallocate(emsfc_lw)
         deallocate(pfull)
         deallocate(qv)
         deallocate(at)
         deallocate(cc)
         deallocate(conv)
         deallocate(dtau_s)
         deallocate(dtau_c)
         deallocate(dem_s)
         deallocate(dem_c)
         deallocate(phalf)
         deallocate(totalcldarea)
         deallocate(meanptop)
         deallocate(meantaucld)
         deallocate(boxtau)
         deallocate(boxptop)
         deallocate(fq_isccp)
      end select day_time

#endif

!---- qps diagnostics:

#ifndef scm
      call eau_cfqps
     +         ( ksdm , n_physics , nagrid ,   nlm
     +,         pcos1 ,     pcos2 ,  psin1 , psin2
     +,          pl2g ,     bpgl2 ,    bpg ,    tl
     +,           cld ,       clu ,   cldt ,  cldw
     +,          cldi ,      cldm ,  cldpc ,   ncw
     +,            cw ,        ci ,  effrw , effri
     +,            pc    )

      call rad_qps
     +         (        ksdm ,  n_physics ,    nagrid ,          nlm
     +,                pcos1 ,      pcos2 ,     psin1 ,        psin2
     +,             tauw_vis ,   taui_vis ,  taum_vis ,     taut_vis
     +,             tauw_irw ,   taui_irw ,  taum_irw ,     taut_irw
     +,                 pl2g ,     tgeff4 ,     lwtoa ,      lwtoacl
     +,              lwupsfc ,    lwdnsfc , lwdnsfccl ,     lwnetsfc
     +,           lwnetsfccl ,      lwtom ,   lwtomcl , pblfulw(:,1)
     +,         pblfdlw(:,1) ,      swinc ,     swtoa ,      swtoacl
     +,             swnettoa , swnettoacl ,   swdnsfc ,    swdnsfccl
     +,             swnetsfc , swnetsfccl ,     swtom ,      swtomcl
     +,               radvdc ,     radndc ,    radvbc ,       radnbc
     +,              radvdcc ,    radndcc ,   radvbcc ,      radnbcc
     +,                scosz ,       sfac ,    alvdrc ,       alvdfc
     +,               alndrc ,     alndfc ,       atl ,        atlcl
     +,                  asl ,      aslcl ,      lon0 ,         lat0
     +,           climo3(:,2:nlm+1) 
#ifdef rad_isccp
     +               , isccp
#endif
     +         )
#endif
      call rad_pbps(ksdm, n_physics, n_pbp, nagrid, nlm
     +,              swnetsfc, lwnetsfc, lwtoa, swnettoa
     +,              swinc, lwdnsfc, swdnsfc, lwupsfc
     +,              swtoa, swdnsfccl, lwdnsfccl, lwtoacl
     +,              swtoacl, swnetsfccl
     +,              atl, atlcl, asl, aslcl, climo3(:,2:nlm+1))


!---- formats:
  100 format(2(1x,i5),9(1x,e15.8))
  101 format(2(1x,i4),9(1x,e15.8))
  102 format(2(1x,i5),8(1x,f8.3))
  103 format(1x,i5,4(1x,e15.8))
  
!     write(6,*) '--- exit rad_control:'

      end subroutine rad_control

!-----------------------------------------------------------------------
