!-----------------------------------------------------------------------

      subroutine rad_driver
     +               (     len ,       nlm ,       pl2
     +,                    tl2 ,        pl ,       dpl
     +,                     tl ,        ql ,      qcwl
     +,                   qcil ,       o3l ,      acld
     +,                  effrw ,     effri ,        ts
     +,                   amu0 ,      slr0 , sfc_asdir
     +,              sfc_asdif ,  sfc_emis ,    acldmx
     +,                     s0 , tcldw_vis , tcldi_vis
     +,              tcldw_irw , tcldi_irw ,      fdsw
     +,                   fusw ,      fdlw ,      fulw
     +,                 fdswcl ,    fuswcl ,    fdlwcl
     +,                 fulwcl ,     swinc ,    radvbc
     +,                 radvdc ,    radnbc ,    radndc
     +,                radvbcc ,   radvdcc ,   radnbcc
     +,                radndcc ,    alvdrc ,    alvdfc
     +,                 alndrc ,    alndfc
     +               )

      use kinds
      use rad_params
      
      implicit none

!-----------------------------------------------------------------------

! rad_driver calls the long-wave (rad_lwr) and short-wave (rad_swr)
! parameterization of radiation.
! Laura D. Fowler/slikrock (02-01-01).

! send comments to laura@atmos.colostate.edu
! and norm@atmos.colostate.edu

! subroutines called:

!      rad_lwr : parameterization of long-wave radiation.
!      rad_swr : parameterization of shor-wave radiation.

! argument list variables:
!     input arguments:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  len        !length of sub domain.
     &, nlm        !number of layers.

      real (kind=dbl_kind), intent(in)::
     &  s0         !solar constant                             (W/m^2).

      real (kind=dbl_kind), intent(in), dimension(len)::
     &  ts         !surface temperature                            (K).
     &, amu0       !cosine of solar zenith angle                   (-).
     &, slr0       !fraction of daylight                           (-).
     &, acldmx     !maximum cloud fraction                         (-).

      real (kind=dbl_kind), dimension(len,mbs)::
     &  sfc_asdir  !spectral surface albedo (direct beam)          (-).
     &, sfc_asdif  !spectral surface albedo (diffuse beam)         (-).

      real (kind=dbl_kind), dimension(len,mbir)::
     &  sfc_emis     !spectral surface emissivity                  (-).

      real (kind=dbl_kind), intent(in), dimension(len,nlm)::
     &  pl         !layer pressure                               (hPa).
     &, dpl        !layer thickness                              (hPa).
     &, tl         !temperature                                    (K).
     &, ql         !specific humidity                          (kg/kg).
     &, qcwl       !cloud water mixing ratio                   (kg/kg).
     &, qcil       !cloud ice mixing ratio                     (kg/kg).
     &, o3l        !ozone mixing ratio                         (kg/kg).
     &, acld       !cloud fraction                                 (-).
     &, effrw      !effective radius of water droplets            (mu).
     &, effri      !effective radius of ice droplets              (mu).

      real (kind=dbl_kind), intent(in), dimension(len,nlm+1)::
     &  pl2        !level pressure                               (hPa).
     &, tl2        !temperature at layer edges                     (K).

!     outputs arguments:
!     ------------------
      real (kind=dbl_kind), intent(out), dimension(len)::
     &  alvdrc     !sfc visible direct albedo                      (-).
     &, alvdfc     !sfc visible diffuse albedo                     (-).
     &, alndrc     !sfc near-ir direct albedo                      (-).
     &, alndfc     !sfc near-ir diffuse albedo                     (-).
     &, radvbc     !sfc all-sky vis direct net SW              (W/m^2).
     &, radvdc     !sfc all-sky vis diffuse net SW             (W/m^2).
     &, radnbc     !sfc all-sky near-ir direct net SW          (W/m^2).
     &, radndc     !sfc all-sky near-ir diffuse net SW         (W/m^2).
     &, radvbcc    !sfc clear-sky visible direct net SW        (W/m^2).
     &, radvdcc    !sfc clear-sky visible diffuse net SW       (W/m^2).
     &, radnbcc    !sfc clear-sky near-ir direct net SW        (W/m^2).
     &, radndcc    !sfc clear-sky near-ir diffuse net SW       (W/m^2).
     &, swinc      !toa incident solar radiation               (W/m^2).

      real (kind=dbl_kind), intent(out), dimension(len,nlm)::
     &  tcldw_vis   !optical depth of water clouds (sw: visible)   (-).
     &, tcldi_vis   !optical depth of ice clouds (sw: visible)     (-).
     &, tcldw_irw   !optical depth of water clouds (lw: window)    (-).
     &, tcldi_irw   !optical depth of ice clouds (lw: window)      (-).

      real (kind=dbl_kind), intent(out), dimension(len,nlm+1)::
     &  fdsw       !downward SW flux                           (W/m^2).
     &, fusw       !upward SW flux                             (W/m^2).
     &, fdlw       !downward LW flux                           (W/m^2).
     &, fulw       !upward LW flux                             (W/m^2).
     &, fdswcl     !downward clear-sky SW flux                 (W/m^2).
     &, fuswcl     !upward clear-sky SW flux                   (W/m^2).
     &, fdlwcl     !downward clear-sky LW flux                 (W/m^2).
     &, fulwcl     !upward clear-sky LW flux                   (W/m^2).

! local variables:

      logical (kind=log_kind)::
     &  sel_rules_lw
     &, sel_rules_sw

      logical (kind=log_kind), dimension(len)::
     &  bitx       !scans daytime grid-points                      (-).
     
      integer (kind=int_kind)::
     &  i, l
     &, nday       !number of daytime grid-points .

      integer (kind=int_kind), dimension(len)::
     &  iday       !location of daytime grid-points.
     
      real (kind=dbl_kind)::
     &  umco2      !concentration of CO2 (ppm).
     &, umch4      !concentration of CH4      .
     &, umn2o      !concentration of N2O      .
      data umco2,umch4,umn2o /330.,1.6,0.28/

      real (kind=dbl_kind), dimension(len)::
     &  den        !multiplying factor.
     &, slr        !fraction of day.

      real (kind=dbl_kind), dimension(len,nlm)::
     &  rmix       !water vapor mixing ratio                   (kg/kg).
     &, cwrho      !density of cloud water                     (g/m^3).
     &, cirho      !density of cloud ice                       (g/m^3).
     &, o3mix      !ozone mixing ratio                         (kg/kg).
#ifdef overlap
     &, b1         !cloud overlap parameter                         (-).
     &, b2         !cloud overlap parameter                         (-).
     &, b3         !cloud overlap parameter                         (-).
     &, b4         !cloud overlap parameter                         (-).
#endif

! local allocatable arrays:

      real (kind=dbl_kind), dimension(:), allocatable::
     &  ts_day     !as ts, but for daytime grid-points             (K).
     &, amu0_day   !as amu0,but for daytime grid-points            (-).
     &, slr_day    !as slr,but for daytime grid-points             (-).
     &, acldmx_day !as cldmx, but for daytime grid-points          (-).

      real (kind=dbl_kind), dimension(:,:), allocatable::
     &  pl_day     !as pl,but for daytime grid-points            (hPa).
     &, dpl_day    !as dpl,but for daytime grid-points           (hPa).
     &, tl_day     !as tl,but for daytime grid-points              (K).
     &, rmix_day   !as rmix,but for daytime grid-points        (kg/kg).
     &, cwrho_day  !as cwrho,but for daytime grid-points       (g/m^3).
     &, cirho_day  !as cirho,but for daytime grid-points       (g/m^3).
     &, o3mix_day  !as o3mix,but for daytime grid-points       (kg/kg).
     &, acld_day   !as acld,but for daytime grid-points            (-).
     &, effrw_day  !as effrw_day, but for daytime grid-points     (mu).
     &, effri_day  !as effri_day, but for daytime grid-points     (mu).
     &, asdir_day  !as sfc_asdir, but for daytime grid-points      (-).
     &, asdif_day  !as sfc_asdif, but for daytime grid-pointd      (-).
#ifdef overlap
     &, b1_day     !cloud overlap parameter                        (-).
     &, b2_day     !cloud overlap parameter                        (-).
     &, b3_day     !cloud overlap parameter                        (-).
     &, b4_day     !cloud overlap parameter                        (-).
#endif

      real (kind=dbl_kind), dimension(:), allocatable::      
     &  alvdr_day  !as alvdr, but for daytime grid-points          (-).
     &, alvdf_day  !as alvdf, but for daytime grid-points          (-).
     &, alndr_day  !as alndr, but for daytime grid-points          (-).
     &, alndf_day  !as alndf, but for daytime grid-points          (-).
     &, radvbc_day !as radvbc, but for daytime grid-points     (W/m^2).
     &, radvdc_day !as radvdc, but for daytime grid-points     (W/m^2).
     &, radnbc_day !as radnbc, but for daytime grid-points     (W/m^2).
     &, radndc_day !as radndc, but for daytime grid-points     (W/m^2).
     &, radvbcc_day!as radvbcc, but for daytime grid-points    (W/m^2).
     &, radvdcc_day!as radvdcc, but for daytime grid-points    (W/m^2).
     &, radnbcc_day!as radnbc, but for daytime grid-points     (W/m^2).
     &, radndcc_day!as radndc, but for daytime grid-points     (W/m^2).
     &, swinc_day  !as swinc, but for daytime grid-points      (W/m^2).

      real (kind=dbl_kind), dimension(:,:), allocatable::
     &  tcldw_day  !
     &, tcldi_day  !

      real (kind=dbl_kind), dimension(:,:), allocatable::
     &  pl2_day    !as pl2_loc, but for daytime grid-points      (hPa).
     &, fdsw_day   !as fdsw, but for daytime grid points       (W/m^2).
     &, fdswcl_day !as fdswcl, but for daytime grid points     (W/m^2).
     &, fusw_day   !as fusw, but for daytime grid points       (W/m^2).
     &, fuswcl_day !as fuswcl, but for daytime grid points     (W/m^2).

!-----------------------------------------------------------------------

!     write(6,*) '--- enter rad_driver:'

      sel_rules_sw = .false.
      sel_rules_lw = .false.

!---- initialization of output arrays:

      alvdrc(:)   = radc0 ; alvdfc(:)   = radc0
      alndrc(:)   = radc0 ; alndfc(:)   = radc0

      radvbc(:)   = radc0 ; radvdc(:)   = radc0
      radnbc(:)   = radc0 ; radndc(:)   = radc0
      radvbcc(:)  = radc0 ; radvdcc(:)  = radc0
      radnbcc(:)  = radc0 ; radndcc(:)  = radc0

      tcldw_vis(:,:) = radc0 ; tcldi_vis(:,:) = radc0
      tcldw_irw(:,:) = radc0 ; tcldi_irw(:,:) = radc0

      swinc(:)    = radc0 ; fdsw(:,:)   = radc0
      fdswcl(:,:) = radc0 ; fusw(:,:)   = radc0
      fuswcl(:,:) = radc0 ; fdlw(:,:)   = radc0
      fdlwcl(:,:) = radc0 ; fulw(:,:)   = radc0
      fulwcl(:,:) = radc0

!--- random overlap of the cloud fraction:

#ifdef overlap
      do i = 1, len
         b1(i,1) = 1.0 - acld(i,1)
         b3(i,1) = 1.0
         do l = 2,nlm
            if (acld(i,l-1).eq.1.0) then
               b1(i,l) = 1.0
            else
               b1(i,l) = (1.0 - max(acld(i,l),acld(i,l-1))) /
     +                   (1.0 - acld(i,l-1))
            endif
            if (acld(i,l-1).eq.0.0) then
               b3(i,l) = 1.0
            else
               b3(i,l) = min(acld(i,l),acld(i,l-1)) /
     +                acld(i,l-1)
            endif
         enddo
 
         b2(i,nlm) = 1.0 - acld(i,nlm)
         b4(i,nlm) = 1.0
         do l = 1,nlm-1
            if (acld(i,l+1).eq.1.0) then
               b2(i,l) = 1.0
            else
               b2(i,l) = (1.0 - max(acld(i,l),acld(i,l+1))) /
     +                   (1.0 - acld(i,l+1))
            endif
            if (acld(i,l+1).eq.0.0) then
               b4(i,l) = 1.0
            else
               b4(i,l) = min(acld(i,l),acld(i,l+1)) /
     +                   acld(i,l+1)
            endif
         enddo
      enddo
#endif

!---- units conversion:

      do l = 1, nlm
         den(:)     = pl(:,l)*100/(287.*tl(:,l))
         rmix(:,l)  = ql(:,l)/(1.-ql(:,l))
#ifdef overlap
         cwrho(:,l) = den*1000.*qcwl(:,l)
         cirho(:,l) = den*1000.*qcil(:,l)
!        cwrho(:,l) = radc0
!        cirho(:,l) = radc0     
#else
         cwrho(:,l) = den*1000.*qcwl(:,l)*acld(:,l)
         cirho(:,l) = den*1000.*qcil(:,l)*acld(:,l)
!        cwrho(:,l) = radc0
!        cirho(:,l) = radc0     
#endif
         o3mix(:,l) = o3l(:,l)
      enddo                  

!---- call to the long-wave radiation code:

      call rad_lwr
     +         (      len ,       nlm ,          pl2 ,      tl2
     +,                pl ,       dpl ,           tl ,     rmix
     +,             cwrho ,     cirho ,        o3mix ,     acld
     +,             effrw ,     effri ,           ts ,   acldmx
     +,             umco2 ,     umch4 ,        umn2o , sfc_emis
     +,         tcldw_irw , tcldi_irw ,         fdlw ,   fdlwcl
     +,              fulw ,    fulwcl , sel_rules_lw
#ifdef overlap
     +,                b1 ,        b2 ,           b3 ,       b4
#endif
     +         )
     
!--   call to the short-wave radiation code:

!--   note: slr needs to be modified to accomodate the difference in
!           s0 between the original radiation code and that in BUGS2.
      slr(:) = slr0(:) * s0/1339.945

!--   note: computation of the shortwave radiative heating rates and
!     fluxes are made for daytime grid-points only: 
      bitx(:) = amu0(:) .ge. 0.01
      nday = 0
      do i = 1, len
         if(bitx(i)) then 
            nday       = nday + 1
            iday(nday) = i 
         endif
      enddo
      if(nday .eq. 0) goto 1000

      allocate(ts_day(nday))
      allocate(amu0_day(nday))
      allocate(slr_day(nday))
      allocate(acldmx_day(nday))
      allocate(asdir_day(nday,mbs))
      allocate(asdif_day(nday,mbs))
      allocate(pl_day(nday,nlm))
      allocate(dpl_day(nday,nlm))
      allocate(tl_day(nday,nlm))
      allocate(rmix_day(nday,nlm))
      allocate(cwrho_day(nday,nlm))
      allocate(cirho_day(nday,nlm))
      allocate(o3mix_day(nday,nlm))
      allocate(acld_day(nday,nlm))
      allocate(effrw_day(nday,nlm))
      allocate(effri_day(nday,nlm))
#ifdef overlap
      allocate(b1_day(nday,nlm))
      allocate(b2_day(nday,nlm))
      allocate(b3_day(nday,nlm))
      allocate(b4_day(nday,nlm))
#endif
      allocate(pl2_day(nday,nlm+1))


      allocate(radvbc_day(nday))
      allocate(radvdc_day(nday))
      allocate(radnbc_day(nday))
      allocate(radndc_day(nday))
      allocate(radvbcc_day(nday))
      allocate(radvdcc_day(nday))
      allocate(radnbcc_day(nday))
      allocate(radndcc_day(nday))
      allocate(alvdr_day(nday))
      allocate(alvdf_day(nday))
      allocate(alndr_day(nday))
      allocate(alndf_day(nday))
      allocate(swinc_day(nday))
      allocate(tcldw_day(nday,nlm))
      allocate(tcldi_day(nday,nlm))
      allocate(fdsw_day(nday,nlm+1))
      allocate(fdswcl_day(nday,nlm+1))      
      allocate(fusw_day(nday,nlm+1))
      allocate(fuswcl_day(nday,nlm+1))           

      alvdr_day(1:nday) = sfc_asdir(iday(1:nday),1)
      alvdf_day(1:nday) = sfc_asdif(iday(1:nday),1)
      alndr_day(1:nday) = sfc_asdir(iday(1:nday),2)
      alndf_day(1:nday) = sfc_asdif(iday(1:nday),2)

      ts_day(1:nday)          = ts(iday(1:nday))
      amu0_day(1:nday)        = amu0(iday(1:nday))
      slr_day(1:nday)         = slr(iday(1:nday))
      acldmx_day(1:nday)      = acldmx(iday(1:nday))
      asdir_day(1:nday,1:mbs) = sfc_asdir(iday(1:nday),1:mbs)
      asdif_day(1:nday,1:mbs) = sfc_asdif(iday(1:nday),1:mbs)
      pl_day(1:nday,1:nlm)    = pl(iday(1:nday),1:nlm)
      dpl_day(1:nday,1:nlm)   = dpl(iday(1:nday),1:nlm)
      tl_day(1:nday,1:nlm)    = tl(iday(1:nday),1:nlm)
      rmix_day(1:nday,1:nlm)  = rmix(iday(1:nday),1:nlm)
      cwrho_day(1:nday,1:nlm) = cwrho(iday(1:nday),1:nlm)
      cirho_day(1:nday,1:nlm) = cirho(iday(1:nday),1:nlm)
      o3mix_day(1:nday,1:nlm) = o3mix(iday(1:nday),1:nlm)
      acld_day(1:nday,1:nlm)  = acld(iday(1:nday),1:nlm)
      effrw_day(1:nday,1:nlm) = effrw(iday(1:nday),1:nlm)
      effri_day(1:nday,1:nlm) = effri(iday(1:nday),1:nlm)

!---- swinc is only a diagnostic:

      swinc_day(1:nday)       = amu0_day(1:nday)*slr_day(1:nday)
      swinc_day(1:nday)       = swinc_day(1:nday)*1339.945

#ifdef overlap
      b1_day(1:nday,1:nlm)    = b1(iday(1:nday),1:nlm)
      b2_day(1:nday,1:nlm)    = b2(iday(1:nday),1:nlm)
      b3_day(1:nday,1:nlm)    = b3(iday(1:nday),1:nlm)
      b4_day(1:nday,1:nlm)    = b4(iday(1:nday),1:nlm)
#endif
      pl2_day(1:nday,1:nlm+1) = pl2(iday(1:nday),1:nlm+1)

      call rad_swr
     +          (        nday ,         nlm ,    pl2_day ,      pl_day
     +,               dpl_day ,      tl_day ,   rmix_day ,   cwrho_day
     +,             cirho_day ,   o3mix_day ,     ts_day ,    amu0_day
     +,               slr_day ,   asdir_day ,  asdif_day ,    acld_day
     +,            acldmx_day ,   effrw_day ,  effri_day ,       umco2
     +,                 umch4 ,       umn2o ,  tcldw_day ,   tcldi_day
     +,              fdsw_day ,  fdswcl_day ,   fusw_day ,  fuswcl_day
     +,            radvbc_day , radvbcc_day , radvdc_day , radvdcc_day
     +,            radnbc_day , radnbcc_day , radndc_day , radndcc_day
     +,          sel_rules_sw
#ifdef overlap
     +,                b1_day ,      b2_day ,     b3_day ,      b4_day
#endif
     +          )

      alvdrc(iday(1:nday))   = alvdr_day(1:nday)
      alvdfc(iday(1:nday))   = alvdf_day(1:nday)
      alndrc(iday(1:nday))   = alndr_day(1:nday)
      alndfc(iday(1:nday))   = alndf_day(1:nday)
      radvbc(iday(1:nday))   = radvbc_day(1:nday)
      radvdc(iday(1:nday))   = radvdc_day(1:nday)
      radnbc(iday(1:nday))   = radnbc_day(1:nday)
      radndc(iday(1:nday))   = radndc_day(1:nday)
      radvbcc(iday(1:nday))  = radvbcc_day(1:nday)
      radvdcc(iday(1:nday))  = radvdcc_day(1:nday)
      radnbcc(iday(1:nday))  = radnbcc_day(1:nday)
      radndcc(iday(1:nday))  = radndcc_day(1:nday)
      swinc(iday(1:nday))    = swinc_day(1:nday)

      tcldw_vis(iday(1:nday),:)  = tcldw_day(1:nday,:)
      tcldi_vis(iday(1:nday),:)  = tcldi_day(1:nday,:)

      fdsw(iday(1:nday),:)   = fdsw_day(1:nday,:)
      fdswcl(iday(1:nday),:) = fdswcl_day(1:nday,:)
      fusw(iday(1:nday),:)   = fusw_day(1:nday,:)
      fuswcl(iday(1:nday),:) = fuswcl_day(1:nday,:)

      deallocate(ts_day)
      deallocate(amu0_day)
      deallocate(slr_day)
      deallocate(acldmx_day)
      deallocate(asdir_day)
      deallocate(asdif_day)
      deallocate(pl_day)
      deallocate(dpl_day)
      deallocate(tl_day)
      deallocate(rmix_day)
      deallocate(cwrho_day)
      deallocate(cirho_day)
      deallocate(o3mix_day)
      deallocate(acld_day)
      deallocate(effrw_day)
      deallocate(effri_day)
#ifdef overlap
      deallocate(b1_day)
      deallocate(b2_day)
      deallocate(b3_day)
      deallocate(b4_day)
#endif
      deallocate(pl2_day)
      deallocate(radvbc_day)
      deallocate(radvdc_day)
      deallocate(radnbc_day)
      deallocate(radndc_day)
      deallocate(radvbcc_day)
      deallocate(radvdcc_day)
      deallocate(radnbcc_day)
      deallocate(radndcc_day)
      deallocate(alvdr_day)
      deallocate(alvdf_day)
      deallocate(alndr_day)
      deallocate(alndf_day)
      deallocate(swinc_day)
      deallocate(tcldw_day)
      deallocate(tcldi_day)
      deallocate(fdsw_day)
      deallocate(fdswcl_day)      
      deallocate(fusw_day)
      deallocate(fuswcl_day)

 1000 continue

      end subroutine rad_driver

!----------------------------------------------------------------------
