!-----------------------------------------------------------------------

      subroutine rad_lwr
     +                (     ncol ,       nlm ,        pp ,    tt2
     +,                      ppl ,        dp ,        tt ,   rmix
     +,                    cwrho ,     cirho ,     o3mix , cldamt
     +,                    effrw ,     effri ,        ts , cldmax
     +,                    umco2 ,     umch4 ,     umn2o ,     es
     +,                tcldw_irw , tcldi_irw ,      fdlw , fdlwcl
     +,                     fulw ,    fulwcl , sel_rules
#ifdef overlap
     +,                       b1 ,        b2 ,        b3 ,     b4
#endif
     +                )
     
      use kinds
      use rad_params
      use rad_planck

      implicit none

!-----------------------------------------------------------------------

! references:
! rad_lwr replaces crclwr written by G. Stephens. bugs_lwr computes the
! downward and upward longwave radiative fluxes, and longwave heating
! rates.
! Laura D. Fowler (slikrock/08-23-96).

! send comments to laura@slikrock.atmos.colostate.edu and
! norm@atmos.colostate.edu.

! modifications:

! * moved the computation of the all-sky and clear-sky radiative heating
!   rates to rad_driver.
!   Laura D. Fowler and Phil Partain/slikrock (01-27-99).

! * added effective radii of cloud droplets and ice crystals that are
!   dependent on the cloud water and cloud ice contents.
!   Laura D. Fowler/slikrock (06-08-00).

! * cleaned up the argument list to remove variables related to short
!   wave radiative transfer.
!   Laura D. Fowler/slikrock (02-01-00).

! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! subroutines called:

!     pscalei         : pressure scaling.
!     cloudg          : computes cloud optical properties of water/ice
!                       clouds.
!     gascon_ckd_parm : water vapor continuum absorption.
!     plank           : computes planck function.
!     comscp1         : combines optical properties for gray absorption
!                       (clouds and water vapor continuum).
!     comscp2         : combines optical properties for non-gray gaseous
!                       absorption.
!     gases           : computes gases absorption.
!     two_rt_lw       : two-stream parameterization.

! argument list variables:
! all arrays indexed as nlm correspond to variables defined in the
! middle of layers. all arrays indexed as nlm+1 correspond to variables
! defined at levels at the top and bottom of layers.

!     input arguments:
!     ----------------
      logical (kind=log_kind), intent(in)::
     &  sel_rules

      integer (kind=int_kind), intent(in)::
     &  ncol  !length of sub-domain.
     &, nlm   !number of layers.

      real (kind=dbl_kind), intent(in)::
     &  umco2 !concentration of CO2                              (ppm).
     &, umch4 !concentration of CH4                              (???).
     &, umn2o !concentration of N2o                              (???).

      real (kind=dbl_kind), intent(in),  dimension(ncol)::
     &  ts    !surface temperature                                 (K).
     &, cldmax!maximum cloud fraction                              (-). 

      real (kind=dbl_kind), intent(in), dimension(ncol,nlm)::
     &  ppl   !layer pressure                                    (hPa).
     &, dp    !layer thickness                                   (hPa).
     &, tt    !temperature                                         (K).
     &, rmix  !water vapor mixing ratio                        (kg/kg).
     &, cwrho !cloud water mixing ratio                        (g/m^3).
     &, cirho !cloud ice mixing ratio                          (g/m^3).
     &, o3mix !ozone mixing ratio                              (kg/kg).
     &, cldamt!cloud fraction                                      (-).
     &, effrw !effective radius of water droplets                 (mu).
     &, effri !effective radius of ice crystals                   (mu).
#ifdef overlap
     &, b1    !cloud overlap parameter                             (-).
     &, b2    !cloud overlap parameter                             (-).
     &, b3    !cloud overlap parameter                             (-).
     &, b4    !cloud overlap parameter                             (-).
#endif

      real (kind=dbl_kind), intent(in), dimension(ncol,nlm+1)::
     &  pp    !level pressure                                    (hPa).
     &, tt2   !temperature at layer edges                          (K).

      real (kind=dbl_kind), intent(in), dimension(ncol,mbir)::
     &  es    !spectral surface emissivity                         (-).

!     output arguments:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(ncol,nlm)::
     &  tcldi_irw!visible optical depth of ice clouds               (-).
     &, tcldw_irw!visible optical depth of water cloud              (-).

      real (kind=dbl_kind), intent(out), dimension(ncol,nlm+1):: 
     &  fdlw  !downward LW flux                                (W/m^2).
     &, fdlwcl!downward clear-ksy LW flux                      (W/m^2).
     &, fulw  !upward LW flux                                  (W/m^2).
     &, fulwcl!upward clear-sky LW flux                        (W/m^2).

! local variables:

      integer (kind=int_kind)::    
     &  i     !horizontal index.
     &, l     !vertical index.
     &, ib    !index of spectral interval.
     &, ig    !index of k-distribution.
     &, ibmbs !index of LW spectral interval.

      integer (kind=int_kind), dimension(ncol,nlm)::
     &  ip1   !used in conjunction with pressure weigthing.
     &, ip2   !used in conjunction with pressure weigthing.

      real (kind=dbl_kind)::
     &  hk    !weighted spectral solar constant                (W/m^2).
     &, tmax  !temperature threshold                               (K).                        .
     &, eps   !threshold for cloud optical properties                 .
     &, pdist
      data eps,tmax,pdist /1.e-05,340.,2./
          
      real (kind=dbl_kind), dimension(mbir)::
     &  kg    !nb of k-distributions per spectral intervals.  
      data kg /2,3,4,4,3,5,2,10,12,7,7,8/

      real (kind=dbl_kind), dimension(4,mbir)::
     &  bcoef !fitted coefficients for computation of the
!              Planck function as a function of the temperature.

      real (kind=dbl_kind), dimension(mbir)::
     &  asym_wat!spectral asymmetry factor of water clouds.
     &, asym_ice!spectral asymmetry factor of ice clouds.

      real (kind=dbl_kind), dimension(mb)::
     &  cnrw    !real part of refractive index (Water clouds).
     &, cniw    !imaginary part of refractive index (Water clouds).
     &, cnri    !real part of refractive index (Ice clouds).
     &, cnii    !imaginary part of refractive indec (Ice clouds).
     &, xlam    !center of spectral band.

      real (kind=dbl_kind), dimension(ncol,nlm):: 
     &  ttem    !local temperature                                 (K).
     &, pkd     !
     &, tau1    !all-sky optical depth                             (-).
     &, tauclr1 !clear-sky optical depth                           (-).
     &, tau     !all-sky optical depth                             (-).
     &, tauclr  !clear-sky optical depth                           (-).
     &, taer    !aerosol optical depth                             (-).
     &, tray    !rayley optical depth                              (-).
     &, tg      !gases optical depth                               (-).
     &, tgm     !wv continuum optical depth                        (-).
     &, tcldi   !ice cloud optical depth                           (-).
     &, tcldw   !water cloud optical depth                         (-).
     &, wc      !all-sky single scattering albedo                  (-).
     &, wcclr   !clear-sky single scattering albedo                (-).
     &, waer    !aerosol single scattering albedo                  (-).
     &, wray    !rayley single scattering albedo                   (-).
     &, wcldi   !ice cloud single scattering albedo                (-).
     &, wcldw   !water cloud single scattering albedo              (-).
     &, asym    !all-sky asymmetry factor                          (-).
     &, asyclr  !clear-sky asymmetry factor                        (-).
     &, asyaer  !aerosol asymmetry factor                          (-).
     &, asycldi !ice cloud asymmetry factor                        (-).
     &, asycldw !water cloud asymmetry factor                      (-).
     &, fwclr   !
     &, fwcld   !

      real (kind=dbl_kind), dimension(ncol,nlm+1)::
     &  bf      !planck function for layers                    (W/m^2).
     &, fdg     !spectral downward flux                        (W/m^2).
     &, fdgcl   !spectral clear-sky downward flux              (W/m^2).
     &, fug     !spectral upward flux                          (W/m^2).
     &, fugcl   !spectral clear-sky upward flux                (W/m^2).

!     new planck function coefficients for above ir bands:
      data ((bcoef(i,ib),i=1,4), ib=1,mbir)/
     + -20.505924  ,  0.28733411 , -0.0013492457 ,  2.1273992e-06,
     + -21.046537  ,  0.30242115 , -0.0014625419 ,  2.3870096e-06,
     + -38.637063  ,  0.58203037 , -0.0029747389 ,  5.1784662e-06,
     + -16.145962  ,  0.26795971 , -0.0015193826 ,  2.9546527e-06,
     +  -8.0902446 ,  0.16953655 , -0.0011700223 ,  2.7019124e-06,
     +   1.3734172 ,  0.029527221, -0.00049561409,  1.6827241e-06,
     +  16.679324  , -0.17339655 ,  0.00027930627,  1.1720939e-06,
     +  17.751777  , -0.23418230 ,  0.00083621277, -1.5364067e-07,
     +  14.782737  , -0.22588400 ,  0.00099624277, -6.2372181e-07,
     +   7.5331770 , -0.14872300 ,  0.00084728739, -7.0478254e-07,
     +   0.53663800, -0.038073786,  0.00038212883, -3.6746469e-07,
     +  -1.8897969 ,  0.017366085,  0.00011727887, -1.2047530e-07 /

!     longwave asymmetry parameters:
!     (assumes: re=10 for water; re=30 for ice)
      data asym_wat /0.8200, 0.8547, 0.8619, 0.8683, 0.8723, 0.8703
     +,              0.8566, 0.8040, 0.7463, 0.6579, 0.5103, 0.1279 /
      data asym_ice /0.8524, 0.8791, 0.9022, 0.8797, 0.8637, 0.8722
     +,              0.8609, 0.8168, 0.7663, 0.6584, 0.6172, 0.3585 /

!---  cnrw and cniw (water clouds):
      data cnrw/1.3422,1.3281,1.3174,1.2901,1.3348,1.3700,1.3191,1.2821
     &,         1.3160,1.3030,1.2739,1.2319,1.1526,1.1981,1.3542,1.4917
     &,         1.5463,1.8718/
      data cniw/6.4790e-9,1.3417e-06,1.2521e-4,7.1533e-4,4.2669e-2
     &,         4.3785e-3,1.3239e-2 ,1.5536e-2,5.3894e-2,3.4346e-2
     &,         3.7490e-2,4.7442e-2 ,1.2059e-1,3.3546e-1,4.1698e-1
     &,         4.0674e-1,3.6362e-1 ,5.2930e-1/

!--- cnri and cnii (ice clouds):
      data cnri/1.3266,1.2986,1.2826,1.2556,1.2963,1.3956 
     &,         1.3324,1.2960,1.3121,1.3126,1.2903,1.2295
     &,         1.1803,1.5224,1.5572,1.5198,1.4993,1.7026/
      data cnii/7.0696e-9,9.1220e-7,1.2189e-4,5.7648e-4,4.3144e-2
     &,         8.2935e-3,1.5540e-2,2.5594e-2,5.9424e-2,5.1511e-2
     &,         4.0325e-2,4.7994e-2,2.3834e-1,3.0697e-1,1.1852e-1
     &,         4.3048e-2,6.3218e-2,1.5843e-1/

!---- spectral band center:
      data xlam/0.45  ,1.0   ,1.6  ,2.2  ,3.0   ,3.75  ,4.878 ,5.556
     &,         6.452 ,7.547 ,8.511,9.615,11.236,13.605,16.529,21.277 
     &,         29.412,71.403/

!-----------------------------------------------------------------------

!---- 0. initialize output arrays:

      tcldw_irw(:,:) = radc0 ; tcldi_irw(:,:) = radc0
      fdlw(:,:)      = radc0 ; fdlwcl(:,:)    = radc0
      fulw(:,:)      = radc0 ; fulwcl(:,:)    = radc0

      do l = 1, nlm
         do i = 1, ncol
            ttem(i,l) = min(tmax,tt(i,l))
          enddo
      enddo

!--   pressure scaling:

      call pscalei
     +        (ncol , nlm , ppl , pkd
     +,         ip1 , ip2
     +        )

!---- 1. loop over the mbir spectral intervals starts here:

      do ib = mbs+1, mb                 
         ibmbs = ib - mbs
         
         tray(:,:)   = 0.
         wray(:,:)   = 0.
         taer(:,:)   = 0.
         waer(:,:)   = 0.
         asyaer(:,:) = 1.

         tcldw(:,:) = radc0 ; wcldw(:,:) = radc0 ; asycldw(:,:) = radc0
         tcldi(:,:) = radc0 ; wcldi(:,:) = radc0 ; asycldi(:,:) = radc0

!---- 1.1 optical properties of water and ice clouds (as in crclwr for
!        now):

         call cloudg
     +           (   ncol ,    nlm  ,    mb ,    ib
     +,                pp ,     tt  , cwrho , effrw
     +,             pdist ,   cnrw  ,  cniw ,  cnri
     +,              cnii ,   xlam  , tcldw , wcldw
     +,           asycldw , .false.
     +           )

         call cloudg
     +           (   ncol ,   nlm   ,    mb ,    ib
     +,                pp ,    tt   , cirho , effri
     +,             pdist ,  cnrw   ,  cniw ,  cnri
     +,              cnii ,  xlam   , tcldi , wcldi
     +,           asycldi , .true.
     +           )

!     the asymmetry factor for water and ice clouds are fixed as for now
!     functions of the spectral intervals:

         do l = 1, nlm
            do i = 1, ncol
               if(cwrho(i,l) .ge. eps) asycldw(i,l) = asym_wat(ibmbs)
               if(cirho(i,l) .ge. eps) asycldi(i,l) = asym_ice(ibmbs)
            enddo
         enddo    

!---- 1.2 water vapor continuum:             

         call gascon_ckd
     +           (ncol , nlm, ib ,   pp
     +,            ppl ,  dp, tt , rmix
     +,            tgm
     +           )
!        call gascon_robs
!    +           (ncol , nlm ,   ib , ppl
!    +,             dp ,  tt , rmix , tgm
!    +           ) 

!---- 1.3 planck function:

!        call plank
!    +           (          ncol ,           nlm  ,             ts
!    +,                      tt2 , bcoef(1,ibmbs) , bcoef(2,ibmbs)
!    +,           bcoef(3,ibmbs) , bcoef(4,ibmbs)
!    +,           bf
!    +           )
         call planck
     +           (ncol, nlm,ibmbs,ts,tt,bf)
!        do i = 1, ncol
!           write(6,102) ibmbs,ts(i),bf(i,nlm+1)
!        enddo
  102 format(i3,2(1x,e15.8))

!---- 1.4 combines single-scattering properties for gray absorption:

         call comscp1
     +           (   ncol ,     nlm ,  taer ,   tcldi
     +,             tcldw ,     tgm ,  tray ,    waer
     +,             wcldi ,   wcldw ,  wray ,  asyaer
     +,           asycldi , asycldw ,  tau1 , tauclr1
     +,              asym ,  asyclr , fwcld ,   fwclr
     +           )
           
!---- loop over the k-probability distributions starts here:

         do ig = 1, kg(ibmbs)

!---- 1.5 gaseous absorption:

            call gases
     +              ( ncol ,   nlm ,    ib ,    ig
     +,                 pp ,    dp ,    tt ,  rmix
     +,              o3mix , umco2 , umch4 , umn2o
     +,                 hk ,    tg ,   pkd ,   ip1
     +,                ip2
     +              )

!---- 1.6 combines all single-scattering properties:

            call comscp2
     +              (  ncol ,  nlm ,      tg , fwcld
     +,               fwclr , tau1 , tauclr1 ,   tau
     +,              tauclr ,   wc ,   wcclr
     +              )

!---- 1.7 two-stream approximation:
#ifdef overlap
            call two_rt_lw_sel
     +              ( ncol ,    nlm , mbs , mbir
     +,                 ib , tauclr ,  es ,   bf
     +,              fugcl ,  fdgcl
     +              )

            call two_rt_lw_iter
     +              (     ncol ,   nlm ,    mbs ,   mbir
     +,                     ib ,cldamt ,     wc ,  wcclr
     +,                   asym ,asyclr ,    tau , tauclr
     +,                     es ,    bf ,    fug ,    fdg
     +,              sel_rules ,    b1 ,     b2 ,     b3
     +,                     b4
     +              )
#else
            call two_rt_lw
     +              (     ncol , nlm ,  mbs , mbir
     +,                     ib ,  wc , asym ,  tau
     +,                     es ,  bf ,  fug ,  fdg
     +,              sel_rules
     +              )

            call two_rt_lw
     +              (     ncol ,   nlm ,    mbs ,   mbir
     +,                     ib , wcclr , asyclr , tauclr
     +,                     es ,    bf ,  fugcl ,  fdgcl
     +,              sel_rules
     +              )
#endif
            fdlw(:,:)   = fdlw(:,:)   + fdg(:,:)*hk
            fulw(:,:)   = fulw(:,:)   + fug(:,:)*hk
            fdlwcl(:,:) = fdlwcl(:,:) + fdgcl(:,:)*hk
            fulwcl(:,:) = fulwcl(:,:) + fugcl(:,:)*hk

!---- save the optical depths of water and ice clouds in the infrared
!     window:
            select case (ib)
            
               case(12)            
                  tcldw_irw(:,:) = tcldw(:,:)
                  tcldi_irw(:,:) = tcldi(:,:)
            
            end select

         enddo ! end k-distribution

      enddo ! end spectral interval      

!---- formats:
  100 format(i3,8(1x,e15.8))

      end subroutine rad_lwr
      
c-----------------------------------------------------------------------
