!------------------------------------------------------------------------------

       subroutine rad_sounding
     +               (  nlen ,     len ,     nlm
     +,                 npbl ,     pl2 ,    bpl2
     +,                  tl2 ,      pl ,     bpl
     +,                   th ,      ql ,      cw
     +,                   ci ,   pblcw ,   pblci
     +,               climo3 , pl2_rad , tl2_rad
     +,               pl_rad , dpl_rad ,  tl_rad
     +,               ql_rad ,  cw_rad ,  ci_rad
     +,               o3_rad
     +               )

      use kinds
      use rad_params

      implicit none

!-----------------------------------------------------------------------
! REFERENCES:
! rad_sounding sets the atmospheric variables needed by the radiation
! codes. In particular, it adds a sponge layer between the top-of-the
! model and the top-of-the-atmosphere.
! Laura D. Fowler/slikrock (02-01-01).

! send comments to laura@atmos.colostate.edu

! MODIFICATIONS:
!     none.

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMONS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  nlen     !length of horizontal domain with ghost cells.
     &, len      !length of horizontal domain without ghost cells.
     &, nlm      !number of layers.
     &, npbl     !number of layers above the PBL top.

      real (kind=dbl_kind), intent(in), dimension(nlen,nlm)::
     &  pl       !layer pressure                                  (Pa).
     &, bpl      !(pl/p0)**kapa                                    (-).
     &, th       !layer potential temperature                      (K).
     &, ql       !water vapor mixing ratio                     (kg/kg).
     &, cw       !cloud water mixing ratio                     (kg/kg).
     &, ci       !cloud ice mixing ratio                       (kg/kg).
     
      real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1)::
     &  pl2      !pressure at layer edges                         (Pa).
     &, bpl2     !(pl2/p0)**kapa at layer edges                    (-).
     &, tl2      ! temperature at layer edges             (K).
     &, climo3   !climatological ozone mixing ratio            (kg/kg).

      real (kind=dbl_kind), dimension(nlen,npbl)::
     &   pblcw       !pbl cloud water mixing ratio             (kg/kg).
     &,  pblci       !pbl cloud ice mixing ratio               (kg/kg).
 
!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(len,nlm+1)::
     &  pl_rad   !layer pressure                                 (hPa).
     &, dpl_rad  !layer pressure thickness                       (hPa).
     &, tl_rad   !layer temperature                                (K).
     &, ql_rad   !water vapor mixing ratio                     (kg/kg).
     &, cw_rad   !cloud water mixing ratio                     (kg/kg).
     &, ci_rad   !cloud ice mixing ratio                       (kg/kg).
     &, o3_rad   !ozone mixing ratio                           (kg/kg).

      real (kind=dbl_kind), intent(out), dimension(len,nlm+2)::
     &  pl2_rad  !pressure at layer edges                        (hPa).
     &, tl2_rad  !temperature at layer edge                        (K).

! LOCAL VARIABLES:
      
      integer (kind=int_kind)::
     &  i, l, lpbl, nlm_rad

      real (kind=dbl_kind)::
     &  ptop_rad = 1.0_dbl_kind!                                  (Pa).

      real (kind=dbl_kind) ::
     &  f2
      real (kind=dbl_kind), dimension(len,nlm+1)::
     &  bpl_rad
      real (kind=dbl_kind), dimension(len,nlm+2)::
     &  bpl2_rad

!-----------------------------------------------------------------------

      lpbl = nlm-npbl ; nlm_rad = nlm+1

      f2 = radc1/(p0_sfc**kappa)

      do i = 1, len

         pl2_rad(i,1)            = ptop_rad*0.01
         pl2_rad(i,2:nlm_rad+1)  = pl2(i,1:nlm+1)*0.01
         bpl2_rad(i,1)           = f2*exp(kappa*log(ptop_rad))
         bpl2_rad(i,2:nlm_rad+1) = bpl2(i,1:nlm+1) 

         pl_rad(i,1)          = pl2_rad(i,2)
     +                        - (pl2_rad(i,2)-pl2_rad(i,1))/2.
         pl_rad(i,2:nlm_rad)  = pl(i,1:nlm)*0.01
         bpl_rad(i,1)         = f2*exp(kappa*log(pl_rad(i,1)*100.))
         bpl_rad(i,2:nlm_rad) = bpl(i,1:nlm)
         
         do l = 1, nlm_rad
            dpl_rad(i,l)        = pl2_rad(i,l+1)-pl2_rad(i,l)
         enddo

!-- 4/2008:  fix out of bounds array error in tl2_rad assignment
         !tl2_rad(i,1:nlm_rad+1) = tl2(i,1:nlm+1)
         tl2_rad(i,1) = tl2(i,1)
         tl2_rad(i,2:nlm_rad+1) = tl2(i,1:nlm+1)

!-- 4/2008:  use isothermal rather than isentropic assumption to fill top layer
!--          of tl_rad
         !tl_rad(i,1)            = th(i,1)*bpl_rad(i,1)
         tl_rad(i,1) = tl2(i,1)
         
         tl_rad(i,2:nlm_rad)    = th(i,1:nlm)*bpl(i,1:nlm)

         ql_rad(i,1)            = ql(i,1)
         ql_rad(i,2:nlm_rad)    = ql(i,1:nlm)

         cw_rad(i,1)              = radc0
!        cw_rad(i,2:nlm_rad)      = cw(i,1:nlm)
         cw_rad(i,2:lpbl+1)       = cw(i,1:lpbl)
         cw_rad(i,lpbl+2:nlm_rad) = pblcw(i,1:npbl)+cw(i,lpbl+1:nlm) 

         ci_rad(i,1)              = radc0
!        ci_rad(i,2:nlm_rad)      = ci(i,1:nlm)
         ci_rad(i,2:lpbl+1)       = ci(i,1:lpbl)
         ci_rad(i,lpbl+2:nlm_rad) = pblci(i,1:npbl)+ci(i,lpbl+1:nlm) 

         o3_rad(i,1:nlm_rad)      = climo3(i,1:nlm+1)

      enddo

      return
      end subroutine rad_sounding
      
!-----------------------------------------------------------------------
      
