!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module sigma_coordinate

!--------------------------------------------------------------------------
!
!  this module defines the number of vertical levels and includes
!     subroutines related to vertical advection.
!--------------------------------------------------------------------------

	use kinds
	use physical_parameters
        use model_parameters
	use domain_decomposition
	use abort

	implicit none
	save

#ifdef no_stratosphere
        integer (kind=int_kind), parameter ::  nfree    = 16
        integer (kind=int_kind), parameter ::  nstrat = 0
        real (kind=dbl_kind),  parameter ::    p_top = 1.0e2
        real (kind=dbl_kind),  parameter ::    p_interface = p_top
#else
        integer (kind=int_kind), parameter ::  nfree    = 28
        integer (kind=int_kind), parameter ::  nstrat = 14
        real (kind=dbl_kind),  parameter ::    p_top = 100.
        real (kind=dbl_kind),  parameter ::    p_interface = 1.0e4
#endif

#ifdef no_pbl
        integer (kind=int_kind), parameter ::  npbl   = 0
#else
        integer (kind=int_kind), parameter ::  npbl   = 1
#endif
        integer (kind=int_kind), parameter ::  nlm = nfree + npbl
        integer (kind=int_kind), parameter ::  ntrop  = nlm - (nstrat+npbl)
        integer (kind=int_kind), parameter ::  lpbl = nlm-npbl



	integer (kind=int_kind), parameter ::  nlmp1  = nlm + 1
	integer (kind=int_kind)            ::  nltp1 ! number of sigma levels
                                                     ! lt 0.1
        real (kind=dbl_kind), dimension(nlmp1)     :: sigl2
        real (kind=dbl_kind), dimension(nlm)       :: dsig, invdsig


	public ::
     *    init_vertical_coordinate,
     *    vert_flux_divergence,
     *    vert_flux_divergence_mom,
     *    vert_deriv_plg,
     *    vert_deriv_plgl2


!-------------------------------------------------------------------
!  VARIABLE DEFINITIONS
!-------------------------------------------------------------------
!  NOTE : layer centers and layer edges are numbered from top to bottom
!
!            sigl2(1)
!               sig(1)
!            sigl2(2)
!               sig(2)
!               .
!               .
!               sig(nlm)
!            sigl2(nlmp1)
!
!       nlm             : total number of vertical layer
!       nlmp1           : total number of layer edges
!       sigl2           : sigma level defined at the layer edges
!       dsig            : sigma layer thickness (dsig(k) = sigl2(k+1) - sig(k))
!       invdsig         : 1/dsig
!       sig             : sigma value @ layer center (changes with time and space)
!       nstrat          : number of stratospheric layers
!       ntrop           : number of layers in troposphere (excluding pbl)
!       npbl            : number of layers in pbl
!       p_interface     : pressure of interface seperating 'stratosphere' from 'troposphere' (Pa)
!       p_top           : pressure of top of model (Pa)
!       sigma_top       : sigma level specified for model TOA
!---------------------------------------------------------------------


!**************************************************************************

        contains

!**************************************************************************


!======================================================================
! BEGINNING OF INIT_VERTICAL_COORDINATE
!======================================================================
	subroutine init_vertical_coordinate

!---------------------------------------------------------------------------
!  this subroutine is called as a part of the grid initialization
!  it defines the vertical sigma levels a the layer edges and
!  computes the sigma thicknesses of each layer.
!  tdr, 3-5-98
!---------------------------------------------------------------------------

	implicit none

	integer (kind=int_kind) :: k,i,ioerr
 
 
!---------------------------------------------------------------------------
! LOG:
!  current set up for a pure sigma coordinate (Phillips, 1957)
!	pressure @ layer edge = sigl2 * surface pressure
!	tdr, 3-15-98
!
!  option for generalized sigma coordinate added 5-20-98, tdr
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! NOTES:
!  coordinate direction is increasing sigma with increasing pressure
!  and increasing index with increasing sigma
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! specify sigma levels at runtime, read in from data file
!---------------------------------------------------------------------------
	if(my_task.eq.0) then

	  open(unit=2,file=trim(datapath)//'data/sigma.defined'
     *,        form='formatted'
     *,        status='old',action='read',iostat=ioerr)
          if (ioerr /= 0) then
            write(*,*) '*** init_vertical_coordinate: '
     *,       'sigma_coordinate file error ',ioerr
            stop
          endif
	  write(6,*) ' SIGMA IS DEFINED FROM DATA FILE '
	  do k=1,nlmp1
	    read(2,*)  i,sigl2(i)
	    write(6,*) i,sigl2(i)
	  enddo
	  close(2)

!---------------------------------------------------------------------------
! sanity check
!---------------------------------------------------------------------------
#ifndef no_pbl
	if(my_task.eq.0) then
	  if(maxval(sigl2).gt.2.0_dbl_kind+1.0e-10_dbl_kind) then
	    write(6,*) ' error 1 in sigma.defined'
	    write(6,*) ' sigma.defined inconsistent with vertical coordinate'
	    call abort_code
	  endif
	endif
#else
	if(my_task.eq.0) then
          if(maxval(sigl2).gt.1.0_dbl_kind+1.0e-10_dbl_kind) then
            write(6,*) ' error 2 in sigma.defined'
            write(6,*) ' sigma.defined inconsistent with vertical coordinate'
	    call abort_code
          endif
        endif
#endif

!---------------------------------------------------------------------------
! find number of sigma levels lt 0.1 (used for get_waterl2 in hstatc)
!---------------------------------------------------------------------------
#ifdef no_stratosphere
        nltp1 = 0
        do k = 1,nlmp1
           if(sigl2(k).lt.0.1_dbl_kind) nltp1 = nltp1 + 1
        enddo
#else
        nltp1 = nstrat
#endif



!---------------------------------------------------------------------------
! compute delta sigma
!---------------------------------------------------------------------------
	  do k=1,nlm
	    dsig(k) = sigl2(k+1) - sigl2(k)
	  enddo

!---------------------------------------------------------------------------
!  store the inverse of delta sigma
!---------------------------------------------------------------------------
	  invdsig(:) = c1 / dsig(:)

!---------------------------------------------------------------------------
! echo info to screen
!---------------------------------------------------------------------------
	  write(6,*)
	  write(6,*)
#ifndef no_pbl
	  write(6,*) ' GENERALIZED SIGMA COORDINATE IS BEING USED'
#else
	  write(6,*) ' PHILLIPS SIGMA COORDINATE IS BEING USED'
#endif
	  write(6,*)
	  write(6,*) '   defined sigma levels'
	  do k=1,nlmp1
	    write(6,*) '    ',k,sigl2(k)
	  enddo
	  write(6,*)
	  write(6,*) '   defined deltas and 1/deltas'
	  do k=1,nlm
	    write(6,*) '    ',k,dsig(k),invdsig(k)
	  enddo
	  write(6,*)
	  write(6,*) '      p_top  :',p_top
	  write(6,*)

	endif	! if(my_task.eq.0)

#ifdef mpi
	call MPI_BCAST(nltp1,1,mpi_int_type,0,MPI_COMM_ATMOS,ierr)
	call MPI_BCAST(sigl2,nlmp1,mpi_float_type,0,MPI_COMM_ATMOS,ierr)
	call MPI_BCAST(dsig ,nlm,mpi_float_type,0,MPI_COMM_ATMOS,ierr)
	call MPI_BCAST(invdsig,nlm,mpi_float_type,0,MPI_COMM_ATMOS,ierr)
#endif


	end subroutine init_vertical_coordinate
!===========================================================================
! END OF INIT_VERTICAL_COORDINATE
!===========================================================================


!===========================================================================
! BEGINNING OF VERT_FLUX_DIVERGENCE
!===========================================================================
	subroutine vert_flux_divergence(
#ifndef no_pbl
     *		tr,
#endif
     *		trl2,
     *		dsigdt,
#ifndef no_pbl
     *		entrainment,
     *		cumulusmf,
     *		pleak,
#endif
     *		xout   )

!---------------------------------------------------------------------------
!  NOTE: special option for treatment of flux across pbl top
!  by choosing the vertical_upwind_pbl option, the flux across the
!  pbl top is done backward in space.
!---------------------------------------------------------------------------

        implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm,nlm)  , intent(in)  :: 
     *			tr
#endif

        real (kind=dbl_kind), dimension(iim,jjm,nlmp1), intent(in)  :: 
     *			trl2,
     *			dsigdt

#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm)      , intent(in)  :: 
     *			entrainment,
     *			cumulusmf,
     *			pleak
#endif
!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(iim,jjm,nlm)  , intent(out) :: 
     *			xout

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm) :: 
     *			work1, 
     *			work2
#endif

	real (kind=dbl_kind), dimension(iim,jjm,nlmp1) :: 
     *			work_l2

        integer (kind=int_kind) :: k

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------
	xout = c0
	work_l2 = c0
#ifndef no_pbl
	work1 = c0
	work2 = c0
#endif

!---------------------------------------------------------------------------
! compute flux at layer edges
!---------------------------------------------------------------------------
	do k=1,nlmp1
	  work_l2(:,:,k) = trl2(:,:,k) * dsigdt(:,:,k)
	enddo

#ifndef no_pbl
!---------------------------------------------------------------------------
! upwind E and Mb seperately across pbl top
!---------------------------------------------------------------------------
#ifdef vertical_upwind_pbl
	k = nstrat+ntrop+1
        where(entrainment+pleak.ge.c0)
          work1 = grav*(entrainment+pleak)
        elsewhere
          work2 = grav*(entrainment+pleak)
        endwhere
	work2 = work2 - grav*cumulusmf

        work_l2(:,:,k) =   work1 * trl2(:,:,k)
     *                   + work2 * tr(:,:,k  )
#endif
#endif

	do k=1,nlm
	 xout(:,:,k) = (work_l2(:,:,k+1) - work_l2(:,:,k))*invdsig(k)
	enddo
	
	end subroutine vert_flux_divergence
!===========================================================================
! END OF VERT_FLUX_DIVERGENCE
!===========================================================================


!===========================================================================
! BEGINNING OF VERT_FLUX_DIVERGENCE_MOM
!===========================================================================
        subroutine vert_flux_divergence_mom(
     *          ndim,
     *          u,
     *          dsigdt,
     *		invpthick,
#ifndef no_pbl
     *          entrainment,
     *          cumulusmf,
     *          pleak,
#endif
     *          xout )

!---------------------------------------------------------------------------
!  PURPOSE: find the tendency of velocity due to vertical motion. the input
!    variable, u, may be the normal or tangential component. the curl and 
!    divergence of xout will be the appropriate forcing for the vorticity
!    and divergence equation, respectively.
!
!  NOTE : the values computed at the layer edges are not actually fluxes
!	see Arakawa and Lamb 1977 for more details
!---------------------------------------------------------------------------

        implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::
     *                  ndim

        real (kind=dbl_kind), dimension(ndim,iim,jjm,nlm)  , intent(in)  :: 
     *                  u

        real (kind=dbl_kind), dimension(ndim,iim,jjm,nlmp1), intent(in)  :: 
     *                  dsigdt

	real (kind=dbl_kind), dimension(ndim,iim,jjm,nlm  ), intent(in)  ::
     *                  invpthick

#ifndef no_pbl
        real (kind=dbl_kind), dimension(ndim,iim,jjm)      , intent(in)  :: 
     *                  entrainment,
     *                  cumulusmf,
     *                  pleak
#endif

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(ndim,iim,jjm,nlm)  , intent(out) :: 
     *                  xout

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
#ifndef no_pbl
        real (kind=dbl_kind), dimension(ndim,iim,jjm) :: 
     *		        work1, 
     *		        work2
#endif

        real (kind=dbl_kind), dimension(ndim,iim,jjm,nlmp1) :: 
     * 			workp
     *,			workm
#ifndef no_pbl
     *, 		u_up
#endif

        integer (kind=int_kind) :: k

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------
	xout = c0
	workp = c0
	workm = c0
#ifndef no_pbl
	work1 = c0
	work2 = c0
	u_up = c0

!---------------------------------------------------------------------------
! get upstream velocity
!---------------------------------------------------------------------------
	do k=2,nlm
	  where(dsigdt(:,:,:,k).gt.0) 
            u_up(:,:,:,k)=u(:,:,:,k-1)
	  elsewhere
	    u_up(:,:,:,k)=u(:,:,:,k  )
	endwhere
	enddo
#endif

!---------------------------------------------------------------------------
! compute quantities at bottom edge of each layer
!---------------------------------------------------------------------------
        do k=1,nlm-1
          workp(:,:,:,k) =  -p5*( u(:,:,:,k+1) - u(:,:,:,k  )) 
     *			              * dsigdt(:,:,:,k+1)
	enddo

!---------------------------------------------------------------------------
! compute quantities at top edge of each layer
!   somewhat redundant, but makes upwinding easier to understand
!---------------------------------------------------------------------------
	do k=2,nlm
          workm(:,:,:,k) =  -p5*( u(:,:,:,k  ) - u(:,:,:,k-1))
     *                                * dsigdt(:,:,:,k  )
        enddo

#ifndef no_pbl
!---------------------------------------------------------------------------
! upwind pbl if requested
!   the layer just above the pbl is nstrat+ntrop
!   work1 contains the upwind velocity relative to E+P
!   work2 contains the upwind velocity relative to CMF
!---------------------------------------------------------------------------
#ifdef vertical_upwind_pbl_mom
	work1 = c0
	work2 = c0
	where(entrainment(:,:,:)+pleak(:,:,:).gt.c0)
	  work1(:,:,:) = entrainment(:,:,:)+pleak(:,:,:)
	elsewhere
	  work2(:,:,:) = entrainment(:,:,:)+pleak(:,:,:)
	endwhere
	work2(:,:,:) = work2(:,:,:) - cumulusmf(:,:,:)

!---------------------------------------------------------------------------
! modify the quantity at the bottom of the layer just above the pbl top
!---------------------------------------------------------------------------
	k = nstrat+ntrop
 	workp(:,:,:,k) =  -c1*( u(:,:,:,k+1) - u(:,:,:,k) ) * grav * work2

!---------------------------------------------------------------------------
! modify the quantity at the top of the layer just below the pbl top
!---------------------------------------------------------------------------
	k = nstrat+ntrop+1
 	workm(:,:,:,k) =  -c1*( u(:,:,:,k) - u(:,:,:,k-1) ) * grav * work1

#endif
#endif

!---------------------------------------------------------------------------
! compute the mean of the layer edge quantities
!---------------------------------------------------------------------------
	do k=1,nlm

          xout(:,:,:,k) = (workp(:,:,:,k) + workm(:,:,:,k)) *
     *                       invdsig(k) * invpthick(:,:,:,k)
	enddo

	end subroutine vert_flux_divergence_mom
!===========================================================================
! END OF VERT_FLUX_DIVERGENCE_MOM
!===========================================================================


!===========================================================================
! BEGINNING OF VERT_DERIV_PLG
!===========================================================================
	subroutine vert_deriv_plg(xin,plg,xout)

        implicit none

        real (kind=dbl_kind), dimension(iim,jjm,nlm), intent(in) :: 
     *                             xin, 
     *                             plg

        real (kind=dbl_kind), dimension(iim,jjm,nlmp1), intent(out) :: 
     *                             xout

        integer (kind=int_kind) :: k
 
        do k=2,nlm
        xout(:,:,k) = (xin(:,:,k)-xin(:,:,k-1))/
     *                (plg(:,:,k)-plg(:,:,k-1))
        enddo
        xout(:,:,1) = c0
        xout(:,:,nlmp1) = c0

        end subroutine vert_deriv_plg
!===========================================================================
! END OF VERT_DERIV_PLG
!===========================================================================


!===========================================================================
! BEGINNING OF VERT_DERIV_PLGL2
!===========================================================================
	subroutine vert_deriv_plgl2(xin,plgl2,xout)

	implicit none

	real (kind=dbl_kind), dimension(iim,jjm,nlmp1), intent(in) :: 
     *                            xin, 
     *                            plgl2

	real (kind=dbl_kind), dimension(iim,jjm,nlm) , intent(out):: 
     *                            xout

	integer (kind=int_kind) :: k
 
	do k=1,nlm
	xout(:,:,k) =  (xin(:,:,k+1)  -  xin(:,:,k))/
     *		      (plgl2(:,:,k+1) - plgl2(:,:,k))
	enddo

	end subroutine vert_deriv_plgl2
!===========================================================================
! END OF VERT_DERIV_PLGL2
!===========================================================================

        end module sigma_coordinate

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
