!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module adjustments

!-----------------------------------------------------------------------
! PURPOSE: call a series of routines which adjust the state of the 
!    model. the routines return the adjusted state (not a time rate of
!    change.  Examples include dry convective adjustment and hole filling
!-----------------------------------------------------------------------

        use kinds
        use physical_parameters
	use domain_decomposition
        use model_parameters
        use model_control
        use prognostics
	use hstatc
        use tracer_names
        use vertical_diffusion
        use pbp_module
        use utility
#ifdef eau_ng
        use eau_liqfillng
#endif

        implicit none
	private
	save

	real (kind = dbl_kind), dimension(iim,jjm,nsdm) ::
     *			dcinc

	real (kind = dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *			dcadt,
     *			dcadq,
     *                  diffth_f, diffh_f, diffq_f

        real (kind=dbl_kind), dimension(nwalls,iim,jjm,nlm,nsdm) :: 
     *                  diffun_f, diffut_f

	real (kind = dbl_kind), dimension(iim,jjm,nlm,ntracer,nsdm) ::
     *			fill_rate

	public :: 	make_adjustments,
     *			init_adjustments,
     *		  	dcinc,
     *		  	dcadt,
     *		  	dcadq,
     *                  diffth_f,
     *                  diffun_f,
     *                  diffut_f,
     *			fill_rate

!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF INIT_ADJUSTMENTS
!======================================================================
	subroutine init_adjustments
	implicit none

	dcinc = c0
	dcadt = c0
	dcadq = c0
        diffth_f = c0
        diffh_f = c0
        diffq_f = c0
        diffun_f = c0
        diffut_f = c0
	fill_rate = c0

	end subroutine init_adjustments
!======================================================================
! END OF INIT_ADJUSTMENTS
!======================================================================


!======================================================================
! BEGINNING OF MAKE_ADJUSTMENTS
!======================================================================
        subroutine make_adjustments
        implicit none

        real (kind=dbl_kind), dimension(iim,jjm,nlm) :: workdc

!-----------------------------------------------------------------------
! we require (p/p0)**kappa at time level n4, so call press
!-----------------------------------------------------------------------
	do ksdm = 1,nsdm
        call press(
     *          plgl2 (:,:,: ,ksdm),
     *          plg   (:,:,: ,ksdm),
     *          bpgl2 (:,:,: ,ksdm),
     *          bpg   (:,:,: ,ksdm),
     *          ps    (:,:,n4,ksdm),
     *          pb    (:,:,n4,ksdm),
     *          iim, jjm)
	enddo
	  

!-----------------------------------------------------------------------
! call dry convective adjustment
!-----------------------------------------------------------------------
 	call dcadj

!-----------------------------------------------------------------------
! call hole filing
!-----------------------------------------------------------------------
#ifdef eau_ng
      call eau_fill_global
#else
        call hole_filler
#endif

!-----------------------------------------------------------------------
! call pbl mixing
!-----------------------------------------------------------------------
	if (npbl > 1) call mix_pbl

!-----------------------------------------------------------------------
! add other adjustment procedures here
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! adjustment diagnostics
!-----------------------------------------------------------------------

!----------------------------------------------------------------------------
! dry convective adjustment quantities
!----------------------------------------------------------------------------
	do ksdm = 1,nsdm
        workdc = dcadt(:,:,:,ksdm) / pthick(:,:,:,n4,ksdm)

        call pbp2sum(workdc, iim*jjm, c1, ipbp2_tntdc, ksdm, iim*jjm)

        workdc = dcadq(:,:,:,ksdm) / pthick(:,:,:,n4,ksdm)
        call pbp2sum(workdc, iim*jjm, c1, ipbp2_tnqdc, ksdm, iim*jjm)

        call pbp1sum(dcinc(1,1,ksdm), c1, ipbp1_dryconvinc, ksdm, iim*jjm)
        enddo

	end subroutine make_adjustments
!======================================================================
! END OF MAKE_ADJUSTMENTS
!======================================================================


!======================================================================
! BEGINNING OF DCADJ
!======================================================================
	subroutine dcadj

!-----------------------------------------------------------------------
! PURPOSE : instanteously adjust the column of air to be hydrostatically
!       stable. "Mix" potential temperature and water vapor.
!-----------------------------------------------------------------------

	implicit none

!-----------------------------------------------------------------------
! nlb allows the dca to "skip" the lowest nlb layers
! set to zero at the moment, this allow dcadj to mix across the pbl
!-----------------------------------------------------------------------
        integer (kind = int_kind), parameter :: nlb = 0

        integer (kind = int_kind) :: i,j,l,ninst,ii,lp1,
     *                               lbeg,lend,lmod,lend1,lbeg1

        real (kind = dbl_kind) :: sum1, sum2, sum3, sum4, dp,
     *                            tem, tem1,dctt, dcqq

        integer (kind = int_kind), allocatable, dimension(:,:) :: indx

        logical, dimension(iim,jjm) :: bitx, bitc
	logical  equal


!----------------------------------------------------------------------------
! do loop over (iim,jjm) blocks of data
!----------------------------------------------------------------------------
	do ksdm = 1, nsdm

!----------------------------------------------------------------------------
! initialize arrays (this routines works on chuncks of ksdm)
!  these are global arrays when are saved for diagnostics
!----------------------------------------------------------------------------
        DCADT(:,:,:,ksdm) = 0.0
        DCADQ(:,:,:,ksdm) = 0.0
	DCINC(:,:,ksdm) = 0.0

!----------------------------------------------------------------------------
! BUILD AN INDEX LIST FOR USE IN THE I-LOOP BELOW.
!   bitx is set true if the column is unstable somewhere
!----------------------------------------------------------------------------
      do j = 1,jjm
      do i = 1,iim
         bitx(i,j) = th(i,j,2,n4,ksdm).gt.th(i,j,1,n4,ksdm)
      enddo
      enddo

      DO  L=3,NLM-NLB
	 do j = 1,jjm
         do i = 1,iim
            bitc(i,j) = th(i,j,l,n4,ksdm).gt.th(i,j,l-1,n4,ksdm)
            bitx(i,j) = bitx(i,j).or.bitc(i,j)
         enddo
	 enddo
      enddo

!----------------------------------------------------------------------------
!   ninst is the number of columns with instability
!----------------------------------------------------------------------------
      ninst = 0
      do j = 1,jjm
      do i = 1,iim
         if(bitx(i,j)) then
            dcinc(i,j,ksdm) = 1.0
            ninst = ninst + 1
         endif
      enddo
      enddo

!----------------------------------------------------------------------------
! if no instability, then cycle
!----------------------------------------------------------------------------
      IF(NINST .EQ. 0) cycle

!----------------------------------------------------------------------------
! allocate space for indx
!----------------------------------------------------------------------------
      allocate(indx(ninst,2))

!----------------------------------------------------------------------------
!  indx is a collection of indices of the unstable columns
!----------------------------------------------------------------------------
      ii = 0
      do j = 1,jjm
      do i = 1,iim
         if(bitx(i,j)) then
            ii = ii + 1
            indx(ii,1) = i
	    indx(ii,2) = j
         endif
      enddo
      enddo

!----------------------------------------------------------------------------
!  unstable point loop
!----------------------------------------------------------------------------
      DO II=1,NINST
         I     = INDX(II,1)
	 J     = INDX(II,2)
         EQUAL = .TRUE.
         LBEG  = 1

!----------------------------------------------------------------------------
!  SEARCH FOR UNSTABLE LAYER AND SET ITS UPPER AND LOWER
!  LIMITS, 'LBEG' AND 'LEND'
!----------------------------------------------------------------------------
    1    L = LBEG
    2    LP1 = L + 1
         IF(TH(I,J,L,n4,ksdm) .GT. TH(I,J,L+1,n4,ksdm)) GOTO 3
         IF(TH(I,J,L,n4,ksdm) .LT. TH(I,J,L+1,n4,ksdm)) EQUAL = .FALSE.
         L = LP1
         IF(L .LT. NLM-NLB) GOTO 2

    3    LEND = L
         IF(LP1 .EQ. NLM-NLB .AND. EQUAL) GOTO 9
         IF(LEND .NE. LBEG) GOTO 4
         LBEG = LP1
         IF(LBEG .EQ. NLM-NLB) GOTO 9
         EQUAL = .TRUE.
         GOTO 1

!----------------------------------------------------------------------------
!     ADJUST THE POTENTIAL TEMPERATURE AND WATER IN UNSTABLE REGION
!       adjustment conserves enthalpy (cp T) and water mass
!----------------------------------------------------------------------------
    4    SUM1 = 0.0
         SUM2 = 0.0
         SUM3 = 0.0
         SUM4 = 0.0

!----------------------------------------------------------------------------
!   sum1 is integrated temperature
!   sum2 is integrated mass x (p/p0)**kapa
!   sum3 is integrated water vapor mass
!   sum4 is integrated mass
!----------------------------------------------------------------------------
         DO LMOD=LBEG,LEND
	    DP   = PTHICK(I,J,LMOD,n4,ksdm) * DSIG(LMOD)
            SUM1 = SUM1 + TH(I,J,LMOD,n4,ksdm)*bpg(I,J,LMOD,ksdm) * dp
            SUM2 = SUM2 + DP * bpg(I,J,LMOD,ksdm) 
            SUM3 = SUM3 + TRACER(I,J,LMOD,iq,n4,ksdm) * dp
            SUM4 = SUM4 + dp
         enddo

!----------------------------------------------------------------------------
!   tem is the mixed potential temperature
!   tem1 is the mixed water vapor mixing ratio
!----------------------------------------------------------------------------
         TEM  = SUM1/SUM2
         TEM1 = SUM3/SUM4

         DO LMOD=LBEG,LEND
            DCTT = TH(I,J,LMOD,n4,ksdm)
            DCQQ = TRACER(I,J,LMOD,iq,n4,ksdm)
            TH(I,J,LMOD,n4,ksdm) = TEM
            TRACER(I,J,LMOD,iq,n4,ksdm) = TEM1
            DCTT = TH(I,J,LMOD,n4,ksdm) - DCTT
            DCQQ = TRACER(I,J,LMOD,iq,n4,ksdm) - DCQQ
            DCADT(I,J,LMOD,ksdm) = DCADT(I,J,LMOD,ksdm) + DCTT
            DCADQ(I,J,LMOD,ksdm) = DCADQ(I,J,LMOD,ksdm) + DCQQ
         enddo

         IF(LBEG .EQ. 1 .AND. LEND .EQ. NLM-NLB) GOTO 9
         LEND1 = LEND + 1
         IF(LBEG .EQ. 1) GOTO 7
         LBEG1 = LBEG - 1
         IF(LEND .EQ. NLM-NLB) GOTO 8

!----------------------------------------------------------------------------
!     TEST STABILITY OF ADJUSTED LAYER WITH RESPECT TO LAYERS
!     ABOVE AND BELOW IT
!----------------------------------------------------------------------------
         IF(TH(I,J,LBEG1,n4,ksdm) .LT. TEM) LBEG = LBEG1
         IF(TH(I,J,LEND1,n4,ksdm) .GT. TEM) LEND = LEND1
         IF(LBEG .EQ. LBEG1 .OR. LEND .EQ. LEND1)GOTO 4
         LBEG = LEND
         GOTO 1

    7    IF(TH(I,J,LEND1,n4,ksdm) .LT. TEM) LBEG = LEND
         EQUAL = .TRUE.
         LEND = LEND1
         GOTO 1

    8    IF(TH(I,J,LBEG1,n4,ksdm) .GE. TEM) GOTO 9
         LBEG = LBEG1
         GOTO 4

    9    CONTINUE
      enddo   ! do ii = 1,ninst

      deallocate(indx)

      enddo	! do ksdm = 1,nsdm

!-----------------------------------------------------------------------
! convert dcadt and dcadq to rate of change
!-----------------------------------------------------------------------
	dcadt(:,:,:,:) = pthick(:,:,:,n4,:)*dcadt(:,:,:,:) * invdt 
	dcadq(:,:,:,:) = pthick(:,:,:,n4,:)*dcadq(:,:,:,:) * invdt

      end subroutine dcadj
!======================================================================
! END OF DCADJ
!======================================================================


!======================================================================
! BEGINNING OF HOLE_FILLER
!======================================================================
	subroutine hole_filler

!-----------------------------------------------------------------
! PURPOSE : prevent negative values from propagating
!  The method here is to fill hole one layer at a time. The possibility
!  exists that the total amount of substance in a layer may be negative.
!  In this case, the hole filler will be non-conservative. The amount of
!  non-conservative hole filling has been monitored and is in fact a
!  negligible source of error to the budgets of the water species.
!-----------------------------------------------------------------

	implicit none
	
	integer (kind = int_kind ) :: nt, k, isum

	real (kind = dbl_kind) :: 
     *			total = 0.0_dbl_kind,
     *			neg   = 0.0_dbl_kind,
     *			pos   = 0.0_dbl_kind,
     *			factor = 0.0_dbl_kind

	real (kind = dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *			work = 0.0_dbl_kind,
     *			mask = 0.0_dbl_kind

        real (kind = dbl_kind), dimension(iim,jjm,nsdm) :: 
     *                  area_mask,
     *                  one_2d   = 1.0_dbl_kind
        real (kind=dbl_kind) :: fill_lim

!-----------------------------------------------------------------
!  initialize
!-----------------------------------------------------------------
	area_mask = c1 
	isum = iim*jjm*nlm*nsdm
	fill_rate = c0

!-----------------------------------------------------------------
!  compute mass
!-----------------------------------------------------------------
	do k=1,nlm
	  mask(:,:,k,:) = area_mask(:,:,:)*pthick(:,:,k,n4,:)*dsig(k)
	enddo

!-----------------------------------------------------------------
!  loop over tracers
!-----------------------------------------------------------------
	do nt = 1,ntracer

!-----------------------------------------------------------------
!  skip if requested
!-----------------------------------------------------------------
	  if (fill_status(nt).eq.0) cycle

!-----------------------------------------------------------------
!  zero ghost cells
!-----------------------------------------------------------------
	  do k=1,nlm
 	    !work(:,:,k,:) = tracer(:,:,k,nt,n4,:)

! fill_lim allows the pbl water vapor to not be set to zero	
        fill_lim = c0
        if (k==nlm .and. nt==iq) fill_lim=1.e-10
 	    work(:,:,k,:) = tracer(:,:,k,nt,n4,:) - fill_lim
	  enddo

!-----------------------------------------------------------------
!  stop if a negative occurs
!-----------------------------------------------------------------
	  if(fill_status(nt).eq.2) then
	    if(minval(work).lt.c0) then
	       write(6,*) my_task, ' negative encountered for tracer ',nt
	       write(6,*) '      this tracer is named : ',tracer_name(nt)
	       write(6,*) '      aborting'
	       stop
	    endif
	  endif

!-----------------------------------------------------------------
!  fill holes
!-----------------------------------------------------------------
	  if(fill_status(nt).eq.1) then
	     !total = global_real_sum(work(:,:,:,:), mask(:,:,:,:), isum)
	     total = SUM(work*mask)
	     where(work(:,:,:,:).lt.c0) 
	        fill_rate(:,:,:,nt,:) = - work(:,:,:,:) * invdt
	     work(:,:,:,:) = c0
	     endwhere
	     !pos   = global_real_sum(work(:,:,:,:), mask(:,:,:,:), isum)
	     pos   = SUM(work*mask)
	     neg = total - pos

 	     if(total.lt.c0) then
              if(my_task.eq.0) 
     +           write(51,*) 'nsecond = ',nsecond_dyn, 
     +                       ' tracer ',tracer_name(nt),', punting'
              if(my_task.eq.0)
     +           write(51,*) 'nsecond = ',nsecond_dyn, 
     +                       ' total = ',total,' pos = ',pos,' neg = neg'
	        if(my_task.eq.0)write(6,*)' total<0, fill non-conservatively'
	        total = c1
	        neg   = c0
	        tracer(:,:,:,nt,n4,:) = c0
	        where(work.gt.c0)
	           fill_rate(:,:,:,nt,:) = - work(:,:,:,:) * invdt
	        endwhere	
	     endif

	     if(neg.ne.c0) then
	        factor = total / pos
              if(my_task.eq.0)
     +           write(51,*) 'nsecond = ',nsecond_dyn, 
     +                       'tracer ',tracer_name(nt),' factor= ',factor
	        tracer(:,:,:,nt,n4,:) = factor * work(:,:,:,:) + fill_lim
	        where(work.gt.c0)
	           fill_rate(:,:,:,nt,:) = -(c1 - factor) * work(:,:,:,:) * invdt
! sometimes the holes are so small they are missed by the truncation error
!  from computing total and pos. So we will fill all remaining holes - 
!  these are negligible holes that have slipped under the radar, so to speak
             where(tracer(:,:,:,nt,n4,:) .lt. c0) tracer(:,:,:,nt,n4,:) = c0
	        endwhere	
	     endif
	  endif		! if(fill_status(nt).eq.1)

	enddo		! do nt = 1,ntracer
	  
	end subroutine hole_filler
!======================================================================
! END OF HOLE_FILLER
!======================================================================


!======================================================================
! BEGINNING OF MIX_PBL
!======================================================================
	subroutine mix_pbl
	
!-----------------------------------------------------------------------
! PURPOSE:  for npbl>1, this subroutine will vertically diffuse a number
!    of model variables throughout the PBL.  These variables are:
!        moist static energy (h)
!        total water mixing ratio (r)
!        relative vorticity (eta)
!        divergence (div)
!        
!    h and r are mixed instead of potential temperature, because under
!    saturated conditions theta is not conserved.  Also, the new eta 
!    and div values are found by mixing the two momentum components 
!    located at the cell walls (normal and tangential wind components).
!    
!    The diffusion is implicit, so the equations for each layer of the 
!    PBL are solved my standard matrix methods.  
!
!    M. Branson 2/2001
!-----------------------------------------------------------------------

	integer (kind = int_kind) :: i, j, k, iwall

	real (kind = dbl_kind ), dimension(iim,jjm) :: 
     *		sump, sumth, sumdiv, sumeta

        real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) :: 
     *          eta_frc, div_frc, work,
     +          h, htmp, qtmp

        real (kind=dbl_kind), dimension(iim,jjm,nlmp1,nsdm) :: 
     *     thflux, hflux, qflux, tl2, gaml2

!        real (kind=dbl_kind), dimension(nwalls,iim,jjm,nlm,nsdm) :: 
!     *                      untmp, uttmp

        real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: 
     *                      intth

        real (kind=dbl_kind), dimension(nwalls,iim,jjm,nsdm) :: 
     *                      intun, intut

        real (kind=dbl_kind), dimension(nlm,iim,jjm,nsdm) :: 
     *      cmixhsav, dmixhsav, emixhsav

!        real (kind=dbl_kind), dimension(nlm,nwalls,iim,jjm,nsdm) :: 
!     *  	         bmixm_w

        real (kind=dbl_kind), dimension(nlm,nwalls,iim,jjm,nsdm) :: 
     *  	         cmixm_w2, dmixm_w2, emixm_w2

	logical, parameter :: l_debug = .false.

#ifdef generalized_sigma

!-----------------------------------------------------------------
! added by mdb:  compute the moist static energies 
!-----------------------------------------------------------------
	do ksdm = 1, nsdm
          call get_h(
     *               phi(:,:,:,ksdm),
     *               bpg(:,:,:,ksdm),
     *               th(:,:,:,n3,ksdm),
     *               tracer(:,:,:,1,n3,ksdm),
     *               qsl2(:,:,:,ksdm),
     *               h(:,:,:,ksdm)
     *               )
        enddo

!-----------------------------------------------------------------
! added by mdb:  do small-scale mixing for theta instead of a 
!   mass-weighted average
!-----------------------------------------------------------------
        intth = c0
        intun = c0
        intut = c0

!-----------------------------------------------------------------
! solve simple tridiagonal system of equation to do diffusion 
! (simple k-theory) of potential temperature between the pbl 
! layers.
!-----------------------------------------------------------------
        call DIFF_COEFF

!-----------------------------------------------------------------
! save the matrix coefficients (they get overwritten by dgtsl)
!-----------------------------------------------------------------
        cmixhsav = cmixh
        dmixhsav = dmixh
        emixhsav = emixh

!-----------------------------------------------------------------
! diffuse h 
!-----------------------------------------------------------------
        cmixh = cmixhsav
        dmixh = dmixhsav
        emixh = emixhsav

        call MATRIX_SOLVE (h, htmp)

!-----------------------------------------------------------------
! diffuse q in same way as h
!-----------------------------------------------------------------
        cmixh = cmixhsav
        dmixh = dmixhsav
        emixh = emixhsav

        work(:,:,:,:) = tracer(:,:,:,1,n3,:)
        call MATRIX_SOLVE (work, qtmp)

!-----------------------------------------------------------------
! solve simple tridiagonal system of equation to do diffusion 
! (simple k-theory) of momentum between the pbl layers.  must 
! solve system of equations for both wind components (normal and
! tangential)
!-----------------------------------------------------------------
        call DIFF_COEFF_MOM

!-----------------------------------------------------------------
! save the matrix coefficients (they get overwritten by dgtsl)
!-----------------------------------------------------------------
        cmixm_w2 = cmixm_w
        dmixm_w2 = dmixm_w
        emixm_w2 = emixm_w

! tdr - reference to un must be removed
!       call MATRIX_SOLVE_CELL_WALLS (un, untmp)
! endtdr

        cmixm_w = cmixm_w2
        dmixm_w = dmixm_w2
        emixm_w = emixm_w2

! tdr - reference to ut must be removed
!       call MATRIX_SOLVE_CELL_WALLS (ut, uttmp)
! endtdr

	do ksdm = 1, nsdm
	  sump = c0
	  sumth = c0
	  sumdiv = c0
	  sumeta = c0
	  do k = nlm - npbl + 1, nlm
! old method:  just do mass-weighted averaging of theta, eta and div
!	    dp    = plgl2(:,:,k+1,ksdm) - plgl2(:,:,k,ksdm)
!	    sump  = sump  + dp
!	    sumth = sumth + th(:,:,k,n4,ksdm) * dp
!	    sumdiv = sumdiv + div(:,:,k,n4,ksdm) * dp
!	    sumeta = sumeta + eta(:,:,k,n4,ksdm) * dp

 	    do i=1,iim
 	      do j=1,jjm
                diffh_f(i,j,k,ksdm) = 
     +            (pthick(i,j,k,n4,ksdm)*htmp(i,j,k,ksdm) - 
     +            pthick(i,j,k,n3,ksdm)*h(i,j,k,ksdm))/dt
                diffq_f(i,j,k,ksdm) = 
     +            (pthick(i,j,k,n4,ksdm)*qtmp(i,j,k,ksdm) - 
     +            pthick(i,j,k,n3,ksdm)*tracer(i,j,k,1,n3,ksdm))/dt


                do iwall = 1,nwalls
! tdr - references to un and ut must be removed
!                 diffun_f(iwall,i,j,k,ksdm) = 
!    +             (untmp(iwall,i,j,k,ksdm) - un(iwall,i,j,k,ksdm))/dt
!                 diffut_f(iwall,i,j,k,ksdm) = 
!    +             (uttmp(iwall,i,j,k,ksdm) - ut(iwall,i,j,k,ksdm))/dt
! endtdr

                enddo
              enddo
            enddo
	  enddo

! compute the fluxes (go from ground up, use fact that flx at sfc = 0)
          thflux(:,:,nlmp1,ksdm) = c0
          hflux(:,:,nlmp1,ksdm) = c0
          qflux(:,:,nlmp1,ksdm) = c0
 	  do i=1,iim
 	    do j=1,jjm
              do k = nlm,nlm-npbl+1,-1
                hflux(i,j,k,ksdm) = hflux(i,j,k+1,ksdm)
     *            + diffh_f(i,j,k,ksdm)*dsig(k)/grav
                qflux(i,j,k,ksdm) = qflux(i,j,k+1,ksdm)
     *            + diffq_f(i,j,k,ksdm)*dsig(k)/grav
              enddo
            enddo
          enddo

! compute the theta flux from the h and q fluxes
          tl2 = thl2*bpgl2      ! temp at layer edges
 	  do i=1,iim
 	    do j=1,jjm
              do k = nlm,nlm-npbl+1,-1
                if (tracer(i,j,k,1,n3,ksdm) < qsl2(i,j,k,ksdm)) then
                  thflux(i,j,k,ksdm) = 
     *             (hflux(i,j,k,ksdm)-hltm*qflux(i,j,k,ksdm)) /
     *              (spec_heat_cp*bpgl2(i,j,k,ksdm))
                else
                  gaml2(i,j,k,ksdm) = gamfac*qsl2(i,j,k,ksdm) /
     *                     (tl2(i,j,k,ksdm)*tl2(i,j,k,ksdm))
                  thflux(i,j,k,ksdm) = hflux(i,j,k,ksdm) /
     *              (spec_heat_cp*bpgl2(i,j,k,ksdm)*
     *              (1.+gaml2(i,j,k,ksdm)))
                endif

              enddo

! now compute diffth_f from the theta fluxes computed from h-flux,r-flux
              do k = nlm,nlm-npbl+1,-1
                diffth_f(i,j,k,ksdm) = (thflux(i,j,k,ksdm) -
     *           thflux(i,j,k+1,ksdm)) * grav/dsig(k)
              enddo

            enddo
          enddo
         
!-----------------------------------------------------------------
! convert un, ut forcings to div and eta forcings
!-----------------------------------------------------------------
	  do k = nlm - npbl + 1, nlm
!tdrmod
!           call CURL_OF_FLUX(
!    *            diffut_f   (:,:,:,k,ksdm),
!    *            c_dist     (:,:,:,ksdm),
!    *            areai      (:,:,ksdm),
!    *            eta_frc    (:,:,k,ksdm))
!           call DIV_OF_FLUX(
!    *            diffun_f   (:,:,:,k,ksdm),
!    *            c_dist     (:,:,:,ksdm),
!    *            areai      (:,:,ksdm),
!    *            div_frc    (:,:,k,ksdm))
!endtdrmod
          enddo

	  do k = nlm - npbl + 1, nlm
!-- old mass-weighted averaging scheme
!            th(:,:,k,n4,ksdm) = sumth / sump
!            div(:,:,k,n4,ksdm) = sumdiv / sump
!            eta(:,:,k,n4,ksdm) = sumeta / sump
	    do i=1,iim
	      do j=1,jjm
 	        th(i,j,k,n4,ksdm) = th(i,j,k,n4,ksdm) +
     +            diffth_f(i,j,k,ksdm)*dt/pthick(i,j,k,n4,ksdm)
 	        div(i,j,k,n4,ksdm) = div(i,j,k,n4,ksdm) +
     +            div_frc(i,j,k,ksdm)*dt
 	        eta(i,j,k,n4,ksdm) = eta(i,j,k,n4,ksdm) +
     +            eta_frc(i,j,k,ksdm)*dt

                tracer(i,j,k,1,n4,ksdm) = tracer(i,j,k,1,n4,ksdm) +
     +            diffq_f(i,j,k,ksdm)*dt/pthick(i,j,k,n4,ksdm)
 	      enddo
  	    enddo
	  enddo

! sanity check (theta diffusion)
          if (l_debug) then
	  do i=1,iim
	    do j=1,jjm
              do k = 1,nlm
                intth(i,j,ksdm) = intth(i,j,ksdm)+
     +            diffth_f(i,j,k,ksdm)*dsig(k)
                do iwall = 1,nwalls
                  intun(iwall,i,j,ksdm) = intun(iwall,i,j,ksdm)+
     +              diffun_f(iwall,i,j,k,ksdm)*dsig(k)
                  intut(iwall,i,j,ksdm) = intut(iwall,i,j,ksdm)+
     +              diffut_f(iwall,i,j,k,ksdm)*dsig(k)
                enddo
              enddo
              if (abs(intth(i,j,ksdm)) > 1.e-10) then
                write(*,*) 'intth = ',intth(i,j,ksdm),i,j,ksdm
!                stop
              endif
            enddo
          enddo
          endif
	enddo  ! ksdm

        if (l_debug) then
          call FLUX_SUM_CELL_WALLS (intun,1)
          call FLUX_SUM_CELL_WALLS (intut,2)
        endif    ! l_debug

!        read(*,*)

#endif

	end subroutine mix_pbl
!======================================================================
! END OF MIX_PBL
!======================================================================

!======================================================================
! BEGINNING OF MATRIX_SOLVE
!======================================================================
	subroutine matrix_solve (xin, xout)

        real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) :: 
     *  	         xin, xout
	integer (kind = int_kind) :: i, j, k, info

	do ksdm = 1, nsdm
          do k = 1,nlm
            do i = 1,iim
              do j = 1,jjm
                bmixh(k,i,j,ksdm) = pthick(i,j,k,n3,ksdm)
     +            *xin(i,j,k,ksdm)
              enddo
            enddo
          enddo
        enddo 

	do ksdm = 1, nsdm
 	  do i=1,iim
 	    do j=1,jjm
 	      call DGTSL(nlm,cmixh(1,i,j,ksdm),dmixh(1,i,j,ksdm),
     +                   emixh(1,i,j,ksdm),bmixh(1,i,j,ksdm),info)
              if (info .ne. 0) then
                write(*,*) 'info = ',info,i,j,ksdm
                stop
              endif
              do k = 1,nlm
                xout(i,j,k,ksdm) = bmixh(k,i,j,ksdm)
              enddo
 	    enddo
 	  enddo
        enddo

	end subroutine matrix_solve

!======================================================================
! BEGINNING OF MATRIX_SOLVE_CELL_WALLS
!======================================================================
	subroutine matrix_solve_cell_walls (xin, xout)

!-----------------------------------------------------------------------
! PURPOSE:  Follow the template of average_to_cell_walls to solve 
!    the matrix of diffusion equations for momentum in the PBL.
!    (i.e., the walls must be done in a certain grid-specific order)
!    -- M. Branson, 11/2000
!-----------------------------------------------------------------------

        real (kind=dbl_kind), dimension(nwalls,iim,jjm,nlm,nsdm) :: 
     *  	         xin, xout

        real (kind=dbl_kind), dimension(nlm,nwalls,iim,jjm,nsdm) :: 
     *  	         bmixm_w

	integer (kind = int_kind) :: i, j, k, iwall, info

!----------------------------------------------------------------------------
! INITIALIZE
!----------------------------------------------------------------------------
        xout = 0.0

	do ksdm = 1, nsdm

        do j=jpad+1,jjm-jpad
          do i=ipad+1,iim-ipad
            do iwall = 1,nwalls
              do k = 1,nlm
                bmixm_w(k,iwall,i,j,ksdm) = xin(iwall,i,j,k,ksdm)
              enddo

 	      call dgtsl(nlm,cmixm_w(1,iwall,i,j,ksdm),
     +            dmixm_w(1,iwall,i,j,ksdm),emixm_w(1,iwall,i,j,ksdm),
     +            bmixm_w(1,iwall,i,j,ksdm),info)
              if (info .ne. 0) then
                write(*,*) 'info = ',info,i,j,ksdm
!                stop
                xout(iwall,i,j,k,ksdm) = c0
              else
                do k = 1,nlm
                  xout(iwall,i,j,k,ksdm) = bmixm_w(k,iwall,i,j,ksdm)
                enddo
              endif
 	    enddo
 	  enddo
        enddo

        j=jjm
        do i=ipad+1,iim-1
          do iwall = 2,3
            do k = 1,nlm
              bmixm_w(k,iwall,i,j,ksdm) = xin(iwall,i,j,k,ksdm)
            enddo

 	    call dgtsl(nlm,cmixm_w(1,iwall,i,j,ksdm),
     +          dmixm_w(1,iwall,i,j,ksdm),emixm_w(1,iwall,i,j,ksdm),
     +          bmixm_w(1,iwall,i,j,ksdm),info)
            if (info .ne. 0) then
              write(*,*) 'info = ',info,i,j,ksdm
!              stop
              xout(iwall,i,j,k,ksdm) = c0
            else
              do k = 1,nlm
                xout(iwall,i,j,k,ksdm) = bmixm_w(k,iwall,i,j,ksdm)
              enddo
            endif
          enddo
        enddo

        i=iim
        do j=jpad+1,jjm-1
          do iwall = 1,2
            do k = 1,nlm
              bmixm_w(k,iwall,i,j,ksdm) = xin(iwall,i,j,k,ksdm)
            enddo

 	    call dgtsl(nlm,cmixm_w(1,iwall,i,j,ksdm),
     +          dmixm_w(1,iwall,i,j,ksdm),emixm_w(1,iwall,i,j,ksdm),
     +          bmixm_w(1,iwall,i,j,ksdm),info)
            if (info .ne. 0) then
              write(*,*) 'info = ',info,i,j,ksdm
!              stop
              xout(iwall,i,j,k,ksdm) = c0
            else
              do k = 1,nlm
                xout(iwall,i,j,k,ksdm) = bmixm_w(k,iwall,i,j,ksdm)
              enddo
            endif
          enddo
        enddo

        i = iim; j = jjm; iwall = 2
        do k = 1,nlm
          bmixm_w(k,iwall,i,j,ksdm) = xin(iwall,i,j,k,ksdm)
        enddo

 	call dgtsl(nlm,cmixm_w(1,iwall,i,j,ksdm),
     +      dmixm_w(1,iwall,i,j,ksdm),emixm_w(1,iwall,i,j,ksdm),
     +      bmixm_w(1,iwall,i,j,ksdm),info)
        if (info .ne. 0) then
          write(*,*) 'info = ',info,i,j,ksdm
!          stop
          xout(iwall,i,j,k,ksdm) = c0
        else
          do k = 1,nlm
            xout(iwall,i,j,k,ksdm) = bmixm_w(k,iwall,i,j,ksdm)
          enddo
        endif

        i = 1; j = jjm-1
        do iwall = 2,3
          do k = 1,nlm
            bmixm_w(k,iwall,i,j,ksdm) = xin(iwall,i,j,k,ksdm)
          enddo

 	  call dgtsl(nlm,cmixm_w(1,iwall,i,j,ksdm),
     +        dmixm_w(1,iwall,i,j,ksdm),emixm_w(1,iwall,i,j,ksdm),
     +        bmixm_w(1,iwall,i,j,ksdm),info)
          if (info .ne. 0) then
            write(*,*) 'info = ',info,i,j,ksdm
!            stop
            xout(iwall,i,j,k,ksdm) = c0
          else
            do k = 1,nlm
              xout(iwall,i,j,k,ksdm) = bmixm_w(k,iwall,i,j,ksdm)
            enddo
          endif
        enddo

        i = iim-1; j = 1
        do iwall = 1,3
          do k = 1,nlm
            bmixm_w(k,iwall,i,j,ksdm) = xin(iwall,i,j,k,ksdm)
          enddo

 	  call dgtsl(nlm,cmixm_w(1,iwall,i,j,ksdm),
     +        dmixm_w(1,iwall,i,j,ksdm),emixm_w(1,iwall,i,j,ksdm),
     +        bmixm_w(1,iwall,i,j,ksdm),info)
          if (info .ne. 0) then
            write(*,*) 'info = ',info,i,j,ksdm
!            stop
            xout(iwall,i,j,k,ksdm) = c0
          else
            do k = 1,nlm
              xout(iwall,i,j,k,ksdm) = bmixm_w(k,iwall,i,j,ksdm)
            enddo
          endif
        enddo

        enddo ! ksdm loop

	end subroutine matrix_solve_cell_walls
!======================================================================
! END OF MATRIX_SOLVE_CELL_WALLS
!======================================================================

!======================================================================
! BEGINNING OF FLUX_SUM_CELL_WALLS
!======================================================================
	subroutine flux_sum_cell_walls (xin,itype)

        integer, intent(in) :: itype
	integer (kind = int_kind) :: i, j, iwall, ksdm

        real (kind=dbl_kind), dimension(nwalls,iim,jjm,nsdm) :: 
     *  	         xin
        character(16), dimension(2) :: warnstring

        warnstring(1) = 'intun > 1.e-10 :'
        warnstring(2) = 'intut > 1.e-10 :'

	do ksdm = 1, nsdm

        do j=jpad+1,jjm-jpad
          do i=ipad+1,iim-ipad
            do iwall = 1,nwalls
              if (abs(xin(iwall,i,j,ksdm)) > 1.e-10) then
                write(*,*) warnstring(itype),xin(iwall,i,j,ksdm),
     *            iwall,i,j,ksdm
                stop
              endif
            enddo
          enddo
        enddo

        j=jjm
        do i=ipad+1,iim-1
          do iwall = 2,3
            if (abs(xin(iwall,i,j,ksdm)) > 1.e-10) then
              write(*,*) warnstring(itype),xin(iwall,i,j,ksdm),
     *          iwall,i,j,ksdm
              stop
            endif
          enddo
        enddo

        i=iim
        do j=jpad+1,jjm-1
          do iwall = 1,2
            if (abs(xin(iwall,i,j,ksdm)) > 1.e-10) then
              write(*,*) warnstring(itype),xin(iwall,i,j,ksdm),
     *          iwall,i,j,ksdm
              stop
            endif
          enddo
        enddo

        i = iim; j = jjm; iwall = 2
        if (abs(xin(iwall,i,j,ksdm)) > 1.e-10) then
          write(*,*) warnstring(itype),xin(iwall,i,j,ksdm),
     *      iwall,i,j,ksdm
          stop
        endif

        i = 1; j = jjm-1
        do iwall = 2,3
          if (abs(xin(iwall,i,j,ksdm)) > 1.e-10) then
            write(*,*) warnstring(itype),xin(iwall,i,j,ksdm),
     *        iwall,i,j,ksdm
            stop
          endif
        enddo

        i = iim-1; j = 1
        do iwall = 1,3
          if (abs(xin(iwall,i,j,ksdm)) > 1.e-10) then
            write(*,*) warnstring(itype),xin(iwall,i,j,ksdm),
     *        iwall,i,j,ksdm
            stop
          endif
        enddo

        enddo ! ksdm loop

	end subroutine flux_sum_cell_walls
!======================================================================
! END OF FLUX_SUM_CELL_WALLS
!======================================================================
	end module adjustments
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

