
!======================================================================
! BEGIN CUMULUS_FRICTION MODULE
!======================================================================

module cumulus_friction

   use kinds
   use domain_decomposition
   use physical_parameters
   use sigma_coordinate
   use prognostics

   implicit none

   private

! save lamda and masflx from eau_cup_control

   real (kind=dbl_kind), dimension(iim,jjm,nlm-1,nlm,nsdm), public :: &
      lamda  ! fractional entrainment rate                 (m-1)
   real (kind=dbl_kind), dimension(iim,jjm,nlm,nlm,nsdm), public :: &
      masflx ! !cloud base convective mass flux           (kg/m2/s)

#ifdef cup_friction
! eta on hexagon centers (for one cloud base only and subdomain only)
   real (kind=dbl_kind), dimension(iim,jjm,nlm,nlm) :: &
      eta2

! eta on hexagon corners (for one cloud base only and subdomain only)
   real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,nlm) :: &
      eta2_c

! structure for mass flux info on corners
   type :: cupfrict
      integer (kind=int_kind) :: &
         ncup    ! number of convective points for cloud base and subdomain
      real (kind=dbl_kind), pointer, dimension(:,:) :: &
         eta2_cf       !  convective mass flux
      integer (kind=int_kind), pointer, dimension(:) :: &
         i_indx, &     ! i index for convective points
         j_indx, &     ! j index for convective points
         n_indx, &     ! corner index for convective points
         k_indx        ! cloud top index for convective points
   end type cupfrict

   type (cupfrict), dimension(nlm,nsdm) :: corners
   logical (kind=log_kind), save :: first = .true.

! public member functions
   public :: cumulus_friction_init, &
             cumulus_friction_compute

   contains

!======================================================================
! BEGIN SUBROUTINE CUMULUS_FRICTION_INIT
!======================================================================
   subroutine cumulus_friction_init

! This routine is called on the eau_cup_control timestep to get the
!   convective mass flux on the hexagon corners. These values remain
!   unchanged until eau_cup_control is called again

      implicit none

! local variables
      integer (kind=int_kind) :: jjj, ksdm, l, k, n, i, j ! loop indices
      integer (kind=int_kind) :: count                    ! counter

      if(.not.first) then
         do jjj = 1,nlm
            do ksdm = 1,nsdm
               if(corners(jjj,ksdm)%ncup > 0) then
                  deallocate(corners(jjj,ksdm)%eta2_cf)
                  deallocate(corners(jjj,ksdm)%i_indx)
                  deallocate(corners(jjj,ksdm)%j_indx)
                  deallocate(corners(jjj,ksdm)%k_indx)
                  deallocate(corners(jjj,ksdm)%n_indx)
               endif
            enddo
         enddo
      endif
      first = .false.

      do jjj = 1,nlm   ! cloud base loop
! Wrap lamda and masflx
         call wrap( max_level, &
                    iim,       &
                    jjm,       &
                    nlm-1,     &
                    nsdm,      &
                    lamda(:,:,:,jjj,:) )
         call wrap( max_level, &
                    iim,       &
                    jjm,       &
                    nlm,       &
                    nsdm,      &
                    masflx(:,:,:,jjj,:) )


         do ksdm = 1,nsdm

! compute mass flux
            call etacal

! average to the corners
            do k = 1,nlm
               do l = 1,nlm
                  call average_to_cell_corners(       &
                           ipad,                      &
                           iim,                       &
                           jjm,                       &
                           ndata,                     &
                           ncorners,                  &
                           s_weights(:,:,:,:,ksdm),   &
                           eta2(:,:,l,k),             &
                           eta2_c(:,:,:,l,k) )
               enddo
            enddo

! count the convective corner points
            corners(jjj,ksdm)%ncup = 0
            do l = 1,nlm-1
               do j = 1,jjm
                  do i = 1,iim
                     do n = 1,ncorners
                        if(maxval(eta2_c(n,i,j,:,l)) > c0 )     &
                           corners(jjj,ksdm)%ncup = corners(jjj,ksdm)%ncup + 1
                     enddo
                  enddo
               enddo
            enddo
            if(corners(jjj,ksdm)%ncup > 0) then
             allocate(corners(jjj,ksdm)%eta2_cf(corners(jjj,ksdm)%ncup,nlm))
             allocate(corners(jjj,ksdm)%i_indx(corners(jjj,ksdm)%ncup))
             allocate(corners(jjj,ksdm)%j_indx(corners(jjj,ksdm)%ncup))
             allocate(corners(jjj,ksdm)%k_indx(corners(jjj,ksdm)%ncup))
             allocate(corners(jjj,ksdm)%n_indx(corners(jjj,ksdm)%ncup))
             count = 0
             do l = 1,nlm-1
                do j = 1,jjm
                   do i = 1,iim
                      do n = 1,ncorners
                         if(maxval(eta2_c(n,i,j,:,l)) > c0 )     then
                            count = count + 1
                            corners(jjj,ksdm)%n_indx(count) = n
                            corners(jjj,ksdm)%i_indx(count) = i
                            corners(jjj,ksdm)%j_indx(count) = j
                            corners(jjj,ksdm)%k_indx(count) = l
                            corners(jjj,ksdm)%eta2_cf(count,:) = &
                              eta2_c(n,i,j,:,l)
                          endif
                      enddo
                   enddo
                enddo
             enddo
            endif
         enddo
      enddo

      contains 

      subroutine etacal
! This subroutine computes the convective mass flux for all convective points
!   in  subdomain ksdm with cloud base jjj

         integer (kind=int_kind) :: ktyp       ! loop indices
         integer (kind=int_kind) :: ktop         ! highest cloud top

         eta2(:,:,:,:) = c0

         do j = 1,jjm
            do i = 1,iim

! Is there convection at this point? If so then ktop < nlm. Find the highest
!   cloud top and set ktop to that level
               ktop = nlm
               do l = jjj-1,1,-1
                  if(masflx(i,j,l,jjj,ksdm) > 1.e-16_dbl_kind)   &
                          ktop = min(ktop,l)
               enddo
               if(ktop < nlm) then

! Convection exists = compute the mass flux
                  DO KTYP=KTOP,jjj-1
                     IF (MASFLX(I,j,KTYP,jjj,ksdm) .NE. 0.) THEN
                        DO K=jjj, ktyp, -1
                           eta2(I,j,k,KTYP) = masflx(i,j,ktyp,jjj,ksdm)   &
                               * (1.+ LAMDA(I,j,KTYP,jjj,ksdm)            &
                               * (phil2(i,j,k,ksdm)                       &
                               -  phil2(i,j,jjj,ksdm))*gravi)
                        enddo
                     ENDIF
                  enddo

               endif
            enddo
         enddo


      end subroutine etacal

   end subroutine cumulus_friction_init

!======================================================================
! END SUBROUTINE CUMULUS_FRICTION_INIT
!======================================================================


!======================================================================
! BEGIN SUBROUTINE CUMULUS_FRICTION_COMPUTE
!======================================================================
   subroutine cumulus_friction_compute

! This routine computes the convective redistribution of momentum for 
!   a column.

! variables averaged to corners
      real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,nsdm) :: phil_c
      real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,nsdm) :: phil2_c

      integer (kind=int_kind) :: count, k
      integer (kind=int_kind) :: jjj, ksdm, n, i, j, jmin, l  ! indices
      real (kind=dbl_kind), dimension(nspace,nlm) :: dutem  ! column friction rate
      real (kind=dbl_kind), dimension(nlm) :: etatem  ! column mask flux
      real (kind=dbl_kind), dimension(nlm) ::      &
             zz,zz1 ! column mid layer, layer base heights
      real (kind=dbl_kind), dimension(nlm) ::      &
             imassl ! column inverse mass

! average geopotential to corners
      do ksdm = 1,nsdm
         do l = 1,nlm
            call average_to_cell_corners(       &
                     ipad,                      &
                     iim,                       &
                     jjm,                       &
                     ndata,                     &
                     ncorners,                  &
                     s_weights(:,:,:,:,ksdm),   &
                     phi(:,:,l,ksdm),           &
                     phil_c(:,:,:,l,ksdm) )
            call average_to_cell_corners(       &
                     ipad,                      &
                     iim,                       &
                     jjm,                       &
                     ndata,                     &
                     ncorners,                  &
                     s_weights(:,:,:,:,ksdm),   &
                     phil2(:,:,l,ksdm),         &
                     phil2_c(:,:,:,l,ksdm) )
         enddo
      enddo

      do jjj = 1,nlm
         jmin = min(jjj,lpbl)
         do ksdm = 1,nsdm
            do count = 1,corners(jjj,ksdm)%ncup
               n = corners(jjj,ksdm)%n_indx(count)
               i = corners(jjj,ksdm)%i_indx(count)
               j = corners(jjj,ksdm)%j_indx(count)
               k = corners(jjj,ksdm)%k_indx(count)
               etatem(:) = corners(jjj,ksdm)%eta2_cf(count,:)
               zz(1:nlm) = phil_c(n,i,j,1:nlm,ksdm)*gravi
               zz1(1:nlm-1) = phil2_c(n,i,j,2:nlm,ksdm)*gravi
               imassl(1:nlm) = invpthick_c(n,i,j,1:nlm,ksdm)*grav
               call cufrict
               u_src(:,n,i,j,1:nlm-1,ksdm) =      &
                  u_src(:,n,i,j,1:nlm-1,ksdm) + dutem(:,1:nlm-1)
            enddo
         enddo
      enddo

! the cumulus friction needs wrapping
      call wrap_vertices(nspace,nlm,u_src(1,1,1,1,1,1))

      contains

      subroutine cufrict
! This subroutine computes the cumulus friction rate of change of momemtum
!   due to clouds of one cloud base level for one grid column

      real (kind=dbl_kind) :: zb, zk, zkb1, zkb2, zzctop   ! geopotential variables
      real (kind=dbl_kind) :: plamda               ! fractional entrainment rate
      real (kind=dbl_kind) :: tem
      real (kind=dbl_kind), dimension(nspace) :: umean               
      real (kind=dbl_kind), dimension(nspace, nlm) ::     &
                              ucc             ! in-cloud wind value
      real (kind=dbl_kind), dimension(nspace, nlm) ::            &
                              ul2             ! layer base wind
      real (kind=dbl_kind), dimension(nspace, nlm+1) ::            &
                              flux1, flux2
      real(dbl_kind), parameter :: amiu = 1.0_dbl_kind

      dutem(:,:) = c0

! this portion based on eau_cuptracers

!---- cloud base:

      zb = zz1(jjj-1)
      zzctop = zz(k)
      ucc(:,jjj) = u(:,n,i,j,jjj,n3,ksdm)
      ul2(:,1:nlm-1) =                   &
           0.5*(u(:,n,i,j,1:nlm-1,n3,ksdm) + u(:,n,i,j,2:nlm,n3,ksdm))
      ul2(:,nlm) = u(:,n,i,j,nlm,n3,ksdm)

!---- higher cloud levels:

      plamda = (etatem(jjj-1)/etatem(jjj) - 1._dbl_kind)     &
                      / (zz1(jjj-2)-zb)
      do l = jjj-1, k+1, -1
         zk   = zz1(l-1)-zz1(l)
         zkb1 = zz1(l-1)-zb
         zkb2 = zz1(l)-zb
         ucc(:,l) =                                                   &
               ((1.0+plamda*zkb2)*ucc(:,l+1) + plamda*ul2(:,l)*zk)    &
               / (1.0+plamda*zkb1)
      enddo

!---- cloud-top controlled by amiu:

      zk     = zzctop-zz1(k)
      zkb1   = zzctop-zb
      zkb2   = zz1(k)-zb
      umean(:) = 0.5*(u(:,n,i,j,k,n3,ksdm)+ul2(:,k))
      ucc(:,k) =                                      &
                       ((1.0+plamda*zkb2)*ucc(:,k+1)  &
                     + plamda*umean*zk+amiu             &
                     * plamda*(zz(k)-zb)*umean)         &
                     / (1.0+plamda*zkb1                 &
                     + amiu*plamda*(zz(k)-zb))          

      tem = imassl(k)*etatem(k)
      dutem(:,k) = dutem(:,k)  + tem*(ucc(:,k)-umean(:) )

! this portion based on eau_cupsubs

      flux1(:,1:nlm+1) = c0
      flux2(:,1:nlm+1) = c0

      umean(:) = 0.5 *(u(:,n,i,j,k,n3,ksdm)-ul2(:,k))
      do l = k+2, jmin
         flux1(:,l) = etatem(l) * (ul2(:,l-1)-u(:,n,i,j,l-1,n3,ksdm))
      enddo
      flux1(:,k+1) = etatem(k+1) * (ul2(:,k)-umean(:))
      do l = k+1, jmin
         flux2(:,l) = etatem(l) * (ul2(:,l-1)-u(:,n,i,j,l,n3,ksdm))
      enddo
      do l = k, jmin
         dutem(:,l) = dutem(:,l) - (flux1(:,l+1)-flux2(:,l)) * imassl(l)
      enddo

      end subroutine cufrict

   end subroutine cumulus_friction_compute

!======================================================================
! END SUBROUTINE CUMULUS_FRICTION_COMPUTE
!======================================================================

#endif
end module cumulus_friction
