!======================================================================
! PROGRAM BUGS
!======================================================================

	program bugs

!-----------------------------------------------------------------------
! PURPOSE : driver for the single-column model
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! DYNAMICAL CORE : LOG, LINEAGE and SUPPORT
!
!	This numerical model is not supported. Feel free to use it for
!	  academic purposes. If think you have found an error in the
!	  code, please inform us. We welcome any comments you might have
!	  on any aspect of the model.
!
!	We have put substantial effort into making this model readable
!	  and understandable. We have done this by using Fortran 90
!	  modules, defining variables, and using many comments. Anyone
!	  who takes the time to study the code and read the background
!	  papers should be able to use the model without many problems.
!
!	The original version of this SWM was written my Ross Heikes
!	  as a part of a Master's Thesis at CSU (1993). The basic
!	  2nd order accurate operators (Laplacian, Flux-Diverengce, and
!	  Jacobian) were included in the original version
!
!	While the original papers (listed below) use the twisted
!	  icosahedal grid, this swm does not used the twisted grid, it
!	  uses the spherical geodesic grid without the twisting.
!
!	The code was largely rewritten by Todd Ringler (1999) to form
!	  the basis of a dynamical core. Various operators related to
!	  calculating fluxes at cell walls were added.
!
!	A domain decomposition algorithm was implemented to make use
!	  of massively parallel computers. June 2000.
!
!       A new dynamical core based on Ringler and Randall (2002a,b)
!         was implement in April 2002.
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! DYNAMICAL CORE : REFERENCES 
!   on this swallow water model 
!	Heikes, R., 1993: The shallow water equations on a spherical 
!	   geodesic grid, Department of Atmospheric Science, Colorado 
!	   State University, Technical Report 524, pp. 175. 
!
!	Heikes, R. and R. A. Randall, 1995a: Numerical integration 
!	  of the shallow-water equations on a twisted icosahedral 
!	  grid. Part I: Basic design and results of tests. Mon. Wea. 
!	  Rev., 123, 1862-1880.
!
!	Heikes, R. and R. A. Randall, 1995b: Numerical integration of 
!	  the shallow-water equations on a twisted icosahedral grid. 
!	  Part II: A detailed description of the grid and an analysis 
!	  of numerical accuracy. Mon. Wea. Rev., 123, 1881-1887.
!
!   some relevant paper preceding the work of Heikes and Randall
!	Masuda, Y., and H. Ohnishi, 1986: An integration scheme of 
!	  the primitive equations model with an icosahedral-hexagonal 
!	  grid system and its application to the shallow water equations. 
!	  Short- and Medium-Range Numerical Weather Prediction. 
!	  Japan Meteorological Society, Tokyo, 317-326.
!
!	Sadourny, R., A. Arakawa, and Y. Mintz, 1968: Integration of the 
!	  nondivergent barotropic vorticity equation with an icosahedal-
!	  hexagonal grid for the sphere. Mon. Wea. Rev., 96, 351-356.
!
!	Williamson, D. L., 1968: Integration of the barotropic vorticity equation 
!	  on a spherical geodesic grid. Tellus, 20, 642-653.
!
!   work which uses this swm as a basis for further development
!	Ringler, T.D, R.P. Heikes, and D.A. Randall, 1998: Solving 
!	  the primitive equations on a spherical geodesic grid: A 
!	  technical report to a new class of dynamical cores. Department 
!	  of Atmospheric Science, Colorado State University, 
!	  Technical Report 665, pp. 62.
!
!	Ringler, T.D. and D.A. Randall, 2002a: A Potential Enstrophy
!	   and Energy Conserving Numerical Scheme for Solution of
!	   the Shallow-Water Equations on a Geodesic Grid. Mon. Wea.
!	   Rev., 130, 1397-1410.
!
!       Ringler, T.D. and D.A. Randall, 2002b: The ZM-grid: An
!	   Alternative to the Z-grid. Mon. Wea. Rev., 130, 1411-1422.
!-----------------------------------------------------------------------


	use physical_parameters
	use model_parameters
	use domain_decomposition
	use domain_decomposition_sfc
	use sigma_coordinate
	use model_control
	use timer
	use initialize
	use source_terms
	use post_step_module
	use adjustments
	use pbp_module
	use qp_pbp_diagnostics_module

#ifdef link_to_gcm_physics
	use pbp_module_sfc
        use bc_update
	use flux_coupler_module
        use sfc_control_module
        use fluxcoupler_fields
	use initialize_pcontrol
	use sfc_init
        use step_scm
#endif

	implicit none

        logical (kind=log_kind) :: fc_restart=.true.

!-------------------------------------------------------------------------
! INITIALIZATION MPI if NECESSARY
!-------------------------------------------------------------------------
	call timer_start('total')
	call timer_start('model init')

!-------------------------------------------------------------------------
! INITIALIZATION MODEL STATE
!-------------------------------------------------------------------------
	call initialize_model

!-------------------------------------------------------------------------
! update time
!-------------------------------------------------------------------------
	  call timer_start('model crl')
          call time_manager

!------------------------------------------------------------------
! based on the time step, get pointers
!------------------------------------------------------------------
          call get_pointers
	  call timer_stop('model crl')

#ifdef link_to_gcm_physics

!------------------------------------------------------------------
! initialize pcontrol: init cumulus kinetic energy and climatolical
!  ozone
!------------------------------------------------------------------
         call init_pcontrol

!------------------------------------------------------------------
! initialize the flux coupler
!------------------------------------------------------------------
         call init_flux_coupler

!------------------------------------------------------------------
! compute the time varying boundary conditions
!------------------------------------------------------------------
 	call bc_update_manager

!------------------------------------------------------------------
! call the flux coupler to compute diagnostic information such
!  as stratus cloudiness and surface albedo needed for the initial
!  call to pcontrol
!------------------------------------------------------------------
         call flux_coupler(fc_restart)
#endif

!-----------------------------------------------------------------------
! initialize the qp and pbp table information
!-----------------------------------------------------------------------
        call initialize_pbpinfo
#ifdef link_to_gcm_physics
        call initialize_pbpinfo_sfc
#endif

!-----------------------------------------------------------------------
! initialize the qp and pbp files
!-----------------------------------------------------------------------

!  for the atmosphere
        call initialize_pbpfiles(simulation_tag)
#ifdef link_to_gcm_physics
        call initialize_pbpfiles_sfc(simulation_tag)
#endif

!-----------------------------------------------------------------------
! initialize the global mean file
!-----------------------------------------------------------------------
!	call initialize_globinfo

        call timer_stop('model init')

!-------------------------------------------------------------------------
! INTEGRATION : stop model based on logical halt
!-------------------------------------------------------------------------
        do while(.not.l_halt)

!-------------------------------------------------------------------------
! initialize sources
!-------------------------------------------------------------------------
          call timer_start('sources')
	  call init_sources
          call timer_stop('sources')

#ifdef link_to_gcm_physics
!-------------------------------------------------------------------------
! compute the physics time rates of change
!   pcontrol uses time level n3, pthick(n3)*dsig (or plgl2, etc.)
!   can avoid "ghost cells" by masking with grid_center_mask(iim,jjm,ksmd)
!-------------------------------------------------------------------------
#ifdef eau_ng
          call pcontrol_ng
#else
          call pcontrol
#endif

!-------------------------------------------------------------------------
! compute the surface physics, get surface fluxes
!-------------------------------------------------------------------------
	  call flux_coupler(fc_restart)
#endif

 	  call step_dt_scm   ! combined pre_step and step_dt for scm
!          write(*,*) 'tracer(nt) = ',tracer(1,1,:,1,n4,1)
!          read(*,*)

!-------------------------------------------------------------------------
! make adjustments of state of atmosphere (like dcadj and hole filling)
!-------------------------------------------------------------------------
          call timer_start('adjust')
 	  call make_adjustments
          call timer_stop('adjust')

!-----------------------------------------------------------------
! accumulate time average (qp) and time series (pbp) diagnostics
!-----------------------------------------------------------------
        call timer_start('qp')
	call qp_pbp_diagnostics(nsecond_dyn)
        call timer_stop('qp')

!-------------------------------------------------------------------------
! output results based on logicals
!-------------------------------------------------------------------------
          call timer_start('model crl')

!-----------------------------------------------------------------
! compute diagnostically-derived quantities based on new time step
!-----------------------------------------------------------------
	call post_step

!-----------------------------------------------------------------------
! update fields that may be needed in the flux coupler
!-----------------------------------------------------------------------
        spdm_atm(:,:,:) = spdm(:,:,:)
        shear1_atm(:,:,:) = shear1(:,:,:)

          call timer_stop('model crl')

	  if(l_pbp) call write_pbp_diagnostics
#ifdef link_to_gcm_physics
	  if(l_pbp) call write_pbp_diagnostics_sfc
#endif

!-------------------------------------------------------------------------
! update time
!-------------------------------------------------------------------------
          call timer_start('model crl')
          call time_manager

!------------------------------------------------------------------
! based on the time step, get pointers
!------------------------------------------------------------------
          call get_pointers
          call timer_stop('model crl')

!------------------------------------------------------------------
! based on the time step, update the boundary conditions
!------------------------------------------------------------------
#ifdef link_to_gcm_physics
          if(l_bc_update) call bc_update_manager

!------------------------------------------------------------------
! if we wrote a restart, recompute albedo and stratus for restartability
!------------------------------------------------------------------
	  if(fc_restart) call flux_coupler(fc_restart)
#endif
        enddo   ! do while(.not.l_halt)

!-------------------------------------------------------------------------
! WRITE OUT TIMING
!-------------------------------------------------------------------------
	call timer_write
        call timer_stop('total')

	end program bugs

!======================================================================
! END OF PROGRAM BUGS
!======================================================================
