module scm_module

!  This module contains Single Column Models (SCM) variables and
!     utility routines.
!  Scm control input is read in from a file data/scm_control, which
!     determines which forcings are prescribe and where the data is
!     found.

!  To add a new prescribed forcing:
!     1. add a variable to each of the variable
!        classes below with a common string reflecting the variable name
!     2. If a profile, allocate it in scm_init for time levels 1 and 2 and
!        the interpolated time level
!     3. Add statements to read in its control variables from scm_control
!        in scm_init
!     4. Add statements to read in the forcing at time level 2 in scm_init
!     5. Add statements to copy time level 2 to time level 1, and to read
!        in a new time level 2 in scm_read
!     6. Add statements to perform time interpolation in scm_interp
!     7. If a profile, add statements to vertically interpolate it in scm_interp
!  In general, for the above steps you can use an existing variable as a 
!     template
!
!     8. Add code to use this field in the model. This may require development
!        of unique code on the users part.
!
!     The routines describe above are all part of this module

  use kinds
  use sigma_coordinate
  use prognostics
  use tracer_names
  use time_manager_module
  use netcdf_util
      
  implicit none
  private
  save

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  user defined parameters - location of the grid column (in radians)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  real (kind=dbl_kind), public ::  &
    lat_scm, &
    lon_scm

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  Variable type definition: These strings are found in each of the 
!   variable classes below and their definition is common to all the classes.
!   Included are the units expected for the forcing data of that field
!    Profiles are functions of pressure (vertical coordinate).
!     *div*   - horizontal wind divergence on a pressure surface profile (s^-1)
!     *qadvt* - horizontal water vapor advective tendency profile (kg/kg/s)
!     *tadvt* - horizontal temperature advective tendency profiles(K/s)
!     *ptend* - surface pressure tendency (hPa/hr)
!               Note! Pressure tendency has no time interpolation
!     *fss*   - surface sensible heat flux (W/m**2)
!     *fws*   - surface latent heat flux (W/m**2)
!     *u*     - zonal wind profile (m/s)
!     *v*     - meridional wind profile (m/s)
!     *tg*    - surface boundary temperature (K)
!     *alb*   - surface broadband albedo (-)
!     *rad*   - total radiative heating profile (K/day)
!     *qin*   - upstream water vapor profile (kg/kg)
!     *tin*   - upstream temperature profile (K)
!     *tauadv*- horizonal advective tendency timescale profile (s)
!     *vqadvt*- vertical water vapor advective tendency profile (kg/kg/s)
!     *vtadvt*- vertical temperature advective tendency profile (K/s)

!    Variable class - switches: what scm forcings are to be prescribed
  logical (kind=log_kind), public ::  &
    scm_div,       &
    scm_qadvt,     &
    scm_tadvt,     &
    scm_ptend,     &
    scm_fss,       &
    scm_fws,       &
    scm_u,         &
    scm_v,         &
    scm_t,         &
    scm_q,         &
    scm_tg,        &
    scm_alb,       &
    scm_ps,        &
    scm_rad,       &
    scm_qin,       &
    scm_tin,       &
    scm_tauadv,    &
    scm_vqadvt,    &
    scm_vtadvt,    &
    scm_presfluxes

  character (len=3), public ::  &
    scm_mode            ! forcing mode for scm      

  character (len=15), public ::  &
    scm_prefix          ! obs data file prefix      

!-- Variable class - variables for time interpolation : preceding time level
!                                                        (time level 1)
!-- profiles - defined on input pressure levels
  real (kind=dbl_kind), dimension(:), allocatable ::  &
    div_scm1, &
    qadvt_scm1, &
    tadvt_scm1, &
    u_scm1, &
    v_scm1, &
    t_scm1, &
    q_scm1, &
    qin_scm1, &
    tin_scm1, &
    rad_scm1, &
    tauadv_scm1, &
    vqadvt_scm1, &
    vtadvt_scm1
!-- surface variables
  real (kind=dbl_kind) ::  &
    fss_scm1, &
    fws_scm1, &
    tg_scm1, &
    alb_scm1, &
    ps_scm1

!-- Variable class - variables for time interpolation : subsequent time level
!                                                        (time level 1)
!-- profiles - defined on input pressure levels
  real (kind=dbl_kind), dimension(:), allocatable ::  &
    div_scm2, &
    qadvt_scm2, &
    tadvt_scm2, &
    u_scm2, &
    v_scm2, &
    t_scm2, &
    q_scm2, &
    qin_scm2, &
    tin_scm2, &
    rad_scm2, &
    tauadv_scm2, &
    vqadvt_scm2, &
    vtadvt_scm2
!-- surface variables
  real (kind=dbl_kind) ::  &
    fss_scm2, &
    fws_scm2, &
    tg_scm2, &
    alb_scm2, &
    ps_scm2

!-- Variable class - time interpolated variables
!-- profiles - defined on input pressure levels
  real (kind=dbl_kind), dimension(:), allocatable ::  &
    div_scm, &
    qadvt_scm, &
    tadvt_scm, &
    u_scm, &
    v_scm, &
    t_scm, &
    q_scm, &
    qin_scm, &
    tin_scm, &
    rad_scm, &
    tauadv_scm, &
    vqadvt_scm, &
    vtadvt_scm

  real (kind=dbl_kind), dimension(:), allocatable ::  x

!-- surface variables
  real (kind=dbl_kind), dimension(iim,jjm,nsdm), public ::  &
    ptend_scm = c0, &
    fss_scm = c0, &
    fws_scm = c0, &
    tg_scm = c0, &
    alb_scm = c0, &
    ps_scm = c0

!-- Variable class : time and vertically interpolated profiles
  real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm), public ::  &
    div_terp = c0, &
    qadvt_terp = c0, &
    tadvt_terp = c0, &
    rad_terp = c0, &
    vqadvt_terp = c0, &
    vtadvt_terp = c0, &
    tin_terp = c0, &
    qin_terp = c0, &
    u_terp = c0, &
    v_terp = c0, &
    t_terp = c0, &
    q_terp = c0, &
    tauadv_terp = c0

!-- Additional temporary profiles used in diagnostics
  real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm), public ::   &  
    qadvt_tem, &
    tadvt_tem, &
    vqadvt_tem, &
    vtadvt_tem, &
    trlxterm, &
    qrlxterm, &
    trevterm, &
    qrevterm

!-- Scm control variables
  integer (kind=int_kind), public ::  &
    ntscm,           &  ! number of time records in input profiles
    nlev_scm,        &  ! number of levels in input profiles
    scm_time_indx,   &  ! current scm time index 
    scm_gridpt_type, &  ! current grid point type  
    ioerr     ! file validation
  real (kind=dbl_kind), dimension(:), allocatable, public ::  &
    plev_scminit  ! pressure levels input profiles defined at (hPa)
  real (kind=dbl_kind), dimension(:), allocatable ::  &
    plev_scm, &  ! modified vertical pressure levels for vertical interpolation
    plev2_scm    ! layer-edge pressure levels for vertical interpolation
  logical (kind=log_kind) ::  &
    top2bot  ! if true, input profiles order from top of atmosphere to bottom

  real (kind=dbl_kind) ::  &
    scm_interval, &  ! interval (in hours) between input time levels
    scm_tauinit, &   ! initial time (in hours) in the scm time series
    tau_scm, &       ! scm time at time level 2 (hours)
    scmtfac, &       ! time interpolation weight
    scmtfaci        ! time interpolation weight

!-- work arrays for time interpolation
  real (kind=dbl_kind), dimension(:), allocatable ::          &
    tt_scm, th_scm, sht_scm
      	        
!-- tau values from netcdf file
  real (kind=dbl_kind), dimension(:), allocatable, public :: tau_scm_all 

!-- new derived types for scm driver data
  type :: scm_data_type_1
     character (len=30) :: name
     character (len=30) :: units
     character (len=80) :: long_name
     real (kind=real_kind), pointer :: data(:)
  end type
  
  type :: scm_data_type_2
     character (len=30) :: name
     character (len=30) :: units
     character (len=80) :: long_name
     real (kind=real_kind), pointer :: data(:,:)
  end type

  TYPE (scm_data_type_1) :: scm_data1d(7)
  TYPE (scm_data_type_2) :: scm_data2d(17)


!-- declare public subroutines
  public :: scm_init, scm_read, scm_interp

!*****************************************************************

  contains

!*****************************************************************

!======================================================================
! BEGINNING OF SCM_INIT
!======================================================================
  subroutine scm_init(tau)

!-- This subroutine determines what data will be used to drive the SCM, 
!-- opens the necessary files and reads the data for the first time step.

    implicit none

!-- Argument list
    real (kind=dbl_kind), intent(in) :: tau

!-- Local variables
    character (len=50) :: ncdfinfile

!-- what files do we open and use? Get this info from scm_control
    open(unit=2,file='data/scm_control',form='formatted',  &
         status='old',action='read',iostat=ioerr)
    if (ioerr /= 0) then
      write(*,*) '*** scm_init: scm_control file error ',ioerr
      stop
    endif

    read(2,*) scm_gridpt_type
    read(2,*) scm_mode
    read(2,*) scm_presfluxes
    read(2,*) scm_rad
    read(2,*) scm_alb

! get vertical structure of scm data
!    read(2,*) nlev_scm,top2bot
!    read(2,*) (plev_scminit(l),l=1,nlev_scm)
!    read(2,*) lon_scm
!    read(2,*) lat_scm
    read(2,*) ncdfinfile
    close(2)

!-- NEW:  read data from new netcdf input file
    call read_netcdf(ncdfinfile)
                       
    write(*,*) 'nlev_scm = ',nlev_scm
    write(*,*) 'plev_scminit from netcdf file = ',plev_scminit
    write(*,*) 'lat_scm from netcdf file = ',lat_scm
    write(*,*) 'lon_scm from netcdf file = ',lon_scm

    allocate(plev_scm(nlev_scm+1))
    allocate(plev2_scm(nlev_scm+1))
    plev_scminit = plev_scminit * 100.    ! mb --> Pa
      
    scm_interval = tau_scm_all(2)-tau_scm_all(1)
!    write(*,*) 'scm_interval = ',scm_interval
!    write(*,*) 'scm_data1d(:)%name = ',scm_data1d(:)%name
!    write(*,*) 'scm_data2d(:)%name = ',scm_data2d(:)%name

    lat_scm = lat_scm * pi /180.
    lon_scm = lon_scm * pi /180.

!-- allocate profile arrays
    allocate (tt_scm(nlev_scm),th_scm(nlev_scm),sht_scm(nlev_scm))
    allocate (div_scm(nlev_scm),div_scm1(nlev_scm),div_scm2(nlev_scm))
    allocate (qadvt_scm(nlev_scm),qadvt_scm1(nlev_scm),qadvt_scm2(nlev_scm))
    allocate (tadvt_scm(nlev_scm),tadvt_scm1(nlev_scm),tadvt_scm2(nlev_scm))
    allocate (u_scm(nlev_scm),u_scm1(nlev_scm),u_scm2(nlev_scm))
    allocate (v_scm(nlev_scm),v_scm1(nlev_scm),v_scm2(nlev_scm))
    allocate (t_scm(nlev_scm),t_scm1(nlev_scm),t_scm2(nlev_scm))
    allocate (q_scm(nlev_scm),q_scm1(nlev_scm),q_scm2(nlev_scm))
    allocate (qin_scm(nlev_scm),qin_scm1(nlev_scm),qin_scm2(nlev_scm))
    allocate (tin_scm(nlev_scm),tin_scm1(nlev_scm),tin_scm2(nlev_scm))
    allocate (rad_scm(nlev_scm),rad_scm1(nlev_scm),rad_scm2(nlev_scm))
    allocate (tauadv_scm(nlev_scm),tauadv_scm1(nlev_scm),tauadv_scm2(nlev_scm))
    allocate (vqadvt_scm(nlev_scm),vqadvt_scm1(nlev_scm),vqadvt_scm2(nlev_scm))
    allocate (vtadvt_scm(nlev_scm),vtadvt_scm1(nlev_scm),vtadvt_scm2(nlev_scm))
      
    allocate (x(nlev_scm))

!-- get first time level values
    scm_time_indx = 0


    do while (abs(tau-tau_scm) > scm_interval)
 
      scm_time_indx = scm_time_indx+1
      tau_scm = tau_scm_all(scm_time_indx)
      scm_tauinit = tau_scm
!      if (tau_scm > tau) then
      if (tau_scm-tau > 0.1) then
        write(*,*) 'scm_init: model tau out of SCM tau range',  &
                   ' (tau < first tau_scm). tau = ',tau,        &
                   ' tau_scm = ',tau_scm
        stop
      endif

      if ((scm_mode == 'rev') .or. (scm_mode == 'REV')) then
        qadvt_scm2(:) = scm_data2d(9)%data(:,scm_time_indx)
        tadvt_scm2(:) = scm_data2d(8)%data(:,scm_time_indx)
        ptend_scm = 0.0
      elseif ((scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
        qadvt_scm2(:) = scm_data2d(9)%data(:,scm_time_indx)
        tadvt_scm2(:) = scm_data2d(8)%data(:,scm_time_indx)
        div_scm2(:) = scm_data2d(4)%data(:,scm_time_indx)
      elseif ((scm_mode == 'rlx') .or. (scm_mode == 'RLX')) then
        tin_scm2(:) = scm_data2d(1)%data(:,scm_time_indx)
        qin_scm2(:) = scm_data2d(2)%data(:,scm_time_indx)
        tauadv_scm2(:) = scm_data2d(3)%data(:,scm_time_indx)
        div_scm2(:) = scm_data2d(4)%data(:,scm_time_indx)
      elseif ((scm_mode == 'rvx') .or. (scm_mode == 'RVX')) then
        qadvt_scm2(:) = scm_data2d(9)%data(:,scm_time_indx)
        tadvt_scm2(:) = scm_data2d(8)%data(:,scm_time_indx)
        tauadv_scm2(:) = scm_data2d(3)%data(:,scm_time_indx)
        ptend_scm = 0.0
      endif

      if (scm_presfluxes) then
        fss_scm2 = scm_data1d(2)%data(scm_time_indx)
        fws_scm2 = scm_data1d(1)%data(scm_time_indx)
        tg_scm2 = scm_data1d(7)%data(scm_time_indx)
      endif

      q_scm2(:) = scm_data2d(7)%data(:,scm_time_indx)
      t_scm2(:) = scm_data2d(10)%data(:,scm_time_indx)
      u_scm2(:) = scm_data2d(11)%data(:,scm_time_indx)
      v_scm2(:) = scm_data2d(12)%data(:,scm_time_indx)

      if (scm_rad) then
         rad_scm2(:) = scm_data2d(17)%data(:,scm_time_indx)
         !write(*,*) 'Radiative heating profile not available for this ARM dataset'
         !write(*,*) ' ==> stopping!'
         !stop
         !read(86,*,end=9001) (rad_scm2(l),l=1,nlev_scm)
      endif
      if (scm_alb) then
         write(*,*) 'Surface albedo data not available for this ARM dataset'
         write(*,*) ' ==> stopping!'
         stop
!         read(87,*,end=9001) alb_scm2
      endif

      if (scm_time_indx > ntscm) then
        write(*,*) 'scm_init: no more SCM data, stopping'
        stop
      endif
      
    enddo
      
    end subroutine scm_init
!======================================================================
! END OF SCM_INIT
!======================================================================


!======================================================================
! BEGINNING OF SCM_READ
!======================================================================
    subroutine scm_read

!-- This subroutine cycles the last time level of SCM data
!-- and reads the new time level

    implicit none

!-- local variables
    logical (kind=log_kind) :: scm_stop
    integer (kind=int_kind) :: nstep     ! time step counter

    nstep = step_count
    scm_stop = l_halt

    if (nstep == 0 .or.  &
        mod(tau-scm_tauinit+0.001_dbl_kind,scm_interval) < 0.01_dbl_kind) then
!-- move second time level values to first time level
      div_scm1(:) = div_scm2(:)
      tadvt_scm1(:) = tadvt_scm2(:)
      qadvt_scm1(:) = qadvt_scm2(:)
      vtadvt_scm1(:) = vtadvt_scm2(:)
      vqadvt_scm1(:) = vqadvt_scm2(:)
      u_scm1(:) = u_scm2(:)
      v_scm1(:) = v_scm2(:)
      t_scm1(:) = t_scm2(:)
      q_scm1(:) = q_scm2(:)
      rad_scm1(:) = rad_scm2(:)
      tin_scm1(:) = tin_scm2(:)
      tauadv_scm1(:) = tauadv_scm2(:)
      qin_scm1(:) = qin_scm2(:)
      tg_scm1 = tg_scm2
      alb_scm1 = alb_scm2
      ps_scm1 = ps_scm2
      fss_scm1 = fss_scm2
      fws_scm1 = fws_scm2

!-- get second time level values
      scm_time_indx = scm_time_indx+1
      if (scm_time_indx > ntscm) then
        write(*,*) '>>> scm_read: no more SCM data, stopping'
        write(*,*) '>>>   ntscm = ',ntscm,' scm_time_indx = ',scm_time_indx
        stop
      endif
      tau_scm = tau_scm_all(scm_time_indx)
        
      if ((scm_mode == 'rev') .or. (scm_mode == 'REV')) then
        qadvt_scm2(:) = scm_data2d(9)%data(:,scm_time_indx)
        tadvt_scm2(:) = scm_data2d(8)%data(:,scm_time_indx)
        ptend_scm = 0.0
      elseif ((scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
        qadvt_scm2(:) = scm_data2d(9)%data(:,scm_time_indx)
        tadvt_scm2(:) = scm_data2d(8)%data(:,scm_time_indx)
        div_scm2(:) = scm_data2d(4)%data(:,scm_time_indx)
        ptend_scm = scm_data1d(6)%data(scm_time_indx-1)
        ptend_scm = ptend_scm *100./ 3600.
      elseif ((scm_mode == 'rlx') .or. (scm_mode == 'RLX')) then
        tin_scm2(:) = scm_data2d(1)%data(:,scm_time_indx)
        qin_scm2(:) = scm_data2d(2)%data(:,scm_time_indx)
        tauadv_scm2(:) = scm_data2d(3)%data(:,scm_time_indx)
        div_scm2(:) = scm_data2d(4)%data(:,scm_time_indx)
        ptend_scm = scm_data1d(6)%data(scm_time_indx-1)
        ptend_scm = ptend_scm *100./ 3600.
      elseif ((scm_mode == 'rvx') .or. (scm_mode == 'RVX')) then
        qadvt_scm2(:) = scm_data2d(9)%data(:,scm_time_indx)
        tadvt_scm2(:) = scm_data2d(8)%data(:,scm_time_indx)
        tauadv_scm2(:) = scm_data2d(3)%data(:,scm_time_indx)
        ptend_scm = 0.0
      endif

      if (scm_presfluxes) then
        fss_scm2 = scm_data1d(2)%data(scm_time_indx)
        fws_scm2 = scm_data1d(1)%data(scm_time_indx)
        tg_scm2 = scm_data1d(7)%data(scm_time_indx)
      endif

      q_scm2(:) = scm_data2d(7)%data(:,scm_time_indx)
      t_scm2(:) = scm_data2d(10)%data(:,scm_time_indx)
      u_scm2(:) = scm_data2d(11)%data(:,scm_time_indx)
      v_scm2(:) = scm_data2d(12)%data(:,scm_time_indx)

      if (scm_rad) then
        rad_scm2(:) = scm_data2d(17)%data(:,scm_time_indx)
        !write(*,*) 'Radiative heating profile not available for this ARM dataset'
        !write(*,*) ' ==> stopping!'
        !stop
        !read(86,*,end=9001) (rad_scm2(l),l=1,nlev_scm)
      endif
      if (scm_alb) then
         write(*,*) 'Surface albedo data not available for this ARM dataset'
         write(*,*) ' ==> stopping!'
         stop
!         read(87,*,end=9001) alb_scm2
      endif

    endif  ! if...(top of subroutine)

    end subroutine scm_read
!======================================================================
! END OF SCM_READ
!======================================================================


!======================================================================
! BEGINNING OF SCM_INTERP
!======================================================================
    subroutine scm_interp

!-- This subroutine interpolates the scm forcing meteorological
!-- variables between their read times and interpolates profiles
!-- to the model sigma levels

    implicit none

!-- local variables
    real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) :: tem_terp
    integer (kind=int_kind) :: l

!-- get time interpolation weights
    !scmtfac = mod(tau-scm_tauinit+5.e-3_dbl_kind,scm_interval) / scm_interval
    scmtfac = mod(tau-scm_tauinit,scm_interval) / scm_interval
    scmtfaci = 1. - scmtfac

!-- set up pressure levels for vertical interpolation
    do l = 1,nlev_scm
      plev_scm(l) = min(plgl2(1,1,nlm+1,1), plev_scminit(l) )
    enddo
    plev_scm(nlev_scm+1) = plgl2(1,1,nlm+1,1)

!-- mb:  define layer edge pressures so that scm_terp will be consistent
!--    with the way it is used to initialize the scm (program scm_icatm.f)
    do l = 2,nlev_scm
      plev2_scm(l) = 0.5 * (plev_scminit(l-1)+plev_scminit(l))
    enddo
    plev2_scm(1) = 2.*plev_scminit(1) - plev_scminit(2)
    plev2_scm(nlev_scm+1) = plgl2(1,1,nlm+1,1)     
               
    tadvt_tem = 0.0
    qadvt_tem = 0.0
    qadvt_terp = 0.0
    tadvt_terp = 0.0
    trlxterm = 0.0
    qrlxterm = 0.0
    trevterm = 0.0
    qrevterm = 0.0

    if ((scm_mode == 'rev') .or. (scm_mode == 'REV') .or.  &
        (scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
	  qadvt_scm(:) = scmtfaci * qadvt_scm1(:) + scmtfac * qadvt_scm2(:)
	  call scm_terp( qadvt_scm(:), tem_terp(1,1,:,1) )
	  qadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      qadvt_terp(1,1,:,1) = qadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) 

      tadvt_scm(:) = scmtfaci * tadvt_scm1(:) + scmtfac * tadvt_scm2(:)
      call scm_terp( tadvt_scm(:), tem_terp(1,1,:,1) )
      tadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      tadvt_terp(1,1,:,1) = tadvt_terp(1,1,:,1) + tem_terp(1,1,:,1)  &
                               / bpg(1,1,:,1)
!      elseif ((scm_mode == 'rlx') .or. (scm_mode == 'RLX') .or.
!     *        (scm_mode == 'rvx') .or. (scm_mode == 'RVX')) then
    elseif ((scm_mode == 'rlx') .or. (scm_mode == 'RLX')) then
      qin_scm(:) = scmtfaci * qin_scm1(:) + scmtfac * qin_scm2(:)
      tin_scm(:) = scmtfaci * tin_scm1(:) + scmtfac * tin_scm2(:)
      tauadv_scm(:) = scmtfaci * tauadv_scm1(:) + scmtfac * tauadv_scm2(:)

      call scm_tterp(tt_scm(:), th(1,1,:,n4,1) )
      call scm_qterp(sht_scm(:), tracer(1,1,:,iq,n4,1), th(1,1,:,n4,1),  &
                     tt_scm(:) )
      qadvt_scm(:) = (qin_scm(:)-sht_scm(:)) / tauadv_scm(:)
      call scm_terp( qadvt_scm(:), tem_terp(1,1,:,1) )
      qrlxterm = tem_terp
      qadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      qadvt_terp(1,1,:,1) = qadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) 

      tadvt_scm(:) = (tin_scm(:) - tt_scm(:)) / tauadv_scm(:)
      call scm_terp( tadvt_scm(:), tem_terp(1,1,:,1) )
      trlxterm(1,1,:,1) = tem_terp(1,1,:,1)*bpg(1,1,:,1)
      tadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      tadvt_terp(1,1,:,1) = tadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) 
    endif

    if ((scm_mode == 'rlx') .or. (scm_mode == 'RLX') .or.  & 
          (scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
	  div_scm(:) = scmtfaci * div_scm1(:) + scmtfac * div_scm2(:)
	  call scm_terp( div_scm(:), div_terp(1,1,:,1) )
    endif

!      if ((scm_mode == 'rvx') .or. (scm_mode == 'RVX')) then
!         vqadvt_scm(:) = scmtfaci * vqadvt_scm1(:) + scmtfac * vqadvt_scm2(:)
!         call scm_terp( vqadvt_scm(:), vqadvt_terp(1,1,:,1) )
!         vqadvt_tem(1,1,:,1) = vqadvt_terp(1,1,:,1)
!         vtadvt_scm(:) = scmtfaci * vtadvt_scm1(:) + scmtfac * vtadvt_scm2(:)
!         call scm_terp( vtadvt_scm(:), vtadvt_terp(1,1,:,1) )
!         vtadvt_tem(1,1,:,1) = vtadvt_terp(1,1,:,1)
!         vtadvt_terp(1,1,:,1) = vtadvt_terp(1,1,:,1) / bpg(1,1,:,1)
!      endif
    if ((scm_mode == 'rvx') .or. (scm_mode == 'RVX')) then
      qadvt_scm(:) = scmtfaci * qadvt_scm1(:) + scmtfac * qadvt_scm2(:)
      call scm_terp( qadvt_scm(:), tem_terp(1,1,:,1) )
      qrevterm(1,1,:,1) = tem_terp(1,1,:,1)
      qadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      qadvt_terp(1,1,:,1) = qadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) 

      tadvt_scm(:) = scmtfaci * tadvt_scm1(:) + scmtfac * tadvt_scm2(:)
      call scm_terp( tadvt_scm(:), tem_terp(1,1,:,1) )
      trevterm(1,1,:,1) = tem_terp(1,1,:,1)
      tadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      tadvt_terp(1,1,:,1) = tadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) / bpg(1,1,:,1)

      q_scm(:) = scmtfaci * q_scm1(:) + scmtfac * q_scm2(:)
      t_scm(:) = scmtfaci * t_scm1(:) + scmtfac * t_scm2(:)
      th_scm(:) = t_scm(:) * (p0_sfc/plev_scminit(:))**kappa
      tauadv_scm(:) = scmtfaci * tauadv_scm1(:) + scmtfac * tauadv_scm2(:)

      call scm_tterp( tt_scm(:), th(1,1,:,n4,1) )

      call scm_qterp(sht_scm(:), tracer(1,1,:,iq,n4,1), th(1,1,:,n4,1),  &
                       tt_scm(:) )
      qadvt_scm(:) = (q_scm(:)-sht_scm(:)) / tauadv_scm(:)
      call scm_terp( qadvt_scm(:), tem_terp(1,1,:,1) )
      qrlxterm(1,1,:,1) = tem_terp(1,1,:,1)
      qadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      qadvt_terp(1,1,:,1) = qadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) 

      tadvt_scm(:) = (th_scm(:) - tt_scm(:)) / tauadv_scm(:)
      call scm_terp( tadvt_scm(:), tem_terp(1,1,:,1) )
      trlxterm(1,1,:,1) = tem_terp(1,1,:,1)*bpg(1,1,:,1)
      tadvt_tem(1,1,:,1) = tem_terp(1,1,:,1)
      tadvt_terp(1,1,:,1) = tadvt_terp(1,1,:,1) + tem_terp(1,1,:,1) 
    endif

    if (scm_rad) then
      rad_scm(:) = scmtfaci * rad_scm1(:) + scmtfac * rad_scm2(:)
      call scm_terp( rad_scm(:), rad_terp(1,1,:,1) )
      rad_terp(1,1,:,1) = rad_terp(1,1,:,1) / 86400. 
    endif

    u_scm(:) = scmtfaci * u_scm1(:) + scmtfac * u_scm2(:)
    call scm_terp( u_scm(:), u_terp(1,1,:,1) )

    v_scm(:) = scmtfaci * v_scm1(:) + scmtfac * v_scm2(:)
    call scm_terp( v_scm(:), v_terp(1,1,:,1) )

    t_scm(:) = scmtfaci * t_scm1(:) + scmtfac * t_scm2(:)
!    call scm_terp( t_scm(:), t_terp(1,1,:,1) )
!-- interp theta instead of t (to check vs what is read from ic_atm)
    th_scm(:) = t_scm(:) * (p0_sfc/plev_scminit(:))**kappa
    call scm_terp( th_scm(:), t_terp(1,1,:,1) )

!-- fill data at "top" levels -- assume isothermal atmosphere
    do l = 1,nlm
      if (t_terp(1,1,l,1) == c0)  &
        t_terp(1,1,l,1) = th_scm(1) * (2.*plev_scminit(1) /  &
              (plgl2(1,1,l,1)+plgl2(1,1,l+1,1))) ** kappa
    enddo

    q_scm(:) = scmtfaci * q_scm1(:) + scmtfac * q_scm2(:)
    call scm_terp( q_scm(:), q_terp(1,1,:,1) )

    if (scm_presfluxes) then
      fss_scm = scmtfaci * fss_scm1 + scmtfac * fss_scm2
      fws_scm = scmtfaci * fws_scm1 + scmtfac * fws_scm2
      tg_scm = scmtfaci * tg_scm1 + scmtfac * tg_scm2
    endif
    !write(*,*) 'scm_interp: fss_scm1 = ',fss_scm1,' fss_scm2 = ',fss_scm2,  &
    !           ' fss_scm = ',fss_scm
    !read(*,*)
    if(scm_alb) alb_scm = scmtfaci * alb_scm1 + scmtfac * alb_scm2

!    if(scm_ps) ps_scm = scmtfaci * ps_scm1 + scmtfac * ps_scm2

!*****************************************************************

    contains

!*****************************************************************

! routines for vertical interpolation of scm data

!======================================================================
! BEGINNING OF SCM_TERP
!======================================================================
    subroutine scm_terp( x_SCM, x )

!-- This is a generic vertical interpolation routine that takes data on
!-- the driver dataset levels (x_scm) and interpolates it onto the model
!-- vertical levels (x)

!-- *** modified to match the scheme used to compute the initialization files
!--     (m. branson, 11/03)

    implicit none

!-- argument list variables
    real (kind=dbl_kind), dimension(:), intent(in) ::  &
      x_SCM  ! input data on scm pressure levels
    real (kind=dbl_kind), dimension(:), intent(out) ::  &
      x      ! output data interpolated to gcm levels

!-- local variables
    integer (kind=int_kind) :: l, ll      ! loop indices
    real (kind=dbl_kind) :: pthick, delp

    do l = 1,nlm
      x(l) = c0
      if (plgl2(1,1,l+1,1).ge.plev2_SCM(1)) then  
        pthick = plgl2(1,1,l+1,1) - max(plev2_SCM(1),plgl2(1,1,l,1))
        do ll = 1,nlev_SCM
          delp = max(c0,min( plgl2(1,1,l+1,1), plev2_SCM(ll+1) ) -  &
                     max( plgl2(1,1,l,1), plev2_SCM(ll)))
          x(l) = x(l) + x_SCM(ll) * delp
        enddo
        x(l) = x(l) / pthick
      endif
    enddo

    end subroutine scm_terp
!======================================================================
! END OF SCM_TERP
!======================================================================

!======================================================================
! BEGINNING OF SCM_TTERP
!======================================================================
    subroutine scm_tterp( x_SCM, x )

!-- This subroutine interpolates GCM potential temperature to SCM levels
!-- for purposes of computing relaxation implied horizontal advection

    implicit none

!-- argument list variables
    real (kind=dbl_kind), dimension(:), intent(out) ::  &
      x_SCM  ! output data interpolated to scm pressure levels
    real (kind=dbl_kind), dimension(:), intent(in) ::  &
      x      ! input data on gcm levels

!-- local variables
    integer (kind=int_kind) :: l,ll !loop indices
    integer (kind=int_kind) :: la,lb!interpolation levels above and below
    real (kind=dbl_kind), dimension(nlm+1) :: pltem, xtem
    real (kind=dbl_kind) :: a, b, d

    xtem(1:nlm) = x(1:nlm)
    pltem(1:nlm) = plg(1,1,1:nlm,1)
    xtem(nlm+1) = x(nlm)
    pltem(nlm) = plgl2(1,1,nlm,1)
    pltem(nlm+1) = plgl2(1,1,nlm+1,1)
    la = 1
    do l = 1,nlev_SCM
      lb = la + 1
      do ll = la+1,nlm
        if (pltem(ll).lt.plev_SCM(l)) lb = ll+1
      enddo
      la = lb - 1
      d = pltem(lb) - pltem(la)
      a = (pltem(lb) - plev_scm(l)) / d
      b = 1. - a
      x_SCM(l) = a*xtem(la) + b*xtem(lb)
    enddo

    end subroutine scm_tterp
!======================================================================
! END OF SCM_TTERP
!======================================================================


!======================================================================
! BEGINNING OF SCM_QTERP
!======================================================================
    subroutine scm_qterp( x_SCM, x, t, t_scm )

!-- This subroutine interpolates GCM specific humidity to SCM levels
!-- for purposes of computing relaxation implied horizontal advection
!-- relative humidity is linearly interpolated above the pbl top

!-- 11/03:  fixed mistake in ttem computation (m. branson)

    use eau_util

    implicit none

!-- argument list variables
    real (kind=dbl_kind), dimension(:), intent(in) ::  &
      t_SCM  ! input temperature data to scm pressure levels
    real (kind=dbl_kind), dimension(:), intent(in) ::  &
      t      ! input temperature data on gcm levels
    real (kind=dbl_kind), dimension(:), intent(out) ::  &
      x_SCM  ! out water vapor interpolated to scm pressure levels
    real (kind=dbl_kind), dimension(:), intent(in) ::  &
      x      ! input water vapor data on gcm levels

!-- local variables
    integer (kind=int_kind) :: l,ll  !loop indices
    integer (kind=int_kind) :: la,lb !interpolation levels above and below
    real (kind=dbl_kind), dimension(nlm) :: xtem
    real (kind=dbl_kind), dimension(nlm+1) :: pltem
    real (kind=dbl_kind), dimension(1) :: ttem, qs
    real (kind=dbl_kind) :: a, b, d

    do l = 1,nlm-1
      pltem(l) = plg(1,1,l,1)
      ttem(1) = t(l) * (pltem(l)/p0_sfc)**kappa
      call eau_sat(1, pltem(l), ttem, qsats=qs)
      xtem(l) = x(l) / qs(1)      ! relative humidity
    enddo
    pltem(nlm) = plgl2(1,1,nlm,1)
    pltem(nlm+1) = plgl2(1,1,nlm+1,1)
    ttem(1) = t(nlm) * (pltem(nlm)/p0_sfc)**kappa
    call eau_sat(1, pltem(nlm), ttem, qsats=qs)
    xtem(nlm) = x(nlm) / qs(1)     ! relative humidity

    la = 1
    do l = 1,nlev_SCM
      lb = la + 1
      do ll = la+1,nlm
        if (pltem(ll) < plev_SCM(l)) lb = ll+1
      enddo
      la = lb - 1
      d = pltem(lb) - pltem(la)
      a = (pltem(lb) - plev_scm(l)) / d
      b = 1. - a
      if (la == nlm) then
        x_scm(l) = x(la)  !  mixed layer
      else
        ttem(1) = t_scm(l)*(plev_scm(l)/p0_sfc)**kappa
        call eau_sat(1, plev_scm(l), ttem, qsats=qs)
        x_SCM(l) = (a*xtem(la) + b*xtem(lb)) * qs(1)
      endif
    enddo

    end subroutine scm_qterp
!======================================================================
! END OF SCM_QTERP
!======================================================================

    end subroutine scm_interp
!======================================================================
! END OF SCM_INTERP
!======================================================================


    subroutine read_netcdf(fnm)
  
!-- New routine to read SCM driver data input from netcdf file.
!--(This replaces the individual ascii files that were previously used.)
!-- we know the dimensions will always be time and level (pressure).
!-- put all info into new derived types
  
    character (len=*), intent(in) :: fnm
  
    logical (kind=log_kind) :: exans
    integer (kind=int_kind) :: i, j, icnt0=0, icnt1=0, icnt2=0
    integer (kind=int_kind) :: ndims, nvars, ngatts, unlimdimid
    integer (kind=int_kind) :: status, ncid, timeid, tmpid
    integer (kind=int_kind) :: vartype, varndims, vardimids, varnatts
    real (kind=dbl_kind), dimension(:), allocatable :: x
    real (kind=dbl_kind), dimension(:,:), allocatable :: x2
    character (len=30) :: varname, atttext2
    character (len=80) :: attname, atttext
    
!-- initialize the scm driver data derived types
    call init_scm_type_1(scm_data1d)
    call init_scm_type_2(scm_data2d)

    inquire(file=trim(fnm),exist=exans)
    if (exans) then
      status = NF_OPEN(trim(fnm), NF_NOWRITE, ncid)
      if (status .ne. NF_NOERR) call handle_error(status,"nf_open")
    
      status = NF_INQ(ncid, ndims, nvars, ngatts, unlimdimid)
      if (status .ne. NF_NOERR) call handle_error(status,"nf_inq")
      write(*,*) 'ndims = ',ndims
      write(*,*) 'nvars = ',nvars
      write(*,*) 'ngatts = ',ngatts

      status = NF_INQ_DIMID(ncid,'time',timeid)
      if (status .ne. NF_NOERR) call handle_error(status,"dimid")
      status = NF_INQ_DIMLEN(ncid,timeid,ntscm)
      if (status .ne. NF_NOERR) call handle_error(status,"dimlen")
      status = NF_INQ_DIMID(ncid,'level',tmpid)
      if (status .ne. NF_NOERR) call handle_error(status,"dimid")
      status = NF_INQ_DIMLEN(ncid,tmpid,nlev_scm)
      if (status .ne. NF_NOERR) call handle_error(status,"dimlen")

!-- allocate variable sizes
      allocate(x(ntscm),tau_scm_all(ntscm))
      !allocate(x2(ntscm,nlev_scm))
      allocate(x2(nlev_scm,ntscm))
      allocate(plev_scminit(nlev_scm))
      do i = 1,7
        allocate(scm_data1d(i)%data(ntscm))
      enddo
      do i = 1,17
        !allocate(scm_data2d(i)%data(ntscm,nlev_scm))
        allocate(scm_data2d(i)%data(nlev_scm,ntscm))
      enddo
    
      do i = 1,nvars
        atttext = ''; atttext2 = ''
        status = NF_INQ_VAR(ncid, i, varname, vartype, varndims, vardimids, varnatts)
        if (status .ne. NF_NOERR) call handle_error(status,"nf_inq_var")
        write(*,*) '>>> varname = ',trim(varname), ' type = ',vartype, &
                   ' varndims = ',varndims,' vardimids = ',vardimids
        select case (varndims)
          case (1)
            write(*,*) '1d variable: ',varname
            if (varname == 'time') then
              status = NF_GET_VAR_DOUBLE(ncid,i,tau_scm_all)
            elseif (varname == 'level') then
              status = NF_GET_VAR_DOUBLE(ncid,i,plev_scminit)
            elseif (varname == 'latitude') then
              status = NF_GET_VAR_DOUBLE(ncid,i,lat_scm)
            elseif (varname == 'longitude') then
              status = NF_GET_VAR_DOUBLE(ncid,i,lon_scm)
            endif
          case (2)
            icnt1 = icnt1+1
            scm_data1d(icnt1)%name = varname
            status = NF_GET_VAR_DOUBLE(ncid,i,x)
            scm_data1d(icnt1)%data = x
            do j = 1,varnatts
              status = NF_INQ_ATTNAME(ncid, i, j, attname)
              if (attname == 'long_name') then
                status = NF_GET_ATT_TEXT(ncid, i, attname, atttext)
                scm_data1d(icnt1)%long_name = atttext
              else if (attname == 'units') then
                status = NF_GET_ATT_TEXT(ncid, i, attname, atttext2)
                scm_data1d(icnt1)%units = atttext2
              endif
            enddo
!            write(*,*) '2d variable: ',varname,' units = ',scm_data1d(icnt1)%units
          case (3)
            icnt2 = icnt2+1
            scm_data2d(icnt2)%name = varname
            status = NF_GET_VAR_DOUBLE(ncid,i,x2)
            scm_data2d(icnt2)%data = x2
            do j = 1,varnatts
              status = NF_INQ_ATTNAME(ncid, i, j, attname)
              if (attname == 'long_name') then
                status = NF_GET_ATT_TEXT(ncid, i, attname, atttext)
                scm_data2d(icnt2)%long_name = atttext
              else if (attname == 'units') then
                status = NF_GET_ATT_TEXT(ncid, i, attname, atttext2)
                scm_data2d(icnt2)%units = atttext2
              endif
            enddo
!            write(*,*) '3d variable: ',trim(varname),' units = ',atttext2
        end select
      
      enddo
!      write(*,*) scm_data2d(:)%name
    
      write(*,*) 'ntscm = ',ntscm
!      write(*,*) 'tau_scm from netcdf file = ',tau_scm_all
    
    else
      write(*,*) 'netcdf input file not found'
      stop
    endif

!  write(*,*) 'scm_data1d(2)%data(:) = ',scm_data1d(2)%data(:) 
!  write(*,*) 'scm_data2d(9)%data(1,:) = ',scm_data2d(9)%data(1,:) 
!  read(*,*)
  
    end subroutine read_netcdf


    subroutine init_scm_type_1(scmdata)

    type (scm_data_type_1) :: scmdata(:)
    integer (kind=int_kind) :: i,ilength
 
    ilength = size(scmdata)
    scmdata(:)%name = "no scm name"
    scmdata(:)%units = "no scm units"
    scmdata(:)%long_name = "no scm long name"
    do i=1,ilength
      nullify(scmdata(i)%data)
    enddo
    
    end subroutine init_scm_type_1


    subroutine init_scm_type_2(scmdata)

    type (scm_data_type_2) :: scmdata(:)
    integer (kind=int_kind) :: i,ilength

    ilength = size(scmdata)
    scmdata(:)%name = "no scm name"
    scmdata(:)%units = "no scm units"
    scmdata(:)%long_name = "no scm long name"
    do i=1,ilength
      nullify(scmdata(i)%data)
    enddo

    end subroutine init_scm_type_2

end module scm_module
