!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module source_terms

!-----------------------------------------------------------------------
! PURPOSE: computes sources for prognostic equations. examples include
!    del4 diffusion, pbl pleak term, and Held-Suarez forcing

!---- modifications:
!     added initialization of the elqng source terms for the eau_liqng
!     parameterization of fractional cloudiness.
!     Laura D. Fowler/mcbug (09-06-02).

!-----------------------------------------------------------------------

        use kinds
        use physical_parameters
        use model_parameters
        use model_control
	use dissipation_operator
	use operators_hl
        use wrap_face_data
        use prognostics
	use tracer_names

#ifndef no_pbl
#ifdef link_to_gcm_physics
#ifdef cup_friction
	use cumulus_friction
#endif
#endif

#ifdef gravity_wave_drag
	use gravity_wave_drag_module
#endif
#endif

#ifdef held_suarez
	use held_suarez_forcing_module
#endif

        implicit none
        save

	integer (kind=int_kind) :: k, n

	real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm) ::
     *		work_c

	real (kind=dbl_kind), dimension(iim,jjm) ::
     *		work

	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *		work1,
     *		work2,
     *		work3,
     *		work4

	real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *		work1_3d,
     *		work2_3d,
     *		work3_3d,
     *		work4_3d

	private ::
     *		k,
     *          work1,
     *          work2,  
     *          work3,
     *          work4,
     *		get_del4

	public ::
     *		get_sources,
     *		init_sources

!*****************************************************************

        contains

!*****************************************************************


!======================================================================
! BEGINNING OF GET_SOURCES
!======================================================================
	subroutine get_sources

	implicit none

!---------------------------------------------------------------------
! PURPOSE : a driver for computing various terms on the rhs, all of which
!    (with the exception of th_res) will be integrated Euler Forward
!    
! NOTE : all of the sources are additive, i.e. the contribution to
!    vorticity forcing from, say, gravity wave drag is added into the
!    array, eta_src, which holds the cumulative vorticity forcing. 
!    Therefore, all sources are initialized to zero.
!---------------------------------------------------------------------

!---------------------------------------------------------------------
! compute the del4 diffusion (requires wrapping)
!---------------------------------------------------------------------
 	call get_del4

#ifndef no_pbl
!---------------------------------------------------------------------
! compute the gravity wave drag source terms
!---------------------------------------------------------------------
#ifdef gravity_wave_drag
 	call compute_gravity_wave_drag
#endif

#ifdef cup_friction
        call cumulus_friction_compute
#endif

!---------------------------------------------------------------------
! compute the forcing due to surface stress
!---------------------------------------------------------------------
#ifdef link_to_gcm_physics
  	call compute_surface_stress
#endif
#endif

!---------------------------------------------------------------------
! compute idealized Held-Suarez forcing
!---------------------------------------------------------------------
#ifdef held_suarez
        call held_suarez_forcing
#endif

!---------------------------------------------------------------------
! compute an ad hoc source for tracer fields if no physics
!---------------------------------------------------------------------
#ifndef link_to_gcm_physics
!	call tracer_surface_flux
#else
#ifdef passive_tracers
        do n = 1, npassive
           if(ipassive(n).gt.0) then
              tracer_src(:,:,nlm,ipassive(n),:) =
     +              tracer_src(:,:,nlm,ipassive(n),:)
     +            + tracer_flux(:,:,n,:) * grav
           endif

!---------------------------------------------------------------------
! tracer decay
!---------------------------------------------------------------------

! this is an example of 60 day decay of a tracer

!          tracer_src(:,:,:,ipassive(n),:) =
!    *              tracer_src(:,:,:,ipassive(n),:)
!    *            - tracer(:,:,:,ipassive(n),n3,:) / (60.*86400.)

        enddo
#endif
#endif

	end subroutine get_sources
!======================================================================
! END OF GET_SOURCES
!======================================================================


!======================================================================
! BEGINNING OF INIT_SOURCES
!======================================================================
	subroutine init_sources
	implicit none

	u_src = c0
	th_src = c0
	th_res = c0
	tracer_src = c0
#ifndef no_pbl
	pleak = c0
#endif
#ifdef eau_ng
      elqng_src = c0
#endif

	end subroutine init_sources
!======================================================================
! END OF INIT_SOURCES
!======================================================================


!======================================================================
! BEGINNING OF GET_DEL4
!======================================================================
        subroutine get_del4
	implicit none

!	real (kind=dbl_kind) :: work2_c(nspace,ncorners,iim,jjm,nsdm)
        integer (kind=int_kind), parameter :: norder = 2
 	real (kind=dbl_kind) :: work1_c(nspace,ncorners,iim,jjm,nlm,nsdm)
 	real (kind=dbl_kind) :: work_q(iim,jjm,nlm,nsdm)
	integer k

!       work2_c = c0
!	do k=1,nlm
!	call dissipation(
!    *       u(:,:,:,:,k,n3,:),
!    *       trans_to_X1,
!    *       trans_to_X2,
!    *       work2_c(:,:,:,:,:))	
!	work2_c(1,:,:,:,:) = work2_c(1,:,:,:,:) * grid_corner_mask(:,:,:,:)
!       work2_c(1,:,:,:,:) = work2_c(1,:,:,:,:) * grid_corner_mask(:,:,:,:)
!	u_src(:,:,:,:,k,:) = u_src(:,:,:,:,k,:) + work2_c(:,:,:,:,:)
!	enddo

 	work1_c = c0
 	call laplacian_of_a_vector(
     *         nlm,
     *         norder,
     *         u(:,:,:,:,:,n3,:),
     *         work1_c(:,:,:,:,:,:) )
 	u_src = u_src - visc * work1_c(:,:,:,:,:,:)

      do k = 1,nlm
      call scalar_laplacian_to_the_n(
     *                    norder,
     *                    nlm,
     *                    tracer(:,:,:,n4,iq,:),
     *                    work_q   )
	  tracer_src(:,:,:,iq,:) = tracer_src(:,:,:,iq,:) - 1.e18*work_q
	  enddo

	end subroutine get_del4
!======================================================================
! END OF GET_DEL4
!======================================================================

#ifndef no_pbl
!===========================================================================
! BEGINNING OF COMPUTE_SURFACE_STRESS
!===========================================================================
	subroutine compute_surface_stress
	implicit none

!-----------------------------------------------------------------
!  drag :  surface "drag" coeffient, rho * cu * spdm  (kg/m^2/s)
!  drag_c : after wrapping drag, drag is averaged to cell corners
!  drag_c * V has units of stress (kg/m/s2 == N/m2 == stress)
!  to convert to acceleration multiply drag * V by grav/(pthick*dsig),
!    this takes the surface stress and spreads it through the bottom 
!    layer (which will be the entire pbl when npbl == 1)
!  take the curl (divergence) of this to get the vorticity 
!    (divergence) forcing
!-----------------------------------------------------------------

	work_c = c0

	do ksdm = 1,nsdm

	  work_c(1,:,:,:) = - invpthick_c(:,:,:,nlm,ksdm) *
     *                        invdsig(nlm) *
     *                        drag_c(:,:,:,ksdm) *
     *                        u(1,:,:,:,nlm,n3,ksdm) *
     *                        grav

          work_c(2,:,:,:) = - invpthick_c(:,:,:,nlm,ksdm) *
     *                        invdsig(nlm) *
     *                        drag_c(:,:,:,ksdm) *
     *                        u(2,:,:,:,nlm,n3,ksdm) *
     *                        grav

	  work_c(1,:,:,:) = work_c(1,:,:,:) * grid_corner_mask(:,:,:,ksdm)
          work_c(2,:,:,:) = work_c(2,:,:,:) * grid_corner_mask(:,:,:,ksdm)

	  u_src(:,:,:,:,nlm,ksdm) = u_src(:,:,:,:,nlm,ksdm) + work_c(:,:,:,:)

        enddo

	end subroutine compute_surface_stress
!===========================================================================
! END OF COMPUTE_SURFACE_STRESS
!===========================================================================
#endif

!===========================================================================
! BEGINNING OF TRACER_SURFACE_FLUX
!===========================================================================
	subroutine tracer_surface_flux

	implicit none

	integer (kind = int_kind) :: nt

	real (kind = dbl_kind) :: 
     *			center_point(2,ntracer) = 0.0_dbl_kind, 
     *			radius(ntracer) = 0.0_dbl_kind,
     *			decay(ntracer) = 0.0_dbl_kind,
     *			distance(iim,jjm,nsdm) = 0.0_dbl_kind,
     *			source(iim,jjm,nlm,nsdm) = 0.0_dbl_kind

!----------------------------------------------------------------------------
! set the center of each specification region (lon,lat), radians
!----------------------------------------------------------------------------
        center_point(1,:) = pi
        do nt = 1,ntracer
          center_point(2,nt) = (nt-1)/(nt+1)*pi
        enddo

!----------------------------------------------------------------------------
! set the radius (radians)
!----------------------------------------------------------------------------
        radius(:) = pi/8.0_dbl_kind

!----------------------------------------------------------------------------
! set the decay time scale (inverse seconds)
!----------------------------------------------------------------------------
        decay(:) = c1 / (25.0*86400.0)

!----------------------------------------------------------------------------
! compute distance (radians)
!----------------------------------------------------------------------------
	do nt = 1, ntracer
	  distance(:,:,:) = 
     *           sqrt ( (grid_center_lon(:,:,:) - center_point(1,nt))**2 
     *	               +(grid_center_lat(:,:,:) - center_point(2,nt))**2)
	  where(distance(:,:,:).lt.radius(nt))
	    source(:,:,nlm,:) = (c1 - tracer(:,:,nlm,nt,n3,:))/86400.0
	  endwhere

	  source(:,:,:,:) = source(:,:,:,:) - tracer(:,:,:,nt,n3,:) * decay(nt)

 	  tracer_src(:,:,:,nt,:) = tracer_src(:,:,:,nt,:) + source(:,:,:,:)

	enddo

	end subroutine tracer_surface_flux
!===========================================================================
! END OF TRACER_SURFACE_FLUX
!===========================================================================

	end module source_terms
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
