!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module step

!-----------------------------------------------------------------------
! PURPOSE: call a series of subroutine which each time step a single
!       prognostic variable.  Prognostic variables are held in prognostics.F
!-----------------------------------------------------------------------

	use kinds
        use physical_parameters
        use model_parameters
        use model_control
        use grid_mask
        use grid_metrics
        use operators
	use operators_hl
	use elliptic_solver
        use global_reductions
        use wrap_face_data
        use wrap_vertex_data
        use prognostics
        use timer
        use gather_scatter
        use tracer_names
        use hstatc
        use interpolate_to_latlon
#ifdef fluxcorrectedtransport
        use flux_corrected_transport
#endif
#ifdef eau_ng
        use eau_stepng
#endif

	implicit none
	save

        integer (kind = int_kind) :: i,j,k,nt,icorner,ispace
	real (kind = dbl_kind) :: fr1,fr2,fr3,fr4

	real (kind = dbl_kind), dimension(iim,jjm) ::
     *			work1_2d,
     *                  work2_2d

        real (kind = dbl_kind), dimension(iim,jjm,nlm) ::
     *			work1_3d

#ifndef no_pbl
      real (kind = dbl_kind), dimension(iim,jjm,nlm+3,nsdm) ::
     *			work_wrap=c0
#endif

	real (kind = dbl_kind), dimension(nspace,ncorners,iim,jjm,nlm) ::
     *			work1_c

        real (kind = dbl_kind), dimension(nspace,ncorners,iim,jjm) ::
     *			work2_c,
     *                  work3_c,
     *                  work4_c,
     *                  work5_c

        real (kind = dbl_kind), dimension(       ncorners,iim,jjm) ::
     *                  work6_c

	real (kind = dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *			work_div = 0,
     *			work_rel = 0,
     *			factor = 1.0_dbl_kind

	character*80 tag

	private ::
     *	  i, j, k, nt,
     *                  work1_2d,
     *                  work2_2d,
     *                  work1_3d,
     *                  work1_c,
     *                  work2_c,
     *                  work3_c,
     *                  work4_c,
     *                  work5_c,
     *                  work6_c,
     *                  work_div,
     *                  work_rel,
#ifndef no_pbl
     *			work_wrap,
#endif
     *			factor

#ifdef fluxcorrectedtransport
! FCT mass and mass flux work arrays
#ifndef no_pbl
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION( iim,jjm  ,nsdm) ::
     *   virt_ent ! pleak adjusted entrainment
#endif
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION( iim,jjm, nlm  ,nsdm) ::
     *   mss_low, mss_fct
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION( iim,jjm, nlm, ntprog ,nsdm) ::
     *   mss
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION(nwalls,iim,jjm, nlm,nttend,nsdm)::
     *   msh_f
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION(nwalls,iim,jjm, nlm  ,nsdm) ::
     *   mss_flux_horz, mss_flx1_horz, mss_anti_horz
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION(       iim,jjm, nlmp1,nsdm) ::
     *   mss_flux_vert, mss_edge
#ifndef no_pbl
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION(       iim,jjm, nlmp1,nsdm) ::
     *   mss_flxe_vert
#endif
REAL (KIND=dbl_kind),PRIVATE,DIMENSION(ncorners,iim,jjm, nlm,nsdm) ::
     *   mss_c

! FCT tracer work arrays
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION(       iim,jjm, nlm  ,nsdm) ::
     *   trc_high, trc_fct
      REAL (KIND=dbl_kind),PRIVATE,DIMENSION(       iim,jjm, nlmp1,nsdm) ::
     *   trc_high_edge

#endif

!----------------------------------------------------------------------------
! VARIABLE DESCRIPTION
!----------------------------------------------------------------------------
!       n{4,3,2}        : pointer of time level index {n+1,n,n-1} for prognostics
!       n{3,2,1}_f      : pointer of time level index {n, n-1, n-2} for tendencies
!       ntprog          : number of time level which prognostics must be saved
!       nttend          : number of time level which tendencies  must be saved
!	ntracer		: number of passive tracers to integrate in time
!       div             : divergence (im,jm,ntprog)
!	div_f		: time tendency of divergence (im,jm,nttend)
!	div_src		: rhs source to divergence field
!       eta             : absolute vorticity (im,jm,ntprog)
!       eta_f           : time tendency of eta (im,jm,nttend)
!	eta_src		: rhs source to vorticity field
!       tracer          : passive tracers advected by flow
!       tracer_f        : tendency of passive tracers
!       tracer_src      : rhs source to tracer field
!       psi             : steamfunction at step_count
!       chi             : velocity potential at step_count
!       w{3,2,1}        : weights for time integration
!       step_count      : counter index
!       hs              : surface height (m)
!	work		: local work arrays
!----------------------------------------------------------------------------

!----------------------------------------------------------------------------
! COMMENTS
!	all tendency terms included in the *_f arrays will be integrated
!	using Adams Bashforth 3rd Order integration. all tendency terms
!	included in the *_src arrays will be integrated Euler forward
!----------------------------------------------------------------------------

!**********************************************************************

	contains

!**********************************************************************


!======================================================================
! BEGINNING OF STEP_DT
!======================================================================
      subroutine step_dt(step_count)

	implicit none

        integer (kind=int_kind), intent(in) :: step_count

!--------------------------------------------------------------------------
! UPDATE PROGNOSTICS (psi and chi updated at the end)
!--------------------------------------------------------------------------

 	call timer_start('pressure')
!--------------------------------------------------------------------------
! UPDATE SURFACE PRESSURE
!--------------------------------------------------------------------------
#ifdef fluxcorrectedtransport
        call step_pressure
        call wrap ( max_level,iim,jjm,1,nsdm,ps(:,:,n4,:))
#ifndef no_pbl
        call wrap ( max_level,iim,jjm,1,nsdm,pb(:,:,n4,:))
#endif
#else
	do ksdm = 1, nsdm
          call step_surface_pressure
	enddo

!--------------------------------------------------------------------------
! UPDATE PBL TOP PRESSURE
!   dd.step_pbl_top_pressure has ksdm loop within, the ksdm loop here is
!      broken in two. This is to allow the pbl depth diffusion.
!--------------------------------------------------------------------------
#ifndef no_pbl
        call step_pbl_top_pressure
#endif
#endif

!-----------------------------------------------------------------------
! get pressure thickness at n4 (Arakawa's Pi)
!-----------------------------------------------------------------------
	do ksdm = 1, nsdm
          call get_pthick(
     *          ps         (:,:,n4,ksdm),
#ifndef no_pbl
     *          pb         (:,:,n4,ksdm),
#endif
     *          pthick     (:,:,:,n4,ksdm),
     *          invpthick  (:,:,:,n4,ksdm) )
	enddo
        call timer_stop('pressure')
!--------------------------------------------------------------------------
! END FIRST LOOP OVER KSDM
!  dd.This is now the end of the second ksdm loop since it was split
!--------------------------------------------------------------------------

#ifndef no_pbl
!--------------------------------------------------------------------------
! dsigdt and pleak were updated in step_pbl_top pressure
!	so wrap and average to cell walls
!--------------------------------------------------------------------------
	work_wrap(:,:,1:nlm+1,:) = dsigdt(:,:,:,:)
	work_wrap(:,:,nlm+2,:) = pleak
	work_wrap(:,:,nlm+3,:) = psb_smth
        call wrap ( max_level,iim,jjm,nlm+3,nsdm,work_wrap(:,:,:,:))
	dsigdt(:,:,:,:) = work_wrap(:,:,1:nlm+1,:)
	pleak(:,:,:) = work_wrap(:,:,nlm+2,:)
	psb_smth(:,:,:) = work_wrap(:,:,nlm+3,:)
#else
        call wrap ( max_level,iim,jjm,nlm+1,nsdm,dsigdt(:,:,:,:))
#endif

        do ksdm = 1,nsdm
          do k=1,nlm+1
            call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  dsigdt(:,:,k,ksdm),
     *                  dsigdt_c(:,:,:,k,ksdm))
          enddo

#ifndef no_pbl
        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  pleak(:,:,ksdm),
     *                  pleak_c(:,:,:,ksdm))

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  psb_smth(:,:,ksdm),
     *                  psb_smth_c(:,:,:,ksdm))
#endif
        enddo
#ifndef no_pbl
        call wrap_vertices(1,1,pleak_c(1,1,1,1))
#endif
        call wrap_vertices(1,nlmp1,dsigdt_c(1,1,1,1,1))

!--------------------------------------------------------------------------
! LOOP OVER EACH SUBDOMAIN
!--------------------------------------------------------------------------
	do ksdm = 1, nsdm

!--------------------------------------------------------------------------
! UPDATE MOMENTUM
!--------------------------------------------------------------------------
 	  call timer_start('momentum')
          call step_momentum
 	  call timer_stop('momentum')
	enddo	! do ksdm = 1,nsdm

!--------------------------------------------------------------------------
! UPDATE THETA
!--------------------------------------------------------------------------
 	  call timer_start('theta')
          call step_theta
 	  call timer_stop('theta')

!-----------------------------------------------------------------
! UPDATE TRACERS
!-----------------------------------------------------------------
 	  call timer_start('tracer')
          call step_tracer
 	  call timer_stop('tracer')
 
#ifdef eau_ng 
!-----------------------------------------------------------------
! update the cloud fraction:
!-----------------------------------------------------------------
      call eau_step_acld
      call eau_step_cldmclr
#endif


!----------------------------------------------------------------
! compute relative vorticity and divergence
!----------------------------------------------------------------
        call calculate_rel_and_div(
     *             nlm,
     *             u(:,:,:,:,:,n4,:),
     *             relative(:,:,:,:),
     *             div(:,:,:,n4,:)   )
        do k=1,nlm
        eta(:,:,k,n4,:) = f(:,:,:) + relative(:,:,k,:)
        enddo

!--------------------------------------------------------------------------
! UPDATE PSI and CHI 
!   relative and  div have the global mean subtracted.
!--------------------------------------------------------------------------
	call timer_start('update psi and chi')
        call update_psi_and_chi
	call timer_stop('update psi and chi')
        
!----------------------------------------------------------------
! compute velocity and kinetic energy
!----------------------------------------------------------------
        call calculate_u_and_ke(
     *            nlm                   ,
     *            psi   (    :,:,:,   :),
     *            chi   (    :,:,:,   :),
     *            u     (:,:,:,:,:,n4,:),
     *            ke    (    :,:,:,   :) )

	
!	if(mod(step_count,1080).eq.0) then
!	   tag = trim('pressure')
!	   call write_scalar_data(ps(:,:,n4,:),step_count,tag)
!	   tag = trim('theta')
!          call write_scalar_data(th(:,:,11,n4,:),step_count,tag)
!	   tag = trim('usfc')
!          call write_vector_data(u(:,:,:,:,nlm,n4,:),step_count,tag)
!          tag = trim('utrop')
!          call write_vector_data(u(:,:,:,:,11,n4,:),step_count,tag)
!          tag = trim('usrc_sfc')
!          call write_vector_data(u_src(:,:,:,:,nlm,:),step_count,tag)
!          tag = trim('usrc_trop')
!          call write_vector_data(u_src(:,:,:,:,11,:),step_count,tag)
!	endif

	end subroutine step_dt
!===========================================================================
! END OF STEP_DT
!===========================================================================



!===========================================================================
! BEGINNING OF UPDATE_PSI_AND_CHI
!===========================================================================
      subroutine update_psi_and_chi
      implicit none
      integer (kind=int_kind) :: k,size_2d
      real    (kind=dbl_kind) :: xmean,tot_area
      real    (kind=dbl_kind), dimension(iim,jjm,nsdm) :: one=1.0_dbl_kind
      real    (kind=dbl_kind), dimension(iim,jjm,nsdm) :: area_mask
      real    (kind=dbl_kind),allocatable :: 
     &   x_thick(:,:,:,:),rhs_thick(:,:,:,:)

      ALLOCATE (  x_thick(iim,jjm,2*nlm,nsdm),
     &          rhs_thick(iim,jjm,2*nlm,nsdm))
!-------------------------------------------------------------------------
! prepare for subtracting the global mean
!-------------------------------------------------------------------------
        size_2d = iim*jjm*nsdm
        area_mask = area*grid_center_mask
        tot_area = global_real_sum(area_mask(:,:,:),one(:,:,:),size_2d)
!-------------------------------------------------------------------------
! subtract the global mean
!-------------------------------------------------------------------------
      do k = 1,nlm
         xmean = global_real_sum (relative(:,:,k,:),area_mask(:,:,:),  
     &                                                 size_2d)/tot_area
         relative(:,:,k,:) = relative(:,:,k,:) - xmean

         xmean = global_real_sum (div(:,:,k,n4,:),area_mask(:,:,:), 
     &                                                 size_2d)/tot_area
         div(:,:,k,n4,:) = div(:,:,k,n4,:) - xmean
      enddo
!-------------------------------------------------------------------------
! make thick variables
!-------------------------------------------------------------------------
        x_thick(:,:,    1:  nlm,:) =        psi(:,:,:,:)
      rhs_thick(:,:,    1:  nlm,:) =   relative(:,:,:,:)
        x_thick(:,:,nlm+1:2*nlm,:) =        chi(:,:,:,:)
      rhs_thick(:,:,nlm+1:2*nlm,:) =        div(:,:,:,n4,:)
!-------------------------------------------------------------------------
! solve for psi and chi
!-------------------------------------------------------------------------
      call elliptic (ipad,iim,jjm,2*nlm,nsdm,nweights,  
     &               grid_center_mask(:,:,:),l_weights(:,:,:,:),   
     &               rhs_thick(:,:,:,:),x_thick(:,:,:,:))
!-------------------------------------------------------------------------
! copy back from thick variables
!-------------------------------------------------------------------------
      psi(:,:,:,:) = x_thick(:,:,    1:  nlm,:)
      chi(:,:,:,:) = x_thick(:,:,nlm+1:2*nlm,:)

      DEALLOCATE (x_thick,rhs_thick)

      end subroutine update_psi_and_chi
!======================================================================
! END OF UPDATE_PSI_AND_CHI
!======================================================================


!=======================================================================
! START STEP_SURFACE_PRESSURE
!=======================================================================
        subroutine step_surface_pressure

!-----------------------------------------------------------------------------
! PURPOSE : find tendencies of the surface pressure and integrate it in time.
!-----------------------------------------------------------------------------

	implicit none

!-----------------------------------------------------------------------------
! INITIALIZE
!-----------------------------------------------------------------------------
        work1_2d(:,:) = c0

!-----------------------------------------------------------------------------
! DETERMINE TIME TENDENCY OF SURFACE PRESSURE
!-----------------------------------------------------------------------------
        do k=1,nlm
          work1_2d = work1_2d - div_mass_flux(:,:,k,n3_f,ksdm)*dsig(k)
        enddo
        ps_f(:,:,n3_f,ksdm) = work1_2d(:,:)

!-----------------------------------------------------------------------------
! INCREMENT SURFACE PRESSURE VIA AB3
!-----------------------------------------------------------------------------
        ps(:,:,n4,ksdm) = ps(:,:,n3,ksdm) 
     *			      + dt*(w3*ps_f(:,:,n3_f,ksdm) +
     *                              w2*ps_f(:,:,n2_f,ksdm) +
     *                              w1*ps_f(:,:,n1_f,ksdm)   )

        end subroutine step_surface_pressure
!=======================================================================
! END OF STEP_SURFACE_PRESSURE
!=======================================================================


#ifndef no_pbl
!=======================================================================
! START STEP_PBL_TOP_PRESSURE
!=======================================================================
        subroutine step_pbl_top_pressure

!-----------------------------------------------------------------------------
! PURPOSE : find tendencies of the pbl top pressure and integrate it in time.
!-----------------------------------------------------------------------------

	implicit none

#ifdef generalized_sigma

#ifdef pbl_depth_limiter
        real (kind=dbl_kind), parameter ::
     *                  pbl_min      = 1.0e3,
     *                  pbl_max      = 1.6e4
#else
        real (kind=dbl_kind), parameter ::
     *                  pbl_min      = 8.0e3,
     *                  pbl_max      = 8.0e3
#endif

        integer (kind=int_kind), dimension(iim,jjm) ::
     *                   min_mask=0,
     *                   max_mask=0

        real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *                  pbl_depth = c0

        integer (kind=int_kind) :: norder = 2   ! times del^2 operator applied
        real (kind=dbl_kind) :: fac_pb

      do ksdm = 1,nsdm
!----------------------------------------------------------------------
! sum mass flux divergence from top down to pbl top
!----------------------------------------------------------------------
	pb_f(:,:,n3_f,ksdm) = c0
        do k=1,nstrat+ntrop
           pb_f(:,:,n3_f,ksdm) =    pb_f(:,:,n3_f,ksdm) 
     *                         - div_mass_flux(:,:,k,n3_f,ksdm)*dsig(k)
        enddo

!----------------------------------------------------------------------
! add in the part of the vertical velocity @ the pbl which is to
! be integrated AB3, in general
!----------------------------------------------------------------------
        pb_dsigdt_f(:,:,n3_f,ksdm) = - dsigdt(:,:,nstrat+ntrop+1,ksdm)

!----------------------------------------------------------------------
! increment surface pressure via AB3 or Leap Frog
! this is a provisional value and will be modified if the pbl
! depth is too thin to too thick
!----------------------------------------------------------------------
        pb(:,:,n4,ksdm) = pb(:,:,n3,ksdm) 
     *    + dt*(w3*(pb_f(:,:,n3_f,ksdm) + pb_dsigdt_f(:,:,n3_f,ksdm)) +
     *          w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *          w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )

!---------------------------------------------------------------------
! compute the depth of the pbl
!---------------------------------------------------------------------
        pbl_depth(:,:,ksdm) = ps(:,:,n4,ksdm) - pb(:,:,n4,ksdm)
      enddo

      call wrap ( max_level,
     *             iim,
     *             jjm,
     *             1,
     *             nsdm,
     *             pbl_depth )
      call wrap ( max_level,
     *             iim,
     *             jjm,
     *             1,
     *             nsdm,
     *             pb(:,:,n4,:) )
      call scalar_laplacian_to_the_n(
     *                    norder,
     *                    1,
     *                    pb(:,:,n4,:),
     *                    psb_lapl   )
      fac_pb = (-1)**(norder)* diff_coeff_pb
      psb_smth =  fac_pb * psb_lapl / grav
      pbl_depth(:,:,:) = pbl_depth(:,:,:) + dt*grav*psb_smth(:,:,:)

!---------------------------------------------------------------------
! find mask
!---------------------------------------------------------------------
      do ksdm = 1,nsdm
        min_mask = 0
        max_mask = 0
        where(pbl_depth(:,:,ksdm).lt.pbl_min) min_mask = 1
        where(pbl_depth(:,:,ksdm).gt.pbl_max) max_mask = 1
	
!---------------------------------------------------------------------
! find leak velocity where pbl is less than minimum
!---------------------------------------------------------------------
        where(min_mask(:,:).eq.1)
          pleak(:,:,ksdm) =
     *     + ((ps(:,:,n4,ksdm) - pbl_min) - pb(:,:,n3,ksdm))*invdt
     *     - (w3*(pb_f(:,:,n3_f,ksdm)                             ) +
     *        w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *        w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )
     *     +  w3*grav*(entrainment(:,:,ksdm) - cumulusmf(:,:,ksdm)
     *     +  psb_smth(:,:,ksdm) )
	  pleak(:,:,ksdm) = -pleak(:,:,ksdm)/(grav*w3)
        endwhere

!---------------------------------------------------------------------
! find leak velocity where pbl is greater than maximum
!---------------------------------------------------------------------
        where(max_mask(:,:).eq.1)
          pleak(:,:,ksdm) =
     *     + ((ps(:,:,n4,ksdm) - pbl_max) - pb(:,:,n3,ksdm))*invdt
     *     - (w3*(pb_f(:,:,n3_f,ksdm)                             ) +
     *        w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *        w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )
     *     +  w3*grav*(entrainment(:,:,ksdm) - cumulusmf(:,:,ksdm)
     *     +  psb_smth(:,:,ksdm) )
          pleak(:,:,ksdm) = -pleak(:,:,ksdm)/(grav*w3)
        endwhere

!----------------------------------------------------------------------
! compute new vertical velocity based on new leak
!----------------------------------------------------------------------
        work1_2d(:,:) = psb_smth(:,:,ksdm) + pleak(:,:,ksdm)
	call get_dsigdt(
     *           entrainment(:,:,ksdm),
     *           cumulusmf(:,:,ksdm),
     *           work1_2d(:,:),
     *           div_mass_flux(:,:,:,n3_f,ksdm),
     *           dsigdt(:,:,:,ksdm) )
	
!----------------------------------------------------------------------
! compute new pb_dsigdt_f based on new leak
!----------------------------------------------------------------------
        pb_dsigdt_f(:,:,n3_f,ksdm) = - dsigdt(:,:,nstrat+ntrop+1,ksdm)

!----------------------------------------------------------------------
! compute new pbl top based on new leak
!----------------------------------------------------------------------
        pb(:,:,n4,ksdm) = pb(:,:,n3,ksdm)
     *       + dt*(w3*(pb_f(:,:,n3_f,ksdm) + pb_dsigdt_f(:,:,n3_f,ksdm)) +
     *             w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *             w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm)) )
      enddo
#endif

        end subroutine step_pbl_top_pressure
!=======================================================================
! END STEP_PBL_TOP_PRESSURE
!=======================================================================
#endif

#ifdef fluxcorrectedtransport
!=======================================================================
! START STEP_PRESSURE
!=======================================================================
        subroutine step_pressure

!-----------------------------------------------------------------------------
! PURPOSE : find tendencies of the pbl top pressure and integrate it in time.
!-----------------------------------------------------------------------------

	implicit none

#ifndef no_pbl
#ifdef pbl_depth_limiter
        real (kind=dbl_kind), parameter ::
     *                  pbl_min      = 1.0e3,
     *                  pbl_max      = 1.6e4
#else
        real (kind=dbl_kind), parameter ::
     *                  pbl_min      = 8.0e3,
     *                  pbl_max      = 8.0e3
#endif

        real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *                  pbl_depth = c0
        integer (kind=int_kind) :: norder = 2   ! times del^2 operator applied
        real (kind=dbl_kind) :: fac_pb
#endif
        real (kind=dbl_kind), dimension(iim,jjm,nlm) ::
     *                  zero3d = c0
        real (kind=dbl_kind), dimension(iim,jjm) ::
     *                  zero2d = c0
        integer (kind=int_kind), dimension(iim,jjm) ::
     *                   min_mask=0,
     *                   max_mask=0


        integer (kind=dbl_kind) :: nsd

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  THIS IS THE BEGINNING OF THE FCT CODE
!  the following assumes that ps and pb are wrapped, and only one PBL Layer
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!-----------------------------------------------------------------------
!  set mass
!-----------------------------------------------------------------------
      DO k = 1,nlm
         mss(:,:,k,n3,:) = (plgl2  (:,:,k+1,:)-plgl2  (:,:,k,:))/        
     *                     (sigl2(    k+1  )-sigl2(    k  ))
      ENDDO
!-----------------------------------------------------------------------
!  set mass at cell corners
!-----------------------------------------------------------------------
      mss_c(:,:,:,:,:) = 0.0_dbl_kind
      DO nsd = 1,nsdm
         DO k = 1,nlm
            CALL average_to_cell_corners (ipad,iim,jjm,ndata,ncorners,      
     *                                      s_weights(:,:,:,:,     nsd),    
     *                                      mss      (    :,:,k,n3,nsd),    
     *                                      mss_c    (  :,:,:,k,   nsd))
         ENDDO
      ENDDO
        call wrap_vertices(1,nlm,mss_c(1,1,1,1,1))
!-----------------------------------------------------------------------
!  set mass at layer edges
!-----------------------------------------------------------------------
      mss_edge(:,:,    1,:) = mss(:,:,  1,n3,:)
      DO k = 2,nlm
         mss_edge(:,:,k,:) = (mss(:,:,k-1,n3,:)*(sigl2(k  )-sigl2(k-1))+  
     *                        mss(:,:,k  ,n3,:)*(sigl2(k+1)-sigl2(k  )))/
     *                                              (sigl2(k+1)-sigl2(k-1))
      ENDDO     
      mss_edge(:,:,nlmp1,:) = mss(:,:,nlm,n3,:)

!-----------------------------------------------------------------------
!  set flux-corrected mass
!-----------------------------------------------------------------------
#ifdef no_pbl
      CALL set_mass_fct (                  mss          (    :,:,:,:,:),
     *                                     mss_c        (  :,:,:,:,  :), 
     *                                     mss_edge     (    :,:,:,  :), 
     *                                     u            (:,:,:,:,:,:,:), 
     *                                     dsigdt       (    :,:,:,  :), 
     *                                     mss_low      (    :,:,:,  :), 
     *                                     mss_flux_horz(  :,:,:,:,  :), 
     *                                     mss_flux_vert(    :,:,:,  :),
     *                                     mss_fct      (    :,:,:,  :))
#else
      CALL set_mass_fct (                  mss          (    :,:,:,:,:),
     *                                     mss_c        (  :,:,:,:,  :), 
     *                                     mss_edge     (    :,:,:,  :), 
     *                                     u            (:,:,:,:,:,:,:), 
     *                                     dsigdtu      (    :,:,:,  :), 
     *                                     dsigdte      (    :,:,:,  :), 
     *                                     entrainment  (    :,:,    :), 
     *                                     cumulusmf    (    :,:,    :), 
     *                                     mss_low      (    :,:,:,  :), 
     *                                     mss_flux_horz(  :,:,:,:,  :), 
     *                                     mss_flux_vert(    :,:,:,  :),
     *                                     mss_flxe_vert(    :,:,:,  :),
     *                                     mss_fct      (    :,:,:,  :))
#endif

#ifdef no_pbl
!-----------------------------------------------------------------------
!  set ps
!-----------------------------------------------------------------------
      ps(:,:,n4,:) = p_top
      do k = 1,nlm
         ps(:,:,n4,:) = ps(:,:,n4,:) + 
     *                  mss_fct(:,:,k,:)*(sigl2(k+1)-sigl2(k))
      enddo
#else
!-----------------------------------------------------------------------
!  set ps and pb
!-----------------------------------------------------------------------
      pb(:,:,n4,:) = p_top
      do k = 1,lpbl
         pb(:,:,n4,:) = pb(:,:,n4,:) + 
     *                  mss_fct(:,:,k,:)*(sigl2(k+1)-sigl2(k))
      enddo
      ps(:,:,n4,:) = pb(:,:,n4,:) +
     *               mss_fct(:,:,nlm  ,:)*(sigl2(nlmp1)-sigl2(nlm))
      call wrap ( max_level,
     *             iim,
     *             jjm,
     *             1,
     *             nsdm,
     *             pb(:,:,n4,:) )
      call scalar_laplacian_to_the_n(
     *                    norder,
     *                    1,
     *                    pb(:,:,n4,:),
     *                    psb_lapl   )
      fac_pb = (-1)**(norder)* diff_coeff_pb
      psb_smth =  fac_pb * psb_lapl / grav
      pbl_depth(:,:,:) = (ps(:,:,n4,:)-pb(:,:,n4,:))+dt*grav*psb_smth(:,:,:)

! impose pbl limits
      DO nsd = 1,nsdm
        pleak(:,:,nsd) = c0
        min_mask = 0
        max_mask = 0
        where(pbl_depth(:,:,nsd).lt.pbl_min) min_mask = 1
        where(pbl_depth(:,:,nsd).gt.pbl_max) max_mask = 1
        where(min_mask(:,:).eq.1)
                  pleak(:,:,nsd) =
     *     -(pbl_depth(:,:,nsd) - pbl_min)*invdt*gravi
        endwhere
        where(max_mask(:,:).eq.1)
          pleak(:,:,nsd) =
     *     -(pbl_depth(:,:,nsd) - pbl_max)*invdt*gravi
        endwhere
        virt_ent(:,:,nsd) = entrainment(:,:,nsd) + pleak(:,:,nsd) +
     *                      psb_smth(:,:,nsd)
	call get_dsigdt(
     *           virt_ent(:,:,nsd),
     *           cumulusmf(:,:,nsd),
     *           zero2d(:,:),
     *           zero3d(:,:,:),
     *           dsigdte(:,:,:,nsd) )
      enddo
!-----------------------------------------------------------------------
!  set flux-corrected mass including pleak
!-----------------------------------------------------------------------
      CALL set_mass_fct (                  mss          (    :,:,:,:,:),
     *                                     mss_c        (  :,:,:,:,  :), 
     *                                     mss_edge     (    :,:,:,  :), 
     *                                     u            (:,:,:,:,:,:,:), 
     *                                     dsigdtu      (    :,:,:,  :), 
     *                                     dsigdte      (    :,:,:,  :), 
     *                                     virt_ent     (    :,:,    :), 
     *                                     cumulusmf    (    :,:,    :), 
     *                                     mss_low      (    :,:,:,  :), 
     *                                     mss_flux_horz(  :,:,:,:,  :), 
     *                                     mss_flux_vert(    :,:,:,  :),
     *                                     mss_flxe_vert(    :,:,:,  :),
     *                                     mss_fct      (    :,:,:,  :))
      dsigdt = dsigdtu + dsigdte
     
      mss(:,:,:,n4,:) = mss_fct(:,:,:,:)
!-----------------------------------------------------------------------
!  set ps and pb
!-----------------------------------------------------------------------
      pb(:,:,n4,:) = p_top
      do k = 1,lpbl
         pb(:,:,n4,:) = pb(:,:,n4,:) + 
     *                  mss(:,:,k,n4,:)*(sigl2(k+1)-sigl2(k))
      enddo
      ps(:,:,n4,:) = pb(:,:,n4,:) +
     *               mss(:,:,nlm  ,n4,:)*(sigl2(nlmp1)-sigl2(nlm))
#endif

        end subroutine step_pressure
!=======================================================================
! END STEP_PRESSURE
!=======================================================================
#endif


!===========================================================================
!  START OF STEP_THETA
!===========================================================================
        subroutine step_theta

!---------------------------------------------------------------------------
! PURPOSE
!  compute the time tendency of potential temperature and integrate
!  potential temperature in time
!---------------------------------------------------------------------------

        implicit none

!#ifdef fluxcorrectedtransport

!      th_f(:,:,:,n3_f,:) = th(:,:,:,n3,:)

!      trc_high(:,:,:,:) = w3*th_f(:,:,:,n3_f,:)+
!     *                    w2*th_f(:,:,:,n2_f,:)+
!     *                    w1*th_f(:,:,:,n1_f,:)

!      trc_high_edge(:,:,:,:) = 0.0_dbl_kind
!      DO k = 2,nlmp1-1
!         trc_high_edge(:,:,k,:) = 
!     *              0.5_dbl_kind*(trc_high(:,:,k-1,:)+trc_high(:,:,k,:))
!      ENDDO

!#ifdef no_pbl
!      CALL set_tracer_fct (            th           (  :,:,:,     n3,:),  
!     *                                 trc_high     (  :,:,:,        :),  
!     *                                 trc_high_edge(  :,:,:,        :),  
!     *                                 mss          (  :,:,:,      :,:),  
!     *                                 mss_low      (  :,:,:,        :),  
!     *                                 mss_flux_horz(:,:,:,:,        :),  
!     *                                 mss_flux_vert(  :,:,:,        :),  
!     *                                 trc_fct      (  :,:,:,        :))
!#else
!      CALL set_tracer_fct (            th           (  :,:,:,     n3,:),  
!     *                                 trc_high     (  :,:,:,        :),  
!     *                                 trc_high_edge(  :,:,:,        :),  
!     *                                 mss          (  :,:,:,      :,:),  
!     *                                 mss_low      (  :,:,:,        :),  
!     *                                 virt_ent     (    :,:,        :),  
!     *                                 cumulusmf    (    :,:,        :),  
!     *                                 mss_flux_horz(:,:,:,:,        :),  
!     *                                 mss_flux_vert(  :,:,:,        :),  
!     *                                 mss_flxe_vert(  :,:,:,        :),  
!     *                                 trc_fct      (  :,:,:,        :))
!#endif

!      th(:,:,:,n4,:) = trc_fct(:,:,:,:)
      

!        do k = 1,nlm
!---------------------------------------------------------------------------
! Add the physical sources and sinks
!---------------------------------------------------------------------------
!          th(:,:,k,n4,:) =  th(:,:,k,n4,:)
!     *                 + dt*th_src(:,:,k,:)* invpthick(:,:,k,n4,:)

!	enddo   ! do k = 1,nlm
#ifdef fluxcorrectedtransport
        do ksdm = 1,nsdm

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! compute tendency due to vertical motion
!---------------------------------------------------------------------------
#ifdef no_pbl
        call vert_flux_divergence(
     *          thl2        (:,:,:,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          work1_3d    (:,:,:)     )

        th_f(:,:,:,n3_f,ksdm) =  - work1_3d(:,:,:)
#else
        work1_2d(:,:) = c0
        call vert_flux_divergence(
     *          th          (:,:,:,n3,ksdm),
     *          thl2        (:,:,:,ksdm),
     *          dsigdtu     (:,:,:,ksdm),
     *          work1_2d    (:,:),
     *          work1_2d    (:,:),
     *          work1_2d    (:,:),
     *          work1_3d    (:,:,:)     )

        th_f(:,:,:,n3_f,ksdm) =  - work1_3d(:,:,:)
        work1_3d = c0
!---------------------------------------------------------------------------
! compute tendency due to vertical motion
!---------------------------------------------------------------------------
        work1_2d(:,:) = psb_smth(:,:,ksdm) + pleak(:,:,ksdm)
        call vert_flux_divergence(
     *          th          (:,:,:,n3,ksdm),
     *          thl2        (:,:,:,ksdm),
     *          dsigdte     (:,:,:,ksdm),
     *          entrainment (:,:,ksdm),
     *          cumulusmf   (:,:,ksdm),
     *          work1_2d    (:,:),
     *          work1_3d    (:,:,:)     )
#endif

#else
        do ksdm = 1,nsdm

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! compute tendency due to vertical motion
!---------------------------------------------------------------------------
#ifdef no_pbl
        call vert_flux_divergence(
     *          thl2        (:,:,:,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          work1_3d    (:,:,:)     )
#else
        work1_2d(:,:) = psb_smth(:,:,ksdm) + pleak(:,:,ksdm)
        call vert_flux_divergence(
     *          th          (:,:,:,n3,ksdm),
     *          thl2        (:,:,:,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          entrainment (:,:,ksdm),
     *          cumulusmf   (:,:,ksdm),
     *          work1_2d    (:,:),
     *          work1_3d    (:,:,:)     )
#endif

        th_f(:,:,:,n3_f,ksdm) =  - work1_3d(:,:,:)
        work1_3d = c0
#endif

!---------------------------------------------------------------------------
! begin loop over vertical level k
!---------------------------------------------------------------------------
        do k=1,nlm

!---------------------------------------------------------------------------
! compute flux of theta as theta@corner * mass flux@corner
!---------------------------------------------------------------------------
          work2_c(1,:,:,:) = th_c(:,:,:,k,ksdm) * 
     *                       mass_flux_c(1,:,:,:,k,n3_f,ksdm)
          work2_c(2,:,:,:) = th_c(:,:,:,k,ksdm) *
     *                       mass_flux_c(2,:,:,:,k,n3_f,ksdm)

!---------------------------------------------------------------------------
! take the divergence of the th_c weighted mass flux
!---------------------------------------------------------------------------
        call div_of_a_vector(                                             
     *          ipad,      
     *          iim,          
     *          jjm,               
     *          ndata,
     *          nspace,       
     *          ncorners,     
     *          area_inv(:,:,ksdm),
     *          v_weights(:,:,:,:,:,ksdm),                                
     *          work2_c(:,:,:,:), 
     *          work1_2d(:,:) )

!---------------------------------------------------------------------------
! store in theta_f
!---------------------------------------------------------------------------
          th_f(:,:,k,n3_f,ksdm) = th_f(:,:,k,n3_f,ksdm)
     *	                                   - work1_2d(:,:)

!---------------------------------------------------------------------------
! compute theta at new time level
!---------------------------------------------------------------------------
          th(:,:,k,n4,ksdm) = 
     *                       th(:,:,k,n3,ksdm)*pthick(:,:,k,n3,ksdm) 
     *                     + dt*(w3*th_f(:,:,k,n3_f,ksdm) +
     *                           w2*th_f(:,:,k,n2_f,ksdm) +
     *                           w1*th_f(:,:,k,n1_f,ksdm)   )
     *                     + dt*(th_src(:,:,k,ksdm)-work1_3d(:,:,k))


!---------------------------------------------------------------------------
! divide by mass to get real tracer values (kg/kg)
!---------------------------------------------------------------------------
          th(:,:,k,n4,ksdm) =    th(:,:,k,n4,ksdm) 
     *                                 * invpthick(:,:,k,n4,ksdm)

	enddo   ! do k = 1,nlm
        enddo   ! do ksdm = 1,nsdm
!#endif

	end subroutine step_theta
!===========================================================================
!  END OF STEP_THETA
!===========================================================================


!===========================================================================
!  START OF STEP_TRACER
!===========================================================================
        subroutine step_tracer

!---------------------------------------------------------------------------
! PURPOSE
!  compute the time tendency of tracers and integrate them in time
!---------------------------------------------------------------------------

        implicit none

#ifdef fluxcorrectedtransport

!---------------------------------------------------------------------------
! loop over tracers
!---------------------------------------------------------------------------
	do nt = 1,ntracer

      tracer_f(:,:,:,nt,n3_f,:) = tracer(:,:,:,nt,n3,:)

      trc_high(:,:,:,:) = w3*tracer_f(:,:,:,nt,n3_f,:)+
     *                    w2*tracer_f(:,:,:,nt,n2_f,:)+
     *                    w1*tracer_f(:,:,:,nt,n1_f,:)

      trc_high_edge(:,:,:,:) = 0.0_dbl_kind
      DO k = 2,nlmp1-1
         trc_high_edge(:,:,k,:) = 
     *              0.5_dbl_kind*(trc_high(:,:,k-1,:)+trc_high(:,:,k,:))
      ENDDO

#ifdef no_pbl
      CALL set_tracer_fct (            tracer       (  :,:,:,nt  ,n3,:),
     *                                 trc_high     (  :,:,:,        :),
     *                                 trc_high_edge(  :,:,:,        :),
     *                                 mss          (  :,:,:,      :,:),
     *                                 mss_low      (  :,:,:,        :),  
     *                                 mss_flux_horz(:,:,:,:,        :),
     *                                 mss_flux_vert(  :,:,:,        :),
     *                                 trc_fct      (  :,:,:,        :))
#else
      CALL set_tracer_fct (            tracer       (  :,:,:,nt  ,n3,:),
     *                                 trc_high     (  :,:,:,        :),
     *                                 trc_high_edge(  :,:,:,        :),
     *                                 mss          (  :,:,:,      :,:),
     *                                 mss_low      (  :,:,:,        :),  
     *                                 virt_ent     (    :,:,        :),
     *                                 cumulusmf    (    :,:,        :),
     *                                 mss_flux_horz(:,:,:,:,        :),
     *                                 mss_flux_vert(  :,:,:,        :),
     *                                 mss_flxe_vert(  :,:,:,        :),  
     *                                 trc_fct      (  :,:,:,        :))
#endif

      tracer(:,:,:,nt,n4,:) = trc_fct(:,:,:,:)

        do k = 1,nlm
!---------------------------------------------------------------------------
! Add the physical sources and sinks
!---------------------------------------------------------------------------
          tracer(:,:,k,nt,n4,:) = 
     *                   tracer(:,:,k,nt,n4,:)
     *                 + dt*tracer_src(:,:,k,nt,:)* invpthick(:,:,k,n4,:)

	enddo   ! do k = 1,nlm
	enddo	! do nt=1,ntracer
#else
        do ksdm = 1,nsdm
!---------------------------------------------------------------------------
! loop over tracers
!---------------------------------------------------------------------------
	do nt = 1,ntracer

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! compute tendency due to vertical motion
!---------------------------------------------------------------------------
#ifdef no_pbl
        call vert_flux_divergence(
     *          tracerl2    (:,:,:,nt,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          work1_3d    (:,:,:)     )
#else
        work1_2d(:,:) = psb_smth(:,:,ksdm) + pleak(:,:,ksdm)
        call vert_flux_divergence(
     *          tracer      (:,:,:,nt,n3,ksdm),
     *          tracerl2    (:,:,:,nt,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          entrainment (:,:,ksdm),
     *          cumulusmf   (:,:,ksdm),
     *          work1_2d    (:,:),
     *          work1_3d    (:,:,:)     )
#endif

        tracer_f(:,:,:,nt,n3_f,ksdm) =  - work1_3d(:,:,:)

!---------------------------------------------------------------------------
! begin loop over vertical level k
!---------------------------------------------------------------------------
        do k=1,nlm

!---------------------------------------------------------------------------
! compute flux of tracer as tracer@corner * mass flux@corner
!---------------------------------------------------------------------------
          work2_c(1,:,:,:) = tracer_c(:,:,:,k,nt,ksdm) * 
     *                       mass_flux_c(1,:,:,:,k,n3_f,ksdm)
          work2_c(2,:,:,:) = tracer_c(:,:,:,k,nt,ksdm) *
     *                       mass_flux_c(2,:,:,:,k,n3_f,ksdm)

!---------------------------------------------------------------------------
! take the divergence of the th_c weighted mass flux
!---------------------------------------------------------------------------
        call div_of_a_vector(                                             
     *          ipad,      
     *          iim,          
     *          jjm,               
     *          ndata,
     *          nspace,       
     *          ncorners,     
     *          area_inv(:,:,ksdm),
     *          v_weights(:,:,:,:,:,ksdm),                                
     *          work2_c(:,:,:,:), 
     *          work1_2d(:,:) )

!---------------------------------------------------------------------------
! store in tracer_f
!---------------------------------------------------------------------------
          tracer_f(:,:,k,nt,n3_f,ksdm) = tracer_f(:,:,k,nt,n3_f,ksdm)
     *	                                   - work1_2d(:,:)

!---------------------------------------------------------------------------
! compute tracer at new time level
!---------------------------------------------------------------------------
          tracer(:,:,k,nt,n4,ksdm) = 
     *                       tracer(:,:,k,nt,n3,ksdm)*pthick(:,:,k,n3,ksdm) 
     *                     + dt*(w3*tracer_f(:,:,k,nt,n3_f,ksdm) +
     *                           w2*tracer_f(:,:,k,nt,n2_f,ksdm) +
     *                           w1*tracer_f(:,:,k,nt,n1_f,ksdm)   )
     *                     + dt*tracer_src(:,:,k,nt,ksdm)


!---------------------------------------------------------------------------
! divide by mass to get real tracer values (kg/kg)
!---------------------------------------------------------------------------
          tracer(:,:,k,nt,n4,ksdm) =    tracer(:,:,k,nt,n4,ksdm) 
     *                                 * invpthick(:,:,k,n4,ksdm)

	enddo   ! do k = 1,nlm
	enddo	! do nt=1,ntracer
        enddo   ! do ksdm = 1,nsdm
#endif

        end subroutine step_tracer
!===========================================================================
!  END OF STEP_TRACER
!===========================================================================

!======================================================================
! BEGINNING OF STEP_MOMENTUM
!======================================================================
        subroutine step_momentum

!----------------------------------------------------------------------------
! PURPOSE : find tendencies for the momentum equation
!   and integrated it in time
!----------------------------------------------------------------------------

        implicit none

#ifdef no_pbl
!----------------------------------------------------------------------------
! get contribution due to vertical advection
!----------------------------------------------------------------------------
        call vert_flux_divergence_mom(
     *          ncorners                         ,
     *          u             (1,:,:,:,:,n3,ksdm),
     *          dsigdt_c      (  :,:,:,:,   ksdm),
     *          invpthick_c   (  :,:,:,:,   ksdm),
     *          work1_c       (1,:,:,:,:        )   )

        call vert_flux_divergence_mom(
     *          ncorners                         ,
     *          u             (2,:,:,:,:,n3,ksdm),
     *          dsigdt_c      (  :,:,:,:,   ksdm),
     *          invpthick_c   (  :,:,:,:,   ksdm),
     *          work1_c       (2,:,:,:,:        )   )
#else
!----------------------------------------------------------------------------
! initialize
!----------------------------------------------------------------------------
        work6_c = psb_smth_c(:,:,:,ksdm) + pleak_c(:,:,:,ksdm)

!----------------------------------------------------------------------------
! get contribution due to vertical advection
!----------------------------------------------------------------------------
        call vert_flux_divergence_mom(
     *          ncorners                         ,
     *          u             (1,:,:,:,:,n3,ksdm),
     *          dsigdt_c      (  :,:,:,:,   ksdm),
     *          invpthick_c   (  :,:,:,:,   ksdm),
     *          entrainment_c (  :,:,:,     ksdm),
     *          cumulusmf_c   (  :,:,:,     ksdm),
     *          work6_c       (  :,:,:),
     *          work1_c       (1,:,:,:,:        )   )

        call vert_flux_divergence_mom(
     *          ncorners                         ,
     *          u             (2,:,:,:,:,n3,ksdm),
     *          dsigdt_c      (  :,:,:,:,   ksdm),
     *          invpthick_c   (  :,:,:,:,   ksdm),
     *          entrainment_c (  :,:,:,     ksdm),
     *          cumulusmf_c   (  :,:,:,     ksdm),
     *          work6_c       (  :,:,:),
     *          work1_c       (2,:,:,:,:        )   )
#endif

!----------------------------------------------------------------------------
! get all the tendencies due to horizontal motions
!----------------------------------------------------------------------------
        do k = 1,nlm

!-----------------------------------------------------------------------------
! average vorticity to cell walls
!-----------------------------------------------------------------------------
          call average_pv_to_cell_corners(                                    
     *          ipad,                                                        
     *          iim,                                                         
     *          jjm,                                                         
     *          ncorners,                                                    
     *          nspace,                                                      
     *          ndata,                                                       
     *          eta(:,:,k,n3,ksdm),                                                  
     *          pthick(:,:,k,n3,ksdm),                                              
     *          invpthick(:,:,k,n3,ksdm),                                             
     *          work6_c(:,:,:) )

!----------------------------------------------------------------------------
! compute k cross v
!-----------------------------------------------------------------------------
        call k_cross_v(                                                      
     *          iim,
     *          jjm,
     *          nspace,
     *          ncorners,
     *          u(:,:,:,:,k,n3,ksdm),
     *          work2_c(:,:,:,:) )

        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        do ispace=1,nspace
          work2_c(ispace,icorner,i,j) =   work6_c(       icorner,i,j)        
     *                                  * work2_c(ispace,icorner,i,j)
        enddo
        enddo
	enddo
	enddo

!----------------------------------------------------------------------------
! get contribution due to horizontal pressure gradient force (also KE)
!----------------------------------------------------------------------------
        work2_2d = phi(:,:,k,ksdm) + ke(:,:,k,ksdm)
        call gradient(                                                      
     *          ipad,                                                       
     *          iim,                                                        
     *          jjm,                                                        
     *          ndata,                                                      
     *          nspace,                                                     
     *          ncorners,                                                   
     *          corner_area_inv(:,:,:,ksdm),                                
     *          v_weights(:,:,:,:,:,ksdm),                                  
     *          work2_2d(:,:),                                                      
     *          work3_c(:,:,:,:) )

        call gradient(
     *          ipad,    
     *          iim,           
     *          jjm,
     *          ndata,   
     *          nspace,                
     *          ncorners,
     *          corner_area_inv(:,:,:,ksdm),
     *          v_weights(:,:,:,:,:,ksdm),
     *          pthick(:,:,k,n3,ksdm),                                             
     *          work4_c(:,:,:,:) )
	work4_c(1,:,:,:) = alphasig_c(:,:,:,k,ksdm) * work4_c(1,:,:,:)
        work4_c(2,:,:,:) = alphasig_c(:,:,:,k,ksdm) * work4_c(2,:,:,:)

!----------------------------------------------------------------------------
! get contribution due to horizontal pressure gradient force from PBL 
!----------------------------------------------------------------------------

#ifndef no_stratosphere
        if(k.le.nstrat) work4_c(:,:,:,:) = c0
#endif

#ifdef no_pbl
	      work5_c = c0
#else
         if(k.ge.nstrat+ntrop+1.and.k.le.nlm) then
             work2_2d(:,:) = pb(:,:,n3,ksdm)
             call gradient(
     *          ipad,
     *          iim,
     *          jjm,
     *          ndata,
     *          nspace,  
     *          ncorners,
     *          corner_area_inv(:,:,:,ksdm),
     *          v_weights(:,:,:,:,:,ksdm),
     *          work2_2d(:,:),
     *          work5_c(:,:,:,:) )
              work5_c(1,:,:,:) = alpha_c(:,:,:,k,ksdm) * work5_c(1,:,:,:)
              work5_c(2,:,:,:) = alpha_c(:,:,:,k,ksdm) * work5_c(2,:,:,:)
	  else
	      work5_c = c0
          endif
#endif

!----------------------------------------------------------------------------
! sum all the tendencies
!----------------------------------------------------------------------------
        u_f(:,:,:,:,k,n3_f,ksdm) = + work1_c(:,:,:,:,k)
     *                             - work2_c(:,:,:,:  )
     *                             - work3_c(:,:,:,:  )
     *                             - work4_c(:,:,:,:  )
     *                             - work5_c(:,:,:,:  )

!----------------------------------------------------------------------------
! step momentum in time
!----------------------------------------------------------------------------
        u(:,:,:,:,k,n4,ksdm) =  u(:,:,:,:,k,n3,ksdm) 
     *                          + dt * (w3*u_f(:,:,:,:,k,n3_f,ksdm) +
     *                                  w2*u_f(:,:,:,:,k,n2_f,ksdm) +
     *                                  w1*u_f(:,:,:,:,k,n1_f,ksdm)  )
     *                          + dt *   u_src(:,:,:,:,k,     ksdm)


        enddo   ! do k=1,nlm

        end subroutine step_momentum
!===========================================================================
! END OF STEP_MOMENTUM
!===========================================================================



!===========================================================================
! BEGINNING OF WRAP_PROGNOSTICS
!===========================================================================
	subroutine wrap_prognostics
	implicit none
#ifndef no_pbl
	integer (kind = int_kind), parameter :: 
     *      nvar =  nlm*(4+ntracer) +2
#else
	integer (kind = int_kind), parameter :: 
     *      nvar =  nlm*(4+ntracer) +1
#endif
	real (kind = dbl_kind) :: buffer(iim,jjm,nvar,nsdm) = 0

	i = 0
	j = 0

	i = 1
	do ksdm =1 ,nsdm
          buffer(  1:2,    :,i,ksdm) = ps(  1:2,    :,n4,ksdm)
          buffer(iim-1,    :,i,ksdm) = ps(iim-1,    :,n4,ksdm)
          buffer(    :,  1:2,i,ksdm) = ps(    :,  1:2,n4,ksdm)
          buffer(    :,jjm-1,i,ksdm) = ps(    :,jjm-1,n4,ksdm)
	enddo


#ifdef no_pbl
      i = 1
#else
	  i = 2
        do ksdm =1 ,nsdm
          buffer(  1:2,    :,i,ksdm) = pb(  1:2,    :,n4,ksdm)
          buffer(iim-1,    :,i,ksdm) = pb(iim-1,    :,n4,ksdm)
          buffer(    :,  1:2,i,ksdm) = pb(    :,  1:2,n4,ksdm)
          buffer(    :,jjm-1,i,ksdm) = pb(    :,jjm-1,n4,ksdm)
        enddo
#endif

	do ksdm =1 ,nsdm
	do k=1,nlm
	  j = i + k
	  buffer(  1:2,    :,j,ksdm) = eta(  1:2,    :,k,n4,ksdm)
	  buffer(iim-1,    :,j,ksdm) = eta(iim-1,    :,k,n4,ksdm)
	  buffer(    :,  1:2,j,ksdm) = eta(    :,  1:2,k,n4,ksdm)
          buffer(    :,jjm-1,j,ksdm) = eta(    :,jjm-1,k,n4,ksdm)
	enddo
	enddo

        i = j
        do ksdm =1 ,nsdm
        do k=1,nlm
          j = i + k
          buffer(  1:2,    :,j,ksdm) = div(  1:2,    :,k,n4,ksdm)
          buffer(iim-1,    :,j,ksdm) = div(iim-1,    :,k,n4,ksdm)
          buffer(    :,  1:2,j,ksdm) = div(    :,  1:2,k,n4,ksdm)
          buffer(    :,jjm-1,j,ksdm) = div(    :,jjm-1,k,n4,ksdm)
        enddo
        enddo

        i = j
        do ksdm =1 ,nsdm
        do k=1,nlm
          j = i + k
          buffer(  1:2,    :,j,ksdm) = th(  1:2,    :,k,n4,ksdm)
          buffer(iim-1,    :,j,ksdm) = th(iim-1,    :,k,n4,ksdm)
          buffer(    :,  1:2,j,ksdm) = th(    :,  1:2,k,n4,ksdm)
          buffer(    :,jjm-1,j,ksdm) = th(    :,jjm-1,k,n4,ksdm)
        enddo
        enddo

        i = j
        do ksdm =1 ,nsdm
        do k=1,nlm
          j = i + k
          buffer(  1:2,    :,j,ksdm) = ke(  1:2,    :,k,ksdm)
          buffer(iim-1,    :,j,ksdm) = ke(iim-1,    :,k,ksdm)
          buffer(    :,  1:2,j,ksdm) = ke(    :,  1:2,k,ksdm)
          buffer(    :,jjm-1,j,ksdm) = ke(    :,jjm-1,k,ksdm)
        enddo
        enddo

	i = j
	do ksdm = 1,nsdm
	do nt = 1,ntracer
        do k=1,nlm
          j = i + (nt-1)*nlm + k
          buffer(  1:2,    :,j,ksdm) = tracer(  1:2,    :,k,nt,n4,ksdm)
          buffer(iim-1,    :,j,ksdm) = tracer(iim-1,    :,k,nt,n4,ksdm)
          buffer(    :,  1:2,j,ksdm) = tracer(    :,  1:2,k,nt,n4,ksdm)
          buffer(    :,jjm-1,j,ksdm) = tracer(    :,jjm-1,k,nt,n4,ksdm)
        enddo
	enddo
	enddo

	if(j.ne.nvar) then
	   write(6,*) ' stop 001'
	   stop
	endif

	call wrap ( max_level,
     *             iim,
     *             jjm,
     *             nvar,
     *             nsdm,
     *             buffer(:,:,:,:) )

	i = 0
	j = 0

	i = 1
	do ksdm = 1,nsdm
          ps(    1,    :,n4,ksdm) = buffer(    1,    :,i,ksdm)
          ps(iim  ,    :,n4,ksdm) = buffer(iim  ,    :,i,ksdm)
          ps(    :,    1,n4,ksdm) = buffer(    :,    1,i,ksdm)
          ps(    :,jjm  ,n4,ksdm) = buffer(    :,jjm  ,i,ksdm)
	enddo


#ifdef no_pbl
        i = 1
#else
        i = 2
        do ksdm = 1,nsdm
          pb(    1,    :,n4,ksdm) = buffer(    1,    :,i,ksdm)
          pb(iim  ,    :,n4,ksdm) = buffer(iim  ,    :,i,ksdm)
          pb(    :,    1,n4,ksdm) = buffer(    :,    1,i,ksdm)
          pb(    :,jjm  ,n4,ksdm) = buffer(    :,jjm  ,i,ksdm)
        enddo

#endif

	do ksdm = 1,nsdm
	do k=1,nlm
	  j = i + k
          eta(    1,    :,k,n4,ksdm) = buffer(    1,    :,j,ksdm)
          eta(iim  ,    :,k,n4,ksdm) = buffer(iim  ,    :,j,ksdm)
          eta(    :,    1,k,n4,ksdm) = buffer(    :,    1,j,ksdm)
          eta(    :,jjm  ,k,n4,ksdm) = buffer(    :,jjm  ,j,ksdm)
	enddo
	enddo

        i = j
        do ksdm = 1,nsdm
        do k=1,nlm
          j = i + k
          div(    1,    :,k,n4,ksdm) = buffer(    1,    :,j,ksdm)
          div(iim  ,    :,k,n4,ksdm) = buffer(iim  ,    :,j,ksdm)
          div(    :,    1,k,n4,ksdm) = buffer(    :,    1,j,ksdm)
          div(    :,jjm  ,k,n4,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo
        enddo

        i = j
        do ksdm = 1,nsdm
        do k=1,nlm
          j = i + k
          th(    1,    :,k,n4,ksdm) = buffer(    1,    :,j,ksdm)
          th(iim  ,    :,k,n4,ksdm) = buffer(iim  ,    :,j,ksdm)
          th(    :,    1,k,n4,ksdm) = buffer(    :,    1,j,ksdm)
          th(    :,jjm  ,k,n4,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo
        enddo

        i = j
        do ksdm = 1,nsdm
        do k=1,nlm
          j = i + k
          ke(    1,    :,k,ksdm) = buffer(    1,    :,j,ksdm)
          ke(iim  ,    :,k,ksdm) = buffer(iim  ,    :,j,ksdm)
          ke(    :,    1,k,ksdm) = buffer(    :,    1,j,ksdm)
          ke(    :,jjm  ,k,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo
        enddo

	i = j
	do ksdm = 1,nsdm
        do nt = 1,ntracer
	do k = 1,nlm
            j = i + (nt-1)*nlm + k
            tracer(    1,    :,k,nt,n4,ksdm) = buffer(    1,    :,j,ksdm)
            tracer(iim  ,    :,k,nt,n4,ksdm) = buffer(iim  ,    :,j,ksdm)
            tracer(    :,    1,k,nt,n4,ksdm) = buffer(    :,    1,j,ksdm)
            tracer(    :,jjm  ,k,nt,n4,ksdm) = buffer(    :,jjm  ,j,ksdm)
	enddo
	enddo
	enddo

	if(j.ne.nvar) then
	   write(6,*) ' stop 002'
	   stop
	endif

	end subroutine wrap_prognostics
!===========================================================================
! END OF WRAP_PROGNOSTICS
!===========================================================================

        subroutine write_scalar_data ( xin, tau, vtag)
        implicit none
        real (kind=dbl_kind), intent(in) :: xin(iim,jjm,nsdm)
        integer (kind=int_kind), intent(in) :: tau
        character (len=80), intent(in) :: vtag
        integer (kind=int_kind) :: itau,i,j,ksdm
        character (len=80) :: ctau, ftag
        itau = tau + 10000000
        close(99)
        write(99,fmt='(i8)') itau
        close(99)
        read(99,fmt='(a80)') ctau
        ctau(1:1) = '0'
        close(99)
        ftag = 'dx/' // trim(vtag) // '.' // trim(ctau)
        open(98,file=trim(datapath)//ftag,form='formatted',status='unknown')
        do ksdm=1,nsdm
        do j=1,jjm
        do i=1,iim
        if(grid_center_mask(i,j,ksdm).eq.1.0_dbl_kind) then
          write(98,fmt='(e20.10)') xin(i,j,ksdm)
        endif
        enddo
        enddo
        enddo
        close(98)
        end subroutine write_scalar_data

        subroutine write_vector_data ( xin, tau, vtag)
        implicit none
        real (kind=dbl_kind), intent(in) ::
     *          xin(nspace,ncorners,iim,jjm,nsdm)
        integer (kind=int_kind), intent(in) :: tau
        character (len=80), intent(in) :: vtag
        integer (kind=int_kind) :: itau,i,j,ksdm,icorner
        character (len=80) :: ctau, ftag
        real (kind=dbl_kind) :: t(3), s(3)
        itau = tau + 10000000
        close(99)
        write(99,fmt='(i8)') itau
        close(99)
        read(99,fmt='(a80)') ctau
        ctau(1:1) = '0'
        close(99)
        ftag = 'dx/' // trim(vtag) // '.' // trim(ctau)
        open(98,file=trim(datapath)//ftag,form='formatted',status='unknown')
        do ksdm=1,nsdm
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        if(grid_corner_mask(icorner,i,j,ksdm).eq.1.0_dbl_kind) then
          s(1) = xin(1,icorner,i,j,ksdm)
          s(2) = xin(2,icorner,i,j,ksdm)
          s(3) = 0.0_dbl_kind
          t(:) = matmul(trans_to_X1(:,:,icorner,i,j,ksdm),s(:))
          write(98,fmt='(3e20.10)') t(1), t(2), t(3)
        endif
        enddo
        enddo
        enddo
        enddo
        close(98)
        end subroutine write_vector_data


	end module step

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
