!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module utility

!-----------------------------------------------------------------------
! PURPOSE: global utilities to be used by entire module
!   should only need to use kinds and physical_parameters
!-----------------------------------------------------------------------

	use kinds
	use physical_parameters

	implicit none
        private
	save

	public ::
     *     dgtsl, cslp, invcslp


!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF DGTSL
!======================================================================
      subroutine dgtsl(n,c,d,e,b,info)

!----------------------------------------------------------------------------
! PURPOSE: solve a tridiagonal system
!----------------------------------------------------------------------------

!----------------------------------------------------------------------------
! LOG:
!       general tridiagonal solver adopted from NETLIB
!       tdr 4-2-98
!----------------------------------------------------------------------------

      integer n,info
      real (kind=dbl_kind) :: c(n),d(n),e(n),b(n)

!     dgtsl given a general tridiagonal matrix and a right hand
!     side will find the solution.
!
!     on entry
!
!        n       integer
!                is the order of the tridiagonal matrix.
!
!        c       real (kind=dbl_kind)
!                is the subdiagonal of the tridiagonal matrix.
!                c(2) through c(n) should contain the subdiagonal.
!                on output c is destroyed.
!
!        d       real (kind=dbl_kind)
!                is the diagonal of the tridiagonal matrix.
!                on output d is destroyed.
!
!        e       real (kind=dbl_kind)
!                is the superdiagonal of the tridiagonal matrix.
!                e(1) through e(n-1) should contain the superdiagonal.
!                on output e is destroyed.
!
!        b       real (kind=dbl_kind)
!                is the right hand side vector.
!
!     on return
!
!        b       is the solution vector.
!
!        info    integer
!                = 0 normal value.
!                = k if the k-th element of the diagonal becomes
!                    exactly zero.  the subroutine returns when
!                    this is detected.
!
!     linpack. this version dated 08/14/78 .
!     jack dongarra, argonne national laboratory.
!
!     no externals
!     fortran abs
!
!     internal variables
!
      integer k,kb,kp1,nm1,nm2
      real (kind=dbl_kind) :: t
!     begin block permitting ...exits to 100
!
         info = 0
         c(1) = d(1)
         nm1 = n - 1
         if (nm1 .lt. 1) go to 40
            d(1) = e(1)
            e(1) = 0.0
            e(n) = 0.0

            do 30 k = 1, nm1
               kp1 = k + 1

!              find the largest of the two rows

               if (abs(c(kp1)) .lt. abs(c(k))) go to 10

!                 interchange row

                  t = c(kp1)
                  c(kp1) = c(k)
                  c(k) = t
                  t = d(kp1)
                  d(kp1) = d(k)
                  d(k) = t
                  t = e(kp1)
                  e(kp1) = e(k)
                  e(k) = t
                  t = b(kp1)
                  b(kp1) = b(k)
                  b(k) = t
   10          continue

!              zero elements

               if (c(k) .ne. 0.0) go to 20
                  info = k
!     ............exit
                  go to 100
   20          continue
               t = -c(kp1)/c(k)
               c(kp1) = d(kp1) + t*d(k)
               d(kp1) = e(kp1) + t*e(k)
               e(kp1) = 0.0
               b(kp1) = b(kp1) + t*b(k)
   30       continue
   40    continue
         if (c(n) .ne. 0.0) go to 50
            info = n
         go to 90
   50    continue

!           back solve

            nm2 = n - 2
            b(n) = b(n)/c(n)
            if (n .eq. 1) go to 80
               b(nm1) = (b(nm1) - d(nm1)*b(n))/c(nm1)
               if (nm2 .lt. 1) go to 70
               do 60 kb = 1, nm2
                  k = nm2 - kb + 1
                  b(k) = (b(k) - d(k)*b(k+1) - e(k)*b(k+2))/c(k)
   60          continue
   70          continue
   80       continue
   90    continue
  100 continue

      end subroutine dgtsl
!======================================================================
! END OF DGTSL
!======================================================================

c--------------------

      SUBROUTINE CSLP(psl, ps, theta, phis, len)

c    this subroutine uses the ecmwf algorithm that corrects for warm and
c       cold locations
c--------------------

      implicit none

C    argument list
      integer (kind=int_kind), intent(in) :: len  ! size of domain

      real (kind=dbl_kind), dimension(len), intent(in) ::
     *    ps    ! surface pressure (Pa)
     *,   theta ! surface air potential temperature (K)
     *,   phis  ! surface geopotential (m2/s2)

      real (kind=dbl_kind), dimension(len), intent(out) ::
     *    psl  ! sea level pressure (Pa)

c    local variables
      integer (kind=int_kind) :: i
      real (kind=dbl_kind) :: tem1, tem2, alpha, tstar, t0, tas

      do i = 1,len
         psl(i) = ps(i)
         tas = theta(i) * ( ps(i)/ p0_sfc)**kappa
c   adjust lapse rate and temperatures for extremely warm and cold locations
         if( abs(phis(i)).ge.0.1) then
            alpha = 0.0065 * gas_const_R/grav
            t0 = tas + 0.0065 * phis(i) / grav
            tstar = tas
            if(tstar.le.290.5.and.t0.gt.290.5) then
               alpha = gas_const_R / phis(i) * (290.5 - tstar)
            elseif(tstar.gt.290.5.and.t0.gt.290.5) then
               alpha = 0.0
               tstar = 0.5 * (290.5+tstar)
            elseif(tstar.lt.255.0) then
               tstar = 0.5 * (255.0 + tstar)
            endif
            tem1 = 1./(gas_const_R*tstar)
            tem2 = alpha*phis(i)*tem1
            psl(i) = ps(i)*exp( phis(i)*tem1*
     *            (1.-0.5*tem2 +0.33333333*tem2**2) )
         endif
      enddo


      end subroutine cslp

c--------------------

      SUBROUTINE invCSLP(psl, ps, theta, phis, len)

c    this subroutine inverts the ecmwf algorithm for sea level pressure

      implicit none

      integer (kind=int_kind), intent(in) :: len  ! size of domain

      real (kind=dbl_kind), dimension(len), intent(in) ::
     *    psl   ! sea level pressure (Pa)
     *,   theta ! surface air potential temperature (K)
     *,   phis  ! surface geopotential (m2/s2)

      real (kind=dbl_kind), dimension(len), intent(out) ::
     *    ps   ! surface pressure (Pa)

c    local variables
      integer (kind=int_kind) :: i, n
      real (kind=dbl_kind) :: tem1, tem2, t0
      real (kind=dbl_kind), dimension(len) :: alpha, tstar

! initialize tstar
      do i = 1,len
         alpha(i) = 0.0065*gas_const_R/grav
         tstar(i) = (inv_p0_sfc*psl(i))**kappa * theta(i) -
     *              0.0065_dbl_kind*phis(i)/ grav
      enddo

!  iterate to get tstar
      do n = 1,4
      do i = 1,len
         tem1 = 1./(gas_const_R*tstar(i))
         tem2 = alpha(i)*phis(i)*tem1
         ps(i) = psl(i)*exp( - phis(i)*tem1*
     *         (1.-0.5_dbl_kind*tem2 +0.33333333_dbl_kind*tem2**2) )
         tstar(i) = (inv_p0_sfc*ps(i))**kappa * theta(i)
      enddo
      enddo

      do n = 1,2
      do i = 1,len
         if(tstar(i).lt.255.0_dbl_kind) then
            tstar(i) = 0.5_dbl_kind * (255.0_dbl_kind + tstar(i))
         else if(tstar(i).gt.290.5_dbl_kind) then
            alpha(i) = 0.0_dbl_kind
            tstar(i) = 0.5_dbl_kind * (290.5_dbl_kind + tstar(i))
         else
            t0 = tstar(i) + 0.0065_dbl_kind * phis(i) / grav
            if(t0.gt.290.5_dbl_kind) alpha(i) = gas_const_R / phis(i) *
     *                                 (290.5_dbl_kind - tstar(i))
         endif
         tem1 = 1._dbl_kind/(gas_const_R*tstar(i))
         tem2 = alpha(i)*phis(i)*tem1
         ps(i) = psl(i)*exp( - phis(i)*tem1*
     *         (1.-0.5_dbl_kind*tem2 +0.33333333_dbl_kind*tem2**2) )
         tstar(i) = (inv_p0_sfc*ps(i))**kappa * theta(i)
      enddo
      enddo


      end subroutine invcslp

	end module utility

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
