      MODULE wrap_face_data

! This routine is modified to work with the flux coupler. Basically the 
!  changes to do this are to replace max_level with asgn_level so that
!  wrapping can be done at resolutions finer than the atmosphere grid.
!  Also, the wrapsend%ws and wraprecv%wr variables have been extended so
!  that they are inverses of each other - they define the same connection in
!  reverse directions. This enables us to write the wrap_fc routine to look
!  just like the wrap routine but with messages going in the opposite direction
!  so that the partial remappings in the ghost cells are gathered and added to
!  the appropriate real cell to make the remapping complete
! Don Dazlich - 14 May 2002

      USE kinds
      USE model_parameters
      USE assign_owner

      IMPLICIT NONE
      SAVE

      TYPE wrapSEND_type
         INTEGER (KIND=int_kind) :: length
         INTEGER (KIND=int_kind), POINTER :: wS(:,:,:)
      END TYPE wrapSEND_type
      TYPE wrapRECV_type
         INTEGER (KIND=int_kind) :: length
         INTEGER (KIND=int_kind), POINTER :: wR(:,:,:)
      END TYPE wrapRECV_type

      TYPE (wrapSEND_type) :: wrapSEND(asgn_level)
      TYPE (wrapRECV_type) :: wrapRECV(asgn_level)
      INTEGER (KIND=int_kind) :: wrapPOLE(9,12,asgn_level)

      CONTAINS
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE initialize_wrap ()
! 24 AUGUST 2000
      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: level,nsbdm_level,msg_max(asgn_level)

! ALLOCATE MEMORY FOR wrapSEND AND wrapRECV
      DO level = 1,asgn_level
         nsbdm_level = 5*isdm_level(level)*jsdm_level(level)

! wrapsend%ws and wraprecv_wr now describe 11 types of connections instead of
!   the previous 9 and 6 respectively.
! Definitions:
!   ws(1) - (global) subdomain number to receive the message
!   ws(2) - tag of sent message
!   ws(3) - initial i index of message in sending array
!   ws(4) - initial j index of message in sending array
!   ws(5) (new) - i index increment of message in sending array
!   ws(6) (new) - j index increment of message in sending array
!   ws(7) (was 5) - local subdomain number to receive message in receiving array
!   ws(8) (was 6) - initial i index of message in receiving array
!   ws(9) (was 7) - initial j index of message in receiving array
!   ws(10) (was 8) - i index increment of message in receiving array
!   ws(11) (was 9 )- j index increment of message in receiving array
!   wr(1) - (global) subdomain number of sending message
!   wr(2) - tag of received message
!   wr(3) - initial i index of message in receiving array
!   wr(4) - initial j index of message in receiving array
!   wr(5) - i index increment of message in receiving array
!   wr(6) - i index increment of message in receiving array
!   wr(7) - local subdomain number to send message in sending array
!   wr(8) (new) - initial i index of message in sending array
!   wr(9) (new) - initial j index of message in sending array
!   wr(10) (new) - i index increment of message in sending array
!   wr(11) (new) - i index increment of message in sending array
!
! These definitions should make it clear that to send a message in the reverse
!   direction, one can simply substitute ws for wr and vice versa. That is
!   precisely what is done to subroutine wrap to get the connections
!   for subroutine wrap_fc

         ALLOCATE (wrapSEND(level)%wS(11,6,nsbdm_level))
         wrapSEND(level)%length = nsbdm_level
         wrapSEND(level)%wS(:,:,:) = 0

         ALLOCATE (wrapRECV(level)%wR(11,6,nsbdm_level))
         wrapRECV(level)%length = nsbdm_level
         wrapRECV(level)%wR(:,:,:) = 0
      ENDDO
      wrapPOLE(:,:,:) = 0

      CALL set_connections ()
      msg_max(:) = 0
      CALL set_tags (msg_max)
      CALL set_pole (msg_max)

      END SUBROUTINE initialize_wrap 
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc

      SUBROUTINE set_connections ()
! 23 AUGUST 2000
      IMPLICIT NONE
      INTEGER (KIND=int_kind) :: level,im_t,im5_t,jm_t,
     &                           isdm_t,jsdm_t,iim_t,jjm_t,nsbdm_t,ierr
      INTEGER (KIND=int_kind), ALLOCATABLE :: 
     &                    sbdmn1(:,:,:),sbdmn2(:,:,:,:),sbdmn3(:,:,:,:),
     &                    wrapSEND_count(:),wrapRECV_count(:),
     &                    msgEDGE(:,:,:,:),msgEDGE_count(:)
      INTEGER (KIND=int_kind) :: nghbr_i(4),nghbr_j(4),
     &            start_i(4),start_j(4),di(4),dj(4),diag_i(4),diag_j(4),
     &            i,j,panel,isd,jsd,nsbd,i1,j1,i2,j2,del_i,del_j,
     &            count,count1,count2,nghbr,edge,sndr,diag

      DO level = 1,asgn_level
         IF (l_grid_even) THEN
            im_t = 5*(2+2*2*2**(level-1)) 
            jm_t =   (2+  2*2**(level-1)) 
         ELSE
            im_t = 5*(2+2*3*2**(level-1))
            jm_t =   (2+  3*2**(level-1)) 
         ENDIF
         isdm_t = isdm_level(level)
         jsdm_t = jsdm_level(level)

           im5_t = im_t/5
           iim_t = 2+(im5_t-2)/isdm_t
           jjm_t = 2+(jm_t -2)/jsdm_t
         nsbdm_t = 5*isdm_t*jsdm_t

         ALLOCATE (sbdmn1(2,im_t,jm_t)          ,STAT=ierr)
         ALLOCATE (sbdmn2(2,iim_t,jjm_t,nsbdm_t),STAT=ierr)
         ALLOCATE (sbdmn3(2,iim_t,jjm_t,nsbdm_t),STAT=ierr)
         ALLOCATE (wrapSEND_count(nsbdm_t)      ,STAT=ierr)
         ALLOCATE (wrapRECV_count(nsbdm_t)      ,STAT=ierr)
         ALLOCATE (msgEDGE(2,iim_t-2,4,nsbdm_t) ,STAT=ierr)
         ALLOCATE (msgEDGE_count(nsbdm_t)       ,STAT=ierr)

! INITIALIZE
         nghbr_i = (/iim_t,2,1,2/)
         nghbr_j = (/2,jjm_t,2,1/)
         start_i = (/iim_t-1,2,2,2/)
         start_j = (/2,jjm_t-1,2,2/)
         di = (/0,1,0,1/)
         dj = (/1,0,1,0/)
         diag_i = (/1,iim_t-1,1,iim_t/)
         diag_j = (/1,1,jjm_t-1,jjm_t/)

         sbdmn1(:,:,:)     = 0
         sbdmn2(:,:,:,:)   = 0
         sbdmn3(:,:,:,:)   = 0
         wrapSEND_count(:) = 0
         wrapRECV_count(:) = 0
         msgEDGE(:,:,:,:)  = 0
         msgEDGE_count(:)  = 0

! SET sbdmn1
         sbdmn1(:,1,jm_t-1) = (/1,-1/) ! NORTH POLE
         sbdmn1(:,im_t-1,1) = (/1,-2/) ! SOUTH POLE
         count1 = 0
         count2 = 0
         DO panel = 0,4
            DO jsd = 0,jsdm_t-1
               DO isd = 0,isdm_t-1
                  count1 = count1 + 1
                  DO j = 2,jjm_t-1
                     DO i = 2,iim_t-1
                        count2 = count2 + 1
                 sbdmn1(:,im5_t*panel+(iim_t-2)*isd+i,(jjm_t-2)*jsd+j) =
     &                                                 (/count1,count2/)
                     ENDDO
                  ENDDO
               ENDDO
            ENDDO
         ENDDO

! WRAP sbdmn1
         CALL wrap_sbdmn1 (im_t,jm_t,sbdmn1)

! SET sbdmn2
         nsbd = 0
         DO panel = 0,4
            DO jsd = 0,jsdm_t-1
               DO isd = 0,isdm_t-1
                  nsbd = nsbd + 1
                  DO j = 1,jjm_t
                     DO i = 1,iim_t
                        sbdmn2(:,i,j,nsbd) = 
     &             sbdmn1(:,im5_t*panel+(iim_t-2)*isd+i,(jjm_t-2)*jsd+j)
                     ENDDO
                  ENDDO
                  sbdmn2(:,iim_t,1,nsbd) = -999
                  sbdmn2(:,1,jjm_t,nsbd) = -999
               ENDDO
            ENDDO
         ENDDO

! ELIMINATE CORNER OF EACH SUBDOMAIN IN sbdmn2 WITH A (iim-1,jjm-1) PENTAGON
         DO nsbd = 1,nsbdm_t
            IF(MOD (nsbd-1,isdm_t*jsdm_t)+1.EQ.isdm_t*jsdm_t-jsdm_t)THEN
               sbdmn2(:,iim_t,jjm_t,nsbd) = -999
            ENDIF
            IF(MOD (nsbd-1,isdm_t*jsdm_t)+1.EQ.isdm_t*jsdm_t       )THEN
               sbdmn2(:,iim_t,jjm_t,nsbd) = -999
            ENDIF
         ENDDO

! DETERMINE THE DESTINATION OF AN EDGE MESSAGE
         DO nsbd = 1,nsbdm_t
            DO edge = 1,4
               nghbr = sbdmn2(1,nghbr_i(edge),nghbr_j(edge),nsbd)
               wrapSEND(level)%wS(1,edge,nsbd) = nghbr
               msgEDGE_count(nghbr) = msgEDGE_count(nghbr) + 1
               DO count = 0,iim_t-3
                  i = start_i(edge)+di(edge)*count
                  j = start_j(edge)+dj(edge)*count
                  msgEDGE(:,count+1,msgEDGE_count(nghbr),nghbr) =
     &                                                sbdmn2(:,i,j,nsbd)
               ENDDO
            ENDDO
         ENDDO

! DETERMINE HOW THE EDGE MESSAGES RECEIVED SHOULD BE STORED 
         DO nsbd = 1,nsbdm_t
            DO edge = 1,4
               DO j = 1,jjm_t
                  DO i = 1,iim_t
                     IF (msgEDGE(2,1,edge,nsbd).EQ.
     &                                          sbdmn2(2,i,j,nsbd)) THEN
                        i1 = i
                        j1 = j
                     ENDIF
                     IF (msgEDGE(2,2,edge,nsbd).EQ.
     &                                          sbdmn2(2,i,j,nsbd)) THEN
                        i2 = i
                        j2 = j
                     ENDIF
                  ENDDO
               ENDDO
               wrapRECV(level)%wR(1,edge,nsbd) = msgEDGE(1,1,edge,nsbd)
               wrapRECV(level)%wR(3,edge,nsbd) = i1
               wrapRECV(level)%wR(4,edge,nsbd) = j1
               wrapRECV(level)%wR(5,edge,nsbd) = i2-i1
               wrapRECV(level)%wR(6,edge,nsbd) = j2-j1
            ENDDO
         ENDDO

         DO nsbd = 1,nsbdm_t
! FILL IN NORTH POLE AND SOUTH POLE FOR EACH SUBDOMAIN OF sbdmn3
            IF(MOD (nsbd-1,isdm_t*jsdm_t)+1.EQ.isdm_t*(jsdm_t-1)+1) THEN
               sbdmn3(:,1,jjm_t-1,nsbd) =  (/1,-1/) ! NORTH POLE
            ENDIF
            IF(MOD (nsbd-1,isdm_t*jsdm_t)+1.EQ.isdm_t             ) THEN
               sbdmn3(:,iim_t-1,1,nsbd) =  (/1,-2/) ! SOUTH POLE
            ENDIF
! MARK THE PENTAGONS WITH A SPECIAL TAG SO THAT NO DIAGONAL MESSAGE IS SENT
            IF(Mod (nsbd-1,isdm_t*jsdm_t)+1.EQ.isdm_t*jsdm_t-jsdm_t)THEN
               sbdmn3(:,iim_t,jjm_t,nsbd) = -999
            ENDIF
            IF(Mod (nsbd-1,isdm_t*jsdm_t)+1.EQ.isdm_t*jsdm_t       )THEN
               sbdmn3(:,iim_t,jjm_t,nsbd) = -999
            ENDIF
         ENDDO

! BEGIN TO RECONSTRUCT sbdmn3 USING wrapRECV TO FILL IN THE EDGES
         DO nsbd = 1,nsbdm_t
            DO edge = 1,4
               i  = wrapRECV(level)%wR(3,edge,nsbd)
               j  = wrapRECV(level)%wR(4,edge,nsbd)
               del_i = wrapRECV(level)%wR(5,edge,nsbd)
               del_j = wrapRECV(level)%wR(6,edge,nsbd)
               DO count = 0,iim_t-3
                  i1 = i+del_i*count
                  j1 = j+del_j*count
                  sbdmn3(:,i1,j1,nsbd) = msgEDGE(:,count+1,edge,nsbd)
               ENDDO
            ENDDO
         ENDDO

! SET DIAGONAL MESSAGES...
! DETERMINE LOCATION AND OWNER OF MISSING GHOST POINTS IN sbdmn3 AND
! MODIFY wrapRECV.  REQUEST MISSING GHOST POINT IN sbdmn3 FROM THEIR
! OWNERS AND MODIFY wrapSEND.
         wrapSEND_count(:) = 4
         wrapRECV_count(:) = 4
         DO nsbd = 1,nsbdm_t
            DO diag = 1,4
               IF (sbdmn3(1,diag_i(diag),diag_j(diag),nsbd).EQ.0) THEN
                  sndr = sbdmn2(1,diag_i(diag),diag_j(diag),nsbd)
                  DO i = 1,iim_t
                     DO j = 1,jjm_t
                        IF (sbdmn2(2,i,j,sndr).EQ.
     &                    sbdmn2(2,diag_i(diag),diag_j(diag),nsbd)) THEN
                           i1 = i
                           j1 = j
                        ENDIF
                     ENDDO
                  ENDDO
                  wrapSEND_count(sndr) = wrapSEND_count(sndr) + 1
                  wrapSEND(level)%wS(1,wrapSEND_count(sndr),sndr) = nsbd
                  wrapSEND(level)%wS(3,wrapSEND_count(sndr),sndr) = i1
                  wrapSEND(level)%wS(4,wrapSEND_count(sndr),sndr) = j1
                  wrapSEND(level)%wS(8,wrapSEND_count(sndr),sndr) = 
     &                                                      diag_i(diag)
                  wrapSEND(level)%wS(9,wrapSEND_count(sndr),sndr) = 
     &                                                      diag_j(diag)

                  wrapRECV_count(nsbd) = wrapRECV_count(nsbd) + 1
                  wrapRECV(level)%wR(1,wrapRECV_count(nsbd),nsbd) = sndr
                  wrapRECV(level)%wR(3,wrapRECV_count(nsbd),nsbd) = 
     &                                                      diag_i(diag)
                  wrapRECV(level)%wR(4,wrapRECV_count(nsbd),nsbd) = 
     &                                                      diag_j(diag)
                  wrapRECV(level)%wR(8,wrapRECV_count(nsbd),nsbd) = i1
                  wrapRECV(level)%wR(9,wrapRECV_count(nsbd),nsbd) = j1
               ENDIF
            ENDDO
         ENDDO

         DEALLOCATE (sbdmn1,sbdmn2,sbdmn3,STAT=ierr)
         DEALLOCATE (wrapSEND_count,wrapRECV_count,STAT=ierr)
         DEALLOCATE (msgEDGE,msgEDGE_count,STAT=ierr)
      ENDDO

      END SUBROUTINE set_connections
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE wrap_sbdmn1 (im_t,jm_t,sbdmn1)
      IMPLICIT NONE
      INTEGER (KIND=int_kind) :: im_t,jm_t,sbdmn1(2,im_t,jm_t)
      INTEGER (KIND=int_kind) :: ij,o1,o2,q_t,p_t,im5_t,panel

        p_t =   im_t-1
        q_t =   jm_t-1
      im5_t = 2*jm_t-2

      DO panel = 0,4
         o1 = im5_t*(MOD (panel+0,5))
         o2 = im5_t*(MOD (panel+1,5))

         sbdmn1(:,o1+1      ,q_t) = sbdmn1(:,  1,q_t) ! NORTH POLE
         sbdmn1(:,o1+im5_t-1,  1) = sbdmn1(:,p_t,  1) ! SOUTH POLE

         DO ij = 2,q_t ! OFFSET 1 : TOP LEFT
            sbdmn1(:,o1+ij,jm_t) = sbdmn1(:,o2+2,q_t+2-ij)
         ENDDO                                                                
         DO ij = q_t+1,im5_t-1 ! OFFSET 1 : TOP RIGHT
            sbdmn1(:,o1+ij,jm_t) = sbdmn1(:,o2-q_t+1+ij,2) 
         ENDDO
         DO ij = 2,q_t ! OFFSET 1 : RIGHT EDGE
            sbdmn1(:,o1+im5_t,ij) = sbdmn1(:,o2+im5_t+1-ij,2)  
         ENDDO

         DO ij = 1,q_t-1 ! OFFSET 2 : LEFT EDGE
            sbdmn1(:,o2+1,ij) = sbdmn1(:,o1+q_t+1-ij,q_t)                       
         ENDDO
         DO ij = 2,q_t ! OFFSET 2 : BOTTOM LEFT
            sbdmn1(:,o2+ij,1) = sbdmn1(:,o1+q_t-1+ij,q_t)                
         ENDDO
         DO ij = q_t+1,im5_t-2 ! OFFSET 2 : BOTTOM RIGHT
            sbdmn1(:,o2+ij,1) = sbdmn1(:,o1+im5_t-1,im5_t-ij)      
         ENDDO
      ENDDO

      END SUBROUTINE wrap_sbdmn1
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_tags (msg_max)
! 24 AUGUST 2000
      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: msg_max(asgn_level)
      INTEGER (KIND=int_kind) :: 
     &               level,msg_tag,nsd1,nsd1_owner,msg1,dest,nsd2,msg2,m,
     &               im_t, jm_t, im5_t, iim_t, jjm_t, isdm_t, jsdm_t
      INTEGER (KIND=int_kind), dimension(4) :: i1_start, j1_start, di1, dj1

! the increment and starting index info is no longer hard wired in the wrap
!  routine. We assign these new values to ws and wr here.

      di1 = (/0,1,0,1/)
      dj1 = (/1,0,1,0/)
! GENERATE A UNIQUE TAG FOR EACH MESSAGE SENT
      DO level = 1,asgn_level
! SET im_t AND jm_t
         IF (l_grid_even) THEN
            im_t = 5*(2+2*2*2**(level-1)) 
            jm_t =   (2+  2*2**(level-1)) 
         ELSE
            im_t = 5*(2+2*3*2**(level-1))
            jm_t =   (2+  3*2**(level-1)) 
         ENDIF
         isdm_t = isdm_level(level)
         jsdm_t = jsdm_level(level)

          im5_t = im_t/5
          iim_t = 2+(im5_t-2)/isdm_t
          jjm_t = 2+(jm_t -2)/jsdm_t
         i1_start = (/iim_t-1,2,2,2/)
         j1_start = (/2,jjm_t-1,2,2/)
         msg_tag = 0
         DO nsd1 = 1,wrapSEND(level)%length
            nsd1_owner = sbdmn_owner(level)%owner(nsd1)
            DO msg1 = 1,6

! assigning the new slots in ws that were previously hardwired in wrap
               if(msg1.le.4) then
                  wrapSEND(level)%wS(3,msg1,nsd1) = i1_start(msg1)
                  wrapSEND(level)%wS(4,msg1,nsd1) = j1_start(msg1)
                  wrapSEND(level)%wS(5,msg1,nsd1) = di1(msg1)
                  wrapSEND(level)%wS(6,msg1,nsd1) = dj1(msg1)
               endif
               dest = wrapSEND(level)%wS(1,msg1,nsd1)
               IF (dest.NE.0) THEN
                  IF (nsd1_owner.EQ.sbdmn_owner(level)%owner(dest)) THEN
                     DO nsd2 = 1,sbdmn_assign(nsd1_owner+1,level)%length
                        IF (sbdmn_assign(nsd1_owner+1,level)%sd(nsd2)
     &                                                    .EQ.dest) THEN
                           wrapSEND(level)%wS(7,msg1,nsd1) = nsd2
                        ENDIF
                     ENDDO

! the source indices and increments and target indices and increments
!  for ws and wr correspond. The one is copied to the other.
                     DO msg2 = 1,6
                        IF(wrapRECV(level)%wR(1,msg2,dest).EQ.nsd1) THEN
                           DO m = 8,11
                              wrapSEND(level)%wS(m,msg1,nsd1) = 
     &                                 wrapRECV(level)%wR(m-5,msg2,dest)
                              wrapRECV(level)%wR(m,msg2,dest) = 
     &                                 wrapSEND(level)%wS(m-5,msg1,nsd1)
                           ENDDO
                        ENDIF
                     ENDDO
                  ELSE
                     msg_tag = msg_tag + 1
                     wrapSEND(level)%wS(2,msg1,nsd1) = msg_tag
                     DO msg2 = 1,6
                        IF (wrapRECV(level)%wR(1,msg2,dest).EQ.nsd1)THEN
                           wrapRECV(level)%wR(2,msg2,dest) = msg_tag
                           DO m = 8,11
                              wrapSEND(level)%wS(m,msg1,nsd1) = 
     &                                 wrapRECV(level)%wR(m-5,msg2,dest)
                              wrapRECV(level)%wR(m,msg2,dest) = 
     &                                 wrapSEND(level)%wS(m-5,msg1,nsd1)
                           ENDDO
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDDO

! this loop is added to create a wraprecv(7) that is symmetric with wrapsend(7)
         DO nsd1 = 1,wrapSEND(level)%length
            nsd1_owner = sbdmn_owner(level)%owner(nsd1)
            DO msg1 = 1,6
               dest = wrapRECV(level)%wR(1,msg1,nsd1)
               IF (dest.NE.0) THEN
                  IF (nsd1_owner.EQ.sbdmn_owner(level)%owner(dest)) THEN
                     DO nsd2 = 1,sbdmn_assign(nsd1_owner+1,level)%length
                        IF (sbdmn_assign(nsd1_owner+1,level)%sd(nsd2)
     &                                                    .EQ.dest) THEN
                           wrapRECV(level)%wR(7,msg1,nsd1) = nsd2
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDDO

         msg_max(level) = msg_tag
      ENDDO

      END SUBROUTINE set_tags
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_pole (msg_max)
      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: msg_max(asgn_level)
      INTEGER (KIND=int_kind) :: level,im_t,jm_t,im5_t,
     &                           isdm_t,jsdm_t,iim_t,jjm_t
      INTEGER (KIND=int_kind) :: npole,n,pole_sd(2),pole_owner,
     &                           nghbr_sd(4,2),nghbr_owner,count,
     &                           nsd,pole_nsd(2),nghbr_nsd(4,2),msg_tag
      INTEGER (KIND=int_kind) :: pole_i(2),pole_j(2),
     &                           sndr_i(4),sndr_j(4),dest_i(4),dest_j(4)

      DO level = 1,asgn_level
! SET im_t AND jm_t
         IF (l_grid_even) THEN
            im_t = 5*(2+2*2*2**(level-1)) 
            jm_t =   (2+  2*2**(level-1)) 
         ELSE
            im_t = 5*(2+2*3*2**(level-1))
            jm_t =   (2+  3*2**(level-1)) 
         ENDIF
         isdm_t = isdm_level(level)
         jsdm_t = jsdm_level(level)

          im5_t = im_t/5
          iim_t = 2+(im5_t-2)/isdm_t
          jjm_t = 2+(jm_t -2)/jsdm_t

         pole_i = (/1,iim_t-1/)
         pole_j = (/jjm_t-1,1/)
         sndr_i = (/2,2,iim_t-1,iim_t-1/)
         sndr_j = (/jjm_t-1,jjm_t-1,2,2/)
         dest_i = (/1,iim_t,iim_t,1/)
         dest_j = (/jjm_t,1,1,jjm_t/)

         pole_sd(1) = isdm_t*(  jsdm_t-1)+1 ! SUBDOMAIN NUMBER OF NORTH POLE
         pole_sd(2) = isdm_t*(4*jsdm_t+1)   ! SUBDOMAIN NUMBER OF SOUTH POLE

! SUBDOMAIN NEIGHBORS OF NORTH POLE
         DO n = 1,4
            nghbr_sd(n,1) = isdm_t*(jsdm_t*(n+1)-1)+1
         ENDDO
! SUBDOMAIN NEIGHBORS OF SOUTH POLE
         DO n = 1,4
            nghbr_sd(n,2) = isdm_t*(jsdm_t*(n-1)+1)   
         ENDDO

         count = 0
         msg_tag = msg_max(level)

! SET wrapPOLE FOR THE NORTH AND SOUTH POLE
         DO npole = 1,2
            pole_owner = sbdmn_owner(level)%owner(pole_sd(npole))
            DO nsd = 1,sbdmn_assign(pole_owner+1,level)%length
               IF (pole_sd(npole).EQ.
     &              sbdmn_assign(pole_owner+1,level)%sd(nsd)) THEN
                  pole_nsd(npole) = nsd
               ENDIF
            ENDDO

            DO n = 1,4
               nghbr_owner = sbdmn_owner(level)%owner(nghbr_sd(n,npole))
               DO nsd = 1,sbdmn_assign(nghbr_owner+1,level)%length
                  IF (nghbr_sd(n,npole).EQ.
     &                   sbdmn_assign(nghbr_owner+1,level)%sd(nsd)) THEN
                     nghbr_nsd(n,npole) = nsd
                  ENDIF
               ENDDO

               count = count+1
               wrapPOLE(1,count,level) =  pole_sd(  npole)
               wrapPOLE(2,count,level) = nghbr_sd(n,npole)
               IF (pole_owner.EQ.nghbr_owner) THEN
                  wrapPOLE(3,count,level) = 0
               ELSE
                  msg_tag = msg_tag+1
                  wrapPOLE(3,count,level) = msg_tag
               ENDIF
               wrapPOLE(4,count,level) = pole_i(npole)
               wrapPOLE(5,count,level) = pole_j(npole)
               wrapPOLE(6,count,level) = pole_nsd(npole)
               wrapPOLE(7,count,level) = pole_i(npole)
               wrapPOLE(8,count,level) = pole_j(npole)
               wrapPOLE(9,count,level) = nghbr_nsd(n,npole)
            ENDDO
         ENDDO

! MESSAGES NEED TO ALSO BE SENT TO THE NORTH POLE AND THE SOUTH POLE 
         DO npole = 1,2
            pole_owner = sbdmn_owner(level)%owner(pole_sd(npole))
            DO n = 2,3
               nghbr_owner = sbdmn_owner(level)%owner(nghbr_sd(n,npole))
               count = count+1
               wrapPOLE(1,count,level) = nghbr_sd(n,npole)
               wrapPOLE(2,count,level) =  pole_sd(  npole)
               IF (pole_owner.EQ.nghbr_owner) THEN
                  wrapPOLE(3,count,level) = 0
               ELSE
                  msg_tag = msg_tag+1
                  wrapPOLE(3,count,level) = msg_tag
               ENDIF
               wrapPOLE(4,count,level) = sndr_i(count-8)
               wrapPOLE(5,count,level) = sndr_j(count-8)
               wrapPOLE(6,count,level) = nghbr_nsd(n,npole)
               wrapPOLE(7,count,level) = dest_i(count-8)
               wrapPOLE(8,count,level) = dest_j(count-8)
               wrapPOLE(9,count,level) = pole_nsd(npole)
            ENDDO
         ENDDO
      ENDDO

      END SUBROUTINE set_pole

!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE wrap (level,iim_t,jjm_t,nvar,nsdm_t,x)

      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: level,iim_t,jjm_t,nvar,nsdm_t
      REAL (KIND=dbl_kind) :: x(iim_t,jjm_t,nvar,nsdm_t)

      INTEGER (KIND=int_kind) :: sbdmn_number,edge,diag,pole,                
     &                           edge_size,diag_size,pole_size,               
     &                           nsd,i_start,j_start,di,dj,i,j,               
     &                 nsd1,i1,j1,i1_start,j1_start,di1,dj1,                  
     &                 nsd2,i2_start,j2_start,di2,dj2,i2,j2,count,            
     &                 edge_dest,diag_dest,pole_dest,                         
     &                 edge_tag ,diag_tag ,pole_tag,                          
     &                 edge_src ,diag_src ,pole_src,                          
     &                 count_send_req,count_recv_req,                         
     &                 isdm_t,jsdm_t,sd_np,sd_sp,ierr
      INTEGER (KIND=int_kind), ALLOCATABLE :: send_req(:),recv_req(:)
      INTEGER (KIND=int_kind), ALLOCATABLE ::                                
     &                                  send_status(:,:),recv_status(:,:)

      REAL (KIND=dbl_kind), ALLOCATABLE :: buffer_send_edge(:,:,:,:),        
     &                                     buffer_send_diag(:,:,:),          
     &                                     buffer_send_pole(:,:)
      REAL (KIND=dbl_kind), ALLOCATABLE :: buffer_recv_edge(:,:,:,:),        
     &                                     buffer_recv_diag(:,:,:),          
     &                                     buffer_recv_pole(:,:)
      ALLOCATE (buffer_send_edge(iim_t-2,nvar,4,nsdm_t))
      ALLOCATE (buffer_send_diag(nvar,5:6,nsdm_t))
      ALLOCATE (buffer_send_pole(nvar,12))
      ALLOCATE (buffer_recv_edge(iim_t-2,nvar,4,nsdm_t))
      ALLOCATE (buffer_recv_diag(nvar,5:6,nsdm_t))
      ALLOCATE (buffer_recv_pole(nvar,12))
      ALLOCATE (send_req(6*nsdm_t+12))
      ALLOCATE (recv_req(6*nsdm_t+12))

      buffer_send_edge(:,:,:,:) = 0.0
      buffer_send_diag(  :,:,:) = 0.0
      buffer_send_pole(    :,:) = 0.0
      buffer_recv_edge(:,:,:,:) = 0.0
      buffer_recv_diag(  :,:,:) = 0.0
      buffer_recv_pole(    :,:) = 0.0

      send_req(:) = -999
      recv_req(:) = -999

      edge_size = (iim_t-2)*nvar
      diag_size = nvar
      pole_size = nvar

      count_send_req = 0
      count_recv_req = 0

#ifdef mpi
!     IF (npe_atmos > 1) THEN
! POST RECEIVES FOR EDGES AND DIAGONALS
         DO nsd1 = 1,nsdm_t
            sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd1)
! POST EDGE RECEIVES
            DO edge = 1,4
               IF (wrapRECV(level)%wR(2,edge,sbdmn_number).NE.0) THEN
                   edge_src = sbdmn_owner(level)%                             
     &                    owner(wrapRECV(level)%wR(1,edge,sbdmn_number))
                  edge_tag =    wrapRECV(level)%wR(2,edge,sbdmn_number)
                  count_recv_req = count_recv_req + 1
                  CALL MPI_IRECV (buffer_recv_edge(1,1,edge,nsd1),edge_size, 
     &                      mpi_float_type,edge_src,edge_tag,                 
     &                      mpi_comm_atmos,recv_req(count_recv_req),ierr)
               ENDIF
            ENDDO
! POST DIAGONAL RECEIVES
            DO diag = 5,6
               IF ((wrapRECV(level)%wR(1,diag,sbdmn_number).NE.0).AND.       
     &              (wrapRECV(level)%wR(2,diag,sbdmn_number).NE.0)) THEN
                  diag_src = sbdmn_owner(level)%                             
     &                     owner(wrapRECV(level)%wR(1,diag,sbdmn_number))
                  diag_tag =    wrapRECV(level)%wR(2,diag,sbdmn_number)
                  count_recv_req = count_recv_req + 1
                  CALL MPI_IRECV (buffer_recv_diag(1,diag,nsd1),diag_size,   
     &                      mpi_float_type,diag_src,diag_tag,                 
     &                      mpi_comm_atmos,recv_req(count_recv_req),ierr)
               ENDIF
            ENDDO
         ENDDO ! LOOP OVER nsdm_t SUBDOMAINS

! POST RECEIVES FOR THE POLES
         DO pole = 1,12
            IF ((my_task.EQ.                                                 
     &              sbdmn_owner(level)%owner(wrapPOLE(2,pole,level))).AND.    
     &                                (wrapPOLE(3,pole,level).NE.0)) THEN
               pole_src = sbdmn_owner(level)%owner(wrapPOLE(1,pole,level))
               pole_tag =                      wrapPOLE(3,pole,level)
               count_recv_req = count_recv_req + 1
               CALL MPI_IRECV (buffer_recv_pole(1,pole),pole_size,           
     &                      mpi_float_type,pole_src,pole_tag,                 
     &                      mpi_comm_atmos,recv_req(count_recv_req),ierr)
            ENDIF
         ENDDO
!     ENDIF
#endif

! POST SENDS FOR EDGES AND DIAGONALS
      DO nsd1 = 1,nsdm_t
         sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd1)
! POST SENDS FOR EDGES
         DO edge = 1,4
            i1_start = wrapSEND(level)%wS(3,edge,sbdmn_number)
            j1_start = wrapSEND(level)%wS(4,edge,sbdmn_number)
            di1      = wrapSEND(level)%wS(5,edge,sbdmn_number)
            dj1      = wrapSEND(level)%wS(6,edge,sbdmn_number)
            IF (wrapSEND(level)%wS(2,edge,sbdmn_number).EQ.0) THEN
               nsd2     = wrapSEND(level)%wS(7,edge,sbdmn_number)
               i2_start = wrapSEND(level)%wS(8,edge,sbdmn_number)
               j2_start = wrapSEND(level)%wS(9,edge,sbdmn_number)
               di2      = wrapSEND(level)%wS(10,edge,sbdmn_number)
               dj2      = wrapSEND(level)%wS(11,edge,sbdmn_number)
               DO count = 0,iim_t-3
                  i1 = i1_start      +di1      *count
                  j1 = j1_start      +dj1      *count
                  i2 = i2_start      +di2      *count
                  j2 = j2_start      +dj2      *count
                  x(i2,j2,:,nsd2) = x(i1,j1,:,nsd1)
               ENDDO
            ELSE
               DO count = 0,iim_t-3
                  i1 = i1_start      +di1      *count
                  j1 = j1_start      +dj1      *count
                  buffer_send_edge(count+1,:,edge,nsd1)=x(i1,j1,:,nsd1)
               ENDDO
               edge_dest = sbdmn_owner(level)%                               
     &                     owner(wrapSEND(level)%wS(1,edge,sbdmn_number))
               edge_tag  =      wrapSEND(level)%wS(2,edge,sbdmn_number)
#ifdef mpi
!              IF (npe_atmos > 1) THEN
                  count_send_req = count_send_req + 1
                  CALL MPI_ISEND (buffer_send_edge(1,1,edge,nsd1),edge_size, 
     &                      mpi_float_type,edge_dest,edge_tag,                
     &                      mpi_comm_atmos,send_req(count_send_req),ierr)
!              ENDIF
#endif
            ENDIF
         ENDDO
! POST SENDS FOR DIAGONALS
         DO diag = 5,6
            IF (wrapSEND(level)%wS(1,diag,sbdmn_number).NE.0) THEN
               i1 = wrapSEND(level)%wS(3,diag,sbdmn_number)
               j1 = wrapSEND(level)%wS(4,diag,sbdmn_number)
               IF (wrapSEND(level)%wS(2,diag,sbdmn_number).EQ.0) THEN
                  nsd2 = wrapSEND(level)%wS(7,diag,sbdmn_number)
                  i2   = wrapSEND(level)%wS(8,diag,sbdmn_number)
                  j2   = wrapSEND(level)%wS(9,diag,sbdmn_number)
                  x(i2,j2,:,nsd2) = x(i1,j1,:,nsd1)
               ELSE
                  diag_dest = sbdmn_owner(level)%                            
     &                     owner(wrapSEND(level)%wS(1,diag,sbdmn_number))
                  diag_tag  =   wrapSEND(level)%wS(2,diag,sbdmn_number)
                  buffer_send_diag(:,diag,nsd1) = x(i1,j1,:,nsd1)
#ifdef mpi
!                 IF (npe_atmos > 1) THEN
                     count_send_req = count_send_req + 1
                     CALL MPI_ISEND (buffer_send_diag(1,diag,nsd1),diag_size,
     &                      mpi_float_type,diag_dest,diag_tag,                
     &                      mpi_comm_atmos,send_req(count_send_req),ierr)
!                 ENDIF
#endif
               ENDIF
            ENDIF
         ENDDO
      ENDDO ! LOOP OVER nsdm SUBDOMAINS

! POST SENDS FOR THE POLES
      DO pole = 1,12
         IF (my_task.EQ.sbdmn_owner(level)%owner(wrapPOLE(1,pole,level))) THEN
              i1 = wrapPOLE(4,pole,level)
              j1 = wrapPOLE(5,pole,level)
            nsd1 = wrapPOLE(6,pole,level)
            IF (wrapPOLE(3,pole,level).EQ.0) THEN
                 i2 = wrapPOLE(7,pole,level)
                 j2 = wrapPOLE(8,pole,level)
               nsd2 = wrapPOLE(9,pole,level)
               x(i2,j2,:,nsd2) = x(i1,j1,:,nsd1)
            ELSE
               pole_dest = sbdmn_owner(level)%owner(wrapPOLE(2,pole,level))
               pole_tag  =                       wrapPOLE(3,pole,level)
               buffer_send_pole(:,pole) = x(i1,j1,:,nsd1)
#ifdef mpi
!              IF (npe_atmos > 1) THEN
                  count_send_req = count_send_req + 1
                  CALL MPI_ISEND (buffer_send_pole(1,pole),pole_size,        
     &                      mpi_float_type,pole_dest,pole_tag,                
     &                      mpi_comm_atmos,send_req(count_send_req),ierr)
!              ENDIF
#endif
            ENDIF
         ENDIF
      ENDDO

#ifdef mpi
!     IF (npe_atmos > 1) THEN

         ALLOCATE (send_status(MPI_STATUS_SIZE,count_send_req))
         ALLOCATE (recv_status(MPI_STATUS_SIZE,count_recv_req))
         send_status(:,:) = 0
         recv_status(:,:) = 0

         CALL MPI_WAITALL (count_send_req,send_req,send_status,ierr)
         CALL MPI_WAITALL (count_recv_req,recv_req,recv_status,ierr)

! UNPACK BUFFER FOR EDGE AND DIAGONAL MESSAGES
         DO nsd = 1,nsdm_t
            sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd)
            DO edge = 1,4
               IF (wrapRECV(level)%wR(2,edge,sbdmn_number).NE.0) THEN
                  i_start = wrapRECV(level)%wR(3,edge,sbdmn_number)
                  j_start = wrapRECV(level)%wR(4,edge,sbdmn_number)
                  di      = wrapRECV(level)%wR(5,edge,sbdmn_number)
                  dj      = wrapRECV(level)%wR(6,edge,sbdmn_number)
                  DO count = 0,iim_t-3
                     i = i_start+di*count
                     j = j_start+dj*count
                     x(i,j,:,nsd) = buffer_recv_edge(count+1,:,edge,nsd)
                  ENDDO
               ENDIF
            ENDDO

            DO diag = 5,6
               IF ((wrapRECV(level)%wR(1,diag,sbdmn_number).NE.0).AND.       
     &              (wrapRECV(level)%wR(2,diag,sbdmn_number).NE.0)) THEN
                  i = wrapRECV(level)%wR(3,diag,sbdmn_number)
                  j = wrapRECV(level)%wR(4,diag,sbdmn_number)
                  x(i,j,:,nsd) = buffer_recv_diag(:,diag,nsd)
               ENDIF
            ENDDO
         ENDDO ! LOOP OVER nsdm SUBDOMAINS

! UNPACK BUFFER FOR POLE MESSAGES
         DO pole = 1,12
            IF ((my_task.EQ.                                                 
     &             sbdmn_owner(level)%owner(wrapPOLE(2,pole,level))).AND.     
     &                                (wrapPOLE(3,pole,level).NE.0)) THEN
               i = wrapPOLE(7,pole,level)
               j = wrapPOLE(8,pole,level)
               nsd = wrapPOLE(9,pole,level)
               x(i,j,:,nsd) = buffer_recv_pole(:,pole)
            ENDIF
         ENDDO
!     ENDIF
#endif

! SET DEAD CELLS TO A REPRESENTATIVE VALUE
      isdm_t = isdm_level(level)
      jsdm_t = jsdm_level(level)
      sd_np = isdm_t*(  jsdm_t-1)+1
      sd_sp = isdm_t*(4*jsdm_t+1)
      DO nsd = 1,sbdmn_assign(my_task+1,level)%length
         sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd)
         IF ((sbdmn_number.NE.sd_np).AND.(sbdmn_number.NE.sd_sp)) THEN
            x(iim_t,1,:,nsd) = x(ipad+1,ipad+1,:,nsd)
            x(1,jjm_t,:,nsd) = x(ipad+1,ipad+1,:,nsd)
         ENDIF
         IF (MOD (sbdmn_number-1,isdm_t*jsdm_t)+1.EQ.                        
     &                                         isdm_t*jsdm_t-jsdm_t) THEN
            x(iim_t,jjm_t,:,nsd) = x(iim_t-1,jjm_t,:,nsd)
         ENDIF
         IF (MOD (sbdmn_number-1,isdm_t*jsdm_t)+1.EQ.                        
     &                                           isdm_t*jsdm_t     ) THEN
            x(iim_t,jjm_t,:,nsd) = x(iim_t-1,jjm_t,:,nsd)
         ENDIF
      ENDDO

      DEALLOCATE (buffer_send_edge,buffer_send_diag,buffer_send_pole)
      DEALLOCATE (buffer_recv_edge,buffer_recv_diag,buffer_recv_pole)
      DEALLOCATE (send_req,recv_req)
#ifdef mpi
!     IF (npe_atmos > 1) THEN
         DEALLOCATE (send_status,recv_status)
!     ENDIF
#endif

      END SUBROUTINE wrap

!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE wrap_fc (level,iim_t,jjm_t,nvar,nsdm_t,x)

! this routine does a 'reverse wrap' for sfc to atm flux coupler remapping.
! The paths for the messages are reversed from those in wrap. This is done 
! by switching the references to wrap_send%ws and wrap_recv%wr
! Additionally, instead of replacing the data, since these are partial sums,
! the received messages are added to values already in the target spot.

      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: level,iim_t,jjm_t,nvar,nsdm_t
      REAL (KIND=dbl_kind) :: x(iim_t,jjm_t,nvar,nsdm_t)
      REAL (KIND=dbl_kind) :: xtem(iim_t,jjm_t,nvar,nsdm_t)

      INTEGER (KIND=int_kind) :: sbdmn_number,edge,diag,pole,                
     &                           edge_size,diag_size,pole_size,               
     &                           nsd,i_start,j_start,di,dj,i,j,               
     &                 nsd1,i1,j1,i1_start,j1_start,di1,dj1,                  
     &                 nsd2,i2_start,j2_start,di2,dj2,i2,j2,count,            
     &                 edge_dest,diag_dest,pole_dest,                         
     &                 edge_tag ,diag_tag ,pole_tag,                          
     &                 edge_src ,diag_src ,pole_src,                          
     &                 count_send_req,count_recv_req,                         
     &                 isdm_t,jsdm_t,sd_np,sd_sp,ierr
      INTEGER (KIND=int_kind), ALLOCATABLE :: send_req(:),recv_req(:)
      INTEGER (KIND=int_kind), ALLOCATABLE ::                                
     &                                  send_status(:,:),recv_status(:,:)

      REAL (KIND=dbl_kind), ALLOCATABLE :: buffer_send_edge(:,:,:,:),        
     &                                     buffer_send_diag(:,:,:),          
     &                                     buffer_send_pole(:,:)
      REAL (KIND=dbl_kind), ALLOCATABLE :: buffer_recv_edge(:,:,:,:),        
     &                                     buffer_recv_diag(:,:,:),          
     &                                     buffer_recv_pole(:,:)

      ALLOCATE (buffer_send_edge(iim_t-2,nvar,4,nsdm_t))
      ALLOCATE (buffer_send_diag(nvar,5:6,nsdm_t))
      ALLOCATE (buffer_send_pole(nvar,12))
      ALLOCATE (buffer_recv_edge(iim_t-2,nvar,4,nsdm_t))
      ALLOCATE (buffer_recv_diag(nvar,5:6,nsdm_t))
      ALLOCATE (buffer_recv_pole(nvar,12))
      ALLOCATE (send_req(6*nsdm_t+12))
      ALLOCATE (recv_req(6*nsdm_t+12))

      buffer_send_edge(:,:,:,:) = 0.0
      buffer_send_diag(  :,:,:) = 0.0
      buffer_send_pole(    :,:) = 0.0
      buffer_recv_edge(:,:,:,:) = 0.0
      buffer_recv_diag(  :,:,:) = 0.0
      buffer_recv_pole(    :,:) = 0.0

      send_req(:) = -999
      recv_req(:) = -999

      edge_size = (iim_t-2)*nvar
      diag_size = nvar
      pole_size = nvar

      count_send_req = 0
      count_recv_req = 0
      xtem(:,:,:,:) = 0.0

#ifdef mpi
!     IF (npe_atmos > 1) THEN
! POST RECEIVES FOR EDGES AND DIAGONALS
         DO nsd1 = 1,nsdm_t
            sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd1)
! POST EDGE RECEIVES
            DO edge = 1,4
               IF (wrapSEND(level)%wS(2,edge,sbdmn_number).NE.0) THEN
                   edge_src = sbdmn_owner(level)%                             
     &                    owner(wrapSEND(level)%wS(1,edge,sbdmn_number))
                  edge_tag =    wrapSEND(level)%wS(2,edge,sbdmn_number)
                  count_recv_req = count_recv_req + 1
                  CALL MPI_IRECV (buffer_recv_edge(1,1,edge,nsd1),edge_size, 
     &                      mpi_float_type,edge_src,edge_tag,                 
     &                      mpi_comm_atmos,recv_req(count_recv_req),ierr)
               ENDIF
            ENDDO
! POST DIAGONAL RECEIVES
            DO diag = 5,6
               IF ((wrapSEND(level)%wS(1,diag,sbdmn_number).NE.0).AND.       
     &              (wrapSEND(level)%wS(2,diag,sbdmn_number).NE.0)) THEN
                  diag_src = sbdmn_owner(level)%                             
     &                     owner(wrapSEND(level)%wS(1,diag,sbdmn_number))
                  diag_tag =    wrapSEND(level)%wS(2,diag,sbdmn_number)
                  count_recv_req = count_recv_req + 1
                  CALL MPI_IRECV (buffer_recv_diag(1,diag,nsd1),diag_size,   
     &                      mpi_float_type,diag_src,diag_tag,                 
     &                      mpi_comm_atmos,recv_req(count_recv_req),ierr)
               ENDIF
            ENDDO
         ENDDO ! LOOP OVER nsdm_t SUBDOMAINS

! POST RECEIVES FOR THE POLES
         DO pole = 1,12
            IF ((my_task.EQ.                                                 
     &              sbdmn_owner(level)%owner(wrapPOLE(1,pole,level))).AND.    
     &                                (wrapPOLE(3,pole,level).NE.0)) THEN
               pole_src = sbdmn_owner(level)%owner(wrapPOLE(2,pole,level))
               pole_tag =                      wrapPOLE(3,pole,level)
               count_recv_req = count_recv_req + 1
               CALL MPI_IRECV (buffer_recv_pole(1,pole),pole_size,           
     &                      mpi_float_type,pole_src,pole_tag,                 
     &                      mpi_comm_atmos,recv_req(count_recv_req),ierr)
            ENDIF
         ENDDO
!     ENDIF
#endif

! POST SENDS FOR EDGES AND DIAGONALS
      DO nsd1 = 1,nsdm_t
         sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd1)
! POST SENDS FOR EDGES
         DO edge = 1,4
            i1_start = wrapRECV(level)%wR(3,edge,sbdmn_number)
            j1_start = wrapRECV(level)%wR(4,edge,sbdmn_number)
            di1      = wrapRECV(level)%wR(5,edge,sbdmn_number)
            dj1      = wrapRECV(level)%wR(6,edge,sbdmn_number)
            IF (wrapRECV(level)%wR(2,edge,sbdmn_number).EQ.0) THEN
               nsd2     = wrapRECV(level)%wR(7,edge,sbdmn_number)
               i2_start = wrapRECV(level)%wR(8,edge,sbdmn_number)
               j2_start = wrapRECV(level)%wR(9,edge,sbdmn_number)
               di2      = wrapRECV(level)%wR(10,edge,sbdmn_number)
               dj2      = wrapRECV(level)%wR(11,edge,sbdmn_number)
               DO count = 0,iim_t-3
                  i1 = i1_start      +di1      *count
                  j1 = j1_start      +dj1      *count
                  i2 = i2_start      +di2      *count
                  j2 = j2_start      +dj2      *count
                  xtem(i2,j2,:,nsd2) = xtem(i2,j2,:,nsd2) + x(i1,j1,:,nsd1)
               ENDDO
            ELSE
               DO count = 0,iim_t-3
                  i1 = i1_start      +di1      *count
                  j1 = j1_start      +dj1      *count
                  buffer_send_edge(count+1,:,edge,nsd1)=x(i1,j1,:,nsd1)
               ENDDO
               edge_dest = sbdmn_owner(level)%                               
     &                     owner(wrapRECV(level)%wR(1,edge,sbdmn_number))
               edge_tag  =      wrapRECV(level)%wR(2,edge,sbdmn_number)
#ifdef mpi
!              IF (npe_atmos > 1) THEN
                  count_send_req = count_send_req + 1
                  CALL MPI_ISEND (buffer_send_edge(1,1,edge,nsd1),edge_size, 
     &                      mpi_float_type,edge_dest,edge_tag,                
     &                      mpi_comm_atmos,send_req(count_send_req),ierr)
!              ENDIF
#endif
            ENDIF
         ENDDO
! POST SENDS FOR DIAGONALS
         DO diag = 5,6
            IF (wrapRECV(level)%wR(1,diag,sbdmn_number).NE.0) THEN
               i1 = wrapRECV(level)%wR(3,diag,sbdmn_number)
               j1 = wrapRECV(level)%wR(4,diag,sbdmn_number)
               IF (wrapRECV(level)%wR(2,diag,sbdmn_number).EQ.0) THEN
                  nsd2 = wrapRECV(level)%wR(7,diag,sbdmn_number)
                  i2   = wrapRECV(level)%wR(8,diag,sbdmn_number)
                  j2   = wrapRECV(level)%wR(9,diag,sbdmn_number)
                  xtem(i2,j2,:,nsd2) = xtem(i2,j2,:,nsd2) + x(i1,j1,:,nsd1)
               ELSE
                  diag_dest = sbdmn_owner(level)%                            
     &                     owner(wrapRECV(level)%wR(1,diag,sbdmn_number))
                  diag_tag  =   wrapRECV(level)%wR(2,diag,sbdmn_number)
                  buffer_send_diag(:,diag,nsd1) = x(i1,j1,:,nsd1)
#ifdef mpi
!                 IF (npe_atmos > 1) THEN
                     count_send_req = count_send_req + 1
                     CALL MPI_ISEND (buffer_send_diag(1,diag,nsd1),diag_size,
     &                      mpi_float_type,diag_dest,diag_tag,                
     &                      mpi_comm_atmos,send_req(count_send_req),ierr)
!                 ENDIF
#endif
               ENDIF
            ENDIF
         ENDDO
      ENDDO ! LOOP OVER nsdm SUBDOMAINS

! POST SENDS FOR THE POLES
      DO pole = 1,12
         IF (my_task.EQ.sbdmn_owner(level)%owner(wrapPOLE(2,pole,level))) THEN
              i1 = wrapPOLE(7,pole,level)
              j1 = wrapPOLE(8,pole,level)
            nsd1 = wrapPOLE(9,pole,level)
            IF (wrapPOLE(3,pole,level).EQ.0) THEN
                 i2 = wrapPOLE(4,pole,level)
                 j2 = wrapPOLE(5,pole,level)
               nsd2 = wrapPOLE(6,pole,level)
               xtem(i2,j2,:,nsd2) = xtem(i2,j2,:,nsd2) + x(i1,j1,:,nsd1)
            ELSE
               pole_dest = sbdmn_owner(level)%owner(wrapPOLE(1,pole,level))
               pole_tag  =                       wrapPOLE(3,pole,level)
               buffer_send_pole(:,pole) = x(i1,j1,:,nsd1)
#ifdef mpi
!              IF (npe_atmos > 1) THEN
                  count_send_req = count_send_req + 1
                  CALL MPI_ISEND (buffer_send_pole(1,pole),pole_size,        
     &                      mpi_float_type,pole_dest,pole_tag,                
     &                      mpi_comm_atmos,send_req(count_send_req),ierr)
!              ENDIF
#endif
            ENDIF
         ENDIF
      ENDDO

#ifdef mpi
!     IF (npe_atmos > 1) THEN

         ALLOCATE (send_status(MPI_STATUS_SIZE,count_send_req))
         ALLOCATE (recv_status(MPI_STATUS_SIZE,count_recv_req))
         send_status(:,:) = 0
         recv_status(:,:) = 0

         CALL MPI_WAITALL (count_send_req,send_req,send_status,ierr)
         CALL MPI_WAITALL (count_recv_req,recv_req,recv_status,ierr)

! UNPACK BUFFER FOR EDGE AND DIAGONAL MESSAGES
         DO nsd = 1,nsdm_t
            sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd)
            DO edge = 1,4
               IF (wrapSEND(level)%wS(2,edge,sbdmn_number).NE.0) THEN
                  i_start = wrapSEND(level)%wS(3,edge,sbdmn_number)
                  j_start = wrapSEND(level)%wS(4,edge,sbdmn_number)
                  di      = wrapSEND(level)%wS(5,edge,sbdmn_number)
                  dj      = wrapSEND(level)%wS(6,edge,sbdmn_number)
                  DO count = 0,iim_t-3
                     i = i_start+di*count
                     j = j_start+dj*count
                     xtem(i,j,:,nsd) = xtem(i,j,:,nsd) + 
     &                   buffer_recv_edge(count+1,:,edge,nsd)
                  ENDDO
               ENDIF
            ENDDO

            DO diag = 5,6
               IF ((wrapSEND(level)%wS(1,diag,sbdmn_number).NE.0).AND.       
     &              (wrapSEND(level)%wS(2,diag,sbdmn_number).NE.0)) THEN
                  i = wrapSEND(level)%wS(3,diag,sbdmn_number)
                  j = wrapSEND(level)%wS(4,diag,sbdmn_number)
               xtem(i,j,:,nsd) = xtem(i,j,:,nsd) + buffer_recv_diag(:,diag,nsd)
               ENDIF
            ENDDO
         ENDDO ! LOOP OVER nsdm SUBDOMAINS

! UNPACK BUFFER FOR POLE MESSAGES
         DO pole = 1,12
            IF ((my_task.EQ.                                                 
     &             sbdmn_owner(level)%owner(wrapPOLE(1,pole,level))).AND.     
     &                                (wrapPOLE(3,pole,level).NE.0)) THEN
               i = wrapPOLE(4,pole,level)
               j = wrapPOLE(5,pole,level)
               nsd = wrapPOLE(6,pole,level)
               xtem(i,j,:,nsd) = xtem(i,j,:,nsd) + buffer_recv_pole(:,pole)
            ENDIF
         ENDDO
!     ENDIF
#endif

! SET DEAD CELLS TO A REPRESENTATIVE VALUE
      isdm_t = isdm_level(level)
      jsdm_t = jsdm_level(level)
      sd_np = isdm_t*(  jsdm_t-1)+1
      sd_sp = isdm_t*(4*jsdm_t+1)
      DO nsd = 1,sbdmn_assign(my_task+1,level)%length
         sbdmn_number = sbdmn_assign(my_task+1,level)%sd(nsd)
         IF ((sbdmn_number.NE.sd_np).AND.(sbdmn_number.NE.sd_sp)) THEN
            xtem(iim_t,1,:,nsd) = x(ipad+1,jpad+1,:,nsd)
            xtem(1,jjm_t,:,nsd) = xtem(ipad+1,jpad+1,:,nsd)
         ENDIF
         IF (Mod (sbdmn_number-1,isdm_t*jsdm_t)+1.EQ.
     &                                        isdm_t*jsdm_t-jsdm_t) THEN
            xtem(iim_t,jjm_t,:,nsd) = xtem(ipad+1,jpad+1,:,nsd)
         ENDIF
         IF (Mod (sbdmn_number-1,isdm_t*jsdm_t)+1.EQ.
     &                                          isdm_t*jsdm_t     ) THEN
            xtem(iim_t,jjm_t,:,nsd) = xtem(ipad+1,jpad+1,:,nsd)
         ENDIF
      ENDDO

      x(:,:,:,1:nsdm_t) = x(:,:,:,1:nsdm_t) + xtem(:,:,:,1:nsdm_t)

      DEALLOCATE (buffer_send_edge,buffer_send_diag,buffer_send_pole)
      DEALLOCATE (buffer_recv_edge,buffer_recv_diag,buffer_recv_pole)
      DEALLOCATE (send_req,recv_req)
#ifdef mpi
!     IF (npe_atmos > 1) THEN
         DEALLOCATE (send_status,recv_status)
!     ENDIF
#endif


      END SUBROUTINE wrap_fc
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      END MODULE wrap_face_data
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
