      module qp_module

!  This file initializes, stores, computes and writes the diagnostic output
!     that is global and time averaged. The short-hand term for these
!     diagnostics is 'qp'. QP3 are 3 dimensional diagnostics, QP2 are
!     2 dimensional (horizontal) only.

!  The 6 hourly high-frequency amip variables have been added here since
!     their structure is similar to the qps. They  have analogs in
!     virtually every way to the qp2s that the qp2 comments apply. These
!     variables are called hf.

      use kinds
      use model_parameters
      use domain_decomposition
      use sigma_coordinate
      use tracer_names
      use netcdf_geodesic
      use time_manager_module
      use gather_scatter
      use grid_metrics
      use wrap_vertex_data

      implicit none
      save
      private

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!	PROCEDURE TO ADD A QP VARIABLE TO THE MODEL
!
!       Adding a QP3 (3-dimensional field)
!   	In this file:
!	1. Increase Nqp3 by 1
!	2. Add new entries in the tables for
!          -a variable named iqp3_xxx, where xxx is the name in qp3_name.
!               Add this to the list of iqp3 variables and assign it a value
!               1 larger than the previous one.
!       3. Go to subroutine initialize_qpinfo further down in this file.
!          Add an assignment statement for each of these variables for
!          index iqp3_xxx (these are down in a block):
!  	   -qp3_name (the name of the field conforming, if possible, to 
!          	amip conventions
!          -qp3_units ( the units of the field)
!          -qp3_descr ( A longer description of the field)
!	In the file where the variable resides
!	4. Be sure that a "use qp_module" statement is in the subroutine
!       5. Add a qp3sum statement for the field you wish to store.
!	The description of the qp3sum argument list is in the file with 
!       subroutine qp3sum
!	6. If the added field is a product for which primes are desired
!          a product for the bars must be added also. The names should bear
!          the tags _prime and _bar. Look for the ua_ta_prime example for
!          the other fields that must be added with this one.
!
!       Adding a QP2 (or hf2) (horizontal only field)
!	Use the instructions above, replacing qp3 with qp2 (hf2)
!
!       Hints:
!       -If the field you want to save is a linear combination of model
!        variables you can use multiple qp*sum calls with the appropriate 
!        weights.
!        Example - reflected TOA SW is swinc - dswtop. Use a qp2sum call
!        for swinc with a weight of 1.0 and a call for dswtop with a weight
!        of -1.0
!        NOTE: if you use multiple qp2sum calls to save a diagnostics, all
!          but one call must include the step_sample argument.
!       -If the field you want to save is in a subroutine that is not called
!        every timestep, the weight should include a factor to compensate for
!        that.
!        Example - radiation is called every physics timestep, once every
!        n_physics timesteps. Include a factor of n_physics in the weight
!       -hardwired constants in the argument list must be of the right kind
!        Example - a weight of 1.0 should be written 1.0_dbl_kind
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!  tables of qp information

!   number of installed available qps, change these as more qps are added
      integer (kind=int_kind), parameter, public :: 
     *                                      nqp3  = 133+3*(ntracer-nwater)
     *,                                     nqp2  = 195+3*(ntracer-nwater)
     *,                                     nhf2  = 42
     *,                                     nmjo2 = 16
     *,                                     nmjo3 = 10

!   names are the variable names and conform to amip standards where possible
!   these are assigned in subroutine initialize_qpinfo further down
      character (len=16), dimension(nqp2), public :: qp2_name
      character (len=16), dimension(nqp3), public :: qp3_name
      character (len=16), dimension(nhf2), public :: hf2_name
      character (len=16), dimension(nmjo2), public :: mjo2_name
      character (len=16), dimension(nmjo3), public :: mjo3_name

!  units are the units the fields are stored in and are mks where possible
!   these are assigned in subroutine initialize_qpinfo further down
      character (len=16), dimension(nqp2), public :: qp2_units 
      character (len=16), dimension(nqp3), public :: qp3_units
      character (len=16), dimension(nhf2), public :: hf2_units
      character (len=16), dimension(nmjo2), public :: mjo2_units
      character (len=16), dimension(nmjo3), public :: mjo3_units
      character (len=16), dimension(nqp2), public :: qp2_altunits 

!  descr is a description of the field
!   these are assigned in subroutine initialize_qpinfo further down
      character (len=80), dimension(nqp2), public :: qp2_descr
      character (len=80), dimension(nqp3), public :: qp3_descr
      character (len=80), dimension(nhf2), public :: hf2_descr
      character (len=80), dimension(nmjo2), public :: mjo2_descr
      character (len=80), dimension(nmjo3), public :: mjo3_descr

!  type denotes which type of variable this is (for ezplot)
      character (len=3), dimension(nqp2), public :: qp2_type
      character (len=3), dimension(nqp3), public :: qp3_type
      character (len=3), dimension(nhf2), public :: hf2_type

      integer (kind=int_kind), dimension(nqp2), public :: qp2_indx
      integer (kind=int_kind), dimension(nqp3), public :: qp3_indx
      logical (kind=int_kind), dimension(nqp3), public :: qp3_prodlog = .false.
      integer (kind=int_kind), dimension(nqp3,2), public :: qp3_prodindx
      integer (kind=int_kind), dimension(nhf2), public :: hf2_indx
      integer (kind=int_kind), dimension(nmjo2), public :: mjo2_indx
      integer (kind=int_kind), dimension(nmjo3), public :: mjo3_indx
      logical (kind=log_kind), dimension(nqp2), public :: qp2_log = .false.
      logical (kind=log_kind), dimension(nqp3), public :: qp3_log = .false.
      logical (kind=log_kind), dimension(nhf2), public :: hf2_log = .false.
      logical (kind=log_kind), dimension(nmjo2), public :: mjo2_log = .false.
      logical (kind=log_kind), dimension(nmjo3), public :: mjo3_log = .false.
      logical (kind=log_kind), dimension(nqp2) :: qp2_amip_sampling = .false.
      logical (kind=log_kind), dimension(nqp3) :: qp3_amip_sampling = .false.
      logical (kind=log_kind), dimension(nhf2) :: hf2_amip_sampling = .false.
      logical (kind=log_kind), dimension(nmjo2) :: mjo2_amip_sampling = .false.
      logical (kind=log_kind), dimension(nmjo3) :: mjo3_amip_sampling = .false.
      integer (kind=int_kind), dimension(nqp2) :: qp2_mjo_link = 0
      integer (kind=int_kind), dimension(nqp3) :: qp3_mjo_link = 0

! these will point to  locations in the qp information tables
! their values are assigned sequentially, and the last value is used
!   as the dimension of the above tables
      integer (kind=int_kind) :: i
      integer (kind=int_kind), parameter :: nqp_passive = max(1,ntracer-nwater)
      integer (kind=int_kind), parameter, public ::
     *     iqp2_blent	   = 1
     *,    iqp2_clblp	   = 2
     *,    iqp2_blp	   = 3
     *,    iqp2_rls	   = 4
     *,    iqp2_rss	   = 5
     *,    iqp2_rst	   = 6
     *,    iqp2_rlut	   = 7
     *,    iqp2_rsdt	   = 8
     *,    iqp2_spdm	   = 9
     *,    iqp2_rlds	   = 10
     *,    iqp2_ps	   = 11
     *,    iqp2_huss	   = 12
     *,    iqp2_clbl	   = 13
     *,    iqp2_rsds	   = 14
     *,    iqp2_dryconvinc = 15
     *,    iqp2_hfss       = 16
     *,    iqp2_ts	   = 17
     *,    iqp2_psl	   = 18
     *,    iqp2_tas	   = 19
     *,    iqp2_hfls	   = 20
     *,    iqp2_rsus	   = 21
     *,    iqp2_rlus	   = 22
     *,    iqp2_rsut	   = 23
     *,    iqp2_rsdscs	   = 24
     *,    iqp2_rsuscs	   = 25
     *,    iqp2_rldscs	   = 26
     *,    iqp2_rlutcs	   = 27
     *,    iqp2_rsutcs	   = 28
     *,    iqp2_clt	   = 29
     *,    iqp2_ttrop	   = 30
     *,    iqp2_evspsbl	   = 31
     *,    iqp2_rtmt	   = 32
     *,    iqp2_tauu	   = 33
     *,    iqp2_tauv	   = 34
     *,    iqp2_uas	   = 35
     *,    iqp2_vas	   = 36
     *,    iqp2_tauugwd	   = 37
     *,    iqp2_tauvgwd	   = 38
     *,    iqp2_tasmin	   = 39
     *,    iqp2_tasmax	   = 40
     *,    iqp2_psbmin	   = 41
     *,    iqp2_psbmax	   = 42
     *,    iqp2_leakp	   = 43
     *,    iqp2_leakpinc   = 44
     *,    iqp2_leakn	   = 45
     *,    iqp2_leakninc   = 46
     *,    iqp2_pbldmf	   = 47
     *,    iqp2_fillvq     = 48
     *,    iqp2_fillvcw    = 49
     *,    iqp2_fillvci    = 50
     *,    iqp2_fillvrw    = 51
     *,    iqp2_fillvri    = 52
     *,    iqp2_filliq     = 53
     *,    iqp2_fillicw    = 54
     *,    iqp2_fillici    = 55
     *,    iqp2_fillirw    = 56
     *,    iqp2_filliri    = 57
     *,    iqp2_filllq     = 58
     *,    iqp2_filllcw    = 59
     *,    iqp2_filllci    = 60
     *,    iqp2_filllrw    = 61
     *,    iqp2_filllri    = 62
     *,    iqp2_ke         = 63
     *,    iqp2_en         = 64
     *,    iqp2_cupinc     = 65
     *,    iqp2_cupincpbl  = 66
     *,    iqp2_cupinc1    = 67
     *,    iqp2_cupinc2    = 68
     *,    iqp2_cupinc3    = 69
     *,    iqp2_cupinc4    = 70
     *,    iqp2_cupinc5    = 71
     *,    iqp2_cupinc6    = 72
     *,    iqp2_cupinc7    = 73
     *,    iqp2_cupinc8    = 74
     *,    iqp2_cupinc9    = 75
     *,    iqp2_cupinc10   = 76
     *,    iqp2_cupinc11   = 77
     *,    iqp2_cupinc12   = 78
     *,    iqp2_cupinc13   = 79
     *,    iqp2_cupinc14   = 80
     *,    iqp2_cupinc15   = 81
     *,    iqp2_cupinc16   = 82
     *,    iqp2_prc        = 83
     *,    iqp2_cupcos1    = 84
     *,    iqp2_cupcos2    = 85
     *,    iqp2_cupsin1    = 86
     *,    iqp2_cupsin2    = 87
     *,    iqp2_cupprpbl   = 88
     *,    iqp2_cupcke     = 89
     *,    iqp2_cupcdwork  = 90
     *,    iqp2_cupmb      = 91
     *,    iqp2_cupcape    = 92
     *,    iqp2_dlcupt     = 93
     *,    iqp2_dicupt     = 94
     *,    iqp2_totdlcupt  = 95
     *,    iqp2_totdicupt  = 96
     *,    iqp2_prw        = 97
     *,    iqp2_pblprw     = 98
     *,    iqp2_pblcw      = 99

      integer (kind=int_kind), parameter, public ::
     *     iqp2_pblci      = 100
     *,    iqp2_clwvi      = 101
     *,    iqp2_elqrwt     = 102
     *,    iqp2_clivi      = 103
     *,    iqp2_elqrit     = 104
     *,    iqp2_pblclwvi   = 105
     *,    iqp2_pblelqrwt  = 106
     *,    iqp2_pblclivi   = 107
     *,    iqp2_pblelqrit  = 108
     *,    iqp2_lpblclwvi  = 109
     *,    iqp2_lpblelqrwt = 110
     *,    iqp2_lpblclivi  = 111
     *,    iqp2_lpblelqrit = 112
     *,    iqp2_elqprr     = 113
     *,    iqp2_prsn       = 114
     *,    iqp2_elqpr      = 115
     *,    iqp2_elqcos1    = 116
     *,    iqp2_elqcos2    = 117
     *,    iqp2_elqsin1    = 118
     *,    iqp2_elqsin2    = 119
     *,    iqp2_pr         = 120
     *,    iqp2_prcos1     = 121
     *,    iqp2_prcos2     = 122
     *,    iqp2_prsin1     = 123
     *,    iqp2_prsin2     = 124
     *,    iqp2_elqacld    = 125
     *,    iqp2_meltlev    = 126
     *,    iqp2_press20C   = 127
     *,    iqp2_press40C   = 128
     *,    iqp2_cldhigh    = 129
     *,    iqp2_cldmid     = 130
     *,    iqp2_cldlow     = 131
     *,    iqp2_cldt2d     = 132
     *,    iqp2_cldw2d     = 133
     *,    iqp2_cldi2d     = 134
     *,    iqp2_cldm2d     = 135
     *,    iqp2_ncw        = 136
     *,    iqp2_cwlm1      = 137
     *,    iqp2_effrw      = 138
     *,    iqp2_cil5       = 139
     *,    iqp2_effri      = 140
     *,    iqp2_lwnetsfccl = 141
     *,    iqp2_lwplanck   = 142
     *,    iqp2_lwtom      = 143
     *,    iqp2_lwtomcl    = 144
     *,    iqp2_lwpblfu    = 145
     *,    iqp2_lwpblfd    = 146
     *,    iqp2_swnettoacl = 147
     *,    iqp2_swnetsfccl = 148
     *,    iqp2_swtom      = 149
     *,    iqp2_swtomcl    = 150
     *,    iqp2_swradvdc   = 151
     *,    iqp2_swradndc   = 152
     *,    iqp2_swradvbc   = 153
     *,    iqp2_swradnbc   = 154
     *,    iqp2_swradvdcc  = 155
     *,    iqp2_swradndcc  = 156
     *,    iqp2_swradvbcc  = 157
     *,    iqp2_swradnbcc  = 158
     *,    iqp2_swcosz     = 159
     *,    iqp2_swsfac     = 160
     *,    iqp2_swalvdr    = 161
     *,    iqp2_swalvdf    = 162
     *,    iqp2_swalndr    = 163
     *,    iqp2_swalndf    = 164
     *,    iqp2_codtwvis   = 165
     *,    iqp2_codtivis   = 166
     *,    iqp2_codtmvis   = 167
     *,    iqp2_codtvis    = 168
     *,    iqp2_codtwirw   = 169
     *,    iqp2_codtiirw   = 170
     *,    iqp2_codtmirw   = 171
     *,    iqp2_codtirw    = 172
     *,    iqp2_fhcld1     = 173
     *,    iqp2_fhcld2     = 174
     *,    iqp2_fhcld3     = 175
     *,    iqp2_fhcld4     = 176
     *,    iqp2_fhcld5     = 177
     *,    iqp2_fmcld1     = 178
     *,    iqp2_fmcld2     = 179
     *,    iqp2_fmcld3     = 180
     *,    iqp2_fmcld4     = 181
     *,    iqp2_fmcld5     = 182
     *,    iqp2_flcld1     = 183
     *,    iqp2_flcld2     = 184
     *,    iqp2_flcld3     = 185
     
      integer (kind=int_kind), parameter, public ::
     *     iqp2_flcld4     = 186
     *,    iqp2_flcld5     = 187
     *,    iqp2_psbsmth    = 188
     *,    iqp2_psblapl    = 189
     *,    iqp2_hurs       = 190
     *,    iqp2_dzlrprec   = 191
     *,    iqp2_dzlsprec   = 192
     *,    iqp2_deaultp    = 193
     *,    iqp2_deauliq    = 194
     *,    iqp2_evaptend   = 195
      integer (kind=int_kind), parameter, dimension(nqp_passive),public::
     *     iqp2_passive    = (/(195+i,i=1,nqp_passive)/)
      integer (kind=int_kind), parameter, dimension(nqp_passive),public::
     *     iqp2_passivepbl = (/(195+i,i=nqp_passive+1,2*nqp_passive)/)
      integer (kind=int_kind), parameter, dimension(nqp_passive),public::
     *     iqp2_tracersrc  = (/(195+i,i=2*(nqp_passive)+1,3*nqp_passive)/)

      integer (kind=int_kind), parameter, public ::
     *     iqp3_mask	     = 1
     *,    iqp3_maskamip     = 2
     *,    iqp3_zg	     = 3
     *,    iqp3_ta	     = 4
     *,    iqp3_vorticity    = 5
     *,    iqp3_divergence   = 6
     *,    iqp3_tntsw	     = 7
     *,    iqp3_tntlw	     = 8
     *,    iqp3_hur	     = 9
     *,    iqp3_tntdc	     = 10
     *,    iqp3_qsat         = 11
     *,    iqp3_o3           = 12
     *,    iqp3_tntswcs      = 13
     *,    iqp3_tntlwcs      = 14
     *,    iqp3_ua           = 16
     *,    iqp3_va           = 17
     *,    iqp3_wap          = 18
     *,    iqp3_pressure     = 19
     *,    iqp3_psbg         = 20
     *,    iqp3_tnqmc        = 21
     *,    iqp3_tnqdc        = 22
     *,    iqp3_fill3q       = 23
     *,    iqp3_fill3cw      = 24
     *,    iqp3_fill3ci      = 25
     *,    iqp3_fill3rw      = 26
     *,    iqp3_fill3ri      = 27
     *,    iqp3_cvuva        = 28
     *,    iqp3_uva_bar      = 29
     *,    iqp3_cvuta        = 30
     *,    iqp3_uta_bar      = 31
     *,    iqp3_cvvta        = 32
     *,    iqp3_vta_bar      = 33
     *,    iqp3_cvwapta      = 34
     *,    iqp3_wapta_bar    = 35
     *,    iqp3_cvvzeta      = 36
     *,    iqp3_vzeta_bar    = 37
     *,    iqp3_cvuhusa      = 38
     *,    iqp3_uhusa_bar    = 39
     *,    iqp3_cvvhusa      = 40
     *,    iqp3_vhusa_bar    = 41
     *,    iqp3_cvwhusa      = 42
     *,    iqp3_whusa_bar    = 43
     *,    iqp3_ke           = 44
     *,    iqp3_en           = 45
     *,    iqp3_cke          = 46
     *,    iqp3_cdwork       = 47
     *,    iqp3_masflx       = 48
     *,    iqp3_tntmc        = 49
     *,    iqp3_tnmrc        = 50
     *,    iqp3_dhcup        = 51
     *,    iqp3_dvcup        = 52
     *,    iqp3_dlcup        = 53
     *,    iqp3_drcup        = 54
     *,    iqp3_dicup        = 55
     *,    iqp3_dscup        = 56
     *,    iqp3_dmcup        = 57        
     *,    iqp3_dthcup_ng    = 58
     *,    iqp3_dvcup_ng     = 59
     *,    iqp3_drcup_ng     = 60
     *,    iqp3_dscup_ng     = 61
     *,    iqp3_tcuph        = 62
     *,    iqp3_tcupv        = 63
     *,    iqp3_tcupl        = 64
     *,    iqp3_tcupr        = 65
     *,    iqp3_tcupi        = 66
     *,    iqp3_tcups        = 67
     *,    iqp3_tcupm        = 68
     *,    iqp3_tcupth_cld   = 69
     *,    iqp3_tcupth_clr   = 70
     *,    iqp3_tcupv_cld    = 71
     *,    iqp3_tcupv_clr    = 72
     *,    iqp3_tcupr_cld    = 73
     *,    iqp3_tcupr_clr    = 74
     *,    iqp3_tcups_cld    = 75
     *,    iqp3_tcups_clr    = 76
     *,    iqp3_clw          = 77
     *,    iqp3_rwelq        = 78
     *,    iqp3_cli          = 79
     *,    iqp3_rielq        = 80
     *,    iqp3_hus          = 81
     *,    iqp3_tntlsp       = 82
     *,    iqp3_dqelq        = 83
     *,    iqp3_dvelq        = 84
     *,    iqp3_dlelq        = 85
     *,    iqp3_drelq        = 86
     *,    iqp3_dielq        = 87
     *,    iqp3_dselq        = 88
     *,    iqp3_drwfelq      = 89
     *,    iqp3_drifelq      = 90
     *,    iqp3_cctpelq      = 91
     *,    iqp3_ccwvelq      = 92
     *,    iqp3_ccrwelq      = 93
     *,    iqp3_ccrielq      = 94
     *,    iqp3_ccvvelq      = 95
     *,    iqp3_acldelq      = 96
     *,    iqp3_acldtelq     = 97
     *,    iqp3_dcctpelq     = 98
     *,    iqp3_dccwvelq     = 99
      integer (kind=int_kind), parameter, public ::     
     *     iqp3_dccrwelq     = 100
     *,    iqp3_dccrielq     = 101
     *,    iqp3_dccvvelq     = 102
     *,    iqp3_dacldelq     = 103    
     *,    iqp3_cld          = 104
     *,    iqp3_clu          = 105
     *,    iqp3_cl           = 106
     *,    iqp3_cldw         = 107
     *,    iqp3_cldi         = 108
     *,    iqp3_cldm         = 109
     *,    iqp3_effrw        = 110
     *,    iqp3_effri        = 111
     *,    iqp3_codwvis      = 112
     *,    iqp3_codivis      = 113
     *,    iqp3_codmvis      = 114
     *,    iqp3_codtvis      = 115
     *,    iqp3_codwirw      = 116
     *,    iqp3_codiirw      = 117
     *,    iqp3_codmirw      = 118
     *,    iqp3_codtirw      = 119
     *,    iqp3_cvvzga       = 120
     *,    iqp3_vzga_bar     = 121
     *,    iqp3_cvuua        = 122
     *,    iqp3_uua_bar      = 123
     *,    iqp3_cvvva        = 124
     *,    iqp3_vva_bar      = 125
     *,    iqp3_cvtta        = 126
     *,    iqp3_tta_bar      = 127
     *,    iqp3_tnt          = 128
     *,    iqp3_tnmrd        = 129
     *,    iqp3_tnmmuc       = 130
     *,    iqp3_tnmmvc       = 131
     *,    iqp3_tnmmugwd       = 132
     *,    iqp3_tnmmvgwd       = 133
      integer (kind=int_kind), parameter, dimension(nqp_passive), public ::
     *     iqp3_passive = (/ (133+i,i=1,nqp_passive) /)
      integer (kind=int_kind), parameter, dimension(nqp_passive), public ::
     *     iqp3_dtrccup = (/ (133+i,i=nqp_passive+1,2*nqp_passive)/)
      integer (kind=int_kind), parameter, dimension(nqp_passive), public ::
     *     iqp3_tcuptrc = (/ (133+i,i=2*nqp_passive+1,3*nqp_passive)/)

      integer (kind=int_kind), parameter, public ::
     *     ihf2_va850	= 1
     *,    ihf2_va200	= 2
     *,    ihf2_ua850	= 3
     *,    ihf2_ua200	= 4
     *,    ihf2_rlut	= 5
     *,    ihf2_psl	= 6
     *,    ihf2_tasmax	= 7
     *,    ihf2_tasmin	= 8
     *,    ihf2_prc	= 9
     *,    ihf2_elqpr   = 10
     *,    ihf2_pr      = 11
     *,    ihf2_ke 	= 12
     *,    ihf2_ke250	= 13
     *,    ihf2_en250	= 14
     *,    ihf2_en   	= 15
     *,    ihf2_ta850	= 16
     *,    ihf2_ta500	= 17
     *,    ihf2_ta50 	= 18
     *,    ihf2_zg500  	= 19
     *,    ihf2_hus850	= 20
     *,    ihf2_hus500	= 21
     *,    ihf2_uas   	= 22
     *,    ihf2_vas   	= 23
     *,    ihf2_tas  	= 24
     *,    ihf2_huss 	= 25
     *,    ihf2_wap500 	= 26
     *,    ihf2_va50 	= 27
     *,    ihf2_ua50 	= 28
     *,    ihf2_ps  	= 29
     *,    ihf2_clt   	= 30
     *,    ihf2_tauu 	= 31
     *,    ihf2_tauv 	= 32
     *,    ihf2_prw  	= 33
     *,    ihf2_hfss  	= 34
     *,    ihf2_hfls 	= 35
     *,    ihf2_rsds 	= 36
     *,    ihf2_rsus 	= 37
     *,    ihf2_rlds 	= 38
     *,    ihf2_rlus 	= 39
     *,    ihf2_rsdt 	= 40
     *,    ihf2_rsut 	= 41
     *,    ihf2_zblh 	= 42

      integer (kind=int_kind), parameter, public ::
     *     imjo2_rlut	= 1
     *,    imjo2_chi200	= 2
     *,    imjo2_elqwvt = 3
     *,    imjo2_cuppr	= 4
     *,    imjo2_elqpr	= 5
     *,    imjo2_evap	= 6
     *,    imjo2_hfss	= 7
     *,    imjo2_rss	= 8
     *,    imjo2_rls	= 9
     *,    imjo2_huss	= 10
     *,    imjo2_tas	= 11
     *,    imjo2_ts	= 12
     *,    imjo2_blp	= 13
     *,    imjo2_uas	= 14
     *,    imjo2_vas	= 15
     *,    imjo2_ps	= 16

      integer (kind=int_kind), parameter, public ::
     *     imjo3_mask	= 1
     *,    imjo3_wap	= 2
     *,    imjo3_ua	= 3
     *,    imjo3_va	= 4
     *,    imjo3_tntlat	= 5
     *,    imjo3_tnqlat	= 6
     *,    imjo3_ta	= 7
     *,    imjo3_hus	= 8
     *,    imjo3_tntrad	= 9
     *,    imjo3_zg	= 10

!   number of saved fields
      integer (kind=int_kind), public :: iqp3 = 0, iqp2 = 0, ihf2 = 0,
     *                                   imjo3 = 0, imjo2 = 0, iqp3prod = 0

!   number of samples
      integer (kind=int_kind), dimension(nqp2), public :: iqp_sample
      integer (kind=int_kind), dimension(nhf2), public :: ihf_sample
      integer (kind=int_kind), dimension(nmjo2), public :: imjo2_sample

!   multiplier attribute
      real (kind=dbl_kind), dimension(nqp2), public :: qp2_multfac
      real (kind=dbl_kind), dimension(nqp2), public :: qp2_contmin
      real (kind=dbl_kind), dimension(nqp2), public :: qp2_contint

!   time at start of summation
      integer (kind=int_kind), public :: nsecondinit

!   qp data slots
      real (kind=real_kind), allocatable, dimension(:,:,:,:,:), public :: 
     *                 qp3
      real (kind=dbl_kind), allocatable, dimension(:,:,:,:,:), public :: 
     *                 mjo3
      real (kind=real_kind), allocatable, dimension(:,:,:,:), public ::
     *                 qp2, hf2
      real (kind=dbl_kind), allocatable, dimension(:,:,:,:), public ::
     *                 mjo2

!   pressure levels for vertical interpolation to isobaric surface
#ifdef no_stratosphere
      integer (kind=int_kind), parameter, public :: mm = 17
      real (kind=dbl_kind), dimension(mm), public ::
     * pm = (/ 1000.0, 2000.0, 3000.0, 5000.0, 7000.0, 10000.0
     *,        15000.0, 20000.0, 25000.0
     *,        30000.0, 40000.0, 50000.0, 60000.0, 70000.0
     *,        85000.0, 92500.0, 100000.0 /)
#else
      integer (kind=int_kind), parameter, public :: mm = 21
      real (kind=dbl_kind), dimension(mm), public ::
     * pm = (/ 200.0,300.0,500.0,700.0
     *,        1000.0, 2000.0, 3000.0, 5000.0, 7000.0, 10000.0
     *,        15000.0, 20000.0, 25000.0
     *,        30000.0, 40000.0, 50000.0, 60000.0, 70000.0
     *,        85000.0, 92500.0, 100000.0 /)
#endif
      real (kind=dbl_kind), dimension(mm+1) :: pme

!   pressure levels for vertical interpolation to isentropic surface
      real (kind=dbl_kind), dimension(mm) ::
     * tm 

!  vertical interpolation type
      integer (kind=int_kind) ::
     *         qpterptype = 1  ! 1 for isobaric, 2 for isentropic

!  vertical interpolation variables
      real (kind=dbl_kind), dimension(iimjjm,mm,nsdm), public ::
     *    pm3, pmk3, pfa, pfb
      integer (kind=int_kind), dimension(iimjjm,mm,nsdm) ::
     *    lbnd, lcnd
      real (kind=dbl_kind), dimension(iimjjm,mm+1,nsdm) ::
     *    pme3

!  underlying surface type
       real (kind=real_kind), dimension(iim,jjm,nsdm), public ::
     *           landfrac, seafrac, icefrac

!  indices for latlon interpolation
       integer (kind=int_kind), dimension(iim,jjm,nsdm), public ::
     *           interp_indx

!  unique points in domain
      integer (kind=int_kind), public :: qp_points

!  mjo file units
      integer (kind=int_kind) :: mjo2file=68, mjo3file=69

      public ::
     *  qp2sum, qp3sum, initialize_qpinfo, setqp_wrap, qp3sum_zg
     *, initialize_qpfiles,  write_qp_diagnostics, write_hf_diagnostics
     *, qp3sum_divvort, qp3sum_cld, hf2sum, write_mjo_diagnostics
     *, qp3_product_sum


    

      contains

      subroutine initialize_qpinfo

!  this subroutine assigns all the qp information from the input file atm_qpopts


      implicit none

      integer (kind=int_kind) :: ldummy, n, m, ioerr
      logical (kind=log_kind) :: doqptem
      character (len=16) :: nametem
      character (len=2) :: tnumber
      real (kind=dbl_kind) :: pmktem
      real (kind=dbl_kind), dimension(max_ig) :: indx_max_ig = 0.0
      real (kind=dbl_kind), dimension(im,jm) :: indx_imjm = 0.0
      real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: rinterp_indx = 0.0

! local variables
        integer (kind=int_kind) :: ic3d, icsum, i
        real (kind=dbl_kind) :: nb, psum

! initialize
      qp2_name(:) = " no name qp2"
      qp3_name(:) = " no name qp3"
      hf2_name(:) = " no name hf2"
      mjo2_name(:) = " no name mjo2"
      mjo3_name(:) = " no name mjo3"
      qp2_units(:) = "no units qp2"
      qp3_units(:) = "no units qp3"
      hf2_units(:) = "no units hf2"
      mjo2_units(:) = "no units mjo2"
      mjo3_units(:) = "no units mjo3"
      qp2_altunits(:) = "no altunits qp2"
      qp2_descr(:) = "no description qp2"
      qp3_descr(:) = "no description qp3"
      hf2_descr(:) = "no description hf2"
      mjo2_descr(:) = "no description mjo2"
      mjo3_descr(:) = "no description mjo3"
      qp2_type(:) = "no type qp2"
      qp3_type(:) = "no type qp3"
      hf2_type(:) = "no type hf2"
      qp2_multfac(:) = c1
      qp2_contmin(:) = 1.e36_dbl_kind
      qp2_contint(:) = 1.e36_dbl_kind
      
!  do assignments for variable name, units, description and type
      qp2_name (iqp2_blent ) = "blent"
        qp2_units(iqp2_blent ) = "kg/m^2/s"
        qp2_descr(iqp2_blent ) = "PBL top entrainment rate"
        qp2_type(iqp2_blent ) = "pbl"

        qp2_name (iqp2_clblp ) = "clblp"
        qp2_units(iqp2_clblp ) = "Pa"
        qp2_descr(iqp2_clblp ) = "Boundary layer cloud depth"
        qp2_type(iqp2_clblp ) = "pbl"

        qp2_name (iqp2_blp   ) = "blp"
        qp2_units(iqp2_blp   ) = "Pa"
        qp2_descr(iqp2_blp   ) = "PBL depth"
        qp2_mjo_link(iqp2_blp) = imjo2_blp
        qp2_type(iqp2_blp ) = "pbl"
        qp2_multfac(iqp2_blp ) = 0.01_dbl_kind
        qp2_contmin(iqp2_blp ) = 10.0_dbl_kind
        qp2_contint(iqp2_blp ) = 10.0_dbl_kind
        qp2_altunits(iqp2_blp   ) = "mb"

        qp2_name (iqp2_rls   ) = "rls"
        qp2_units(iqp2_rls   ) = "W/m^2"
        qp2_descr(iqp2_rls   ) = "RAD: SFC all-sky net longwave radiation"
        qp2_mjo_link(iqp2_rls) = imjo2_rls
        qp2_type(iqp2_rls ) = "rad"

        qp2_name (iqp2_rss   ) = "rss"
        qp2_units(iqp2_rss   ) = "W/m^2"
        qp2_descr(iqp2_rss   ) = "RAD: SFC all-sky net shortwave radiation"
        qp2_mjo_link(iqp2_rss) = imjo2_rss
        qp2_type(iqp2_rss ) = "rad"

        qp2_name (iqp2_rst   ) = "rst"
        qp2_units(iqp2_rst   ) = "W/m^2"
        qp2_descr(iqp2_rst   ) = "RAD: TOA all-sky net shortwave radiation"
        qp2_type(iqp2_rst ) = "rad"

        qp2_name (iqp2_rlut  ) = "rlut"
        qp2_units(iqp2_rlut  ) = "W/m^2"
        qp2_descr(iqp2_rlut  ) = "Outgoing longwave radiation"
        qp2_mjo_link(iqp2_rlut) = imjo2_rlut
        qp2_type(iqp2_rlut ) = "rad"

        qp2_name (iqp2_rsdt  ) = "rsdt"
        qp2_units(iqp2_rsdt  ) = "W/m^2"
        qp2_descr(iqp2_rsdt  ) = "TOA incident shortwave radiation"
        qp2_type(iqp2_rsdt ) = "rad"

        qp2_name (iqp2_spdm  ) = "spdm"
        qp2_units(iqp2_spdm  ) = "m/s"
        qp2_descr(iqp2_spdm  ) = "PBL wind speed"
        qp2_type(iqp2_spdm ) = "pbl"

        qp2_name (iqp2_rlds  ) = "rlds"
        qp2_units(iqp2_rlds  ) = "W/m^2"
        qp2_descr(iqp2_rlds  ) = "Surface downwelling longwave radiation"
        qp2_type(iqp2_rlds ) = "rad"

        qp2_name (iqp2_ps    ) = "ps"
        qp2_units(iqp2_ps    ) = "Pa"
        qp2_descr(iqp2_ps    ) = "Surface pressure"
	qp2_amip_sampling(iqp2_ps) = .true.
        qp2_mjo_link(iqp2_ps)  = imjo2_ps
        qp2_type(iqp2_ps ) = "dyn"

        qp2_name (iqp2_huss  ) = "huss"
        qp2_units(iqp2_huss  ) = "kg/kg"
        qp2_descr(iqp2_huss  ) = "Surface (2m) specific humidity"
	qp2_amip_sampling(iqp2_huss) = .true.
        qp2_mjo_link(iqp2_huss) = imjo2_huss
        qp2_type(iqp2_huss ) = "pbl"

        qp2_name (iqp2_clbl  ) = "clbl"
        qp2_units(iqp2_clbl  ) = "-"
        qp2_descr(iqp2_clbl  ) = "EAUcf: Incidence of PBL clouds"
        qp2_type(iqp2_clbl ) = "pbl"
        qp2_multfac(iqp2_clbl ) = 100._dbl_kind
        qp2_contmin(iqp2_clbl ) = 10.0_dbl_kind
        qp2_contint(iqp2_clbl ) = 10.0_dbl_kind
        qp2_altunits(iqp2_clbl) = "%"

        qp2_name (iqp2_rsds  ) = "rsds"
        qp2_units(iqp2_rsds  ) = "W/m^2"
        qp2_descr(iqp2_rsds  ) = "Surface incident shortwave radiation"
        qp2_type(iqp2_rsds ) = "rad"

        qp2_name (iqp2_dryconvinc) = "dryconvinc"
        qp2_units(iqp2_dryconvinc) = "-"
        qp2_descr(iqp2_dryconvinc) = "DRY CONVECTIVE ADJUSTMENT INCIDENCE"
        qp2_type(iqp2_dryconvinc ) = "dyn"

        qp2_name (iqp2_hfss  ) = "hfss"
        qp2_units(iqp2_hfss  ) = "W/m^2"
        qp2_descr(iqp2_hfss  ) = "Surface sensible heat flux"
        qp2_mjo_link(iqp2_hfss) = imjo2_hfss
        qp2_type(iqp2_hfss ) = "pbl"

        qp2_name (iqp2_ts) = "ts "
        qp2_units(iqp2_ts) = "K"
        qp2_descr(iqp2_ts) = "Ground temperature"
        qp2_mjo_link(iqp2_ts) = imjo2_ts
        qp2_type(iqp2_ts ) = "pbl"

        qp2_name (iqp2_psl   ) = "psl"
        qp2_units(iqp2_psl   ) = "Pa"
        qp2_descr(iqp2_psl   ) = "Mean-sea-level pressure"
	qp2_amip_sampling(iqp2_psl) = .true.
        qp2_type(iqp2_psl ) = "dyn"

        qp2_name (iqp2_tas   ) = "tas"
        qp2_units(iqp2_tas   ) = "K"
        qp2_descr(iqp2_tas   ) = "Mixed layer air temperature at surface"
	qp2_amip_sampling(iqp2_tas) = .true.
        qp2_mjo_link(iqp2_tas) = imjo2_tas
        qp2_type(iqp2_tas ) = "pbl"

        qp2_name (iqp2_hfls  ) = "hfls"
        qp2_units(iqp2_hfls  ) = "W/m^2"
        qp2_descr(iqp2_hfls  ) = "Surface latent heat flux"
        qp2_type(iqp2_hfls ) = "pbl"

        qp2_name (iqp2_rsus  ) = "rsus"
        qp2_units(iqp2_rsus  ) = "W/m^2"
        qp2_descr(iqp2_rsus  ) = "Surface reflected shortwave radiation"
        qp2_type(iqp2_rsus ) = "rad"

        qp2_name (iqp2_rlus  ) = "rlus"
        qp2_units(iqp2_rlus  ) = "W/m^2"
        qp2_descr(iqp2_rlus  ) = "Surface upwelling longwave radiation"
        qp2_type(iqp2_rlus ) = "rad"

        qp2_name (iqp2_rsut  ) = "rsut"
        qp2_units(iqp2_rsut  ) = "W/m^2"
        qp2_descr(iqp2_rsut  ) = "TOA reflected shortwave radiation"
        qp2_type(iqp2_rsut ) = "rad"

        qp2_name (iqp2_rsdscs) = "rsdscs"
        qp2_units(iqp2_rsdscs) = "W/m^2"
        qp2_descr(iqp2_rsdscs) = 
     &       "Surface incident clear-sky shortwave radiation"
        qp2_type(iqp2_rsdscs ) = "rad"

        qp2_name (iqp2_rsuscs) = "rsuscs"
        qp2_units(iqp2_rsuscs) = "W/m^2"
        qp2_descr(iqp2_rsuscs) = 
     &       "Surface reflected clear-sky shortwave radiation"
        qp2_type(iqp2_rsuscs ) = "rad"

        qp2_name (iqp2_rldscs) = "rldscs"
        qp2_units(iqp2_rldscs) = "W/m^2"
        qp2_descr(iqp2_rldscs) = 
     &       "Surface downwelling clear-sky longwave radiation"
        qp2_type(iqp2_rldscs ) = "rad"

        qp2_name (iqp2_rlutcs) = "rlutcs"
        qp2_units(iqp2_rlutcs) = "W/m^2"
        qp2_descr(iqp2_rlutcs) = 
     &       "TOA clear-sky outgoing longwave radiation"
        qp2_type(iqp2_rlutcs ) = "rad"

        qp2_name (iqp2_rsutcs) = "rsutcs"
        qp2_units(iqp2_rsutcs) = "W/m^2"
        qp2_descr(iqp2_rsutcs) = "TOA reflected clear-sky shortwave radiation"
        qp2_type(iqp2_rsutcs ) = "rad"

        qp2_name (iqp2_clt) = "clt"
        qp2_units(iqp2_clt) = "%"
        qp2_descr(iqp2_clt) = "Total cloud amount"
        qp2_type(iqp2_clt ) = "rad"

        qp2_name (iqp2_ttrop) = "ttrop"
        qp2_units(iqp2_ttrop) = "K"
        !qp2_descr(iqp2_ttrop) = "Tropopause temperature"
        qp2_descr(iqp2_ttrop) = "Cold-point temperature"
        qp2_type(iqp2_ttrop ) = "dyn"

        qp2_name (iqp2_evspsbl) = "evspsbl"
        qp2_units(iqp2_evspsbl) = "kg/m2/s"
        qp2_descr(iqp2_evspsbl) = "Surface evaporation plus sublimation rate"
        qp2_mjo_link(iqp2_evspsbl) = imjo2_evap
        qp2_type(iqp2_evspsbl) = "pbl"
        qp2_multfac(iqp2_evspsbl) = 86400._dbl_kind
        qp2_contmin(iqp2_evspsbl) = 10.0_dbl_kind
        qp2_contint(iqp2_evspsbl) = 10.0_dbl_kind
        qp2_altunits(iqp2_evspsbl) = "mm/day"

        qp2_name (iqp2_evaptend) = "evaptend"
        qp2_units(iqp2_evaptend) = "kg/kg/s"
        qp2_descr(iqp2_evaptend) = "Evaporation tendency"
        qp2_type(iqp2_evaptend ) = "pbl"

        qp2_name (iqp2_rtmt) = "rtmt"
        qp2_units(iqp2_rtmt) = "W/m2"
        qp2_descr(iqp2_rtmt) = "Net radiation at model top"
        qp2_type(iqp2_rtmt ) = "rad"

        qp2_name (iqp2_tauu) = "tauu"
        qp2_units(iqp2_tauu) = "N/m2"
        qp2_descr(iqp2_tauu) = "Eastward surface wind stress"
        qp2_type(iqp2_tauu ) = "pbl"

        qp2_name (iqp2_tauv) = "tauv"
        qp2_units(iqp2_tauv) = "N/m2"
        qp2_descr(iqp2_tauv) = "Northward surface wind stress"
        qp2_type(iqp2_tauv ) = "pbl"

        qp2_name (iqp2_uas) = "uas"
        qp2_units(iqp2_uas) = "m/s"
        qp2_descr(iqp2_uas) = "Eastward surface wind (10m)"
	qp2_amip_sampling(iqp2_uas) = .true.
        qp2_mjo_link(iqp2_uas) = imjo2_uas
        qp2_type(iqp2_uas ) = "pbl"

        qp2_name (iqp2_vas) = "vas"
        qp2_units(iqp2_vas) = "m/s"
        qp2_descr(iqp2_vas) = "Northward surface wind (10m)"
	qp2_amip_sampling(iqp2_vas) = .true.
        qp2_mjo_link(iqp2_vas) = imjo2_vas
        qp2_type(iqp2_vas ) = "pbl"

        qp2_name (iqp2_tauugwd) = "tauugwd"
        qp2_units(iqp2_tauugwd) = "N/m2"
        qp2_descr(iqp2_tauugwd) = "GWD induced eastward surface wind stress"
        qp2_type(iqp2_tauugwd ) = "dyn"

        qp2_name (iqp2_tauvgwd) = "tauvgwd"
        qp2_units(iqp2_tauvgwd) = "N/m2"
        qp2_descr(iqp2_tauvgwd) = "GWD induced northward surface wind stress"
        qp2_type(iqp2_tauvgwd ) = "dyn"

        qp2_name (iqp2_tasmin) = "tasmin"
        qp2_units(iqp2_tasmin) = "K"
        qp2_descr(iqp2_tasmin) = "Daily minimum surface (2m) air temperature"
        qp2_type(iqp2_tasmin ) = "pbl"

        qp2_name (iqp2_tasmax) = "tasmax"
        qp2_units(iqp2_tasmax) = "K"
        qp2_descr(iqp2_tasmax) = "Daily maximum surface (2m) air temperature"
        qp2_type(iqp2_tasmax ) = "pbl"

        qp2_name (iqp2_psbmin) = "psbmin"
        qp2_units(iqp2_psbmin) = "Pa"
        qp2_descr(iqp2_psbmin) = "Daily minimum PBL depth"
        qp2_type(iqp2_psbmin ) = "pbl"

        qp2_name (iqp2_psbmax) = "psbmax"
        qp2_units(iqp2_psbmax) = "Pa"
        qp2_descr(iqp2_psbmax) = "Daily maximum PBL depth"
        qp2_type(iqp2_psbmax ) = "pbl"

        qp2_name (iqp2_psbsmth) = "psbsmth"
        qp2_units(iqp2_psbsmth) = "kg/m2/s"
        qp2_descr(iqp2_psbsmth) = "pbl depth diffusion rate"
        qp2_type(iqp2_psbsmth ) = "dyn"

        qp2_name (iqp2_psblapl) = "psblapl"
        qp2_units(iqp2_psblapl) = "Pa/m2"
        qp2_descr(iqp2_psblapl) = "rms(pbl depth laplacian)"
        qp2_type(iqp2_psblapl ) = "dyn"

        qp2_name (iqp2_leakp) = "leakp"
        qp2_units(iqp2_leakp) = "kg/m2/s"
        qp2_descr(iqp2_leakp) = "positive pleak"
        qp2_type(iqp2_leakp ) = "pbl"

        qp2_name (iqp2_leakpinc) = "leakpinc"
        qp2_units(iqp2_leakpinc) = "-"
        qp2_descr(iqp2_leakpinc) = "positive pleak incidence"
        qp2_type(iqp2_leakpinc ) = "pbl"

        qp2_name (iqp2_leakn) = "leakn"
        qp2_units(iqp2_leakn) = "kg/m2/s"
        qp2_descr(iqp2_leakn) = "negative pleak"
        qp2_type(iqp2_leakn ) = "pbl"

        qp2_name (iqp2_leakninc) = "leakninc"
        qp2_units(iqp2_leakninc) = "-"
        qp2_descr(iqp2_leakninc) = "negative pleak incidence"
        qp2_type(iqp2_leakninc ) = "pbl"

        qp2_name (iqp2_pbldmf) = "pbldmf"
        qp2_units(iqp2_pbldmf) = "kg/m2/s"
        qp2_descr(iqp2_pbldmf) = "pbl mass flux divergence"
        qp2_type(iqp2_pbldmf ) = "pbl"

        qp2_name (iqp2_fillvq  ) = "fillvq"
        qp2_units(iqp2_fillvq  ) = "kg/m2/s"
        qp2_descr(iqp2_fillvq  ) = "vert. int. water vapor hole fill rate"
        qp2_type(iqp2_fillvq ) = "dyn"

        qp2_name (iqp2_fillvcw  ) = "fillvcw"
        qp2_units(iqp2_fillvcw  ) = "kg/m2/s"
        qp2_descr(iqp2_fillvcw  ) = "vert. int. cloud water hole fill rate"
        qp2_type(iqp2_fillvcw ) = "dyn"

        qp2_name (iqp2_fillvci  ) = "fillvci"
        qp2_units(iqp2_fillvci  ) = "kg/m2/s"
        qp2_descr(iqp2_fillvci  ) = "vert. int. cloud ice hole fill rate"
        qp2_type(iqp2_fillvci ) = "dyn"

        qp2_name (iqp2_fillvrw  ) = "fillvrw"
        qp2_units(iqp2_fillvrw  ) = "kg/m2/s"
        qp2_descr(iqp2_fillvrw  ) = "vert. int. rain hole fill rate"
        qp2_type(iqp2_fillvrw ) = "dyn"

        qp2_name (iqp2_fillvri  ) = "fillvri"
        qp2_units(iqp2_fillvri  ) = "kg/m2/s"
        qp2_descr(iqp2_fillvri  ) = "vert. int. snow hole fill rate"
        qp2_type(iqp2_fillvri ) = "dyn"

        qp2_name (iqp2_filliq  ) = "filliq"
        qp2_units(iqp2_filliq  ) = "-"
        qp2_descr(iqp2_filliq  ) = "water vapor hole fill incidence"
        qp2_type(iqp2_filliq ) = "dyn"

        qp2_name (iqp2_fillicw  ) = "fillicw"
        qp2_units(iqp2_fillicw  ) = "-"
        qp2_descr(iqp2_fillicw  ) = "cloud water hole fill incidence"
        qp2_type(iqp2_fillicw ) = "dyn"

        qp2_name (iqp2_fillici  ) = "fillici"
        qp2_units(iqp2_fillici  ) = "-"
        qp2_descr(iqp2_fillici  ) = "cloud ice hole fill incidence"
        qp2_type(iqp2_fillici ) = "dyn"

        qp2_name (iqp2_fillirw  ) = "fillirw"
        qp2_units(iqp2_fillirw  ) = "-"
        qp2_descr(iqp2_fillirw  ) = "rain hole fill incidence"
        qp2_type(iqp2_fillirw ) = "dyn"

        qp2_name (iqp2_filliri  ) = "filliri"
        qp2_units(iqp2_filliri  ) = "-"
        qp2_descr(iqp2_filliri  ) = "snow hole fill incidence"
        qp2_type(iqp2_filliri ) = "dyn"

        qp2_name (iqp2_filllq  ) = "filllq"
        qp2_units(iqp2_filllq  ) = "-"
        qp2_descr(iqp2_filllq  ) = 
     *     "water vapor hole fill incidence*layers filled"
        qp2_type(iqp2_filllq ) = "dyn"

        qp2_name (iqp2_filllcw  ) = "filllcw"
        qp2_units(iqp2_filllcw  ) = "-"
        qp2_descr(iqp2_filllcw  ) = 
     *     "cloud water hole fill incidence*layers filled"
        qp2_type(iqp2_filllcw ) = "dyn"

        qp2_name (iqp2_filllci  ) = "filllci"
        qp2_units(iqp2_filllci  ) = "-"
        qp2_descr(iqp2_filllci  ) = 
     *     "cloud ice hole fill incidence*layers filled"
        qp2_type(iqp2_filllci ) = "dyn"

        qp2_name (iqp2_filllrw  ) = "filllrw"
        qp2_units(iqp2_filllrw  ) = "-"
        qp2_descr(iqp2_filllrw  ) = 
     *     "rain hole fill incidence*layers filled"
        qp2_type(iqp2_filllrw ) = "dyn"

        qp2_name (iqp2_filllri  ) = "filllri"
        qp2_units(iqp2_filllri  ) = "-"
        qp2_descr(iqp2_filllri  ) = 
     *     "snow hole fill incidence*layers filled"
        qp2_type(iqp2_filllri ) = "dyn"

        qp2_name (iqp2_ke    ) = "ke"
        qp2_units(iqp2_ke    ) = "J/m2"
        qp2_descr(iqp2_ke    ) = "vertically integrated kinetic energy"
	  qp2_amip_sampling(iqp2_ke) = .true.
        qp2_type(iqp2_ke ) = "dyn"

        qp2_name (iqp2_en    ) = "en"
        qp2_units(iqp2_en    ) = "kg/m2/s2"
        qp2_descr(iqp2_en    ) = "vertically integrated enstrophy"
	  qp2_amip_sampling(iqp2_en) = .true.
        qp2_type(iqp2_en ) = "dyn"

#ifdef passive_tracers
        do n = 1,npassive
           if(n.le.9) then
              write(unit=tnumber,fmt='(i1)')n
           else
              write(unit=tnumber,fmt='(i2)')n
           endif
           qp2_name (iqp2_passive(n)) = "vtracer"//trim(tnumber)
           qp2_units(iqp2_passive(n)) = "kg/m2"
           qp2_descr(iqp2_passive(n)) = 
     &             "vertically integrated passive tracer "//trim(tnumber)
           qp2_type(iqp2_passive(n)) = "dyn"

           qp2_name (iqp2_passivepbl(n)) = "pbltracer"//trim(tnumber)
           qp2_units(iqp2_passivepbl(n)) = "kg/kg"
           qp2_descr(iqp2_passivepbl(n)) = 
     &             "pbl passive tracer "//trim(tnumber)
           qp2_type(iqp2_passivepbl(n)) = "dyn"

           qp2_name (iqp2_tracersrc(n)) = "tracersrc"//trim(tnumber)
           qp2_units(iqp2_tracersrc(n)) = "kg/m2/s"
           qp2_descr(iqp2_tracersrc(n)) = 
     &             "surface flux, passive tracer "//trim(tnumber)
           qp2_type(iqp2_tracersrc(n)) = "dyn"
        enddo
#endif

        qp2_name (iqp2_cupinc    ) = "cupinc"
        qp2_units(iqp2_cupinc    ) = "-"
        qp2_descr(iqp2_cupinc    ) =
     *   ":cup: incidence of convection"
        qp2_type(iqp2_cupinc ) = "cup"
        
        qp2_name (iqp2_cupincpbl ) = "cupincpbl"
        qp2_units(iqp2_cupincpbl ) = "-"
        qp2_descr(iqp2_cupincpbl ) =
     *   ":cup: incidence of convection starting at PBL top"
        qp2_type(iqp2_cupincpbl ) = "cup"

        qp2_name (iqp2_prc       ) = "prc"
        qp2_units(iqp2_prc       ) = "kg/m2/s"
        qp2_descr(iqp2_prc       ) =
     *   ":cup: convective precipitation rate"
        qp2_mjo_link(iqp2_prc    ) = imjo2_cuppr
        qp2_type(iqp2_prc ) = "cup"
        qp2_multfac(iqp2_prc ) = 86400._dbl_kind
        qp2_contmin(iqp2_prc ) = 1.0_dbl_kind
        qp2_contint(iqp2_prc ) = 1.0_dbl_kind
        qp2_altunits(iqp2_prc) = "mm/day"
     
        qp2_name (iqp2_cupcos1   ) = "cupcos1"
        qp2_units(iqp2_cupcos1   ) = "kg/m2/s"
        qp2_descr(iqp2_cupcos1   ) =
     *   ":cup: cos of 1st diurnal harmonic of cup precipitation"
        qp2_type(iqp2_cupcos1 ) = "cup"
        
        qp2_name (iqp2_cupcos2   ) = "cupcos2"
        qp2_units(iqp2_cupcos2   ) = "kg/m2/s"
        qp2_descr(iqp2_cupcos2   ) =
     *   ":cup: cos of 2nd diurnal harmonic of cup precipitation"
        qp2_type(iqp2_cupcos2 ) = "cup"

        qp2_name (iqp2_cupsin1   ) = "cupsin1"
        qp2_units(iqp2_cupsin1   ) = "kg/m2/s"
        qp2_descr(iqp2_cupsin1   ) =
     *   ":cup: sin of 1st diurnal harmonic of cup precipitation"
        qp2_type(iqp2_cupsin1 ) = "cup"

        qp2_name (iqp2_cupsin2   ) = "cupsin2"
        qp2_units(iqp2_cupsin2   ) = "kg/m2/s"
        qp2_descr(iqp2_cupsin2   ) =
     *   ":cup: sin of 2nd diurnal harmonic of cup precipitation"
        qp2_type(iqp2_cupsin2 ) = "cup"
        
        qp2_name (iqp2_cupprpbl  ) = "cupprpbl"
        qp2_units(iqp2_cupprpbl  ) = "kg/m2/s"
        qp2_descr(iqp2_cupprpbl  ) =
     *   ":cup: convective precipitation rate for PBL cloud base"
        qp2_type(iqp2_cupprpbl ) = "cup"
        qp2_multfac(iqp2_cupprpbl ) = 86400._dbl_kind
        qp2_contmin(iqp2_cupprpbl ) = 1.0_dbl_kind
        qp2_contint(iqp2_cupprpbl ) = 1.0_dbl_kind
        qp2_altunits(iqp2_cupprpbl) = "mm/day"

        qp2_name (iqp2_cupcke    ) = "cupcke"
        qp2_units(iqp2_cupcke    ) = "J/m2"
        qp2_descr(iqp2_cupcke    ) =
     *   ":cup: cumulus kinetic energy at PBL top"
        qp2_type(iqp2_cupcke ) = "cup"

        qp2_name (iqp2_cupcdwork ) = "cupcdwork"
        qp2_units(iqp2_cupcdwork ) = "J/kg)"
        qp2_descr(iqp2_cupcdwork ) =
     *   ":cup: convective cloud work function at PBL top"
        qp2_type(iqp2_cupcdwork ) = "cup"
     
        qp2_name (iqp2_cupmb     ) = "cupmb"
        qp2_units(iqp2_cupmb     ) = "kg/m2/s"
        qp2_descr(iqp2_cupmb     ) =
     *   ":cup: convective mass flux at PBL top"
        qp2_type(iqp2_cupmb ) = "cup"

        qp2_name (iqp2_cupcape   ) = "cape"
        qp2_units(iqp2_cupcape   ) = "J/kg"
        qp2_descr(iqp2_cupcape   ) =
     *   ":cup: conditional available potential energy"
        qp2_type(iqp2_cupcape ) = "cup"

        qp2_name (iqp2_dlcupt    ) = "dlcupt"
        qp2_units(iqp2_dlcupt    ) = "kg/m2/s"
        qp2_descr(iqp2_dlcupt    ) =
     *   ":cup: vertically integrated convective detrainment of cloud water"
        qp2_type(iqp2_dlcupt ) = "cup"

        qp2_name (iqp2_dicupt    ) = "dicupt"
        qp2_units(iqp2_dicupt    ) = "kg/m2/s"
        qp2_descr(iqp2_dicupt    ) =
     *   ":cup: vertically integrated convective detrainment of cloud ice"
        qp2_type(iqp2_dicupt ) = "cup"

        qp2_name (iqp2_totdlcupt ) = "totdlcupt"
        qp2_units(iqp2_totdlcupt ) = "kg/m2/s"
        qp2_descr(iqp2_totdlcupt ) =
     *   ":cup: vertically integrated total convective tendency of cloud water"
        qp2_type(iqp2_totdlcupt ) = "cup"

        qp2_name (iqp2_totdicupt    ) = "totdicupt"
        qp2_units(iqp2_totdicupt    ) = "kg/m2/s"
        qp2_descr(iqp2_totdicupt    ) =
     *   ":cup: vertically integrated total convective tendency of cloud ice"
        qp2_type(iqp2_totdicupt ) = "cup"

        qp2_name (iqp2_cupinc1   ) = "cupinc1"
        qp2_units(iqp2_cupinc1   ) = "-"
        qp2_descr(iqp2_cupinc1   ) =
     *   ":cup: cloud test - convectively unstable PBL cloud base"
        qp2_type(iqp2_cupinc1 ) = "cup"

        qp2_name (iqp2_cupinc2   ) = "cupinc2"
        qp2_units(iqp2_cupinc2   ) = "-"
        qp2_descr(iqp2_cupinc2   ) =
     *   ":cup: cloud test - convective unstable cloud bases"
        qp2_type(iqp2_cupinc2 ) = "cup"

        qp2_name (iqp2_cupinc3   ) = "cupinc3"
        qp2_units(iqp2_cupinc3   ) = "-"
        qp2_descr(iqp2_cupinc3   ) =
     *   ":cup: cloud test - after entrainment rate (PBL)"
        qp2_type(iqp2_cupinc3 ) = "cup"

        qp2_name (iqp2_cupinc4   ) = "cupinc4"
        qp2_units(iqp2_cupinc4   ) = "-"
        qp2_descr(iqp2_cupinc4   ) =
     *   ":cup: cloud test - after entrainment rate (ALL)"
        qp2_type(iqp2_cupinc4 ) = "cup"

        qp2_name (iqp2_cupinc5   ) = "cupinc5"
        qp2_units(iqp2_cupinc5   ) = "-"
        qp2_descr(iqp2_cupinc5   ) =
     *   ":cup: cloud test - after cloud tests (PBL)"
        qp2_type(iqp2_cupinc5 ) = "cup"

        qp2_name (iqp2_cupinc6   ) = "cupinc6"
        qp2_units(iqp2_cupinc6   ) = "-"
        qp2_descr(iqp2_cupinc6   ) =
     *   ":cup: cloud test - after cloud tests (ALL)"
        qp2_type(iqp2_cupinc6 ) = "cup"

        qp2_name (iqp2_cupinc7   ) = "cupinc7"
        qp2_units(iqp2_cupinc7   ) = "-"
        qp2_descr(iqp2_cupinc7   ) =
     *   ":cup: cloud test - after moist static energy (PBL)"
        qp2_type(iqp2_cupinc7 ) = "cup"

        qp2_name (iqp2_cupinc8   ) = "cupinc8"
        qp2_units(iqp2_cupinc8   ) = "-"
        qp2_descr(iqp2_cupinc8   ) =
     *   ":cup: cloud test - after moist static energy (ALL)"
        qp2_type(iqp2_cupinc8 ) = "cup"

        qp2_name (iqp2_cupinc9   ) = "cupinc9"
        qp2_units(iqp2_cupinc9   ) = "-"
        qp2_descr(iqp2_cupinc9   ) =
     *   ":cup: cloud test - after water budget (PBL)"
        qp2_type(iqp2_cupinc9 ) = "cup"

        qp2_name (iqp2_cupinc10  ) = "cupinc10"
        qp2_units(iqp2_cupinc10  ) = "-"
        qp2_descr(iqp2_cupinc10  ) =
     *   ":cup: cloud test - after water budget (ALL)"
        qp2_type(iqp2_cupinc10 ) = "cup"

        qp2_name (iqp2_cupinc11   ) = "cupinc11"
        qp2_units(iqp2_cupinc11   ) = "-"
        qp2_descr(iqp2_cupinc11   ) =
     *   ":cup: cloud test - after normalized mass flux (PBL)"
        qp2_type(iqp2_cupinc11 ) = "cup"

        qp2_name (iqp2_cupinc12   ) = "cupinc12"
        qp2_units(iqp2_cupinc12   ) = "-"
        qp2_descr(iqp2_cupinc12   ) =
     *   ":cup: cloud test - after normalized mass flux (ALL)"
        qp2_type(iqp2_cupinc12 ) = "cup"

        qp2_name (iqp2_cupinc13  ) = "cupinc13"
        qp2_units(iqp2_cupinc13  ) = "-"
        qp2_descr(iqp2_cupinc13  ) =
     *   ":cup: cloud test - after cloud work function (PBL)"
        qp2_type(iqp2_cupinc13 ) = "cup"

        qp2_name (iqp2_cupinc14   ) = "cupinc14"
        qp2_units(iqp2_cupinc14   ) = "-"
        qp2_descr(iqp2_cupinc14   ) =
     *   ":cup: cloud test - after cloud work function (ALL)"
        qp2_type(iqp2_cupinc14 ) = "cup"

        qp2_name (iqp2_cupinc15   ) = "cupinc15"
        qp2_units(iqp2_cupinc15   ) = "-"
        qp2_descr(iqp2_cupinc15   ) =
     *   ":cup: cloud test - after eau cupcld (PBL)"
        qp2_type(iqp2_cupinc15 ) = "cup"

        qp2_name (iqp2_cupinc16   ) = "cupinc16"
        qp2_units(iqp2_cupinc16   ) = "-"
        qp2_descr(iqp2_cupinc16   ) =
     *   ":cup: cloud test - after eau cupcld (ALL)"
        qp2_type(iqp2_cupinc16 ) = "cup"

        qp2_name (iqp2_prw       ) = "prw"
        qp2_units(iqp2_prw       ) = "kg/m2"
        qp2_descr(iqp2_prw       ) = 
     *   ":elq: precipitable water"
        qp2_mjo_link(iqp2_prw    ) = imjo2_elqwvt
        qp2_type(iqp2_prw ) = "elq"

        qp2_name (iqp2_pblprw    ) = "pblprw"
        qp2_units(iqp2_pblprw    ) = "kg/m2"
        qp2_descr(iqp2_pblprw    ) = 
     *   "elq: pbl precipitable water"
        qp2_type(iqp2_pblprw ) = "elq"

        qp2_name (iqp2_hurs  ) = "hurs"
        qp2_units(iqp2_hurs  ) = "-"
        qp2_descr(iqp2_hurs  ) = "Surface (2m) relative humidity"
        qp2_amip_sampling(iqp2_hurs) = .true.
        qp2_type(iqp2_hurs ) = "elq"

        qp2_name (iqp2_dzlrprec  ) = "dzlrprec"
        qp2_units(iqp2_dzlrprec  ) = "kg/m2/s"
        qp2_descr(iqp2_dzlrprec  ) = "drizzle liquid"
        qp2_type(iqp2_dzlrprec ) = "elq"

        qp2_name (iqp2_dzlsprec  ) = "dzlsprec"
        qp2_units(iqp2_dzlsprec  ) = "kg/m2/s"
        qp2_descr(iqp2_dzlsprec  ) = "drizzle frozen"
        qp2_type(iqp2_dzlsprec ) = "elq"

        qp2_name (iqp2_deaultp  ) = "deaultp"
        qp2_units(iqp2_deaultp  ) = "K/s"
        qp2_descr(iqp2_deaultp  ) = "drizzle heating rate"
        qp2_type(iqp2_deaultp ) = "elq"

        qp2_name (iqp2_deauliq  ) = "deauliq"
        qp2_units(iqp2_deauliq  ) = "kg/kg/s"
        qp2_descr(iqp2_deauliq  ) = "drizzle moistening rate"
        qp2_type(iqp2_deauliq ) = "elq"

        qp2_name (iqp2_pblcw     ) = "pblcw"
        qp2_units(iqp2_pblcw     ) = "kg/m2"
        qp2_descr(iqp2_pblcw     ) = 
     *   ":elq: diagnostic pbl cloud water path"
        qp2_type(iqp2_pblcw ) = "elq"

        qp2_name (iqp2_pblci     ) = "pblci"
        qp2_units(iqp2_pblci     ) = "kg/m2"
        qp2_descr(iqp2_pblci     ) = 
     *   ":elq: diagnostic pbl cloud ice path"
        qp2_type(iqp2_pblci ) = "elq"

        qp2_name (iqp2_clwvi     ) = "clwvi"
        qp2_units(iqp2_clwvi     ) = "kg/m2"
        qp2_descr(iqp2_clwvi     ) = 
     *   ":elq: vertically integrated cloud water"
        qp2_type(iqp2_clwvi ) = "elq"
     
        qp2_name (iqp2_elqrwt    ) = "elqrwt"
        qp2_units(iqp2_elqrwt    ) = "kg/m2"
        qp2_descr(iqp2_elqrwt    ) = 
     *   ":elq: vertically-integrated rain"
        qp2_type(iqp2_elqrwt ) = "elq"

        qp2_name (iqp2_clivi     ) = "clivi"
        qp2_units(iqp2_clivi     ) = "kg/m2"
        qp2_descr(iqp2_clivi     ) = 
     *   ":elq: vertically integrated cloud ice"
        qp2_type(iqp2_clivi ) = "elq"

        qp2_name (iqp2_elqrit    ) = "elqrit"
        qp2_units(iqp2_elqrit    ) = "kg/m2"
        qp2_descr(iqp2_elqrit    ) = 
     *   ":elq: vertically-integrated snow"
        qp2_type(iqp2_elqrit ) = "elq"

        qp2_name (iqp2_pblclwvi  ) = "pblclwvi"
        qp2_units(iqp2_pblclwvi  ) = "kg/m2"
        qp2_descr(iqp2_pblclwvi  ) = 
     *   ":elq: pbl vertically integrated cloud water"
        qp2_type(iqp2_pblclwvi ) = "elq"
     
        qp2_name (iqp2_pblelqrwt  ) = "pblelqrwt"
        qp2_units(iqp2_pblelqrwt  ) = "kg/m2"
        qp2_descr(iqp2_pblelqrwt  ) = 
     *   ":elq: pbl vertically-integrated rain"
        qp2_type(iqp2_pblelqrwt ) = "elq"

        qp2_name (iqp2_pblclivi   ) = "pblclivi"
        qp2_units(iqp2_pblclivi   ) = "kg/m2"
        qp2_descr(iqp2_pblclivi   ) = 
     *   ":elq: pbl vertically integrated cloud ice"
        qp2_type(iqp2_pblclivi ) = "elq"

        qp2_name (iqp2_pblelqrit  ) = "pblelqrit"
        qp2_units(iqp2_pblelqrit  ) = "kg/m2"
        qp2_descr(iqp2_pblelqrit  ) = 
     *   ":elq: pbl vertically-integrated snow"
        qp2_type(iqp2_pblelqrit ) = "elq"

        qp2_name (iqp2_lpblclwvi  ) = "lpblclwvi"
        qp2_units(iqp2_lpblclwvi  ) = "kg/m2"
        qp2_descr(iqp2_lpblclwvi  ) = 
     *   ":elq: lpbl vertically integrated cloud water"
        qp2_type(iqp2_lpblclwvi ) = "elq"
     
        qp2_name (iqp2_lpblelqrwt  ) = "lpblelqrwt"
        qp2_units(iqp2_lpblelqrwt  ) = "kg/m2"
        qp2_descr(iqp2_lpblelqrwt  ) = 
     *   ":elq: lpbl vertically-integrated rain"
        qp2_type(iqp2_lpblelqrwt ) = "elq"

        qp2_name (iqp2_lpblclivi   ) = "lpblclivi"
        qp2_units(iqp2_lpblclivi   ) = "kg/m2"
        qp2_descr(iqp2_lpblclivi   ) = 
     *   ":elq: lpbl vertically integrated cloud ice"
        qp2_type(iqp2_lpblclivi ) = "elq"

        qp2_name (iqp2_lpblelqrit  ) = "lpblelqrit"
        qp2_units(iqp2_lpblelqrit  ) = "kg/m2"
        qp2_descr(iqp2_lpblelqrit  ) = 
     *   ":elq: lpbl vertically-integrated snow"
        qp2_type(iqp2_lpblelqrit ) = "elq"

        qp2_name (iqp2_elqprr    ) = "elqprr"
        qp2_units(iqp2_elqprr    ) = "kg/m2/s"
        qp2_descr(iqp2_elqprr    ) =
     *   ":elq: stratiform precipitation rate of rain"
        qp2_type(iqp2_elqprr ) = "elq"
        qp2_multfac(iqp2_elqprr ) = 86400._dbl_kind
        qp2_contmin(iqp2_elqprr ) = 1.0_dbl_kind
        qp2_contint(iqp2_elqprr ) = 1.0_dbl_kind
        qp2_altunits(iqp2_elqprr) = "mm/day"

        qp2_name (iqp2_prsn      ) = "prsn"
        qp2_units(iqp2_prsn      ) = "kg/m2/s"
        qp2_descr(iqp2_prsn      ) =
     *   ":elq: stratiform precipitation rate of snow"
        qp2_type(iqp2_prsn ) = "elq"
        qp2_multfac(iqp2_prsn ) = 86400._dbl_kind
        qp2_contmin(iqp2_prsn ) = 1.0_dbl_kind
        qp2_contint(iqp2_prsn ) = 1.0_dbl_kind
        qp2_altunits(iqp2_prsn) = "mm/day"

        qp2_name (iqp2_elqpr     ) = "elqpr"
        qp2_units(iqp2_elqpr     ) = "kg/m2/s"
        qp2_descr(iqp2_elqpr     ) =
     *   ":elq: stratiform precipitation rate"
        qp2_mjo_link(iqp2_elqpr  ) = imjo2_elqpr
        qp2_type(iqp2_elqpr ) = "elq"
        qp2_multfac(iqp2_elqpr ) = 86400._dbl_kind
        qp2_contmin(iqp2_elqpr ) = 1.0_dbl_kind
        qp2_contint(iqp2_elqpr ) = 1.0_dbl_kind
        qp2_altunits(iqp2_elqpr) = "mm/day"
        
        qp2_name (iqp2_elqcos1   ) = "elqcos1"
        qp2_units(iqp2_elqcos1   ) = "kg/m2/s"
        qp2_descr(iqp2_elqcos1   ) =
     *   ":elq: cos of 1st diurnal harmonic of elq precipitation"
        qp2_type(iqp2_elqcos1 ) = "elq"
        
        qp2_name (iqp2_elqcos2   ) = "elqcos2"
        qp2_units(iqp2_elqcos2   ) = "kg/m2/s"
        qp2_descr(iqp2_elqcos2   ) =
     *   ":elq: cos of 2nd diurnal harmonic of elq precipitation"
        qp2_type(iqp2_elqcos2 ) = "elq"

        qp2_name (iqp2_elqsin1   ) = "elqsin1"
        qp2_units(iqp2_elqsin1   ) = "kg/m2/s"
        qp2_descr(iqp2_elqsin1   ) =
     *   ":elq: sin of 1st diurnal harmonic of elq precipitation"
        qp2_type(iqp2_elqsin1 ) = "elq"

        qp2_name (iqp2_elqsin2   ) = "elqsin2"
        qp2_units(iqp2_elqsin2   ) = "kg/m2/s"
        qp2_descr(iqp2_elqsin2   ) =
     *   ":elq: sin of 2nd diurnal harmonic of elq precipitation"
        qp2_type(iqp2_elqsin2 ) = "elq"

        qp2_name (iqp2_pr        ) = "pr"
        qp2_units(iqp2_pr        ) = "kg/m2/s"
        qp2_descr(iqp2_pr        ) =
     *   ":elq: total precipitation rate"
        qp2_type(iqp2_pr ) = "elq"
        qp2_multfac(iqp2_pr ) = 86400._dbl_kind
        qp2_contmin(iqp2_pr ) = 1.0_dbl_kind
        qp2_contint(iqp2_pr ) = 1.0_dbl_kind
        qp2_altunits(iqp2_pr) = "mm/day"

        qp2_name (iqp2_prcos1    ) = "prcos1"
        qp2_units(iqp2_prcos1    ) = "kg/m2/s"
        qp2_descr(iqp2_prcos1    ) =
     *   ":elq: cos of 1st diurnal harmonic of total precipitation"
        qp2_type(iqp2_prcos1 ) = "elq"
        
        qp2_name (iqp2_prcos2    ) = "prcos2"
        qp2_units(iqp2_prcos2    ) = "kg/m2/s"
        qp2_descr(iqp2_prcos2    ) =
     *   ":elq: cos of 2nd diurnal harmonic of total precipitation"
        qp2_type(iqp2_prcos2 ) = "elq"

        qp2_name (iqp2_prsin1    ) = "prsin1"
        qp2_units(iqp2_prsin1    ) = "kg/m2/s"
        qp2_descr(iqp2_prsin1    ) =
     *   ":elq: sin of 1st diurnal harmonic of total precipitation"
        qp2_type(iqp2_prsin1 ) = "elq"

        qp2_name (iqp2_prsin2    ) = "prsin2"
        qp2_units(iqp2_prsin2    ) = "kg/m2/s"
        qp2_descr(iqp2_prsin2    ) =
     *   ":elq: sin of 2nd diurnal harmonic of total precipitation"
        qp2_type(iqp2_prsin2 ) = "elq"

        qp2_name (iqp2_elqacld   ) = "elqacld"
        qp2_units(iqp2_elqacld   ) = "-"
        qp2_descr(iqp2_elqacld   ) =
     *   ":elq: vertically-integrated cloud fraction"
        qp2_type(iqp2_elqacld ) = "elq"

        qp2_name (iqp2_meltlev   ) = "meltlev"
        qp2_units(iqp2_meltlev   ) = "Pa"
        qp2_descr(iqp2_meltlev   ) =
     *   ":elq: pressure level of the melting level"
        qp2_type(iqp2_meltlev ) = "elq"

        qp2_name (iqp2_press20C  ) = "press20C"
        qp2_units(iqp2_press20C  ) = "Pa"
        qp2_descr(iqp2_press20C  ) =
     *   ":elq: pressure level of the -20C"
        qp2_type(iqp2_press20C ) = "elq"

        qp2_name (iqp2_press40C  ) = "press40C"
        qp2_units(iqp2_press40C  ) = "Pa"
        qp2_descr(iqp2_press40C  ) =
     *   ":elq: pressure level of the -40C"
        qp2_type(iqp2_press40C ) = "elq"

        qp2_name (iqp2_cldhigh   ) = "cldhigh"
        qp2_units(iqp2_cldhigh   ) = "%"
        qp2_descr(iqp2_cldhigh   ) =
     *   ":rad cf: fraction of high-level clouds"
        qp2_type(iqp2_cldhigh ) = "rad"
     
        qp2_name (iqp2_cldmid    ) = "cldmid"
        qp2_units(iqp2_cldmid    ) = "%"
        qp2_descr(iqp2_cldmid    ) =
     *   ":rad cf: fraction of mid-level clouds"
        qp2_type(iqp2_cldmid ) = "rad"

        qp2_name (iqp2_cldlow    ) = "cldlow"
        qp2_units(iqp2_cldlow    ) = "%"
        qp2_descr(iqp2_cldlow    ) =
     *   ":rad cf: fraction of low-level clouds"
        qp2_type(iqp2_cldlow ) = "rad"

        qp2_name (iqp2_cldt2d    ) = "cldt2d"
        qp2_units(iqp2_cldt2d    ) = "%"
        qp2_descr(iqp2_cldt2d    ) =
     *   ":rad cf: total cloud fraction"
        qp2_type(iqp2_cldt2d ) = "rad"

        qp2_name (iqp2_cldw2d    ) = "cldw2d"
        qp2_units(iqp2_cldw2d    ) = "%"
        qp2_descr(iqp2_cldw2d    ) =
     *   ":rad cf: total fraction of water clouds"
        qp2_type(iqp2_cldw2d ) = "rad"

        qp2_name (iqp2_cldi2d    ) = "cldi2d"
        qp2_units(iqp2_cldi2d    ) = "%"
        qp2_descr(iqp2_cldi2d    ) =
     *   ":rad cf: total fraction of ice clouds"
        qp2_type(iqp2_cldi2d ) = "rad"

        qp2_name (iqp2_cldm2d    ) = "cldm2d"
        qp2_units(iqp2_cldm2d    ) = "%"
        qp2_descr(iqp2_cldm2d    ) =
     *   ":rad cf: total fraction of mixed-phase clouds"
        qp2_type(iqp2_cldm2d ) = "rad"

        qp2_name (iqp2_ncw       ) = "ncw"
        qp2_units(iqp2_ncw       ) = "per m3"
        qp2_descr(iqp2_ncw       ) =
     *   ":rad cf: number concentration"
        qp2_type(iqp2_ncw ) = "rad"

        qp2_name (iqp2_cwlm1     ) = "cwlm1"
        qp2_units(iqp2_cwlm1     ) = "g/kg"
        qp2_descr(iqp2_cwlm1     ) =
     *   ":rad cf: cloud water in layer nlm-1"
        qp2_type(iqp2_cwlm1 ) = "rad"

        qp2_name (iqp2_effrw     ) = "effrw"
        qp2_units(iqp2_effrw     ) = "mu"
        qp2_descr(iqp2_effrw     ) =
     *   ":rad cf: effective radius of water clouds in layer nlm-1"
        qp2_type(iqp2_effrw ) = "rad"

        qp2_name (iqp2_cil5      ) = "cil5"
        qp2_units(iqp2_cil5      ) = "g/kg"
        qp2_descr(iqp2_cil5      ) =
     *   ":rad cf: cloud ice in layer 5"
        qp2_type(iqp2_cil5 ) = "rad"

        qp2_name (iqp2_effri     ) = "effri"
        qp2_units(iqp2_effri     ) = "mu"
        qp2_descr(iqp2_effri     ) =
     *   ":rad cf: effective radius of ice clouds in layer 5"
        qp2_type(iqp2_effri ) = "rad"

        qp2_name (iqp2_codtwvis  ) = "codtwvis"
        qp2_units(iqp2_codtwvis  ) = "-"
        qp2_descr(iqp2_codtwvis  ) =
     *   ":rad cf: total optical depth of water clouds - visible"
        qp2_type(iqp2_codtwvis ) = "rad"

        qp2_name (iqp2_codtivis  ) = "codtivis"
        qp2_units(iqp2_codtivis  ) = "-"
        qp2_descr(iqp2_codtivis  ) =
     *   ":rad cf: total optical depth of ice clouds - visible"
        qp2_type(iqp2_codtivis ) = "rad"

        qp2_name (iqp2_codtmvis  ) = "codtmvis"
        qp2_units(iqp2_codtmvis  ) = "-"
        qp2_descr(iqp2_codtmvis  ) =
     *   ":rad cf: total optical depth of mixed-phase clouds - visible"
        qp2_type(iqp2_codtmvis ) = "rad"

        qp2_name (iqp2_codtvis  )  = "codtvis"
        qp2_units(iqp2_codtvis  )  = "-"
        qp2_descr(iqp2_codtvis  )  =
     *   ":rad cf: total optical depth of total clouds - visible"
        qp2_type(iqp2_codtvis ) = "rad"

        qp2_name (iqp2_codtwirw  ) = "codtwirw"
        qp2_units(iqp2_codtwirw  ) = "-"
        qp2_descr(iqp2_codtwirw  ) =
     *   ":rad cf: total optical depth of water clouds - infrared"
        qp2_type(iqp2_codtwirw ) = "rad"

        qp2_name (iqp2_codtiirw  ) = "codtiirw"
        qp2_units(iqp2_codtiirw  ) = "-"
        qp2_descr(iqp2_codtiirw  ) =
     *   ":rad cf: total optical depth of ice clouds - infrared"
        qp2_type(iqp2_codtiirw ) = "rad"

        qp2_name (iqp2_codtmirw  ) = "codtmirw"
        qp2_units(iqp2_codtmirw  ) = "-"
        qp2_descr(iqp2_codtmirw  ) =
     *   ":rad cf: total optical depth of mixed-phase clouds - infrared"
        qp2_type(iqp2_codtmirw ) = "rad"

        qp2_name (iqp2_codtirw  ) = "codtirw"
        qp2_units(iqp2_codtirw  ) = "-"
        qp2_descr(iqp2_codtirw  ) =
     *   ":rad cf: total optical depth of total clouds - infrared"
        qp2_type(iqp2_codtirw ) = "rad"

        qp2_name (iqp2_lwnetsfccl) = "lwnetsfccl"
        qp2_units(iqp2_lwnetsfccl) = "W/m^2"
        qp2_descr(iqp2_lwnetsfccl) =
     *   ":rad lw: sfc clear-sky net longwave radiation"
        qp2_type(iqp2_lwnetsfccl ) = "rad"

        qp2_name (iqp2_lwplanck  ) = "lwplanck"
        qp2_units(iqp2_lwplanck  ) = "W/m^2"
        qp2_descr(iqp2_lwplanck  ) =
     *   ":rad lw: sfc planck function"
        qp2_type(iqp2_lwplanck ) = "rad"

        qp2_name (iqp2_lwtom     ) = "lwtom"
        qp2_units(iqp2_lwtom     ) = "W/m^2"
        qp2_descr(iqp2_lwtom     ) =
     *   ":rad lw: top-of-model all-sky outgoing longwave radiation"
        qp2_type(iqp2_lwtom ) = "rad"

        qp2_name (iqp2_lwtomcl   ) = "lwtomcl"
        qp2_units(iqp2_lwtomcl   ) = "W/m^2"
        qp2_descr(iqp2_lwtomcl   ) =
     *   ":rad lw: top-of-model clear-sky outgoing longwave radiation"
        qp2_type(iqp2_lwtomcl ) = "rad"

        qp2_name (iqp2_lwpblfu   ) = "lwpblfu"
        qp2_units(iqp2_lwpblfu   ) = "W/m^2"
        qp2_descr(iqp2_lwpblfu   ) =
     *   ":rad lw: pbl-top all-sky upward longwave radiation"
        qp2_type(iqp2_lwpblfu ) = "rad"

        qp2_name (iqp2_lwpblfd   ) = "lwpblfd"
        qp2_units(iqp2_lwpblfd   ) = "W/m^2"
        qp2_descr(iqp2_lwpblfd   ) =
     *   ":rad lw: pbl-top all-sky downward longwave radiation"
        qp2_type(iqp2_lwpblfd ) = "rad"
     
        qp2_name (iqp2_swnettoacl   ) = "swnettoacl"
        qp2_units(iqp2_swnettoacl   ) = "W/m^2"
        qp2_descr(iqp2_swnettoacl   ) =
     *   ":rad sw: toa clear-sky net shortwave radiation"
        qp2_type(iqp2_swnettoacl ) = "rad"

        qp2_name (iqp2_swnetsfccl) = "swnetsfccl"
        qp2_units(iqp2_swnetsfccl) = "W/m^2"
        qp2_descr(iqp2_swnetsfccl) =
     *   ":rad sw: sfc clear-sky net shortwave radiation"
        qp2_type(iqp2_swnetsfccl ) = "rad"

        qp2_name (iqp2_swtom     ) = "swtom"
        qp2_units(iqp2_swtom     ) = "W/m^2"
        qp2_descr(iqp2_swtom     ) =
     *   ":rad sw: top of model all-sky reflected shortwave radiation"
        qp2_type(iqp2_swtom ) = "rad"

        qp2_name (iqp2_swtomcl   ) = "swtomcl"
        qp2_units(iqp2_swtomcl   ) = "W/m^2"
        qp2_descr(iqp2_swtomcl   ) =
     *   ":rad sw: top of model clear-sky reflected shortwave radiation"
        qp2_type(iqp2_swtomcl ) = "rad"

        qp2_name (iqp2_swradvdc  ) = "swradvdc"
        qp2_units(iqp2_swradvdc  ) = "W/m^2"
        qp2_descr(iqp2_swradvdc  ) =
     *   ":rad sw: sfc visible diffuse all-sky net sw radiation"
        qp2_type(iqp2_swradvdc ) = "rad"

        qp2_name (iqp2_swradndc  ) = "swradndc"
        qp2_units(iqp2_swradndc  ) = "W/m^2"
        qp2_descr(iqp2_swradndc  ) =
     *   ":rad sw: sfc near-ir diffuse all-sky net sw radiation"
        qp2_type(iqp2_swradndc ) = "rad"

        qp2_name (iqp2_swradvbc  ) = "swradvbc"
        qp2_units(iqp2_swradvbc  ) = "W/m^2"
        qp2_descr(iqp2_swradvbc  ) =
     *   ":rad sw: sfc visible direct all-sky net sw radiation"
        qp2_type(iqp2_swradvbc ) = "rad"

        qp2_name (iqp2_swradnbc  ) = "swradnbc"
        qp2_units(iqp2_swradnbc  ) = "W/m^2"
        qp2_descr(iqp2_swradnbc  ) =
     *   ":rad sw: sfc near-ir direct all-sky net sw radiation"
        qp2_type(iqp2_swradnbc ) = "rad"

        qp2_name (iqp2_swradvdcc ) = "swradvdcc"
        qp2_units(iqp2_swradvdcc ) = "W/m^2"
        qp2_descr(iqp2_swradvdcc ) =
     *   ":rad sw: sfc visible diffuse clear-sky net sw radiation"
        qp2_type(iqp2_swradvdcc ) = "rad"

        qp2_name (iqp2_swradndcc ) = "swradndcc"
        qp2_units(iqp2_swradndcc ) = "W/m^2"
        qp2_descr(iqp2_swradndcc ) =
     *   ":rad sw: sfc near-ir diffuse clear-sky net sw radiation"
        qp2_type(iqp2_swradndcc ) = "rad"

        qp2_name (iqp2_swradvbcc ) = "swradvbcc"
        qp2_units(iqp2_swradvbcc ) = "W/m^2"
        qp2_descr(iqp2_swradvbcc ) =
     *   ":rad sw: sfc visible direct clear-sky net sw radiation"
        qp2_type(iqp2_swradvbcc ) = "rad"

        qp2_name (iqp2_swradnbcc ) = "swradnbcc"
        qp2_units(iqp2_swradnbcc ) = "W/m^2"
        qp2_descr(iqp2_swradnbcc ) =
     *   ":rad sw: sfc near-ir direct clear-sky net sw radiation"
        qp2_type(iqp2_swradnbcc ) = "rad"

        qp2_name (iqp2_swcosz    ) = "swcosz"
        qp2_units(iqp2_swcosz    ) = "-"
        qp2_descr(iqp2_swcosz    ) =
     *   ":rad sw: cosine of solar zenith angle"
        qp2_type(iqp2_swcosz ) = "rad"

        qp2_name (iqp2_swsfac    ) = "swsfac"
        qp2_units(iqp2_swsfac    ) = "-"
        qp2_descr(iqp2_swsfac    ) =
     *   ":rad sw: correction for length of day"
        qp2_type(iqp2_swsfac ) = "rad"

        qp2_name (iqp2_swalvdr   ) = "swalvdr"
        qp2_units(iqp2_swalvdr   ) = "-"
        qp2_descr(iqp2_swalvdr   ) =
     *   ":rad sw: sfc visible direct albedo (-)"
        qp2_type(iqp2_swalvdr ) = "rad"
    
        qp2_name (iqp2_swalvdf   ) = "swalvdf"
        qp2_units(iqp2_swalvdf   ) = "-"
        qp2_descr(iqp2_swalvdf   ) =
     *   ":rad sw: sfc visible diffuse albedo (-)"
        qp2_type(iqp2_swalvdf ) = "rad"

        qp2_name (iqp2_swalndr   ) = "swalndr"
        qp2_units(iqp2_swalndr   ) = "-"
        qp2_descr(iqp2_swalndr   ) =
     *   ":rad sw: sfc near-ir direct albedo (-)"
        qp2_type(iqp2_swalndr ) = "rad"
    
        qp2_name (iqp2_swalndf   ) = "swalndf"
        qp2_units(iqp2_swalndf   ) = "-"
        qp2_descr(iqp2_swalndf   ) =
     *   ":rad sw: sfc near-ir diffuse albedo (-)"
        qp2_type(iqp2_swalndf ) = "rad"

#ifdef rad_isccp
        qp2_name (iqp2_fhcld1    ) = "fhcld1"
        qp2_units(iqp2_fhcld1    ) = "-"
        qp2_descr(iqp2_fhcld1    ) =
     *   ":isccp: frequency of high-level clouds - tau < 0.3"
        qp2_type(iqp2_fhcld1 ) = "isc"

        qp2_name (iqp2_fhcld2    ) = "fhcld2"
        qp2_units(iqp2_fhcld2    ) = "-"
        qp2_descr(iqp2_fhcld2    ) =
     *   ":isccp: frequency of high-level clouds - 0.3 < tau < 3.6"
        qp2_type(iqp2_fhcld2 ) = "isc"

        qp2_name (iqp2_fhcld3    ) = "fhcld3"
        qp2_units(iqp2_fhcld3    ) = "-"
        qp2_descr(iqp2_fhcld3    ) =
     *   ":isccp: frequency of high-level clouds - 3.6 < tau < 23"
        qp2_type(iqp2_fhcld3 ) = "isc"

        qp2_name (iqp2_fhcld4    ) = "fhcld4"
        qp2_units(iqp2_fhcld4    ) = "-"
        qp2_descr(iqp2_fhcld4    ) =
     *   ":isccp: frequency of high-level clouds - 23 < tau < 379"
        qp2_type(iqp2_fhcld4 ) = "isc"

        qp2_name (iqp2_fhcld5    ) = "fhcld5"
        qp2_units(iqp2_fhcld5    ) = "-"
        qp2_descr(iqp2_fhcld5    ) =
     *   ":isccp: frequency of high-level clouds - tau > 379"
        qp2_type(iqp2_fhcld5 ) = "isc"

        qp2_name (iqp2_fmcld1    ) = "fmcld1"
        qp2_units(iqp2_fmcld1    ) = "-"
        qp2_descr(iqp2_fmcld1    ) =
     *   ":isccp: frequency of mid-level clouds - tau < 0.3"
        qp2_type(iqp2_fmcld1 ) = "isc"

        qp2_name (iqp2_fmcld2    ) = "fmcld2"
        qp2_units(iqp2_fmcld2    ) = "-"
        qp2_descr(iqp2_fmcld2    ) =
     *   ":isccp: frequency of mid-level clouds - 0.3 < tau < 3.6"
        qp2_type(iqp2_fmcld2 ) = "isc"

        qp2_name (iqp2_fmcld3    ) = "fmcld3"
        qp2_units(iqp2_fmcld3    ) = "-"
        qp2_descr(iqp2_fmcld3    ) =
     *   ":isccp: frequency of mid-level clouds - 3.6 < tau < 23"
        qp2_type(iqp2_fmcld3 ) = "isc"

        qp2_name (iqp2_fmcld4    ) = "fmcld4"
        qp2_units(iqp2_fmcld4    ) = "-"
        qp2_descr(iqp2_fmcld4    ) =
     *   ":isccp: frequency of mid-level clouds - 23 < tau < 379"
        qp2_type(iqp2_fmcld4 ) = "isc"

        qp2_name (iqp2_fmcld5    ) = "fmcld5"
        qp2_units(iqp2_fmcld5    ) = "-"
        qp2_descr(iqp2_fmcld5    ) =
     *   ":isccp: frequency of mid-level clouds - tau > 379"
        qp2_type(iqp2_fmcld5 ) = "isc"

        qp2_name (iqp2_flcld1    ) = "flcld1"
        qp2_units(iqp2_flcld1    ) = "-"
        qp2_descr(iqp2_flcld1    ) =
     *   ":isccp: frequency of low-level clouds - tau < 0.3"
        qp2_type(iqp2_flcld1 ) = "isc"

        qp2_name (iqp2_flcld2    ) = "flcld2"
        qp2_units(iqp2_flcld2    ) = "-"
        qp2_descr(iqp2_flcld2    ) =
     *   ":isccp: frequency of low-level clouds - 0.3 < tau < 3.6"
        qp2_type(iqp2_flcld2 ) = "isc"

        qp2_name (iqp2_flcld3    ) = "flcld3"
        qp2_units(iqp2_flcld3    ) = "-"
        qp2_descr(iqp2_flcld3    ) =
     *   ":isccp: frequency of low-level clouds - 3.6 < tau < 23"
        qp2_type(iqp2_flcld3 ) = "isc"

        qp2_name (iqp2_flcld4    ) = "flcld4"
        qp2_units(iqp2_flcld4    ) = "-"
        qp2_descr(iqp2_flcld4    ) =
     *   ":isccp: frequency of low-level clouds - 23 < tau < 379"
        qp2_type(iqp2_flcld4 ) = "isc"

        qp2_name (iqp2_flcld5    ) = "flcld5"
        qp2_units(iqp2_flcld5    ) = "-"
        qp2_descr(iqp2_flcld5    ) =
     *   ":isccp: frequency of low-level clouds - tau > 379"
        qp2_type(iqp2_flcld5 ) = "isc"
#endif
!--------------------------------------------------------------------------
! start qp3s
!--------------------------------------------------------------------------
        qp3_name (iqp3_mask  ) = "mask"
        qp3_units(iqp3_mask  ) = "-"
        qp3_descr(iqp3_mask  ) = 
     *	   "fraction of time pressure surface is above earths surface"
        qp3_mjo_link(iqp3_mask) = imjo3_mask
        qp3_type(iqp3_mask ) = "dyn"

        qp3_name (iqp3_pressure  ) = "pressure"
        qp3_units(iqp3_pressure  ) = "Pa"
        qp3_descr(iqp3_pressure  ) = "Pressure "
        qp3_type(iqp3_pressure ) = "dyn"

        qp3_name (iqp3_zg    ) = "zg"
        qp3_units(iqp3_zg    ) = "m"
        qp3_descr(iqp3_zg    ) = "Geopotential height"
	  qp3_amip_sampling(iqp3_zg) = .true.
        qp3_mjo_link(iqp3_zg) = imjo3_zg
        qp3_type(iqp3_zg ) = "dyn"

        qp3_name (iqp3_ta    ) = "ta"
        qp3_units(iqp3_ta    ) = "K"
        qp3_descr(iqp3_ta    ) = "Air Temperature"
	  qp3_amip_sampling(iqp3_ta) = .true.
        qp3_mjo_link(iqp3_ta) = imjo3_ta
        qp3_type(iqp3_ta ) = "dyn"

        qp3_name (iqp3_vorticity ) = "vorticity"
        qp3_units(iqp3_vorticity ) = "s^-1"
        qp3_descr(iqp3_vorticity ) = "Relative vorticity"
	  qp3_amip_sampling(iqp3_vorticity) = .true.
        qp3_type(iqp3_vorticity ) = "dyn"

        qp3_name (iqp3_divergence) = "divergence"
        qp3_units(iqp3_divergence) = "s^-1"
        qp3_descr(iqp3_divergence) = "Divergence"
	  qp3_amip_sampling(iqp3_divergence) = .true.
        qp3_type(iqp3_divergence ) = "dyn"

        qp3_name (iqp3_tntsw ) = "tntsw"
        qp3_units(iqp3_tntsw ) = "K/s "
        qp3_descr(iqp3_tntsw ) = "Temperature tendency due to SW radiation"
        qp3_mjo_link(iqp3_tntsw) = imjo3_tntrad
        qp3_type(iqp3_tntsw ) = "rad"

        qp3_name (iqp3_tntlw ) = "tntlw"
        qp3_units(iqp3_tntlw ) = "K/s"
        qp3_descr(iqp3_tntlw ) = "Temperature tendency due to LW radiation"
        qp3_mjo_link(iqp3_tntlw) = imjo3_tntrad
        qp3_type(iqp3_tntlw ) = "rad"

        qp3_name (iqp3_hur   ) = "hur"
        qp3_units(iqp3_hur   ) = "-"
        qp3_descr(iqp3_hur   ) = "Relative Humidity"
	  qp3_amip_sampling(iqp3_hur) = .true.
        qp3_type(iqp3_hur ) = "dyn"

        qp3_name (iqp3_tntdc ) = "tntdc"
        qp3_units(iqp3_tntdc ) = "K/s"
        qp3_descr(iqp3_tntdc ) = 
     *     "Temperature tendency due to dry convective processes"
        qp3_type(iqp3_tntdc ) = "dyn"

        qp3_name (iqp3_qsat     ) = "qsat"
        qp3_units(iqp3_qsat     ) = "kg/kg"
        qp3_descr(iqp3_qsat     ) = "saturation mixing ratio"
        qp3_type(iqp3_qsat ) = "dyn"

        qp3_name (iqp3_o3       ) = "o3"
        qp3_units(iqp3_o3       ) = "kg/kg"
        qp3_descr(iqp3_o3       ) = "ozone mixing ratio"
        qp3_type(iqp3_o3 ) = "dyn"

        qp3_name (iqp3_tntswcs  ) = "tntswcs"
        qp3_units(iqp3_tntswcs  ) = "K/s"
        qp3_descr(iqp3_tntswcs  ) = "RAD: Atmospheric clear-sky SW heating rate"
        qp3_type(iqp3_tntswcs ) = "rad"

        qp3_name (iqp3_tntlwcs  ) = "tntlwcs"
        qp3_units(iqp3_tntlwcs  ) = "K/s"
        qp3_descr(iqp3_tntlwcs  ) = "RAD: Atmospheric clear-sky LW heating rate"
        qp3_type(iqp3_tntlwcs ) = "rad"

        qp3_name (iqp3_tnt  ) = "tnt"
        qp3_units(iqp3_tnt  ) = "K/s"
        qp3_descr(iqp3_tnt  ) = 
     &        "Temperature tendency due to total diabatic heating"
        qp3_type(iqp3_tnt ) = "rad"

        qp3_name (iqp3_tnmrd  ) = "tnmrd"
        qp3_units(iqp3_tnmrd  ) = "(kg/kg)/s"
        qp3_descr(iqp3_tnmrd  ) = "Moisture tendency due to diabatic processes"
        qp3_type(iqp3_tnmrd ) = "rad"

        qp3_name (iqp3_cl       ) = "cl"
        qp3_units(iqp3_cl       ) = "%"
        qp3_descr(iqp3_cl       ) = "Cloud fraction"
        qp3_type(iqp3_cl ) = "rad"

        qp3_name (iqp3_ua    ) = "ua"
        qp3_units(iqp3_ua    ) = "m/s"
        qp3_descr(iqp3_ua    ) = "Eastward wind"
	  qp3_amip_sampling(iqp3_ua) = .true.
        qp3_mjo_link(iqp3_ua) = imjo3_ua
        qp3_type(iqp3_ua ) = "dyn"

        qp3_name (iqp3_va    ) = "va"
        qp3_units(iqp3_va    ) = "m/s"
        qp3_descr(iqp3_va    ) = "Northward wind"
	  qp3_amip_sampling(iqp3_va) = .true.
        qp3_mjo_link(iqp3_va) = imjo3_va
        qp3_type(iqp3_va ) = "dyn"

        qp3_name (iqp3_wap   ) = "wap"
        qp3_units(iqp3_wap   ) = "Pa/s"
        qp3_descr(iqp3_wap   ) = "Vertical motion"
	  qp3_amip_sampling(iqp3_wap) = .true.
        qp3_mjo_link(iqp3_wap) = imjo3_wap
        qp3_type(iqp3_wap ) = "dyn"

        qp3_name (iqp3_maskamip ) = "maskamip"
        qp3_units(iqp3_maskamip ) = "-"
        qp3_descr(iqp3_maskamip ) = 
     *	   "fraction of time pressure surface is above surface, 6 hour sampling"
	qp3_amip_sampling(iqp3_maskamip) = .true.
        qp3_type(iqp3_maskamip ) = "dyn"

        qp3_name (iqp3_psbg  ) = "psbg"
        qp3_units(iqp3_psbg  ) = "%"
        qp3_descr(iqp3_psbg  ) = 
     *	   "pressure surface below ground"
        qp3_type(iqp3_psbg ) = "dyn"

        qp3_name (iqp3_tnqdc ) = "tnqdc"
        qp3_units(iqp3_tnqdc ) = "ks/ks/s"
        qp3_descr(iqp3_tnqdc ) = 
     *"Water vapor tendency, dry convective adjustment                "
        qp3_type(iqp3_tnqdc ) = "dyn"

        qp3_name (iqp3_fill3q  ) = "fill3q"
        qp3_units(iqp3_fill3q  ) = "kg/kg/s"
        qp3_descr(iqp3_fill3q  ) = "water vapor hole fill rate"
        qp3_type(iqp3_fill3q ) = "dyn"

        qp3_name (iqp3_fill3cw  ) = "fill3cw"
        qp3_units(iqp3_fill3cw  ) = "kg/kg/s"
        qp3_descr(iqp3_fill3cw  ) = "cloud water hole fill rate"
        qp3_type(iqp3_fill3cw ) = "dyn"

        qp3_name (iqp3_fill3ci  ) = "fill3ci"
        qp3_units(iqp3_fill3ci  ) = "kg/kg/s"
        qp3_descr(iqp3_fill3ci  ) = "cloud ice hole fill rate"
        qp3_type(iqp3_fill3ci ) = "dyn"

        qp3_name (iqp3_fill3rw  ) = "fill3rw"
        qp3_units(iqp3_fill3rw  ) = "kg/kg/s"
        qp3_descr(iqp3_fill3rw  ) = "rain hole fill rate"
        qp3_type(iqp3_fill3rw ) = "dyn"

        qp3_name (iqp3_fill3ri  ) = "fill3ri"
        qp3_units(iqp3_fill3ri  ) = "kg/kg/s"
        qp3_descr(iqp3_fill3ri  ) = "snow hole fill rate"
        qp3_type(iqp3_fill3ri ) = "dyn"

        qp3_name (iqp3_ke  ) = "ke"
        qp3_units(iqp3_ke  ) = "J/kg"
        qp3_descr(iqp3_ke  ) = "kinetic energy"
	  qp3_amip_sampling(iqp3_ke) = .true.
        qp3_type(iqp3_ke ) = "dyn"

        qp3_name (iqp3_en  ) = "en"
        qp3_units(iqp3_en  ) = "s-2"
        qp3_descr(iqp3_en  ) = "enstrophy"
	  qp3_amip_sampling(iqp3_en) = .true.
        qp3_type(iqp3_en ) = "dyn"

#ifdef passive_tracers
        do n = 1,npassive
           if(n.le.9) then
              write(unit=tnumber,fmt='(i1)')n
           else
              write(unit=tnumber,fmt='(i2)')n
           endif
           qp3_name (iqp3_passive(n)) = "tracer"//trim(tnumber)
           qp3_units(iqp3_passive(n)) = "kg/kg"
           qp3_descr(iqp3_passive(n)) = "Passive tracer "//trim(tnumber)
 	   qp3_amip_sampling(iqp3_passive(n)) = .false.
           qp3_type(iqp3_passive(n)) = "dyn"

           qp3_name (iqp3_dtrccup(n)) = "dtrccup"//trim(tnumber)
	     qp3_units(iqp3_dtrccup(n)) = "kg/kg/s"
	     qp3_descr(iqp3_dtrccup(n)) =   
     *     ":cup: cloud-top detrainment of passive tracer"//trim(tnumber)
           qp3_type(iqp3_dtrccup(n)) = "cup"

           qp3_name (iqp3_tcuptrc(n)) = "tcuptrc"//trim(tnumber)
	     qp3_units(iqp3_tcuptrc(n)) = "kg/kg/s"
	     qp3_descr(iqp3_tcuptrc(n)) =   
     *     ":cup: large-scale subsidence of passive tracer"//trim(tnumber)
           qp3_type(iqp3_tcuptrc(n)) = "cup"
        enddo
#endif

        qp3_name (iqp3_cke) = "cke"
        qp3_units(iqp3_cke) = "J/m^2"
        qp3_descr(iqp3_cke) = ":cup: convective kinetic energy"
        qp3_type(iqp3_cke) = "cup"
        
        qp3_name (iqp3_cdwork) = "cdwork"
        qp3_units(iqp3_cdwork) = "J/kg"
        qp3_descr(iqp3_cdwork) = ":cup: convective cloud work function"
        qp3_type(iqp3_cdwork) = "cup"

        qp3_name (iqp3_masflx) = "masflx"
        qp3_units(iqp3_masflx) = "kg/kg/s"
        qp3_descr(iqp3_masflx) = ":cup: convective mass flux"
        qp3_type(iqp3_masflx) = "cup"

        qp3_name (iqp3_tntmc) = "tntmc"
        qp3_units(iqp3_tntmc) = "K/s"
        qp3_descr(iqp3_tntmc) =
     *   ":cup: temperature tendency due to moist convective processes"
        qp3_mjo_link(iqp3_tntmc) = imjo3_tntlat
        qp3_type(iqp3_tntmc) = "cup"
     
        qp3_name (iqp3_tnmrc) = "tnmrc"
        qp3_units(iqp3_tnmrc) = "kg/kg/s"
        qp3_descr(iqp3_tnmrc) =
     *   ":cup: moisture tendency due to convective processes"
        qp3_type(iqp3_tnmrc) = "cup"
     
        qp3_name (iqp3_dhcup) = "dhcup"
        qp3_units(iqp3_dhcup) = "J/kg/s"
        qp3_descr(iqp3_dhcup) =
     *   ":cup: cloud-top detrainment of moist static energy"
        qp3_type(iqp3_dhcup) = "cup"

        qp3_name (iqp3_dvcup) = "dvcup"
        qp3_units(iqp3_dvcup) = "kg/kg/s"
        qp3_descr(iqp3_dvcup) =
     *   ":cup: cloud-top detrainment of water vapor"
        qp3_mjo_link(iqp3_dvcup) = imjo3_tnqlat
        qp3_type(iqp3_dvcup) = "cup"

        qp3_name (iqp3_dlcup) = "dlcup"
        qp3_units(iqp3_dlcup) = "kg/kg/s"
        qp3_descr(iqp3_dlcup) =
     *   ":cup: cloud-top detrainment of cloud water"
        qp3_type(iqp3_dlcup) = "cup"
 
        qp3_name (iqp3_drcup) = "drcup"
        qp3_units(iqp3_drcup) = "kg/kg/s"
        qp3_descr(iqp3_drcup) =
     *   ":cup: cloud-top detrainment of rain"
        qp3_type(iqp3_drcup) = "cup"

        qp3_name (iqp3_dicup) = "dicup"
        qp3_units(iqp3_dicup) = "kg/kg/s"
        qp3_descr(iqp3_dicup) =
     *   ":cup: cloud-top detrainment of cloud ice"
        qp3_type(iqp3_dicup) = "cup"

        qp3_name (iqp3_dscup) = "dscup"
        qp3_units(iqp3_dscup) = "kg/kg/s"
        qp3_descr(iqp3_dscup) =
     *   ":cup: cloud-top detrainment of snow"
        qp3_type(iqp3_dscup) = "cup"

        qp3_name (iqp3_dmcup) = "dmcup"
        qp3_units(iqp3_dmcup) = "per s"
        qp3_descr(iqp3_dmcup) =
     *   ":cup: cloud-top detrainment of cloudy air"
        qp3_type(iqp3_dmcup) = "cup"

        qp3_name (iqp3_dthcup_ng) = "dthcup_ng"
        qp3_units(iqp3_dthcup_ng) = "K/s"
        qp3_descr(iqp3_dthcup_ng) =
     *   ":cup: cloud-top detrainment of cld minus clr potential temperature"
        qp3_type(iqp3_dthcup_ng) = "cup"

        qp3_name (iqp3_dvcup_ng) = "dvcup_ng"
        qp3_units(iqp3_dvcup_ng) = "kg/kg/s"
        qp3_descr(iqp3_dvcup_ng) =
     *   ":cup: cloud-top detrainment of cld minus clr water vapor"
        qp3_type(iqp3_dvcup_ng) = "cup"

        qp3_name (iqp3_drcup_ng) = "drcup_ng"
        qp3_units(iqp3_drcup_ng) = "kg/kg/s"
        qp3_descr(iqp3_drcup_ng) =
     *   ":cup: cloud-top detrainment of cld minus clr rain"
        qp3_type(iqp3_drcup_ng) = "cup"

        qp3_name (iqp3_dscup_ng) = "dscup_ng"
        qp3_units(iqp3_dscup_ng) = "kg/kg/s"
        qp3_descr(iqp3_dscup_ng) =
     *   ":cup: cloud-top detrainment of cld minus clr snow"
        qp3_type(iqp3_dscup_ng) = "cup"

        qp3_name (iqp3_tcuph) = "tcuph"
        qp3_units(iqp3_tcuph) = "J/kg/s"
        qp3_descr(iqp3_tcuph) =
     *   ":cup: ls compensating subsidence of moist static energy"
        qp3_type(iqp3_tcuph) = "cup"

        qp3_name (iqp3_tcupv) = "tcupv"
        qp3_units(iqp3_tcupv) = "kg/kg/s"
        qp3_descr(iqp3_tcupv) =
     *   ":cup: ls compensating subsidence of water vapor"
        qp3_mjo_link(iqp3_tcupv) = imjo3_tnqlat
        qp3_type(iqp3_tcupv) = "cup"

        qp3_name (iqp3_tcupl) = "tcupl"
        qp3_units(iqp3_tcupl) = "kg/kg/s"
        qp3_descr(iqp3_tcupl) =
     *   ":cup: ls compensating subsidence of cloud water"
        qp3_type(iqp3_tcupl) = "cup"

        qp3_name (iqp3_tcupr) = "tcupr"
        qp3_units(iqp3_tcupr) = "kg/kg/s"
        qp3_descr(iqp3_tcupr) =
     *   ":cup: ls compensating subsidence of rain"
        qp3_type(iqp3_tcupr) = "cup"

        qp3_name (iqp3_tcupi) = "tcupi"
        qp3_units(iqp3_tcupi) = "kg/kg/s"
        qp3_descr(iqp3_tcupi) =
     *   ":cup: ls compensating subsidence of cloud ice"
        qp3_type(iqp3_tcupi) = "cup"

        qp3_name (iqp3_tcups) = "tcups"
        qp3_units(iqp3_tcups) = "kg/kg/s"
        qp3_descr(iqp3_tcups) =
     *   ":cup: ls compensating subsidence of snow"
        qp3_type(iqp3_tcups) = "cup"

        qp3_name (iqp3_tcupm) = "tcupm"
        qp3_units(iqp3_tcupm) = "per s"
        qp3_descr(iqp3_tcupm) =
     *   ":cup: ls compensating subsidence of cloudy air"
        qp3_type(iqp3_tcupm) = "cup"

        qp3_name (iqp3_tcupth_cld) = "tcupth_cld"
        qp3_units(iqp3_tcupth_cld) = "K/s"
        qp3_descr(iqp3_tcupth_cld) =
     *   ":cup: ls compensating subsidence of cldmclr potential temp in cld"
        qp3_type(iqp3_tcupth_cld) = "cup"

        qp3_name (iqp3_tcupth_clr) = "tcupth_clr"
        qp3_units(iqp3_tcupth_clr) = "K/s"
        qp3_descr(iqp3_tcupth_clr) =
     *   ":cup: ls compensating subsidence of cldmclr potential temp in clr"
        qp3_type(iqp3_tcupth_clr) = "cup"

        qp3_name (iqp3_tcupv_cld) = "tcupv_cld"
        qp3_units(iqp3_tcupv_cld) = "kg/kg/s"
        qp3_descr(iqp3_tcupv_cld) =
     *   ":cup: ls compensating subsidence of cldmclr water vapor in cld"
        qp3_type(iqp3_tcupv_cld) = "cup"

        qp3_name (iqp3_tcupv_clr) = "tcupv_clr"
        qp3_units(iqp3_tcupv_clr) = "kg/kg/s"
        qp3_descr(iqp3_tcupv_clr) =
     *   ":cup: ls compensating subsidence of cldmclr water vapor in clr"
        qp3_type(iqp3_tcupv_clr) = "cup"

        qp3_name (iqp3_tcupr_cld) = "tcupr_cld"
        qp3_units(iqp3_tcupr_cld) = "kg/kg/s"
        qp3_descr(iqp3_tcupr_cld) =
     *   ":cup: ls compensating subsidence of cldmclr rain in cld"
        qp3_type(iqp3_tcupr_cld) = "cup"

        qp3_name (iqp3_tcupr_clr) = "tcupr_clr"
        qp3_units(iqp3_tcupr_clr) = "kg/kg/s"
        qp3_descr(iqp3_tcupr_clr) =
     *   ":cup: ls compensating subsidence of cldmclr rain in clr"
        qp3_type(iqp3_tcupr_clr) = "cup"

        qp3_name (iqp3_tcups_cld) = "tcups_cld"
        qp3_units(iqp3_tcups_cld) = "kg/kg/s"
        qp3_descr(iqp3_tcups_cld) =
     *   ":cup: ls compensating subsidence of cldmclr snow in cld"
        qp3_type(iqp3_tcups_cld) = "cup"

        qp3_name (iqp3_tcups_clr) = "tcups_clr"
        qp3_units(iqp3_tcups_clr) = "kg/kg/s"
        qp3_descr(iqp3_tcups_clr) =
     *   ":cup: ls compensating subsidence of cldmclr snow in clr"
        qp3_type(iqp3_tcups_clr) = "cup"

        qp3_name (iqp3_clw) = "clw"
        qp3_units(iqp3_clw) = "kg/kg"
        qp3_descr(iqp3_clw) = ":elq: cloud liquid water"
        qp3_type(iqp3_clw) = "elq"

        qp3_name (iqp3_rwelq) = "rwelq"
        qp3_units(iqp3_rwelq) = "kg/kg"
        qp3_descr(iqp3_rwelq) = ":elq: rain mixing ratio"
        qp3_type(iqp3_rwelq) = "elq"

        qp3_name (iqp3_cli) = "cli"
        qp3_units(iqp3_cli) = "kg/kg"
        qp3_descr(iqp3_cli) = ":elq: cloud ice"
        qp3_type(iqp3_cli) = "elq"

        qp3_name (iqp3_rielq) = "rielq"
        qp3_units(iqp3_rielq) = "kg/kg"
        qp3_descr(iqp3_rielq) = ":elq: snow mixing ratio"
        qp3_type(iqp3_rielq) = "elq"

        qp3_name (iqp3_hus) = "hus"
        qp3_units(iqp3_hus) = "kg/kg"
        qp3_descr(iqp3_hus) = ":elq: specific humidity"
	  qp3_amip_sampling(iqp3_hus) = .true.
        qp3_type(iqp3_hus) = "elq"

        qp3_name (iqp3_tntlsp) = "tntlsp"
        qp3_units(iqp3_tntlsp) = "K/s"
        qp3_descr(iqp3_tntlsp) =
     *   ":elq: temperature tendency due to stratiform clouds"
        qp3_mjo_link(iqp3_tntlsp) = imjo3_tntlat
        qp3_type(iqp3_tntlsp) = "elq"

        qp3_name (iqp3_dqelq) = "dqelq"
        qp3_units(iqp3_dqelq) = "kg/kg/s"
        qp3_descr(iqp3_dqelq) = ":elq: large-scale tendency of total water"
        qp3_type(iqp3_dqelq) = "elq"

        qp3_name (iqp3_dvelq) = "dvelq"
        qp3_units(iqp3_dvelq) = "kg/kg/s"
        qp3_descr(iqp3_dvelq) = ":elq: large-scale tendency of water vapor"
        qp3_mjo_link(iqp3_dvelq)  = imjo3_tnqlat
        qp3_type(iqp3_dvelq) = "elq"

        qp3_name (iqp3_dlelq) = "dlelq"
        qp3_units(iqp3_dlelq) = "kg/kg/s"
        qp3_descr(iqp3_dlelq) = ":elq: large-scale tendency of cloud water"
        qp3_type(iqp3_dlelq) = "elq"

        qp3_name (iqp3_drelq) = "drelq"
        qp3_units(iqp3_drelq) = "kg/kg/s"
        qp3_descr(iqp3_drelq) = ":elq: large-scale tendency of rain"
        qp3_type(iqp3_drelq) = "elq"

        qp3_name (iqp3_dielq) = "dielq"
        qp3_units(iqp3_dielq) = "kg/kg/s"
        qp3_descr(iqp3_dielq) = ":elq: large-scale tendency of cloud ice"
        qp3_type(iqp3_dielq) = "elq"

        qp3_name (iqp3_dselq) = "dselq"
        qp3_units(iqp3_dselq) = "kg/kg/s"
        qp3_descr(iqp3_dselq) = ":elq: large-scale tendency of snow"
        qp3_type(iqp3_dselq) = "elq"
     
        qp3_name (iqp3_drwfelq) = "drwfelq"
        qp3_units(iqp3_drwfelq) = "kg/kg/s"
        qp3_descr(iqp3_drwfelq) =
     *   ":elq: tendency of rain due to precipitation"
        qp3_type(iqp3_drwfelq) = "elq"

        qp3_name (iqp3_drifelq) = "drifelq"
        qp3_units(iqp3_drifelq) = "kg/kg/s"
        qp3_descr(iqp3_drifelq) =
     *   ":elq: tendency of snow due to precipitation"
        qp3_type(iqp3_drifelq) = "elq"

        qp3_name (iqp3_cctpelq) = "cctpelq"
        qp3_units(iqp3_cctpelq) = "K"
        qp3_descr(iqp3_cctpelq) =
     *   ":elq: cloudy minus cloud-free temperature"
        qp3_type(iqp3_cctpelq) = "elq"

        qp3_name (iqp3_ccwvelq) = "ccwvelq"
        qp3_units(iqp3_ccwvelq) = "kg/kg"
        qp3_descr(iqp3_ccwvelq) =
     *   ":elq: cloudy minus cloud-free water vapor mixing ratio"
        qp3_type(iqp3_ccwvelq) = "elq"

        qp3_name (iqp3_ccrwelq) = "ccrwelq"
        qp3_units(iqp3_ccrwelq) = "kg/kg"
        qp3_descr(iqp3_ccrwelq) =
     *   ":elq: cloudy minus cloud-free rain mixing ratio"
        qp3_type(iqp3_ccrwelq) = "elq"

        qp3_name (iqp3_ccrielq) = "ccrielq"
        qp3_units(iqp3_ccrielq) = "kg/kg"
        qp3_descr(iqp3_ccrielq) =
     *   ":elq: cloudy minus cloud-free snow mixing ratio"
        qp3_type(iqp3_ccrielq) = "elq"

        qp3_name (iqp3_ccvvelq) = "ccvvelq"
        qp3_units(iqp3_ccvvelq) = "Pa/s"
        qp3_descr(iqp3_ccvvelq) =
     *   ":elq: cloudy minus cloud-free vertical velocity"
        qp3_type(iqp3_ccvvelq) = "elq"

        qp3_name (iqp3_acldelq) = "acldelq"
        qp3_units(iqp3_acldelq) = " "
        qp3_descr(iqp3_acldelq) =
     *   ":elq: cloud fraction"
        qp3_type(iqp3_acldelq) = "elq"

        qp3_name (iqp3_acldtelq) = "acldtelq"
        qp3_units(iqp3_acldtelq) = " "
        qp3_descr(iqp3_acldtelq) =
     *   ":elq: accumulated cloud fraction"
        qp3_type(iqp3_acldtelq) = "elq"

        qp3_name(iqp3_dcctpelq) = "dcctpelq"
        qp3_units(iqp3_dcctpelq) = "K/s"
        qp3_descr(iqp3_dcctpelq) =
     *   ":elq: tendency of cld minus clr temperature"
        qp3_type(iqp3_dcctpelq) = "elq"

        qp3_name (iqp3_dccwvelq) = "dccwvelq"
        qp3_units(iqp3_dccwvelq) = "kg/kg/s"
        qp3_descr(iqp3_dccwvelq) =
     *   ":elq: tendency of cld minus clr water vapor mixing ratio"
        qp3_type(iqp3_dccwvelq) = "elq"

        qp3_name (iqp3_dccrwelq) = "dccrwelq"
        qp3_units(iqp3_dccrwelq) = "kg/kg/s"
        qp3_descr(iqp3_dccrwelq) =
     *   ":elq: tendency of cld minus clr rain mixing ratio"
        qp3_type(iqp3_dccrwelq) = "elq"

        qp3_name (iqp3_dccrielq) = "dccrielq"
        qp3_units(iqp3_dccrielq) = "kg/kg/s"
        qp3_descr(iqp3_dccrielq) =
     *   ":elq: tendency of cld minus clr snow mixing ratio"
        qp3_type(iqp3_dccrielq) = "elq"

        qp3_name (iqp3_dccvvelq) = "dccvvelq"
        qp3_units(iqp3_dccvvelq) = "Pa/s2"
        qp3_descr(iqp3_dccvvelq) =
     *   ":elq: tendency of cld minus clr vertical velocity"
        qp3_type(iqp3_dccvvelq) = "elq"

        qp3_name (iqp3_dacldelq) = "dacldelq"
        qp3_units(iqp3_dacldelq) = "per second"
        qp3_descr(iqp3_dacldelq) = ":elq: tendency of cloud fraction"
        qp3_type(iqp3_dacldelq) = "elq"

        qp3_name (iqp3_cld) = "cld"
        qp3_units(iqp3_cld) = "%"
        qp3_descr(iqp3_cld) = ":rad cf: fraction of stratiform clouds"
        qp3_type(iqp3_cld) = "rad"
     
        qp3_name (iqp3_clu) = "clu"
        qp3_units(iqp3_clu) = "%"
        qp3_descr(iqp3_clu) = ":rad cf: fraction of convective clouds"
        qp3_type(iqp3_clu) = "rad"
     
        qp3_name (iqp3_cldw) = "cldw"
        qp3_units(iqp3_cldw) = "%"
        qp3_descr(iqp3_cldw) = ":rad cf: fraction of water clouds"
        qp3_type(iqp3_cldw) = "rad"
     
        qp3_name (iqp3_cldi) = "cldi"
        qp3_units(iqp3_cldi) = "%"
        qp3_descr(iqp3_cldi) = ":rad cf: fraction of ice clouds"
        qp3_type(iqp3_cldi) = "rad"

        qp3_name (iqp3_cldm) = "cldm"
        qp3_units(iqp3_cldm) = "%"
        qp3_descr(iqp3_cldm) = ":rad cf: fraction of mixed phase clouds"
        qp3_type(iqp3_cldm) = "rad"

        qp3_name (iqp3_effrw) = "effrw"
        qp3_units(iqp3_effrw) = "microns"
        qp3_descr(iqp3_effrw) = ":rad cf: effective radius of water clouds"
        qp3_type(iqp3_effrw) = "rad"

        qp3_name (iqp3_effri) = "effri"
        qp3_units(iqp3_effri) = "microns"
        qp3_descr(iqp3_effri) = ":rad cf: effective radius of ice clouds"
        qp3_type(iqp3_effri) = "rad"

        qp3_name (iqp3_codwvis) = "codwvis"
        qp3_units(iqp3_codwvis) = "per Pa"
        qp3_descr(iqp3_codwvis) =
     *   ":rad cf: optical depth of water clouds - visible"
        qp3_type(iqp3_codwvis) = "rad"

        qp3_name (iqp3_codivis) = "codivis"
        qp3_units(iqp3_codivis) = "per Pa"
        qp3_descr(iqp3_codivis) =
     *   ":rad cf: optical depth of ice clouds - visible"
        qp3_type(iqp3_codivis) = "rad"

        qp3_name (iqp3_codmvis) = "codmvis"
        qp3_units(iqp3_codmvis) = "per Pa"
        qp3_descr(iqp3_codmvis) =
     *   ":rad cf: optical depth of mixed-phase clouds - visible"
        qp3_type(iqp3_codmvis) = "rad"

        qp3_name (iqp3_codtvis) = "codtvis"
        qp3_units(iqp3_codtvis) = "per Pa"
        qp3_descr(iqp3_codtvis) =
     *   ":rad cf: optical depth of total cloud fraction - visible"
        qp3_type(iqp3_codtvis) = "rad"

        qp3_name (iqp3_codwirw) = "codwirw"
        qp3_units(iqp3_codwirw) = "per Pa"
        qp3_descr(iqp3_codwirw) =
     *   ":rad cf: optical depth of water clouds - infrared window"
        qp3_type(iqp3_codwirw) = "rad"

        qp3_name (iqp3_codiirw) = "codiirw"
        qp3_units(iqp3_codiirw) = "per Pa"
        qp3_descr(iqp3_codiirw) =
     *   ":rad cf: optical depth of ice clouds - infrared window"
        qp3_type(iqp3_codiirw) = "rad"

        qp3_name (iqp3_codmirw) = "codmirw"
        qp3_units(iqp3_codmirw) = "per Pa"
        qp3_descr(iqp3_codmirw) =
     *   ":rad cf: optical depth of mixed-phase clouds - infrared window"
        qp3_type(iqp3_codmirw) = "rad"

        qp3_name (iqp3_codtirw) = "codtirw"
        qp3_units(iqp3_codtirw) = "per Pa"
        qp3_descr(iqp3_codtirw) =
     *   ":rad cf: optical depth of total cloud fraction - infrared window"
        qp3_type(iqp3_codtirw) = "rad"

        qp3_name (iqp3_tnmmugwd    ) = "tnmmugwd"
        qp3_units(iqp3_tnmmugwd     ) = "m/s^2"
        qp3_descr(iqp3_tnmmugwd     ) = "Eastward momentum tendency due to gravity wave drag"
        qp3_type(iqp3_tnmmugwd ) = "dyn"

        qp3_name (iqp3_tnmmvgwd    ) = "tnmmvgwd"
        qp3_units(iqp3_tnmmvgwd     ) = "m/s^2"
        qp3_descr(iqp3_tnmmvgwd     ) = "Northward momentum tendency due to gravity wave drag"
        qp3_type(iqp3_tnmmvgwd ) = "dyn"

        qp3_name (iqp3_tnmmuc    ) = "tnmmuc"
        qp3_units(iqp3_tnmmuc     ) = "m/s^2"
        qp3_descr(iqp3_tnmmuc     ) = "Eastward momentum tendency due to convection"
        qp3_type(iqp3_tnmmuc ) = "cup"

        qp3_name (iqp3_tnmmvc    ) = "tnmmvc"
        qp3_units(iqp3_tnmmvc     ) = "m/s^2"
        qp3_descr(iqp3_tnmmvc     ) = "Northward momentum tendency due to convection"
        qp3_type(iqp3_tnmmvc ) = "cup"

! this is an example for products and covariances: 
!     products occur with a pair of fields
        qp3_name (iqp3_cvuva) = "cvuva"
        qp3_units(iqp3_cvuva) = "m2/s2"
        qp3_descr(iqp3_cvuva) = "Covariance of eastward and northward winds"
        qp3_prodlog(iqp3_cvuva) = .true.
        qp3_prodindx(iqp3_cvuva,1) = iqp3_ua
        qp3_prodindx(iqp3_cvuva,2) = iqp3_va
	  qp3_amip_sampling(iqp3_cvuva) = .true.
        qp3_type(iqp3_cvuva) = "dyn"

        qp3_name (iqp3_uva_bar) = "uva_bar"
        qp3_units(iqp3_uva_bar) = "m2/s2"
        qp3_descr(iqp3_uva_bar) = "product of mean eastward and northward winds"
        qp3_prodlog(iqp3_uva_bar) = .true.
        qp3_prodindx(iqp3_uva_bar,1) = iqp3_ua
        qp3_prodindx(iqp3_uva_bar,2) = iqp3_va
	  qp3_amip_sampling(iqp3_uva_bar) = .true.
        qp3_type(iqp3_uva_bar) = "dyn"

        qp3_name (iqp3_cvuta) = "cvuta"
        qp3_units(iqp3_cvuta) = "K m/s"
        qp3_descr(iqp3_cvuta) = "Covariance of eastward wind and temperature"
        qp3_prodlog(iqp3_cvuta) = .true.
        qp3_prodindx(iqp3_cvuta,1) = iqp3_ua
        qp3_prodindx(iqp3_cvuta,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_cvuta) = .true.
        qp3_type(iqp3_cvuta) = "dyn"

        qp3_name (iqp3_uta_bar) = "uta_bar"
        qp3_units(iqp3_uta_bar) = "K m/s"
        qp3_descr(iqp3_uta_bar) = 
     &           "product of mean eastward wind and temperature"
        qp3_prodlog(iqp3_uta_bar) = .true.
        qp3_prodindx(iqp3_uta_bar,1) = iqp3_ua
        qp3_prodindx(iqp3_uta_bar,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_uta_bar) = .true.
        qp3_type(iqp3_uta_bar) = "dyn"

        qp3_name (iqp3_cvvta) = "cvvta"
        qp3_units(iqp3_cvvta) = "K m/s"
        qp3_descr(iqp3_cvvta) = "Covariance of northward wind and temperature"
        qp3_prodlog(iqp3_cvvta) = .true.
        qp3_prodindx(iqp3_cvvta,1) = iqp3_va
        qp3_prodindx(iqp3_cvvta,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_cvvta) = .true.
        qp3_type(iqp3_cvvta) = "dyn"

        qp3_name (iqp3_vta_bar) = "vta_bar"
        qp3_units(iqp3_vta_bar) = "K m/s"
        qp3_descr(iqp3_vta_bar) = 
     &           "Product of mean northward wind and temperature"
        qp3_prodlog(iqp3_vta_bar) = .true.
        qp3_prodindx(iqp3_vta_bar,1) = iqp3_va
        qp3_prodindx(iqp3_vta_bar,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_vta_bar) = .true.
        qp3_type(iqp3_vta_bar) = "dyn"

        qp3_name (iqp3_cvwapta) = "cvwapta"
        qp3_units(iqp3_cvwapta) = "K Pa/s"
        qp3_descr(iqp3_cvwapta) = 
     &         "Covariance of vertical motion and temperature"
        qp3_prodlog(iqp3_cvwapta) = .true.
        qp3_prodindx(iqp3_cvwapta,1) = iqp3_wap
        qp3_prodindx(iqp3_cvwapta,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_cvwapta) = .true.
        qp3_type(iqp3_cvwapta) = "dyn"

        qp3_name (iqp3_wapta_bar) = "wapta_bar"
        qp3_units(iqp3_wapta_bar) = "K Pa/s"
        qp3_descr(iqp3_wapta_bar) = 
     &        "Product of mean vertical motion and temperature"
        qp3_prodlog(iqp3_wapta_bar) = .true.
        qp3_prodindx(iqp3_wapta_bar,1) = iqp3_wap
        qp3_prodindx(iqp3_wapta_bar,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_wapta_bar) = .true.
        qp3_type(iqp3_wapta_bar) = "dyn"

        qp3_name (iqp3_cvvzeta) = "cvvzeta"
        qp3_units(iqp3_cvvzeta) = "m/s2"
        qp3_descr(iqp3_cvvzeta) = 
     &         "Covariance of northward wind and relative vorticity"
        qp3_prodlog(iqp3_cvvzeta) = .true.
        qp3_prodindx(iqp3_cvvzeta,1) = iqp3_va
        qp3_prodindx(iqp3_cvvzeta,2) = iqp3_vorticity
	  qp3_amip_sampling(iqp3_cvvzeta) = .true.
        qp3_type(iqp3_cvvzeta) = "dyn"

        qp3_name (iqp3_vzeta_bar) = "vzeta_bar"
        qp3_units(iqp3_vzeta_bar) = "m/s2"
        qp3_descr(iqp3_vzeta_bar) = 
     &       "Product of mean northward wind and relative vorticity"
        qp3_prodlog(iqp3_vzeta_bar) = .true.
        qp3_prodindx(iqp3_vzeta_bar,1) = iqp3_va
        qp3_prodindx(iqp3_vzeta_bar,2) = iqp3_vorticity
	  qp3_amip_sampling(iqp3_vzeta_bar) = .true.
        qp3_type(iqp3_vzeta_bar) = "dyn"

        qp3_name (iqp3_cvuhusa) = "cvuhusa"
        qp3_units(iqp3_cvuhusa) = "kg/kg * m/s"
        qp3_descr(iqp3_cvuhusa) =
     &       "Covariance of eastward wind and specific humidity"
        qp3_prodlog(iqp3_cvuhusa) = .true.
        qp3_prodindx(iqp3_cvuhusa,1) = iqp3_ua
        qp3_prodindx(iqp3_cvuhusa,2) = iqp3_hus
	  qp3_amip_sampling(iqp3_cvuhusa) = .true.
        qp3_type(iqp3_cvuhusa) = "dyn"

        qp3_name (iqp3_uhusa_bar) = "uhusa_bar"
        qp3_units(iqp3_uhusa_bar) = "kg/kg * m/s"
        qp3_descr(iqp3_uhusa_bar) = 
     &      "Product of mean eastward wind and specific humidity"
        qp3_prodlog(iqp3_uhusa_bar) = .true.
        qp3_prodindx(iqp3_uhusa_bar,1) = iqp3_ua
        qp3_prodindx(iqp3_uhusa_bar,2) = iqp3_hus
	  qp3_amip_sampling(iqp3_uhusa_bar) = .true.
        qp3_type(iqp3_uhusa_bar) = "dyn"

        qp3_name (iqp3_cvvhusa) = "cvvhusa"
        qp3_units(iqp3_cvvhusa) = "kg/kg * m/s"
        qp3_descr(iqp3_cvvhusa) =
     &       "Covariance of northward wind and specific humidity"
        qp3_prodlog(iqp3_cvvhusa) = .true.
        qp3_prodindx(iqp3_cvvhusa,1) = iqp3_va
        qp3_prodindx(iqp3_cvvhusa,2) = iqp3_hus
	  qp3_amip_sampling(iqp3_cvvhusa) = .true.
        qp3_type(iqp3_cvvhusa) = "dyn"

        qp3_name (iqp3_vhusa_bar) = "vhusa_bar"
        qp3_units(iqp3_vhusa_bar) = "kg/kg * m/s"
        qp3_descr(iqp3_vhusa_bar) = 
     &      "Product of mean northward wind and specific humidity"
        qp3_prodlog(iqp3_vhusa_bar) = .true.
        qp3_prodindx(iqp3_vhusa_bar,1) = iqp3_va
        qp3_prodindx(iqp3_vhusa_bar,2) = iqp3_hus
	  qp3_amip_sampling(iqp3_vhusa_bar) = .true.
        qp3_type(iqp3_vhusa_bar) = "dyn"

        qp3_name (iqp3_cvwhusa) = "cvwhusa"
        qp3_units(iqp3_cvwhusa) = "kg/kg * Pa/s"
        qp3_descr(iqp3_cvwhusa) = 
     &       "Covariance of vertical motion and specific humidity"
        qp3_prodlog(iqp3_cvwhusa) = .true.
        qp3_prodindx(iqp3_cvwhusa,1) = iqp3_wap
        qp3_prodindx(iqp3_cvwhusa,2) = iqp3_hus
	  qp3_amip_sampling(iqp3_cvwhusa) = .true.
        qp3_type(iqp3_cvwhusa) = "dyn"

        qp3_name (iqp3_whusa_bar) = "whusa_bar"
        qp3_units(iqp3_whusa_bar) = "kg/kg * Pa/s"
        qp3_descr(iqp3_whusa_bar) = 
     &      "Product of mean vertical motion and specific humidity"
        qp3_prodlog(iqp3_whusa_bar) = .true.
        qp3_prodindx(iqp3_whusa_bar,1) = iqp3_wap
        qp3_prodindx(iqp3_whusa_bar,2) = iqp3_hus
	  qp3_amip_sampling(iqp3_whusa_bar) = .true.
        qp3_type(iqp3_whusa_bar) = "dyn"

        qp3_name (iqp3_cvuua) = "cvuua"
        qp3_units(iqp3_cvuua) = "m2/s2"
        qp3_descr(iqp3_cvuua) = "Covariance of eastward wind and eastward wind"
        qp3_prodlog(iqp3_cvuua) = .true.
        qp3_prodindx(iqp3_cvuua,1) = iqp3_ua
        qp3_prodindx(iqp3_cvuua,2) = iqp3_ua
	  qp3_amip_sampling(iqp3_cvuua) = .true.
        qp3_type(iqp3_cvuua) = "dyn"

        qp3_name (iqp3_uua_bar) = "uua_bar"
        qp3_units(iqp3_uua_bar) = "m2/s2"
        qp3_descr(iqp3_uua_bar) = 
     &       "product of mean eastward wind and eastward wind"
        qp3_prodlog(iqp3_uua_bar) = .true.
        qp3_prodindx(iqp3_uua_bar,1) = iqp3_ua
        qp3_prodindx(iqp3_uua_bar,2) = iqp3_ua
	  qp3_amip_sampling(iqp3_uua_bar) = .true.
        qp3_type(iqp3_uua_bar) = "dyn"

        qp3_name (iqp3_cvvva) = "cvvva"
        qp3_units(iqp3_cvvva) = "m2/s2"
        qp3_descr(iqp3_cvvva) = 
     &        "Covariance of northward wind and northward wind"
        qp3_prodlog(iqp3_cvvva) = .true.
        qp3_prodindx(iqp3_cvvva,1) = iqp3_va
        qp3_prodindx(iqp3_cvvva,2) = iqp3_va
	  qp3_amip_sampling(iqp3_cvvva) = .true.
        qp3_type(iqp3_cvvva) = "dyn"

        qp3_name (iqp3_vva_bar) = "vva_bar"
        qp3_units(iqp3_vva_bar) = "m2/s2"
        qp3_descr(iqp3_vva_bar) = 
     &       "product of mean northward wind and northward wind"
        qp3_prodlog(iqp3_vva_bar) = .true.
        qp3_prodindx(iqp3_vva_bar,1) = iqp3_va
        qp3_prodindx(iqp3_vva_bar,2) = iqp3_va
	  qp3_amip_sampling(iqp3_vva_bar) = .true.
        qp3_type(iqp3_vva_bar) = "dyn"

        qp3_name (iqp3_cvtta) = "cvtta"
        qp3_units(iqp3_cvtta) = "K2"
        qp3_descr(iqp3_cvtta) = "Covariance of temperature and temperature"
        qp3_prodlog(iqp3_cvtta) = .true.
        qp3_prodindx(iqp3_cvtta,1) = iqp3_ta
        qp3_prodindx(iqp3_cvtta,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_cvtta) = .true.
        qp3_type(iqp3_cvtta) = "dyn"

        qp3_name (iqp3_tta_bar) = "tta_bar"
        qp3_units(iqp3_tta_bar) = "K2"
        qp3_descr(iqp3_tta_bar) = 
     &       "product of mean temperature and temperature"
        qp3_prodlog(iqp3_tta_bar) = .true.
        qp3_prodindx(iqp3_tta_bar,1) = iqp3_ta
        qp3_prodindx(iqp3_tta_bar,2) = iqp3_ta
	  qp3_amip_sampling(iqp3_tta_bar) = .true.
        qp3_type(iqp3_tta_bar) = "dyn"

        qp3_name (iqp3_cvvzga) = "cvvzga"
        qp3_units(iqp3_cvvzga) = "m2/s"
        qp3_descr(iqp3_cvvzga) = 
     &        "Covariance of northward wind and geopotential height"
        qp3_prodlog(iqp3_cvvzga) = .true.
        qp3_prodindx(iqp3_cvvzga,1) = iqp3_va
        qp3_prodindx(iqp3_cvvzga,2) = iqp3_zg
	  qp3_amip_sampling(iqp3_cvvzga) = .true.
        qp3_type(iqp3_cvvzga) = "dyn"

        qp3_name (iqp3_vzga_bar) = "vzga_bar"
        qp3_units(iqp3_vzga_bar) = "m2/s"
        qp3_descr(iqp3_vzga_bar) = 
     &       "product of mean northward wind and geopotential height"
        qp3_prodlog(iqp3_vzga_bar) = .true.
        qp3_prodindx(iqp3_vzga_bar,1) = iqp3_va
        qp3_prodindx(iqp3_vzga_bar,2) = iqp3_zg
	  qp3_amip_sampling(iqp3_vzga_bar) = .true.
        qp3_type(iqp3_vzga_bar) = "dyn"



        if(.not.amip_sampling) then
           qp3_amip_sampling = .false.
           qp2_amip_sampling = .false.
        endif

!--------------------------------------------------------------------------
! start hf2s
!--------------------------------------------------------------------------
        hf2_name (ihf2_va850 ) = "va850"
        hf2_units(ihf2_va850  ) = "m/s"
        hf2_descr(ihf2_va850  ) = "Northward wind at 850 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_va850) = .true.

        hf2_name (ihf2_va200 ) = "va200"
        hf2_units(ihf2_va200  ) = "m/s"
        hf2_descr(ihf2_va200  ) = "Northward wind at 200 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_va200) = .true.

        hf2_name (ihf2_ua850 ) = "ua850"
        hf2_units(ihf2_ua850  ) = "m/s"
        hf2_descr(ihf2_ua850  ) = "Eastward wind at 850 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_ua850) = .true.

        hf2_name (ihf2_ua200 ) = "ua200"
        hf2_units(ihf2_ua200  ) = "m/s"
        hf2_descr(ihf2_ua200  ) = "Eastward wind at 200 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_ua200) = .true.

        hf2_name (ihf2_rlut  ) = "rlut"
        hf2_units(ihf2_rlut  ) = "W/m^2"
        hf2_descr(ihf2_rlut  ) = "Outgoing longwave radiation - 6 hourly"

        hf2_name (ihf2_psl   ) = "psl"
        hf2_units(ihf2_psl   ) = "Pa"
        hf2_descr(ihf2_psl   ) = "Mean-sea-level pressure - 6 hourly"
	hf2_amip_sampling(ihf2_psl) = .true.

        hf2_name (ihf2_tasmin) = "tasmin"
        hf2_units(ihf2_tasmin) = "K"
        hf2_descr(ihf2_tasmin) = "6 hourly minimum surface air temperature"

        hf2_name (ihf2_tasmax) = "tasmax"
        hf2_units(ihf2_tasmax) = "K"
        hf2_descr(ihf2_tasmax) = "6 hourly maximum surface air temperature"

        hf2_name (ihf2_prc   ) = "prc"
        hf2_units(ihf2_prc   ) = "kg/m2/s"
        hf2_descr(ihf2_prc   ) = 
     *   "-cup- convective precipitation rate - 6 hourly"

        hf2_name (ihf2_elqpr ) = "elqpr"
        hf2_units(ihf2_elqpr ) = "kg/m2/s"
        hf2_descr(ihf2_elqpr ) =
     *   "-elq- stratiform precipitation rate - 6 hourly"

        hf2_name (ihf2_pr    ) = "pr"
        hf2_units(ihf2_pr    ) = "kg/m2/s"
        hf2_descr(ihf2_pr    ) =
     *   "Total precipitation rate- 6 hourly"

        hf2_name (ihf2_ke    ) = "ke"
        hf2_units(ihf2_ke    ) = "J/m/2"
        hf2_descr(ihf2_ke    ) = 
     &   "vertically integrated kinetic energy - 6 hourly"
	  hf2_amip_sampling(ihf2_ke) = .true.

        hf2_name (ihf2_en    ) = "en"
        hf2_units(ihf2_en    ) = "kg/m2/s2"
        hf2_descr(ihf2_en    ) = "vertically integrated enstrophy"
	  hf2_amip_sampling(ihf2_en) = .true.

        hf2_name (ihf2_ke250 ) = "ke250"
        hf2_units(ihf2_ke250 ) = "J/kg"
        hf2_descr(ihf2_ke250 ) = "250 mb kinetic energy"
	  hf2_amip_sampling(ihf2_ke250) = .true.

        hf2_name (ihf2_en250 ) = "en250"
        hf2_units(ihf2_en250 ) = "s-2"
        hf2_descr(ihf2_en250 ) = "250 mb enstrophy"
	hf2_amip_sampling(ihf2_en250) = .true.

        hf2_name (ihf2_ta850 ) = "ta850"
        hf2_units(ihf2_ta850  ) = "K"
        hf2_descr(ihf2_ta850  ) = "Air temperature at 850 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_ta850) = .true.

        hf2_name (ihf2_ta500 ) = "ta500"
        hf2_units(ihf2_ta500  ) = "K"
        hf2_descr(ihf2_ta500  ) = "Air temperature at 500 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_ta500) = .true.

        hf2_name (ihf2_ta50 ) = "ta50"
        hf2_units(ihf2_ta50  ) = "K"
        hf2_descr(ihf2_ta50  ) = "Air temperature at 50 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_ta50) = .true.

        hf2_name (ihf2_zg500 ) = "zg500"
        hf2_units(ihf2_zg500  ) = "m"
        hf2_descr(ihf2_zg500  ) = "Geopotential height at 500 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_zg500) = .true.

        hf2_name (ihf2_hus850 ) = "hus850"
        hf2_units(ihf2_hus850  ) = "kg/kg"
        hf2_descr(ihf2_hus850  ) = "Specific humidity at 850 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_hus850) = .true.

        hf2_name (ihf2_hus500 ) = "hus500"
        hf2_units(ihf2_hus500  ) = "kg/kg"
        hf2_descr(ihf2_hus500  ) = "Specific humidity at 500 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_hus500) = .true.

        hf2_name (ihf2_uas ) = "uas"
        hf2_units(ihf2_uas  ) = "m/s"
        hf2_descr(ihf2_uas  ) = "Surface (10m) eastward wind - 6 hourly"
	hf2_amip_sampling(ihf2_uas) = .true.

        hf2_name (ihf2_vas ) = "vas"
        hf2_units(ihf2_vas  ) = "m/s"
        hf2_descr(ihf2_vas  ) = "Surface (10m) northward wind - 6 hourly"
	hf2_amip_sampling(ihf2_vas) = .true.

        hf2_name (ihf2_tas ) = "tas"
        hf2_units(ihf2_tas  ) = "K"
        hf2_descr(ihf2_tas  ) = "Surface (2m) temperature - 6 hourly"
	hf2_amip_sampling(ihf2_tas) = .true.

        hf2_name (ihf2_huss ) = "huss"
        hf2_units(ihf2_huss  ) = "kg/kg"
        hf2_descr(ihf2_huss  ) = "Surface (2m) specific humidity - 6 hourly"
	hf2_amip_sampling(ihf2_huss) = .true.

        hf2_name (ihf2_wap500 ) = "wap500"
        hf2_units(ihf2_wap500  ) = "Pa/s"
        hf2_descr(ihf2_wap500  ) = "Vertical motion at 500 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_wap500) = .true.

        hf2_name (ihf2_va50 ) = "va50"
        hf2_units(ihf2_va50  ) = "m/s"
        hf2_descr(ihf2_va50  ) = "Northward wind at 50 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_va50) = .true.

        hf2_name (ihf2_ua50 ) = "ua50"
        hf2_units(ihf2_ua50  ) = "m/s"
        hf2_descr(ihf2_ua50  ) = "Eastward wind at 50 hPa - 6 hourly"
	hf2_amip_sampling(ihf2_ua50) = .true.

        hf2_name (ihf2_ps ) = "ps"
        hf2_units(ihf2_ps  ) = "Pa"
        hf2_descr(ihf2_ps  ) = "Surface pressure - 6 hourly"
	hf2_amip_sampling(ihf2_ps) = .true.

        hf2_name (ihf2_clt ) = "clt"
        hf2_units(ihf2_clt  ) = "%"
        hf2_descr(ihf2_clt  ) = "Total cloud cover - 6 hourly"

        hf2_name (ihf2_tauu ) = "tauu"
        hf2_units(ihf2_tauu  ) = "N/m2"
        hf2_descr(ihf2_tauu  ) = "Eastward surface wind stress - 6 hourly"

        hf2_name (ihf2_tauv ) = "tauv"
        hf2_units(ihf2_tauv  ) = "N/m2"
        hf2_descr(ihf2_tauv  ) = "Northward surface wind stress - 6 hourly"

        hf2_name (ihf2_prw ) = "prw"
        hf2_units(ihf2_prw  ) = "kg/m2"
        hf2_descr(ihf2_prw  ) = "Precipitable water - 6 hourly"

        hf2_name (ihf2_hfss ) = "hfss"
        hf2_units(ihf2_hfss  ) = "W/m2"
        hf2_descr(ihf2_hfss  ) = "Surface sensible heat flux - 6 hourly"

        hf2_name (ihf2_hfls ) = "hfls"
        hf2_units(ihf2_hfls  ) = "W/m2"
        hf2_descr(ihf2_hfls  ) = "Surface latent heat flux - 6 hourly"

        hf2_name (ihf2_rsds ) = "rsds"
        hf2_units(ihf2_rsds  ) = "W/m2"
        hf2_descr(ihf2_rsds  ) = 
     &           "Surface incident shortwave radiation - 6 hourly"

        hf2_name (ihf2_rsus ) = "rsus"
        hf2_units(ihf2_rsus  ) = "W/m2"
        hf2_descr(ihf2_rsus  ) =
     &            "Surface reflected shortwave radiation - 6 hourly"

        hf2_name (ihf2_rlds ) = "rlds"
        hf2_units(ihf2_rlds  ) = "W/m2"
        hf2_descr(ihf2_rlds  ) = 
     &        "Surface downwelling longwave radiation - 6 hourly"

        hf2_name (ihf2_rlus ) = "rlus"
        hf2_units(ihf2_rlus  ) = "W/m2"
        hf2_descr(ihf2_rlus  ) = 
     &         "Surface upwelling longwave radiation - 6 hourly"

        hf2_name (ihf2_rsdt ) = "rsdt"
        hf2_units(ihf2_rsdt  ) = "W/m2"
        hf2_descr(ihf2_rsdt  ) = "TOA incident shortwave radiation - 6 hourly"

        hf2_name (ihf2_rsut ) = "rsut"
        hf2_units(ihf2_rsut  ) = "W/m2"
        hf2_descr(ihf2_rsut  ) = "TOA reflected shortwave radiation - 6 hourly"

        hf2_name (ihf2_zblh ) = "zblh"
        hf2_units(ihf2_zblh  ) = "m"
        hf2_descr(ihf2_zblh  ) = "Planetary boundary layer height - 6 hourly"
	hf2_amip_sampling(ihf2_zblh) = .true.

!--------------------------------------------------------------------------
! start mjo2s
!--------------------------------------------------------------------------
        mjo2_name (imjo2_rlut  ) = "rlut"
        mjo2_units(imjo2_rlut  ) = "W/m^2"
        mjo2_descr(imjo2_rlut  ) = 
     &          "RAD: TOA all-sky outgoing longwave radiation"

        mjo2_name (imjo2_chi200  ) = "chi200"
        mjo2_units(imjo2_chi200  ) = "m^2/s"
        mjo2_descr(imjo2_chi200  ) =  "200 mb velocity potential"

        mjo2_name (imjo2_elqwvt) = "elqwvt"
        mjo2_units(imjo2_elqwvt) = "kg/m^2"
        mjo2_descr(imjo2_elqwvt) =
     *   ":elq: vertically-integrated water vapor"

        mjo2_name (imjo2_cuppr ) = "cuppr"
        mjo2_units(imjo2_cuppr ) = "kg/m2/s"
        mjo2_descr(imjo2_cuppr ) = ":cup: cumulus precipitation rate"

        mjo2_name (imjo2_elqpr ) = "elqpr"
        mjo2_units(imjo2_elqpr ) = "kg/m2/s"
        mjo2_descr(imjo2_elqpr ) = ":elq: stratiform precipitation rate"

        mjo2_name (imjo2_evap) = "evap"
        mjo2_units(imjo2_evap) = "kg/m2/s"
        mjo2_descr(imjo2_evap) = "Surface evaporation plus sublimation rate"

        mjo2_name (imjo2_hfss  ) = "hfss"
        mjo2_units(imjo2_hfss  ) = "W/m^2"
        mjo2_descr(imjo2_hfss  ) = "SURFACE SENSIBLE HEAT FLUX"

        mjo2_name (imjo2_rss   ) = "rss"
        mjo2_units(imjo2_rss   ) = "W/m^2"
        mjo2_descr(imjo2_rss   ) = "RAD: SFC all-sky net shortwave radiation"

        mjo2_name (imjo2_rls   ) = "rls"
        mjo2_units(imjo2_rls   ) = "W/m^2"
        mjo2_descr(imjo2_rls   ) = "RAD: SFC all-sky net longwave radiation"

        mjo2_name (imjo2_huss  ) = "huss"
        mjo2_units(imjo2_huss  ) = "kg/kg"
        mjo2_descr(imjo2_huss  ) = "Surface (2m) specific humidity"

        mjo2_name (imjo2_tas   ) = "tas"
        mjo2_units(imjo2_tas   ) = "K"
        mjo2_descr(imjo2_tas   ) = "sfc air temperature"

        mjo2_name (imjo2_ts) = "ts "
        mjo2_units(imjo2_ts) = "K"
        mjo2_descr(imjo2_ts) = "surface lower boundary temperature"

        mjo2_name (imjo2_blp   ) = "blp"
        mjo2_units(imjo2_blp   ) = "Pa"
        mjo2_descr(imjo2_blp   ) = "PBL depth"

        mjo2_name (imjo2_uas) = "uas"
        mjo2_units(imjo2_uas) = "m/s"
        mjo2_descr(imjo2_uas) = "Surface (10m) eastward wind "

        mjo2_name (imjo2_vas) = "vas"
        mjo2_units(imjo2_vas) = "m/s"
        mjo2_descr(imjo2_vas) = "Surface (10m) northward wind"

        mjo2_name (imjo2_ps    ) = "ps"
        mjo2_units(imjo2_ps    ) = "Pa"
        mjo2_descr(imjo2_ps    ) = "Surface pressure"



!--------------------------------------------------------------------------
! start mjo3s
!--------------------------------------------------------------------------
        mjo3_name (imjo3_mask  ) = "mask"
        mjo3_units(imjo3_mask  ) = "-"
        mjo3_descr(imjo3_mask  ) = " "

        mjo3_name (imjo3_wap   ) = "wap"
        mjo3_units(imjo3_wap   ) = "Pa/s"
        mjo3_descr(imjo3_wap   ) = "Vertical motion"

        mjo3_name (imjo3_ua    ) = "ua"
        mjo3_units(imjo3_ua    ) = "m/s"
        mjo3_descr(imjo3_ua    ) = "Zonal wind"

        mjo3_name (imjo3_va    ) = "va"
        mjo3_units(imjo3_va    ) = "m/s"
        mjo3_descr(imjo3_va    ) = "Meriodional wind"

        mjo3_name (imjo3_tntlat    ) = "tntlat"
        mjo3_units(imjo3_tntlat    ) = "K/s"
        mjo3_descr(imjo3_tntlat    ) = 
     &       "Temperature tendency due to latent heating"

        mjo3_name (imjo3_tnqlat    ) = "tnqlat"
        mjo3_units(imjo3_tnqlat    ) = "kg/kg/s"
        mjo3_descr(imjo3_tnqlat    ) = 
     &      "Water vapor tendency due to latent processes"

        mjo3_name (imjo3_ta    ) = "ta"
        mjo3_units(imjo3_ta    ) = "K"
        mjo3_descr(imjo3_ta    ) = "Air Temperature"

        mjo3_name (imjo3_hus       ) = "hus"
        mjo3_units(imjo3_hus       ) = "kg/kg"
        mjo3_descr(imjo3_hus       ) = 
     *"EAUliqNG: Water vapor mixing ratio                             "

        mjo3_name (imjo3_tntrad    ) = "tntrad"
        mjo3_units(imjo3_tntrad    ) = "K/s"
        mjo3_descr(imjo3_tntrad    ) = 
     &       "Temperature tendency due to radiative heating"

        mjo3_name (imjo3_zg    ) = "zg"
        mjo3_units(imjo3_zg    ) = "m"
        mjo3_descr(imjo3_zg    ) = "Geopotential height"


	do n=1,nqp2
	qp2_name(n) = trim(qp2_name(n))
	qp2_units(n) = trim(qp2_units(n))
	qp2_descr(n) = trim(qp2_descr(n))
	qp2_type(n) = trim(qp2_type(n))
	qp2_altunits(n) = trim(qp2_altunits(n))
	enddo

	do n=1,nqp3
        qp3_name(n) = trim(qp3_name(n))
        qp3_units(n) = trim(qp3_units(n))
        qp3_descr(n) = trim(qp3_descr(n))
	qp3_type(n) = trim(qp3_type(n))
	enddo

	do n=1,nhf2
	hf2_name(n) = trim(hf2_name(n))
	hf2_units(n) = trim(hf2_units(n))
	hf2_descr(n) = trim(hf2_descr(n))
	hf2_type(n) = trim(hf2_type(n))
	enddo

	do n=1,nmjo2
	mjo2_name(n) = trim(mjo2_name(n))
	mjo2_units(n) = trim(mjo2_units(n))
	mjo2_descr(n) = trim(mjo2_descr(n))
	enddo

	do n=1,nmjo3
	mjo3_name(n) = trim(mjo3_name(n))
	mjo3_units(n) = trim(mjo3_units(n))
	mjo3_descr(n) = trim(mjo3_descr(n))
	enddo

!  initialize some qp tables
      qp2_indx = 0
      qp3_indx = 0
      hf2_indx = 0
      mjo2_indx = 0
      mjo3_indx = 0
      qp2_log = .false.
      qp3_log = .false.
      hf2_log = .true.
      mjo2_log = .true.
      mjo3_log = .true.

!  always save mask, pressure, geopotential and air temperature
      qp3_log(iqp3_mask) = .true.
      qp3_indx(iqp3_mask) = 1
      qp3_log(iqp3_maskamip) = .true.
      qp3_indx(iqp3_maskamip) = 2
      qp3_log(iqp3_pressure) = .true.
      qp3_indx(iqp3_pressure) = 3
      qp3_log(iqp3_zg) = .true.
      qp3_indx(iqp3_zg) = 4
      qp3_log(iqp3_ta) = .true.
      qp3_indx(iqp3_ta) = 5
      qp3_log(iqp3_ua) = .true.
      qp3_indx(iqp3_ua) = 6
      qp3_log(iqp3_va) = .true.
      qp3_indx(iqp3_va) = 7
      qp3_log(iqp3_wap) = .true.
      qp3_indx(iqp3_wap) = 8
      qp3_log(iqp3_vorticity) = .true.
      qp3_indx(iqp3_vorticity) = 9
      qp3_log(iqp3_divergence) = .true.
      qp3_indx(iqp3_divergence) = 10
      iqp3 = 10



!  read input data
      if(my_task.eq.0) then
         open(unit=2,file=trim(datapath)//'data/atm_qpopts', form='formatted')

         do
           read(2,*, IOSTAT=ioerr)doqptem,ldummy,nametem
	   nametem = trim(nametem)
           if(ioerr.ne.0) exit
           if(doqptem) then
             if (ldummy.eq.1) then
               do n = 1,nqp3
                 if(nametem.eq.qp3_name(n)) then

!   test if this is an automatically saved field
                   if( nametem.eq.qp3_name(iqp3_mask) .or.
     *                 nametem.eq.qp3_name(iqp3_maskamip) .or.
     *                 nametem.eq.qp3_name(iqp3_pressure) .or.
     *                 nametem.eq.qp3_name(iqp3_zg) .or.
     *                 nametem.eq.qp3_name(iqp3_va) .or.
     *                 nametem.eq.qp3_name(iqp3_ua) .or.
     *                 nametem.eq.qp3_name(iqp3_wap) .or.
     *                 nametem.eq.qp3_name(iqp3_vorticity) .or.
     *                 nametem.eq.qp3_name(iqp3_divergence) .or.
     *                 nametem.eq.qp3_name(iqp3_ta) ) exit
                   iqp3 = iqp3 + 1
                   qp3_log(n) = .true.
                   qp3_indx(n) = iqp3
                   exit
                 endif
                 if(n.eq.nqp3)
     *             print *,nametem,' is not an available qp3 diagnostic'
               enddo
             else if (ldummy.eq.0) then
               do n = 1,nqp2
                 if(nametem.eq.qp2_name(n)) then
                   iqp2 = iqp2 + 1
                   qp2_log(n) = .true.
                   qp2_indx(n) = iqp2
                   exit
                 endif
                 if(n.eq.nqp2)
     *             print *,nametem,' is not an available qp2 diagnostic'
               enddo
             endif
           endif
         enddo

         close(2)

	 write(6,*)
         do n = 1,nqp3
           if(qp3_log(n)) then
	     write(6,10) qp3_name(n), qp3_descr(n),qp3_indx(n)
 10          format('Will save qp3: ',a12,a68, '  in qp3 slot  ', i4)
	   endif
         enddo
         write(6,*)
         do n = 1,nqp2
           if(qp2_log(n)) then
             write(6,11) qp2_name(n), qp2_descr(n),qp2_indx(n)
 11          format('Will save qp2: ',a12,a68, '  in qp2 slot  ', i4)
           endif
         enddo
	 write(6,*)
      endif  ! if my_task.eq.0

      do n = 1,nhf2
         ihf2 = ihf2 + 1
         hf2_indx(n) = ihf2
      enddo

      do n = 1,nmjo2
         imjo2 = imjo2 + 1
         mjo2_indx(n) = imjo2
      enddo

      do n = 1,nmjo3
         imjo3 = imjo3 + 1
         mjo3_indx(n) = imjo3
      enddo

! set products true only if both components are true
      n = 0
      do
        n = n + 1
        if(qp3_prodlog(n).and.qp3_log(n)) then
          qp3_log(n) = qp3_log(qp3_prodindx(n,1))
     &           .and.qp3_log(qp3_prodindx(n,2))
          qp3_amip_sampling(n) = qp3_amip_sampling(qp3_prodindx(n,1))
     &           .or.qp3_amip_sampling(qp3_prodindx(n,2))
        endif
        if(n.ge.nqp3)exit
      enddo

!  broadcast information

#ifdef mpi
        call MPI_BCAST(
     *       qp2_indx,
     *       nqp2,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3_indx,
     *       nqp3,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp2_log,
     *       nqp2,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)
        
        call MPI_BCAST(
     *       qp3_log,
     *       nqp3,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp2_name,
     *       nqp2*16,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3_name,
     *       nqp3*16,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp2_units,
     *       nqp2*16,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3_units,
     *       nqp3*16,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp2_descr,
     *       nqp2*80,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3_descr,
     *       nqp3*80,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       iqp2,
     *       1,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       iqp3,
     *       1,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp2_type,
     *       nqp2*3,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       qp3_type,
     *       nqp3*3,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       hf2_type,
     *       nhf2*3,
     *       MPI_CHARACTER,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

#endif


!  allocate the qp arrays

      allocate(qp2(iim, jjm, 0:iqp2, nsdm))
      allocate(hf2(iim, jjm, 0:ihf2, nsdm))
      allocate(qp3(iim, jjm, mm, 0:iqp3, nsdm))
      allocate(mjo2(iim, jjm, 0:imjo2, nsdm))
      allocate(mjo3(iim, jjm, mm, 0:imjo3, nsdm))

      if(qpterptype.eq.1) then
         pme(1) = p_top
         do m = 2,mm
            pme(m) = 0.5 * (pm(m-1) + pm(m))
         enddo
c     isobaric coordinate, these assignments are done once for all
         do m = 1,mm
            pmktem = (pm(m)/p0_sfc)**kappa
            pm3(:,m,:) = pm(m)
            pmk3(:,m,:) = pmktem
         enddo
         do m = 1,mm
            pme3(:,m,:) = pme(m)
         enddo
      endif
      interp_indx = 0

!   assign a sequential index to each grid point
      if(my_task.eq.0) then
         do m = 1,max_ig
            indx_max_ig(m) = m
         enddo
         call scatter_to_grid_global(indx_max_ig(:),indx_imjm(:,:))
      endif
      call scatter_fine(indx_imjm(:,:),rinterp_indx(:,:,:))
      interp_indx(:,:,:) = nint(rinterp_indx(:,:,:))

!----------------------------------------------------------------------
!     calculate pcos and psin for computation of the first and second
!     harmonics of the diurnal cycle.
!----------------------------------------------------------------------


      nb = 86400._dbl_kind / dt
      ic3d   = 2*nb + 2.5
      icsum  =   nb + 0.5

      allocate(psin(ic3d))
      allocate(pcos(ic3d))
      
      do i = 1, ic3d
         pcos(i) = cos(pi2*(i-1)/nb)
         psin(i) = sin(pi2*(i-1)/nb)
      enddo

      psum = 0.0_dbl_kind
      do i = 1, icsum
         psum = psum + pcos(i)*pcos(i)
      enddo
      
      psum = nb/psum
      do i = 1, ic3d
         pcos(i) = pcos(i)*psum
         psin(i) = psin(i)*psum
      enddo
       
      
!----------------------------------------------------------------------

      end subroutine initialize_qpinfo



      subroutine initialize_qpfiles(simulation_tag, grid_mask)

!  this subroutine assigns all the qp information from the input file atm_qpopts

      use netcdf_geodesic

      implicit none

      character*40, intent(in) :: simulation_tag
      real (kind=dbl_kind), intent(in), dimension(iim,jjm,nsdm) ::
     *     grid_mask

      real (kind=dbl_kind), dimension(2,2,iim,jjm,nsdm) :: grid_corner
      real (kind=selected_real_kind(13)), allocatable, dimension(:) ::
     *        qpcenterlat, qpcenterlon, qpcenterarea
      real (kind=selected_real_kind(13)), allocatable, dimension(:,:) :: 
     *        qpcornerlat, qpcornerlon

      character*8, time_label
      character*4, proc_label
      integer (kind=int_kind) :: j, ksdm, n_points, monthofyear

      integer (kind=int_kind), parameter :: corners=6
      real (kind=dbl_kind), dimension(corners,iim,jjm,nsdm) :: 
     *        cornerlat, cornerlon

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

      write(unit=proc_label,fmt='(i4)')
     *                1000+my_task
      proc_label(1:1) = 'p'

!-- process grid corner data
      grid_corner(1,:,:,:,:) = grid_corner_lon(:,:,:,:)
      grid_corner(2,:,:,:,:) = grid_corner_lat(:,:,:,:)
      call wrap_vertices(2,1, grid_corner)
      cornerlon = 0.0
      cornerlon(1:2,:,:,:) = grid_corner(1,1:2,:,:,:)
      cornerlon(3,1:iim-1,:,:) = grid_corner(1,1,2:iim,:,:)
      cornerlon(4,1:iim-1,1:jjm-1,:) = grid_corner(1,2,2:iim,2:jjm,:)
      cornerlon(5,1:iim-1,1:jjm-1,:) = grid_corner(1,1,2:iim,2:jjm,:)
      cornerlon(6,:,1:jjm-1,:) = grid_corner(1,2,:,2:jjm,:)
      cornerlat = 0.0
      cornerlat(1:2,:,:,:) = grid_corner(2,1:2,:,:,:)
      cornerlat(3,1:iim-1,:,:) = grid_corner(2,1,2:iim,:,:)
      cornerlat(4,1:iim-1,1:jjm-1,:) = grid_corner(2,2,2:iim,2:jjm,:)
      cornerlat(5,1:iim-1,1:jjm-1,:) = grid_corner(2,1,2:iim,2:jjm,:)
      cornerlat(6,:,1:jjm-1,:) = grid_corner(2,2,:,2:jjm,:)

!-- fill the zeroes in the np, sp pentagons 
      cornerlon(6,1,jjm-1,:) = cornerlon(5,1,jjm-1,:)
      cornerlat(6,1,jjm-1,:) = cornerlat(5,1,jjm-1,:)
      cornerlon(3,iim-1,1,:) = cornerlon(2,iim-1,1,:)
      cornerlat(3,iim-1,1,:) = cornerlat(2,iim-1,1,:)

      qp_points = 0
      do ksdm = 1,nsdm
        do j = 1,jjm
          do i = 1,iim
            if(grid_mask(i,j,ksdm).gt.0.5_dbl_kind) qp_points = qp_points + 1
          enddo
        enddo
      enddo

      allocate (qpcornerlat(corners,qp_points))
      allocate (qpcenterlat(qp_points))
      allocate (qpcornerlon(corners,qp_points))
      allocate (qpcenterlon(qp_points))
      allocate (qpcenterarea(qp_points))

!  gather unique data
      n_points = 0
      do ksdm = 1,nsdm
        do j = 1,jjm
          do i = 1,iim
             if(grid_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                n_points = n_points + 1
                qpcenterlat(n_points) = grid_center_lat(i,j,ksdm)
                qpcenterlon(n_points) = grid_center_lon(i,j,ksdm)
                qpcenterarea(n_points) = area(i,j,ksdm)
                qpcornerlat(:,n_points) = cornerlat(:,i,j,ksdm)
                qpcornerlon(:,n_points) = cornerlon(:,i,j,ksdm)
             endif
          enddo
        enddo
      enddo

      call ginit_atm_nc (simulation_tag, "qp_output/gatm_"//trim(time_label)
     *                 //trim(proc_label)//".g2.nc", 
     *                 qp2_descr, qp2_name, qp2_units, qp2_type, qp2_log,
     *                 qp2_multfac, qp2_contmin, qp2_contint, qp2_altunits, 
     *                 qp_points, mm, pm, "qp_output/gatm_"//trim(time_label)
     *                 //trim(proc_label)//".g3.nc",
     *                 qp3_descr, qp3_name, qp3_units, qp3_type, qp3_log,
     *		       "hf_output/gatm_"//trim(time_label)
     *                 //trim(proc_label)//".g2.nc",
     *                 hf2_descr, hf2_name, hf2_units, hf2_type, hf2_log,
     *                 qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                 corners, max_ig, qpcenterarea)

      deallocate (qpcornerlat)
      deallocate (qpcenterlat)
      deallocate (qpcornerlon)
      deallocate (qpcenterlon)
      deallocate (qpcenterarea)

      nsecondinit = nsecond_dyn - dt
      iqp_sample = 0
      qp2(:,:,:,:) = c0
      ihf_sample = 0
      hf2(:,:,:,:) = c0
      qp3(:,:,:,:,:) = c0
      imjo2_sample = 0
      mjo2(:,:,:,:) = c0
      mjo3(:,:,:,:,:) = c0

#ifndef no_pbl
      if(my_task.eq.0) then
         open(unit=mjo2file,
     *          file=trim(datapath)//'mjo_output/mjo2_'//trim(time_label),
     *		form='unformatted')
         open(unit=mjo3file,
     *          file=trim(datapath)//'mjo_output/mjo3_'//trim(time_label),
     *		form='unformatted')
      endif
#endif

      end subroutine initialize_qpfiles



      subroutine setqp_wrap

      use prognostics

      IMPLICIT none

      integer (kind=int_kind) :: ksdm

      do ksdm = 1,nsdm

         call setqp(bpg(1,1,1,ksdm),    bpgl2(1,1,1,ksdm),
     *              th(1,1,1,n3,ksdm),  plg(1,1,1,ksdm),
     *              plgl2(1,1,1,ksdm),  ksdm)

      enddo



      end subroutine setqp_wrap



      subroutine setqp ( bpg, bpeg, t, pl, pl2, ksdm )

      IMPLICIT none

c-------------------------------------------------------------------
c     Subroutine SETQP sets up the vertical interpolation of 3D arrays
c       from model sigma surfaces to user selected constant pressure
c       surfaces. SETQP calculates the arrays LBND, which contains the
c       array location of the model level directly above some pressure
c       level, used by subroutines QPVINT, QQVINT, CUPINT and PHVINT,
c       LCND which identifies the model layer a pressure surface
c       lies within for interpolating cloud quantities (CVINT) and also
c       for calculating interpolated geopotential height, PFA and PFB,
c       interpolation weights that go with LBND using linear interpolation
c       in p**kapa, and MASK, which identifies points beneath the earth's
c       surface in the interpolation. Special cases for QPVINT, QQVINT
c       and CUPINT interpolation are for pressure surfaces located between
c       model level NLMM1 and the PBL top, which is interpolated using the
c       PBL value at the PBL top, and for pressure surfaces in the PBL, which
c       are assigned the pbl value.

C     ARGUMENT LIST VARIABLES
C        Input variables:
      integer (kind=int_kind), intent(in) :: ksdm
      real (kind=dbl_kind), intent(in), dimension(iimjjm,nlm) ::
     *     pl, t, bpg
      real (kind=dbl_kind), intent(in), dimension(iimjjm,nlm+1) ::
     *     pl2, bpeg

c     LOCAL VARIABLES:
      real (kind=dbl_kind), dimension(iimjjm,nlm) :: gtem     ! gather array
      real (kind=dbl_kind), dimension(iimjjm,mm) :: masktem
      real (kind=dbl_kind), dimension(iim,jjm,mm) :: mask3
      real (kind=dbl_kind) ::
     *     dkapa, tem
      integer m, l, j, lbeg, lbegnew, item

c----------------------------------------------------------------


c    make 3d array for pressure

      do i = 1,iimjjm
         pme3(i,mm+1,ksdm) =
     *               max(pl2(i,nlm+1), pm3(i,mm,ksdm))
      enddo
      if(qpterptype.eq.2) then
c     isentropic coordinate
c not sure how we want to modify this interpolation type for the no-pbl case
c   but i'll save that for another day
         dkapa = 1./kappa
         lbeg=2
         do m = 1,mm
            do i = 1,iimjjm
               pmk3(i,m,ksdm) = bpg(i,lbeg-1) + (tm(m)-t(i,lbeg-1)) *
     *            (bpg(i,lbeg) - bpg(i,lbeg-1))
     *            /(t(i,lbeg)-t(i,lbeg-1))
            enddo
            lbegnew=nlm-1
            do l = lbeg,nlm-1
               do i = 1,iimjjm
                  if(t(i,l).ge.tm(m)) then
                     pmk3(i,m,ksdm) = bpg(i,l) + (tm(m)-t(i,l)) *
     *                  (bpg(i,l+1)-bpg(i,l))
     *                       /(t(i,l+1)-t(i,l))
                  else
                     lbegnew = min(lbegnew,l-1)
                  endif
               enddo
            enddo
            lbeg = max(2,lbegnew)
            do i = 1,iimjjm
               pmk3(i,m,ksdm) = max(pmk3(i,m,ksdm),bpeg(i,1))
               pmk3(i,m,ksdm) =
     *            min(pmk3(i,m,ksdm),bpeg(i,nlmp1)+float(m)*0.001)
               pm3(i,m,ksdm) = p0_sfc* pmk3(i,m,ksdm)**dkapa
            enddo
         enddo
         do i = 1,iimjjm
            pme3(i,1,ksdm) = p_top
            pme3(i,mm+1,ksdm) =
     *               max(pl2(i,nlm+1), pm3(i,mm,ksdm))
         enddo
         do m = 2,mm
            do i = 1,iimjjm
               pme3(i,m,ksdm) = 0.5 * (pm3(i,m,ksdm) +
     *               pm3(i,m-1,ksdm))
            enddo
         enddo
      endif
C
C    find the index (lbnd) of model layer above pressure pm3
C
      DO M=1,MM
        DO I = 1 , iimjjm
          LBND(I,M,ksdm) = 1
        enddo
#ifdef no_pbl
        DO L = 2,nLM
          DO I = 1 , iimjjm
            IF(bpg(I,L).LT.PMK3(i,M,ksdm))
     *                   LBND(I,M,ksdm) = l
          enddo
        enddo
#else
        DO L = 2,nLM-1
          DO I = 1 , iimjjm
            IF(bpg(I,L).LT.PMK3(i,M,ksdm))
     *                   LBND(I,M,ksdm) = l
          enddo
        enddo
        DO I = 1 , iimjjm
          IF(bpeg(I,nLM).LT.PMK3(i,M,ksdm)) 
     *                     LBND(I,M,ksdm) = nlm
        enddo
#endif
      enddo
C
C     CALCULATE INTERPOLATING FACTORS FOR A LINEAR INTERPOLATION
C       IN P**2/7
C
      do m = 1,mm
        DO I = 1 , iimjjm     !   this loop is a gather
          pfb(i,m,ksdm) = bpg(i,lbnd(i,m,ksdm))
        enddo
      enddo
#ifdef no_pbl
      do l = 1,nlm-1
        DO I = 1 , iimjjm
          gtem(i,l) = bpg(i,l+1)
        enddo
      enddo
      do l=nlm,nlm     !  for boundary layer interpolation
        DO I = 1 , iimjjm
          gtem(i,l) = bpeg(i,l+1)
        enddo
      enddo
#else
      do l = 1,nlm-2
        DO I = 1 , iimjjm
          gtem(i,l) = bpg(i,l+1)
        enddo
      enddo
      do l=nlm-1,nlm     !  for boundary layer interpolation
        DO I = 1 , iimjjm
          gtem(i,l) = bpeg(i,l+1)
        enddo
      enddo
#endif
      do m = 1,mm
        DO I = 1 , iimjjm     !   this loop is a gather
          pfa(i,m,ksdm) = gtem(i,lbnd(i,m,ksdm))
        enddo
      enddo

      DO m = 1,mm
#ifdef no_pbl
        DO I = 1 , iimjjm
          IF (LBND(I,m,ksdm).LE.nlm-1) THEN
            tem = 1.0 / (PFA(I,m,ksdm) - PFB(I,m,ksdm))
            PFA(I,m,ksdm) = tem *
     *             (PMk3(I,m,ksdm) - PFA(I,m,ksdm))
            PFB(I,m,ksdm) = tem *
     *             (PMk3(I,m,ksdm) - PFB(I,m,ksdm))
          ELSE    ! mixed layer

            PFA(I,m,ksdm) = -1.0
            PFB(I,m,ksdm) = 0.0
          END IF
        enddo
#else
        DO I = 1 , iimjjm
          IF (LBND(I,m,ksdm).LE.nlm-1) THEN
            tem = 1.0 / (PFA(I,m,ksdm) - PFB(I,m,ksdm))
            PFA(I,m,ksdm) = tem *
     *             (PMk3(I,m,ksdm) - PFA(I,m,ksdm))
            PFB(I,m,ksdm) = tem *
     *             (PMk3(I,m,ksdm) - PFB(I,m,ksdm))
          ELSE    ! mixed layer

            PFA(I,m,ksdm) = -1.0
            PFB(I,m,ksdm) = 0.0
          END IF
        enddo
#endif
      enddo
C
C     CALCULATE MASK
C
      DO M=1,MM
        DO I = 1 , iimjjm    !  level above earth's sfc

          IF (PMK3(i,M,ksdm).LT.bpeg(I,nLM+1)) THEN
            masktem(I,M) = 1.0
          ELSE        !    level beneath earth's sfc

            PFB(I,M,ksdm) = 0.0
            PFA(I,M,ksdm) = 0.0
            masktem(I,M) = 0.0
          END IF
        enddo
      enddo

         do j = 1,jjm
           item = (j-1)*iim
           mask3(:,j,:) = masktem(item+1:item+iim,:)
         enddo

         qp3(:,:,:,qp3_indx(iqp3_mask),ksdm) =
     *            qp3(:,:,:,qp3_indx(iqp3_mask),ksdm)+ mask3(:,:,:)

         if(mod(nsecond_dyn,21600).eq.0.or.
     *                 .not.qp3_amip_sampling(iqp3_maskamip)) then
            qp3(:,:,:,qp3_indx(iqp3_maskamip),ksdm) =
     *            qp3(:,:,:,qp3_indx(iqp3_maskamip),ksdm) + mask3(:,:,:)
            mjo3(:,:,:,mjo3_indx(imjo3_mask),ksdm) = 
     *            mjo3(:,:,:,mjo3_indx(imjo3_mask),ksdm) + mask3(:,:,:)
         endif

C
C     FIND CLOUD LAYER INTERPOLATION LEVEL FROM QPVINT INDEXING (LBND)
C
      DO  M = 1,MM
        DO I = 1 , iimjjm
          if( (bpeg(i,lbnd(i,m,ksdm)+1).gt.pmk3(i,m,ksdm))
     *             .or.(lbnd(i,m,ksdm).gt.nlm-1) ) then
            LCND(i,m,ksdm) = LBND(i,m,ksdm)
          else
            LCND(i,m,ksdm) = LBND(i,m,ksdm) + 1
          endif
        enddo
      enddo


      end subroutine setqp



      subroutine qp2sum(xin, x_const, index, ksdm, len, ioffset, step_sample)

!  this subroutine accumulates a field in a section in the qp2 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a qp2
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by iqp2_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            qp2 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset
      integer (kind=int_kind), intent(in), optional ::
     *     step_sample

      real (kind=dbl_kind), dimension(len), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend

      real (kind=dbl_kind), dimension(iimjjm) :: x1

#ifdef scm
      return
#endif

      if(qp2_log(index).or.qp2_mjo_link(index).ne.0) then
        if(qp2_amip_sampling(index)) then
          if(mod(nsecond_dyn,21600).ne.0) return
        endif
         
         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         x1 = 0.0
         x1(offset+1:offset+len) = xin(1:len)

         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1

         if(qp2_log(index)) then
! save the qp2 array
! update the sample size
           if(.not.present(step_sample)) then
             if(ksdm.eq.1.and.offset.eq.0)
     *             iqp_sample(index) = iqp_sample(index) + 1
           endif
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             qp2(ibeg:iend,j,qp2_indx(index),ksdm) =
     *          qp2(ibeg:iend,j,qp2_indx(index),ksdm) 
     *        + x_const * x1(ibeg+item:iend+item)
           enddo
         endif
         if(qp2_mjo_link(index).ne.0) then
           if(mjo2_log(qp2_mjo_link(index))) then
! save the mjo2 array
! update the sample size
             if(.not.present(step_sample)) then
               if(ksdm.eq.1.and.offset.eq.0)
     *               imjo2_sample(qp2_mjo_link(index)) =
     *                         imjo2_sample(qp2_mjo_link(index)) + 1
             endif
             do j = jbeg,jend
               item = (j-1)*iim
               ibeg = 1 + max( 0, offset-(j-1)*iim )
               iend = min( iim, offset+len-(j-1)*iim )
               mjo2(ibeg:iend,j,mjo2_indx(qp2_mjo_link(index)),ksdm) =
     *            mjo2(ibeg:iend,j,mjo2_indx(qp2_mjo_link(index)),ksdm) 
     *          + x_const * x1(ibeg+item:iend+item)
             enddo
           endif
         endif


      endif !  if(qp2_log(index).or.qp2_mjo_link(index).ne.0)

      end subroutine qp2sum



      subroutine hf2sum(xin, x_const, index, ksdm, len, ioffset, step_sample)

!  this subroutine accumulates a field in a section in the hf2 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a hf2
!  x_const - a weight to scale xin
!  index - the hf2 array index specified by ihf2_xxx (where xxx is the 
!          field name) in the calling program
!  len - the length of the data domain to store
!  ioffset - (optional) the offset to the starting location in the 
!            hf2 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the hf slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset
      integer (kind=int_kind), intent(in), optional ::
     *     step_sample

      real (kind=dbl_kind), dimension(len), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend

      real (kind=dbl_kind), dimension(iimjjm) :: x1

#ifdef scm
      return
#endif

      if(hf2_log(index)) then
        if(hf2_amip_sampling(index)) then
          if(mod(nsecond_dyn,21600*nhf_freq).ne.0) return
        endif
         
         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
! update the sample size
         if(.not.present(step_sample)) then
         if(ksdm.eq.1.and.offset.eq.0) ihf_sample(index) = ihf_sample(index) + 1
         endif
         x1 = 0.0
         x1(offset+1:offset+len) = xin(1:len)

         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1

! save the hf2 array
         do j = jbeg,jend
           item = (j-1)*iim
           ibeg = 1 + max( 0, offset-(j-1)*iim )
           iend = min( iim, offset+len-(j-1)*iim )
           hf2(ibeg:iend,j,hf2_indx(index),ksdm) =
     *        hf2(ibeg:iend,j,hf2_indx(index),ksdm) 
     *      + x_const * x1(ibeg+item:iend+item)
         enddo


      endif !  if(hf2_log(index))

      end subroutine hf2sum



      subroutine qp3sum(xin, lenx, x_const, index, ksdm, len, ioffset, parray)

!  this subroutine vertically interpolates a field to constant pressure
!      or theta surfaces and accumulates it in a section in the qp3 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a qp3
!  lenx - horizontal dimension of xin
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by iqp3_xxx (where xxx is the 
!          field name) in the calling program
!  ksdm - the subdomain index
!  len - the length of the data domain to store (can't be large than lenx)
!  ioffset - (optional) the offset to the starting location in the 
!            qp3 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section to fill
     *,    lenx    ! horizontal dimension of xin
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset

      real (kind=dbl_kind), dimension(lenx,nlm), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), dimension(lenx,mm), intent(out), optional ::
     *      parray  !  field interpolated to pressure surfaces
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend, m

      real (kind=dbl_kind), dimension(iimjjm,nlm) :: x1
      real (kind=dbl_kind), dimension(iimjjm,mm) :: x2
      real (kind=dbl_kind), dimension(iim,jjm) :: hftem

#ifdef scm
      return
#endif

      if(qp3_log(index).or.qp3_mjo_link(index).ne.0) then
        if(qp3_amip_sampling(index)) then
          if(mod(nsecond_dyn,21600).ne.0) return
        endif

!   test for compatible length arguments
         if(lenx.lt.len) then
            print *,'qp3sum called with incompatible len, len arguments'
            print *,' for qp3 index ',index
            return
         endif
 
         
         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         x1 = 0.0
         x1(offset+1:offset+len,:) = xin(1:len,:)
         x2 = 0.0

!  vertically interpolate the field from sigma surfaces to constant pressure
!        or theta surfaces

         call qpvint

         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1

         if(qp3_log(index)) then
! save the qp3 array
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             qp3(ibeg:iend,j,:,qp3_indx(index),ksdm) =
     *            qp3(ibeg:iend,j,:,qp3_indx(index),ksdm)
     *            + x_const * x2(item+ibeg:item+iend,:)
           enddo
           if(present(parray)) parray(1:len,:) = x2(offset+1:offset+len,:)
         endif

         if(qp3_mjo_link(index).ne.0) then
! save the mjo3 array
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             mjo3(ibeg:iend,j,:,qp3_mjo_link(index),ksdm) =
     *            mjo3(ibeg:iend,j,:,qp3_mjo_link(index),ksdm)
     *            + x_const * x2(item+ibeg:item+iend,:)
           enddo
         endif

       if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         if(index.eq.iqp3_ke) then
           do m = 1,mm
             if(abs(pm(m)-25000._dbl_kind).lt.1._dbl_kind)  then
               do j = jbeg,jend
                 item = (j-1)*iim
                 ibeg = 1 + max( 0, offset-(j-1)*iim )
                 iend = min( iim, offset+len-(j-1)*iim )
                 hftem(ibeg:iend,j) = x_const * x2(item+ibeg:item+iend,m)
               enddo
               call hf2sum(hftem, c1,ihf2_ke250, ksdm, iim*jjm)
             endif
           enddo
         endif

         if(index.eq.iqp3_en) then
           do m = 1,mm
             if(abs(pm(m)-25000._dbl_kind).lt.1._dbl_kind)  then
               do j = jbeg,jend
                 item = (j-1)*iim
                 ibeg = 1 + max( 0, offset-(j-1)*iim )
                 iend = min( iim, offset+len-(j-1)*iim )
                 hftem(ibeg:iend,j) = x_const * x2(item+ibeg:item+iend,m)
               enddo
               call hf2sum(hftem, c1,ihf2_en250, ksdm, iim*jjm)
             endif
           enddo
         endif
      endif ! if(mod(nsecond_dyn,21600*nhf_freq).eq.0)

      endif !  if(qp3_log(index).or.qp3_mjo_link(index).ne.0)

      contains

c-----------------------------------------------------------------------

      SUBROUTINE QPVINT

c-------------------------------------------------------------------

      IMPLICIT none

c-------------------------------------------------------------------
c     Subroutine  QPVINT vertically interpolates arrays to constant
c       pressure surfaces linearly in p**kapa using factors precomputed
c       by SETQP. Special cases are for pressure surfaces located between
c       model level NLMM1 and the PBL top, which is interpolated using the
c       PBL value at the PBL top, and for pressure surfaces in the PBL, which
c       are assigned the pbl value.


c     LOCAL VARIABLES:
      real (kind=dbl_kind), dimension(len,nlm) :: gtem  ! work array
      real (kind=dbl_kind), dimension(len,mm) :: 
     *      qpa   ! gathered array of values above
     *,     qpb   ! gathered array of values below
      integer m, l   !  loop indices

c----------------------------------------------------------------
C
C     PICK OUT VALUES TO INTERPOLATE
C
      do m = 1,mm
         do I = 1, len   !   this loop is a gather
            qpa(i,m) = x1(i+offset, lbnd(i+offset ,m, ksdm))
         enddo
      enddo

      DO l = 1,nlm-1
         do I = 1, len   !   this loop is a gather
            gtem(i,l) = x1(i+offset,l+1)
         enddo
      enddo

      DO I = 1 , len   ! for pbl interpolation
        gtem(I,nLM) = 0.0
      enddo
      do m = 1,mm
         do I = 1, len   !   this loop is a gather
            qpb(i,m) = gtem(i, lbnd(i+offset ,m,ksdm))
         enddo
      enddo
C
C     INTERPOLATE!!
C
      DO m = 1,mm
        do I = 1, len
           x2(I+offset ,m) = (QPB(I,m) * PFB(I+offset ,m,ksdm) -
     *                           QPA(I,m) * PFA(I+offset ,m, ksdm))
        enddo
      enddo

      END subroutine qpvint

      end subroutine qp3sum



      subroutine qp3sum_cld(xin, pc, lenx, x_const, index, ksdm, len, ioffset
     *,                     parray)

!  this subroutine assigns the value of cloudiness for the sigma
!        layer at the constant pressure or theta surface to that
!        surface.

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  xin - the input array to be stored as a qp3
!  pc - stratus cloud base (mb)
!  lenx - horizontal dimension of xin
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by iqp3_xxx (where xxx is the 
!          field name) in the calling program
!  ksdm - the subdomain index
!  len - the length of the data domain to store (can't be large than lenx)
!  ioffset - (optional) the offset to the starting location in the 
!            qp3 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section to fill
     *,    lenx    ! horizontal dimension of xin
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset

      real (kind=dbl_kind), dimension(lenx,nlm), intent(in) ::
     *      xin  !  input field to accumulate
      real (kind=dbl_kind), dimension(lenx), intent(in) ::
     *      pc  !  stratus cloud base (mb)
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor
      real (kind=dbl_kind), dimension(lenx,mm), intent(out), optional ::
     *      parray  !  field interpolated to pressure surfaces

! local arrays
      integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend, m

      real (kind=dbl_kind), dimension(iimjjm,nlm) :: x1
      real (kind=dbl_kind), dimension(iimjjm,mm) :: x2

#ifdef scm
      return
#endif

      if(qp3_log(index).or.qp3_mjo_link(index).ne.0) then
        if(qp3_amip_sampling(index)) then
          if(mod(nsecond_dyn,21600).ne.0) return
        endif

!   test for compatible length arguments
         if(lenx.lt.len) then
            print *,'qp3sum called with incompatible len, len arguments'
            print *,' for qp3 index ',index
            return
         endif
 
         
         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         x1 = 0.0
         x1(offset+1:offset+len,:) = xin(1:len,:)
         x2 = 0.0
         
         do m = 1,mm
            do i = 1,len
               x2(i+offset,m) = x1(i+offset,lcnd(i+offset,m,ksdm))
               if(100._dbl_kind*pc(i).le.pm3(i+offset,m,ksdm))
     *                            x2(i+offset,m) = 0.0_dbl_kind
            enddo
         enddo

         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1

! save the qp3 array
         if(qp3_log(index)) then
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             qp3(ibeg:iend,j,:,qp3_indx(index),ksdm) =
     *            qp3(ibeg:iend,j,:,qp3_indx(index),ksdm)
     *            + x_const * x2(item+ibeg:item+iend,:)
           enddo
           if(present(parray)) parray(1:len,:) = x2(offset+1:offset+len,:)
         endif

         if(qp3_mjo_link(index).ne.0) then
! save the mjo3 array
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             mjo3(ibeg:iend,j,:,qp3_mjo_link(index),ksdm) =
     *            mjo3(ibeg:iend,j,:,qp3_mjo_link(index),ksdm)
     *            + x_const * x2(item+ibeg:item+iend,:)
           enddo
         endif


      endif !  if(qp3_log(index).or.qp3_mjo_link(index).ne.0)

      end subroutine qp3sum_cld




      subroutine qp3sum_zg(ph, t, lenx, bpeg, bpg
     *,                    x_const, index, ksdm, len, ioffset, parray)

!  this subroutine sums a section of geopotential in the qp3 array

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  ph - geopotential
!  t - potential temperature
!  bpeg - layer edge pressure**kappa
!  bpg - mid layer pressure**kappa
!  lenx - horizontal dimension of xin
!  x_const - a weight to scale xin
!  index - the qp2 array index specified by iqp3_xxx (where xxx is the 
!          field name) in the calling program
!  ksdm - the subdomain index
!  len - the length of the data domain to store (can't be large than lenx)
!  ioffset - (optional) the offset to the starting location in the 
!            qp3 field for storage, default is 0.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     index   ! index of the qp slot to save field to
     *,    ksdm    ! subdomain to fill
     *,    len     ! length of xin array section to fill
     *,    lenx    ! horizontal dimension of xin
      integer (kind=int_kind), intent(in), optional ::
     *     ioffset
      real (kind=dbl_kind), dimension(lenx,mm), intent(out), optional ::
     *      parray  !  field interpolated to pressure surfaces

      real (kind=dbl_kind), dimension(iimjjm,nlm+1), intent(in) ::
     *      bpeg   
      real (kind=dbl_kind), dimension(iimjjm,nlm), intent(in) ::
     *      bpg   
      real (kind=dbl_kind), dimension(lenx,nlm), intent(in) ::
     *      ph   !  geopotential
     *,     t    !  potential temperature
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

! local arrays
      integer (kind=int_kind) :: item, offset, j, jbeg, jend, ibeg, iend

      real (kind=dbl_kind), dimension(iimjjm,nlm) :: th1, ph1
      real (kind=dbl_kind), dimension(iimjjm,mm) :: ph2

#ifdef scm
      return
#endif

      if(qp3_log(index).or.qp3_mjo_link(index).ne.0) then
        if(qp3_amip_sampling(index)) then
          if(mod(nsecond_dyn,21600).ne.0) return
        endif
         
!   test for compatible length arguments
         if(lenx.lt.len) then
            print *,'qp3sum called with incompatible len, len arguments'
            print *,' for qp3 index ',index
            return
         endif

         if(present(ioffset)) then
           offset = ioffset
         else
           offset = 0
         endif
         th1 = 0.0
         ph1 = 0.0
         th1(offset+1:offset+len,:) = t(1:len,:)
         ph1(offset+1:offset+len,:) = ph(1:len,:)

!  compute the vertically interpolated geopotential
         call phvint

         jbeg = offset/iim + 1
         jend = (offset+len-1)/iim+1

         if(qp3_log(index)) then
! save the geopotential into the array
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             qp3(ibeg:iend,j,:,qp3_indx(index),ksdm) =
     *            qp3(ibeg:iend,j,:,qp3_indx(index),ksdm)
     *            + x_const * ph2(item+ibeg:item+iend,:)
           enddo
           if(present(parray)) parray(1:len,:) = ph2(offset+1:offset+len,:)
         endif

         if(qp3_mjo_link(index).ne.0) then
! save the mjo3 array
           do j = jbeg,jend
             item = (j-1)*iim
             ibeg = 1 + max( 0, offset-(j-1)*iim )
             iend = min( iim, offset+len-(j-1)*iim )
             mjo3(ibeg:iend,j,:,qp3_mjo_link(index),ksdm) =
     *            mjo3(ibeg:iend,j,:,qp3_mjo_link(index),ksdm)
     *            + x_const * ph2(item+ibeg:item+iend,:)
           enddo
         endif

      endif !  if(qp3_log(index).or.qp3_mjo_link(index).ne.0)

      contains

      subroutine phvint

!  this subroutine computes a consistent vertical interpolation of geopotential

!!!WARNING!!!!
!
! this routine is currently only consistent with the generalized sigma
!    coordinate option, and only for npbl=1
! This routine will only be approximate for other configurations
!
!!!WARNING!!!


      implicit none

!  work arrays
      real (kind=dbl_kind), dimension(mm) :: t1, t2, t3, t4, t5, t6
      real (kind=dbl_kind), dimension(nlm) :: gtem ! work variable for gathering
      real (kind=dbl_kind) :: gravin, pintk
      integer (kind=int_kind) :: l, m

      gravin = 1.0 / grav
      pintk = (p_interface/p0_sfc)**kappa

      do i = 1,len

C
C     INDEX TEMPERATURES TO PROPER LEVELS
C
         do m = 1,mm
            t5(m) = th1(i+offset,lbnd(i+offset,m, ksdm))
         enddo
         DO l = 1,nlm-1
            gtem(l) = th1(i+offset,l+1)
         enddo
         gtem(nlm) = th1(i+offset,nlm)
         do m = 1,mm
            t6(m) = gtem(lbnd(i+offset,m,ksdm))
         enddo
C
C     PREPARE INTERPOLATION IN STRATOSPHERE
C
         do m = 1,mm
            if(pmk3(i+offset,m,ksdm).le.pintk) then
               T4(m) = T5(m) / T6(m)
               t1(m) = log(t4(m))
               T4(m) = T4(m) - 1.0
               t4(m) = abs(t4(m))
               IF (T4(m).GT.0.000001) THEN
                  T6(m) = T5(m) * T1(m) / T4(m)
               ELSE

                  T6(m) = T5(m)
               END IF
            endif
         enddo
C
C     PREPARE TROPOSPHERE
C
         do m = 1,mm
            t2(m) = bpeg(i,lbnd(i+offset,m,ksdm)+1)
         enddo
         DO l = 1,nlm-1
            gtem(l) = bpg(i,l+1)
         enddo
         gtem(nlm) = bpeg(i,nlm+1)
         do m = 1,mm
            t4(m) = gtem(lbnd(i+offset,m,ksdm))
            t3(m) = bpg(i,lbnd(i+offset,m,ksdm))
            if(pmk3(i+offset,m,ksdm).gt.pintk)
     *         T6(m) = ((T2(m) - T3(m)) * T5(m)
     *             + T6(m) * (T4(m) -T2(m))) / (T4(m) - T3(m))
         enddo
C
C     BOTH TROPOSPHERE AND STRATOSPHERE
C
         do m = 1,mm
            t4(m) = bpg(i,lcnd(i+offset,m,ksdm))
            T3(M) = PMK3(i+offset,M,ksdm)
            T6(m) = T6(m) * (T4(m) - T3(m)) * 0.5
            t5(m) = th1(i+offset,lcnd(i+offset,m,ksdm))
            T6(m) = T6(m) + T5(m) * (0.5 * (T4(m) + T3(m)) -T2(m))
            T6(m) = spec_heat_cp * T6(m) * (T4(m) - T3(m)) / (T4(m) - T2(m))
            t2(m) = ph1(i+offset,lcnd(i+offset,m,ksdm))
         enddo
C
C     BOUNDARY LAYER
C
         do m = 1,mm
            t4(m) = bpg(i,nlm)
            t5(m) = th1(i+offset,nlm)
            IF(LCND(I+offset,M,ksdm).GT.nlm-1)
     *         T6(M) = T5(M) * (T4(M) - T3(M)) * spec_heat_cp
         enddo
C
C    FINALLY SUM UP TERMS
C
         DO m = 1,mm
            ph2(I+offset,m) = (T6(m) + T2(m)) * gravin
            if(pmk3(i+offset,m,ksdm).gt.bpeg(i,nlm+1)) ph2(i+offset,m) = 0.0
         enddo

      enddo  ! i loop over grid cell

      end subroutine phvint

      end subroutine qp3sum_zg



      subroutine qp3sum_divvort(u, pthick_c, pl2g
     *,                    x_const, ksdm, div_200
     *,                    diva, zeta_a, ua, va, wap )

!  this subroutine sums the divergence and vorticity  in their qp3 array slots

!!!!!!!!!!Argument list description!!!!!!!!!!!!!!!!!!
!
!  u - wind 
!  pl2g - layer edge pressure
!  x_const - a weight to scale the qp3
!  ksdm - the subdomain index
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      use grid_metrics
      use operators

      implicit none
 
      integer (kind=int_kind), intent(in) :: ksdm
      real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm), intent(in) ::
     *      pthick_c   
      real (kind=dbl_kind), dimension(iim,jjm,nlm+1), intent(in) ::
     *      pl2g   
      real (kind=dbl_kind), dimension(iim,jjm), intent(out) ::
     *      div_200   
      real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm,nlm)
     *,     intent(in) :: u    !  wind at cell corners
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor

!  arrays passed out to form products (on pressure surfaces)
      real (kind=dbl_kind), intent(out), dimension(iim,jjm,mm) ::
     *      ua     ! zonal wind
     *,     va     ! meridional wind
     *,     wap    ! vertical pressure velocity
     *,     diva   ! divergence
     *,     zeta_a ! reloative vorticity

! local arrays
      integer (kind=int_kind) :: k
      real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm,mm) ::
     *		work_u  ! vertically interpolated tangential wind
      real (kind=dbl_kind), dimension(iim,jjm) ::
     *		work     ! divergence or vorticity on a pressure surface
     *,         vvert
     *,         vvert2
      real (kind=dbl_kind), dimension(iim,jjm,mm) :: 
     * 		pmtem 
      real (kind=dbl_kind), dimension(iim,jjm,mm+1) :: 
     * 		pmetem 
      real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm+1) ::
     *		pl2g_c   ! cell wall layer edge pressure
      real (kind=dbl_kind), dimension(ncorners,iim,jjm,mm) ::
     *		pm_c     ! cell wall interpolation pressure 
      real (kind=dbl_kind), dimension(ncorners,iim,jjm,mm+1) ::
     *		pme_c    ! cell wall interpolation layer edge pressure

#ifdef scm
      return
#endif

!  amip sampling is assumied for all quantities in this subroutine.
      if(mod(nsecond_dyn,21600).ne.0.and.amip_sampling) return

!     get pressures on cell walls
        pl2g_c(:,:,:,1) = p_top
        do k = 1,nlm
           pl2g_c(:,:,:,k+1) = pl2g_c(:,:,:,k) + pthick_c(:,:,:,k)*dsig(k)
        enddo

!  if pm3 and pme3 were on an isentropic surface, we would have to wrap the vertices
!   of pm_c and pme_c. For now, we'll assume pm3 and pme3 are on isobaric surfaces
        do k = 1,mm
          pmtem(:,:,k) = pm3(1,k,ksdm)
          pm_c (:,:,:,k) = pm3(1,k,ksdm)
!          do j = 1,jjm
!          do i = 1,iim
!             ij = i + (j-1) * iim
!             pmtem(i,j,k) = pm3(ij,k,ksdm)
!          enddo
!          enddo
!          call average_to_cell_corners(
!    *                  ipad,
!    *                  iim,
!    *                  jjm,
!    *                  ndata,
!    *                  ncorners,
!    *                  s_weights(:,:,:,:,ksdm),
!    *                  pmtem(:,:,k),
!    *                  pm_c(:,:,:,k))
        enddo
        do k = 1,mm+1
          pmetem(:,:,k) = pme3(1,k,ksdm)
          pme_c (:,:,:,k) = pme3(1,k,ksdm)
!          do j = 1,jjm
!          do i = 1,iim
!             ij = i + (j-1) * iim
!             pmetem(i,j,k) = pme3(ij,k,ksdm)
!          enddo
!          enddo
!          call average_to_cell_corners(
!    *                  ipad,
!    *                  iim,
!    *                  jjm,
!    *                  ndata,
!    *                  ncorners,
!    *                  s_weights(:,:,:,:,ksdm),
!    *                  pmetem(:,:,k),
!    *                  pme_c(:,:,:,k))
        enddo

! get vorticity on pressure surfaces and sum in its qp3 slot

! vertically interpolate the wind (both components
          call uvint(u, work_u)

          if(qp3_log(iqp3_vorticity).or.qp3_mjo_link(iqp3_vorticity).ne.0) then
            do k = 1,mm

! compute vorticity on the pressure surface
              call curl_of_a_vector(                  
     *          ipad,                           
     *          iim,                                 
     *          jjm,                             
     *          ndata,                            
     *          nspace,                           
     *          ncorners,                        
     *          area_inv(:,:,ksdm),                 
     *          v_weights(:,:,:,:,:,ksdm),       
     *          work_u(:,:,:,:,k),                     
     *          work)   ! vorticity

! zero out below surface
              where( pmtem(:,:,k).gt.pl2g(:,:,nlm+1) ) work(:,:) = 0.0

! accumulate in the qp slot
              qp3(:,:,k,qp3_indx(iqp3_vorticity),ksdm) =
     *            qp3(:,:,k,qp3_indx(iqp3_vorticity),ksdm)
     *                  + x_const * work(:,:)
              zeta_a(:,:,k) = work(:,:)
              if(qp3_mjo_link(iqp3_vorticity).ne.0) then
! save the mjo3 array
                mjo3(:,:,k,qp3_mjo_link(iqp3_vorticity),ksdm) =
     *               mjo3(:,:,k,qp3_mjo_link(iqp3_vorticity),ksdm)
     *               + x_const * work(:,:)
              endif
            enddo
          endif

! get divergence on pressure surfaces and sum in its qp3 slot


           vvert(:,:) = 0.0
           vvert2(:,:) = 0.0
           do k = 1,mm
! compute divergence on the pressure surface
              call div_of_a_vector(          
     *          ipad,                       
     *          iim,                              
     *          jjm,                                
     *          ndata,                             
     *          nspace,                           
     *          ncorners,                           
     *          area_inv(:,:,ksdm),                    
     *          v_weights(:,:,:,:,:,ksdm),         
     *          work_u(:,:,:,:,k),                   
     *          work)   ! divergence

              vvert(:,:) = vvert2(:,:)
     *                - (pmtem(:,:,k) - pmetem(:,:,k)) * work(:,:)
              vvert2(:,:) = vvert(:,:)
     *                - (pmetem(:,:,k+1) - pmtem(:,:,k)) * work(:,:)

            if(qp3_log(iqp3_divergence)) then
! zero out below surface
              where( pmtem(:,:,k).gt.pl2g(:,:,nlm+1) ) work(:,:) = 0.0

! accumulate in the qp slot
              qp3(:,:,k,qp3_indx(iqp3_divergence),ksdm) =
     *            qp3(:,:,k,qp3_indx(iqp3_divergence),ksdm)
     *                  + x_const * work(:,:)
              diva(:,:,k) = work(:,:)
              if(qp3_mjo_link(iqp3_divergence).ne.0) then
! save the mjo3 array
                mjo3(:,:,k,qp3_mjo_link(iqp3_divergence),ksdm) =
     *               mjo3(:,:,k,qp3_mjo_link(iqp3_divergence),ksdm)
     *               + x_const * work(:,:)
              endif
            endif

            if(abs(pm(k)-20000._dbl_kind).lt.1._dbl_kind)
     *          div_200(:,:) = work(:,:) 

! zero out below surface
              where( pmtem(:,:,k).gt.pl2g(:,:,nlm+1) ) vvert(:,:) = 0.0

! accumulate in the qp slot
              qp3(:,:,k,qp3_indx(iqp3_wap),ksdm) =
     *            qp3(:,:,k,qp3_indx(iqp3_wap),ksdm)
     *                  + x_const * vvert(:,:)
              wap(:,:,k) = vvert(:,:)
              if(qp3_mjo_link(iqp3_wap).ne.0) then
! save the mjo3 array
                mjo3(:,:,k,qp3_mjo_link(iqp3_wap),ksdm) =
     *               mjo3(:,:,k,qp3_mjo_link(iqp3_wap),ksdm)
     *               + x_const * vvert(:,:)
              endif
           enddo

!  get the zonal and meridional winds
 	   do k = 1,mm

!    move winds from corner 1 to cell center - will later average properly
              ua(:,:,k) = work_u(1,1,:,:,k)
              va(:,:,k) = work_u(2,1,:,:,k)

!  zero out below surface and accumulate zonal wind
                 where( pmtem(:,:,k).gt.pl2g(:,:,nlm+1) ) ua(:,:,k) = 0.0
              qp3(:,:,k,qp3_indx(iqp3_ua),ksdm) =
     *            qp3(:,:,k,qp3_indx(iqp3_ua),ksdm)
     *                  + x_const * ua(:,:,k)
                if(qp3_mjo_link(iqp3_ua).ne.0) then
! save the mjo3 array
                  mjo3(:,:,k,qp3_mjo_link(iqp3_ua),ksdm) =
     *                 mjo3(:,:,k,qp3_mjo_link(iqp3_ua),ksdm)
     *                 + x_const * ua(:,:,k)
                endif

!  zero out below surface and accumulate meridional wind
                 where( pmtem(:,:,k).gt.pl2g(:,:,nlm+1) ) va(:,:,k) = 0.0
              qp3(:,:,k,qp3_indx(iqp3_va),ksdm) =
     *            qp3(:,:,k,qp3_indx(iqp3_va),ksdm)
     *                  + x_const * va(:,:,k)
                if(qp3_mjo_link(iqp3_va).ne.0) then
! save the mjo3 array
                  mjo3(:,:,k,qp3_mjo_link(iqp3_va),ksdm) =
     *                 mjo3(:,:,k,qp3_mjo_link(iqp3_va),ksdm)
     *                 + x_const * va(:,:,k)
                endif
         if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
              if(abs(pm(k)-85000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(va(1,1,k), c1,ihf2_va850, ksdm, iim*jjm)
              if(abs(pm(k)-20000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(va(1,1,k), c1,ihf2_va200, ksdm, iim*jjm)
              if(abs(pm(k)-5000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(va(1,1,k), c1,ihf2_va50, ksdm, iim*jjm)
              if(abs(pm(k)-85000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(ua(1,1,k), c1,ihf2_ua850, ksdm, iim*jjm)
              if(abs(pm(k)-20000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(ua(1,1,k), c1,ihf2_ua200, ksdm, iim*jjm)
              if(abs(pm(k)-5000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(ua(1,1,k), c1,ihf2_ua50, ksdm, iim*jjm)
              if(abs(pm(k)-50000._dbl_kind).lt.1._dbl_kind) 
     *              call hf2sum(wap(1,1,k), c1,ihf2_wap500, ksdm, iim*jjm)
         endif
	   enddo

      contains

        subroutine uvint(uv, work_uv)

	implicit none

	real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm,nlm)
     *,          intent(in) :: uv
	real (kind=dbl_kind), dimension(nspace,ncorners,iim,jjm,mm)
     *,          intent(out) :: work_uv

!  local variables
        integer (kind=int_kind) :: m, lss, j, icorners, l
        integer (kind=int_kind), dimension(ncorners,iim,jjm) :: lb, lc, nm
        real (kind=dbl_kind), dimension(ncorners,iim,jjm) :: idpm

!  initialize indexing and interpolation variables
	lb = 1
	lc = 1
	lss = 1
        work_uv = 0.0

!  find nm, the lowest pressure level above the surface
        DO M = 1,MM
          where(pl2g_c(:,:,:,nlm+1).gt.pm_c(:,:,:,m)) nm(:,:,:) = m
        enddo

        DO M = 1,MM
!  find the index of the sigma layer at pressure pme(m+1), lc, 
           do j = 1,jjm
           do i = 1,iim
           do icorners = 1,ncorners
              lss = min(lss, lc(icorners, i, j) )
           enddo
           enddo
           enddo
           DO L=LSS,nLM+1
             where(pl2g_c(:,:,:,l).lt.pme_c(:,:,:,m+1)) lc(:,:,:) = l
           enddo
           IF(M.NE.MM) THEN
              where(pl2g_c(:,:,:,nlm+1).le.pm_c(:,:,:,m+1))
     *                  lc(:,:,:) = nlm
           ELSE
              LC(:,:,:) = nLM
           END IF

!  get the mass flux for pressure layer at m by summing between layer lb and lc
           DO L = LSS,nLM
               where(LB(:,:,:).EQ.L)
                  work_uv(1,:,:,:,M) = work_uv(1,:,:,:,M) + uv(1,:,:,:,L) *
     *               (pl2g_c(:,:,:,L) - PME_c(:,:,:,M))
                  work_uv(2,:,:,:,M) = work_uv(2,:,:,:,M) + uv(2,:,:,:,L) *
     *               (pl2g_c(:,:,:,L) - PME_c(:,:,:,M))
               end where
               where(L.EQ.LC(:,:,:).AND.m.ne.nm(:,:,:))
                  work_uv(1,:,:,:,M) = work_uv(1,:,:,:,M) + uv(1,:,:,:,L) *
     *               (PME_c(:,:,:,M+1) - pl2g_c(:,:,:,L+1))
                  work_uv(2,:,:,:,M) = work_uv(2,:,:,:,M) + uv(2,:,:,:,L) *
     *               (PME_c(:,:,:,M+1) - pl2g_c(:,:,:,L+1))
               end where
               where(LB(:,:,:).LE.L.AND.LC(:,:,:).GE.L)
                  work_uv(1,:,:,:,M) = work_uv(1,:,:,:,M) + uv(1,:,:,:,L) *
     *               (pl2g_c(:,:,:,L+1) - pl2g_c(:,:,:,L))
                  work_uv(2,:,:,:,M) = work_uv(2,:,:,:,M) + uv(2,:,:,:,L) *
     *               (pl2g_c(:,:,:,L+1) - pl2g_c(:,:,:,L))
               end where
           enddo

!  get the pressure layer thickness and use inverse to get interpolated wind
           where(M.LT.NM(:,:,:)) 
              idpm(:,:,:) = 1.0/(PME_c(:,:,:,M+1)-PME_c(:,:,:,M))
           ELSE where
              idpm(:,:,:) = 0.
           END where
           where(M.EQ.NM(:,:,:)) 
     *        idpm(:,:,:) = 1.0/(pl2g_c(:,:,:,nLM+1)-PME_c(:,:,:,M))
           work_uv(1,:,:,:,m) = work_uv(1,:,:,:,m) * idpm(:,:,:)
           work_uv(2,:,:,:,m) = work_uv(2,:,:,:,m) * idpm(:,:,:)

!   set top layer index for next pressure level to current bottom layer index
           LB(:,:,:) = LC(:,:,:)
        enddo


        end subroutine uvint

      end subroutine qp3sum_divvort


      subroutine qp3_product_sum(x1, x2, index, x_const, ksdm)

!  this routine stores the product of two interpolated fields in a qp slot

      real (kind=dbl_kind), dimension(iim,jjm,mm), intent(in) ::
     *      x1, x2   ! input fields
      integer (kind=int_kind), intent(in) :: index
      real (kind=dbl_kind), intent(in) :: x_const  ! scaling factor
      integer (kind=int_kind), intent(in) :: ksdm

      if(qp3_log(index)) then
         if(qp3_amip_sampling(index)) then
            if(mod(nsecond_dyn,21600).ne.0) return
         endif
         qp3(:,:,:,qp3_indx(index),ksdm) =
     *            qp3(:,:,:,qp3_indx(index),ksdm)
     *          + x_const * x1(:,:,:) * x2(:,:,:)
      endif

      end subroutine qp3_product_sum



      subroutine write_qp_diagnostics(grid_mask)

      use netcdf_general_tools

      implicit none

      real (kind=dbl_kind), dimension(iim,jjm,nsdm), intent(in) :: grid_mask

      integer (kind=int_kind) :: j, n, m
      real (kind=real_kind) :: qp_samples_inv, kappatem, mask_samples_inv
      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) :: mask_inv
     *,                                                       mask_inv_amip

      logical (kind=log_kind), dimension(iim,jjm,nsdm) :: l_grid_mask

      allocate (mask_inv(iim,jjm,mm,nsdm))
      allocate (mask_inv_amip(iim,jjm,mm,nsdm))

! convert theta to temperature
      do j = 1,jjm
        do i = 1,iim
          n = i + (j-1) * iim
          qp3(i,j,:,qp3_indx(iqp3_ta),:) = qp3(i,j,:,qp3_indx(iqp3_ta),:) 
     *                  * pmk3(n,:,:)
        enddo
      enddo
      
      if(qpterptype == 1) then
!  no interpolation of pressure
         do m = 1,mm
            qp3(:,:,m,qp3_indx(iqp3_pressure),:) = 
     *               qp3(:,:,m,qp3_indx(iqp3_mask),:) * pm(m)
         enddo
      else
! convert p**kappa to pressure
         kappatem = 1.0 - inv_kappa
         where(qp3(:,:,:,qp3_indx(iqp3_mask),:).gt.0.5_real_kind)
     *       qp3(:,:,:,qp3_indx(iqp3_pressure),:) =
     *           qp3(:,:,:,qp3_indx(iqp3_pressure),:)**inv_kappa * p0_sfc
     *         * qp3(:,:,:,qp3_indx(iqp3_mask),:)**kappatem
      endif

! normalize
      do n = 1,nqp2
         if(qp2_log(n)) then
            if(iqp_sample(n).eq.0) then
               qp_samples_inv = 0.0
            else
               qp_samples_inv = 1. / float(iqp_sample(n))
            endif
            qp2(:,:,qp2_indx(n),:) = qp2(:,:,qp2_indx(n),:) * qp_samples_inv
         endif
      enddo

      mask_inv = c0
      mask_inv_amip = c0
      where(qp3(:,:,1:mm,iqp3_mask,:).ge.0.9)
     *            mask_inv(:,:,1:mm,:) = 1.0 / qp3(:,:,1:mm,iqp3_mask,:)
      where(qp3(:,:,1:mm,iqp3_maskamip,:).ge.0.9)
     *        mask_inv_amip(:,:,1:mm,:) = 1.0 / qp3(:,:,1:mm,iqp3_maskamip,:)

      mask_samples_inv = 1. / qp3(2,2,1,iqp3_mask,1)      
      qp3(:,:,1:mm,iqp3_mask,:) = 
     *       qp3(:,:,1:mm,iqp3_mask,:) * mask_samples_inv
      mask_samples_inv = 1. / qp3(2,2,1,iqp3_maskamip,1)      
      qp3(:,:,1:mm,iqp3_maskamip,:) = 
     *       qp3(:,:,1:mm,iqp3_maskamip,:) * mask_samples_inv
      do n = 3,nqp3
        if(qp3_log(n)) then
           if(qp3_amip_sampling(n)) then
             where(mask_inv_amip(:,:,1:mm,:).gt.1.e-8_dbl_kind) 
               qp3(:,:,1:mm,qp3_indx(n),:) = 
     *           qp3(:,:,1:mm,qp3_indx(n),:) * mask_inv_amip(:,:,1:mm,:)
             else where
               qp3(:,:,1:mm,qp3_indx(n),:) = missing
             end where
           else
             where(mask_inv(:,:,1:mm,:).gt.1.e-8_dbl_kind) 
               qp3(:,:,1:mm,qp3_indx(n),:) = 
     *           qp3(:,:,1:mm,qp3_indx(n),:) * mask_inv(:,:,1:mm,:)
             else where
               qp3(:,:,1:mm,qp3_indx(n),:) = missing
             end where
           endif
         endif
      enddo

! get the primes and bars of product qps
      n = 2
      do
        n = n + 1
        if(qp3_prodlog(n).and.qp3_log(n)) then
!  compute x1_bar * x2_bar, x1_prime*x2_prime
          where(mask_inv_amip(:,:,1:mm,:).gt.1.e-8_dbl_kind) 
            qp3(:,:,1:mm,qp3_indx(n)+1,:) = 
     *          qp3(:,:,1:mm,qp3_indx(qp3_prodindx(n,1)),:)
     *           * qp3(:,:,1:mm,qp3_indx(qp3_prodindx(n,2)),:)
            qp3(:,:,1:mm,qp3_indx(n),:) = 
     *         qp3(:,:,1:mm,qp3_indx(n),:) - qp3(:,:,1:mm,qp3_indx(n)+1,:)
          else where
            qp3(:,:,1:mm,qp3_indx(n),:) = missing
            qp3(:,:,1:mm,qp3_indx(n)+1,:) = missing
          end where
          n = n + 1
        endif
        if(n.ge.nqp3)exit
      enddo

      if(qp3_log(iqp3_psbg)) qp3(:,:,1:mm,qp3_indx(iqp3_psbg),:) = 
     *           100. * (1. - qp3(:,:,1:mm,qp3_indx(iqp3_mask),:) )
      
      l_grid_mask = .false.
      where(grid_mask(:,:,:).gt.0.5_dbl_kind) l_grid_mask(:,:,:) = .true.

!  write the data to a netcdf file
      call gwrt_atm_nc(nsecond_dyn, nsecondinit, nyear0, qp_points, l_grid_mask,
     *                 qp2(:,:,1:iqp2,:), qp2_name, qp2_log, qp2_indx,
     *                 landfrac, seafrac, icefrac, interp_indx,
     *                 qp3(:,:,1:mm,1:iqp3,:), qp3_name, qp3_log, qp3_indx )

!   zero out for next period of summation

      nsecondinit = nsecond_dyn
      qp2 = c0
      qp3 = c0
      iqp_sample = 0

      deallocate(mask_inv)
      deallocate(mask_inv_amip)

      end subroutine write_qp_diagnostics



      subroutine write_hf_diagnostics(grid_mask)

      implicit none

      real (kind=dbl_kind), dimension(iim,jjm,nsdm), intent(in) :: grid_mask

      integer (kind=int_kind) :: n
      real (kind=real_kind) :: hf_samples_inv

      logical (kind=log_kind), dimension(iim,jjm,nsdm) :: l_grid_mask

! normalize
      do n = 1,nhf2
         if(hf2_log(n)) then
            if(ihf_sample(n).ge.1) then
              hf_samples_inv = 1. / float(ihf_sample(n))
              hf2(:,:,hf2_indx(n),:) = hf2(:,:,hf2_indx(n),:) * hf_samples_inv
            endif
         endif
      enddo
      
      l_grid_mask = .false.
      where(grid_mask(:,:,:).gt.0.5_dbl_kind) l_grid_mask(:,:,:) = .true.

!  write the data to a netcdf file
      call gwrt_hf_nc(nsecond_dyn, nyear0, qp_points, l_grid_mask,
     *                 hf2(:,:,1:ihf2,:), hf2_name, hf2_log, hf2_indx,
     *                 landfrac, seafrac, icefrac, interp_indx  )

!   zero out for next period of summation

      hf2 = c0
      ihf_sample = 0

      end subroutine write_hf_diagnostics



      subroutine write_mjo_diagnostics

      use interpolate_to_latlon
      implicit none

      integer (kind=int_kind) :: n, k
      real (kind=dbl_kind), dimension(im,jm) :: work
      real (kind=dbl_kind), dimension(im,jm,nlm) :: work3
      real (kind=dbl_kind) :: mjo_samples_inv

! normalize and write out interpolated 2d diagnostics
      do n = 1,nmjo2
         if(mjo2_log(n)) then
            if(imjo2_sample(n) .gt. 0.0_dbl_kind) then
               mjo_samples_inv = 1. / float(imjo2_sample(n))
            else
               mjo_samples_inv = 1.0_dbl_kind
            endif
            mjo2(:,:,mjo2_indx(n),:) = mjo2(:,:,mjo2_indx(n),:)
     *                 * mjo_samples_inv
            work = c0
            call gather_fine(mjo2(:,:,mjo2_indx(n),:),work(:,:))
            if(my_task.eq.0) call interp_driver_mjo(work, mjo2file)
         endif
      enddo
      mjo2 = c0
      imjo2_sample = 0

! normalize and write out interpolated 3d diagnostics
      do n = 1,nmjo3
         if(mjo3_log(n)) then
            work3 = c0
            do k = 1,mm
               call gather_fine(mjo3(:,:,k,mjo3_indx(n),:),work3(:,:,k))
            enddo
            if(my_task.eq.0) call interp_driver_mjo3(work3, mm, mjo3file)
         endif
      enddo
      mjo3 = c0

      end subroutine write_mjo_diagnostics


      end module qp_module
